/*
 * File: $Id: DicomSpectroWorklistTable.java,v 1.1.1.1 $
 * History
 * Date: 12.12.2016  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2016 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.clinicalcasehandling;

 
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import org.spectrim.batchprocessing.BatchProcessing;

/**
 *
 * @author slotboom
 */
public class DicomSpectroWorklistTable extends JTable implements  ActionListener, MouseListener
{


    public  DicomSpectroWorklistTable( )
    {
        super( );

        Object[][] cellData = { { "1", "2", "3", "4", "5", "6", "7", "8", "9", "0" },
                                { "1", "2", "3", "4", "5", "6", "7", "8", "9", "0" }};
       
        m_tableModel = new DefaultTableModel( );
       
        m_tableModel.addColumn( "<HTML><B>Patient name</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Date of Birth</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Processing status</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Series description</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Study date</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Spectroscopy data type</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Scanner model</B></HTML>" );    
        m_tableModel.addColumn( "<HTML><B>Processing date</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Dicom file reference</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Case UID</B></HTML>" );
        
        setModel( m_tableModel );
        
        // define default behavior of the JTable
        setAutoCreateColumnsFromModel( false );
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );

        constructObject( );

        // Initialize the toggle functions
        for ( int i = 0 ; i < 10; i++ )
        {
            m_ascending[i] = true;
        }
        
    }

    /**
     * Does the rest of the object construction work
     */
    private void constructObject( )
    {
        m_popup = new JPopupMenu( );

        m_selectAllItems = new JMenuItem( "Select all items" );
        m_selectAllItems.addActionListener( this );

        m_deleteSelectedItems = new JMenuItem( "Delete selected items" );
        m_deleteSelectedItems.addActionListener( this );

        // sorting possibilities
        m_sortByPatientName = new JMenuItem( "Sort by patient name" );
        m_sortByPatientName.addActionListener( this );
    
        m_sortByPatientDateOfBirth = new JMenuItem( "Sort by patient name" );
        m_sortByPatientDateOfBirth.addActionListener( this );
        
        m_sortByProcessingStatus = new JMenuItem( "Sort by processing status" );
        m_sortByProcessingStatus.addActionListener( this );
        
        m_sortBySpectroscopyDataType = new JMenuItem( "Sort by Spectroscopy type");
        m_sortBySpectroscopyDataType.addActionListener( this );
        
        m_sortByStudyDate = new JMenuItem( "Sort by study date" );
        m_sortByStudyDate.addActionListener( this );
        
        m_sortByScannerModel = new JMenuItem( "Sort by scanner model" );
        m_sortByScannerModel.addActionListener( this );
        
        m_sortBySeriesDescription = new JMenuItem( "Sort by series description" );
        m_sortBySeriesDescription.addActionListener( this );
        
        m_sortByProcessingDateAndTime = new JMenuItem( "Sort by processing date/time" );
        m_sortByProcessingDateAndTime.addActionListener( this );
        
        m_sortByDicomFileReference = new JMenuItem ("Sort by file reference");
        m_sortByDicomFileReference.addActionListener( this );
        
        m_sortBySpectroscopyCaseUID = new JMenuItem( "Sort by spectroscopy case uid");
        m_sortBySpectroscopyCaseUID.addActionListener( this );
        
        // add to the popup menu
        m_popup.add( m_selectAllItems );
        m_popup.add( m_deleteSelectedItems );

        m_popup.addSeparator( );

        m_popup.add( m_sortByPatientName );
        m_popup.add( m_sortByPatientDateOfBirth );
        m_popup.add( m_sortByProcessingStatus );
        m_popup.add( m_sortBySeriesDescription );
        m_popup.add( m_sortBySpectroscopyDataType );
        m_popup.add( m_sortByStudyDate );
        m_popup.add( m_sortByScannerModel );
        m_popup.add( m_sortByProcessingDateAndTime );
        m_popup.add( m_sortByDicomFileReference );
        m_popup.add( m_sortBySpectroscopyCaseUID );
        
        add( m_popup );

        addMouseListener( this );

    }

    /**
     * Is called whenever a registered component raises an ActionEvent
     */
    public void actionPerformed( ActionEvent e )
    {

        Object sender = e.getSource( );

        if ( e.getSource( ) == BatchProcessing.class )
        {
            String fileName = BatchProcessing.m_file;
            if ( e.getID() == BatchProcessing.THREAD_STARTED )
            {
                changeBatchProcessingStatus( fileName, BatchProcessing.THREAD_STARTED_CMD );
            }
            
            if ( e.getID() == BatchProcessing.THREAD_FINISHED )
            {
                changeBatchProcessingStatus( fileName, BatchProcessing.THREAD_FINISHED_CMD );
            }
        }
        
        if ( sender == m_selectAllItems )
        {
            this.selectAll( );
        }

        
        m_popup.add( m_sortByPatientName );
        
        m_popup.add( m_sortByPatientDateOfBirth );
        
        m_popup.add( m_sortByProcessingStatus );
        
        m_popup.add( m_sortBySeriesDescription );
        
        m_popup.add( m_sortBySpectroscopyDataType );
        
        m_popup.add( m_sortByStudyDate );
        
        m_popup.add( m_sortByScannerModel );
        
        m_popup.add( m_sortByProcessingDateAndTime );
        
        m_popup.add( m_sortByDicomFileReference );
        
        m_popup.add( m_sortBySpectroscopyCaseUID );
        
        
        if ( sender == m_sortByPatientName )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 0, m_ascending[0] );
            m_ascending[0] = !m_ascending[0];
        }

        if ( sender == m_sortByPatientDateOfBirth )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 1, m_ascending[1] );
            m_ascending[1] = !m_ascending[1];
        }

        if ( sender == m_sortByProcessingStatus )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 2, m_ascending[2] );
            m_ascending[2] = !m_ascending[2];
        }

        if ( sender == m_sortBySeriesDescription )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 3, m_ascending[3] );
            m_ascending[3] = !m_ascending[3];
        }

        if ( sender == m_sortBySpectroscopyDataType )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 4, m_ascending[4] );
            m_ascending[4] = !m_ascending[4];
        }
        
        if ( sender == m_sortByStudyDate )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 5, m_ascending[5] );
            m_ascending[5] = !m_ascending[5];
        }

        if ( sender == m_sortByScannerModel )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 6, m_ascending[6] );
            m_ascending[6] = !m_ascending[6];
        }

        if ( sender ==  m_sortByProcessingDateAndTime )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 7, m_ascending[7] );
            m_ascending[7] = !m_ascending[7];
        }

        if ( sender == m_sortByDicomFileReference )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 8, m_ascending[8] );
            m_ascending[8] = !m_ascending[8];
        }
        
        if ( sender ==  m_sortBySpectroscopyCaseUID )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 9, m_ascending[9] );
            m_ascending[9] = !m_ascending[9];
        }
        
        if ( sender == m_deleteSelectedItems )
        {
            deleteSelectedFileReferences( );
        }

    }

    /**
     * Method that sorts the list after image position
     */
    private void numericSortAllRowsBySeriesNumber( )
    {
        int numberOfColumns = m_tableModel.getColumnCount( );
        int numberOfRows    = m_tableModel.getRowCount( );

        Object[][] inputData = new Object[numberOfRows][numberOfColumns];
        Object[][] sortedData = new Object[numberOfRows][numberOfColumns];

        float[] inputSlicePositions = new float[ numberOfRows ];
        float[] orderedSlicePositions = new float[ numberOfRows ];

        int[] order = new int[numberOfRows];

        for ( int i = 0; i < numberOfColumns; i++ )
        {
            for ( int j = 0; j < numberOfRows; j++ )
            {
                inputData[j][i] = m_tableModel.getValueAt( j, i );
            }
        }

        for ( int j = 0; j < numberOfRows; j++ )
        {
            inputSlicePositions[j] = new Float( m_tableModel.getValueAt( j, 2 ).toString( ) ).floatValue( );
        }

        orderedSlicePositions = QuickSort.quickSort( order, inputSlicePositions );

        for ( int i = 0; i < numberOfRows; i++ )
        {
            sortedData[i][0] = inputData[order[i]][0];
            sortedData[i][1] = inputData[order[i]][1];
            sortedData[i][2] = inputData[order[i]][2];
            sortedData[i][3] = inputData[order[i]][3];
            sortedData[i][4] = inputData[order[i]][4];
            sortedData[i][5] = inputData[order[i]][5];
            sortedData[i][6] = inputData[order[i]][6];
            sortedData[i][7] = inputData[order[i]][7];
            sortedData[i][8] = inputData[order[i]][8];
            sortedData[i][9] = inputData[order[i]][9];
        }

        for ( int i = 0; i < numberOfColumns; i++ )
        {
            for ( int j = 0; j < numberOfRows; j++ )
            {
                m_tableModel.setValueAt( sortedData[j][i], j, i );
            }
        }

        m_tableModel.fireTableStructureChanged( );

    }


    /**
     * Method that sorts the list after image position
     */
    private void numericSortAllRowsByInstanceNumber( )
    {
        int numberOfColumns = m_tableModel.getColumnCount( );
        int numberOfRows    = m_tableModel.getRowCount( );

        Object[][] inputData = new Object[numberOfRows][numberOfColumns];
        Object[][] sortedData = new Object[numberOfRows][numberOfColumns];

        float[] inputSlicePositions = new float[ numberOfRows ];
        float[] orderedSlicePositions = new float[ numberOfRows ];

        int[] order = new int[numberOfRows];

        for ( int i = 0; i < numberOfColumns; i++ )
        {
            for ( int j = 0; j < numberOfRows; j++ )
            {
                inputData[j][i] = m_tableModel.getValueAt( j, i );
            }
        }

        for ( int j = 0; j < numberOfRows; j++ )
        {
            inputSlicePositions[j] = new Float( m_tableModel.getValueAt( j, 4 ).toString( ) ).floatValue( );
        }

        orderedSlicePositions = QuickSort.quickSort( order, inputSlicePositions );

        for ( int i = 0; i < numberOfRows; i++ )
        {
            sortedData[i][0] = inputData[order[i]][0];
            sortedData[i][1] = inputData[order[i]][1];
            sortedData[i][2] = inputData[order[i]][2];
            sortedData[i][3] = inputData[order[i]][3];
            sortedData[i][4] = inputData[order[i]][4];
            sortedData[i][5] = inputData[order[i]][5];
            sortedData[i][6] = inputData[order[i]][6];
            sortedData[i][7] = inputData[order[i]][7];
            sortedData[i][8] = inputData[order[i]][8];
            sortedData[i][9] = inputData[order[i]][9];
        }

        for ( int i = 0; i < numberOfColumns; i++ )
        {
            for ( int j = 0; j < numberOfRows; j++ )
            {
                m_tableModel.setValueAt( sortedData[j][i], j, i );
            }
        }

        m_tableModel.fireTableStructureChanged( );

    }


    /**
     * Deletes selected file references, not files!
     */
    private void deleteSelectedFileReferences( )
    {
        int[] selectedRows = this.getSelectedRows( );

        for ( int i = selectedRows.length -1 ; i > -1 ; i-- )
        {
            m_tableModel.removeRow( selectedRows[i] );
        }

        m_tableModel.fireTableStructureChanged( );
    }

    /**
     * Deletes selected file references, not files!
     */
    private void deleteAllTableElements( )
    {
        int numberOfRowsToDelete = m_tableModel.getRowCount( );

        for ( int i = numberOfRowsToDelete - 1 ; i > -1 ; i-- )
        {
            m_tableModel.removeRow( i );
        }

        m_tableModel.fireTableStructureChanged( );
    }


     /**
      *  Regardless of sort order (ascending or descending), null values always appear last.
      *  colIndex specifies a column in model.
      */
    public void sortAllRowsBy( DefaultTableModel model, int colIndex, boolean ascending )
    {
        Vector data = model.getDataVector( );
        Collections.sort( data, new ColumnSorter( colIndex, ascending ) );
        model.fireTableStructureChanged( );
    }

       /**
     * Is called whenever a mouse button is pressed
     */
    public void mousePressed( MouseEvent e )
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
    }

    /**
     * Is called whenever a mouse button is clicked
     */
    public void mouseClicked( MouseEvent e )
    {
    }

    /**
     * Is called whenever a mouse pointer enters
     */
    public void mouseEntered( MouseEvent e )
    {
    }

    /**
     * Is called whenever the mouse pointer exits
     */
    public void mouseExited( MouseEvent e )
    {
    }

    /**
     * Is called whenever a mouse button is released
     */
    public void mouseReleased( MouseEvent e )
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
    }

    /**
     * Sets the cell values of the list. Since it is a public method it can be
     * called from a other object
     */
    public void setListData(  Vector<ClinicalSpectroscopyCaseWorklistDataElement> worklistElements )
    {

        deleteAllTableElements( );

        // declare the m_cellData object again.
        m_cellData = new Object[worklistElements.size( )][10];
        
        Iterator it = worklistElements.iterator( );

        int counter = 0;

        m_popup.add( m_sortByPatientName );
        m_popup.add( m_sortByPatientDateOfBirth );
        m_popup.add( m_sortByProcessingStatus );
        m_popup.add( m_sortBySeriesDescription );
        m_popup.add( m_sortBySpectroscopyDataType );
        m_popup.add( m_sortByStudyDate );
        m_popup.add( m_sortByScannerModel );
        m_popup.add( m_sortByProcessingDateAndTime );
        m_popup.add( m_sortByDicomFileReference );
        m_popup.add( m_sortBySpectroscopyCaseUID );
        
        while ( it.hasNext( ) )
        {
            ClinicalSpectroscopyCaseWorklistDataElement element = (ClinicalSpectroscopyCaseWorklistDataElement)it.next( );

            
            m_tableModel.addColumn( "<HTML><B>Patient name</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Date of Birth</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Processing status</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Series description</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Study date</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Spectroscopy data type</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Scanner model</B></HTML>" );    
            m_tableModel.addColumn( "<HTML><B>Processing date</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Dicom file reference</B></HTML>" );
            m_tableModel.addColumn( "<HTML><B>Case UID</B></HTML>" );
            
            
            m_cellData[counter][0] = element.getPatientName( );
            m_cellData[counter][1] = element.getPatientDateOfBirth( );
            m_cellData[counter][2] = element.getProcessingStatus( ) + "";
            m_cellData[counter][3] = element.getSeriesDescription( );
            m_cellData[counter][4] = element.getStudyDate( );
            m_cellData[counter][5] = element.getSpectroscopyDataType( );
            m_cellData[counter][6] = element.getScannerModel( );
            m_cellData[counter][7] = element.getProcessingDateAndTime( );
            m_cellData[counter][8] = element.getDicomFileReference( );
            m_cellData[counter][9] = element.getM_case_UID( );

            counter++;
        }

        m_tableModel = new DefaultTableModel( m_cellData, m_columnNames );

        setModel( m_tableModel );

        // selectAll( );

        updateUI( );
    }

    
    private void changeBatchProcessingStatus( String fileName, String newStatus )
    {
        for ( int j = 0; j < m_tableModel.getRowCount( ); j++ )
        {
            String fileRef = (String)m_tableModel.getValueAt(j, 8);
            if (fileRef.compareTo(fileName) == 0)
            {
                m_tableModel.setValueAt(newStatus, j, 2);
            }
            
        }
    }
    
    /**
     * Sets the cell values of the list. Since it is a public method it can be
     * called from a other object
     */
    public void changeListDataElement( ClinicalSpectroscopyCaseWorklistDataElement worklistElement )
    {

        for ( int j = 0; j < m_tableModel.getRowCount( ); j++ )
        {
            if ( ( (String)m_tableModel.getValueAt(j, 6 ) ) .compareTo( worklistElement.getDicomFileInstanceUID( ) ) == 0 )
            {
                m_tableModel.setValueAt( worklistElement.getPatientName( ), j, 0 );
                m_tableModel.setValueAt( worklistElement.getPatientDateOfBirth( ), j, 1 );
                m_tableModel.setValueAt( worklistElement.getProcessingStatus( ), j, 2 );
                m_tableModel.setValueAt( worklistElement.getSeriesDescription( ), j, 3 );
                m_tableModel.setValueAt( worklistElement.getStudyDate( ), j, 4 );
                m_tableModel.setValueAt( worklistElement.getSpectroscopyDataType( ), j, 5 );
                m_tableModel.setValueAt( worklistElement.getScannerModel( ), j, 6 );
                m_tableModel.setValueAt( worklistElement.getProcessingDateAndTime( ), j, 7 );
                m_tableModel.setValueAt( worklistElement.getDicomFileReference( ), j, 8 );
                m_tableModel.setValueAt( worklistElement.getM_case_UID( ), j, 9 );
            }
        }
    }

    /**
     * Adds a row to the table
     * @param elementToAdd is a clinical spectroscopy case worklist data element
     */
    public void addRow( ClinicalSpectroscopyCaseWorklistDataElement elementToAdd )
    {
        Object[] rowToAdd = new Object[ m_tableModel.getColumnCount( ) ];

        rowToAdd[0] = elementToAdd.getPatientName( );
        rowToAdd[1] = elementToAdd.getPatientDateOfBirth( );
        rowToAdd[2] = elementToAdd.getProcessingStatus( );
        rowToAdd[3] = elementToAdd.getSeriesDescription( );
        rowToAdd[4] = elementToAdd.getStudyDate( );
        rowToAdd[5] = elementToAdd.getSpectroscopyDataType( );
        rowToAdd[6] = elementToAdd.getScannerModel( );
        rowToAdd[7] = elementToAdd.getProcessingDateAndTime( );
        rowToAdd[8] = elementToAdd.getDicomFileReference( );
        rowToAdd[9] = elementToAdd.getM_case_UID( );

        m_tableModel.addRow( rowToAdd );
        setModel( m_tableModel );
        m_tableModel.fireTableDataChanged( );
        updateUI();
    }

    /**
     * returns whether the selected series contains image data or
     * not
     */
    public boolean isImageSeries( )
    {
        return m_isImageSeries;
    }

     /**
     * Returns the file references of all selected files from the table
     * @return
     */
    public Vector<String> getSelectedFileReferencesFromTable( )
    {
        m_fileSelectedSeriesFileReferences = new Vector<String>( );
        int[] selectedRows = this.getSelectedRows( );

        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_fileSelectedSeriesFileReferences.add( (String)getValueAt( selectedRows[i], 0 ) );
        }

        return m_fileSelectedSeriesFileReferences;
    }

    /**
     * Returns the TreeSet<String> contain the file references
     * in the selected series
     */
    public Vector<String> getSelectedSeriesFileReferences( )
    {
        return m_fileSelectedSeriesFileReferences;
    }


    /**
     * Clears the spectroscopy work list table
     */
    public void clearDicomSpectroWorklistTable( )
    {

        m_tableModel = new DefaultTableModel( );
        
        m_tableModel.addColumn( "<HTML><B>Patient name</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Date of Birth</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Processing status</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Spectroscopy data type</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Study date</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Scanner model</B></HTML>" );    
        m_tableModel.addColumn( "<HTML><B>Series description</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Processing date</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Dicom file reference</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Case UID</B></HTML>" );
        
        setModel( m_tableModel );

        updateUI( );
    }

    // Attributes
    private DefaultTableModel m_tableModel;

    private JPopupMenu m_popup;

    private JMenuItem m_selectAllItems;
    private JMenuItem m_deleteSelectedItems;
    
    private JMenuItem m_sortByPatientName;
    private JMenuItem m_sortByPatientDateOfBirth;   
    private JMenuItem m_sortByProcessingStatus;
    private JMenuItem m_sortBySpectroscopyDataType;
    private JMenuItem m_sortByStudyDate;
    private JMenuItem m_sortByScannerModel;
    private JMenuItem m_sortBySeriesDescription;
    private JMenuItem m_sortByProcessingDateAndTime;
    private JMenuItem m_sortByDicomFileReference;
    private JMenuItem m_sortBySpectroscopyCaseUID;
    
    // needed for a toggle soring functions
    private final boolean[] m_ascending = new boolean[10];

    // Contains the treeset of images belonging to a series study
    private Vector<String> m_fileSelectedSeriesFileReferences;

    // Object to store the list items of the image series data into
    Object[][] m_cellData;
    String[] m_columnNames;

    // variable indicating whether series contains image data or else
    boolean m_isImageSeries;

}