package org.spectrim.data;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;
import org.spectrim.model.GlobalModel;
import org.spectrim.grid.SaturationBand;
import org.spectrim.spectra.Spectra;
import org.spectrim.grid.CSIGrid;
import org.spectrim.imagegraphicobjects.Point3D;
import java.util.ArrayList;
import java.util.Arrays;
import org.dcm4che2.data.Tag;
import org.spectrim.geometricobjects.Cube3D;
import org.spectrim.main.MidasPreprocessedEPSIDataImporterParameterSettingsControl;
import org.spectrim.signalprocessing.FFT;
import org.spectrim.signalprocessing.SignalTools;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import org.spectrim.view.ImageSlice;
import org.spectrim.view.ImageStack;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
/**
 *
 * @author nunopbarros
 */
public class CSIData 
{

//  Spectra variables ################################################# 
    private byte[]  rawDataByte; //byte array from siemens format
    public int      m_numberOfTimeSamplePoints; // Number points per signal(spectrum)
    private double  m_resonanceFrequency; // Resonance frequency
    private double  ppmReference;
    private double  spectralWidth = 1023;
//  Both channel ( alternating abs, disp ) dispersion and absorption channel
//  in time domain (TD)
    protected float[] m_bothChannelsTD;
    protected float[] m_absorptionChannelTDraw;
    protected float[] m_dispersionChannelTDraw;
    public double[]   m_xaxisTD;
//  Both channel ( alternating abs, disp ) dispersion and absorption channel
//  in frequency domain (FD)
    public float[]  m_bothChannelsFD;
    public float[]  m_absorptionChannelFD;
    public float[]  m_dispersionChannelFD;
    public double[] m_xaxisFDppm;
    public double[] m_xaxisFDhz;
//  Excitation volume (Volume-of-Interest(VOI)) variables #################################################     
    public Cube3D   voi; //Contains the vertices of the VOI
    public Point3D  voiPosition = new Point3D(); // Position of the VOI
    private Point3D voiSize = new Point3D(); // Size of the VOI
    private Point3D voiV1 = new Point3D(); // VOI 1st Direction Vector 
    private Point3D voiV2 = new Point3D(); // VOI 2nd Direction Vector
    private Point3D voiV3 = new Point3D(); // VOI 3rd Direction Vector
    private double  voiInPlaneRotation;
//  Grid/Field-of-View(FOV) variables #################################################     
    private Point3D fovPosition = new Point3D(); // Jana: center of the grid
    public Point3D    fovSize; // Size of the FOV
    protected Point3D fovV1 = new Point3D(); // FOV 1st Direction Vector 
    protected Point3D fovV2 = new Point3D(); // FOV 2nd Direction Vector 
    protected Point3D fovV3 = new Point3D(); // FOV 3rd Direction Vector 
    private double    fovInPlaneRotation;
    
    private double    lScanRegionPosTra;
//  Grid: Number Voxels in each direction
    public int        nrVoxelsDir1; // sSpecPara.lFinalMatrixSizePhase
    public int        nrVoxelsDir2; // sSpecPara.lFinalMatrixSizeRead
    public int        nrVoxelsDir3; // sSpecPara.lFinalMatrixSizeSlice
    public int        nrSpectraPerVoxel = 1;
//  Background image variables ################################################# 
    public String     m_ref_imageUID1;
    public String     m_ref_imageUID2;
    public String     m_ref_imageUID3;
    public int        m_refSlice;
    public String     seriesUID;
//  Other variables
    public ArrayList<SaturationBand> saturationBands;
    public CSIGrid    csigrid;
    protected int     fileType;
    protected int     vectorlength;
    protected Spectra spectra;
    private GlobalModel model;
//  DICOM Info    
    private String    patientName;
    private String    patientID = "";
    private String    patientDOB = "unknown";
    private String    seriesDescription;
    private String    protocolName;
    private String    seriesNumber;
    private String    studyDate;
    private String    SOPInstanceUID;
    private String    manufacturer;
    private String    modelName;
    private String    nucleus;
    private double    B0;
    private double    TE;
    private double    TR;
// created by Jana
    private String modality;
    private String studyID;
    private String instanceNumber;
    private String studyDescription;
    private String accessionNumber = "Accession number not known";
    private String patientSex;
    private String studyTime;
    private String seriesTime;
    private String scannerName;
    
    private boolean FFTDone = false;
    private final   String epsi = "EPSI";
    private         String file;
    private int     m_sliceNumberToDisplay;
    
//    /**  
//     * Creates a CSIData object based on imported data from Midas.
//     * @param model
//     * @param midasEPSIdata
//     * @param dimensions
//     * @param sampleFrequency
//     * @param resonanceFrequency
//     * @param TE_EPSI
//     * @param TR_EPSI
//     * @param timeSamples
//     * @param sliceNumberToDisplay
//     */
//    public CSIData( GlobalModel model, float[] midasEPSIdata, 
//                    int[] dimensions, int timeSamples,
//                    float sampleFrequency, double resonanceFrequency,
//                    float TE_EPSI, float TR_EPSI, int sliceNumberToDisplay )
//    {
//
////        patientName = model.getPatientName( );
////        patientID   = model.getPatientID( );
////        patientDOB  = model.getPatientBirthDate( );
////        studyDate           = model.getStudyDate( );
////        seriesDescription   = model.getSeriesDescription( );
////        
////        seriesNumber        = model.getSeriesNumber( );
////        instanceNumber      = model.getInstanceNumber( );
////        protocolName        = model.getStudyDescription( );
////        SOPInstanceUID      = model.getSOPInstanceUID( );
////        manufacturer        = model.getManufacturer( );
////        modelName           = model.getModelName( );
////        
////        modality            = model.getModality( );
////        studyID             = model.getStudyID( );
////        instanceNumber      = model.getInstanceNumber( );
////        studyDescription    = model.getStudyDescription( );
////        if ( !manufacturer.contains( "Philips" ) )
////        {
////            accessionNumber     = model.getAccessionNumber( );
////            seriesDescription   = model.getSeriesDescription( );
////        }
////        
////        patientSex          = model.getPatientSex( );
////        studyTime           = model.getStudyTime( );
////        seriesTime          = model.getSeriesTime( );
////        scannerName         = model.getScannerModelName( );
//        
//        
//        patientName         = "Midas Preprocessed 7T EPSI data";
//        patientID           = "01234567890";
//        patientDOB          = "01/01/1970";
//        studyDate           = "01/01/2021";
//        
//        seriesDescription   = "EPSI 7T Midas preprocessed data";
//        
//        seriesNumber        = "-1";
//        instanceNumber      = "-1";;
//        protocolName        = "EPSI 7T";
//        SOPInstanceUID      = "1.1.1.1.1.1.1.1.1.1.1";
//        manufacturer        = "Siemens Healthineers Erlangen";
//        modelName           = "Terra";
//        
//        modality            = "MR";
//        studyID             = "09876543210";
//               
//        accessionNumber     = "-1";
//        
//        patientSex          = "O";
//        studyTime           = "00:00:00";
//        seriesTime          = "00:00:00";
//        scannerName         = "Terra 1Tx";
//              
//        // For Midas data the FFT is done
//        FFTDone              = false;
//        m_resonanceFrequency = resonanceFrequency;    
//        spectralWidth        = sampleFrequency;
//        B0                   = resonanceFrequency / 42.577480610E+06;
//                
//        TE                   = TE_EPSI;
//        TR                   = TR_EPSI;
//        
//        m_numberOfTimeSamplePoints = timeSamples;
//        ppmReference    = 3.0f;
//        nucleus         = "1H";
//        nrVoxelsDir2    = dimensions[1]; // Nr Rows    01.2021 added correct?
//        nrVoxelsDir1    = dimensions[0]; // Nr Columns 01.2021 added correct?
//        nrVoxelsDir3    = dimensions[2]; // Nr Frames  01.2021 added correct?
//        
//        
//        
//        this.model      = model;
//        m_sliceNumberToDisplay = sliceNumberToDisplay;
//        
////        int nrSlices = 1;                // correct? is water refs also included? 
//        
//        
////      m_ref_imageUID1 = readHeaderValue( headerMetaboliteMapMidas, "(0008,1140)[0]->(0008,1155)" );
////      m_ref_imageUID2 = readHeaderValue( headerMetaboliteMapMidas, "(0008,1140)[1]->(0008,1155)" );
////      m_ref_imageUID3 = readHeaderValue( headerMetaboliteMapMidas, "(0008,1140)[2]->(0008,1155)" );
//
////                DicomHeaderLine headerLine = (DicomHeaderLine) headerMetaboliteMapMidas.get( "(5600,0020)" );
////                rawDataByte = headerMetaboliteMapMidas.ValueByte;
//
//                
//
////                m_resonanceFrequency = Double.valueOf( readHeaderValue(headerMetaboliteMapMidas, "(0018,9098)" ) ) * 1000000;
////                ppmReference = Double.valueOf(readHeaderValue( headerMetaboliteMapMidas, "(0018,9053)") );
////                nucleus = readHeaderValue( headerMetaboliteMapMidas, "(0018,9100)");
//
////                nrVoxelsDir2 = Integer.valueOf( readHeaderValue(headerMetaboliteMapMidas, "(0028,0010)") ); // Nr Rows
////                nrVoxelsDir1 = Integer.valueOf( readHeaderValue(headerMetaboliteMapMidas, "(0028,0011)") ); // Nr Columns
////                nrVoxelsDir3 = Integer.valueOf( readHeaderValue(headerMetaboliteMapMidas, "(0028,0008)") ); // Nr Frames
//               
////                int nrSlices = Integer.valueOf( readHeaderValue( headerMetaboliteMapMidas, "(2001,1018)") ); //Philips format
//        
//
////        // verify whether this is correct for 3D EPSI data
////        if ( nrSlices > 0 ) 
////        {
////            nrSpectraPerVoxel = nrVoxelsDir3 / nrSlices;
////            nrVoxelsDir3 = nrSlices;
////        } 
////        else 
////        {
////            nrSpectraPerVoxel = 1;
////        }
//             
////            NOTE: Currently if a tag has more than 2 levels, only the last two are used as key in the header object.
////            THIS SHOULD CHANGE! Example: tag (5200,9229)->(0028,9110)->(0018,0050) uses the key (0028,9110)->(0018,0050)
//                //FOV variables - information contained in group tags 5200,9229 and 5200,9230:
////            (5200,9229)->(0020,9116)	Plane Orientation Sequence
////            (5200,9229)->(0020,9116)->(0020,0037)	Image Orientation (Patient)
////            (5200,9229)->(0028,9110)	Pixel Measures Sequence
////            (5200,9229)->(0028,9110)->(0018,0050)	Slice Thickness
////            (5200,9229)->(0028,9110)->(0028,0030)	Pixel Spacing
////            (5200,9230)	Per-frame Functional Groups Sequence
////            (5200,9230)->(0020,9111)	Frame Content Sequence
////            (5200,9230)->(0020,9111)->(0018,9074)	Frame Acquisition DateTime
////            (5200,9230)->(0020,9111)->(0018,9151)	Frame Reference DateTime
////            (5200,9230)->(0020,9111)->(0018,9220)	Frame Acquisition Duration
////            (5200,9230)->(0020,9111)->(0020,9156)	Frame Acquisition Number
////            (5200,9230)->(0020,9113)	Plane Position Sequence
////            (5200,9230)->(0020,9113)->(0020,0032)	Image Position (Patient)
//                //VOI variables - tags grouped in tag 0018,9126
////            (0018,9126)	Volume Localization Sequence
////            (0018,9126)[0]->(0018,9104)	Slab Thickness
////            (0018,9126)[0]->(0018,9105)	Slab Orientation
////            (0018,9126)[0]->(0018,9106)	Mid Slab Position
////            (0018,9126)[1]->(0018,9104)	Slab Thickness
////            (0018,9126)[1]->(0018,9105)	Slab Orientation
////            (0018,9126)[1]->(0018,9106)	Mid Slab Position
////            (0018,9126)[2]->(0018,9104)	Slab Thickness
////            (0018,9126)[2]->(0018,9105)	Slab Orientation
////            (0018,9126)[2]->(0018,9106)	Mid Slab Position
//                
//        voiSize = new Point3D( );
//         
//        voiSize.m_x1 = model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).pixelSpacingX * nrVoxelsDir1;
//        voiSize.m_x2 = model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).pixelSpacingY * nrVoxelsDir2;
//        voiSize.m_x3 = model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).sliceThickness * nrVoxelsDir3;
//        
////        voiSize.m_x3 = model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).sliceThickness;
//
////        voiSize.m_x1 = model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingX;
////        voiSize.m_x2 = model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingY;
////        voiSize.m_x3 = model.getImageStack( ).getImages( ).get( 0 ).sliceThickness;
//
//        voiPosition = model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).imagePosition;
//        
//        for ( int i = 0; i < model.getImageStack( ).getImages( ).size( ); i++ )
//        {
//            System.out.println( "Image position pos slice : " + i + " = [ " + 
//                                model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x1 + " , " + 
//                                model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x2 + " , " + 
//                                model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x3 + " ] " );  
//        }
//        
//        voiPosition = model.getImageStack( ).getImages( ).get( 0 ).imagePosition;
//         
//        
//        voiV1 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationX;
//        voiV2 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationY;
//        
//        // direction vector in 3rd direction should be perpendicular to the voiV1, voiV2
//        voiV3 = Point3D.crossProduct( voiV1, voiV2 );
//        
////        voiSize.m_x1 = Double.valueOf( readHeaderValue( headerMetaboliteMapMidas, "(0018,9126)[0]->(0018,9104)" ) );
////        voiSize.m_x2 = Double.valueOf( readHeaderValue( headerMetaboliteMapMidas, "(0018,9126)[1]->(0018,9104)" ) );
////        voiSize.m_x3 = Double.valueOf( readHeaderValue( headerMetaboliteMapMidas, "(0018,9126)[2]->(0018,9104)" ) );        
////        String[] positionTxt = readHeaderValue( headerMetaboliteMapMidas, "(0018,9126)[0]->(0018,9106)").split("\\\\" );
//                
////                if ( positionTxt.length == 3 )
////                {
////                    voiPosition = new Point3D(Double.valueOf( positionTxt[0]), Double.valueOf( positionTxt[1]), Double.valueOf(positionTxt[2] ) );
////                }
////                else
////                {
////                    System.out.println( "Voxel postion could not be read!" );
////                    positionTxt = new String[3];
////                    positionTxt[0] = "0.0";
////                    positionTxt[1] = "0.0";
////                    positionTxt[2] = "0.0";
////                }
//                
////                String[] v1Txt = readHeaderValue( headerMetaboliteMapMidas, "(0018,9126)[0]->(0018,9105)").split("\\\\");
////                String[] v2Txt = readHeaderValue( headerMetaboliteMapMidas, "(0018,9126)[1]->(0018,9105)").split("\\\\");
////                String[] v3Txt = readHeaderValue( headerMetaboliteMapMidas, "(0018,9126)[2]->(0018,9105)").split("\\\\");
////                if ( v1Txt.length == 3 )
////                {
////                    voiV1 = new Point3D( Double.valueOf( v1Txt[0] ), 
////                                         Double.valueOf( v1Txt[1] ), 
////                                         Double.valueOf( v1Txt[2] ) );
////                }
////                else
////                {
////                    voiV1 = new Point3D( 0.0, 0.0, 0.0 );
////                }
////                
////                if ( v2Txt.length == 3 )
////                {
////                    voiV2 = new Point3D( Double.valueOf( v2Txt[0] ), 
////                                         Double.valueOf( v2Txt[1] ), 
////                                         Double.valueOf( v2Txt[2] ) );
////                }
////                else
////                {
////                    voiV2 = new Point3D( 0.0, 0.0, 0.0 );
////                }
////                
////                if ( v3Txt.length == 3 )
////                {
////                    voiV3 = new Point3D( Double.valueOf( v3Txt[0] ), 
////                                         Double.valueOf( v3Txt[1] ), 
////                                         Double.valueOf( v3Txt[2] ) );
////                }
////                else
////                {
////                    voiV3 = new Point3D( 0.0, 0.0, 0.0 );
////                }
// 
////                String[] pixelSpacingTxt = readHeaderValue( headerMetaboliteMapMidas, "(0028,9110)->(0028,0030)" ).split( "\\\\" );
////                double sliceThickness = Double.valueOf( readHeaderValue( headerMetaboliteMapMidas, "(0028,9110)->(0018,0050)" ) );
//                
//                Point3D voxelSpacing = new Point3D( model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).pixelSpacingX, 
//                                                    model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).pixelSpacingY, 
//                                                    model.getImageStack( ).getImages( ).get( m_sliceNumberToDisplay ).sliceThickness );
//
//                fovSize      = new Point3D( );
//                fovSize.m_x1 = voxelSpacing.m_x1 * nrVoxelsDir1; // Adjacent row spacing
//                fovSize.m_x2 = voxelSpacing.m_x2 * nrVoxelsDir2; // Adjacent column spacing
//                fovSize.m_x3 = voxelSpacing.m_x3 * nrVoxelsDir3;
//                
////                fovSize.m_x1 = voxelSpacing.m_x1 ; // Adjacent row spacing
////                fovSize.m_x2 = voxelSpacing.m_x2 ; // Adjacent column spacing
////                fovSize.m_x3 = voxelSpacing.m_x3 ;
//
//
////                String[] orientationFOV = readHeaderValue( headerMetaboliteMapMidas, "(0020,9116)->(0020,0037)").split("\\\\" );
//
// 
//                
////                fovV1 = new Point3D( Double.valueOf( orientationFOV[0] ), 
////                                     Double.valueOf( orientationFOV[1] ), 
////                                     Double.valueOf( orientationFOV[2] ) );
////                fovV2 = new Point3D( Double.valueOf( orientationFOV[3] ), 
////                                     Double.valueOf( orientationFOV[4] ), 
////                                     Double.valueOf( orientationFOV[5] ) );
//                
////                fovV3 = Point3D.crossProduct( fovV1, fovV2 ); //TODO: Needs Phantom Validation
//                
//                fovV1 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationX;
//                fovV2 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationY;
//                fovV3 = Point3D.crossProduct( fovV2, fovV1 ); //test
//                
////              The position of the FOV is given by the conventional Image Position tag. This tag gives the position of the center
////              of the top left voxel. This is not the case in Siemens, where we obtain directly the center of the FOV. Therefore here
////              we need to calculate the center of the FOV.
////                String[] firstPixelPos = readHeaderValue( headerMetaboliteMapMidas, "(0020,9113)->(0020,0032)").split("\\\\" );
////                fovPosition = new Point3D( Double.valueOf( firstPixelPos[0] ), 
////                                           Double.valueOf( firstPixelPos[1] ), 
////                                           Double.valueOf( firstPixelPos[2] ) );
////                
////                fovPosition.translate( Point3D.vectorCombination( fovV1, fovV2, fovV3, 
////                                                                  voxelSpacing.m_x1 * ( nrVoxelsDir1 - 1 ) / 2,
////                                                                  voxelSpacing.m_x2 * ( nrVoxelsDir2 - 1 ) / 2, 
////                                                                  voxelSpacing.m_x3 * ( nrVoxelsDir3 - 1 ) / 2 ) );
//
//                // For EPSI we try now to copy the topleft position of the 
//                // first image.
////                fovPosition = model.getImageStack( ).getImages( ).get( 0 ).imagePosition;
////                
////                fovPosition = new Point3D( 0.0, 0.0 , fovPosition.m_x3 );
//
//                // fovPosition = new Point3D( 0.0, 0.0 , 0.0 );
//                
//                if ( ( voiSize.m_x1 == 0.0 ) && 
//                     ( voiSize.m_x2 == 0.0 ) &&
//                     ( voiSize.m_x3 == 0.0 ) )
//                {
//                    voiSize.m_x1 = fovSize.m_x1;
//                    voiSize.m_x2 = fovSize.m_x2;
//                    voiSize.m_x3 = fovSize.m_x3;      
//                }
//
//// we do not have saturation bands for the time being ...
////                String tagaux;
////                String slabOrient[];
////                String slabPosit[];
////                double slabThick;
////
////                for ( int i = 0; i < 15; i++ ) 
////                {
////                    tagaux = "(0018,9107)[" + i + "]->(0018,9104)"; //Slab thickness
////                    slabThick = Double.valueOf( readHeaderValue( headerMetaboliteMapMidas, tagaux ) );
////                    if ( slabThick > 0 ) 
////                    {
////                        tagaux = "(0018,9107)[" + i + "]->(0018,9105)"; //Slab orientation
////                        slabOrient = readHeaderValue( headerMetaboliteMapMidas, tagaux ).split( "\\\\" );
////                        tagaux = "(0018,9107)[" + i + "]->(0018,9106)"; //Mid slab position
////                        slabPosit = readHeaderValue( headerMetaboliteMapMidas, tagaux).split("\\\\");
////                        saturationBands.add( new SaturationBand( slabThick,
////                                             Double.valueOf( slabPosit[0] ),
////                                             Double.valueOf( slabPosit[1] ),
////                                             Double.valueOf( slabPosit[2] ),
////                                             Double.valueOf( slabOrient[0] ),
////                                             Double.valueOf( slabOrient[1] ),
////                                             Double.valueOf( slabOrient[2] ) ) );
////                    }
////                }
////        }
//
////        if ( nrVoxelsDir1 == 0 ) 
////        {
////            nrVoxelsDir1 = 1;
////        }
////        
////        if ( nrVoxelsDir2 == 0 ) 
////        {
////             nrVoxelsDir2 = 1;
////        }
////        
////        if ( nrVoxelsDir3 == 0 ) 
////        {
////            nrVoxelsDir3 = 1;
////        }
//
//        // Create excited volume voxel
//        double dim1 = voiSize.m_x1 / 2;
//        double dim2 = voiSize.m_x2 / 2;
//        double dim3 = voiSize.m_x3 / 2;
//        
//
//        
//        Point3D pc1 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, -dim3 );
//        Point3D pc2 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, dim3 );
//        
////        Point3D center = new Point3D( ( pc1.m_x1 + pc2.m_x1 )/2.0, 
////                                      ( pc1.m_x2 + pc2.m_x2 )/2.0,
////                                      ( pc2.m_x3 ) );
//        
//         Point3D center = new Point3D( ( pc1.m_x1 + pc2.m_x1 )/2.0, 
//                                       ( pc1.m_x2 + pc2.m_x2 )/2.0,
//                                       ( pc1.m_x3 + pc2.m_x3 )/2.0 );
//        
//         
////         Point3D center = new Point3D( 0.0, 0.0, 0.0 );
//         
//        // now the grid can be computed!
////        csigrid = new CSIGrid( fovPosition, fovV1, fovV2, fovV3,
////                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3,
////                               fovSize.m_x1, fovSize.m_x2, fovSize.m_x3 );
//
//        Point3D shift = new Point3D( 0.0, 0.0, 0.0 );
//        
//        csigrid = new CSIGrid( shift, center, fovV1, fovV2, fovV3,
//                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3,
//                               fovSize.m_x1, fovSize.m_x2, fovSize.m_x3 );
//
//        Point3D p5 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, -dim3 );
//        Point3D p3 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, dim3 );
//        Point3D p6 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, -dim2, -dim3 );
//        Point3D p1 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, dim2, -dim3 );
//        Point3D p8 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, dim3 );
//        Point3D p2 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, -dim3 );
//        Point3D p4 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, dim2, dim3 );
//        Point3D p7 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, -dim2, dim3 );
//        
//        voiPosition = new Point3D( 0.0, 0.0, -voiPosition.m_x3 );
//        
//        p1.translate( voiPosition );
//        p2.translate( voiPosition );
//        p3.translate( voiPosition );
//        p4.translate( voiPosition );
//        p5.translate( voiPosition );
//        p6.translate( voiPosition );
//        p7.translate( voiPosition );
//        p8.translate( voiPosition );
//
//        voi = new Cube3D( p1, p2, p3, p4, p5, p6, p7, p8 );
//
//        csigrid.setExcitationVoxel( voi );
//
//// No saturation bands in this version ...        
////        for ( int i = 0; i < saturationBands.size( ); i++ ) 
////        {
////            csigrid.addSaturationBand( saturationBands.get( i ).p1, 
////                                       saturationBands.get( i ).p2, 
////                                       saturationBands.get( i ).p3, 
////                                       saturationBands.get( i ).p4, 
////                                       saturationBands.get( i ).p5,
////                                       saturationBands.get( i ).p6, 
////                                       saturationBands.get( i ).p7, 
////                                       saturationBands.get( i ).p8);
////        }
//
//        // Midas data are available in FD. Calss Spectra requires TD.
//
//        int cntr = 0;
//        
//        m_absorptionChannelTDraw = new float[ midasEPSIdata.length / 2  ];
//        m_dispersionChannelTDraw = new float[ midasEPSIdata.length / 2 ];
//        
//        float[] dummyAbsFD  = new float[timeSamples];
//        float[] dummyDispFD = new float[timeSamples];
////        float[] dummyAbsTD  = new float[timeSamples];
////        float[] dummyDispTD = new float[timeSamples];
//
//        int totalNumberOfSpectra = nrVoxelsDir1 * nrVoxelsDir2 * nrVoxelsDir3;
//        
//        for ( int k = 0; k < totalNumberOfSpectra; k++ )
//        {
//            if( k % 2 == 0 )
//            {      
//                for ( int i = 0; i < timeSamples; i++ )
//                {
//                    dummyAbsFD[i]  = midasEPSIdata[ ( k * timeSamples ) + 2 * i ];
//                    dummyDispFD[i] = midasEPSIdata[ ( k * timeSamples ) + 2 * i + 1 ];     
//                }
//            }
//            else
//            { 
//                float[] abso = new float[timeSamples];
//                float[] disp = new float[timeSamples];
//                
//                for ( int i = 0; i < timeSamples; i++ )
//                {
//                    abso[i] = midasEPSIdata[ ( k * timeSamples ) + 2 * i ];
//                    disp[i] = midasEPSIdata[ ( k * timeSamples ) + 2 * i + 1 ];
//                }
//                
//                for ( int i = 0; i < timeSamples; i++ )
//                {
//                    dummyAbsFD[i]  = abso[ ( i + ( timeSamples / 2 ) - 1 ) % ( timeSamples )];
//                    dummyDispFD[i] = disp[ ( i + ( timeSamples / 2 ) - 1 ) % ( timeSamples )];
//                }
//            }
//            
//            float[][] vaux = new float[timeSamples][2];
//            
//            vaux = FD2TD_EPSI( dummyAbsFD, dummyDispFD );
//            
//            for ( int t = 0; t < timeSamples; t++ )  
//            {
//                m_absorptionChannelTDraw[ k * timeSamples + t ] = vaux[t][0] / timeSamples;
//                m_dispersionChannelTDraw[ k * timeSamples + t ] = -vaux[t][1] / timeSamples;
//            }
//            
//            // for ( int i= 0; i< )
//            
//            
////            if ( ( k > totalNumberOfSpectra / 3 ) && ( k < 20 + totalNumberOfSpectra / 3 ) )
////            {
////                XYPlotFrame xypf = new XYPlotFrame( "Check FD signal - " + k, null );
////                float[] xaxis = new float[dummyAbsTD.length];
////                for ( int i = 0; i < dummyDispTD.length; i++ )
////                {
////                    xaxis[i] = (float)i;
////                }
////                xypf.addCurve( new Curve( xaxis, dummyAbsTD ), true );
////            }
//            
//            
//        }
//        
//        float[][][][] fourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
//        float[][][][] fourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
//        
//        // Midas has a different ordering of the data than spectrIm expects.
//        // Build an easy to reformat data structure
//        cntr=0;
//        
//        for ( int i = 0; i < nrVoxelsDir1; i++ )
//        {
//            for ( int j = 0; j < nrVoxelsDir2; j++ )
//            {
//                for ( int k = 0; k < nrVoxelsDir3; k++ )
//                {
//                    for ( int t = 0; t < timeSamples; t++ )
//                    {
//                        
//                        fourDdataReal[i][j][k][t] = m_absorptionChannelTDraw[ cntr ];
//                        fourDdataImag[i][j][k][t] = m_dispersionChannelTDraw[ cntr ];
//                        cntr++;
//                    }
//                }
//            }
//        }
//        
//        // now put it in different order
//        cntr=0;
//        
//        for ( int k = 0; k < nrVoxelsDir3; k++ )
//        {
//            for ( int i = 0; i < nrVoxelsDir1; i++ )
//            {
//                for ( int j = 0; j < nrVoxelsDir2; j++ )
//                {
//                    for ( int t = 0; t < timeSamples; t++ )
//                    {
//                        m_absorptionChannelTDraw[ cntr ] = fourDdataReal[i][j][k][t];
//                        m_dispersionChannelTDraw[ cntr ] = fourDdataImag[i][j][k][t];
//                        cntr++;
//                    }
//                }
//            }
//        }
//        
//        fourDdataReal = null;
//        fourDdataImag = null;
//        
//        spectra = new Spectra( m_absorptionChannelTDraw, 
//                               m_dispersionChannelTDraw,
//                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, 
//                               spectralWidth, m_resonanceFrequency, 
//                               ppmReference, FFTDone );
//        
//        // model = new GlobalModel( );
//        
//        model.setCSIData( this );
//        model.setSpectra( spectra );
//        model.setCSIGrid( csigrid );
//
//    }
    
    private float[][] TD2FD_EPSI( float[] abs, float[] disp ) 
    {
        float[][] vaux = new float[abs.length][2];
        
        FFT fft = new FFT( );
        
        for ( int m = 0; m < abs.length; m++ ) 
        {
            vaux[m][0] = abs[m];
            vaux[m][1] = disp[m];
        }

        vaux = fft.fft_1d( vaux );
        
        float[] absorptionFD = new float[abs.length];
        float[] dispersionFD = new float[abs.length];
        
        for ( int t = 0; t < (abs.length); t++ ) 
        {
            int d = ( t + (abs.length) / 2 ) % ( abs.length );
            absorptionFD[d] = (float) vaux[t][0];
            dispersionFD[d] = (float) vaux[t][1];
        }
        
        for ( int m = 0; m < abs.length; m++ )
        {
            vaux[m][0] = absorptionFD[m];
            vaux[m][1] = dispersionFD[m];
        }
        
        return vaux;
    }
    
    
    
    
    
    public float[][] FD2TD_EPSI( float[] abs, float[] disp ) 
    {
        float[][] vaux = new float[abs.length][2];

        for ( int m = 0; m < abs.length; m++ ) 
        {
            int d = (m + 1 + abs.length / 2) % abs.length;
            
            vaux[d][0] = abs[abs.length - 1 - m];
            vaux[d][1] = disp[disp.length - 1 - m];
        }
        
        FFT fft = new FFT( );
        vaux = fft.fft_1d( vaux );

        return vaux;
    }
    
    
//    /**  
//     * Creates a CSIData object based on imported data from Midas.
//     * @param model
//     * @param midasEPSIdata
//     * @param dimensions
//     * @param sampleFrequency
//     * @param resonanceFrequency
//     * @param TE_EPSI
//     * @param TR_EPSI
//     * @param timeSamples
//     * @param sliceNumberToDisplay
//     */
//    public CSIData( GlobalModel model, float[] midasEPSIdata, 
//                    int[] dimensions, int timeSamples,
//                    float sampleFrequency, double resonanceFrequency,
//                    float TE_EPSI, float TR_EPSI, int sliceNumberToDisplay )
//    {
//        
//        patientName         = "Midas Preprocessed 7T EPSI data";
//        patientID           = "01234567890";
//        patientDOB          = "01-01-1970";
//        studyDate           = "01-01-2021";
//        
//        seriesDescription   = "EPSI-7T-Midas-preprocessed";
//        
//        seriesNumber        = "-1";
//        instanceNumber      = "-1";;
//        protocolName        = "EPSI 7T";
//        SOPInstanceUID      = "1.1.1.1.1.1.1.1.1.1.1";
//        manufacturer        = "Siemens Healthineers Erlangen";
//        modelName           = "Terra";
//        
//        modality            = "MR";
//        studyID             = "09876543210";
//               
//        accessionNumber     = "-1";
//        
//        patientSex          = "O";
//        studyTime           = "00:00:00";
//        seriesTime          = "00:00:00";
//        scannerName         = "Terra 1Tx";
//              
//        // For Midas data the FFT is done
//        FFTDone              = false;
//        m_resonanceFrequency = resonanceFrequency;    
//        spectralWidth        = sampleFrequency;
//        B0                   = resonanceFrequency / 42.577480610E+06;
//                
//        TE                   = TE_EPSI;
//        TR                   = TR_EPSI;
//        
//        m_numberOfTimeSamplePoints = timeSamples;
//        ppmReference    = 3.0f;
//        nucleus         = "1H";
//        
//        nrVoxelsDir1    = dimensions[0]; // Nr Columns 01.2021 added correct?
//        nrVoxelsDir2    = dimensions[1]; // Nr Rows    01.2021 added correct?
//        nrVoxelsDir3    = dimensions[2]; // Nr Frames  01.2021 added correct?
//        
//        this.model      = model;
//        m_sliceNumberToDisplay = sliceNumberToDisplay;
//        
//        // int nrSlices = 1;                // correct? is water refs also included? 
//        
//        
////      m_ref_imageUID1 = readHeaderValue( headerMetaboliteMapMidas, "(0008,1140)[0]->(0008,1155)" );
////      m_ref_imageUID2 = readHeaderValue( headerMetaboliteMapMidas, "(0008,1140)[1]->(0008,1155)" );
////      m_ref_imageUID3 = readHeaderValue( headerMetaboliteMapMidas, "(0008,1140)[2]->(0008,1155)" );
//
//        
//        // EPSI single voxel size = same size as MRSI map
//        System.out.println( "pixel size = [ " + model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingX + " , " + 
//                                                model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingY + " , " + 
//                                                model.getImageStack( ).getImages( ).get( 0 ).sliceThickness + " ] " );
//
//        voiSize = new Point3D( );
//         
//        voiSize.m_x1 = model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingX * nrVoxelsDir1;
//        voiSize.m_x2 = model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingY * nrVoxelsDir2;
//        voiSize.m_x3 = model.getImageStack( ).getImages( ).get( 0 ).sliceThickness * nrVoxelsDir3;
//        
//        System.out.println( "voiSize (total excited volume)  = [ " + voiSize.m_x1 + " , " + 
//                                                                     voiSize.m_x2 + " , " + 
//                                                                     voiSize.m_x3 + " ] " );  
//        
//        System.out.println( "Image[0] position " + " = [ " + model.getImageStack( ).getImages( ).get( 0 ).imagePosition.m_x1 + " , " + 
//                                                             model.getImageStack( ).getImages( ).get( 0 ).imagePosition.m_x2 + " , " + 
//                                                             model.getImageStack( ).getImages( ).get( 0 ).imagePosition.m_x3 + " ] " );  
//        
//        
//        double minCoordinateDimension1 = Float.POSITIVE_INFINITY;
//        double maxCoordinateDimension1 = -Float.POSITIVE_INFINITY;
//        double minCoordinateDimension2 = Float.POSITIVE_INFINITY;
//        double maxCoordinateDimension2 = -Float.POSITIVE_INFINITY;
//        double minCoordinateDimension3 = Float.POSITIVE_INFINITY;
//        double maxCoordinateDimension3 = -Float.POSITIVE_INFINITY;
//        
//        
//        
//        ////// TEST CODE NUNO /////        
//        voiV1 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationX;
//        voiV2 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationY;
//        // direction vector in 3rd direction should be perpendicular to the voiV1, voiV2
//        voiV3 = Point3D.crossProduct( voiV1, voiV2 );        
//        fovV1 = voiV1.Clone();
//        fovV2 = voiV2.Clone();
//        fovV3 = voiV3.Clone();
//        
//        Point3D voxelSpacing = new Point3D( model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingX, 
//                                            model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingY, 
//                                            model.getImageStack( ).getImages( ).get( 0 ).sliceThickness );
//
//        fovSize      = new Point3D( );
//        fovSize.m_x1 = voxelSpacing.m_x1 * nrVoxelsDir1; // Adjacent row spacing
//        fovSize.m_x2 = voxelSpacing.m_x2 * nrVoxelsDir2; // Adjacent column spacing
//        fovSize.m_x3 = voxelSpacing.m_x3 * nrVoxelsDir3;
//        
//        Point3D center = model.getImageStack( ).getImages( ).get( 0 ).imagePosition;
//        Point3D shift = Point3D.vectorCombination(voiV1, voiV2, voiV3, (nrVoxelsDir1-1)/2, (nrVoxelsDir2-1)/2, (nrVoxelsDir3-1)/2);
//        center.addPoint(shift);
//
//        
//        csigrid = new CSIGrid( center, fovV1, fovV2, fovV3,
//                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3,
//                               fovSize.m_x1, fovSize.m_x2, fovSize.m_x3 );
//        /////////////
//        
//        for ( int i = 0; i < model.getImageStack( ).getImages( ).size( ); i++ )
//        {
//            
//            System.out.println( "Image position pos slice : " + i + " = [ " + 
//                                model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x1 + " , " + 
//                                model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x2 + " , " + 
//                                model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x3 + " ] " );
//            
//            if ( minCoordinateDimension1 > model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x1 )
//            {
//                minCoordinateDimension1 = model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x1;
//            }
//            
//            if ( maxCoordinateDimension1 < model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x1 )
//            {
//                maxCoordinateDimension1 = model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x1;
//            }
//            
//            if ( minCoordinateDimension2 > model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x2 )
//            {
//                minCoordinateDimension2 = model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x2;
//            }
//            
//            if ( maxCoordinateDimension2 < model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x2 )
//            {
//                maxCoordinateDimension2 = model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x2;
//            }
//            
//            if ( minCoordinateDimension3 > model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x3 )
//            {
//                minCoordinateDimension3 = model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x3;
//            }
//            
//            if ( maxCoordinateDimension3 < model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x3 )
//            {
//                maxCoordinateDimension3 = model.getImageStack( ).getImages( ).get( i ).imagePosition.m_x3;
//            }
//            
//        }
//        
//        System.out.println( "minCoordinateDimension1 = " + minCoordinateDimension1 );
//        System.out.println( "maxCoordinateDimension1 = " + maxCoordinateDimension1 );
//        System.out.println( "minCoordinateDimension2 = " + minCoordinateDimension2 );
//        System.out.println( "maxCoordinateDimension2 = " + maxCoordinateDimension2 );
//        System.out.println( "minCoordinateDimension3 = " + minCoordinateDimension3 );
//        System.out.println( "maxCoordinateDimension3 = " + maxCoordinateDimension3 );
//        
////        double shift_z = 0.0 * model.getImageStack( ).getImages( ).get( 0 ).sliceThickness;
////        
////        voiV1 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationX;
////        voiV2 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationY;
////        
////        // direction vector in 3rd direction should be perpendicular to the voiV1, voiV2
////        voiV3 = Point3D.crossProduct( voiV1, voiV2 );
//////        voiV3 = Point3D.crossProduct( voiV2, voiV1 );
////        
////        Point3D voxelSpacing = new Point3D( model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingX, 
////                                            model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingY, 
////                                            model.getImageStack( ).getImages( ).get( 0 ).sliceThickness );
////
////        fovSize      = new Point3D( );
////        fovSize.m_x1 = voxelSpacing.m_x1 * nrVoxelsDir1; // Adjacent row spacing
////        fovSize.m_x2 = voxelSpacing.m_x2 * nrVoxelsDir2; // Adjacent column spacing
////        fovSize.m_x3 = voxelSpacing.m_x3 * nrVoxelsDir3;
////                
////                
////        fovV1 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationX;
////        fovV2 = model.getImageStack( ).getImages( ).get( 0 ).imageOrientationY;
////        //fovV3 = Point3D.crossProduct( fovV2, fovV1 ); // test
////        fovV3 = Point3D.crossProduct( fovV1, fovV2 );        
////        if ( ( voiSize.m_x1 == 0.0 ) && 
////             ( voiSize.m_x2 == 0.0 ) &&
////             ( voiSize.m_x3 == 0.0 ) )
////        {
////            voiSize.m_x1 = fovSize.m_x1;
////            voiSize.m_x2 = fovSize.m_x2;
////            voiSize.m_x3 = fovSize.m_x3;      
////        }
////        
////        voiSize = fovSize.Clone();
////        
////        // Create excited voxel
//        double dim1 = voiSize.m_x1 / 2;
//        double dim2 = voiSize.m_x2 / 2;
//        double dim3 = voiSize.m_x3 / 2;
////
////        Point3D pc1 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, -dim3 );
////        Point3D pc2 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, dim3 );
////        
////        System.out.println( "pc1 " + " = [ " + pc1.m_x1 + " , " + pc1.m_x2 + " , " + pc1.m_x3 + " ] " );  
////        System.out.println( "pc2 " + " = [ " + pc2.m_x1 + " , " + pc2.m_x2 + " , " + pc2.m_x3 + " ] " );  
////        
//////        Point3D center = new Point3D( ( pc1.m_x1 + pc2.m_x1 )/2.0, 
//////                                      ( pc1.m_x2 + pc2.m_x2 )/2.0,
//////                                      ( pc2.m_x3 ) );
////        
////         Point3D center = new Point3D( ( pc1.m_x1 + pc2.m_x1 )/2.0, 
////                                       ( pc1.m_x2 + pc2.m_x2 )/2.0,
////                                       ( pc1.m_x3 + pc2.m_x3 )/2.0 );
////        
////         //Now add position first image
////         center.addPoint(model.getImageStack( ).getImages( ).get( 0 ).imagePosition);
////         
////        //center = new Point3D(0.0,0.0,84.4244);
////         
////        System.out.println( "Voi center " + " = [ " + center.m_x1 + " , " + 
////                                                      center.m_x2 + " , " + 
////                                                      center.m_x3 + " ] " );  
////         
//////         Point3D center = new Point3D( 0.0, 0.0, 0.0 );
////         
////        // now the grid can be computed!
//////        csigrid = new CSIGrid( fovPosition, fovV1, fovV2, fovV3,
//////                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3,
//////                               fovSize.m_x1, fovSize.m_x2, fovSize.m_x3 );
////
//////        Point3D shift = new Point3D( 0.0, 0.0, shift_z );
////        Point3D shift = new Point3D( 0.0, 0.0, 0.0 );
//////        csigrid = new CSIGrid( shift, center, fovV1, fovV2, fovV3,
//////                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3,
//////                               fovSize.m_x1, fovSize.m_x2, fovSize.m_x3 );
////        csigrid = new CSIGrid( center, fovV1, fovV2, fovV3,
////                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3,
////                               fovSize.m_x1, fovSize.m_x2, fovSize.m_x3 );
//
//        Point3D p5 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, -dim3 );
//        Point3D p3 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, dim3 );
//        Point3D p6 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, -dim2, -dim3 );
//        Point3D p1 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, dim2, -dim3 );
//        Point3D p8 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, dim3 );
//        Point3D p2 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, -dim3 );
//        Point3D p4 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, dim2, dim3 );
//        Point3D p7 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, -dim2, dim3 );
//       
////        voiPosition = new Point3D( 0.0, 0.0, shift_z );
//        voiPosition = center.Clone();
//        
//        p1.translate( voiPosition );
//        p2.translate( voiPosition );
//        p3.translate( voiPosition );
//        p4.translate( voiPosition );
//        p5.translate( voiPosition );
//        p6.translate( voiPosition );
//        p7.translate( voiPosition );
//        p8.translate( voiPosition );
//
//        System.out.println( "Excited volume p1 = [ " + p4.m_x1 + " , " + p4.m_x2 + " , " + p4.m_x3 + " ] " );  
//        System.out.println( "Excited volume p2 = [ " + p3.m_x1 + " , " + p3.m_x2 + " , " + p3.m_x3 + " ] " );  
//        System.out.println( "Excited volume p3 = [ " + p2.m_x1 + " , " + p2.m_x2 + " , " + p2.m_x3 + " ] " );  
//        System.out.println( "Excited volume p4 = [ " + p1.m_x1 + " , " + p1.m_x2 + " , " + p1.m_x3 + " ] " );  
//        System.out.println( "Excited volume p5 = [ " + p8.m_x1 + " , " + p8.m_x2 + " , " + p8.m_x3 + " ] " );  
//        System.out.println( "Excited volume p6 = [ " + p7.m_x1 + " , " + p7.m_x2 + " , " + p7.m_x3 + " ] " );  
//        System.out.println( "Excited volume p7 = [ " + p6.m_x1 + " , " + p6.m_x2 + " , " + p6.m_x3 + " ] " );  
//        System.out.println( "Excited volume p8 = [ " + p5.m_x1 + " , " + p5.m_x2 + " , " + p5.m_x3 + " ] " );  
//        
//        // voi = new Cube3D( p1, p2, p3, p4, p5, p6, p7, p8 );
//
//        voi = new Cube3D( p4, p3, p2, p1, p8, p7, p6, p5 );
//        
//        csigrid.setExcitationVoxel( voi );
//
//        // Midas data are available in FD. Calss Spectra requires TD.
//
//        int cntr = 0;
//        
//        m_absorptionChannelTDraw = new float[ midasEPSIdata.length / 2  ];
//        m_dispersionChannelTDraw = new float[ midasEPSIdata.length / 2 ];
//        
//        for ( int i=0;i<midasEPSIdata.length/2;i++)
//        {
//            m_absorptionChannelTDraw[i] = midasEPSIdata[2*i];
//            m_dispersionChannelTDraw[i] = midasEPSIdata[2*i+1];
//        }
//        
//        float[] dummyAbsFD  = new float[timeSamples];
//        float[] dummyDispFD = new float[timeSamples];
//
//        int totalNumberOfSpectra = nrVoxelsDir1 * nrVoxelsDir2 * nrVoxelsDir3;
//        
//
//        float[][][][] fourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
//        float[][][][] fourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
//        
////        // Midas has a different ordering of the data than spectrIm expects.
////        // Build an easy to reformat data structure         
//        for ( int k = 0; k < nrVoxelsDir3; k++ )
//        {  
//            for ( int i = 0; i < nrVoxelsDir1; i++ )
//            {
//                for ( int j = 0; j < nrVoxelsDir2; j++ )
//                {
//                  
//                    for ( int t = 0; t < timeSamples; t++ )
//                    {
//                        fourDdataReal[i][j][k][t] = m_absorptionChannelTDraw[ cntr ];
//                        fourDdataImag[i][j][k][t] = m_dispersionChannelTDraw[ cntr ];
//                        cntr++;
//                    }
//                }
//            }
//        }
//
//        // now compute the TD signals in order to comply to Nuno's constructor call
//        float[] abso = new float[timeSamples];
//        float[] disp = new float[timeSamples];
//        
//        for ( int k = 0; k < nrVoxelsDir3; k++ )
//        {  
//            for ( int i = 0; i < nrVoxelsDir1; i++ )
//            {
//                for ( int j = 0; j < nrVoxelsDir2; j++ )
//                {
//                    for ( int t = 0; t < timeSamples; t++ )  
//                    {
//                        abso[t] = fourDdataReal[i][j][k][ t ];
//                        disp[t] = fourDdataImag[i][j][k][ t ];
//                    }
//                    float[][] vaux = new float[timeSamples][2];
//
//                    vaux = FD2TD_EPSI( abso, disp );
//
//                    for ( int t = 0; t < timeSamples; t++ )  
//                    {
//                        fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
//                        fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
//                    }
//                }
//            }
//        }
//
//        cntr=0;
//        
//        // now put it in different order
//        for ( int k = nrVoxelsDir3-1; k > -1; k-- )
//        {
//            for ( int i = 0; i < nrVoxelsDir1; i++ )
//            {
//                for ( int j = 0; j < nrVoxelsDir2; j++ )
//                {
//               
//                    for ( int t = 0; t < timeSamples; t++ )
//                    { 
//                        m_absorptionChannelTDraw[ cntr ] = fourDdataReal[i][j][k][t];
//                        m_dispersionChannelTDraw[ cntr ] = fourDdataImag[i][j][k][t];
//                        cntr++;
//                    }
//                }
//            }
//        }
//        
//        fourDdataReal = null;
//        fourDdataImag = null;
//        
//        spectra = new Spectra( m_absorptionChannelTDraw, 
//                               m_dispersionChannelTDraw,
//                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, 
//                               spectralWidth, m_resonanceFrequency, 
//                               ppmReference, FFTDone );
//        spectra.setDefaultCombinationFunction( Spectra.CombinationFunction.MEAN );
//        // model = new GlobalModel( );
//        
//        model.setCSIData( this );
//        model.setSpectra( spectra );
//        model.setCSIGrid( csigrid );
//
//    }
    
    
    /**  
     * Creates a CSIData object based on imported data from Midas.
     * @param model
     * @param midasEPSIdata
     * @param dimensions
     * @param sampleFrequency
     * @param resonanceFrequency
     * @param TE_EPSI
     * @param TR_EPSI
     * @param timeSamples
     * @param sliceNumberToDisplay
     * @param preProcessingType
     * @param filterSize
     */
    public CSIData( GlobalModel model, float[] midasEPSIdata, 
                    int[] dimensions, int timeSamples,
                    float sampleFrequency, double resonanceFrequency,
                    float TE_EPSI, float TR_EPSI, int sliceNumberToDisplay,
                    int preProcessingType, int filterSize, float ppmReference )
    {
        
        patientName         = "Midas Preprocessed 7T EPSI data";
        patientID           = "01234567890";
        patientDOB          = "19700101";
        studyDate           = "20210101";
        
        seriesDescription   = "EPSI-7T-Midas-preprocessed";
        
        seriesNumber        = "-1";
        instanceNumber      = "-1";;
        protocolName        = "EPSI 7T";
        SOPInstanceUID      = "1.1.1.1.1.1.1.1.1.1.1";
        manufacturer        = "Siemens Healthineers Erlangen";
        modelName           = "Terra";
        
        modality            = "MR";
        studyID             = "09876543210";
               
        accessionNumber     = "-1";
        
        patientSex          = "O";
        studyTime           = "00:00:00";
        seriesTime          = "00:00:00";
        scannerName         = "Terra 1Tx";
              
        // For Midas data the FFT is done
        FFTDone              = false;
        m_resonanceFrequency = resonanceFrequency;    
        spectralWidth        = sampleFrequency;
        B0                   = resonanceFrequency / 42.577480610E+06;
                
        TE                   = TE_EPSI;
        TR                   = TR_EPSI;
        
        m_numberOfTimeSamplePoints = timeSamples;
        // ppmReference    = 3.0f;
        nucleus         = "1H";
        
        nrVoxelsDir1    = dimensions[0]; // Nr Columns 01.2021 added correct?
        nrVoxelsDir2    = dimensions[1]; // Nr Rows    01.2021 added correct?
        nrVoxelsDir3    = dimensions[2]; // Nr Frames  01.2021 added correct?
        
        this.model      = model;
        m_sliceNumberToDisplay = sliceNumberToDisplay;
               
        // EPSI single voxel size = same size as MRSI map
//        System.out.println( "pixel size = [ " + model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingX + " , " + 
//                                                model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingY + " , " + 
//                                                model.getImageStack( ).getImages( ).get( 0 ).sliceThickness + " ] " );

        voiSize = new Point3D( );
         
        voiSize.m_x1 = model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingX * nrVoxelsDir1;
        voiSize.m_x2 = model.getImageStack( ).getImages( ).get( 0 ).pixelSpacingY * nrVoxelsDir2;
        voiSize.m_x3 = model.getImageStack( ).getImages( ).get( 0 ).sliceThickness * nrVoxelsDir3;
        
//        System.out.println( "voiSize (total excited volume)  = [ " + voiSize.m_x1 + " , " + 
//                                                                     voiSize.m_x2 + " , " + 
//                                                                     voiSize.m_x3 + " ] " );  
//        
//        System.out.println( "Image[0] position " + " = [ " + model.getImageStack( ).getImages( ).get( 0 ).imagePosition.m_x1 + " , " + 
//                                                             model.getImageStack( ).getImages( ).get( 0 ).imagePosition.m_x2 + " , " + 
//                                                             model.getImageStack( ).getImages( ).get( 0 ).imagePosition.m_x3 + " ] " );  
               
        csigrid = createGridLikeImage(  model.getImageStack( ) );

        // Midas data are available in FD. Constructor needs Spectra in TD.

        int cntr = 0;
        
        m_absorptionChannelTDraw = new float[ midasEPSIdata.length / 2  ];
        m_dispersionChannelTDraw = new float[ midasEPSIdata.length / 2 ];
        
        for ( int i=0;i<midasEPSIdata.length/2;i++)
        {
            m_absorptionChannelTDraw[i] = midasEPSIdata[2*i];
            m_dispersionChannelTDraw[i] = midasEPSIdata[2*i+1];
        }
        
        float[] dummyAbsFD  = new float[timeSamples];
        float[] dummyDispFD = new float[timeSamples];

        int totalNumberOfSpectra = nrVoxelsDir1 * nrVoxelsDir2 * nrVoxelsDir3;
        
        float[][][][] fourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
        float[][][][] fourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
        
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.NO_SPATIAL_FILTERING_AT_IMPORT_ID )
        {
            fourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            fourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];
        
            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = fourDdataReal[i][j][k][ t ];
                            disp[t] = fourDdataImag[i][j][k][ t ];
                        }
                    
                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            }

            cntr=0;
        
            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
        }

            if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_NEAREST_NEIGHBOURS_ONLY_2D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
    
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
                                              
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
 
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            } 
            
            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                System.out.println( "Applying moving median 2D nearest neighbours only filtering at import." );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int median_cntr = 0;
                            float[] averageArrayReal = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];
                            float[] averageArrayImag = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];

                            for ( int q = -filterSize; q <= filterSize; q++ )
                            {
                                averageArrayReal[median_cntr] = fourDdataReal[i+q][j][k][t];
                                averageArrayImag[median_cntr] = fourDdataImag[i+q][j][k][t];
                                median_cntr++;
                            }
                            for ( int q = -filterSize; q <= filterSize; q++ )
                            {
                                averageArrayReal[median_cntr] = fourDdataReal[i][j+q][k][t];
                                averageArrayImag[median_cntr] = fourDdataImag[i][j+q][k][t];
                                median_cntr++;
                            }
                            
                            fourDdataReal[i][j][k][t] = SignalTools.getMedian( averageArrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getMedian( averageArrayImag );
                            
                        } 
                    }   
                }
            }
                        
            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
            fourDdataReal = null;
            fourDdataImag = null;
            
        }
        
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_NEAREST_NEIGHBOURS_ONLY_2D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
            
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
                                              
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            }
            
            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                System.out.println( "Applying moving average 2D nearest neighbours only filtering at import." );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int mean_cntr = 0;
                            float[] averageArrayReal = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];
                            float[] averageArrayImag = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];

                            for ( int q = -filterSize; q <= filterSize; q++ )
                            {
                                averageArrayReal[mean_cntr] = fourDdataReal[i+q][j][k][t];
                                averageArrayImag[mean_cntr] = fourDdataImag[i+q][j][k][t];
                                mean_cntr++;
                            }
                            for ( int q = -filterSize; q <= filterSize; q++ )
                            {
                                averageArrayReal[mean_cntr] = fourDdataReal[i][j+q][k][t];
                                averageArrayImag[mean_cntr] = fourDdataImag[i][j+q][k][t];
                                mean_cntr++;
                            }
                            
                            fourDdataReal[i][j][k][t] = SignalTools.getAverage( averageArrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getAverage( averageArrayImag );
                            
                        } 
                    }   
                }
            }
                        
            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
            fourDdataReal = null;
            fourDdataImag = null;
        }
                
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_FULL_2D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
    
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
                                              
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
 
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            }             
            
            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                System.out.println( "Applying 2D all neighbours moving median filtering at import" );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int median_cntr = 0;
                            float[] medianarrayReal = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            float[] medianarrayImag = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            
                            for ( int p = -filterSize; p <= filterSize; p++ )
                            {
                                for ( int q = -filterSize; q <= filterSize; q++ )
                                {
                                    medianarrayReal[median_cntr] = fourDdataReal[i+p][j+q][k][t];
                                    medianarrayImag[median_cntr] = fourDdataImag[i+p][j+q][k][t];
                                    median_cntr++;
                                }
                            }
                            
                            fourDdataReal[i][j][k][t] = SignalTools.getMedian( medianarrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getMedian( medianarrayImag );
                            
                        } 
                    }   
                }
            }
                     
            cntr=0;

            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
            fourDdataReal = null;
            fourDdataImag = null;
            
        }
        
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_FULL_2D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
            
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
            
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            }   
            
            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                System.out.println( "Applying moving median filtering at import" );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int median_cntr = 0;
                            float[] averageArrayReal = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            float[] averageArrayImag = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            
                            for ( int p = -filterSize; p <= filterSize; p++ )
                            {
                                for ( int q = -filterSize; q <= filterSize; q++ )
                                {
                                    averageArrayReal[median_cntr] = fourDdataReal[i+p][j+q][k][t];
                                    averageArrayImag[median_cntr] = fourDdataImag[i+p][j+q][k][t];
                                    median_cntr++;
                                }
                            }
                            
                            fourDdataReal[i][j][k][t] = SignalTools.getAverage( averageArrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getAverage( averageArrayImag );
                            
                        } 
                    }   
                }
            }

            cntr=0;

            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
        }
        
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_FULL_3D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
    
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
                                              
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            }   
            
            for ( int k = filterSize; k < nrVoxelsDir3 - filterSize; k++ )
            {  
                System.out.println( "Applying moving median filtering at import" );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int median_cntr = 0;
                            float[] medianArrayReal = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            float[] medianArrayImag = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            
                            for ( int p = -filterSize; p <= filterSize; p++ )
                            {
                                for ( int q = -filterSize; q <= filterSize; q++ )
                                {
                                    for ( int r = -filterSize; r <= filterSize; r++ )
                                    {
                                        medianArrayReal[median_cntr] = fourDdataReal[i+p][j+q][k+r][t];
                                        medianArrayImag[median_cntr] = fourDdataImag[i+p][j+q][k+r][t];
                                        median_cntr++;
                                    }
                                }
                            }
                            
                            fourDdataReal[i][j][k][t] = SignalTools.getMedian( medianArrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getMedian( medianArrayImag );
                            
                        } 
                    }   
                }
            }

            cntr = 0;

            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
            fourDdataReal = null;
            fourDdataImag = null;
            
        }
        
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_FULL_3D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
            
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
                                              
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            } 
            
            for ( int k = filterSize; k < nrVoxelsDir3 - filterSize; k++ )
            {  
                System.out.println( "Applying moving median filtering at import" );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int average_cntr = 0;
                            float[] averageArrayReal = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            float[] averageArrayImag = new float[ ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) * ( 2 * filterSize + 1 ) ];
                            
                            for ( int p = -filterSize; p <= filterSize; p++ )
                            {
                                for ( int q = -filterSize; q <= filterSize; q++ )
                                {
                                    for ( int r = -filterSize; r <= filterSize; r++ )
                                    {
                                        averageArrayReal[average_cntr] = fourDdataReal[i+p][j+q][k+r][t];
                                        averageArrayImag[average_cntr] = fourDdataImag[i+p][j+q][k+r][t];
                                        average_cntr++;
                                    }
                                }
                            }
                            
                            fourDdataReal[i][j][k][t] = SignalTools.getAverage( averageArrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getAverage( averageArrayImag );
                            
                        } 
                    }   
                }
            }

            cntr=0;

            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
        }

        
        

        
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_NEAREST_NEIGHBOURS_ONLY_3D_MEDIAN_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
    
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
                                              
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            }
            
            
            for ( int k = filterSize; k < nrVoxelsDir3 - filterSize; k++ )
            {  
                System.out.println( "Applying 3D moving median nearest neighbours only filtering at import" );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int median_cntr = 0;
                            float[] medianArrayReal = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];
                            float[] medianArrayImag = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];
                            
                            for ( int r = -filterSize; r <= filterSize; r++ )
                            {
                                medianArrayReal[median_cntr] = fourDdataReal[i+r][j][k][t];
                                medianArrayImag[median_cntr] = fourDdataImag[i+r][j][k][t];
                                median_cntr++;
                            }
                            
                            for ( int r = -filterSize; r <= filterSize; r++ )
                            {
                                medianArrayReal[median_cntr] = fourDdataReal[i][j+r][k][t];
                                medianArrayImag[median_cntr] = fourDdataImag[i][j+r][k][t];
                                median_cntr++;
                            }
                            
                            for ( int r = -filterSize; r <= filterSize; r++ )
                            {
                                medianArrayReal[median_cntr] = fourDdataReal[i][j][k+r][t];
                                medianArrayImag[median_cntr] = fourDdataImag[i][j][k+r][t];
                                median_cntr++;
                            }
                            
                            fourDdataReal[i][j][k][t] = SignalTools.getMedian( medianArrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getMedian( medianArrayImag );
                            
                        } 
                    }   
                }
            }
                        
            cntr = 0;

            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
            fourDdataReal = null;
            fourDdataImag = null;
            
        }
        
        if ( preProcessingType == MidasPreprocessedEPSIDataImporterParameterSettingsControl.MOVING_NEAREST_NEIGHBOURS_ONLY_3D_AVERAGE_SPATIAL_FILTERING_AT_IMPORT_ID ) // in plane moving median averaging ...
        {
            
            // Now filter the data:
            float[][][][] dummyFourDdataReal = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            float[][][][] dummyFourDdataImag = new float[nrVoxelsDir1][nrVoxelsDir2][nrVoxelsDir3][timeSamples];
            cntr = 0;
                                              
            dummyFourDdataReal = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_absorptionChannelTDraw );
            dummyFourDdataImag = splitRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, m_dispersionChannelTDraw );
            
            // Apply filtering in TD!
            // Now compute the TD signals in order to comply to Nuno's constructor call
            float[] abso = new float[timeSamples];
            float[] disp = new float[timeSamples];

            for ( int k = 0; k < nrVoxelsDir3; k++ )
            {  
                for ( int i = 0; i < nrVoxelsDir1; i++ )
                {
                    for ( int j = 0; j < nrVoxelsDir2; j++ )
                    {
                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            abso[t] = dummyFourDdataReal[i][j][k][ t ];
                            disp[t] = dummyFourDdataImag[i][j][k][ t ];
                        }

                        float[][] vaux = new float[timeSamples][2];

                        vaux = FD2TD_EPSI( abso, disp );

                        for ( int t = 0; t < timeSamples; t++ )  
                        {
                            fourDdataReal[i][j][k][t] = vaux[t][0] / timeSamples;
                            fourDdataImag[i][j][k][t] = -vaux[t][1] / timeSamples;
                        }
                    }
                }
            }
            
            
            for ( int k = filterSize; k < nrVoxelsDir3 - filterSize; k++ )
            {  
                System.out.println( "Applying moving average 3D nearest neighbours only filtering at import" );
                for ( int j = filterSize; j < nrVoxelsDir2 - filterSize ; j++ )
                {
                    for ( int i = filterSize; i < nrVoxelsDir1 - filterSize; i++ )
                    {  
                        
                        for ( int t = 0; t < timeSamples; t++ )
                        {
                            int average_cntr = 0;
                            float[] averageArrayReal = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];
                            float[] averageArrayImag = new float[ ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) + ( 2 * filterSize + 1 ) ];
                            
                            for ( int r = -filterSize; r <= filterSize; r++ )
                            {
                                averageArrayReal[average_cntr] = fourDdataReal[i+r][j][k][t];
                                averageArrayImag[average_cntr] = fourDdataImag[i+r][j][k][t];
                                average_cntr++;
                            }
                            
                            for ( int r = -filterSize; r <= filterSize; r++ )
                            {
                                averageArrayReal[average_cntr] = fourDdataReal[i][j+r][k][t];
                                averageArrayImag[average_cntr] = fourDdataImag[i][j+r][k][t];
                                average_cntr++;
                            }
                            
                            for ( int r = -filterSize; r <= filterSize; r++ )
                            {
                                averageArrayReal[average_cntr] = fourDdataReal[i][j][k+r][t];
                                averageArrayImag[average_cntr] = fourDdataImag[i][j][k+r][t];
                                average_cntr++;
                            }
                                                        
                            fourDdataReal[i][j][k][t] = SignalTools.getAverage( averageArrayReal );
                            fourDdataImag[i][j][k][t] = SignalTools.getAverage( averageArrayImag );
                            
                        } 
                    }   
                }
            }

            cntr=0;

            m_absorptionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataReal );
            m_dispersionChannelTDraw = mergeRawData( nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, timeSamples, fourDdataImag );
            
        }
        
        
        fourDdataReal = null;
        fourDdataImag = null;
        
        spectra = new Spectra( m_absorptionChannelTDraw, 
                               m_dispersionChannelTDraw,
                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, 
                               spectralWidth, m_resonanceFrequency, 
                               ppmReference, FFTDone );
        
        spectra.setDefaultCombinationFunction( Spectra.CombinationFunction.MEAN );
        
        model.setCSIData( this );
        model.setSpectra( spectra );
        
        model.setCSIGrid( csigrid );
        model.calcGridIntersections( );

    }
    
    protected float[][][][] splitRawData( int gridSize1, int gridSize2, int gridSize3, int timeSize, float[] absIn ) 
    {
        float[][][][] absOut = new float[gridSize1][gridSize2][gridSize3][timeSize];      
        for ( int k = 0; k < gridSize3; k++ ) 
        {
            for ( int j = 0; j < gridSize2; j++ ) 
            {
                for ( int i = 0; i < gridSize1; i++ ) 
                {
                    for ( int t = 0; t < (timeSize); t++ ) 
                    {
                        absOut[i][j][k][t]  = absIn[ k * gridSize1 * gridSize2 * timeSize
                                                     + j * gridSize1 * timeSize
                                                     + i * timeSize + t ];
                       
                    }
                }
            }
        }
        return absOut;
    }
    
    protected float[] mergeRawData( int gridSize1, int gridSize2, int gridSize3, int timeSize, float[][][][] absIn ) 
    {
        float[] absOut = new float[gridSize1*gridSize2*gridSize3*timeSize];
        
        for ( int k = 0; k < gridSize3; k++ ) 
        {
            for ( int j = 0; j < gridSize2; j++ ) 
            {
                for ( int i = 0; i < gridSize1; i++ ) 
                {
                    for ( int t = 0; t < (timeSize); t++ ) 
                    {
                        int aLoc = k * gridSize1 * gridSize2 * timeSize
                                   + j * gridSize1 * timeSize
                                   + i * timeSize + t;
                        absOut[ aLoc ] = absIn[i][j][k][t];
                    }
                }
            }
        }
        return absOut;
    }   
    
    public CSIGrid createGridLikeImage( ImageStack myImageStack )
    {
        
        CSIGrid newgrid = null;
        
        if ( myImageStack != null ) 
        {
            int[] rasterDimensions = myImageStack.getDimensions();
            ImageSlice firstImage =myImageStack.getImages().get(0);
            double spacingBetweenSlices = (myImageStack.getImages().get(1).imagePosition.subtractPoint(myImageStack.getImages().get(0).imagePosition)).length();
            
            Point3D voxelsize = new Point3D(firstImage.pixelSpacingX,firstImage.pixelSpacingY,spacingBetweenSlices);
            
            Point3D FOV = new Point3D(rasterDimensions[0]*voxelsize.m_x1, rasterDimensions[1]*voxelsize.m_x2, rasterDimensions[2]*voxelsize.m_x3);
           
            Point3D V1 = firstImage.imageOrientationX.normalize();
            Point3D V2 = firstImage.imageOrientationY.normalize();
            Point3D V3 = Point3D.crossProduct(V1, V2).normalize();
           
            Point3D center = firstImage.imagePosition;
            // I dont fully remmember but only z direction needs a half voxel shift.
            Point3D halfVoxelShift = Point3D.vectorCombination(V1, V2, V3, 0*voxelsize.m_x1/2, 0*voxelsize.m_x2/2, voxelsize.m_x3/2);
            Point3D topLeft2center = Point3D.vectorCombination(V1, V2, V3, FOV.m_x1/2, FOV.m_x2/2, FOV.m_x3/2);
            center = center.addPoint(topLeft2center);
            center = center.subtractPoint(halfVoxelShift);
            newgrid = new CSIGrid( center, 
                                   V1.scalarMultiplication(voxelsize.m_x1), 
                                   V2.scalarMultiplication(voxelsize.m_x2), 
                                   V3.scalarMultiplication(voxelsize.m_x3), 
                                   rasterDimensions[0], rasterDimensions[1], 
                                   rasterDimensions[2],
                                   FOV.m_x1, FOV.m_x2, FOV.m_x3 );
           
            // Create excited volume voxel
            double dim1 = FOV.m_x1 / 2;
            double dim2 = FOV.m_x2 / 2;
            double dim3 = FOV.m_x3 / 2;

            Point3D p5 = Point3D.vectorCombination(V1, V2, V3, -dim1, -dim2, -dim3);
            Point3D p3 = Point3D.vectorCombination(V1, V2, V3, dim1, dim2, dim3);
            Point3D p6 = Point3D.vectorCombination(V1, V2, V3, dim1, -dim2, -dim3);
            Point3D p1 = Point3D.vectorCombination(V1, V2, V3, -dim1, dim2, -dim3);
            Point3D p8 = Point3D.vectorCombination(V1, V2, V3, -dim1, -dim2, dim3);
            Point3D p2 = Point3D.vectorCombination(V1, V2, V3, dim1, dim2, -dim3);
            Point3D p4 = Point3D.vectorCombination(V1, V2, V3, -dim1, dim2, dim3);
            Point3D p7 = Point3D.vectorCombination(V1, V2, V3, dim1, -dim2, dim3);

            p1.translate(center);
            p2.translate(center);
            p3.translate(center);
            p4.translate(center);
            p5.translate(center);
            p6.translate(center);
            p7.translate(center);
            p8.translate(center);

            
//            System.out.println( "Excitation volume p1 = [ " + p1.m_x1 + " , " + p1.m_x2 + " , " + p1.m_x3 + " ] " );  
//            System.out.println( "Excitation volume p2 = [ " + p2.m_x1 + " , " + p2.m_x2 + " , " + p2.m_x3 + " ] " );  
//            System.out.println( "Excitation volume p3 = [ " + p3.m_x1 + " , " + p3.m_x2 + " , " + p3.m_x3 + " ] " );  
//            System.out.println( "Excitation volume p4 = [ " + p4.m_x1 + " , " + p4.m_x2 + " , " + p4.m_x3 + " ] " );  
//            System.out.println( "Excitation volume p5 = [ " + p5.m_x1 + " , " + p5.m_x2 + " , " + p5.m_x3 + " ] " );  
//            System.out.println( "Excitation volume p6 = [ " + p6.m_x1 + " , " + p6.m_x2 + " , " + p6.m_x3 + " ] " );  
//            System.out.println( "Excitation volume p7 = [ " + p7.m_x1 + " , " + p7.m_x2 + " , " + p7.m_x3 + " ] " );  
//            System.out.println( "Excitation volume p8 = [ " + p8.m_x1 + " , " + p8.m_x2 + " , " + p8.m_x3 + " ] " ); 
            
            
            Cube3D excitationVOI = new Cube3D( p1, p2, p3, p4, p5, p6, p7, p8 );
            
            newgrid.setExcitationVoxel( excitationVOI );
           
        }
        
        return newgrid;
    }
    
    
    
    
    public CSIData( DicomHeader header, String file ) 
    {
        this.file = file;
        saturationBands = new ArrayList<>();
        DicomHeaderLine lineaux = (DicomHeaderLine) header.get( "(0008,0016)" );
        if (lineaux != null) 
        {
            if ( lineaux.Value.contentEquals( "1.3.12.2.1107.5.9.1" ) ) 
            {
                fileType = 1; //Siemens Format              
            } 
            else if ( lineaux.Value.contentEquals( "1.2.840.10008.5.1.4.1.1.4.2" ) ) 
            {
                fileType = 2; //Standard Format
            }
            
            if ( fileType != 0 ) 
            {
                importCSIdata( header );
                
                
                if ( seriesDescription.toLowerCase( ).contains( epsi.toLowerCase( ) )  == false )
                //if ( seriesDescription.contains( new CharSequence("EPSI_MRSI") ) != 0 )
                {    
                    processRawSpectra( );
                }
                else
                {
                    processRawEPSIDicomFormattedSpectra( );
                }
                
                if ( nrSpectraPerVoxel > 1 ) 
                { //TEMPORARY: NOW WE ONLY USE THE WATER_SUP SPECTRA. IN PHILIPS NWS IS IN THE SECOND HALF.
                    int l = m_absorptionChannelTDraw.length / nrSpectraPerVoxel;
                    float[] abs2 = new float[l];
                    float[] disp2 = new float[l];
                    System.arraycopy( m_absorptionChannelTDraw, 0, abs2, 0, l );
                    System.arraycopy( m_dispersionChannelTDraw, 0, disp2, 0, l );
                    spectra = new Spectra( abs2, disp2, nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, spectralWidth, m_resonanceFrequency, ppmReference, FFTDone );
                } 
                else 
                {
                    spectra = new Spectra( m_absorptionChannelTDraw, 
                                           m_dispersionChannelTDraw,
                                           nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, 
                                           spectralWidth, m_resonanceFrequency, 
                                           ppmReference, FFTDone );
                }
                model = new GlobalModel( );
                model.setCSIData( this );
                model.setSpectra( spectra );
                model.setCSIGrid( csigrid );
            }
        }
    }

    public GlobalModel getModel( ) 
    {
        return model;
    }

    public Spectra getSpectra( ) 
    {
        return spectra;
    }

    public String getFile( ) 
    {
        return file;
    }

    public void setFile( String fileReference )
    {
        file = fileReference;
    }
            
    public void processRawSpectra( ) 
    {
        byte[] dummy = new byte[4];

        if ( protocolName.contains( "csi_laser_spiral" ) )
        {
            spectralWidth = spectralWidth * 2f;
            System.out.println( "Protocol name contains string 'csi_laser_spiral' : bandwidth doubled!" );
        }
        
        // Adaptation for case that oversampling 
        // is not removed by Siemens scanner
        // The number of sample points must be verified instead of computation 
        // of the number of "nrBits" (mentioned should be bytes (and not bits)?
        int NrBits = 8;
        m_numberOfTimeSamplePoints = rawDataByte.length / ( NrBits * nrVoxelsDir1 * nrVoxelsDir2 * nrVoxelsDir3 * nrSpectraPerVoxel );
        
//        int NrBits = rawDataByte.length / ( m_numberOfTimeSamplePoints * nrVoxelsDir1 * nrVoxelsDir2 * nrVoxelsDir3 * nrSpectraPerVoxel );
//        NrBits = NrBits/2;
        vectorlength = m_numberOfTimeSamplePoints;
        m_bothChannelsTD = new float[this.rawDataByte.length / ( NrBits / 2 )];
        m_absorptionChannelTDraw = new float[this.rawDataByte.length / NrBits];
        m_dispersionChannelTDraw = new float[this.rawDataByte.length / NrBits];

        float max = Float.MIN_VALUE;
// 2097151
        // TODO: Adapt to different lengths
        // for ( int i = 0; i < ( this.rawDataByte.length / (NrBits / 2 ) ); i++ ) 
        for ( int i = 0; i < ( this.rawDataByte.length / ( NrBits / 2) ); i++ ) 
        {          
            dummy[0] = this.rawDataByte[4 * i + 3];
            dummy[1] = this.rawDataByte[4 * i + 2];
            dummy[2] = this.rawDataByte[4 * i + 1];
            dummy[3] = this.rawDataByte[4 * i + 0];
            
            m_bothChannelsTD[i] = org.spectrim.lib.Convert.byte2float( dummy, 0 );
         
        }
            
        for ( int i = 0; i < m_bothChannelsTD.length / 2; i++ ) 
        {
            m_absorptionChannelTDraw[i] = m_bothChannelsTD[2 * i];
            if ( fileType == 1 ) 
            {
                m_dispersionChannelTDraw[i] = -m_bothChannelsTD[2 * i + 1];
            } 
            else if ( fileType == 2 ) 
            {
                m_dispersionChannelTDraw[i] = m_bothChannelsTD[2 * i + 1];
            }
        }

    }

    public void processRawEPSIDicomFormattedSpectra( ) 
    {
        byte[] dummy = new byte[4];

        int NrBytes = 8;
        
        // In this type of data these tags need to be set manually!
        nrVoxelsDir3 = 1;
        spectralWidth = spectralWidth/100.0;
        
        m_numberOfTimeSamplePoints = rawDataByte.length / ( NrBytes * nrVoxelsDir1 * nrVoxelsDir2 * nrVoxelsDir3 * nrSpectraPerVoxel );
        
        vectorlength = m_numberOfTimeSamplePoints;
        m_bothChannelsTD = new float[this.rawDataByte.length / ( NrBytes / 2 )];
        m_absorptionChannelTDraw = new float[this.rawDataByte.length / NrBytes];
        m_dispersionChannelTDraw = new float[this.rawDataByte.length / NrBytes];

        float max = Float.MIN_VALUE;

        for ( int i = 0; i < ( this.rawDataByte.length / ( NrBytes / 2) ); i++ ) 
        {          
            dummy[0] = this.rawDataByte[4 * i + 3];
            dummy[1] = this.rawDataByte[4 * i + 2];
            dummy[2] = this.rawDataByte[4 * i + 1];
            dummy[3] = this.rawDataByte[4 * i + 0];
            
            m_bothChannelsTD[i] = org.spectrim.lib.Convert.byte2float( dummy, 0 );
         
        }
            
        for ( int i = 0; i < m_bothChannelsTD.length / 2; i++ ) 
        {
            m_absorptionChannelTDraw[i] = m_bothChannelsTD[2 * i];
            if ( fileType == 1 )
            {
                if ( protocolName.toLowerCase( ).contains( epsi.toLowerCase( ) )  == false )
                // if ( protocolName.contains( "EPSI_MRSI" ) )
                {
                     m_dispersionChannelTDraw[i] = m_bothChannelsTD[2 * i + 1];
                }
                else
                {
                    m_dispersionChannelTDraw[i] = -m_bothChannelsTD[2 * i + 1];
                }
            } 
            else if ( fileType == 2 ) 
            {
                m_dispersionChannelTDraw[i] = m_bothChannelsTD[2 * i + 1];
            }
        }

    }
    
    
    
    
    public void zeroFilling(int vectorsize) {
        if ((Math.abs(Math.log(vectorsize) / Math.log(2)) % 1) != 0) {
            System.out.println("The vector size should be a power of 2");
        } else {
            vectorlength = vectorsize;
            spectra.zerofilling(m_numberOfTimeSamplePoints - vectorsize);
        }
    }

    public String getPatientName() {
        return patientName;
    }

    public String getPatientID() {
        return patientID;
    }

    /**
     * Returns the patients date of birth.
     *
     * @return
     */
    public String getPatientDOB() {
        return patientDOB;
    }

    public String getStudyDate() {
        return studyDate;
    }

    public String getSeriesDescription() {
        return seriesDescription;
    }

    public String getSeriesNumber() {
        return seriesNumber;
    }

    public String getInstanceNumber() {
        return instanceNumber;
    }

    public String getSOPInstanceUID() {
        return SOPInstanceUID;
    }

    public double getB0() {
        return B0;
    }

    public double getTE() {
        return TE;
    }

    public double getTR() {
        return TR;
    }

    public double getBandwidth() {
        return spectralWidth;
    }

    public String getManufacturer() {
        return manufacturer;
    }

    public void setManufacturer( String manufacturer )
    {
        this.manufacturer = manufacturer;
    }
    
    public String getModelName() {
        return modelName;
    }

    public String getNucleus() {
        return nucleus;
    }

    // Jana
    public String getModality() {
        return modality;
    }

    public String getStudyID() {
        return studyID;
    }

    protected boolean isHeaderValue(DicomHeader header, String tag) {
        DicomHeaderLine headerLine;
        headerLine = (DicomHeaderLine) header.get(tag);
        return (headerLine != null);

    }

    protected String readHeaderValue(DicomHeader header, String tag) {
        DicomHeaderLine headerLine;
        headerLine = (DicomHeaderLine) header.get(tag);
        if (headerLine != null) {
            if ( headerLine.Value != null )
            {
                String val = headerLine.Value.replace("\"", "");            
                return val;
            }
            else
            {
                // System.out.println("No value found for tag: " + tag);
                return "0";
            }
        } else {
            // System.out.println("No value found for tag: " + tag);
            return "0";
        }
    }

 /**
     * Add hoc solution for reading Dicom Philips Tags
     * @param header 
     */
    public void importCSIdata( DicomHeader header ) 
    {
        DicomHeaderLine headerLine;

        patientName = readHeaderValue(header, "(0010,0010)");
        patientID = readHeaderValue(header, "(0010,0020)");
        patientDOB = readHeaderValue(header, "(0010,0030)");
        studyDate = readHeaderValue(header, "(0008,0020)");
        seriesDescription = readHeaderValue(header, "(0008,103E)");
        seriesNumber = readHeaderValue(header, "(0020,0011)");
        instanceNumber = readHeaderValue(header, "(0020,0013)");
        protocolName = readHeaderValue(header, "(0018,1030)");
        SOPInstanceUID = readHeaderValue(header, "(0008,0018)");
        manufacturer = readHeaderValue(header, "(0008,0070)");
        modelName = readHeaderValue(header, "(0008,1090)");
        // added by Jana
        modality = readHeaderValue(header, "(0008,0060)");
        studyID = readHeaderValue(header, "(0020,0010)");
        instanceNumber = readHeaderValue(header, "(0020,0013)");
        studyDescription = readHeaderValue(header, "(0008,1030)");
        if ( !manufacturer.contains( "Philips" ) )
        {
            accessionNumber = readHeaderValue(header, "(0008,0050)" );
            seriesDescription = readHeaderValue(header, "(0018,1030)" );
        }
        patientSex = readHeaderValue(header, "(0010,0040)");
        studyTime = readHeaderValue(header, "(0008,0030)");
        seriesTime = readHeaderValue(header, "(0008,0031)");
        scannerName = readHeaderValue(header, "(0008,1090)");
        
        if ( fileType == 1 ) 
        {
            B0 = Double.valueOf(readHeaderValue(header, "sProtConsistencyInfo.flNominalB0"));
            if (B0 == 0) {
                B0 = Double.valueOf(readHeaderValue(header, "sTXSPEC.flKDynMagnitudeMax"));
            }
            TE = Double.valueOf(readHeaderValue(header, "alTE[0]")) / 1000;
            TR = Double.valueOf(readHeaderValue(header, "alTR[0]")) / 1000;
            m_ref_imageUID1 = readHeaderValue(header, "tReferenceImage0");
            m_ref_imageUID2 = readHeaderValue(header, "tReferenceImage1");
            m_ref_imageUID3 = readHeaderValue(header, "tReferenceImage2");
            
            // is table offset according to 
            lScanRegionPosTra = Double.parseDouble( "" + readHeaderValue( header, "lScanRegionPosTra") );
            
            headerLine = (DicomHeaderLine) header.get("(7FE1,1010)");
            rawDataByte = headerLine.ValueByte;
            
            m_resonanceFrequency = Double.valueOf(readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].lFrequency")); //Hz

            if (readHeaderValue(header, "sCoilSelectMeas.aTxCoilSelectData[0].tNucleus").matches("1H")
                    || readHeaderValue(header, "sCoilSelectMeas.aRxCoilSelectData[0].tNucleus").matches("1H")
                    || readHeaderValue(header, "sRXSPEC.asNucleusInfo[0].tNucleus").matches("1H")
                    || readHeaderValue(header, "asCoilSelectMeas[0].tNucleus").matches("1H")
                    || readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].tNucleus").matches("1H")) {
                ppmReference = 4.7;
                nucleus = "1H";
            };//Include other nucleus

            if (readHeaderValue(header, "sCoilSelectMeas.aTxCoilSelectData[0].tNucleus").matches("2H")
                    || readHeaderValue(header, "sCoilSelectMeas.aRxCoilSelectData[0].tNucleus").matches("2H")
                    || readHeaderValue(header, "sRXSPEC.asNucleusInfo[0].tNucleus").matches("2H")
                    || readHeaderValue(header, "asCoilSelectMeas[0].tNucleus").matches("2H")
                    || readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].tNucleus").matches("2H")) {
                ppmReference = 4.7;
                nucleus = "2H";
            };
            
            if (readHeaderValue(header, "sCoilSelectMeas.aTxCoilSelectData[0].tNucleus").matches("31P")
                    || readHeaderValue(header, "sCoilSelectMeas.aRxCoilSelectData[0].tNucleus").matches("31P")
                    || readHeaderValue(header, "sRXSPEC.asNucleusInfo[0].tNucleus").matches("31P")
                    || readHeaderValue(header, "asCoilSelectMeas[0].tNucleus").matches("31P")
                    || readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].tNucleus").matches("31P")) {
                ppmReference = 0.0;
                nucleus = "31P";
            };
            
            if (readHeaderValue(header, "sCoilSelectMeas.aTxCoilSelectData[0].tNucleus").matches("13C")
                    || readHeaderValue(header, "sCoilSelectMeas.aRxCoilSelectData[0].tNucleus").matches("13C")
                    || readHeaderValue(header, "sRXSPEC.asNucleusInfo[0].tNucleus").matches("13C")
                    || readHeaderValue(header, "asCoilSelectMeas[0].tNucleus").matches("13C")
                    || readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].tNucleus").matches("13C")) {
                ppmReference = 0.0;
                nucleus = "13C";
            };
            
            if (readHeaderValue(header, "sCoilSelectMeas.aTxCoilSelectData[0].tNucleus").matches("23Na")
                    || readHeaderValue(header, "sCoilSelectMeas.aRxCoilSelectData[0].tNucleus").matches("23Na")
                    || readHeaderValue(header, "sRXSPEC.asNucleusInfo[0].tNucleus").matches("23Na")
                    || readHeaderValue(header, "asCoilSelectMeas[0].tNucleus").matches("23Na")
                    || readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].tNucleus").matches("23Na")) {
                ppmReference = 0.0;
                nucleus = "23Na";
            };
            
            if ( readHeaderValue(header, "sCoilSelectMeas.aTxCoilSelectData[0].tNucleus").matches("19F")
                    || readHeaderValue(header, "sCoilSelectMeas.aRxCoilSelectData[0].tNucleus").matches("19F")
                    || readHeaderValue(header, "sRXSPEC.asNucleusInfo[0].tNucleus").matches("19F")
                    || readHeaderValue(header, "asCoilSelectMeas[0].tNucleus").matches("19F")
                    || readHeaderValue(header, "sTXSPEC.asNucleusInfo[0].tNucleus").matches("19F")) {
                ppmReference = 0.0;
                nucleus = "19F";
            };
            
            NumberFormat nf = new DecimalFormat( "###.##" );
            
            // Set the default values for possible HLSVDpro
            TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
            defaults.load();
            defaults.setProperty( TDFDFittingDefaults.HLSVD_PPM_REFERENCE_VALUE, "" + ppmReference );
            defaults.setProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY, "" + nf.format( ppmReference - 0.3f ) );
            defaults.setProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY, "" + nf.format( ppmReference + 0.3f ) );
            defaults.save();
            //NOTE:
            //FOV variables - tags starting with sSliceArray;
            //VOI variables - tags starting with sSpecPara.sVoI
            m_numberOfTimeSamplePoints = Integer.valueOf(readHeaderValue(header, "sSpecPara.lVectorSize"));
            nrVoxelsDir1 = Integer.valueOf(readHeaderValue(header, "sSpecPara.lFinalMatrixSizeRead"));
            nrVoxelsDir2 = Integer.valueOf(readHeaderValue(header, "sSpecPara.lFinalMatrixSizePhase"));
            nrVoxelsDir3 = Integer.valueOf(readHeaderValue(header, "sSpecPara.lFinalMatrixSizeSlice"));
            nrSpectraPerVoxel = 1;
            
            if ( seriesDescription.toLowerCase( ).contains( epsi.toLowerCase( ) )  == true )
            // if ( protocolName.contains( "EPSI_MRSI" ) )
            {
                nrVoxelsDir3 = 1;
            }
            
            double voiSagPosition = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sPosition.dSag"));
            double voiCorPosition = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sPosition.dCor"));
            double voiTraPosition = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sPosition.dTra"));
            
            // Correct for the table position which is given by lScanRegionPosTra
            voiPosition = new Point3D(voiSagPosition, voiCorPosition, voiTraPosition + lScanRegionPosTra);

            //TODO: Check if this works with more than one slice... Each slice should have its position
            double fovSagPosition = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].sPosition.dSag"));
            double fovCorPosition = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].sPosition.dCor"));
            double fovTraPosition = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].sPosition.dTra"));
            fovPosition = new Point3D(fovSagPosition, fovCorPosition, fovTraPosition);

            voiInPlaneRotation = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.dInPlaneRot"));
            fovInPlaneRotation = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].dInPlaneRot"));

            double norm2VoiPlaneX = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sNormal.dSag"));
            double norm2VoiPlaneY = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sNormal.dCor"));
            double norm2VoiPlaneZ = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sNormal.dTra"));

            voiSize = new Point3D();

//          Readout and phase encoding directions change in Sagital and Coronal prescriptions  
            if (norm2VoiPlaneZ >= norm2VoiPlaneY && norm2VoiPlaneZ >= norm2VoiPlaneX) {//Tranversal case
                voiSize.m_x1 = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.dReadoutFOV"));
                voiSize.m_x2 = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.dPhaseFOV"));
            } else {//Sagital and Coronal cases
                voiSize.m_x2 = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.dReadoutFOV"));
                voiSize.m_x1 = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.dPhaseFOV"));
            }

            voiSize.m_x3 = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.dThickness"));

//            Normal to the plane (or V3)
            voiV3 = new Point3D(norm2VoiPlaneX, norm2VoiPlaneY, norm2VoiPlaneZ);

////            Starting V1 (before rotation) - (direction of increasing columns)
            if (norm2VoiPlaneZ > norm2VoiPlaneY && norm2VoiPlaneZ > norm2VoiPlaneX) {
                voiV1 = Point3D.normalize(new Point3D(voiV3.m_x3, 0, -voiV3.m_x1)); //Transversal case
            } else if (norm2VoiPlaneY > norm2VoiPlaneX && norm2VoiPlaneY > norm2VoiPlaneZ) {
                voiV1 = Point3D.normalize(new Point3D(voiV3.m_x2, -voiV3.m_x1, 0)); //Coronal case
            } else {
                voiV1 = Point3D.normalize(new Point3D(-voiV3.m_x2, voiV3.m_x1, 0)); //Sagital case
            }

//            Rotate V1
            if (voiInPlaneRotation != 0) {
                voiV1 = Point3D.rotate3D(voiV1, voiV3, -voiInPlaneRotation);
            }

//            Obtain the 2nd Vector (direction of increasing rows)
            if (norm2VoiPlaneX > norm2VoiPlaneY && norm2VoiPlaneX > norm2VoiPlaneZ) {
                voiV2 = Point3D.crossProduct(voiV1, voiV3); //Sagital case (The sagital case has a left-handed oriented referencial)
            } else {
                voiV2 = Point3D.crossProduct(voiV3, voiV1); //Transversal and Coronal cases
            }
            voiV2.normalize();

            double norm2FovPlaneX = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sNormal.dSag"));
            double norm2FovPlaneY = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sNormal.dCor"));
            double norm2FovPlaneZ = Double.valueOf(readHeaderValue(header, "sSpecPara.sVoI.sNormal.dTra"));

            fovSize = new Point3D();
            if (norm2FovPlaneZ >= norm2FovPlaneY && norm2FovPlaneZ >= norm2FovPlaneX) {//Tranversal case
                fovSize.m_x1 = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].dReadoutFOV"));
                fovSize.m_x2 = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].dPhaseFOV"));
            } else {//Sagital and Coronal cases
                fovSize.m_x2 = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].dReadoutFOV"));
                fovSize.m_x1 = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].dPhaseFOV"));
            }

            fovSize.m_x3 = Double.valueOf(readHeaderValue(header, "sSliceArray.asSlice[0].dThickness"));

//            Normal to the plane (or V3)
            fovV3 = new Point3D(norm2FovPlaneX, norm2FovPlaneY, norm2FovPlaneZ);

////            Starting V1 (before rotation) - (direction of increasing columns)
            if (norm2FovPlaneZ > norm2FovPlaneY && norm2FovPlaneZ > norm2FovPlaneX) {
                fovV1 = Point3D.normalize(new Point3D(fovV3.m_x3, 0, -fovV3.m_x1)); //Transversal case
            } else if (norm2FovPlaneY > norm2FovPlaneX && norm2FovPlaneY > norm2FovPlaneZ) {
                fovV1 = Point3D.normalize(new Point3D(fovV3.m_x2, -fovV3.m_x1, 0)); //Coronal case
            } else {
                fovV1 = Point3D.normalize(new Point3D(-fovV3.m_x2, fovV3.m_x1, 0)); //Sagital case
            }

//            Rotate V1
            if (fovInPlaneRotation != 0) {
                fovV1 = Point3D.rotate3D(fovV1, fovV3, -fovInPlaneRotation);
            }

//            Obtain the 2nd Vector (direction of increasing rows)
            if (norm2FovPlaneX > norm2FovPlaneY && norm2FovPlaneX > norm2FovPlaneZ) {
                fovV2 = Point3D.crossProduct(fovV1, fovV3); //Sagital case (The sagital case has a left-handed oriented referencial)
            } else {
                fovV2 = Point3D.crossProduct(fovV3, fovV1); //Transversal and Coronal cases
            }
            fovV2.normalize();

            double dwellTime = Double.valueOf(readHeaderValue(header, "sRXSPEC.alDwellTime[0]")); //nanoseconds
            String removeOversampling = readHeaderValue(header, "sSpecPara.ucRemoveOversampling");
            if (removeOversampling.matches("0x1")) {
                spectralWidth = 1000000000 / (dwellTime * 2); //Hz
            } else {
                spectralWidth = 1000000000 / (dwellTime); //Hz
            }

            String tagaux;
            double taux;
            double o1aux;
            double o2aux;
            double o3aux;
            double p1aux;
            double p2aux;
            double p3aux;

            for (int i = 0; i < 15; i++) {
                tagaux = "sRSatArray.asElm[" + i + "].dThickness";
                taux = Double.valueOf(readHeaderValue(header, tagaux));
                if (taux != 0) {
                    tagaux = "sRSatArray.asElm[" + i + "].sNormal.dSag";
                    o1aux = Double.valueOf(readHeaderValue(header, tagaux));
                    tagaux = "sRSatArray.asElm[" + i + "].sNormal.dCor";
                    o2aux = Double.valueOf(readHeaderValue(header, tagaux));
                    tagaux = "sRSatArray.asElm[" + i + "].sNormal.dTra";
                    o3aux = Double.valueOf(readHeaderValue(header, tagaux));

                    tagaux = "sRSatArray.asElm[" + i + "].sPosition.dSag";
                    p1aux = Double.valueOf(readHeaderValue(header, tagaux));
                    tagaux = "sRSatArray.asElm[" + i + "].sPosition.dCor";
                    p2aux = Double.valueOf(readHeaderValue(header, tagaux));
                    tagaux = "sRSatArray.asElm[" + i + "].sPosition.dTra";
                    p3aux = Double.valueOf(readHeaderValue(header, tagaux));
                    saturationBands.add(new SaturationBand(taux,
                            Double.valueOf(p1aux),
                            Double.valueOf(p2aux),
                            Double.valueOf(p3aux),
                            Double.valueOf(o1aux),
                            Double.valueOf(o2aux),
                            Double.valueOf(o3aux)));
                }
            }

        } 
        else if ( fileType == 2 ) 
        {
            if ( !readHeaderValue( header, "(0028,9108)" ).matches( "COMPLEX" ) ) 
            {
                model.printTxt( "Data representation not COMPLEX! (DICOM Tag (0028,9108))" );
            } 
            else 
            {
                //    MJ
                String fft = readHeaderValue( header, "(2005,1340)" );
                if ( fft.equals( "POST_FT" ) ) 
                {
                    FFTDone = true;
                }
                spectralWidth = Double.valueOf( readHeaderValue( header, "(0018,9052)" ) ); // spectral width in Hz
                B0 = Double.valueOf( readHeaderValue( header, "(0018,0087)" ) );
                if ( B0 == 0 ) 
                {
                    B0 = Double.valueOf( readHeaderValue( header, "(2001,1085)" ) );
                }
                
                TE = Double.valueOf( readHeaderValue( header, "(0018,9114)->(0018,9082)" ) );
                TR = Double.valueOf( readHeaderValue( header, "(0018,9112)->(0018,0080)" ) );
                m_ref_imageUID1 = readHeaderValue( header, "(0008,1140)[0]->(0008,1155)" );
                m_ref_imageUID2 = readHeaderValue( header, "(0008,1140)[1]->(0008,1155)" );
                m_ref_imageUID3 = readHeaderValue( header, "(0008,1140)[2]->(0008,1155)" );

                headerLine = (DicomHeaderLine) header.get( "(5600,0020)" );
                rawDataByte = headerLine.ValueByte;
                m_numberOfTimeSamplePoints = Integer.valueOf( readHeaderValue( header, "(0028,9002)" ) );

                m_resonanceFrequency = Double.valueOf( readHeaderValue(header, "(0018,9098)" ) ) * 1000000;
                ppmReference = Double.valueOf(readHeaderValue( header, "(0018,9053)") );
                nucleus = readHeaderValue( header, "(0018,9100)");

                nrVoxelsDir2 = Integer.valueOf( readHeaderValue(header, "(0028,0010)") ); // Nr Rows
                nrVoxelsDir1 = Integer.valueOf( readHeaderValue(header, "(0028,0011)") ); // Nr Columns
                nrVoxelsDir3 = Integer.valueOf( readHeaderValue(header, "(0028,0008)") ); // Nr Frames
               
                int nrSlices = Integer.valueOf( readHeaderValue(header, "(2001,1018)") ); //Philips format
                
                if ( nrSlices > 0 ) 
                {
                    nrSpectraPerVoxel = nrVoxelsDir3 / nrSlices;
                    nrVoxelsDir3 = nrSlices;
                } 
                else 
                {
                    nrSpectraPerVoxel = 1;
                }

                //NOTE: Currently if a tag has more than 2 levels, only the last two are used as key in the header object.
//            THIS SHOULD CHANGE! Example: tag (5200,9229)->(0028,9110)->(0018,0050) uses the key (0028,9110)->(0018,0050)
                //FOV variables - information contained in group tags 5200,9229 and 5200,9230:
//            (5200,9229)->(0020,9116)	Plane Orientation Sequence
//            (5200,9229)->(0020,9116)->(0020,0037)	Image Orientation (Patient)
//            (5200,9229)->(0028,9110)	Pixel Measures Sequence
//            (5200,9229)->(0028,9110)->(0018,0050)	Slice Thickness
//            (5200,9229)->(0028,9110)->(0028,0030)	Pixel Spacing
//            (5200,9230)	Per-frame Functional Groups Sequence
//            (5200,9230)->(0020,9111)	Frame Content Sequence
//            (5200,9230)->(0020,9111)->(0018,9074)	Frame Acquisition DateTime
//            (5200,9230)->(0020,9111)->(0018,9151)	Frame Reference DateTime
//            (5200,9230)->(0020,9111)->(0018,9220)	Frame Acquisition Duration
//            (5200,9230)->(0020,9111)->(0020,9156)	Frame Acquisition Number
//            (5200,9230)->(0020,9113)	Plane Position Sequence
//            (5200,9230)->(0020,9113)->(0020,0032)	Image Position (Patient)
                //VOI variables - tags grouped in tag 0018,9126
//            (0018,9126)	Volume Localization Sequence
//            (0018,9126)[0]->(0018,9104)	Slab Thickness
//            (0018,9126)[0]->(0018,9105)	Slab Orientation
//            (0018,9126)[0]->(0018,9106)	Mid Slab Position
//            (0018,9126)[1]->(0018,9104)	Slab Thickness
//            (0018,9126)[1]->(0018,9105)	Slab Orientation
//            (0018,9126)[1]->(0018,9106)	Mid Slab Position
//            (0018,9126)[2]->(0018,9104)	Slab Thickness
//            (0018,9126)[2]->(0018,9105)	Slab Orientation
//            (0018,9126)[2]->(0018,9106)	Mid Slab Position
                voiSize = new Point3D( );
                voiSize.m_x1 = Double.valueOf( readHeaderValue( header, "(0018,9126)[0]->(0018,9104)" ) );
                voiSize.m_x2 = Double.valueOf( readHeaderValue( header, "(0018,9126)[1]->(0018,9104)" ) );
                voiSize.m_x3 = Double.valueOf( readHeaderValue( header, "(0018,9126)[2]->(0018,9104)" ) );
                
                String[] positionTxt = readHeaderValue( header, "(0018,9126)[0]->(0018,9106)").split("\\\\" );
                
                if ( positionTxt.length == 3 )
                {
                    voiPosition = new Point3D(Double.valueOf( positionTxt[0]), Double.valueOf( positionTxt[1]), Double.valueOf(positionTxt[2] ) );
                }
                else
                {
                    System.out.println( "Voxel postion could not be read!" );
                    positionTxt = new String[3];
                    positionTxt[0] = "0.0";
                    positionTxt[1] = "0.0";
                    positionTxt[2] = "0.0";
                }
                
                String[] v1Txt = readHeaderValue(header, "(0018,9126)[0]->(0018,9105)").split("\\\\");
                String[] v2Txt = readHeaderValue(header, "(0018,9126)[1]->(0018,9105)").split("\\\\");
                String[] v3Txt = readHeaderValue(header, "(0018,9126)[2]->(0018,9105)").split("\\\\");
                if ( v1Txt.length == 3 )
                {
                    voiV1 = new Point3D( Double.valueOf( v1Txt[0] ), 
                                         Double.valueOf( v1Txt[1] ), 
                                         Double.valueOf( v1Txt[2] ) );
                }
                else
                {
                    voiV1 = new Point3D( 0.0, 0.0, 0.0 );
                }
                
                if ( v2Txt.length == 3 )
                {
                    voiV2 = new Point3D( Double.valueOf( v2Txt[0] ), 
                                         Double.valueOf( v2Txt[1] ), 
                                         Double.valueOf( v2Txt[2] ) );
                }
                else
                {
                    voiV2 = new Point3D( 0.0, 0.0, 0.0 );
                }
                
                if ( v3Txt.length == 3 )
                {
                    voiV3 = new Point3D( Double.valueOf( v3Txt[0] ), 
                                         Double.valueOf( v3Txt[1] ), 
                                         Double.valueOf( v3Txt[2] ) );
                }
                else
                {
                    voiV3 = new Point3D( 0.0, 0.0, 0.0 );
                }
                
                
                String[] pixelSpacingTxt = readHeaderValue( header, "(0028,9110)->(0028,0030)" ).split( "\\\\" );
                double sliceThickness = Double.valueOf( readHeaderValue( header, "(0028,9110)->(0018,0050)" ) );
                Point3D voxelSpacing = new Point3D( Double.valueOf( pixelSpacingTxt[0] ), 
                                                    Double.valueOf( pixelSpacingTxt[1] ), 
                                                    sliceThickness );

//                if (sliceThickness == 0d) { // DO WE REALLY NEED THIS???
//                    sliceThickness = Double.valueOf(PixelSpacing[0]);
//                }

                fovSize      = new Point3D( );
                fovSize.m_x1 = voxelSpacing.m_x1 * nrVoxelsDir1; // Adjacent row spacing
                fovSize.m_x2 = voxelSpacing.m_x2 * nrVoxelsDir2; // Adjacent column spacing
                fovSize.m_x3 = sliceThickness * nrVoxelsDir3;

                String[] orientationFOV = readHeaderValue( header, "(0020,9116)->(0020,0037)").split("\\\\" );
                fovV1 = new Point3D( Double.valueOf( orientationFOV[0] ), 
                                     Double.valueOf( orientationFOV[1] ), 
                                     Double.valueOf( orientationFOV[2] ) );
                fovV2 = new Point3D( Double.valueOf( orientationFOV[3] ), 
                                     Double.valueOf( orientationFOV[4] ), 
                                     Double.valueOf( orientationFOV[5] ) );
                
                fovV3 = Point3D.crossProduct( fovV1, fovV2 ); //TODO: Needs Phantom Validation
                fovV3 = Point3D.crossProduct( fovV2, fovV1 ); //test
                
//              The position of the FOV is given by the conventional Image Position tag. This tag gives the position of the center
//              of the top left voxel. This is not the case in Siemens, where we obtain directly the center of the FOV. Therefore here
//              we need to calculate the center of the FOV.
                String[] firstPixelPos = readHeaderValue( header, "(0020,9113)->(0020,0032)").split("\\\\" );
                fovPosition = new Point3D( Double.valueOf( firstPixelPos[0] ), 
                                           Double.valueOf( firstPixelPos[1] ), 
                                           Double.valueOf( firstPixelPos[2] ) );
                
                fovPosition.translate( Point3D.vectorCombination( fovV1, fovV2, fovV3, 
                                                                  voxelSpacing.m_x1 * ( nrVoxelsDir1 - 1 ) / 2,
                                                                  voxelSpacing.m_x2 * ( nrVoxelsDir2 - 1 ) / 2, 
                                                                  voxelSpacing.m_x3 * ( nrVoxelsDir3 - 1 ) / 2 ) );



                if ( ( voiSize.m_x1 == 0.0 ) && 
                     ( voiSize.m_x2 == 0.0 ) &&
                     ( voiSize.m_x3 == 0.0 ) )
                {
                    voiSize.m_x1 = fovSize.m_x1;
                    voiSize.m_x2 = fovSize.m_x2;
                    voiSize.m_x3 = fovSize.m_x3;      
                }




//                // THE NEXT RULES SHOULD NOT BE NEEDED GIVEN THAT WE HAVE NOW A CLEAR SEPARATION BETWEEN FOV AND VOI
//                // AND THAT THE DIRECTIONS OF THE FOV SHOULD NOT DEPEND ON THE VOLUME OF THE VOI...
//                if (voiSize.m_x2 > 0 && voiSize.m_x3 > 0) {//TO DO 14 Jun 2017: Validate this rule with Phantom data
//                    fovV1 = new Point3D(Double.valueOf(orientationFOV[0]), Double.valueOf(orientationFOV[1]), Double.valueOf(orientationFOV[2]));
//                    fovV2 = new Point3D(Double.valueOf(orientationFOV[3]), Double.valueOf(orientationFOV[4]), Double.valueOf(orientationFOV[5]));
//                    fovV3 = Point3D.crossProduct(fovV1, fovV2); //TO DO: Needs Phantom Validation
//
//                    //BEGIN Jana: first pixel expected in -N/2.*pixel_size+pixel_size/2 i.e. in -(N-1)/2.*pixel_size
//                    // new center is  first_pixel_pos-(-(N-1)/2.*pixel_size) i.e. first_pixel_pos+(N-1)/2.*pixel_size
//                    String[] firstPixelPos = readHeaderValue(header, "(0020,9113)->(0020,0032)").split("\\\\");
//                    fovPosition = new Point3D();
//                    fovPosition.m_x1 = Double.valueOf(firstPixelPos[0]) + Double.valueOf(PixelSpacing[0]) * (nrVoxelsDir1 - 1) / 2.;
//                    fovPosition.m_x2 = Double.valueOf(firstPixelPos[1]) + Double.valueOf(PixelSpacing[1]) * (nrVoxelsDir2 - 1) / 2.;
//                    // not sure about this (if water and metabolite in one file, it would fail
//                    //m_dTraVoxelPositionGrid = Double.valueOf(firstPixelPos[2])-sliceThickness * (m_dir3Points-1)/2.;  
//                    // therefore 
//                    fovPosition.m_x3 = Double.valueOf(firstPixelPos[2]);
//                    //END Jana
//                } else {
//                    // For Philips: do not why row and column are flipped in "(0020,9116)->(0020,0037)" 
//                    // maybe different for other vendors
//                    fovV2 = new Point3D(Double.valueOf(orientationFOV[0]), Double.valueOf(orientationFOV[1]), Double.valueOf(orientationFOV[2]));
//                    fovV1 = new Point3D(Double.valueOf(orientationFOV[3]), Double.valueOf(orientationFOV[4]), Double.valueOf(orientationFOV[5]));
//                    
//                    //TODO 14 Jun 2017:
//                    //Check If FOV.m_x1 needs to change with FOV.m_x2 and same thing for other variables like grid dims and so on... 
//                    fovPosition = voiPosition.Clone();
//                }
                String tagaux;
                String slabOrient[];
                String slabPosit[];
                double slabThick;

                for ( int i = 0; i < 15; i++ ) 
                {
                    tagaux = "(0018,9107)[" + i + "]->(0018,9104)"; //Slab thickness
                    slabThick = Double.valueOf( readHeaderValue( header, tagaux ) );
                    if ( slabThick > 0 ) 
                    {
                        tagaux = "(0018,9107)[" + i + "]->(0018,9105)"; //Slab orientation
                        slabOrient = readHeaderValue( header, tagaux ).split( "\\\\" );
                        tagaux = "(0018,9107)[" + i + "]->(0018,9106)"; //Mid slab position
                        slabPosit = readHeaderValue(header, tagaux).split("\\\\");
                        saturationBands.add( new SaturationBand( slabThick,
                                             Double.valueOf( slabPosit[0] ),
                                             Double.valueOf( slabPosit[1] ),
                                             Double.valueOf( slabPosit[2] ),
                                             Double.valueOf( slabOrient[0] ),
                                             Double.valueOf( slabOrient[1] ),
                                             Double.valueOf( slabOrient[2] ) ) );
                    }
                }

            }
        }

        if ( nrVoxelsDir1 == 0 ) 
        {
            nrVoxelsDir1 = 1;
        }
        
        if ( nrVoxelsDir2 == 0 ) 
        {
             nrVoxelsDir2 = 1;
        }
        
        if ( nrVoxelsDir3 == 0 ) 
        {
            nrVoxelsDir3 = 1;
        }

        csigrid = new CSIGrid( fovPosition, fovV1, fovV2, fovV3,
                               nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3,
                               fovSize.m_x1, fovSize.m_x2, fovSize.m_x3 );

        // Create excited volume voxel
        double dim1 = voiSize.m_x1 / 2;
        double dim2 = voiSize.m_x2 / 2;
        double dim3 = voiSize.m_x3 / 2;

        Point3D p5 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, -dim3 );
        Point3D p3 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, dim3 );
        Point3D p6 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, -dim2, -dim3 );
        Point3D p1 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, dim2, -dim3 );
        Point3D p8 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, -dim2, dim3 );
        Point3D p2 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, dim2, -dim3 );
        Point3D p4 = Point3D.vectorCombination( voiV1, voiV2, voiV3, -dim1, dim2, dim3 );
        Point3D p7 = Point3D.vectorCombination( voiV1, voiV2, voiV3, dim1, -dim2, dim3 );

        p1.translate( voiPosition );
        p2.translate( voiPosition );
        p3.translate( voiPosition );
        p4.translate( voiPosition );
        p5.translate( voiPosition );
        p6.translate( voiPosition );
        p7.translate( voiPosition );
        p8.translate( voiPosition );

        voi = new Cube3D( p1, p2, p3, p4, p5, p6, p7, p8 );

        csigrid.setExcitationVoxel( voi );

        for ( int i = 0; i < saturationBands.size( ); i++ ) 
        {
            csigrid.addSaturationBand( saturationBands.get( i ).p1, 
                                       saturationBands.get( i ).p2, 
                                       saturationBands.get( i ).p3, 
                                       saturationBands.get( i ).p4, 
                                       saturationBands.get( i ).p5,
                                       saturationBands.get( i ).p6, 
                                       saturationBands.get( i ).p7, 
                                       saturationBands.get( i ).p8);
        }
    }

    /**
     * Return the array which contains the real as well as the imaginary channel
     * of the CSIData object.
     *
     * @return
     */
    public float[] getComplexResponse( ) 
    {
        return m_bothChannelsTD;
    }

    /**
     * Returns the absorption channel only of this CSIData object.
     *
     * @return
     */
    public float[] getAbsorptionChannelTDraw( ) 
    {
        return m_absorptionChannelTDraw;
    }

    /**
     * Returns the dispersion channel only of this CSIData object.
     *
     * @return
     */
    public float[] getDispersionChannelTDraw( ) 
    {
        return m_dispersionChannelTDraw;
    }

    /**
     * This method adds the complex time domain responses of singleInstance to
     * the already existing time responses.
     *
     * @param singleInstance
     */
    void addResponse( float[] responseToAdd ) 
    {

        for ( int i = 0; i < m_bothChannelsTD.length; i++ ) 
        {
            m_bothChannelsTD[i] = m_bothChannelsTD[i] + responseToAdd[i];
        }

        for ( int i = 0; i < ( m_bothChannelsTD.length / 2 ) - 1; i++ ) 
        {
            m_absorptionChannelTDraw[i] = m_bothChannelsTD[2 * i];
            m_dispersionChannelTDraw[i] = m_bothChannelsTD[2 * i + 1];
        }

        // Spectra object should be updated as well.
        spectra = new Spectra( m_absorptionChannelTDraw, 
                               m_dispersionChannelTDraw,
                               nrVoxelsDir1, 
                               nrVoxelsDir2, 
                               nrVoxelsDir3, 
                               spectralWidth,
                               m_resonanceFrequency, 
                               ppmReference, 
                               FFTDone );

        model = new GlobalModel( );
        model.setSpectra( spectra );
        model.setCSIData( this );
        model.setCSIGrid( csigrid );
    }

    /**
     * This method subtracts the complex time domain responses of singleInstance
     * to the already existing time responses.
     *
     * @param singleInstance
     */
    void subtractResponse(float[] responseToSubtract) {
        // but conjugate
        for (int i = 0; i < (m_bothChannelsTD.length / 2) - 1; i++) {
            m_bothChannelsTD[2 * i] = m_bothChannelsTD[2 * i] - responseToSubtract[2 * i];
            m_bothChannelsTD[2 * i + 1] = m_bothChannelsTD[2 * i + 1] - responseToSubtract[2 * i + 1];
        }

        for (int i = 0; i < (m_bothChannelsTD.length / 2) - 1; i++) {
            m_absorptionChannelTDraw[i] = m_bothChannelsTD[2 * i];
            m_dispersionChannelTDraw[i] = m_bothChannelsTD[2 * i + 1];
        }

        // Spectra object should be updated as well
        spectra = new Spectra(m_absorptionChannelTDraw, m_dispersionChannelTDraw,
                nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, spectralWidth,
                m_resonanceFrequency, ppmReference, FFTDone);

        // finally the model
        model = new GlobalModel();
        model.setSpectra(spectra);
        model.setCSIData(this);
        model.setCSIGrid(csigrid);
    }

    /**
     * Invert the TD response.
     */
    void invertResponse() {
        // inversion and conjugation: only invert the real part
        for (int i = 0; i < m_bothChannelsTD.length / 2 - 1; i++) {
            m_bothChannelsTD[2 * i] = -m_bothChannelsTD[2 * i];
        }
        // conjugation
        for (int i = 0; i < (m_bothChannelsTD.length / 2) - 1; i++) {
            m_absorptionChannelTDraw[i] = m_bothChannelsTD[2 * i];
            m_dispersionChannelTDraw[i] = m_bothChannelsTD[2 * i + 1];
        }

        // Spectra object should be updated as well
        spectra = new Spectra(m_absorptionChannelTDraw, m_dispersionChannelTDraw,
                nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, spectralWidth,
                m_resonanceFrequency, ppmReference, FFTDone);

        // finally the model
        model = new GlobalModel();
        model.setSpectra(spectra);
        model.setCSIData(this);
        model.setCSIGrid(csigrid);
    }

    /**
     * Conjugates the signals.
     */
    public void conjugate() {
        // inversion and conjugation: only invert the real part
        for (int i = 0; i < m_bothChannelsTD.length / 2 - 1; i++) {
            m_bothChannelsTD[2 * i + 1] = -m_bothChannelsTD[2 * i + 1];
        }

        // conjugation
        for (int i = 0; i < (m_bothChannelsTD.length / 2) - 1; i++) {
            m_absorptionChannelTDraw[i] = m_bothChannelsTD[2 * i];
            m_dispersionChannelTDraw[i] = m_bothChannelsTD[2 * i + 1];
        }

        // Spectra object should be updated as well
        spectra = new Spectra(m_absorptionChannelTDraw, m_dispersionChannelTDraw,
                nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, spectralWidth,
                m_resonanceFrequency, ppmReference, FFTDone);

        // Finally the model
        model = new GlobalModel();
        model.setSpectra(spectra);
        model.setCSIData(this);
        model.setCSIGrid(csigrid);
    }

    /**
     * Sets all responses to zero without changing their dimensions.
     */
    void setResponseToZero() {
        // inversion and conjugation: only invert the real part
        for (int i = 0; i < m_bothChannelsTD.length - 1; i++) {
            m_bothChannelsTD[i] = 0f;
        }

        // Set all responses also to zero...
        for (int i = 0; i < (m_bothChannelsTD.length / 2) - 1; i++) {
            m_absorptionChannelTDraw[i] = 0f;
            m_dispersionChannelTDraw[i] = 0f;
        }

        // Spectra object should be updated as well
        spectra = new Spectra(m_absorptionChannelTDraw, m_dispersionChannelTDraw,
                nrVoxelsDir1, nrVoxelsDir2, nrVoxelsDir3, spectralWidth,
                m_resonanceFrequency, ppmReference, FFTDone);

        // Finally the model
        model = new GlobalModel();
        model.setSpectra(spectra);
        model.setCSIData(this);
        model.setCSIGrid(csigrid);
    }
    
    /**
     * Added as utility attribute to access the study description tag.
     * @return 
     */
    public String getStudyDescription( ) 
    {
       return studyDescription;
    }

    public String getAccessionNumber() {
        return accessionNumber;
    }

    public String getPatientBirthDate() {
       return patientDOB;
    }

    public String getPatientSex() {
        return patientSex;
    }

    public String getStudyTime() {
        return studyTime;
    }

    public String getSeriesTime() {
        return seriesTime;
    }

    public String getScannerName() {
        return scannerName;
    }

    public String getProtocolName() {
        return protocolName;
    }

}
