/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.decisionforest;

import org.spectrim.signalprocessing.RandomMinimizerDouble;
import org.spectrim.decisionforest.structure.Node;

/**
 *
 * @author nunopbarros
 */
public class CentroidFinder extends RandomMinimizerDouble {

    private Node node;

    public CentroidFinder(Node node) {
        this.node = node;
    }

    @Override
    public double fnc(double[] par) {

        double fncVal = 0;

        int nrvaraux = node.getNrVar();
        double[] pointaux = new double[nrvaraux];
        int res;
        int sizedataelem = node.getDataElems().size();
        
        // Clear DataElems from Children
        for (int i = 0; i < node.getNrChildren(); i++) {
            node.getChild(i).clearDataElems();
        }
        
        // Split data for node's children acording to variables 
        for (int j = 0; j < sizedataelem; j++) {
            for (int k = 0; k < nrvaraux; k++) {
                pointaux[k] = node.getDataSet().get(node.getDataElems().get(j)).variables[node.getVarNode().get(k)];
            }
            res = split(par, pointaux);
            node.getChild(res).addDataElem(node.getDataElems().get(j), true);//calc entropy here
        }

        // Calculate Information Gain after Split
        fncVal = node.getEntropy();

        for (int i = 0; i < node.getNrChildren(); i++) {
            fncVal -= ((double) node.getChild(i).getDataElems().size() / (double) sizedataelem) * node.getChild(i).getEntropy();
        }
//        System.out.println(fncVal);

        if (fncVal == 0) {
            return Double.POSITIVE_INFINITY;
        } else {
            return 1 / fncVal;
        }

    }

    int split(double[] paramlist, double[] point) {
        int res = 0;
        int nrvaraux = node.getNrVar();
        double mindist = 0;
        double distaux;
        double[] v1 = new double[nrvaraux];

        for (int i = 0; i < node.getNrChildren(); i++) {
            for (int j = 0; j < nrvaraux; j++) {
                v1[j] = paramlist[i * nrvaraux + j];
            }

            distaux = distance(v1, point);
            if (i == 0) {
                res = 0;
                mindist = distaux;
            } else if (distaux < mindist) {
                res = i;
                mindist = distaux;
            }
        }

        return res;
    }

    double distance(double[] v1, double[] v2) {
        double val = 0;

        for (int i = 0; i < v1.length; i++) {
            val += Math.abs(v1[i] - v2[i]);
        }
        return (double) val;
    }
}
