    /*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.decisionforest.structure;

import org.spectrim.lib.DoubleRange;
import com.mongodb.BasicDBObject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ThreadLocalRandom;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.spectrim.decisionforest.data.DataSet;
import org.spectrim.decisionforest.exceptions.DimSizeBiggerThanDatasetSize;
import org.spectrim.decisionforest.CentroidFinder;
import org.spectrim.decisionforest.CentroidWeightFinder;
import org.spectrim.decisionforest.results.ResultMap;
import org.spectrim.decisionforest.results.ResultSet;
import org.spectrim.decisionforest.results.ResultVect;

/**
 *
 * @author nunopbarros
 */
public class Node implements java.io.Serializable {

    public int id;
    public int level;
//    private int subsetdim;
    public double infgain;
    public DataSet dataset;
    public ArrayList<Integer> dataelem; // positions of elements from Forest Dataset that were "dropped" in a certain node
    public ArrayList<Integer> varnode;
//    public int nrvarnode;
    public ArrayList<Node> childList;
//    public int nrchildren;
    public ArrayList<Double> paramnode;
    public String endnodeclass;
    public HashMap<String, Integer> trainingPopulation = new HashMap<>();
    private int nrElements;
    public int nrElementsTraining;
    public Tree parent;
    public double entropy;
    public double[] centroidWeights;
    public static final int WEAKLEARN_IGMAX = 0;
    public static final int WEAKLEARN_MEAN = 1;
    public static final int WEAKLEARN_MEDIAN = 2;

    public Node(Tree tree, int id, int level)
            throws DimSizeBiggerThanDatasetSize {
        this.parent = tree;
        this.id = id;
        this.level = level;
        this.childList = new ArrayList<>();
        this.varnode = new ArrayList<>();
        this.setRandomVarSet();
        this.dataelem = new ArrayList<>();
        this.paramnode = new ArrayList<>();
    }

    public Node() {
    }

    public BasicDBObject save() {

        BasicDBObject dbvarnode = new BasicDBObject();
        for (int i = 0; i < this.varnode.size(); i++) {
            dbvarnode.append(String.valueOf(i), this.varnode.get(i));
        }
        BasicDBObject dbchildList = new BasicDBObject();
        for (int i = 0; i < this.childList.size(); i++) {
            dbchildList.append(String.valueOf(i), this.childList.get(i).getId());
        }
        BasicDBObject dbparamnode = new BasicDBObject();
        for (int i = 0; i < this.paramnode.size(); i++) {
            dbparamnode.append(String.valueOf(i), this.paramnode.get(i));
        }
        BasicDBObject dbcentroidweights = new BasicDBObject();
        for (int i = 0; i < this.centroidWeights.length; i++) {
            dbcentroidweights.append(String.valueOf(i), this.centroidWeights[i]);
        }

        BasicDBObject node = new BasicDBObject().
                append("id", this.id).
                append("level", this.level).
                append("infgain", this.infgain).
                append("varnode", dbvarnode).
                //                append("nrvarnode", this.nrvarnode).
                //            append("childList", dbchildList).
                //                append("nrchildren", this.nrchildren).
                append("paramnode", dbparamnode).
                append("endnodeclass", this.endnodeclass).
                append("nrElements", this.nrElementsTraining).
                append("centroidWeights", dbcentroidweights);

        List<BasicDBObject> listNodeChildren = new ArrayList<>();
        for (Node child : childList) {
            listNodeChildren.add(child.save());
        }
        node.append("children", listNodeChildren);

        List<BasicDBObject> listTrainingPopulation = new ArrayList<>();
        Iterator<String> iter = this.trainingPopulation.keySet().iterator();
        int i = 0;
        while (iter.hasNext()) {
            String klasseaux = iter.next();
            listTrainingPopulation.add(new BasicDBObject().append("" + i, klasseaux + "%%%" + this.trainingPopulation.get(klasseaux)));
            i++;
        }
        node.append("trainingPopulation", listTrainingPopulation);
        return node;
    }

    public int getLevel() {
        return this.level;
    }

    public int getId() {
        return this.id;
    }

    public void addChild() throws DimSizeBiggerThanDatasetSize {
        Node nodechild = new Node(parent, parent.nodenum, this.getLevel() + 1);
        parent.nodenum++;
        this.childList.add(nodechild);
    }

    public Node getChild(int pos) {
        if (childList.isEmpty()) {
            return null;
        } else {
            return this.childList.get(pos);
        }
    }

    public boolean hasChildren() {
        if (childList.isEmpty()) {
            return false;
        } else {
            return true;
        }
    }

    public int getNrChildren() {
        return this.childList.size();
    }

    private void setRandomVarSet() throws DimSizeBiggerThanDatasetSize {
        int randn;
        ThreadLocalRandom randgen = ThreadLocalRandom.current();
        ArrayList<Integer> notUsedVars = new ArrayList<>();
        for (int i = 0; i < parent.nrVars; i++) {
            notUsedVars.add(i);
        }

        if (parent.nrVars == parent.nrVarsNode) {
            this.varnode = (ArrayList<Integer>) notUsedVars;
        } else if (parent.nrVarsNode > parent.nrVars) {
            throw new DimSizeBiggerThanDatasetSize();
        } else {
            for (int i = 0; i < parent.nrVarsNode; i++) {
                randn = randgen.nextInt(notUsedVars.size());
                this.varnode.add(notUsedVars.get(randn));
                notUsedVars.remove(randn);
            }
        }
//        System.out.println("Node Id: " + this.id + " level: " + this.level + " vars node: (" + this.varnode.toString() + ")");
    }

    public void setDataSet(DataSet dataset) {
        this.dataset = dataset;
    }

    public DataSet getDataSet() {
        return this.dataset;
    }

    public void setDataElems(ArrayList<Integer> dataelem, boolean update) {
        this.dataelem = (ArrayList<Integer>) dataelem.clone();
        if (update) {
            this.updateCurrentPopulation();
        }
    }

    public boolean hasDataElems() {
        return !this.dataelem.isEmpty();
    }

    public void addDataElem(Integer newelem, boolean updatePop) {
        this.dataelem.add(newelem);
        if (updatePop) {
            String klasse = this.dataset.get(newelem).klasse;
            if (trainingPopulation.containsKey(klasse)) {
                trainingPopulation.put(klasse, trainingPopulation.get(klasse) + 1);
            } else {
                trainingPopulation.put(klasse, 1);
            }
            this.nrElements++;
        }
    }

    public void clearDataElems() {
        this.dataelem.clear();
        nrElements = 0;
        trainingPopulation.clear();
    }

    public ArrayList<Integer> getDataElems() {
        return this.dataelem;
    }

    public ArrayList<Integer> getVarNode() {
        return this.varnode;
    }

    public int split(double[] paramlist, double[] point) {
        int res = 0;
        double mindist = 0;
        double distaux;
        int varnode = this.paramnode.size() / this.childList.size();
        double[] v1 = new double[varnode];

        for (int i = 0; i < this.getNrChildren(); i++) {
            for (int j = 0; j < varnode; j++) {
                v1[j] = paramlist[i * varnode + j];
            }

//            distaux = centroidWeights[i] * distance(v1, point);
            distaux = distance(v1, point);
            if (i == 0) {
                res = 0;
                mindist = distaux;
            } else if (distaux < mindist) {
                res = i;
                mindist = distaux;
            }
        }

        return res;
    }

    public double distance(double[] v1, double[] v2) {
        double val = 0;

        for (int i = 0; i < v1.length; i++) {
            val += Math.abs(v1[i] - v2[i]);
        }
        return (double) val;
    }

    public int getNrVar() {
        return parent.nrVarsNode;
    }

    public final void train(int nriter, int nrsamples, int minsize, int weaklearner, boolean optimizeWeights) {
        int nrchildnodes = parent.nchilds;
        int sizedataelem = dataelem.size();
        int nrClasses = 0;
//        HashMap<String, Integer> counter = new HashMap<>();
//        Iterator<String> iter;

//        for (int n = 0; n < sizedataelem; n++) {
//            String label = dataset.get(dataelem.get(n)).klasse;
//            if (counter.containsKey(label)) {
//                counter.put(label, counter.get(label) + 1);
//            } else {
//                counter.put(label, 1);
//            }
//        }
        if (sizedataelem == 0 && trainingPopulation.size() != 0) {
            System.out.println("Ouooooo");
        }
//       Stop division of nodes when nr of dataelements is smaller than a certain limit to avoid overfitting.
        if (sizedataelem > minsize) {
            nrClasses = trainingPopulation.size();

//       Given the nr of children adapt the nr of child nodes. It is nonsense to have more child nodes than the nr of existant classes.
            if (nrClasses < nrchildnodes) {
//                int extra = nrchildren - nrClasses;
//                for (int k = 0; k < extra; k++) {
////                    childList.remove(k);
//                    nrchildren--;
//                }
                nrchildnodes = nrClasses;
            }
        }

//       Check if node only has elements of one class. In that case is impossible to reduce entropy.
        if (nrClasses > 1 && !parent.isMaxSplitReached()) { // Terminal node?
            parent.incrementSplitCounter();
            for (int i = 0; i < nrchildnodes; i++) {
                try {
                    this.addChild();
                } catch (DimSizeBiggerThanDatasetSize ex) {
                    Logger.getLogger(Node.class.getName()).log(Level.SEVERE, null, ex);
                }
            }

            for (int j = 0; j < nrchildnodes; j++) {
                this.getChild(j).setDataSet(dataset);
            }

            // Determine Mean, Min and Max of variables.
            double[] mindata = new double[parent.nrVarsNode];
            double[] maxdata = new double[parent.nrVarsNode];
            double[] meanPar = new double[parent.nrVarsNode * nrchildnodes];
            double[] resPar = null;
            double valaux;

            for (int j = 0; j < sizedataelem; j++) {
                for (int k = 0; k < parent.nrVarsNode; k++) {
                    valaux = (double) dataset.get(dataelem.get(j)).variables[varnode.get(k)];
                    if (j == 0) {
                        mindata[k] = valaux;
                        maxdata[k] = valaux;
                    }
                    if (valaux < mindata[k]) {
                        mindata[k] = valaux;
                    }
                    if (valaux > maxdata[k]) {
                        maxdata[k] = valaux;
                    }
                }
            }

            DoubleRange[] parameterRanges = new DoubleRange[parent.nrVarsNode * nrchildnodes];
            for (int k = 0; k < nrchildnodes; k++) {
                for (int j = 0; j < parent.nrVarsNode; j++) {
                    parameterRanges[j + k * parent.nrVarsNode] = new DoubleRange(mindata[j], maxdata[j]);
                    meanPar[j + k * parent.nrVarsNode] = (mindata[j] + maxdata[j]) / 2;
                }
            }

            if (Arrays.equals(mindata, maxdata)) { // If using these variables we cannot 
//                divide the points but we have more than one class we just pass it to
//                ONE child node, which may contain different variables and may be able
//                to have some information gain (the extra child nodes are deleted further)
                resPar = new double[parent.nrVarsNode]; // No parameters are needed
                for (int j = 0; j < sizedataelem; j++) {
                    this.getChild(0).addDataElem(this.dataelem.get(j), true);
                }
                infgain = 0;

            } else {
                if (weaklearner == Node.WEAKLEARN_MEAN || weaklearner == Node.WEAKLEARN_MEDIAN) {
//               Get #NrChildren biggest classes (It is already guaranteed that there aren't 
//                    more childs than classes)         
                    LinkedHashMap<String, Integer> orderedClasses = sortByValue(trainingPopulation);
                    Object[] biggestClasses = orderedClasses.keySet().toArray();
                    resPar = new double[parent.nrVarsNode * nrchildnodes];
                    if (weaklearner == Node.WEAKLEARN_MEAN) {
//                  Determine mean centers of the #NrChildren biggest classes

                        for (int l = 0; l < nrchildnodes; l++) {
                            for (int k = 0; k < parent.nrVarsNode; k++) {
                                for (int j = 0; j < sizedataelem; j++) {
                                    valaux = (double) dataset.get(dataelem.get(j)).variables[varnode.get(k)];
                                    String classe = dataset.get(dataelem.get(j)).klasse;
                                    if (classe.matches((String) biggestClasses[l])) {
                                        resPar[k + l * parent.nrVarsNode] += valaux / trainingPopulation.get(classe);
                                    }
                                }
                            }
                        }

                    } else { //MEDIAN

                        for (int l = 0; l < nrchildnodes; l++) {
                            for (int k = 0; k < parent.nrVarsNode; k++) {
                                ArrayList<Double> vals2order = new ArrayList<>();
                                for (int j = 0; j < sizedataelem; j++) {
                                    valaux = (double) dataset.get(dataelem.get(j)).variables[varnode.get(k)];
                                    String classe = dataset.get(dataelem.get(j)).klasse;
                                    if (classe.matches((String) biggestClasses[l])) {
                                        vals2order.add(valaux);
                                    }
                                }
                                Collections.sort(vals2order);
                                resPar[k + l * parent.nrVarsNode] = vals2order.get((vals2order.size() - 1) / 2); // get median
                            }
                        }

                    }

                } else if (weaklearner == Node.WEAKLEARN_IGMAX) {

                    CentroidFinder cFinder = new CentroidFinder(this);
                    cFinder.setStartParameters(meanPar); // the start parameter has no effect...
                    cFinder.setParameterRanges(parameterRanges);
//                    IGmax.setNumberOfSteps(nrchildren, 3);
                    cFinder.performRandomMinimization(nrsamples, nriter, false);
                    resPar = cFinder.getMinimizingParameters();
                    for (int i = 0; i < this.getNrChildren(); i++) {
                        this.getChild(i).clearDataElems();
                    }
                }
                this.paramnode.clear();
                for (int j = 0; j < nrchildnodes * parent.nrVarsNode; j++) {
                    this.paramnode.add(resPar[j]);
                }

//                // START TEMP ANALYSIS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//                centroidWeights = new double[nrchildnodes];
//                for (int h = 0; h < nrchildnodes; h++) {
//                    centroidWeights[h] = 1;
//                }
//                double[] pointaux2 = new double[parent.nrVarsNode];
//                for (int j = 0; j < sizedataelem; j++) {
//                    for (int k = 0; k < parent.nrVarsNode; k++) {
//                        pointaux2[k] = dataset.get(dataelem.get(j)).variables[varnode.get(k)];
//                    }
//                    int res = split(resPar, pointaux2);
//                    this.getChild(res).addDataElem(this.dataelem.get(j), true);
//                }
//
//                double infgainBefore = this.getEntropy();
//                for (int j = 0; j < nrchildnodes; j++) {
//                    if (!this.getChild(j).dataelem.isEmpty()) {
//                        infgainBefore -= ((double) this.getChild(j).dataelem.size() / (double) sizedataelem) * this.getChild(j).getEntropy();
//                    }
//                }
//                for (int i = 0; i < this.getNrChildren(); i++) {
//                    this.getChild(i).clearDataElems();
//                }
//                // END TEMP ANALYSIS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                //Centroid Weight Optimization
                if (optimizeWeights) {
                    CentroidWeightFinder cWFinder = new CentroidWeightFinder(this);
                    double[] startWeights = new double[nrchildnodes];
                    DoubleRange[] weightRanges = new DoubleRange[nrchildnodes];
                    for (int h = 0; h < nrchildnodes; h++) {
                        weightRanges[h] = new DoubleRange(0d, 1d);
                        startWeights[h] = 1;
                    }
                    cWFinder.setStartParameters(startWeights); // the start parameter has no effect...
                    cWFinder.setParameterRanges(weightRanges);
                    cWFinder.performRandomMinimization(nrsamples, nriter, false);
                    centroidWeights = cWFinder.getMinimizingParameters();

                    for (int i = 0; i < this.getNrChildren(); i++) {
                        this.getChild(i).clearDataElems();
                    }
                } else {
                    centroidWeights = new double[nrchildnodes];
                    for (int h = 0; h < nrchildnodes; h++) {
                        centroidWeights[h] = 1;
                    }
                }

                double[] pointaux = new double[parent.nrVarsNode];
                for (int j = 0; j < sizedataelem; j++) {
                    for (int k = 0; k < parent.nrVarsNode; k++) {
                        pointaux[k] = dataset.get(dataelem.get(j)).variables[varnode.get(k)];
                    }
                    int res = split(resPar, pointaux);
                    this.getChild(res).addDataElem(this.dataelem.get(j), true);
                }

                infgain = this.getEntropy();
                for (int j = 0; j < nrchildnodes; j++) {
                    if (!this.getChild(j).dataelem.isEmpty()) {
                        infgain -= ((double) this.getChild(j).dataelem.size() / (double) sizedataelem) * this.getChild(j).getEntropy();
                    }
                }
//                System.out.println("InfGain Before and After opt weights:\t" + infgainBefore + "\t" + infgain);
            }

            for (int k = this.childList.size() - 1; k >= 0; k--) {
                if (!childList.get(k).hasDataElems()) {
                    this.childList.remove(k);
                }
            }
            for (Node childList1 : this.childList) {
                parent.addNode2Train(childList1);
            }
//                System.out.println("Nr of children: " + nrchildren + "# nrClasses: " + nrClasses + "# Inf. Gain: " + infgain + "# Nr. Elements: " + sizedataelem);
//                System.out.println("Param: " + Arrays.toString(resPar));
//            }
        } else { //FINAL NODE case

//            iter = counter.keySet().iterator();
//            int nrmax = 0;
//            while (iter.hasNext()) {
//                String klasseaux = iter.next();
//                int nr = counter.get(klasseaux);
//                trainingPopulation.put(klasseaux, nr);
//                this.nrElements += nr;
//                if (nr > nrmax) {
//                    nrmax = nr;
//                    endnodeclass = klasseaux;
//                }
//            }
            int nrmax = 0;
            for (Map.Entry<String, Integer> entry : trainingPopulation.entrySet()) {
                int value = entry.getValue();
                if (value > nrmax) {
                    nrmax = value;
                    endnodeclass = entry.getKey();
                }
            }
//            if(nrElements>minsize&&this.trainingPopulation.size()==2&&!this.parent.isMaxSplitReached()){
//                System.out.println("¿Whaaat?");
//            }
            nrElementsTraining = nrElements;
//            System.out.println(this.level + "\t" + this.nrElements + "\t" + this.trainingPopulation.size() + "\t" + endnodeclass + "\t" + (double)trainingPopulation.get(endnodeclass)/(double)nrElements +"\t"+parent.isMaxSplitReached());
        }
        this.dataelem.clear();
    }

    public final static <K, V extends Comparable<? super V>> LinkedHashMap<K, V> sortByValue(Map<K, V> map) {
        List<Map.Entry<K, V>> list
                = new LinkedList<>(map.entrySet());
        Collections.sort(list, new Comparator<Map.Entry<K, V>>() {
            @Override
            public int compare(Map.Entry<K, V> o1, Map.Entry<K, V> o2) {
                return -(o1.getValue()).compareTo(o2.getValue());
            }
        });

        LinkedHashMap<K, V> result = new LinkedHashMap<>();
        for (Map.Entry<K, V> entry : list) {
            result.put(entry.getKey(), entry.getValue());
        }
        return result;
    }

    private void updateCurrentPopulation() {
        this.trainingPopulation.clear();
        this.nrElements = dataelem.size();
        for (Integer i : dataelem) {
            String klasse = this.dataset.get(i).klasse;
            if (trainingPopulation.containsKey(klasse)) {
                trainingPopulation.put(klasse, trainingPopulation.get(klasse) + 1);
            } else {
                trainingPopulation.put(klasse, 1);
            }
        }
    }

    public final double getEntropy() {
//        HashMap<String, Integer> counter = new HashMap<>();
//        DataVect datavectaux;
        double res = 0;
        double probval;
//        int totalnr;
//        Set<String> klasseset;

//        totalnr = this.dataelem.size();
//
//        for (int i = 0; i < totalnr; i++) {
//            datavectaux = this.dataset.get(this.dataelem.get(i));
//            if (counter.containsKey(datavectaux.klasse)) {
//                counter.put(datavectaux.klasse, counter.get(datavectaux.klasse) + 1);
//            } else {
//                counter.put(datavectaux.klasse, 1);
//            }
//        }
        if (nrElements == 0) {
            res = 0;
        } else {
            for (Map.Entry<String, Integer> entry : trainingPopulation.entrySet()) {
                probval = ((double) entry.getValue() / (double) this.nrElements);
                res = res - (probval * (double) Math.log(probval));
            }
        }
//        System.out.println(this.id + " " + res);
        return res;
    }

    public final void test(ResultSet results, double minreliab) {

        int res;
        int sizedataelem = this.dataelem.size();

        int indexaux;
        double[] paramsplit = new double[paramnode.size()];

        for (int j = 0; j < this.childList.size(); j++) {
            this.getChild(j).setDataSet(dataset);
        }

        if (!this.dataelem.isEmpty()) {
            if (!this.hasChildren() && this.trainingPopulation.isEmpty()) {
                System.out.println("Houston...");
            }
            if (this.endnodeclass == null && this.hasChildren() && !this.paramnode.isEmpty()) { // Terminal node?

                if (this.childList.size() > 1) {
                    for (int j = 0; j < paramnode.size(); j++) {
                        paramsplit[j] = this.paramnode.get(j);
                    }
                    for (int j = 0; j < sizedataelem; j++) {
                        double[] pointaux = new double[paramnode.size() / childList.size()];
                        for (int k = 0; k < paramnode.size() / childList.size(); k++) {
                            if (this.dataelem.get(j) == null) {
                                System.out.println("Nullallalal");
                            }
                            pointaux[k] = this.dataset.get(this.dataelem.get(j)).variables[this.varnode.get(k)];
                        }
                        res = split(paramsplit, pointaux);
//                        System.out.println("children nr:"+this.childList.size()+ " res"+res + "level:"+this.level);
                        this.getChild(res).addDataElem(this.dataelem.get(j), false);
                    }
                } else {
                    for (int j = 0; j < sizedataelem; j++) {
                        this.getChild(0).addDataElem(this.dataelem.get(j), false);
                    }
                }

            } else if (!trainingPopulation.isEmpty() && this.endnodeclass != null) {
                if ((double) this.trainingPopulation.get(this.endnodeclass)
                        / (double) this.nrElementsTraining >= minreliab) {

                    for (int j = 0; j < sizedataelem; j++) {

                        indexaux = this.dataelem.get(j);
                        Iterator<String> iter = this.trainingPopulation.keySet().iterator();
                        if (results.containsKey(indexaux)) {
                            while (iter.hasNext()) {
                                String klasseaux = iter.next();
                                if (results.get(indexaux).containsKey(klasseaux)) {
                                    if (klasseaux.matches(endnodeclass)) {
                                        results.get(indexaux).get(klasseaux).nr++;
                                    }
                                    results.get(indexaux).get(klasseaux).probability
                                            += (double) this.trainingPopulation.get(klasseaux)
                                            / (double) this.nrElementsTraining;
                                } else {
                                    ResultVect resVectaux = new ResultVect();
                                    if (klasseaux.matches(endnodeclass)) {
                                        resVectaux.nr = 1;
                                    }
                                    resVectaux.probability = (double) this.trainingPopulation.get(klasseaux)
                                            / (double) this.nrElementsTraining;
                                    results.get(indexaux).put(klasseaux, resVectaux);
                                }
                            }

                        } else {
                            ResultMap resMapaux = new ResultMap();
                            while (iter.hasNext()) {
                                String klasseaux = iter.next();
                                ResultVect resVectaux = new ResultVect();
                                if (klasseaux.matches(endnodeclass)) {
                                    resVectaux.nr = 1;
                                }
                                resVectaux.probability = (double) this.trainingPopulation.get(klasseaux)
                                        / (double) this.nrElementsTraining;
                                resMapaux.put(klasseaux, resVectaux);
                            }
                            results.put(indexaux, (ResultMap) resMapaux);
                        }
                    }
                }
            }
        }

    }

    // Make a function for a counter with max performance
    // avoid repeated calculations of entropy and probabilities
    @Override
    public String toString() {
        return "id: " + this.id + ", level: " + this.level + ", children: " + this.childList.size();
    }
}
