/*
 * File: $Id: Dicom3DSlicerControl.java,v 1.6 2006/11/09 10:05:39 hans Exp $
 * History
 * Date: 03.05.2006  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2006 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * Dicom3DSlicerControl.java
 *
 * Created on May 3, 2006, 12:24 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.spectrim.dicom;

import org.spectrim.lib.Dialogs;
import org.spectrim.lib.DualProgressBar;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 *
 * @author hans
 */
public class Dicom3DSlicerControl extends JPanel implements ActionListener
{


    
    /** 
     * Creates a new instance of Dicom3DSlicerControl. 
     * @param parent
     */
    public Dicom3DSlicerControl( JFrame parent )
    {
        m_parent = parent;
        setUpGui( );
    }



    /** 
     * Creates a new instance of Dicom3DSlicerControl.
     * @param parent
     * @param imageSeriesData
     * @param activationOverlays
     * @param dtiFiberImageOverlays
     * @param dtiFiberColors
     */
    public Dicom3DSlicerControl( JFrame parent, Dicom3DImageSeriesData imageSeriesData,
                                 boolean[][][] activationOverlays,
                                 boolean[][][] dtiFiberImageOverlays,
                                 Color[][][] dtiFiberColors )
    {

        m_activationImageOverlays = activationOverlays;
        m_dtiFiberImageOverlays = dtiFiberImageOverlays;
        m_dtiFiberColors = dtiFiberColors;

        m_parent = parent;
        m_dicom3DData = imageSeriesData;

    }


      /**
     * Returns the number of integers.
     * @return
     */
    public int getOriginalNumberOfImages( )
    {
        return m_dicom3DData.getNumberOfImages( );
    }

    /**
     * Returns the original image width parameter.
     * @return
     */
    public int getOriginalImageWidth( )
    {
        return m_dicom3DData.getFileInfo( 0 ).getImageWidth( );
    }

    /**
     * Returns the original image height parameter.
     * @return
     */
    public int getOriginalImageHeight( )
    {
        return m_dicom3DData.getFileInfo( 0 ).getImageHeight( );
    }


    /**
     * Sets up the GUI of the Dicom3DSlicerControl.
     */
    private void setUpGui( )
    {
        m_topPanel = new JPanel( );
        
        m_topPanel.setLayout( new BorderLayout( ) );
        
        m_upperPanelLeft = new JPanel( );
        m_upperPanelLeft.setLayout( new BorderLayout( ) );
        m_upperPanelLeft.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
                
        m_lowerPanelLeft = new JPanel( );
        m_lowerPanelLeft.setLayout( new BorderLayout( ) );
        m_lowerPanelLeft.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        m_upperPanelRight = new JPanel( );
        m_upperPanelRight.setLayout( new BorderLayout( ) );
        m_upperPanelRight.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
        
        m_lowerPanelRight = new JPanel( );
        m_lowerPanelRight.setLayout( new BorderLayout( ) );
        m_lowerPanelRight.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );

        m_1_2_directionView = new DicomSliceCanvasPolyDraw( m_parent );
        m_1_2_directionView.setBorder( BorderFactory.createLineBorder( Color.BLACK ) );
        m_1_2_directionView.setImage( new int[256*256], 256, 256, 0 );
        m_1_2_directionView.addActionListener( this );
        
        m_1_2_directionView.setDTIFiberOverlays( m_dtiFiberImageOverlays, m_dtiFiberColors );
        
        m_1_2_directionView.setVisible( true );

        m_1_2_directionView.enableFlipAndRotateMenuItems( false );
        
        m_1_3_directionView = new DicomSliceCanvasPolyDraw( m_parent );
        m_1_3_directionView.setImage( new int[256*256], 256, 256, 0 );
        m_1_3_directionView.setBorder( BorderFactory.createLineBorder( Color.BLACK ) );
        m_1_3_directionView.addActionListener( this );
        

        m_1_3_directionView.setVisible( true );
        m_1_3_directionView.enableFlipAndRotateMenuItems( false );
        
        m_2_3_directionView = new DicomSliceCanvasPolyDraw( m_parent );
        m_2_3_directionView.setImage( new int[256*256], 256, 256, 0 );
        m_2_3_directionView.setBorder( BorderFactory.createLineBorder( Color.BLACK ) );
        m_2_3_directionView.addActionListener( this );
        

        m_2_3_directionView.setDTIFiberOverlays( m_dtiFiberImageOverlays, m_dtiFiberColors );

        m_2_3_directionView.setVisible( true );
        m_2_3_directionView.enableFlipAndRotateMenuItems( false );
        
        m_fourthView = new DicomSliceCanvasPolyDraw( m_parent );
        m_fourthView.setImage( new int[256*256], 256, 256, 0 );
        m_fourthView.setBorder( BorderFactory.createLineBorder( Color.BLACK ) );
        m_fourthView.addActionListener( this );
        
        m_fourthView.setVisible( true );
        
        m_lowerCollectPanel = new JPanel( ); 
        m_lowerCollectPanel.setLayout( new BorderLayout( ) );
        m_lowerCollectPanel.setBackground( Color.WHITE );
        
        m_upperCollectPanel = new JPanel( );
        m_upperCollectPanel.setLayout( new BorderLayout( ) );
        m_upperCollectPanel.setBackground( Color.WHITE );
        
        m_upperPanelLeft.add( m_1_2_directionView, BorderLayout.WEST );
        m_upperPanelLeft.setBackground( Color.RED );
        
        m_upperPanelRight.add( m_1_3_directionView , BorderLayout.EAST );
        m_upperPanelRight.setBackground( Color.YELLOW );
        
        m_upperCollectPanel.add( m_upperPanelLeft, BorderLayout.WEST );
        m_upperCollectPanel.add( m_upperPanelRight, BorderLayout.EAST );
        
        m_lowerPanelLeft.add( m_2_3_directionView, BorderLayout.WEST );
        m_lowerPanelLeft.setBackground( Color.GREEN );
        
        m_lowerPanelRight.add( m_fourthView, BorderLayout.EAST );
        m_lowerPanelRight.setBackground( Color.CYAN );
        m_lowerCollectPanel.add( m_lowerPanelLeft, BorderLayout.WEST );
        m_lowerCollectPanel.add( m_lowerPanelRight, BorderLayout.EAST );
        
        m_topPanel.add( m_upperCollectPanel, BorderLayout.NORTH );
        m_topPanel.add( m_lowerCollectPanel, BorderLayout.SOUTH );
        
        add( m_topPanel );
        
        m_1_2_directionView.addActionListener( m_1_3_directionView );
        m_1_2_directionView.addActionListener( m_2_3_directionView );

        m_1_3_directionView.addActionListener( m_1_2_directionView );
        m_1_3_directionView.addActionListener( m_2_3_directionView );

        m_2_3_directionView.addActionListener( m_1_2_directionView );
        m_2_3_directionView.addActionListener( m_1_3_directionView );
        
    }
    
    /**
     * Sets the DTI fiber overlays
     * @param dtiFiberImageOverlays
     * @param colors
     */
    public void setDTIFiberOverlays( boolean[][][] dtiFiberImageOverlays, Color[][][] colors )
    {
        m_dicom3DData.setDTIFiberOverlays( dtiFiberImageOverlays, colors );
        m_1_2_directionView.setDTIFiberOverlays( m_dicom3DData.getDTIFiberImageOverlays_12( ), m_dicom3DData.getDTIColorOverlay_12( ) );
        m_1_2_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_12( ) );
        
        m_1_3_directionView.setDTIFiberOverlays( m_dicom3DData.getDTIFiberImageOverlays_13( ), m_dicom3DData.getDTIColorOverlay_13( ) );
        m_1_3_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_13( ) );
        
        m_2_3_directionView.setDTIFiberOverlays( m_dicom3DData.getDTIFiberImageOverlays_23( ), m_dicom3DData.getDTIColorOverlay_23( ) );
        m_2_3_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_23( ) );

        m_dtiFiberImageOverlays = dtiFiberImageOverlays;
        m_dtiFiberColors = colors;
    }

    /**
     * Sets the fMRI activation overlays.
     * @param activationOverlays
     */
    public void setfMRIActiovationOverlays( boolean[][][] activationOverlays )
    {
        m_dicom3DData.setfMRIActiovationOverlays( activationOverlays );

        m_1_2_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_12( ) );
        m_1_3_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_13( ) );
        m_2_3_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_23( ) );

        m_activationImageOverlays = activationOverlays;
    }

    
    /**
     * Sets the file references of the 3D dataset
     * @param fileRefs
     * @param activationImageOverlays
     * @param dtiFiberImageOverlays
     * @param dtiFiberColors
     */
    public void setDicom3DFileReferences( LinkedList<String> fileRefs,
                                          boolean[][][] activationImageOverlays,
                                          boolean[][][] dtiFiberImageOverlays,
                                          Color[][][] dtiFiberColors )
    {
        m_selected3DFileReferences  = fileRefs;
        m_activationImageOverlays   = activationImageOverlays;
        m_dtiFiberImageOverlays     = dtiFiberImageOverlays;
        m_dtiFiberColors            = dtiFiberColors;

        loadAndDisplay3DDataSet( );

    }
    
    /**
     * Sets the file references of the 4th view dataset.
     * @param fileRefs
     */    
    public void setFourthViewFileReferences( LinkedList<String> fileRefs )
    {
        m_fourthViewFileReferences = fileRefs;
        loadAndDisplayFourthView( );
    }


    /**
     * Method is called after successful loading of 3D data.
     * @param showPopup
     */
    public void inititialize3DViews( boolean showPopup )
    {
        m_3DMatrixSize = m_dicom3DData.get3DMatrixSize( );

        if ( m_dicom3DData != null )
        {
            int[] ima = m_dicom3DData.getImagePixelData_1_2( m_3DMatrixSize / 2 );
            FileInfo finfo = m_dicom3DData.getImageFileInfoObject_1_2( m_3DMatrixSize / 2 );
            LinkedHashMap siemensInfo = m_dicom3DData.getSiemensSpecificFileInfomationObject_1_2( m_3DMatrixSize / 2 );

            m_1_2_directionView.setCurrentlySelectedImageDTIOverlayNumber( m_3DMatrixSize / 2 );
            m_1_2_directionView.setCurrentlySelectedImagefMRIOverlayNumber( m_3DMatrixSize / 2 );

            m_1_2_directionView.setImage( ima, m_3DMatrixSize, m_3DMatrixSize, 0, finfo, siemensInfo );


            m_dicom3DData.setDirectionVectors_1_2( m_1_2_directionView.getImageOrientationUnitVectorX( ),
                                                   m_1_2_directionView.getImageOrientationUnitVectorY( ) );

            m_dicom3DData.set3DFieldOfViewParameters( m_1_2_directionView.m_FOVX,
                                                      m_1_2_directionView.m_FOVY,
                                                      new Double( finfo.getSliceThickness( ) ).floatValue( ) );

            m_1_2_directionView.setVisible( true );
            m_1_2_directionView.enableImagePaging( false );


            m_1_2_directionView.setDTIFiberOverlays( m_dicom3DData.getDTIFiberImageOverlays_12( ), m_dicom3DData.getDTIColorOverlay_12( ) );
            m_1_2_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_12( ) );

            m_1_2_directionView.displayCrossLines( Color.GREEN, Color.YELLOW );

            m_1_2_directionView.drawImage( false );

            ///////////////////////////////////////

            m_1_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( m_3DMatrixSize / 2 );
            m_1_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( m_3DMatrixSize / 2 );

            m_1_3_directionView.setImage( m_dicom3DData.getImagePixelData_1_3( m_3DMatrixSize/2 ), m_3DMatrixSize, m_3DMatrixSize, 0 );

            m_1_3_directionView.setComputedGeometricSliceAttributes( m_dicom3DData.getDirectionVector_1_3_X( ),
                                                                     m_dicom3DData.getDirectionVector_1_3_Y( ), 
                                                                     m_1_2_directionView.compute3DCoordinate(m_3DMatrixSize/2, m_3DMatrixSize/2 ), 
                                                                     m_3DMatrixSize/2, m_3DMatrixSize/2,
                                                                     m_dicom3DData.getFOVY_12( ), m_dicom3DData.getFOVZ_12( ) );
                
            
            
            m_1_3_directionView.setDTIFiberOverlays( m_dicom3DData.getDTIFiberImageOverlays_13( ), m_dicom3DData.getDTIColorOverlay_13( ) );


            m_1_3_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_13( ) );

            m_1_3_directionView.displayCrossLines( Color.GREEN, Color.RED );
            m_1_3_directionView.setVisible( true );
            m_1_3_directionView.enableImagePaging( false );
            m_1_3_directionView.drawImage( false );

            ////////////////////////////////////////

            m_2_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( m_3DMatrixSize / 2 );
            m_2_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( m_3DMatrixSize / 2 );

            m_2_3_directionView.setImage( m_dicom3DData.getImagePixelData_2_3( m_3DMatrixSize/2 ), m_3DMatrixSize, m_3DMatrixSize, 0 );
            
            m_2_3_directionView.setComputedGeometricSliceAttributes( m_dicom3DData.getDirectionVector_2_3_X( ),
                                                                     m_dicom3DData.getDirectionVector_2_3_Y( ), 
                                                                     m_1_2_directionView.compute3DCoordinate( m_3DMatrixSize / 2, m_3DMatrixSize / 2 ), 
                                                                     m_3DMatrixSize / 2, m_3DMatrixSize / 2,
                                                                     m_dicom3DData.getFOVZ_12( ), m_dicom3DData.getFOVX_12( ) );
                
            m_2_3_directionView.setDTIFiberOverlays( m_dicom3DData.getDTIFiberImageOverlays_23( ), m_dicom3DData.getDTIColorOverlay_23( ) );

            m_2_3_directionView.setActivationOverlays( m_dicom3DData.getFMRIActivationImageOverlays_23( ) );

            m_2_3_directionView.displayCrossLines( Color.YELLOW, Color.RED );
            m_2_3_directionView.setVisible( true );
            m_2_3_directionView.enableImagePaging( false );
            m_2_3_directionView.drawImage( false );
            if ( showPopup == true )
            {
                Dialogs d = new Dialogs( );
                Dialogs.messageBox( m_parent, "<HTML> Loading of 3D dataset finished </HTML>" );
            }
        }
        else
        {
            System.out.println( "Could not perform 3D dataset loading : could not create (1)" );
        }
    }
    
    /**
     * Sets the file references of the 3D dataset
     */
    public void loadAndDisplay3DDataSet( )
    {
        if ( m_selected3DFileReferences != null )
        {
            m_dicom3DData = new Dicom3DImageSeriesData( m_parent,
                                                        m_selected3DFileReferences,
                                                        m_activationImageOverlays,
                                                        m_dtiFiberImageOverlays,
                                                        m_dtiFiberColors );

            m_dicom3DData.addActionListener( this );

            m_dicom3DData.create3DDataInSwingworkerThread( );
            
        }
        else
        {
            System.out.println( "Could not perform 3D dataset loading : could not create 3D-dataset!" );
        }
    }
    
    
    /**
     * Sets the file references of the 3D dataset
     */
    public void loadAndDisplayFourthView( )
    {
        if ( m_fourthViewFileReferences != null )
        {
            m_fourthViewData = new DicomImageSeriesData( m_selected3DFileReferences );
        
            m_fourthView.setDicomImages( m_fourthViewData );
            
            m_fourthView.enableImagePaging( true );
            m_fourthView.displayDicomImage( 0 );
            m_fourthView.displayCrossLines( Color.CYAN, Color.CYAN );
            m_fourthView.drawImage( false );
        }
    }

    

    /**
     * Handles the dicomization of the complete image stack 1_3.
     */
    public void handleDicomizationOfImagesStack_1_3( )
    {

        m_selectedImageFrame = 13;

        m_dicomFileCreatorOptionsFrame = new DicomFileCreatorOptionsFrame( "Export frame 13", m_parent, m_dicom3DData.getImageFileInfoObject_1_2( 0 ) );
        m_dicomFileCreatorOptionsFrame.addActionListener( this );
    }


   /**
    *
    */
    public void handleDicomizationOfImagesStack_1_2( )
    {
        m_selectedImageFrame = 12;

        m_dicomFileCreatorOptionsFrame = new DicomFileCreatorOptionsFrame( "Export frame 12", m_parent, m_dicom3DData.getImageFileInfoObject_1_2( 0 ) );
        m_dicomFileCreatorOptionsFrame.addActionListener( this );
    }

    /**
     * Performs the regridding and DICOM file writing of upper left image stacks
     * Functionality is meant to create e.g axial slice stacks from sagitally
     * recorded image stacks.
     */
    public void handleRegriddingOfImagesStack_1_2_InWorkerThread( )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Regridding and writing Dicom files of upper left view .."  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            public Object construct( )
            {
                handleRegriddingOfImagesStack_1_2( );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Warning: Do not sent to PACS system: not all DICOM tags are being set properly!" );
            }
        };
        worker.start( );
    }

   ////////////////////////////////////////////////////////////////////////////
    // Heart of the regridding routines: the following routines enable the
    // user to make e.g. an axial image stack from a 3D sagitaldata set etcetera
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Handles the dicomization of the complete image stack.
     */
    public void handleRegriddingOfImagesStack_1_2( )
    {

        DicomDefaults fdef = DicomDefaults.getInstance( );
        String workingDir = fdef.getProperty( "Dicom mount path" );
        if ( m_org_series_description == null )
        {
            m_org_series_description = new String( m_dicom3DData.getFileInfo( 0 ).getSeriesDescription( ) );
        }

        for ( int i = 0; i < m_dicom3DData.get3DMatrixSize( ); i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / m_dicom3DData.get3DMatrixSize( ) ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            String patientName = m_dicom3DData.getImageFileInfoObject_1_2( i ).getPatientName( );
            String accessionNumber = m_dicom3DData.getImageFileInfoObject_1_2( i ).getAccessionNumber( );
            String examDate =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyDate( );
            String examTime =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyTime( );
            //int seriesNumber =  new Integer( m_dicom3DData.getImageFileInfoObject_1_2( i ).getSeriesNumber( ).trim( ) );
           
            int[] pixelData = m_dicom3DData.getImagePixelData_1_2( i );

            FileInfo finfo;

            if ( i < m_dicom3DData.getNumberOfImages( ) )
            {
                finfo = m_dicom3DData.getFileInfo( i );
            }
            else
            {
                finfo = m_dicom3DData.getFileInfo( 0 );
            }
            
            // the instance number is new (slices are created containing zero pixels values
            // in case the original number of slice was not a power of two

            finfo.setInstanceNumber( "" + i );

            String instanceNumberString = "";
            int instanceNumber = new Integer( finfo.getInstanceNumber( ).trim( ) ).intValue( );

            if ( ( instanceNumber < 10 ) && ( instanceNumber > -1 ) )
            {
                instanceNumberString = "000" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 100 ) && ( instanceNumber > 9 ) )
            {
                instanceNumberString = "00" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 999 ) && ( instanceNumber > 99 ) )
            {
                instanceNumberString = "0" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 9999 ) && ( instanceNumber > 999 ) )
            {
               instanceNumberString = "" + finfo.getInstanceNumber( ).trim( );
            }

            String seriesNumberString = "";

            int seriesNumber = new Integer( finfo.getSeriesNumber( ).trim( ) ).intValue( );

            if ( ( seriesNumber < 10 ) && ( seriesNumber > -1 ) )
            {
                seriesNumberString = "000" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 100 ) && ( seriesNumber > 9 ) )
            {
                 seriesNumberString = "00" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 999 ) && ( seriesNumber > 99 ) )
            {
                seriesNumberString = "0" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 9999 ) && ( seriesNumber > 999 ) )
            {
                seriesNumberString = "" + finfo.getSeriesNumber( ).trim( );
            }

            String dstPath;
            String dstFileName;

            dstPath = workingDir + System.getProperty( "file.separator" ) +
                      m_org_series_description.trim( ) + "_regridded_1_2" + "_" + m_dicom3DData.get3DMatrixSize( ) + "_" + m_dicom3DData.get3DMatrixSize( ) + "_";

            dstFileName = dstPath + System.getProperty( "file.separator" ) +
                          finfo.getPatientName( ).replace( ' ', '_' ) + "_" +
                          m_org_series_description.trim( ) + "-" +
                          "regridded_1_2" + "_" + m_dicom3DData.get3DMatrixSize( ) + "_" +
                          m_dicom3DData.get3DMatrixSize( ) + "_" +
                          seriesNumberString + "-" +
                          instanceNumberString + ".dcm";

            String dstPathCmd = dstPath;

            dstFileName =  eliminateIllegalFileSystemCharacters( dstFileName );

            dstPathCmd = eliminateIllegalFileSystemCharacters( dstPathCmd );

            System.out.println( dstPath );

            // test whether the path exists; if not than create it
            File path = new File( dstPathCmd );
            if ( path.exists( ) == false )
            {
                System.out.println( "Instance # : " + i + "Creating  path : " + dstPath );
                path.mkdir( );
            }

            // try to write the new DICOM file

            // first update all DICOM tags for new image series
            finfo.setImageHeight( m_dicom3DData.get3DMatrixSize( ) );
            finfo.setImageWidth( m_dicom3DData.get3DMatrixSize( ) );

            finfo.setPixelWidth( "" + new Float( finfo.getPixelWidth( ).trim( ) ) );
            finfo.setPixelHeight( "" + new Float( finfo.getPixelHeight( ).trim( ) ) );

            finfo.setPixelSpacing(  finfo.getPixelWidth() + '\\' + finfo.getPixelHeight( ) );

            finfo.setSeriesNumber( "" +  1110 );

            // some reformatted image data do not contain a slice position tag
            if ( finfo.getSliceLocation( ).compareTo( "N.A." ) == 0 )
            {
                finfo.setSliceLocation( "" + new Float( finfo.getInstanceNumber( ) ).floatValue( ) );
            }

            long currentTime  = System.currentTimeMillis( );
            String uid_prefix = "1.3.12.2.1107.5.2";

            DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
            String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );

            String imageInstanceUID = uid_prefix + IP_address + currentTime;

            finfo.setSOPInstanceUID( imageInstanceUID );
            finfo.setSeriesInstanceUID( uid_prefix + IP_address + ( currentTime - 4 ) );

            finfo.setSeriesDescription( "Regridded-version_1_2_" + finfo.getImageWidth( ) + "x" + finfo.getImageHeight( ) + "_of_" + finfo.getSeriesDescription( ) );

            writeAsTrueDicom( dstFileName, finfo, pixelData );

        }
    }


    /**
     * Handles the dicomization of the complete image stack.
     */
    public void handleRegriddingOfImagesStack_1_3( )
    {

        DicomDefaults fdef = DicomDefaults.getInstance( );
        String workingDir = fdef.getProperty( "Dicom mount path" );

        if ( m_org_series_description == null )
        {
            m_org_series_description = new String( m_dicom3DData.getFileInfo( 0 ).getSeriesDescription( ) );
        }

        for ( int i = 0; i < m_dicom3DData.get3DMatrixSize( ); i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / m_dicom3DData.get3DMatrixSize( ) ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            String patientName = m_dicom3DData.getImageFileInfoObject_1_2( i ).getPatientName( );
            String accessionNumber = m_dicom3DData.getImageFileInfoObject_1_2( i ).getAccessionNumber( );
            String examDate =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyDate( );
            String examTime =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyTime( );
            //int seriesNumber =  new Integer( m_dicom3DData.getImageFileInfoObject_1_2( i ).getSeriesNumber( ).trim( ) );

            int[] pixelData = m_dicom3DData.getImagePixelData_1_3( i );

            FileInfo finfo;

            if ( i < m_dicom3DData.getNumberOfImages( ) )
            {
                finfo = m_dicom3DData.getFileInfo( i );
            }
            else
            {
                finfo = m_dicom3DData.getFileInfo( 0 );
            }

            // the instance number is new (slices are created containing zero pixels values
            // in case the original number of slice was not a power of two

            finfo.setInstanceNumber( "" + i );

            String instanceNumberString = "";
            int instanceNumber = new Integer( finfo.getInstanceNumber( ).trim( ) ).intValue( );

            if ( ( instanceNumber < 10 ) && ( instanceNumber > -1 ) )
            {
                instanceNumberString = "000" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 100 ) && ( instanceNumber > 9 ) )
            {
                instanceNumberString = "00" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 999 ) && ( instanceNumber > 99 ) )
            {
                instanceNumberString = "0" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 9999 ) && ( instanceNumber > 999 ) )
            {
               instanceNumberString = "" + finfo.getInstanceNumber( ).trim( );
            }

            String seriesNumberString = "";

            int seriesNumber = new Integer( finfo.getSeriesNumber( ).trim( ) ).intValue( );

            if ( ( seriesNumber < 10 ) && ( seriesNumber > -1 ) )
            {
                seriesNumberString = "000" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 100 ) && ( seriesNumber > 9 ) )
            {
                 seriesNumberString = "00" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 999 ) && ( seriesNumber > 99 ) )
            {
                seriesNumberString = "0" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 9999 ) && ( seriesNumber > 999 ) )
            {
                seriesNumberString = "" + finfo.getSeriesNumber( ).trim( );
            }

            String dstPath;
            String dstFileName;

            dstPath = workingDir + System.getProperty( "file.separator" ) +
                      m_org_series_description.trim( ) + "_regridded_1_3" + "_" + m_dicom3DData.get3DMatrixSize( ) + "_" + m_dicom3DData.get3DMatrixSize( ) + "_";

            dstFileName = dstPath + System.getProperty( "file.separator" ) +
                          finfo.getPatientName( ).replace( ' ', '_' ) + "_" +
                          m_org_series_description.trim( ) + "-" +
                          "regridded_1_3" + "_" + m_dicom3DData.get3DMatrixSize( ) + "_" +
                          m_dicom3DData.get3DMatrixSize( ) + "_" +
                          seriesNumberString + "-" +
                          instanceNumberString + ".dcm";

            String dstPathCmd = dstPath;

            dstFileName =  eliminateIllegalFileSystemCharacters( dstFileName );

            dstPathCmd = eliminateIllegalFileSystemCharacters( dstPathCmd );

            System.out.println( dstPath );

            // test whether the path exists; if not than create it
            File path = new File( dstPathCmd );
            if ( path.exists( ) == false )
            {
                System.out.println( "Instance # : " + i + "Creating  path : " + dstPath );
                path.mkdir( );
            }

            // try to write the new DICOM file

            // first update all DICOM tags for new image series
            finfo.setImageHeight( m_dicom3DData.get3DMatrixSize( ) );
            finfo.setImageWidth( m_dicom3DData.get3DMatrixSize( ) );

            finfo.setPixelWidth( "" + new Float( finfo.getPixelWidth( ).trim( ) ) );
            finfo.setPixelHeight( "" + new Float( finfo.getPixelHeight( ).trim( ) ) );

            finfo.setPixelSpacing(  finfo.getPixelWidth() + '\\' + finfo.getPixelHeight( ) );

            finfo.setSeriesNumber( "" +  1111 );

            // some reformatted image data do not contain a slice position tag
            if ( finfo.getSliceLocation( ).compareTo( "N.A." ) == 0 )
            {
                finfo.setSliceLocation( "" + new Float( finfo.getInstanceNumber( ) ).floatValue( ) );
            }

            long currentTime  = System.currentTimeMillis( );
            String uid_prefix = "1.3.12.2.1107.5.2";

            DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
            String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );

            String imageInstanceUID = uid_prefix + IP_address + currentTime;

            finfo.setSOPInstanceUID( imageInstanceUID );
            finfo.setSeriesInstanceUID( uid_prefix + IP_address + ( currentTime - 4 ) );

            finfo.setSeriesDescription( "Regridded-version_1_3_" + finfo.getImageWidth( ) + "x" + finfo.getImageHeight( ) + "_of_" + finfo.getSeriesDescription( ) );

            writeAsTrueDicom( dstFileName, finfo, pixelData );

        }
    }


    /**
     * Handles the dicomization of the complete image stack.
     */
    public void handleRegriddingOfImagesStack_2_3( )
    {

        DicomDefaults fdef = DicomDefaults.getInstance( );
        String workingDir = fdef.getProperty( "Dicom mount path" );

        if ( m_org_series_description == null )
        {
            m_org_series_description = new String( m_dicom3DData.getFileInfo( 0 ).getSeriesDescription( ) );
        }

        for ( int i = 0; i < m_dicom3DData.get3DMatrixSize( ); i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / m_dicom3DData.get3DMatrixSize( ) ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            String patientName = m_dicom3DData.getImageFileInfoObject_1_2( i ).getPatientName( );
            String accessionNumber = m_dicom3DData.getImageFileInfoObject_1_2( i ).getAccessionNumber( );
            String examDate =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyDate( );
            String examTime =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyTime( );
            //int seriesNumber =  new Integer( m_dicom3DData.getImageFileInfoObject_1_2( i ).getSeriesNumber( ).trim( ) );

            int[] pixelData = m_dicom3DData.getImagePixelData_2_3( i );

            FileInfo finfo;

            if ( i < m_dicom3DData.getNumberOfImages( ) )
            {
                finfo = m_dicom3DData.getFileInfo( i );
            }
            else
            {
                finfo = m_dicom3DData.getFileInfo( 0 );
            }

            // the instance number is new (slices are created containing zero pixels values
            // in case the original number of slice was not a power of two

            finfo.setInstanceNumber( "" + i );

            String instanceNumberString = "";
            int instanceNumber = new Integer( finfo.getInstanceNumber( ).trim( ) ).intValue( );

            if ( ( instanceNumber < 10 ) && ( instanceNumber > -1 ) )
            {
                instanceNumberString = "000" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 100 ) && ( instanceNumber > 9 ) )
            {
                instanceNumberString = "00" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 999 ) && ( instanceNumber > 99 ) )
            {
                instanceNumberString = "0" + finfo.getInstanceNumber( ).trim( );
            }

            if ( ( instanceNumber < 9999 ) && ( instanceNumber > 999 ) )
            {
               instanceNumberString = "" + finfo.getInstanceNumber( ).trim( );
            }

            String seriesNumberString = "";

            int seriesNumber = new Integer( finfo.getSeriesNumber( ).trim( ) ).intValue( );

            if ( ( seriesNumber < 10 ) && ( seriesNumber > -1 ) )
            {
                seriesNumberString = "000" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 100 ) && ( seriesNumber > 9 ) )
            {
                 seriesNumberString = "00" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 999 ) && ( seriesNumber > 99 ) )
            {
                seriesNumberString = "0" + finfo.getSeriesNumber( ).trim( );
            }

            if ( ( seriesNumber < 9999 ) && ( seriesNumber > 999 ) )
            {
                seriesNumberString = "" + finfo.getSeriesNumber( ).trim( );
            }

            String dstPath;
            String dstFileName;

            dstPath = workingDir + System.getProperty( "file.separator" ) +
                      m_org_series_description.trim( ) + "_regridded_2_3" + "_" + m_dicom3DData.get3DMatrixSize( ) + "_" + m_dicom3DData.get3DMatrixSize( ) + "_";

            dstFileName = dstPath + System.getProperty( "file.separator" ) +
                          finfo.getPatientName( ).replace( ' ', '_' ) + "_" +
                          m_org_series_description.trim( ) + "-" +
                          "regridded_2_3" + "_" + m_dicom3DData.get3DMatrixSize( ) + "_" +
                          m_dicom3DData.get3DMatrixSize( ) + "_" +
                          seriesNumberString + "-" +
                          instanceNumberString + ".dcm";

            String dstPathCmd = dstPath;

            dstFileName =  eliminateIllegalFileSystemCharacters( dstFileName );

            dstPathCmd = eliminateIllegalFileSystemCharacters( dstPathCmd );

            System.out.println( dstPath );

            // test whether the path exists; if not than create it
            File path = new File( dstPathCmd );
            if ( path.exists( ) == false )
            {
                System.out.println( "Instance # : " + i + "Creating  path : " + dstPath );
                path.mkdir( );
            }

            // try to write the new DICOM file

            // first update all DICOM tags for new image series
            finfo.setImageHeight( m_dicom3DData.get3DMatrixSize( ) );
            finfo.setImageWidth( m_dicom3DData.get3DMatrixSize( ) );

            finfo.setPixelWidth( "" + new Float( finfo.getPixelWidth( ).trim( ) ) );
            finfo.setPixelHeight( "" + new Float( finfo.getPixelHeight( ).trim( ) ) );

            finfo.setPixelSpacing(  finfo.getPixelWidth( ) + '\\' + finfo.getPixelHeight( ) );

            finfo.setSeriesNumber( "" +  1112 );

            // some reformatted image data do not contain a slice position tag
            if ( finfo.getSliceLocation( ).compareTo( "N.A." ) == 0 )
            {
                finfo.setSliceLocation( "" + new Float( finfo.getInstanceNumber( ) ).floatValue( ) );
            }

            long currentTime  = System.currentTimeMillis( );
            String uid_prefix = "1.3.12.2.1107.5.2";

            DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
            String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );

            String imageInstanceUID = uid_prefix + IP_address + currentTime;

            finfo.setSOPInstanceUID( imageInstanceUID );
            finfo.setSeriesInstanceUID( uid_prefix + IP_address + ( currentTime - 4 ) );

            finfo.setSeriesDescription( "Regridded-version_2_3_" + finfo.getImageWidth( ) + "x" + finfo.getImageHeight( ) + "_of_" + finfo.getSeriesDescription( ) );

            writeAsTrueDicom( dstFileName, finfo, pixelData );

        }
    }



    /**
     * Writes the FileInfo together with the pixel information into a true
     * DICOM file.
     */
    public int writeAsTrueDicom( String fileName, FileInfo finfo, int[] pixelData )
    {
        try
        {
            System.out.println( ">>> fileName : " + fileName );
            RandomAccessFile fout = new RandomAccessFile( fileName, "rw" );

            byte[] leadingZeroes = new byte[128];

            fout.write( leadingZeroes );

            String dicm = new String( "DICM" ).trim( );

            fout.write( dicm.getBytes( ) );

            finfo.writePatientRelatedTags( fout );
            finfo.writeStudySeriesInstancesRelatedTags( fout );
            finfo.writeMRRelatedTags( fout );
            finfo.writeGeometryRelatedTags( fout );
            finfo.writeOtherTags( fout );

            writePixelInformation( fout, pixelData );

            fout.close( );

            return IO_OK;
        }
        catch ( IOException ex )
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }

        return 0;
    }


    /**
     * Depending on the type of the pixel information,
     * @param fout
     */
    private void writePixelInformation( RandomAccessFile fout, int[] pixelInformation ) throws IOException
    {

        //int outp0 = Integer.rotateLeft( DicomDecoder.PIXEL_DATA, 8 );
        fout.writeInt( Integer.rotateLeft( Integer.reverseBytes( DicomDecoder.PIXEL_DATA ), 16 ) );

        fout.writeShort( DicomDecoder.OW );

        byte[] pixels2write = new byte[ 2 * pixelInformation.length ];

        for ( int i = 0; i < pixelInformation.length; i++ )
        {
            // System.out.println( "pixel information : " + pixelInformation[ i ] );
            org.spectrim.lib.Convert.int2wordl( pixels2write, 2 * i, pixelInformation[ i ] );
        }

        byte[] byteArr = new byte[2];

        fout.write( byteArr );

        fout.writeInt( Integer.reverseBytes( pixels2write.length ) );

        fout.write( pixels2write );

    }


    ////////////////////////////////////////////////////////////////////////////


    /**
     * Performs the regridding and DICOM file writing of upper right image stacks
     * Functionality is meant to create e.g axial slice stacks from sagital
     * recorded image stacks.
     */
    public void handleRegriddingOfImagesStack_1_3_InWorkerThread( )
    {
        
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Regridding and writing Dicom files of upper right view .."  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            @Override
            public Object construct( )
            {
                handleRegriddingOfImagesStack_1_3( );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Warning: Do not sent to PACS system: not all DICOM tags are being set properly!" );
            }
        };
        worker.start( );
    }

    /**
     * Performs the regridding and DICOM file writing of lower left image stacks
     * Functionality is meant to create e.g axial slice stacks from sagitally
     * recorded image stacks.
     */
    public void handleRegriddingOfImagesStack_2_3_InWorkerThread( )
    {

        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Regridding and writing Dicom files of lower left view .."  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            @Override
            public Object construct( )
            {
                handleRegriddingOfImagesStack_2_3( );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Warning: Do not sent to PACS system: not all DICOM tags are being set properly!" );
            }
        };
        worker.start( );
    }


    /**
     * Handles the dicomization of the complete image stack 1_3.
     */
    public void handleDicomizationOfImagesStack_2_3( )
    {
        m_selectedImageFrame = 23;

        m_dicomFileCreatorOptionsFrame = new DicomFileCreatorOptionsFrame( "Export frame 23", m_parent, m_dicom3DData.getImageFileInfoObject_1_2( 0 ) );
        m_dicomFileCreatorOptionsFrame.addActionListener( this );
    }



    /**
     * Performs the dicomization in an separate thread to keep the 
     * GUI responsive!
     * @param sentToSelectedDicomNetworkNode
     * @param includeGeometryTags
     * @param seriesNumberToUseForDicomization
     * @param seriesDescriptionForDicomization
     * @param totalNumberOfSlices
     */
    public void redicomizeCompleteImageStack_1_2_InWorkerThread(  final boolean sentToSelectedDicomNetworkNode,
                                                                  final boolean includeGeometryTags,
                                                                  final int seriesNumberToUseForDicomization,
                                                                  final String seriesDescriptionForDicomization,
                                                                  final int totalNumberOfSlices )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ) ,false );
        m_dualProgressBar.setTitle( "Dicomizing files"  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            public Object construct( )
            {
                handleDicomizationOfImagesStack_1_2( sentToSelectedDicomNetworkNode,
                                                     includeGeometryTags,
                                                     seriesNumberToUseForDicomization,
                                                     seriesDescriptionForDicomization,
                                                     totalNumberOfSlices );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Redicomizing image stack finished!" );
            }
        };
        worker.start( );
    }

    /**
     * Performs the dicomization in an separate thread to keep the GUI responsive!
     * @param sentToSelectedDicomNetworkNode
     * @param includeGeometryTags
     * @param seriesNumberToUseForDicomization
     * @param seriesDescriptionForDicomization
     * @param totalNumberOfSlices
     */
    public void redicomizeCompleteImageStack_1_3_InWorkerThread(  final boolean sentToSelectedDicomNetworkNode,
                                                                  final boolean includeGeometryTags,
                                                                  final int seriesNumberToUseForDicomization,
                                                                  final String seriesDescriptionForDicomization,
                                                                  final int totalNumberOfSlices )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ), false );
        m_dualProgressBar.setTitle( "Dicomizing files"  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            @Override
            public Object construct( )
            {
                handleDicomizationOfImagesStack_1_3( sentToSelectedDicomNetworkNode,
                                                     includeGeometryTags,
                                                     seriesNumberToUseForDicomization,
                                                     seriesDescriptionForDicomization,
                                                     totalNumberOfSlices );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Redicomizing image stack finished!" );
            }
        };
        worker.start( );
    }


    /**
     * Performs the dicomization in an seperate thread to keep the GUI responsive!
     * @param sentToSelectedDicomNetworkNode
     * @param includeGeometryTags
     * @param seriesNumberToUseForDicomization
     * @param seriesDescriptionForDicomization
     * @param totalNumberOfSlices
     */
    public void redicomizeCompleteImageStack_2_3_InWorkerThread(  final boolean sentToSelectedDicomNetworkNode,
                                                                  final boolean includeGeometryTags,
                                                                  final int seriesNumberToUseForDicomization,
                                                                  final String seriesDescriptionForDicomization,
                                                                  final int totalNumberOfSlices )
    {
        m_dualProgressBar = new DualProgressBar( m_parent, 100, 100, new Dimension( 900, 150 ),false );
        m_dualProgressBar.setTitle( "Dicomizing files"  );

        m_dualProgressBar.setUpperProgress( 0 );
        m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( false );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Dicomizing image number : " + 0 + "</HTML>" );

        final org.spectrim.lib.SwingWorker worker = new org.spectrim.lib.SwingWorker( )
        {

            public Object construct( )
            {
                handleDicomizationOfImagesStack_2_3( sentToSelectedDicomNetworkNode,
                                                     includeGeometryTags,
                                                     seriesNumberToUseForDicomization,
                                                     seriesDescriptionForDicomization,
                                                     totalNumberOfSlices );
                return null;
            }

            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                // After calculation set enabled again
                m_dualProgressBar.setVisible( false );
                Dialogs.messageBox( m_parent, "Redicomizing image stack finished!" );
            }
        };
        worker.start( );
    }

     /**
     * Replaces all blanks by "\  " in order to be interpreted
     * correctly by the Windows operating system
     * @param cmd
     * @return
     */
    private String eliminateIllegalFileSystemCharacters( String cmd )
    {

        // first get rid ofm blancs
        cmd = cmd.trim( );

        cmd = cmd.replace( ' ', '_' );
        cmd = cmd.replace( ',', '_' );
        cmd = cmd.replace( ';', '_' );
        cmd = cmd.replace( '%', '_' );
        //cmd.replace( '*', '_' );

        cmd = cmd.replace( 'é', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'è', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'ö', 'o' ); // problematic on some linux systems
        cmd = cmd.replace( 'ü', 'u' ); // problematic on some linux systems
        cmd = cmd.replace( 'à', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'ä', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ö', 'O' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ü', 'U' ); // problematic on some linux systems

        cmd = cmd.replace( '@', '_' ); // problematic on some linux systems
        cmd = cmd.replace( '#', '_' ); // problematic on some linux systems

        cmd = cmd.replace( '?', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '¢', 'c' ); // problematic on some linux systems
        cmd = cmd.replace( '!', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '§', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '¦', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '$', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '[', '_' );  // problematic on some linux systems
        cmd = cmd.replace( ']', '_' );  // problematic on some linux systems

        cmd = cmd.replace( '^', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems

        cmd = cmd.replace( 'ç', 'c' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '"', '_' );  // problematic on some linux systems

        cmd = cmd.replace(">", "_" );
        cmd = cmd.replace("<", "_" );

        cmd = cmd.replace("____", "_" );
        cmd = cmd.replace("___", "_" );
        cmd = cmd.replace("__", "_" );

        return cmd;
    }


    /**
     * Handles the dicomization of the complete image stack.
     * @param sentToSelectedDicomNetworkNode
     * @param includeGeometryTags
     * @param seriesNumberToUseForDicomization
     * @param seriesDescriptionForDicomization
     * @param totalNumberOfSlices
     */
    public void handleDicomizationOfImagesStack_1_2( boolean sentToSelectedDicomNetworkNode,
                                                     boolean includeGeometryTags,
                                                     int seriesNumberToUseForDicomization,
                                                     String seriesDescriptionForDicomization,
                                                     int totalNumberOfSlices )
    {
        // switch off crossline displaying..
        m_1_2_directionView.setDisplayCrossLineMode( false );


        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );
        String seriesInstanceUID = uid_prefix + IP_address + currentTime;


        for ( int i = 0; i < m_dicom3DData.get3DMatrixSize( ); i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / m_dicom3DData.get3DMatrixSize( ) ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            String patName = m_dicom3DData.getImageFileInfoObject_1_2( i ).getPatientName( );
            String accessionNumber = m_dicom3DData.getImageFileInfoObject_1_2( i ).getAccessionNumber( );
            String examDate =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyDate( );
            String examTime =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyTime( );
            int seriesNumber =  new Integer( m_dicom3DData.getImageFileInfoObject_1_2( i ).getSeriesNumber( ).trim() );
            //int instanceNumber =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getInstanceNumber( );

            int instanceNumber =  i + 1;

            DicomNetworkNodeDefaults fdef = DicomNetworkNodeDefaults.getInstance( );

            String workingDir = fdef.getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

            // set the proper image information and display it together with associated overlays ..
            m_1_2_directionView.setImage( m_dicom3DData.getImagePixelData_1_2( (int)i ), m_3DMatrixSize, m_3DMatrixSize, 0 );

            m_1_2_directionView.setCurrentlySelectedImageDTIOverlayNumber( i );
            m_1_2_directionView.setCurrentlySelectedImagefMRIOverlayNumber( i );
            m_1_2_directionView.drawImage( false );

            // Prepare the file system for the new files to be created ..
            String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;
            imageSubDirString = eliminateIllegalFileSystemCharacters( imageSubDirString );

            imageSubDirString = workingDir +  System.getProperty( "file.separator" ) + imageSubDirString;

            File subDir = new File( imageSubDirString );

            if ( !subDir.exists( ) )
            {
                subDir.mkdir( );
            }

            // create the screenshots ..
            String localImageFileNameBase = System.getProperty( "file.separator" ) +
                                            patName + "_" + accessionNumber + "_" + examDate +
                                            "_" + examTime + "_" + ( seriesNumber + 1000 ) + "_" + instanceNumber ;

            localImageFileNameBase = imageSubDirString + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( localImageFileNameBase );

            // now paint the graphics to the image
            BufferedImage image = new BufferedImage( m_1_2_directionView.getWidth( ) ,
                                                     m_1_2_directionView.getHeight( ),
                                                     BufferedImage.TYPE_3BYTE_BGR );

            Graphics g = image.createGraphics( );
            g.translate( 0, 0 );
            m_1_2_directionView.printAll( g );

            if ( g != null )
            {
                g.dispose( );
            }

            String totalJPGFileName = localImageFileNameBase + ".jpg";

            File f = new File( totalJPGFileName );

            try
            {
                ImageIO.write( image, "jpg", f );
            }
            catch ( IOException e )
            {
                System.out.print( "Problem writing local image file in DicomFormattedScreenShotCreator::createIntermediateImageOfScreenshot :\n" );
                e.printStackTrace( );
            }

            DicomFormattedScreenShotCreator dicomCreator = new DicomFormattedScreenShotCreator( m_1_2_directionView,
                                                                                                m_dicom3DData.getImageFileInfoObject_1_2( i  ),
                                                                                                includeGeometryTags,
                                                                                                totalJPGFileName,
                                                                                                seriesNumberToUseForDicomization,
                                                                                                seriesDescriptionForDicomization,
                                                                                                seriesInstanceUID, instanceNumber );

            String m_dicomFileName = dicomCreator.getStampedDicomFileReference( );

            if ( sentToSelectedDicomNetworkNode == true )
            {
                int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );
                DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, dicomCreator.getStampedDicomFileReference( ) );
                dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );
            }

        }

        m_1_2_directionView.setDisplayCrossLineMode( true );

        inititialize3DViews( false );

    }


     /**
     * Handles the dicomization of the complete image stack.
     * @param sentToSelectedDicomNetworkNode
     * @param includeGeometryTags
     * @param seriesNumberToUseForDicomization
     * @param seriesDescriptionForDicomization
     * @param totalNumberOfSlices
     */
    public void handleDicomizationOfImagesStack_1_3( boolean sentToSelectedDicomNetworkNode,
                                                     boolean includeGeometryTags,
                                                     int seriesNumberToUseForDicomization,
                                                     String seriesDescriptionForDicomization,
                                                     int totalNumberOfSlices )
    {
        // switch off crossline displaying..
        m_1_3_directionView.setDisplayCrossLineMode( false );


        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );
        String seriesInstanceUID = uid_prefix + IP_address + currentTime;


        for ( int i = 0; i < m_dicom3DData.get3DMatrixSize( ); i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / m_dicom3DData.get3DMatrixSize( ) ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            String patName = m_dicom3DData.getImageFileInfoObject_1_2( i ).getPatientName( );
            String accessionNumber = m_dicom3DData.getImageFileInfoObject_1_2( i ).getAccessionNumber( );
            String examDate =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyDate( );
            String examTime =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyTime( );
            int seriesNumber =  new Integer( m_dicom3DData.getImageFileInfoObject_1_2( i ).getSeriesNumber( ).trim( ) );

            // we cannot use this number because it contains sometimes the same number:
            // due to this we end up with a mess
            //int instanceNumber =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getInstanceNumber( );
            int instanceNumber =  i;
            
            DicomNetworkNodeDefaults fdef = DicomNetworkNodeDefaults.getInstance( );

            String workingDir = fdef.getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

            // set the proper image information and display it together with associated overlays ..
            m_1_3_directionView.setImage( m_dicom3DData.getImagePixelData_1_3( (int)i ), m_3DMatrixSize, m_3DMatrixSize, 0 );

            m_1_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( i );
            m_1_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( i );
            m_1_3_directionView.drawImage( false );

            // Prepare the file system for the new files to be created ..
            String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;
            imageSubDirString = eliminateIllegalFileSystemCharacters( imageSubDirString );

            imageSubDirString = workingDir +  System.getProperty( "file.separator" ) + imageSubDirString;

            File subDir = new File( imageSubDirString );

            if ( !subDir.exists( ) )
            {
                subDir.mkdir( );
            }

            // create the screenshots ..
            String localImageFileNameBase = System.getProperty( "file.separator" ) +
                                            patName + "_" + accessionNumber + "_" + examDate +
                                            "_" + examTime + "_" + ( seriesNumber + 1000 ) + "_" + instanceNumber ;

            localImageFileNameBase = imageSubDirString + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( localImageFileNameBase );

            // now paint the graphics to the image
            BufferedImage image = new BufferedImage( m_1_3_directionView.getWidth( ) ,
                                                     m_1_3_directionView.getHeight( ),
                                                     BufferedImage.TYPE_3BYTE_BGR );

            Graphics g = image.createGraphics( );
            g.translate( 0, 0 );
            m_1_3_directionView.printAll( g );

            if ( g != null )
            {
                g.dispose( );
            }

            String totalJPGFileName = localImageFileNameBase + ".jpg";

            File f = new File( totalJPGFileName );

            try
            {
                ImageIO.write( image, "jpg", f );
            }
            catch ( IOException e )
            {
                System.out.print( "Problem writing local image file in DicomFormattedScreenShotCreator::createIntermediateImageOfScreenshot :\n" );
                e.printStackTrace( );
            }

            DicomFormattedScreenShotCreator dicomCreator = new DicomFormattedScreenShotCreator( m_1_3_directionView,
                                                                                                m_dicom3DData.getImageFileInfoObject_1_2( i  ),
                                                                                                includeGeometryTags,
                                                                                                totalJPGFileName,
                                                                                                seriesNumberToUseForDicomization,
                                                                                                seriesDescriptionForDicomization,
                                                                                                seriesInstanceUID, instanceNumber );

            String m_dicomFileName = dicomCreator.getStampedDicomFileReference( );

            if ( sentToSelectedDicomNetworkNode == true )
            {
                int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );
                DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, dicomCreator.getStampedDicomFileReference( ) );
                dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );
            }

        }

        m_1_3_directionView.setDisplayCrossLineMode( true );

        inititialize3DViews( false );

    }


    /**
     * Handles the dicomization of the complete image stack.
     * @param sentToSelectedDicomNetworkNode
     * @param seriesDescriptionForDicomization
     * @param totalNumberOfSlices
     */
    public void handleDicomizationOfImagesStack_2_3( boolean sentToSelectedDicomNetworkNode,
                                                     boolean includeGeometryTags,
                                                     int seriesNumberToUseForDicomization,
                                                     String seriesDescriptionForDicomization,
                                                     int totalNumberOfSlices )
    {
        // switch off crossline displaying..
        m_2_3_directionView.setDisplayCrossLineMode( false );


        String uid_prefix = "1.3.12.2.1107.5.2";

        DicomNetworkNodeDefaults dnwnd = DicomNetworkNodeDefaults.getInstance( );
        String IP_address = dnwnd.getProperty( DicomNetworkNodeDefaults.LOCAL_HOST_IP_ADDRESS );
        long currentTime = System.currentTimeMillis( );
        String seriesInstanceUID = uid_prefix + IP_address + currentTime;


        for ( int i = 0; i < m_dicom3DData.get3DMatrixSize( ); i++ )
        {

            m_dualProgressBar.setLowerProgress( (int)( 100.0 * (double)i / m_dicom3DData.get3DMatrixSize( ) ) );
            m_dualProgressBar.setLowerLabel( "    Dicomizing image number : " +  i );

            String patName = m_dicom3DData.getImageFileInfoObject_1_2( i ).getPatientName( );
            String accessionNumber = m_dicom3DData.getImageFileInfoObject_1_2( i ).getAccessionNumber( );
            String examDate =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyDate( );
            String examTime =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getStudyTime( );
            int seriesNumber =  new Integer( m_dicom3DData.getImageFileInfoObject_1_2( i ).getSeriesNumber( ).trim( ) ).intValue( );
            // int instanceNumber =  m_dicom3DData.getImageFileInfoObject_1_2( i ).getInstanceNumber( );

            int instanceNumber =  i;

            DicomNetworkNodeDefaults fdef = DicomNetworkNodeDefaults.getInstance( );

            String workingDir = fdef.getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

            // set the proper image information and display it together with associated overlays ..
            m_2_3_directionView.setImage( m_dicom3DData.getImagePixelData_2_3( (int)i ), m_3DMatrixSize, m_3DMatrixSize, 0 );

            m_2_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( i );
            m_2_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( i );
            m_2_3_directionView.drawImage( false );

            // Prepare the file system for the new files to be created ..
            String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;
            imageSubDirString = eliminateIllegalFileSystemCharacters( imageSubDirString );

            imageSubDirString = workingDir +  System.getProperty( "file.separator" ) + imageSubDirString;

            File subDir = new File( imageSubDirString );

            if ( !subDir.exists( ) )
            {
                subDir.mkdir( );
            }

            // create the screenshots ..
            String localImageFileNameBase = System.getProperty( "file.separator" ) +
                                            patName + "_" + accessionNumber + "_" + examDate +
                                            "_" + examTime + "_" + ( seriesNumber + 1000 ) + "_" + instanceNumber ;

            localImageFileNameBase = imageSubDirString + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( localImageFileNameBase );

            // now paint the graphics to the image
            BufferedImage image = new BufferedImage( m_2_3_directionView.getWidth( ) ,
                                                     m_2_3_directionView.getHeight( ),
                                                     BufferedImage.TYPE_3BYTE_BGR );

            Graphics g = image.createGraphics( );
            g.translate( 0, 0 );
            m_2_3_directionView.printAll( g );

            if ( g != null )
            {
                g.dispose( );
            }

            String totalJPGFileName = localImageFileNameBase + ".jpg";

            File f = new File( totalJPGFileName );

            try
            {
                ImageIO.write( image, "jpg", f );
            }
            catch ( IOException e )
            {
                System.out.print( "Problem writing local image file in DicomFormattedScreenShotCreator::createIntermediateImageOfScreenshot :\n" );
                e.printStackTrace( );
            }

            DicomFormattedScreenShotCreator dicomCreator = new DicomFormattedScreenShotCreator( m_2_3_directionView,
                                                                                                m_dicom3DData.getImageFileInfoObject_1_2( i  ),
                                                                                                includeGeometryTags,
                                                                                                totalJPGFileName,
                                                                                                seriesNumberToUseForDicomization,
                                                                                                seriesDescriptionForDicomization,
                                                                                                seriesInstanceUID, instanceNumber );

            String m_dicomFileName = dicomCreator.getStampedDicomFileReference( );

            if ( sentToSelectedDicomNetworkNode == true )
            {
                int selectedRemoteDicomNodeNumber = DicomNetworkNodes.getInstance( ).getSelectedRemoteDicomNodeNumber( );
                DicomFileOverNetworkTransferrer dcmTransferrer = new DicomFileOverNetworkTransferrer( selectedRemoteDicomNodeNumber, dicomCreator.getStampedDicomFileReference( ) );
                dcmTransferrer.performDicomSendOfLocalDicomFileToSelectedRemoteNode( );
            }

        }

        m_2_3_directionView.setDisplayCrossLineMode( true );

        inititialize3DViews( false );

    }


    /**
     * Handles the events by evoking them in control 12
     * @param e
     */
    private void handleEventsForImageControl_1_2( ActionEvent e )
    {

        if ( e.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_AND_SEND_BUTTON_PRESSED )
        {
            int seriesNumberToUseForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
            String seriesDescriptionForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

            if ( e.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
            {
                redicomizeCompleteImageStack_1_2_InWorkerThread( true, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
            else
            {
                redicomizeCompleteImageStack_1_2_InWorkerThread( true, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
        }

        if ( e.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_ONLY_BUTTON_PRESSED )
        {

            int seriesNumberToUseForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
            String seriesDescriptionForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

            if ( e.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
            {
                redicomizeCompleteImageStack_1_2_InWorkerThread( false, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
            else
            {
                redicomizeCompleteImageStack_1_2_InWorkerThread( false, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
        }
    }

    /**
     * Handles the events by invoking them in control 13
     * @param e
     */
    private void handleEventsForImageControl_1_3( ActionEvent e )
    {
        if ( e.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_AND_SEND_BUTTON_PRESSED )
        {
            int seriesNumberToUseForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
            String seriesDescriptionForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

            if ( e.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
            {
                redicomizeCompleteImageStack_1_3_InWorkerThread( true, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization , m_3DMatrixSize );
            }
            else
            {
                redicomizeCompleteImageStack_1_3_InWorkerThread( true, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization , m_3DMatrixSize );
            }
        }

        if ( e.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_ONLY_BUTTON_PRESSED )
        {

            int seriesNumberToUseForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
            String seriesDescriptionForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

            if ( e.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
            {
                redicomizeCompleteImageStack_1_3_InWorkerThread( false, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
            else
            {
                redicomizeCompleteImageStack_1_3_InWorkerThread( false, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
        }
    }

    /**
     * Handles the events by invoking them in control 13
     * @param e
     */
    private void handleEventsForImageControl_2_3( ActionEvent e )
    {
        if ( e.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_AND_SEND_BUTTON_PRESSED )
        {
            int seriesNumberToUseForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
            String seriesDescriptionForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

            if ( e.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
            {
                redicomizeCompleteImageStack_2_3_InWorkerThread( true, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
            else
            {
                redicomizeCompleteImageStack_2_3_InWorkerThread( true, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
        }

        if ( e.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_DICOMIZE_ONLY_BUTTON_PRESSED )
        {

            int seriesNumberToUseForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesNumberForDicomization( );
            String seriesDescriptionForDicomization = m_dicomFileCreatorOptionsFrame.getUserSelectedSeriesDescriptionForDicomization( );

            if ( e.getActionCommand( ).compareTo( DicomFileCreatorOptionsFrame.ALSO_INCLUDE_SPATIAL_TAGS ) == 0 )
            {
                redicomizeCompleteImageStack_2_3_InWorkerThread( false, true, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
            else
            {
                redicomizeCompleteImageStack_2_3_InWorkerThread( false, false, seriesNumberToUseForDicomization, seriesDescriptionForDicomization, m_3DMatrixSize );
            }
        }
    }




    /**
     * Called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        Object sender = e.getSource( );


        if ( sender == m_dicomFileCreatorOptionsFrame )
        {
            if ( e.getID( ) == DicomFileCreatorOptionsFrame.NOTIFY_CANCEL_BUTTON_PRESSED_EVENT )
            {
                m_dicomFileCreatorOptionsFrame.dispose( );
            }

            if ( m_selectedImageFrame == 12 )
            {
                handleEventsForImageControl_1_2( e );
            }

            if ( m_selectedImageFrame == 13 )
            {
                handleEventsForImageControl_1_3( e );
            }

            if ( m_selectedImageFrame == 23 )
            {
                handleEventsForImageControl_2_3( e );
            }

        }

        if ( sender ==  m_1_2_directionView )
        {
            if ( ( e.getID( ) == DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT ) && 
                 ( e.getActionCommand( ).compareTo( DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT_ACTION_COMMAND ) == 0 ) )
            {
                
                int newX = m_1_2_directionView.getLastMouseClickXPosition( );
                int newY = m_1_2_directionView.getLastMouseClickYPosition( );
                
                float newXCorrected = newX /  m_1_2_directionView.getZoom( );
                float newYCorrected = newY /  m_1_2_directionView.getZoom( );


                m_1_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newXCorrected );
                m_1_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newXCorrected );

                m_1_3_directionView.setImage( m_dicom3DData.getImagePixelData_1_3( (int)( newXCorrected ) ), m_3DMatrixSize, m_3DMatrixSize, 0 );

                m_1_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newXCorrected );
                m_1_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newXCorrected );

                m_1_3_directionView.setComputedGeometricSliceAttributes( m_dicom3DData.getDirectionVector_1_3_X( ),
                                                                         m_dicom3DData.getDirectionVector_1_3_Y( ), 
                                                                         m_1_2_directionView.compute3DCoordinate( m_3DMatrixSize/2, m_3DMatrixSize/2 ), 
                                                                         newX, newY,
                                                                         m_dicom3DData.getFOVY_12( ), m_dicom3DData.getFOVZ_12( ) );
                   
                m_1_3_directionView.updateCrossLineY( newY );
                m_1_3_directionView.drawImage( false );


                m_2_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newYCorrected );
                m_2_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newYCorrected );

                m_2_3_directionView.setImage( m_dicom3DData.getImagePixelData_2_3( (int)( newYCorrected ) ), m_3DMatrixSize, m_3DMatrixSize, 0 );
                
                
                m_2_3_directionView.setComputedGeometricSliceAttributes( m_dicom3DData.getDirectionVector_2_3_X( ),
                                                                         m_dicom3DData.getDirectionVector_2_3_Y( ), 
                                                                         m_1_2_directionView.compute3DCoordinate( m_3DMatrixSize / 2, m_3DMatrixSize / 2 ), 
                                                                         newX, newY,
                                                                         m_dicom3DData.getFOVZ_12( ), m_dicom3DData.getFOVX_12( ) );
             
                m_2_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newYCorrected );
                m_2_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newYCorrected );

                m_2_3_directionView.updateCrossLineY( newX );
                m_2_3_directionView.drawImage( false );
                
            }
        }
        
        if ( sender ==  m_1_3_directionView )
        {
            if ( ( e.getID( ) == DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT ) && 
                 ( e.getActionCommand( ).compareTo( DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT_ACTION_COMMAND ) == 0 ) )
            {
                int newX = m_1_3_directionView.getLastMouseClickXPosition( );
                int newY = m_1_3_directionView.getLastMouseClickYPosition( );
                
                float newXCorrected = newX /  m_1_3_directionView.getZoom( );
                float newYCorrected = newY /  m_1_3_directionView.getZoom( );
                
                
                m_1_2_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newXCorrected );
                m_1_2_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newXCorrected );


                m_1_2_directionView.setImage( m_dicom3DData.getImagePixelData_1_2( (int)newXCorrected ), m_3DMatrixSize, m_3DMatrixSize, 0,
                                              m_dicom3DData.getImageFileInfoObject_1_2( (int)newXCorrected ), 
                                              m_dicom3DData.getSiemensSpecificFileInfomationObject_1_2( (int)newXCorrected ) );
                                              
                m_1_2_directionView.updateCrossLineY( newY );
                m_1_2_directionView.drawImage( false );
                
                m_2_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newYCorrected );
                m_2_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newYCorrected );
                
                m_2_3_directionView.setImage( m_dicom3DData.getImagePixelData_2_3( (int)newYCorrected ), m_3DMatrixSize, m_3DMatrixSize, 0 );

                m_2_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newYCorrected );
                m_2_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newYCorrected );

                m_2_3_directionView.updateCrossLineX( newX );
                m_2_3_directionView.drawImage( false );
            }
        }
        
        if ( sender ==  m_2_3_directionView )
        {
            if ( ( e.getID( ) == DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT ) && 
                 ( e.getActionCommand( ).compareTo( DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT_ACTION_COMMAND ) == 0 ) )
            {
                int newX = m_2_3_directionView.getLastMouseClickXPosition( );
                int newY = m_2_3_directionView.getLastMouseClickYPosition( );

                float newXCorrected = newX /  m_2_3_directionView.getZoom( );
                float newYCorrected = newY /  m_2_3_directionView.getZoom( );

                m_1_2_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newXCorrected );
                m_1_2_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newXCorrected );

                m_1_2_directionView.setImage( m_dicom3DData.getImagePixelData_1_2( (int)newXCorrected ), m_3DMatrixSize, m_3DMatrixSize, 0,
                                              m_dicom3DData.getImageFileInfoObject_1_2( (int)newXCorrected ), 
                                              m_dicom3DData.getSiemensSpecificFileInfomationObject_1_2( (int)newXCorrected ) );
            
                m_1_2_directionView.updateCrossLineX( newY );
                m_1_2_directionView.drawImage( false );

                m_1_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newYCorrected );
                m_1_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newYCorrected );

                m_1_3_directionView.setImage( m_dicom3DData.getImagePixelData_1_3( (int)newYCorrected ), m_3DMatrixSize, m_3DMatrixSize, 0 );
                m_1_3_directionView.updateCrossLineX( newX );

                m_1_3_directionView.setCurrentlySelectedImageDTIOverlayNumber( (int)newYCorrected );
                m_1_3_directionView.setCurrentlySelectedImagefMRIOverlayNumber( (int)newYCorrected );

                m_1_3_directionView.drawImage( false );
            }
        }
        
        if ( sender ==  m_fourthView )
        {
            if ( ( e.getID( ) == DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT ) && 
                 ( e.getActionCommand( ).compareTo( DicomSliceCanvas.CROSS_HAIR_CHANGE_EVENT_ACTION_COMMAND ) == 0 ) )
            {
                int newX = m_fourthView.getLastMouseClickXPosition( );
                int newY = m_fourthView.getLastMouseClickYPosition( );

                m_fourthView.updateCrossLineX( newX );
                m_fourthView.updateCrossLineY( newY );
                
                
                m_fourthView.setImageTopTitle( m_fourthView.get3DCoordinateAsString( ) );
               
                m_fourthView.drawImage( false );
                
            }
        }
        
        if ( sender == m_dicom3DData )
        {
            if ( e.getID( ) ==  Dicom3DImageSeriesData.THREE_D_VIEWER_DATASET_LOADED )
            {
                notifyActionListeners( THREE_D_VIEWER_DATASET_LOADED );
                inititialize3DViews( true );
            }

            if ( e.getID( ) ==  Dicom3DImageSeriesData.THREE_D_VIEWER_DATASET_LOAD_ERROR )
            {
                org.spectrim.lib.Dialogs.messageBox( m_parent, "Could not load the 3D data" );
            }
        }
    }


    /**
     * Adds an action listener to the list of listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }


     /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }


    /**
     * Returns a reference on the DicomSliceCanvasPolyDraw m_1_2_directionView( )
     * @return
     */
    public DicomSliceCanvasPolyDraw get_1_2_directionView( )
    {
        return m_1_2_directionView;
    }

    /**
     * Returns a reference on the DicomSliceCanvasPolyDraw m_1_3_directionView( )
     * @return
     */
    public DicomSliceCanvasPolyDraw get_1_3_directionView( )
    {
        return m_1_3_directionView;
    }

    /**
     * Returns a reference on the DicomSliceCanvasPolyDraw m_1_3_directionView( )
     * @return
     */
    public DicomSliceCanvasPolyDraw get_2_3_directionView( )
    {
        return m_2_3_directionView;
    }

    // Attributes
    private JFrame m_parent;
    
    private Dicom3DImageSeriesData m_dicom3DData;
    private DicomImageSeriesData m_fourthViewData;
    
    private DicomSliceCanvasPolyDraw m_1_2_directionView;
    private DicomSliceCanvasPolyDraw m_1_3_directionView;
    private DicomSliceCanvasPolyDraw m_2_3_directionView;
    
    private DicomSliceCanvasPolyDraw m_fourthView;
    
    private JPanel m_topPanel;
    
    private JPanel m_upperPanelLeft;
    private JPanel m_lowerPanelLeft;
    private JPanel m_upperPanelRight;
    private JPanel m_lowerPanelRight;
    
    private JPanel m_upperCollectPanel;
    private JPanel m_lowerCollectPanel;
    
    private LinkedList<String> m_selected3DFileReferences;
    private LinkedList<String> m_fourthViewFileReferences;
        
    private int m_3DMatrixSize;

    private boolean[][][] m_activationImageOverlays;
    private boolean[][][] m_dtiFiberImageOverlays;
    private Color[][][]   m_dtiFiberColors;


    DicomFileCreatorOptionsFrame m_dicomFileCreatorOptionsFrame;

    private int m_selectedImageFrame;

    private DualProgressBar m_dualProgressBar;


    // For proper serialization
    private static final long serialVersionUID = 6482325998221223L;
    
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );

    public final static int THREE_D_VIEWER_DATASET_LOADED = 0;

    public final static int IO_OK = 0;

    String m_org_series_description = null;


}
