/*
 * File: $Id: DicomFormattedScreenShotCreator.java,v 1.6 2009/14/05 13:28:39 hans Exp $
 * History
 * Date: 14.05.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.dicom;

import org.spectrim.lib.Globals;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import javax.imageio.ImageIO;
import javax.swing.JComponent;

/**
 * This class handles the creation of a Dicom format file of a screen shot
 * of a particular part of a running JAVA application.
 * @author slotboom
 */
public class DicomFormattedScreenShotCreator
{

    /**
     * Default constructor of DicomFormattedScreenShotCreator.
     * @param comp
     */
    public DicomFormattedScreenShotCreator( JComponent comp )
    {
        m_component = comp;
    }

    /**
     * Default constructor of DicomFormattedScreenShotCreator.
     * @param comp
     * @param fileInfo
     * @param imageDescription
     * @param includeGeometryTags
     * @param g
     */
    public DicomFormattedScreenShotCreator( JComponent comp, FileInfo fileInfo, 
                                            String imageDescription, 
                                            boolean includeGeometryTags )
    {
        m_component = comp;
        m_dicomInformationAssociation = fileInfo;
        m_imageDescription = imageDescription;
        m_includeGeometryTags = includeGeometryTags;

        String patName          = m_dicomInformationAssociation.getPatientName( );
        String accessionNumber  = m_dicomInformationAssociation.getAccessionNumber( );
        String examDate         = m_dicomInformationAssociation.getStudyDate( );
        String examTime         = m_dicomInformationAssociation.getStudyTime( );
        int seriesNumber        = Integer.parseInt(m_dicomInformationAssociation.getSeriesNumber( ).trim( ) );
        m_currentInstanceNumber = Integer.parseInt(m_dicomInformationAssociation.getInstanceNumber( ).trim( ) );
        
        String imageDir = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

        String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;

        imageSubDirString = imageDir + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( imageSubDirString );

        File imageSubDir = new File( imageSubDirString );

        if ( !imageSubDir.exists( ) )
        {
            imageSubDir.mkdir( );
        }

        if ( imageSubDir.exists( ) )
        {
            m_localImageFileNameBase = patName + "_" + accessionNumber + "_" + examDate +
                                       "_" + examTime + "_" + ( seriesNumber  + 1000 ) + "_" + m_currentInstanceNumber ;

            m_localImageFileNameBase = imageSubDirString + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( m_localImageFileNameBase );


            // Do the dicom conversion!
            createDicomFileInLocalFileSystem( );
        }
    }



    /**
     * Default constructor of DicomFormattedScreenShotCreator;
     * NB: instance number has to be set by caller, is not necessarily defined
     * by Dicom file association!!
     * @param comp
     * @param fileInfo
     * @param includeGeometryTags
     * @param jpgFileName
     * @param seriesNumberToUseForDicomization
     * @param seriesDescription
     * @param seriesInstanceUID
     * @param instanceNumber
     */
    public DicomFormattedScreenShotCreator( JComponent comp, FileInfo fileInfo, 
                                            boolean includeGeometryTags,
                                            String jpgFileName, 
                                            int seriesNumberToUseForDicomization,
                                            String seriesDescription, 
                                            String seriesInstanceUID, 
                                            int instanceNumber )
    {
        m_component = comp;
        m_dicomInformationAssociation = fileInfo;
        m_imageDescription = seriesDescription;
        m_includeGeometryTags = includeGeometryTags;

        String patName         = m_dicomInformationAssociation.getPatientName( );
        String accessionNumber = m_dicomInformationAssociation.getAccessionNumber( );
        String examDate        = m_dicomInformationAssociation.getStudyDate( );
        String examTime        = m_dicomInformationAssociation.getStudyTime( );

        int seriesNumber       = new Integer( m_dicomInformationAssociation.getSeriesNumber( ).trim( ) );

        m_currentInstanceNumber = instanceNumber;

        String imageDir = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

        String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;

        imageSubDirString = imageDir + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( imageSubDirString );

        File imageSubDir = new File( imageSubDirString );

        if ( !imageSubDir.exists( ) )
        {
            imageSubDir.mkdir( );
        }

        if ( imageSubDir.exists( ) )
        {
            m_localImageFileNameBase = patName + "_" + accessionNumber + "_" + examDate +
                                       "_" + examTime + "_" + ( seriesNumber  + 1000 ) + "_" + m_currentInstanceNumber ;

            m_localImageFileNameBase = imageSubDirString + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( m_localImageFileNameBase );


            // do the dicom conversion!
            createDicomFileInLocalFileSystem( jpgFileName, seriesNumberToUseForDicomization, seriesDescription, seriesInstanceUID );
        }
    }



    /**
     * Default constructor of DicomFormattedScreenShotCreator
     * NB instance number has to be set by caller, is not necessarily defined.
     * by Dicom file association!!
     * @param comp
     * @param fileInfo
     * @param includeGeometryTags
     * @param jpgFileName
     * @param seriesNumberToUseForDicomization
     * @param seriesDescription
     * @param seriesInstanceUID
     */
    public DicomFormattedScreenShotCreator( JComponent comp, FileInfo fileInfo, 
                                            boolean includeGeometryTags,
                                            String jpgFileName, 
                                            int seriesNumberToUseForDicomization,
                                            String seriesDescription, 
                                            String seriesInstanceUID )
    {
        m_component = comp;
        m_dicomInformationAssociation = fileInfo;
        m_imageDescription = seriesDescription;
        m_includeGeometryTags = includeGeometryTags;

        String patName          = m_dicomInformationAssociation.getPatientName( );
        String accessionNumber  = m_dicomInformationAssociation.getAccessionNumber( );
        String examDate         = m_dicomInformationAssociation.getStudyDate( );
        String examTime         = m_dicomInformationAssociation.getStudyTime( );
        int    seriesNumber     = new Integer( m_dicomInformationAssociation.getSeriesNumber( ).trim( ) );
        m_currentInstanceNumber = new Integer( m_dicomInformationAssociation.getInstanceNumber( ).trim( ) ).intValue( );

        String imageDir = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );

        String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;

        imageSubDirString = imageDir + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( imageSubDirString );

        File imageSubDir = new File( imageSubDirString );

        if ( !imageSubDir.exists( ) )
        {
            imageSubDir.mkdir( );
        }

        if ( imageSubDir.exists( ) )
        {
            m_localImageFileNameBase = patName + "_" + accessionNumber + "_" + examDate +
                                       "_" + examTime + "_" + ( seriesNumber  + 1000 ) + "_" + m_currentInstanceNumber ;

            m_localImageFileNameBase = imageSubDirString + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( m_localImageFileNameBase );


            // do the dicom conversion!
            createDicomFileInLocalFileSystem( jpgFileName, seriesNumberToUseForDicomization, seriesDescription, seriesInstanceUID );
        }
    }


     /**
      *
      * Creates a screen shot of a given JComponent
      *
      * @param intermediate_file_format file format of the secondary capture, currently only jpg
      * @param dicomInformationAssociation from the file all essential Dicom storage attributes are derived
      */
    public void createIntermediateImageOfScreenshot( String intermediate_file_format,
                                                     FileInfo dicomInformationAssociation )
    {
        m_dicomInformationAssociation = dicomInformationAssociation;
        // currently only jpg-s are allowed
        if ( intermediate_file_format.compareTo( "jpg" ) == 0 )
        {
            String patName = dicomInformationAssociation.getPatientName( );
            String accessionNumber = dicomInformationAssociation.getAccessionNumber( );
            String examDate = dicomInformationAssociation.getStudyDate( );
            String examTime = dicomInformationAssociation.getStudyTime( );
            int seriesNumber = new Integer( dicomInformationAssociation.getSeriesNumber( ).trim( ) );
            m_currentInstanceNumber = new Integer( dicomInformationAssociation.getInstanceNumber( ).trim( ) );

            
            // String workingDir = System.getProperty( "user.dir" );
            String imageDir = DicomNetworkNodeDefaults.getInstance( ).getProperty( DicomNetworkNodeDefaults.CURRENT_INTERMEDIATE_DICOM_CREATION_DIRECTORY );
            
            String imageSubDirString  = patName + "_" + examDate + "_" + accessionNumber;

            imageSubDirString = imageDir + System.getProperty( "file.separator" ) + eliminateIllegalFileSystemCharacters( imageSubDirString );

            File imageSubDir = new File( imageSubDirString );

            if ( !imageSubDir.exists( ) )
            {
                imageSubDir.mkdir( );
            }
            
            m_localImageFileNameBase = imageSubDir + System.getProperty( "file.separator" ) +
                                       patName + "_" + accessionNumber + "_" + examDate +
                                       "_" + examTime + "_" + ( seriesNumber + 1000 ) + "_" + m_currentInstanceNumber ;
            
            //System.out.println( ">>>> Instance number = " + m_currentInstanceNumber );
            
            m_localImageFileNameBase = eliminateIllegalFileSystemCharacters( m_localImageFileNameBase );
            
            m_localFile = new File( m_localImageFileNameBase + "." + intermediate_file_format );


            BufferedImage image = new BufferedImage( m_component.getWidth( ) , m_component.getHeight( ),
                                                     BufferedImage.TYPE_3BYTE_BGR );

            Graphics g = image.createGraphics( );
            g.translate( 0, 0 );
            m_component.printAll( g );

            if ( g != null )
            {
                g.dispose( );
            }

            try
            {
                ImageIO.write( image, intermediate_file_format, m_localFile );
            }
            catch ( IOException e )
            {
                System.out.print( "Problem writing local image file in DicomFormattedScreenShotCreator::createIntermediateImageOfScreenshot :\n" );
            }
        }
    }


    /**
     *
     * @return true if Dicom file could be written to local file system.
     */
     public boolean createDicomFileInLocalFileSystem( )
     {

         if ( createJpg2DcmConfigFile( null ) == 0 )
         {
            return performJPGTODicomConversion( null );
         }

         return false;
     }


    /**
     * Creates a new Dicom file in the file system.
     * @param jpgFileName
     * @param seriesNumberToUseForDicomization
     * @param seriesDescription
     * @param seriesInstanceUID
     * @return true if Dicom file could be written to local file system.
     */
    public boolean createDicomFileInLocalFileSystem( String jpgFileName, 
                                                     int seriesNumberToUseForDicomization, 
                                                     String seriesDescription, 
                                                     String seriesInstanceUID )
    {

        if ( createJpg2DcmConfigFile( jpgFileName, seriesNumberToUseForDicomization, seriesDescription, seriesInstanceUID ) == 0 )
        {
           return performJPGTODicomConversion( jpgFileName );
        }

        return false;
    }

    /**
     * 
     * Creates a new Dicom file in local file system.
     * 
     * @param jpgFileName
     * @param imageDescription
     * @return true if Dicom file could be written to local file system
     */
     public boolean createDicomFileInLocalFileSystem( String jpgFileName, String imageDescription )
     {
         m_imageDescription = imageDescription;
         if ( createJpg2DcmConfigFile( jpgFileName ) == 0 )
         {
            return performJPGTODicomConversion( jpgFileName );
         }
         return false;
     }


    /**
     * Creates a new Dicom file in local file system.
     * @param imageDescription
     * @return true if Dicom file could be written to local file system
     */
     public boolean createDicomFileInLocalFileSystem( String imageDescription )
     {
         m_imageDescription = imageDescription;
         if ( createJpg2DcmConfigFile( imageDescription ) == 0 )
         {
            return performJPGTODicomConversion( imageDescription );
         }
         return false;
     }


    /**
     * Creates a automatic configuration file which is used to perform the
     * the jpg -> Dicom conversion; This method creates all necessary tags
     * needed for a valid dcm-file.
     * @param jpgFileName
     * @return 
     */
     public int createJpg2DcmConfigFile( String jpgFileName )
     {
        if ( jpgFileName == null )
        {
            m_configFileName = m_localImageFileNameBase + ".cfg";
        }
        else
        {
            m_configFileName = jpgFileName.substring( 0, jpgFileName.lastIndexOf(".") ) + ".cfg";
        }
        try
        {

            FileWriter fileWriter = new FileWriter( m_configFileName );
            fileWriter.write( "# jpg2dcm Default Configuration for encapsulating JPEG Baseline streams into\n" );
            fileWriter.write( "# DICOM Secondary Capture Image objects\n" );
            fileWriter.write( "# (s. DICOM Part 3, A.32.7 Video Photographic Image IOD)\n\n");
            fileWriter.write( "# Patient Module Attributes \n");
            fileWriter.write( "# Patient's Name\n");
            fileWriter.write( "00100010:" + m_dicomInformationAssociation.getPatientName( ) + "\n" );
            fileWriter.write( "# Patient ID\n");
            fileWriter.write( "00100020:" + m_dicomInformationAssociation.getPatientID( ) + "\n" );
            fileWriter.write( "# Issuer of Patient ID\n" );
            fileWriter.write( "#00100021:" );
            fileWriter.write( "# Patient's Birth Date\n" );
            fileWriter.write( "00100030:" + m_dicomInformationAssociation.getPatientBirthDate( ) + "\n" );
            fileWriter.write( "# Patient's Sex\n" );
            fileWriter.write( "00100040:" + m_dicomInformationAssociation.getPatientSex( ) + "\n\n" );
            fileWriter.write( "# General Study Module Attributes\n" );
            fileWriter.write( "# Study Instance UID\n" );
            fileWriter.write( "#0020000D:" + m_dicomInformationAssociation.getStudyInstanceUID( ) + "\n" );
            fileWriter.write( "# Study Date\n" );
            fileWriter.write( "00080020:" + m_dicomInformationAssociation.getStudyDate( ) + "\n" );
            fileWriter.write( "# Study Time\n" );
            fileWriter.write( "00080030:" + m_dicomInformationAssociation.getStudyTime( ) + "\n" );
            fileWriter.write( "# Referring Physician's Name\n" );
            fileWriter.write( "00080090:" + "DIN-Bildlabor\n" );
            fileWriter.write( "# Study ID\n" );
            fileWriter.write( "00200010:" + m_dicomInformationAssociation.getStudyID( ) + "\n" );
            fileWriter.write( "# Accession Number\n" );
            fileWriter.write( "00080050:" + m_dicomInformationAssociation.getAccessionNumber( ) + "\n" );
            fileWriter.write( "# Study Description\n" );
            fileWriter.write( "00081030:" + m_dicomInformationAssociation.getStudyDescription( ) + "\n" );
            fileWriter.write( "\n" );
            fileWriter.write( "# General Series Module Attributes\n" );
            fileWriter.write( "# Modality\n" );
            fileWriter.write( "00080060:MR\n" ); // for the time being keep this setting: most important is that it does display well
            fileWriter.write( "# Series Instance UID\n" );
            //fileWriter.write( "0020000E:" + m_dicomInformationAssociation.getSeriesInstanceUID( ) + "\n" );
            fileWriter.write( "# Series description\n" );
            fileWriter.write( "0008103E:" + m_imageDescription + "\n" );
            
            //fileWriter.write( "0008103E:" + "ENCODING" + "\n" );
            
            fileWriter.write( "# Series Number\n" );
            fileWriter.write( "00200011:" + ( m_dicomInformationAssociation.getSeriesNumber( ).trim( ) + 200 ) + "\n" );

            fileWriter.write( "# Series Date\n" );
            fileWriter.write( "00080021:" + ( m_dicomInformationAssociation.getSeriesDate( ).trim( ) ) + "\n" );

            fileWriter.write( "# Series Time\n" );
            fileWriter.write( "00080031:" + m_dicomInformationAssociation.getSeriesTime( ).trim( ) + "\n" );

            //fileWriter.write( "00200011:" + seriesNumberToUseForDicomization + 1000 + "\n" );
            
            
            fileWriter.write( "# General Equipment Module Attributes" + "\n" );
            fileWriter.write( "# Manufacturer" + "\n" );
            fileWriter.write( "00080070:" + "© DIN Bildlabor 2003-2010" + "\n" );
            
            fileWriter.write( "# SC Equipment Module Attributes\n" );
            fileWriter.write( "# Conversion Type\n" );
            fileWriter.write( "00080064:SI" );

            fileWriter.write( "# General Image Module Attributes\n" );
            fileWriter.write( "# Instance Number\n" );
            fileWriter.write( "00200013:" + m_currentInstanceNumber + "\n" );
            
            //System.out.println( ">>>> Instance number =" + m_currentInstanceNumber );

            fileWriter.write( "# Image Pixel Module Attributes (detected from JPEG file, if not specified)" + "\n" );
            fileWriter.write( "# Samples per Pixel\n");
            fileWriter.write( "#00280002:" + m_dicomInformationAssociation.getSamplesPerPixel( ) + "\n" );
            fileWriter.write( "# Photometric Interpretation\n");
            fileWriter.write( "#00280004:" + m_dicomInformationAssociation.getPhotometricInterpretation( ) + "\n" );
            fileWriter.write( "# Planar Configuration\n" );
            fileWriter.write( "#00280006:" + m_dicomInformationAssociation.getPlanarConfiguration() + "\n" );
            fileWriter.write( "# Rows\n" );
            fileWriter.write( "#00280010:" + m_dicomInformationAssociation.getImageHeight( ) + "\n" );
            fileWriter.write( "# Columns\n" );
            fileWriter.write( "#00280011:" + m_dicomInformationAssociation.getImageWidth( ) + "\n" );
            fileWriter.write( "# Bits Allocated\n");
            fileWriter.write( "#00280100:" + m_dicomInformationAssociation.getFileType( ) + "\n" );
            fileWriter.write( "# Bits Stored\n" );
            fileWriter.write( "#00280101:" + m_dicomInformationAssociation.getBitsStored( ) +"\n" );
            fileWriter.write( "# High Bit\n" );
            fileWriter.write( "#00280102:" + m_dicomInformationAssociation.getHighBit( ) + "\n" );
            fileWriter.write( "# Pixel Representation\n" );
            fileWriter.write( "#00280103:" + m_dicomInformationAssociation.getPixelRepresentation( ) + "\n\n" );
            fileWriter.write( "# SOP Common Module Attributes\n" );
            fileWriter.write( "# SOP Class UID\n" );
            fileWriter.write( "00080016:1.2.840.10008.5.1.4.1.1.7\n" ); // secondary capture
            fileWriter.write( "# SOP Instance UID\n");
            fileWriter.write( "#00080018:" + "sop_instance_iud" + "\n" );

            Globals globs = Globals.getInstance( );

            fileWriter.write( "# Software version" + "\n" );
            fileWriter.write( "00181020:" + globs.APP_NAME + " " + globs.APP_VERSION + "\n" );
                
            
            
            // user also wants the geometry information tags to be added
            if ( m_includeGeometryTags == true )
            {     
                
                String imagePosition = m_dicomInformationAssociation.getImagePositionPatient( ).replaceAll( "\\\\", "\\\\\\\\" );
                String imageOrientation = m_dicomInformationAssociation.getImageOrientationPatient( ).replaceAll( "\\\\", "\\\\\\\\" );
                
                fileWriter.write( "# Slice location" + "\n" );
                fileWriter.write( "00201041:" + m_dicomInformationAssociation.getSliceLocation( ) + "\n" );
                fileWriter.write( "# Image orientation" + "\n" );
                fileWriter.write( "00200037:" + imageOrientation + "\n" );
                fileWriter.write( "# Image position" + "\n" );
                fileWriter.write( "00200032:" + imagePosition + "\n" );
                fileWriter.write( "# Slice thickness" + "\n" );
                fileWriter.write( "00180050:" + m_dicomInformationAssociation.getSliceThickness( ) + "\n" );
                
                fileWriter.write( "# Image rows" + "\n" );
                fileWriter.write( "00280010:" + m_dicomInformationAssociation.getImageHeight( ) + "\n" );
                fileWriter.write( "# Image colums" + "\n" );
                fileWriter.write( "00280011:" + m_dicomInformationAssociation.getImageWidth( ) + "\n" );
                
                fileWriter.write( "# Pixel spacing" + "\n" );
                fileWriter.write( "00280030:" + m_dicomInformationAssociation.getPixelSpacing( ) + "\n" );
                
                fileWriter.write( "# Image modality" + "\n" );
                fileWriter.write( "00080060:" + m_dicomInformationAssociation.getImageModality( ) + "\n" );
                
                fileWriter.write( "# Sequence name" + "\n" );
                fileWriter.write( "00180024:" + m_dicomInformationAssociation.getSequenceName( ) + "\n" );
                
                if ( m_dicomInformationAssociation.getImageModality( ).compareTo("MR") == 0 )
                {
                
                    fileWriter.write( "# Protocol name" + "\n" );
                    fileWriter.write( "00181030:" + m_dicomInformationAssociation.getProtocolName( ) + "\n" );
                
                    fileWriter.write( "# Echo time" + "\n" );
                    fileWriter.write( "00180081:" + m_dicomInformationAssociation.getEchoTime( ) + "\n" );
                
                    fileWriter.write( "# TR" + "\n" );
                    fileWriter.write( "00180080:" + m_dicomInformationAssociation.getTR( ) + "\n" );

                    fileWriter.write( "# Phase encoding direction" + "\n" );
                    fileWriter.write( "00181312:" + m_dicomInformationAssociation.getPhaseEncodingDirection( ) + "\n" );

                    fileWriter.write( "# Flip angle" + "\n" );
                    fileWriter.write( "00181312:" + m_dicomInformationAssociation.getFlipAngle( ) + "\n" );
                
                    fileWriter.write( "# Echo train length" + "\n" );
                    fileWriter.write( "00180091:" + m_dicomInformationAssociation.getEchoTrainLength( ) + "\n" );
                    
                    fileWriter.write( "# Percent sampling" + "\n" );
                    fileWriter.write( "00180093:" + m_dicomInformationAssociation.getPercentSampling( ) + "\n" );
                    
                    fileWriter.write( "# Percent FOV" + "\n" );
                    fileWriter.write( "00180094:" + m_dicomInformationAssociation.getPercentPhaseFOV( ) + "\n" );
                    
                    fileWriter.write( "# Pixel bandwidth" + "\n" );
                    fileWriter.write( "00180095:" + m_dicomInformationAssociation.getPixelBandwidth( ) + "\n" );
                   
                    fileWriter.write( "# Samples per pixel" + "\n" );
                    fileWriter.write( "00280002:" + m_dicomInformationAssociation.getSamplesPerPixel( ) + "\n" );

                    fileWriter.write( "# Transmit coil name" + "\n" );
                    fileWriter.write( "00181251:" + m_dicomInformationAssociation.getTransmitCoilName( ) + "\n" );

                    fileWriter.write( "# Transmit coil name" + "\n" );
                    fileWriter.write( "00181251:" + m_dicomInformationAssociation.getTransmitCoilName( ) + "\n" );

                    fileWriter.write( "# Acquisition type" + "\n" );
                    fileWriter.write( "00180023:" + m_dicomInformationAssociation.getAcquisitionType( ) + "\n" );

                    fileWriter.write( "# Scanning sequence" + "\n" );
                    fileWriter.write( "00180020:" + m_dicomInformationAssociation.getScanningSequence( ) + "\n" );
                  
                    fileWriter.write( "# Sequence variant" + "\n" );
                    fileWriter.write( "00180021:" + m_dicomInformationAssociation.getSequenceVariant( ) + "\n" );
                    
                    fileWriter.write( "# Scan options" + "\n" );
                    fileWriter.write( "00180022:" + m_dicomInformationAssociation.getScanOptions( ) + "\n" );   
                }
            }
            
            fileWriter.close( );

        }
        catch ( IOException e )
        {
            System.out.println( "Error creating jpg2dcm config file : " + m_configFileName );
            return -1;
        }
        return 0;
    }


     /**
       * Creates a automatic configuration file which is used to perform the
       * the jpg -> Dicom conversion; This method creates all necessary tags
       * needed for a valid dcm-file.
     * @param jpgFileName
     * @param seriesNumberToUseForDicomization
     * @param seriesDescription
     * @param seriesInstanceUID
     * @return 
       */
     public int createJpg2DcmConfigFile( String jpgFileName, int seriesNumberToUseForDicomization, String seriesDescription, String seriesInstanceUID )
     {
        if ( jpgFileName == null )
        {
            m_configFileName = m_localImageFileNameBase + ".cfg";
        }
        else
        {
            m_configFileName = jpgFileName.substring( 0, jpgFileName.lastIndexOf( "." ) ) + ".cfg";
        }
        try
        {

            FileWriter fileWriter = new FileWriter( m_configFileName );
            fileWriter.write( "# jpg2dcm Default Configuration for encapsulating JPEG Baseline streams into\n" );
            fileWriter.write( "# DICOM Secondary Capture Image objects\n" );
            fileWriter.write( "# (s. DICOM Part 3, A.32.7 Video Photographic Image IOD)\n\n");
            fileWriter.write( "# Patient Module Attributes \n");
            fileWriter.write( "# Patient's Name\n");
            fileWriter.write( "00100010:" + m_dicomInformationAssociation.getPatientName( ) + "\n" );
            fileWriter.write( "# Patient ID\n");
            fileWriter.write( "00100020:" + m_dicomInformationAssociation.getPatientID( ) + "\n" );
            fileWriter.write( "# Issuer of Patient ID\n" );
            fileWriter.write( "#00100021:" );
            fileWriter.write( "# Patient's Birth Date\n" );
            fileWriter.write( "00100030:" + m_dicomInformationAssociation.getPatientBirthDate( ) + "\n" );
            fileWriter.write( "# Patient's Sex\n" );
            fileWriter.write( "00100040:" + m_dicomInformationAssociation.getPatientSex( ) + "\n\n" );
            fileWriter.write( "# General Study Module Attributes\n" );
            fileWriter.write( "# Study Instance UID\n" );
            fileWriter.write( "#0020000D:" + m_dicomInformationAssociation.getStudyInstanceUID( ) + "\n" );
            fileWriter.write( "# Study Date\n" );
            fileWriter.write( "00080020:" + m_dicomInformationAssociation.getStudyDate( ) + "\n" );
            fileWriter.write( "# Study Time\n" );
            fileWriter.write( "00080030:" + m_dicomInformationAssociation.getStudyTime( ) + "\n" );
            fileWriter.write( "# Referring Physician's Name\n" );
            fileWriter.write( "00080090:" + "DIN-Bildlabor\n" );
            fileWriter.write( "# Study ID\n" );
            fileWriter.write( "00200010:" + m_dicomInformationAssociation.getStudyID( ) + "\n" );
            fileWriter.write( "# Accession Number\n" );
            fileWriter.write( "00080050:" + m_dicomInformationAssociation.getAccessionNumber( ) + "\n" );
            fileWriter.write( "# Study Description\n" );
            fileWriter.write( "00081030:" + m_dicomInformationAssociation.getStudyDescription( ) + "\n" );
            fileWriter.write( "\n" );
            fileWriter.write( "# General Series Module Attributes\n" );
            fileWriter.write( "# Modality\n" );
            fileWriter.write( "00080060:MR\n" ); // for the time being keep this setting: most important is that it does display well
            fileWriter.write( "# Series Instance UID\n" );
            fileWriter.write( "0020000E:" + seriesInstanceUID + "\n" );
            fileWriter.write( "# Series description\n" );
            fileWriter.write( "0008103E:" + seriesDescription + "\n" );

            fileWriter.write( "# Series Number\n" );
            fileWriter.write( "00200011:" + seriesNumberToUseForDicomization + "\n" );
            //fileWriter.write( "00200011:" + seriesNumberToUseForDicomization + 1000 + "\n" );


            fileWriter.write( "# General Equipment Module Attributes" + "\n" );
            fileWriter.write( "# Manufacturer" + "\n" );
            fileWriter.write( "00080070:" + "© DIN Bildlabor 2003-2009" + "\n" );

            fileWriter.write( "# SC Equipment Module Attributes\n" );
            fileWriter.write( "# Conversion Type\n" );
            fileWriter.write( "00080064:SI" );

            fileWriter.write( "# General Image Module Attributes\n" );
            fileWriter.write( "# Instance Number\n" );
            fileWriter.write( "00200013:" + m_currentInstanceNumber + "\n" );

            //System.out.println( ">>>> Instance number =" + m_currentInstanceNumber );

            fileWriter.write( "# Image Pixel Module Attributes (detected from JPEG file, if not specified)" + "\n" );
            fileWriter.write( "# Samples per Pixel\n");
            fileWriter.write( "#00280002:" + m_dicomInformationAssociation.getSamplesPerPixel( ) + "\n" );
            fileWriter.write( "# Photometric Interpretation\n");
            fileWriter.write( "#00280004:" + m_dicomInformationAssociation.getPhotometricInterpretation( ) + "\n" );
            fileWriter.write( "# Planar Configuration\n" );
            fileWriter.write( "#00280006:" + m_dicomInformationAssociation.getPlanarConfiguration() + "\n" );
            fileWriter.write( "# Rows\n" );
            fileWriter.write( "#00280010:" + m_dicomInformationAssociation.getImageHeight( ) + "\n" );
            fileWriter.write( "# Columns\n" );
            fileWriter.write( "#00280011:" + m_dicomInformationAssociation.getImageWidth( ) + "\n" );
            fileWriter.write( "# Bits Allocated\n");
            fileWriter.write( "#00280100:" + m_dicomInformationAssociation.getFileType( ) + "\n" );
            fileWriter.write( "# Bits Stored\n" );
            fileWriter.write( "#00280101:" + m_dicomInformationAssociation.getBitsStored( ) +"\n" );
            fileWriter.write( "# High Bit\n" );
            fileWriter.write( "#00280102:" + m_dicomInformationAssociation.getHighBit( ) + "\n" );
            fileWriter.write( "# Pixel Representation\n" );
            fileWriter.write( "#00280103:" + m_dicomInformationAssociation.getPixelRepresentation( ) + "\n\n" );
            fileWriter.write( "# SOP Common Module Attributes\n" );
            fileWriter.write( "# SOP Class UID\n" );
            fileWriter.write( "00080016:1.2.840.10008.5.1.4.1.1.7\n" ); // secondary capture
            fileWriter.write( "# SOP Instance UID\n");
            fileWriter.write( "#00080018:" + "sop_instance_iud" + "\n" );

            fileWriter.write( "# Software version" + "\n" );

            Globals globs = Globals.getInstance( );

            fileWriter.write( "00181020:" + globs.APP_NAME + " " + globs.APP_VERSION + "\n" );

            // user also wants the geometry information tags to be added
            if ( m_includeGeometryTags == true )
            {

                String imagePosition = m_dicomInformationAssociation.getImagePositionPatient( ).replaceAll( "\\\\", "\\\\\\\\" );
                String imageOrientation = m_dicomInformationAssociation.getImageOrientationPatient( ).replaceAll( "\\\\", "\\\\\\\\" );

                fileWriter.write( "# Slice location" + "\n" );
                fileWriter.write( "00201041:" + m_dicomInformationAssociation.getSliceLocation( ) + "\n" );
                fileWriter.write( "# Image orientation" + "\n" );
                fileWriter.write( "00200037:" + imageOrientation + "\n" );
                fileWriter.write( "# Image position" + "\n" );
                fileWriter.write( "00200032:" + imagePosition + "\n" );
                fileWriter.write( "# Slice thickness" + "\n" );
                fileWriter.write( "00180050:" + m_dicomInformationAssociation.getSliceThickness( ) + "\n" );

                fileWriter.write( "# Image rows" + "\n" );
                fileWriter.write( "00280010:" + m_dicomInformationAssociation.getImageHeight( ) + "\n" );
                fileWriter.write( "# Image colums" + "\n" );
                fileWriter.write( "00280011:" + m_dicomInformationAssociation.getImageWidth( ) + "\n" );

                fileWriter.write( "# Pixel spacing" + "\n" );
                fileWriter.write( "00280030:" + m_dicomInformationAssociation.getPixelSpacing( ) + "\n" );

                fileWriter.write( "# Image modality" + "\n" );
                fileWriter.write( "00080060:" + m_dicomInformationAssociation.getImageModality( ) + "\n" );

                fileWriter.write( "# Sequence name" + "\n" );
                fileWriter.write( "00180024:" + m_dicomInformationAssociation.getSequenceName( ) + "\n" );

                if ( m_dicomInformationAssociation.getImageModality( ).compareTo( "MR" ) == 0 )
                {

                    fileWriter.write( "# Protocol name" + "\n" );
                    fileWriter.write( "00181030:" + m_dicomInformationAssociation.getProtocolName( ) + "\n" );

                    fileWriter.write( "# Echo time" + "\n" );
                    fileWriter.write( "00180081:" + m_dicomInformationAssociation.getEchoTime( ) + "\n" );

                    fileWriter.write( "# TR" + "\n" );
                    fileWriter.write( "00180080:" + m_dicomInformationAssociation.getTR( ) + "\n" );

                    fileWriter.write( "# Phase encoding direction" + "\n" );
                    fileWriter.write( "00181312:" + m_dicomInformationAssociation.getPhaseEncodingDirection( ) + "\n" );

                    fileWriter.write( "# Flip angle" + "\n" );
                    fileWriter.write( "00181312:" + m_dicomInformationAssociation.getFlipAngle( ) + "\n" );

                    fileWriter.write( "# Echo train length" + "\n" );
                    fileWriter.write( "00180091:" + m_dicomInformationAssociation.getEchoTrainLength( ) + "\n" );

                    fileWriter.write( "# Percent sampling" + "\n" );
                    fileWriter.write( "00180093:" + m_dicomInformationAssociation.getPercentSampling( ) + "\n" );

                    fileWriter.write( "# Percent FOV" + "\n" );
                    fileWriter.write( "00180094:" + m_dicomInformationAssociation.getPercentPhaseFOV( ) + "\n" );

                    fileWriter.write( "# Pixel bandwidth" + "\n" );
                    fileWriter.write( "00180095:" + m_dicomInformationAssociation.getPixelBandwidth( ) + "\n" );

                    fileWriter.write( "# Samples per pixel" + "\n" );
                    fileWriter.write( "00280002:" + m_dicomInformationAssociation.getSamplesPerPixel( ) + "\n" );

                    fileWriter.write( "# Transmit coil name" + "\n" );
                    fileWriter.write( "00181251:" + m_dicomInformationAssociation.getTransmitCoilName( ) + "\n" );

                    fileWriter.write( "# Transmit coil name" + "\n" );
                    fileWriter.write( "00181251:" + m_dicomInformationAssociation.getTransmitCoilName( ) + "\n" );

                    fileWriter.write( "# Acquisition type" + "\n" );
                    fileWriter.write( "00180023:" + m_dicomInformationAssociation.getAcquisitionType( ) + "\n" );

                    fileWriter.write( "# Scanning sequence" + "\n" );
                    fileWriter.write( "00180020:" + m_dicomInformationAssociation.getScanningSequence( ) + "\n" );

                    fileWriter.write( "# Sequence variant" + "\n" );
                    fileWriter.write( "00180021:" + m_dicomInformationAssociation.getSequenceVariant( ) + "\n" );

                    fileWriter.write( "# Scan options" + "\n" );
                    fileWriter.write( "00180022:" + m_dicomInformationAssociation.getScanOptions( ) + "\n" );
                }
            }

            fileWriter.close( );

        }
        catch ( IOException e )
        {
            System.out.println( "Error creating jpg2dcm config file : " + m_configFileName );
            return -1;
        }
        return 0;
    }
     
    /**
     * Replaces all blanks by "\  " in order to be interpreted 
     * correctly by the Windows operating system.
     * @param cmd 
     * @return 
     */
    private String generateProperWindowsCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }

     /**
     * Replaces all blanks by "\  " in order to be interpreted 
     * correctly by the Windows operating system
     * @param cmd 
     * @return 
     */
    private String eliminateIllegalFileSystemCharacters( String cmd )
    {
        
        // first get rid ofm blancs
        cmd = cmd.trim( );
        
        cmd = cmd.replace( ' ', '_' );
        cmd = cmd.replace( ',', '_' );
        cmd = cmd.replace( ';', '_' );
        cmd = cmd.replace( '%', '_' );
        //cmd.replace( '*', '_' );
        
        cmd = cmd.replace( 'é', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'è', 'e' ); // problematic on some linux systems
        cmd = cmd.replace( 'ö', 'o' ); // problematic on some linux systems
        cmd = cmd.replace( 'ü', 'u' ); // problematic on some linux systems
        cmd = cmd.replace( 'à', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'ä', 'a' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ö', 'O' ); // problematic on some linux systems
        cmd = cmd.replace( 'Ü', 'U' ); // problematic on some linux systems
        
        cmd = cmd.replace( '@', '_' ); // problematic on some linux systems
        cmd = cmd.replace( '#', '_' ); // problematic on some linux systems
        
        cmd = cmd.replace( '?', '_' );  // problematic on some linux systems
        
        cmd = cmd.replace( '¢', 'c' ); // problematic on some linux systems
        cmd = cmd.replace( '!', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '§', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '¦', '_' );  // problematic on some linux systems
        
        cmd = cmd.replace( '$', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '[', '_' );  // problematic on some linux systems
        cmd = cmd.replace( ']', '_' );  // problematic on some linux systems
        
        cmd = cmd.replace( '^', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems
        
        cmd = cmd.replace( 'ç', 'c' );  // problematic on some linux systems
        cmd = cmd.replace( '~', '_' );  // problematic on some linux systems
        cmd = cmd.replace( '"', '_' );  // problematic on some linux systems
        
        cmd = cmd.replace("____", "_" );
        cmd = cmd.replace("___", "_" );
        cmd = cmd.replace("__", "_" );
        
        return cmd;
    }
    
    
    /**
     * Starts the fit that corresponds to the current status of the model
     * and the loaded measured spectrum
     *
     * @param jpgFilename
     * @param configFileName
     * @param dicomFileName
     */
     /**
     * Performs the action conversion.
     */
//    private boolean performJPGTODicomConversion( )
//    {
//
//        String workingDir = System.getProperty( "user.dir" );
//
//        int returnValue;
//        String cmd = "";
//
//        //DicomSliceCanvasPolyDraw draw = getCurrentSliceCanvasPolyDraw( );
//        String patInfo = m_dicomInformationAssociation.getPatientName( );
//        patInfo = patInfo + " (DOB:" + m_dicomInformationAssociation.getPatientBirthDate( ) + ")";
//        String accessionNumber = "Accession number : " + m_dicomInformationAssociation.getAccessionNumber( );
//        String studyDate = "Date " + m_dicomInformationAssociation.getStudyDate( );
//
//        m_dicomFileName = m_localImageFileNameBase + "_stamped.dcm";
//
//        BufferedImage img = null;
//        try
//        {
//            img = ImageIO.read( new File( m_localImageFileNameBase + ".jpg" ) );
//
//            Graphics g = img.getGraphics( );
//
//            g.setColor( Color.BLUE );
//            g.drawString( patInfo + " / " + studyDate + " / " + accessionNumber, 1, img.getHeight( ) - 25 );
//            g.drawString( "Secondary Capture by " + System.getProperty( "user.name" ) + " - For scientific purposes only - DIN-Bildlabor - Inselspital Bern", 1, img.getHeight( ) - 10 );
//
//            ImageIO.write( img, "jpg", new File( ( m_localImageFileNameBase + "_stamped.jpg" ) ) );
//
//        } catch ( IOException e )
//        {
//            // nop
//            System.out.printf( "IOException occurred \n" );
//            e.printStackTrace( );
//            return false;
//        }
//
//        
//        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "linux" ) == true )
//        {
//            cmd = workingDir + "/lib/jpg2dcm -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomFileName + "\n";
//        }
//        
//        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "windows" ) == true )
//        {
//            cmd = workingDir + "\\lib\\jpg2dcm.bat -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomFileName + "\n";
//            cmd = generateProperWindowsCommand( cmd );
//        }
//        
//        //cmd = workingDir + "/lib/jpg2dcm.bat -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomFileName + "\n";
//
//        System.out.println( "Processing following system command : " + cmd );
//
//        try
//        {
//            Process p = Runtime.getRuntime( ).exec( cmd );
//
//            InputStream in = p.getInputStream( );
//            int c;
//            while ( ( c = in.read( ) ) != -1 )
//            {
//                System.out.print( (char)c ) ;
//            }
//
//            returnValue = p.waitFor( );
//            in.close( );
//        }
//        catch ( IOException ioX )
//        {
//            // not handled yet
//            System.out.println( "IOException occurred" );
//            ioX.printStackTrace( );
//            return false;
//        }
//        catch ( InterruptedException intX )
//        {
//            // not handled yet
//            System.out.println( "InterruptedException occurred" );
//            intX.printStackTrace( );
//            return false;
//        }
//
//        return true;
//        
//    }

 
    /**
     * Performs the conversion from jpg to Dicom.
     */
    private boolean performJPGTODicomConversion( String jpgFileReference )
    {

        String workingDir = System.getProperty( "user.dir" );

        int returnValue;
        String cmd = "";

        //DicomSliceCanvasPolyDraw draw = getCurrentSliceCanvasPolyDraw( );
        String patInfo = m_dicomInformationAssociation.getPatientName( );
        patInfo = patInfo + " (DOB:" + m_dicomInformationAssociation.getPatientBirthDate( ) + ")";
        String accessionNumber = "Accession number : " + m_dicomInformationAssociation.getAccessionNumber( );
        String studyDate = "Date " + m_dicomInformationAssociation.getStudyDate( );

        m_dicomFileName = m_localImageFileNameBase + "_stamped.dcm";

        BufferedImage img = null;
        try
        {
            if ( jpgFileReference == null )
            {
                img = ImageIO.read( new File( m_localImageFileNameBase + ".jpg" ) );
            }
            else
            {
                img = ImageIO.read( new File( jpgFileReference ) );
            }
            Graphics g = img.getGraphics( );

            g.setColor( Color.BLUE );
            g.drawString( patInfo + " / " + studyDate + " / " + accessionNumber, 1, img.getHeight( ) - 25 );
            g.drawString( "Secondary Capture by " + System.getProperty( "user.name" ) + " - For scientific purposes only - DIN-Bildlabor - Inselspital Bern", 1, img.getHeight( ) - 10 );

            ImageIO.write( img, "jpg", new File( ( m_localImageFileNameBase + "_stamped.jpg" ) ) );

        } catch ( IOException e )
        {
            // nop
            System.out.printf( "IOException occurred \n" );
            return false;
        }

        
        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "linux" ) == true )
        {
            cmd = workingDir + "/lib/jpg2dcm -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomFileName + "\n";
        }
        
        if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "windows" ) == true )
        {
            cmd = workingDir + "\\lib\\jpg2dcm.bat -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomFileName + "\n";
            cmd = generateProperWindowsCommand( cmd );
        }
        
        //cmd = workingDir + "/lib/jpg2dcm.bat -c " + m_configFileName + " --uid-prefix " + " 1.3.12.2.1107.5.2 " + m_localImageFileNameBase + "_stamped.jpg" + " " + m_dicomFileName + "\n";

        System.out.println( "Processing following system command : " + cmd );

        try
        {
            Process p = Runtime.getRuntime( ).exec( cmd );

            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 )
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
        }
        catch ( IOException ioX )
        {
            // not handled yet
            System.out.println( "IOException occurred" );
            return false;
        }
        catch ( InterruptedException intX )
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
            return false;
        }

        return true;
    }
    
    
    /**
     * File reference of stamped Dicom file.
     * @return
     */
    public String getStampedDicomFileReference( )
    {
        return m_dicomFileName;
    }




    // Attributes
    private JComponent  m_component;
    private String      m_imageDescription;
    private File        m_localFile;
    private String      m_localImageFileNameBase;
    private FileInfo    m_dicomInformationAssociation;
    private String      m_configFileName;
    private String      m_dicomFileName;
    private int         m_currentInstanceNumber;
    private String      m_workingDir;
    private boolean     m_includeGeometryTags;
}