/*
 * File: $Id: DicomNetworkNode.java,v 1.6 2009/15/05 13:28:39 hans Exp $
 * History
 * Date: 15.05.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */



package org.spectrim.dicom;

/**
 *
 * @author slotboom
 */
public class DicomNetworkNode
{
    
    /**
     * Default constructor
     */
    public DicomNetworkNode( )
    {

    }

    /**
     * Constructor of DicomNetworkNode
     * @param receiveAEtitle
     * @param ipaddress
     * @param portNumber
     */
    public DicomNetworkNode( String nodeName, String receiveAEtitle, String ipaddress, int receivePortNumber )
    {
        m_nodeName = nodeName;
        m_receiveAEtitle = receiveAEtitle;

        //define default for m_transmitAEtitle
        m_transmitAEtitle = "NOT DEFINED";

        m_ipaddress = ipaddress;
        m_dicomReceivePortNumber = receivePortNumber;

        //define default for m_dicomTransmitPortNumber
        m_dicomTransmitPortNumber = -1;
    }


    /**
     * 
     * Dicom node definition: for local host construction: two AETitles can be
     * defined and two portnumbers on which dicom listining and sending takes 
     * place 
     * @param nodeName
     * @param receiveAEtitle
     * @param transmitAEtitle
     * @param ipaddress
     * @param receivePortNumber
     * @param transmitPortNumber
     */
    public DicomNetworkNode( String nodeName, String receiveAEtitle, String transmitAEtitle, 
                             String ipaddress, int receivePortNumber, int transmitPortNumber )
    {
        m_nodeName = nodeName;
        m_receiveAEtitle = receiveAEtitle;
        m_transmitAEtitle = transmitAEtitle;
        m_ipaddress = ipaddress;
        m_dicomReceivePortNumber = receivePortNumber;
        m_dicomTransmitPortNumber = transmitPortNumber;
    }


    /**
     * Sets the node name
     * @param portNumber
     */
    public void setNodeName( String nodeName )
    {
        m_nodeName = nodeName;
    }

    /**
     * Sets the port number on which the dicom node listens
     * @param portNumber
     */
    public void setDicomReceivePortNumber( int portNumber )
    {
        m_dicomReceivePortNumber = portNumber;
    }

    /**
     * Sets the transmit port number
     * @param portNumber
     */
    public void setDicomTransmitPortNumber( int portNumber )
    {
        m_dicomTransmitPortNumber = portNumber;
    }


    /**
     * Sets the ip-address of the Dicom network node
     * @param portNumber
     */
    public void setIPaddressNr( String ipaddress )
    {
        m_ipaddress = ipaddress;
    }
    
    /**
     * Sets the dicom aetitle
     * @param portNumber
     */
    public void setReceiveAEtitle( String aetitle )
    {
        m_receiveAEtitle = aetitle;
    }

    /**
     * Sets the dicom aetitle
     * @param portNumber
     */
    public void setTransmitAEtitle( String aetitle )
    {
        m_transmitAEtitle = aetitle;
    }

    /**
     * Returns the port number on which the dicom node listens
     * @return portNumber
     */
    public int getDicomReceivePortNumber( )
    {
        return m_dicomReceivePortNumber;
    }

     /**
     * Returns the transmit port number
     * @return portNumber
     */
    public int getDicomTransmitPortNumber( )
    {
        return m_dicomTransmitPortNumber;
    }


     /**
     * Returns the node name
     * @return portNumber
     */
    public String getNodeName( )
    {
        return m_nodeName;
    }


    /**
     * Returns the ip address
     * @return ip-address of node
     */
    public String getIPaddressr( )
    {
        return m_ipaddress;
    }
    
    /**
     * Returns the aetitle
     * @return m_aetitle
     */
    public String getReceiveAEtitle( )
    {
        return m_receiveAEtitle;
    }
    
    /**
     * Returns the aetitle
     * @return m_aetitle
     */
    public String getTransmitAEtitle( )
    {
        return m_transmitAEtitle;
    }



    // Atrributes
    private String m_nodeName;
    private String m_receiveAEtitle;
    private String m_transmitAEtitle;

    private String m_ipaddress;

    private int m_dicomReceivePortNumber;
    private int m_dicomTransmitPortNumber;
}
