 /*
 * DicomSiemensSpecificFileHeaderViewer.java
 *
 * Created on February 11, 2004, 10:12 AM
 */

package org.spectrim.dicom;

import java.awt.Point;
import java.awt.BorderLayout;

import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Iterator;
import java.util.LinkedHashMap;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;


/**
 * 
 * Utility class for viewing the content of a Siemens specific Dicom tags.
 * 
 * @author  hans
 */
public class DicomSiemensSpecificFileHeaderViewer extends JFrame implements ActionListener 
{
    
    /** 
     * Creates a new instance of DicomSiemensSpecificFileHeaderViewer. 
     * @param parent
     * @param siemensInfo
     */
    public DicomSiemensSpecificFileHeaderViewer( JFrame parent, 
                                                 LinkedHashMap siemensInfo ) 
    {
        
        m_displayPanel = new JPanel( );
        
        m_displayPanel.setLayout( new BorderLayout( ) );
               
        m_attributeTable = new JTable( );
                
        m_attributeTableModel = new DefaultTableModel( );
        
        m_attributeTableModel.addColumn( "<HTML><B>Siemens attribute</B></HTML>" );
        m_attributeTableModel.addColumn( "<HTML><B>Value</B></HTML>" );

        m_attributeTable.setModel( m_attributeTableModel );
        
        JTableHeader tabHeader = m_attributeTable.getTableHeader( );
               
        createTableInfo( siemensInfo );
        
        m_attributeTable.updateUI( );
        
        m_displayPanel.add( tabHeader, BorderLayout.NORTH );
        
        m_displayPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_mainScrollPane = new JScrollPane( m_attributeTable,
                                            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                            JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        // define the button panel
        JPanel m_buttonPanel = new JPanel( );
        m_okButton = new JButton( "OK" );
        m_okButton.addActionListener( this );
        m_buttonPanel.add( m_okButton );
        
        m_collectionPanel = new JPanel( );
        m_collectionPanel.setLayout( new BoxLayout( m_collectionPanel, BoxLayout.Y_AXIS ) );
        // add the two subpanels to the collection panel
        m_collectionPanel.add( m_mainScrollPane );
        m_collectionPanel.add( m_buttonPanel );
        
        m_collectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        setContentPane( m_collectionPanel );
        this.setPreferredSize( new Dimension( 900, 550 ) );
        this.setTitle( "Siemens Specific Dicom Header Information" ); 
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = parent.getLocation( );
        Point centerParent = new Point( parent.getX( ) + parent.getWidth( ) / 2,
                                        parent.getY( ) + parent.getHeight( ) / 2 );


        pack( );
        setVisible( true );
    }
    
    /**
     * Handles action events eg pressing a button etc.
     * @param actionEvent
     */
    @Override
    public void actionPerformed( ActionEvent actionEvent ) 
    {
       Object sender = actionEvent.getSource( );
        
	if ( sender == m_okButton ) 
        {
            setVisible( false );
            this.dispose( );
        }
    }
    
    /**
     * Creates a table of the data contained in 
     * the Siemens header information hash table.
     * @param ht
     */
    public void createTableInfo( LinkedHashMap ht )
    {
        String name;
        Iterator it = ht.keySet( ).iterator( );
        while ( it.hasNext( ) )
        {
            String key = (String)it.next( );
            Object[] row = new Object[]{ key, (String)ht.get(key) };
            m_attributeTableModel.addRow( row );
        }
    }

    // Attributes for the displayer
    JPanel m_collectionPanel;
    JPanel m_displayPanel;
    JScrollPane m_mainScrollPane;
    
    JPanel m_buttonPanel;
    JButton m_okButton;
    
    JTable m_attributeTable;
    DefaultTableModel m_attributeTableModel;
    
    // For proper serialization
    private static final long serialVersionUID = 1481295898889393L;
}
