/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.geometricobjects;

import org.spectrim.imagegraphicobjects.Point3D;
import java.util.ArrayList;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.LUDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.commons.math3.linear.SingularMatrixException;

/**
 *
 * @author nunopbarros
 */
public class Cube3D 
{

    public Point3D p1;
    public Point3D p2;
    public Point3D p3;
    public Point3D p4;
    public Point3D p5;
    public Point3D p6;
    public Point3D p7;
    public Point3D p8;

    public Cube3D( Point3D p1, Point3D p2, Point3D p3, Point3D p4, 
                   Point3D p5, Point3D p6, Point3D p7, Point3D p8 ) 
    {
        this.p1 = p1;
        this.p2 = p2;
        this.p3 = p3;
        this.p4 = p4;
        this.p5 = p5;
        this.p6 = p6;
        this.p7 = p7;
        this.p8 = p8;
    }
    
    public double[] getDimensions( )
    {
        double dim1 = p6.subtractPoint(p5).length( );
        double dim2 = p1.subtractPoint(p5).length( );
        double dim3 = p8.subtractPoint(p5).length( );
        return new double[] { dim1,dim2,dim3 };
    }

    public Point3D getCenter( ) 
    {
        Point3D p = new Point3D( );
        p.m_x1 = ( p1.m_x1 + p2.m_x1 + p3.m_x1 + p4.m_x1 + p5.m_x1 + p6.m_x1 + p7.m_x1 + p8.m_x1 ) / 8;
        p.m_x2 = ( p1.m_x2 + p2.m_x2 + p3.m_x2 + p4.m_x2 + p5.m_x2 + p6.m_x2 + p7.m_x2 + p8.m_x2 ) / 8;
        p.m_x3 = ( p1.m_x3 + p2.m_x3 + p3.m_x3 + p4.m_x3 + p5.m_x3 + p6.m_x3 + p7.m_x3 + p8.m_x3 ) / 8;
        return p;
    }

    public boolean isInside( Cube3D specvox, double marg ) 
    {
        boolean ans = true;
        Point3D v1 = p6.subtractPoint(p5);
        Point3D v2 = p1.subtractPoint(p5);
        Point3D v3 = p8.subtractPoint(p5);
        ArrayList<RealVector> constants = new ArrayList<>();

        double[][] matrixData = 
        {
            { v1.m_x1, -v2.m_x1, v3.m_x1 },
            { v1.m_x2, -v2.m_x2, v3.m_x2 },
            { v1.m_x3, -v2.m_x3, v3.m_x3 } };
        RealMatrix n = new Array2DRowRealMatrix( matrixData );

        constants.add( new ArrayRealVector(new double[]{specvox.p1.m_x1 - p5.m_x1, specvox.p1.m_x3 - p5.m_x2, specvox.p1.m_x3 - p5.m_x3}, false ) );
        constants.add( new ArrayRealVector(new double[]{specvox.p2.m_x1 - p5.m_x1, specvox.p2.m_x3 - p5.m_x2, specvox.p2.m_x3 - p5.m_x3}, false ) );
        constants.add( new ArrayRealVector(new double[]{specvox.p3.m_x1 - p5.m_x1, specvox.p3.m_x3 - p5.m_x2, specvox.p3.m_x3 - p5.m_x3}, false ) );
        constants.add( new ArrayRealVector(new double[]{specvox.p4.m_x1 - p5.m_x1, specvox.p4.m_x3 - p5.m_x2, specvox.p4.m_x3 - p5.m_x3}, false ) );
        constants.add( new ArrayRealVector(new double[]{specvox.p5.m_x1 - p5.m_x1, specvox.p5.m_x3 - p5.m_x2, specvox.p5.m_x3 - p5.m_x3}, false ) );
        constants.add( new ArrayRealVector(new double[]{specvox.p6.m_x1 - p5.m_x1, specvox.p6.m_x3 - p5.m_x2, specvox.p6.m_x3 - p5.m_x3}, false ) );
        constants.add( new ArrayRealVector(new double[]{specvox.p7.m_x1 - p5.m_x1, specvox.p7.m_x3 - p5.m_x2, specvox.p7.m_x3 - p5.m_x3}, false ) );
        constants.add( new ArrayRealVector(new double[]{specvox.p8.m_x1 - p5.m_x1, specvox.p8.m_x3 - p5.m_x2, specvox.p8.m_x3 - p5.m_x3}, false ) );
        DecompositionSolver solver = new LUDecomposition( n ).getSolver( );

        for ( int i = 0; i < constants.size( ); i++ ) 
        {
            try 
            {
                RealVector solution = solver.solve(constants.get(i));
                double[] sol;
                sol = solution.toArray();
                if ( sol[0] < ( marg) || sol[0] > ( 1d - marg )
                  || sol[1] < ( marg) || sol[1] > ( 1d - marg )
                  || sol[2] <  (marg) || sol[2] > ( 1d - marg ) ) 
                {
                    ans = false;
                    i = constants.size( );
                }
            } 
            catch ( Exception ex ) 
            {
                ex.printStackTrace( );
            }
        }
        return ans;
    }

    public boolean isCenterInside( Cube3D specvox ) 
    {
        boolean ans = true;
        Point3D v1 = p6.subtractPoint( p5 );
        Point3D v2 = p1.subtractPoint( p5 );
        Point3D v3 = p8.subtractPoint( p5 );

        double[][] matrixData = 
        {
            { v1.m_x1, v2.m_x1, v3.m_x1 },
            { v1.m_x2, v2.m_x2, v3.m_x2 },
            { v1.m_x3, v2.m_x3, v3.m_x3 } 
        };
        RealMatrix n = new Array2DRowRealMatrix( matrixData );
        Point3D centervox = specvox.getCenter( );
        RealVector constant = new ArrayRealVector( new double[]{ centervox.m_x1 - p5.m_x1, centervox.m_x2 - p5.m_x2, centervox.m_x3 - p5.m_x3}, false );

        DecompositionSolver solver = new LUDecomposition( n ).getSolver( );

        try 
        {
            RealVector solution = solver.solve(constant);
            double[] sol;
            sol = solution.toArray();
            if ( sol[0] < 0d || sol[0] > 1d
                    || sol[1] < 0d || sol[1] > 1d
                    || sol[2] < 0d || sol[2] > 1d) 
            {
                ans = false;
            }
        } 
        catch ( SingularMatrixException ex ) 
        {
            //ex.printStackTrace( );
            ans = false;
        }

        return ans;
    }

    public Cube3D add(Cube3D cube2) 
    {
        Point3D r1 = this.p1.addPoint( cube2.p1 );
        Point3D r2 = this.p2.addPoint( cube2.p2 );
        Point3D r3 = this.p3.addPoint( cube2.p3 );
        Point3D r4 = this.p4.addPoint( cube2.p4 );
        Point3D r5 = this.p5.addPoint( cube2.p5 );
        Point3D r6 = this.p6.addPoint( cube2.p6 );
        Point3D r7 = this.p7.addPoint( cube2.p7 );
        Point3D r8 = this.p8.addPoint( cube2.p8 );
        return new Cube3D( r1, r2, r3, r4, r5, r6, r7, r8 );
    }

    public ArrayList<Line> getEdges( ) 
    {
        
        ArrayList<Line> res = new ArrayList<>( );
        res.add( new Line( p1, p2 ) );
        res.add( new Line( p4, p3 ) );
        res.add( new Line( p5, p6 ) );
        res.add( new Line( p8, p7 ) );
        res.add( new Line( p5, p1 ) );
        res.add( new Line( p6, p2 ) );
        res.add( new Line( p8, p4 ) );
        res.add( new Line( p7, p3 ) );
        res.add( new Line( p1, p4 ) );
        res.add( new Line( p2, p3 ) );
        res.add( new Line( p5, p8 ) );
        res.add( new Line( p6, p7 ) );
        
        return res;
    }

    public boolean hasNullPoint( ) 
    {
        return ( p1 == null
                || p2 == null
                || p3 == null
                || p4 == null
                || p5 == null
                || p6 == null
                || p7 == null
                || p8 == null );
    }
}
