/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.grid;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JColorChooser;
import javax.swing.JFileChooser;
import javax.swing.JSlider;
import javax.swing.KeyStroke;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.spectrim.model.GlobalModel;
import org.spectrim.settings.SettingsGUI;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import org.spectrim.tdfdfit.TextFileWriter;

/**
 * This GUI enables the user to set the thresholds for images segmentation
 * to find out the fraction of GM, WM and CSF pixels in the spectroscopic 
 * voxel.
 * 
 * Currently only valid for single voxel spectroscopy.
 * 
 * @author johan
 */

public class ImageSegmentationWithinSelectedSpectroscopicRegionFrame extends javax.swing.JFrame 
{    
    
    // Attributes of the class
    private int m_minImagePixValue;
    private int m_maxImagePixValue;
    private int m_minExcitedVolumePixValue;
    private int m_maxExcitedVolumePixValue;
    
    private Color m_gmOverlayColor = new Color( 1f, 0f, 0f, 0.1f );
    private Color m_wmOverlayColor = new Color( 0f, 1f, 0f, 0.1f );
    private Color m_csfOverlayColor = new Color( 0f, 0f, 1f, 0.1f );
    
    private GlobalModel m_globalModel;
    
    private ChangeListener  m_changeListener;
    private KeyListener     m_keyListener;
    private ItemListener    m_itemListener;
    // private String          m_tissueOrderingString;
    private int[]           m_orderedIntensityValues = new int[6];
    private boolean         m_tissueOrderChanging = false;
    
    
    public final static String GM_ST_WM_ST_CSF = "GM < WM < CSF";
    public final static String GM_ST_CSF_ST_WM = "GM < CSF < WM";
    
    public final static String WM_ST_GM_ST_CSF = "WM < GM < CSF";
    public final static String WM_ST_CSF_ST_GM = "WM < CSF < GM";
    
    public final static String CSF_ST_WM_ST_GM = "CSF < WM < GM";
    public final static String CSF_ST_GM_ST_WM = "CSF < GM < WM";
    
    private String m_selectedTissueOrdering = GM_ST_WM_ST_CSF;
    
    float m_physPixelWidth = 0f;
    float m_physPixelHeight = 0f;
    float m_physPixelDepth = 0f;
    float m_imageGap = 0f;
    /**
     * Creates new form ImageSegmentationWithinSelectedSpectroscopicRegionFrame.
     * @param model
     */
    public ImageSegmentationWithinSelectedSpectroscopicRegionFrame( GlobalModel model ) 
    {
        
        int[] dims = model.getImageStack( ).getDimensions( );
            
        m_physPixelWidth = model.getImageStack( ).get( 0 ).pixelSpacingX;
        m_physPixelHeight = model.getImageStack( ).get( 0 ).pixelSpacingY;
        m_physPixelDepth = model.getImageStack( ).get( 0 ).sliceThickness;
        
        m_imageGap = (float)model.getImageStack( ).getSpacingBetweenSlices( );
        
        initComponents( );
        m_globalModel = model;
        
        
        m_tissueTypePixelIntensityOrderingComboBox.setSelectedIndex( 1 );
        
        m_selectedTissueOrdering = "";
        
        setImagePixelIntensityRange( );
        getAndSetTDFDFitDefaultValues( );
        
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String defaultCSFMaxPixValue = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MAXIMAL_PIXEL_VALUE );
        if ( defaultCSFMaxPixValue != null )
        {
            m_maximalCSFPixValueSlider.setValue( new Integer( defaultCSFMaxPixValue ) ); 
        }
        
        String defaultGMMaxPixValue =  defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MAXIMAL_PIXEL_VALUE );
        if ( defaultGMMaxPixValue != null )
        {
            m_maximalGMPixValueSlider.setValue( new Integer( defaultGMMaxPixValue ) ); 
        }
        
        String defaultWMMaxPixValue =  defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MAXIMAL_PIXEL_VALUE );    
        if ( defaultWMMaxPixValue != null )
        {
            m_maximalWMPixValueSlider.setValue( new Integer( defaultWMMaxPixValue ) ); 
        }
        
        String defaultCSFMinPixValue = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MINIMAL_PIXEL_VALUE );
        if ( defaultCSFMinPixValue != null )
        {
            m_minimalCSFPixValueSlider.setValue( new Integer( defaultCSFMinPixValue ) ); 
        }
        
        String defaultGMMinPixValue =  defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MINIMAL_PIXEL_VALUE );
        if ( defaultGMMinPixValue != null )
        {
            m_minimalGMPixValueSlider.setValue( new Integer( defaultGMMinPixValue ) ); 
        }
        
        String defaultWMMinPixValue =  defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MINIMAL_PIXEL_VALUE );
        if ( defaultWMMinPixValue != null )
        {
            m_minimalWMPixValueSlider.setValue( new Integer( defaultWMMinPixValue ) ); 
        }
        
        int transparency = -1;
        String defaultTransparency = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_DISPLAY_TRANSPARENCY );
        if ( defaultTransparency != null )
        {
            transparency = new Integer( defaultTransparency );
            m_transparancyOverlayPixValueSlider.setValue( new Integer( defaultTransparency ) );
            m_transparencyValueFormattedTextField.setValue( new Integer( defaultTransparency ) );
        }
        
        int gmRedInt = -1;
        int gmBlueInt = -1;
        int gmGreenInt = -1;
        
        
        String gmRedIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_RED_COMPONENT );
        if ( gmRedIntStr != null )
        {
            gmRedInt = new Integer( gmRedIntStr );
        }
        
        String gmBlueIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_BLUE_COMPONENT );
        if ( gmBlueIntStr != null )
        {
            gmBlueInt = new Integer( gmBlueIntStr );
        }
        
        String gmGreenIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_GREEN_COMPONENT );
        if ( gmGreenIntStr != null )
        {
            gmGreenInt = new Integer( gmGreenIntStr );
        }   
        
        int wmRedInt = -1;
        int wmBlueInt = -1;
        int wmGreenInt = -1;
        
        
        String wmRedIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_RED_COMPONENT );
        if ( wmRedIntStr != null )
        {
            wmRedInt = new Integer( wmRedIntStr );
        }
        
        String wmBlueIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_BLUE_COMPONENT );
        if ( wmBlueIntStr != null )
        {
            wmBlueInt = new Integer( wmBlueIntStr );
        }
        
        String wmGreenIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_GREEN_COMPONENT );
        if ( wmGreenIntStr != null )
        {
            wmGreenInt = new Integer( wmGreenIntStr );
        }
        
        int csfRedInt = -1;
        int csfBlueInt = -1;
        int csfGreenInt = -1;
        
        
        String csfRedIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_RED_COMPONENT );
        if ( csfRedIntStr != null )
        {
            csfRedInt = new Integer( csfRedIntStr );
        }
        
        String csfBlueIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_BLUE_COMPONENT );
        if ( csfBlueIntStr != null )
        {
            csfBlueInt = new Integer( csfBlueIntStr );
        }
        
        String csfGreenIntStr = defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_GREEN_COMPONENT );
        if ( csfGreenIntStr != null )
        {
            csfGreenInt = new Integer( csfGreenIntStr );
        }
        
        if ( ( gmRedInt != -1) && ( gmBlueInt != -1 ) && 
             ( gmGreenInt != -1 ) &&  ( transparency != -1 ) )
        {
            m_gmOverlayColor = new Color( gmRedInt, gmGreenInt, gmBlueInt, transparency );
            Color buttonColor = new Color( gmRedInt, gmGreenInt, gmBlueInt );
            m_GMColorOverlaySelectionButton.setForeground( buttonColor );
        }
        
        if ( ( wmRedInt != -1 ) && ( wmBlueInt != -1 ) && 
             ( wmGreenInt != -1 ) &&  ( transparency != -1) )
        {
            m_wmOverlayColor = new Color( wmRedInt, wmGreenInt, wmBlueInt, transparency );
            Color buttonColor = new Color( wmRedInt, wmGreenInt, wmBlueInt );
            m_WMColorOverlaySelectionButton.setForeground( buttonColor );
        }
        
        if ( ( csfRedInt != -1 ) && ( csfBlueInt != -1 ) && 
             ( csfGreenInt != -1 ) &&  ( transparency != -1 ) )
        {
            m_csfOverlayColor = new Color( csfRedInt, csfGreenInt, csfBlueInt, transparency );
            Color buttonColor = new Color( csfRedInt, csfGreenInt, csfBlueInt );
            
            m_CSFColorOverlaySelectionButton.setForeground( buttonColor );
        }
        
        // Assumed are default colors; not saved in defaults file!
        m_globalModel.getCSIGrid( ).setSegmentationGMOverlayColor( m_gmOverlayColor );
        m_globalModel.getCSIGrid( ).setSegmentationWMOverlayColor( m_wmOverlayColor );
        m_globalModel.getCSIGrid( ).setSegmentationCSFOverlayColor( m_csfOverlayColor );
        
        
        String defaultTissueOrdering = defs.getProperty( TDFDFittingDefaults.TISSUE_TYPE_ORDER );
        
        if ( defaultTissueOrdering != null )
        {
            m_selectedTissueOrdering = defaultTissueOrdering;
            m_tissueTypePixelIntensityOrderingComboBox.getModel( ).setSelectedItem( m_selectedTissueOrdering );
        }
        
        
        if ( m_selectedTissueOrdering.compareTo( "GM < WM < CSF" ) == 0 )
        {
            m_orderedIntensityValues[0] = m_minimalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[1] = m_maximalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[2] = m_minimalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[3] = m_maximalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[4] = m_minimalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[5] = m_maximalCSFPixValueSlider.getValue( );           
        }
        
        if ( m_selectedTissueOrdering.compareTo( "GM < CSF < WM" ) == 0 )
        {
            m_orderedIntensityValues[0] = m_minimalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[1] = m_maximalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[2] = m_minimalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[3] = m_maximalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[4] = m_minimalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[5] = m_maximalWMPixValueSlider.getValue( );
        }
        
        if ( m_selectedTissueOrdering.compareTo( "WM < GM < CSF" ) == 0 )
        {
            m_orderedIntensityValues[0] = m_minimalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[1] = m_maximalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[2] = m_minimalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[3] = m_maximalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[4] = m_minimalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[5] = m_maximalCSFPixValueSlider.getValue( );
        }
        
        if ( m_selectedTissueOrdering.compareTo( "WM < CSF < GM" ) == 0 )
        {
            m_orderedIntensityValues[0] = m_minimalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[1] = m_maximalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[2] = m_minimalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[3] = m_maximalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[4] = m_minimalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[5] = m_maximalGMPixValueSlider.getValue( );
        }
        
        if ( m_selectedTissueOrdering.compareTo( "CSF < GM < WM" ) == 0 )
        {
            m_orderedIntensityValues[0] = m_minimalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[1] = m_maximalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[2] = m_minimalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[3] = m_maximalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[4] = m_minimalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[5] = m_maximalWMPixValueSlider.getValue( );
        }
        
        if ( m_selectedTissueOrdering.compareTo( "CSF < WM < GM" ) == 0 )
        {
            m_orderedIntensityValues[0] = m_minimalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[1] = m_maximalCSFPixValueSlider.getValue( );
            m_orderedIntensityValues[2] = m_minimalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[3] = m_maximalWMPixValueSlider.getValue( );
            m_orderedIntensityValues[4] = m_minimalGMPixValueSlider.getValue( );
            m_orderedIntensityValues[5] = m_maximalGMPixValueSlider.getValue( );
        }
        
        getRootPane( ).setDefaultButton( m_acceptAndQuitButton );
                  
        Action acceptAndExit = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e ) 
            {
                acceptAndQuit( );
            }
        };

        Action exitNoAccept = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e ) 
            {
               dispose( );
            }
        };

        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( "ENTER" ),
                                           "acceptAndExit");
        
        getRootPane( ).getActionMap( ).put( "acceptAndExit",
                                             acceptAndExit  );

        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( "ESCAPE" ),
                                           "Exit without accepting changes" );
        getRootPane( ).getActionMap( ).put( "Exit without accepting changes",
                                             exitNoAccept );
       
        pack( );
        setVisible( true );
        updateGUI( );
        
    }

    /**
     * Sets the minimum and maximum image pixel intensity. 
     */
    public void setImagePixelIntensityRange( )
    {
        if ( ( m_globalModel != null ) && (  m_globalModel.getImageStack( ) != null ) )
        {
            m_maxImagePixValue = m_globalModel.getImageStack( ).getMaximumPixelIntensityOfStackAsInt( );
            m_minImagePixValue = m_globalModel.getImageStack( ).getMinimumPixelIntensityOfStackAsInt( );
            m_maxExcitedVolumePixValue = m_globalModel.getCSIGrid( ).getMaximumPixelIntensityOfExcitedVolumeAsInt( );
            m_minExcitedVolumePixValue = m_globalModel.getCSIGrid( ).getMinimumPixelIntensityOfExcitedVolumeAsInt( );
        }
        
        m_minPixValLabel.setText( "" + m_minImagePixValue );
        m_maxPixValLabel.setText( "" + m_maxImagePixValue );
        
        m_maxPixValExcitedVolumeLabel.setText( "" + m_maxExcitedVolumePixValue );
        m_minPixValExcitedVolumeLabel.setText( "" + m_minExcitedVolumePixValue );
        
        
        m_maximalCSFPixValueSlider.setMaximum( m_maxExcitedVolumePixValue );
        m_maximalCSFPixValueSlider.setMinimum( m_minExcitedVolumePixValue );
        
        m_maximalGMPixValueSlider.setMaximum( m_maxExcitedVolumePixValue );
        m_maximalGMPixValueSlider.setMinimum( m_minExcitedVolumePixValue );
        
        m_maximalWMPixValueSlider.setMaximum( m_maxExcitedVolumePixValue );
        m_maximalWMPixValueSlider.setMinimum( m_minExcitedVolumePixValue );
        
        m_minimalCSFPixValueSlider.setMaximum( m_maxExcitedVolumePixValue );
        m_minimalCSFPixValueSlider.setMinimum( m_minExcitedVolumePixValue );
        
        m_minimalGMPixValueSlider.setMaximum( m_maxExcitedVolumePixValue );
        m_minimalGMPixValueSlider.setMinimum( m_minExcitedVolumePixValue );
        
        m_minimalWMPixValueSlider.setMaximum( m_maxExcitedVolumePixValue );
        m_minimalWMPixValueSlider.setMinimum( m_minExcitedVolumePixValue );
        
    }
    
    
    /**
     * Handles the events that user wants to set. 
     */
    public void activateItemListeners( )
    {
        m_itemListener = new ItemListener( )
        {
            @Override
            public void itemStateChanged( ItemEvent itemEv ) 
            {
            
                if ( itemEv.getSource( ) == m_tissueTypePixelIntensityOrderingComboBox )
                {
                   
                    m_selectedTissueOrdering = itemEv.getItem( ).toString( );
                    
                    if ( itemEv.getStateChange( ) == ItemEvent.SELECTED )
                    {
                        
                        if ( itemEv.getItem( ).toString( ).compareTo( "GM < WM < CSF" ) == 0 )
                        {
                            m_tissueOrderChanging = true;
                            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[0] );
                            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );
                            
                            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[1] );
                            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );
                            
                            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[2] );
                            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );
                            
                            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[3] );
                            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );
                            
                            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[4] );
                            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );
                            
                            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[5] );
                            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );
                            
                            m_selectedTissueOrdering = "GM < WM < CSF";
                            m_tissueOrderChanging = false;
                        }
                        
                        if ( itemEv.getItem( ).toString( ).compareTo( "GM < CSF < WM" ) == 0 )
                        {
                            m_tissueOrderChanging = true;
                            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[0] );
                            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );
                            
                            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[1] );
                            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );
                            
                            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[4] );
                            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );
                            
                            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[5] );
                            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );
                            
                            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[2] );
                            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );
                            
                            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[3] );
                            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );
                            
                            m_selectedTissueOrdering =  "GM < CSF < WM";
                            m_tissueOrderChanging = false;
                        }
                        if ( itemEv.getItem( ).toString( ).compareTo( "WM < GM < CSF" ) == 0 )
                        {
                            m_tissueOrderChanging = true;
                            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[2] );
                            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );
                            
                            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[3] );
                            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );
                            
                            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[0] );
                            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );
                            
                            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[1] );
                            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );
                            
                            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[4] );
                            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );
                            
                            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[5] );
                            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );
                            
                            m_selectedTissueOrdering = "WM < GM < CSF";
                            m_tissueOrderChanging = false;
                        }
                        
                        if ( itemEv.getItem( ).toString( ).compareTo( "WM < CSF < GM" ) == 0 )
                        {
                            m_tissueOrderChanging = true;
                            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[4] );
                            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );
                            
                            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[5] );
                            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );
                            
                            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[0] );
                            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );
                            
                            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[1] );
                            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );
                            
                            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[2] );
                            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );
                            
                            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[3] );
                            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );
                            
                            m_selectedTissueOrdering = "WM < CSF < GM";
                            m_tissueOrderChanging = false;
                        }
                        if ( itemEv.getItem( ).toString( ).compareTo( "CSF < GM < WM" ) == 0 )
                        {
                            m_tissueOrderChanging = true;
                            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[2] );
                            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );
                            
                            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[3] );
                            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );
                            
                            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[4] );
                            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );
                            
                            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[5] );
                            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );
                            
                            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[0] );
                            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );
                            
                            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[1] );
                            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );
                            
                            m_selectedTissueOrdering = "CSF < GM < WM";
                            
                            m_tissueOrderChanging = false;
                        }
                        if ( itemEv.getItem( ).toString( ).compareTo( "CSF < WM < GM" ) == 0 )
                        {
                            m_tissueOrderChanging = true;
                            
                            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[4] );
                            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );
                            
                            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[5] );
                            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );
                            
                            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[2] );
                            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );
                            
                            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[3] );
                            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );
                            
                            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[0] );
                            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );
                            
                            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[1] );
                            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );
                            
                            m_selectedTissueOrdering = "CSF < WM < GM";
                            
                            m_tissueOrderChanging = false;
                        }
                        
                    }
                }  
            }
            
        };
        
        m_tissueTypePixelIntensityOrderingComboBox.addItemListener( m_itemListener );
        
    }
    
    /**
     * This should initialize the GUI properly!
     * @param tissueOrdering 
     */
    private void initializeSliders( String tissueOrdering )
    {
 
        if ( tissueOrdering.compareTo( "GM < WM < CSF" ) == 0 )
        {
            m_tissueOrderChanging = true;
            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[0] );
            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );

            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[1] );
            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );

            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[2] );
            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );

            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[3] );
            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );

            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[4] );
            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );

            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[5] );
            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );

            m_selectedTissueOrdering = "GM < WM < CSF";
            m_tissueOrderChanging = false;
        }

        if ( tissueOrdering.compareTo( "GM < CSF < WM" ) == 0 )
        {
            m_tissueOrderChanging = true;
            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[0] );
            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );

            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[1] );
            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );

            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[4] );
            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );

            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[5] );
            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );

            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[2] );
            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );

            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[3] );
            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );

            m_selectedTissueOrdering =  "GM < CSF < WM";
            m_tissueOrderChanging = false;
        }
        if ( tissueOrdering.compareTo( "WM < GM < CSF" ) == 0 )
        {
            m_tissueOrderChanging = true;
            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[2] );
            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );

            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[3] );
            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );

            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[0] );
            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );

            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[1] );
            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );

            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[4] );
            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );

            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[5] );
            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );

            m_selectedTissueOrdering = "WM < GM < CSF";
            m_tissueOrderChanging = false;
        }

        if ( tissueOrdering.compareTo( "WM < CSF < GM" ) == 0 )
        {
            m_tissueOrderChanging = true;
            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[4] );
            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );

            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[5] );
            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );

            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[0] );
            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );

            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[1] );
            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );

            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[2] );
            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );

            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[3] );
            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );

            m_selectedTissueOrdering = "WM < CSF < GM";
            m_tissueOrderChanging = false;
        }
        if ( tissueOrdering.compareTo( "CSF < GM < WM" ) == 0 )
        {
            m_tissueOrderChanging = true;
            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[2] );
            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );

            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[3] );
            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );

            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[4] );
            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );

            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[5] );
            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );

            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[0] );
            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );

            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[1] );
            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );

            m_selectedTissueOrdering = "CSF < GM < WM";

            m_tissueOrderChanging = false;
        }

        if ( tissueOrdering.compareTo( "CSF < WM < GM" ) == 0 )
        {
            m_tissueOrderChanging = true;

            m_minimalGMPixValueSlider.setValue( m_orderedIntensityValues[4] );
            m_minimalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[4] );

            m_maximalGMPixValueSlider.setValue( m_orderedIntensityValues[5] );
            m_maximalGMPixValueFormattedTextField.setValue( m_orderedIntensityValues[5] );

            m_minimalWMPixValueSlider.setValue( m_orderedIntensityValues[2] );
            m_minimalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[2] );

            m_maximalWMPixValueSlider.setValue( m_orderedIntensityValues[3] );
            m_maximalWMPixValueFormattedTextField.setValue( m_orderedIntensityValues[3] );

            m_minimalCSFPixValueSlider.setValue( m_orderedIntensityValues[0] );
            m_minimalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[0] );

            m_maximalCSFPixValueSlider.setValue( m_orderedIntensityValues[1] );
            m_maximalCSFPixValueFormattedTextField.setValue( m_orderedIntensityValues[1] );

            m_selectedTissueOrdering = "CSF < WM < GM";

            m_tissueOrderChanging = false;
        }
    }
    
    
    /**
     * Routine handling the consistency of the intervals set. 
     */
    private void checkConsistencyOfSegmentationIntervals( String sliderTypeUsed, String targetValue )
    {
        int minCSF = m_minimalCSFPixValueSlider.getValue( );
        int maxCSF = m_maximalCSFPixValueSlider.getValue( );
        int minGM = m_minimalGMPixValueSlider.getValue( );
        int maxGM = m_maximalGMPixValueSlider.getValue( );
        int minWM = m_minimalWMPixValueSlider.getValue( );
        int maxWM = m_maximalWMPixValueSlider.getValue( );
        
        
    }
    
    
    /**
     * Define the ChangeListener in order to detect user made changes in the
     * sliders.
     */
    public void activateKeyboardListeners( )
    { 
        m_keyListener = new KeyListener( ) 
        {        
            @Override
            public void keyPressed( KeyEvent keyEvent )
            {                
                if ( keyEvent.getKeyCode( ) == KeyEvent.VK_ENTER )
                {
                    if ( keyEvent.getComponent( ) == m_minimalCSFPixValueFormattedTextField )
                    {
                        int val = Integer.parseInt( "" + m_minimalCSFPixValueFormattedTextField.getText( ) );
                        m_minimalCSFPixValueSlider.setValue( val );
                    }
                    
                    if ( keyEvent.getComponent( ) == m_maximalCSFPixValueFormattedTextField )
                    {
                        int val = Integer.parseInt( "" + m_maximalCSFPixValueFormattedTextField.getText( ) );
                        m_maximalCSFPixValueSlider.setValue( val );
                    }
                    
                    if ( keyEvent.getComponent( ) == m_minimalGMPixValueFormattedTextField )
                    {
                        int val = Integer.parseInt( "" + m_minimalGMPixValueFormattedTextField.getText( ) );
                        m_minimalGMPixValueSlider.setValue( val );
                    }
                    
                    if ( keyEvent.getComponent( ) == m_maximalGMPixValueFormattedTextField )
                    {
                        int val = Integer.parseInt( "" + m_maximalGMPixValueFormattedTextField.getText( ) );
                        m_maximalGMPixValueSlider.setValue( val );
                    }
                    
                     if ( keyEvent.getComponent( ) == m_minimalWMPixValueFormattedTextField )
                    {
                        int val = Integer.parseInt( "" + m_minimalWMPixValueFormattedTextField.getText( ) );
                        m_minimalWMPixValueSlider.setValue( val );
                    }
                    
                    if ( keyEvent.getComponent( ) == m_maximalWMPixValueFormattedTextField )
                    {
                        int val = Integer.parseInt( "" + m_maximalWMPixValueFormattedTextField.getText( ) );
                        m_maximalWMPixValueSlider.setValue( val );
                    }
                    
                    if ( keyEvent.getComponent( ) ==  m_transparencyValueFormattedTextField )
                    {
                        int val = Integer.parseInt( "" + m_transparencyValueFormattedTextField.getText( ) );
                        m_transparancyOverlayPixValueSlider.setValue( val );
                    }
                }
            }

            @Override
            public void keyReleased( KeyEvent keyEvent ) 
            {
                // not defined
            }

            @Override
            public void keyTyped( KeyEvent keyEvent ) 
            {
                // not defined
            }

        };
                
        this.m_minimalCSFPixValueFormattedTextField.addKeyListener( m_keyListener );
        this.m_maximalCSFPixValueFormattedTextField.addKeyListener( m_keyListener );
        this.m_minimalGMPixValueFormattedTextField.addKeyListener( m_keyListener );
        this.m_maximalGMPixValueFormattedTextField.addKeyListener( m_keyListener );
        this.m_minimalWMPixValueFormattedTextField.addKeyListener( m_keyListener );
        this.m_maximalWMPixValueFormattedTextField.addKeyListener( m_keyListener );
        this.m_transparencyValueFormattedTextField.addKeyListener( m_keyListener );
    }
    
    
    /**
     * Define the ChangeListener in order to detect user made changes in the
     * sliders.
     */
    public void activateChangeListeners( )
    {
        // common listener for all sliders
        m_changeListener = new ChangeListener( )
        {
            @Override
            public void stateChanged( ChangeEvent event )
            {
               // update text field when the slider value changes
               JSlider source = (JSlider) event.getSource( );
               if ( m_tissueOrderChanging == false )
               {
                   // CSF
                   if ( source == m_minimalCSFPixValueSlider )
                   {
                        if ( !source.getValueIsAdjusting( ) )
                        {
                            m_minimalCSFPixValueFormattedTextField.setValue( source.getValue( ) );
                            
                            m_tissueOrderChanging = true;
                            
                            if ( m_selectedTissueOrdering.compareTo( "WM < CSF < GM" ) == 0 )
                            {
                                m_maximalWMPixValueSlider.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                                m_maximalWMPixValueFormattedTextField.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                            }

                           if ( m_selectedTissueOrdering.compareTo( "WM < GM < CSF" ) == 0 )
                            {
                                m_maximalGMPixValueSlider.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                                m_maximalGMPixValueFormattedTextField.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < CSF < WM" ) == 0 )
                            {
                                m_maximalGMPixValueSlider.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                                m_maximalGMPixValueFormattedTextField.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < WM < CSF" ) == 0 )
                            {
                                m_maximalWMPixValueSlider.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                                m_maximalWMPixValueFormattedTextField.setValue( m_minimalCSFPixValueSlider.getValue( ) - 1 );
                            }
                            
                            m_tissueOrderChanging = false;
                            
                            System.out.println( "Slider m_minimalCSFPixValueFormattedTextField moved" );
                            updateGUI( );
                        }
                   }

                   if ( source == m_maximalCSFPixValueSlider )
                   {
                        if ( !source.getValueIsAdjusting( ) )
                        {
                            m_maximalCSFPixValueFormattedTextField.setValue( source.getValue( ) );
                            m_tissueOrderChanging = true;
                            if ( m_selectedTissueOrdering.compareTo( "CSF < WM < GM" ) == 0 )
                            {
                                m_minimalWMPixValueSlider.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                                m_minimalWMPixValueFormattedTextField.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "WM < CSF < GM" ) == 0 )
                            {
                                m_minimalGMPixValueSlider.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                                m_minimalGMPixValueFormattedTextField.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < CSF < WM" ) == 0 )
                            {
                                m_minimalWMPixValueSlider.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                                m_minimalWMPixValueFormattedTextField.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "CSF < GM < WM" ) == 0 )
                            {
                                m_minimalGMPixValueSlider.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                                m_minimalGMPixValueFormattedTextField.setValue( m_maximalCSFPixValueSlider.getValue( ) + 1 );
                            }
                            m_tissueOrderChanging = false;

                            System.out.println( "Slider m_maximalCSFPixValueFormattedTextField moved" );
                            updateGUI( );
                        }
                   }

                   // WM
                   if ( source == m_minimalWMPixValueSlider )
                   {
                        if ( !source.getValueIsAdjusting( ) )
                        {
                            m_minimalWMPixValueFormattedTextField.setValue( source.getValue( ) );
                            m_tissueOrderChanging = true;
                            if ( m_selectedTissueOrdering.compareTo( "CSF < GM < WM" ) == 0 )
                            {
                                m_maximalGMPixValueSlider.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                                m_maximalGMPixValueFormattedTextField.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "CSF < WM < GM" ) == 0 )
                            {
                                m_maximalCSFPixValueSlider.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                                m_maximalCSFPixValueFormattedTextField.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < CSF < WM" ) == 0 )
                            {
                                m_maximalCSFPixValueSlider.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                                m_maximalCSFPixValueFormattedTextField.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < WM < CSF" ) == 0 )
                            {
                                m_maximalGMPixValueSlider.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                                m_maximalGMPixValueFormattedTextField.setValue( m_minimalWMPixValueSlider.getValue( ) - 1 );
                            }
                            m_tissueOrderChanging = false;
                            System.out.println( "Slider m_minimalWMPixValueSlider moved" );
                            updateGUI( );
                       }
                   }
                   
                   if ( source == m_maximalWMPixValueSlider )
                   {
                        if ( !source.getValueIsAdjusting( ) )
                        {
                            m_maximalWMPixValueFormattedTextField.setValue( source.getValue( ) );
                            m_tissueOrderChanging = true;
                            if ( m_selectedTissueOrdering.compareTo( "CSF < WM < GM" ) == 0 )
                            {
                                m_minimalGMPixValueSlider.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                                // also update the FormattedTextField
                                m_minimalGMPixValueFormattedTextField.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "WM < CSF < GM" ) == 0 )
                            {
                                m_minimalCSFPixValueSlider.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                                m_minimalCSFPixValueFormattedTextField.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "WM < GM < CSF" ) == 0 )
                            {
                                m_minimalGMPixValueSlider.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                                m_minimalGMPixValueFormattedTextField.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < WM < CSF" ) == 0 )
                            {
                                m_minimalCSFPixValueSlider.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                                m_minimalCSFPixValueFormattedTextField.setValue( m_maximalWMPixValueSlider.getValue( ) + 1 );
                            }
                            m_tissueOrderChanging = false;
                            System.out.println( "Slider m_maximalWMPixValueSlider moved" );
                            updateGUI( );
                        }
                   }

                   // GM
                   if ( source == m_minimalGMPixValueSlider )
                   {
                        if ( !source.getValueIsAdjusting( ) )
                        {
                            m_minimalGMPixValueFormattedTextField.setValue( source.getValue( ) );
                            m_tissueOrderChanging = true;        
                            if ( m_selectedTissueOrdering.compareTo( "CSF < GM < WM" ) == 0 )
                            {
                                m_maximalCSFPixValueSlider.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                                m_maximalCSFPixValueFormattedTextField.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "WM < GM < CSF" ) == 0 )
                            {
                                m_maximalWMPixValueSlider.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                                m_maximalWMPixValueFormattedTextField.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "CSF < WM < GM" ) == 0 )
                            {
                                m_maximalWMPixValueSlider.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                                m_maximalWMPixValueFormattedTextField.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "WM < CSF < GM" ) == 0 )
                            {
                                m_maximalCSFPixValueSlider.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                                m_maximalCSFPixValueFormattedTextField.setValue( m_minimalGMPixValueSlider.getValue( ) - 1 );
                            }
                            m_tissueOrderChanging = false;
                            System.out.println( "Slider m_minimalGMPixValueSlider moved" );
                            updateGUI( );
                        }
                   }

                   if ( source == m_maximalGMPixValueSlider )
                   {
                        if ( !source.getValueIsAdjusting( ) )
                        {
                            m_maximalGMPixValueFormattedTextField.setValue( source.getValue( ) );
                            m_tissueOrderChanging = true;
                            if ( m_selectedTissueOrdering.compareTo( "CSF < GM < WM" ) == 0 )
                            {
                                m_minimalWMPixValueSlider.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                                m_minimalWMPixValueFormattedTextField.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "WM < GM < CSF" ) == 0 )
                            {
                                m_minimalCSFPixValueSlider.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                                m_minimalCSFPixValueFormattedTextField.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < WM < CSF" ) == 0 )
                            {
                                m_minimalWMPixValueSlider.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                                m_minimalWMPixValueFormattedTextField.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                            }

                            if ( m_selectedTissueOrdering.compareTo( "GM < CSF < WM" ) == 0 )
                            {
                                m_minimalCSFPixValueSlider.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                                m_minimalCSFPixValueFormattedTextField.setValue( m_maximalGMPixValueSlider.getValue( ) + 1 );
                            }
                            m_tissueOrderChanging = false;
                            System.out.println( "Slider m_maximalGMPixValueSlider moved" );
                            updateGUI( );
                        }
                   }
               }
               
               if ( source == m_transparancyOverlayPixValueSlider )
               {
                    if ( !source.getValueIsAdjusting( ) )
                    {
                        m_transparencyValueFormattedTextField.setValue( source.getValue( ) );

                        int redGM = m_gmOverlayColor.getRed( );
                        int blueGM = m_gmOverlayColor.getBlue( );
                        int greenGM = m_gmOverlayColor.getGreen( );

                        m_gmOverlayColor = new Color( redGM, greenGM, blueGM, m_transparancyOverlayPixValueSlider.getValue( ) );

                        int redWM = m_wmOverlayColor.getRed( );
                        int blueWM = m_wmOverlayColor.getBlue( );
                        int greenWM = m_wmOverlayColor.getGreen( );

                        m_wmOverlayColor = new Color( redWM, greenWM, blueWM, m_transparancyOverlayPixValueSlider.getValue( ) );

                        int redCSF = m_csfOverlayColor.getRed( );
                        int blueCSF = m_csfOverlayColor.getBlue( );
                        int greenCSF = m_csfOverlayColor.getGreen( );

                        m_csfOverlayColor = new Color( redCSF, greenCSF, blueCSF, m_transparancyOverlayPixValueSlider.getValue( ) );

                        m_globalModel.getCSIGrid( ).setSegmentationGMOverlayColor( m_gmOverlayColor );
                        m_globalModel.getCSIGrid( ).setSegmentationWMOverlayColor( m_wmOverlayColor );
                        m_globalModel.getCSIGrid( ).setSegmentationCSFOverlayColor( m_csfOverlayColor );

                        System.out.println( "Slider m_transparancyOverlayPixValueSlider moved" );
                        updateGUI( );
                    }
               }

            }  

        };
        
        m_minimalCSFPixValueSlider.addChangeListener( m_changeListener );
        m_maximalCSFPixValueSlider.addChangeListener( m_changeListener );
        m_minimalGMPixValueSlider.addChangeListener( m_changeListener );
        m_maximalGMPixValueSlider.addChangeListener( m_changeListener );
        m_minimalWMPixValueSlider.addChangeListener( m_changeListener );
        m_maximalWMPixValueSlider.addChangeListener( m_changeListener );
        m_transparancyOverlayPixValueSlider.addChangeListener( m_changeListener );

    }
    
    private void updateGUI( ) 
    {
        
        HashMap<String, Integer> myMap = m_globalModel.getCSIGrid( ).prepareImageSegmentColorOverlayComputation( false, 
                                            m_minimalCSFPixValueSlider.getValue( ),
                                            m_maximalCSFPixValueSlider.getValue( ),
                                            m_minimalGMPixValueSlider.getValue( ),
                                            m_maximalGMPixValueSlider.getValue( ),
                                            m_minimalWMPixValueSlider.getValue( ),
                                            m_maximalWMPixValueSlider.getValue( ) );
        
        m_globalModel.updateViewImagePanels( );
        int CSFpix = m_globalModel.getCSIGrid( ).getNumberOfCSFPixelsInExcitedVolume( );
        int GMpix = m_globalModel.getCSIGrid( ).getNumberOfGMPixelsInExcitedVolume( );
        int WMpix = m_globalModel.getCSIGrid( ).getNumberOfWMPixelsInExcitedVolume( );
        int totPix = m_globalModel.getCSIGrid( ).getTotalNumberPixelsInExcitedVolume( );
        m_numberOfCSFPixelsValueLabel.setText( "" + CSFpix );
        m_numberOfGMPixelsValueLabel.setText( "" + GMpix );
        m_numberOfWMPixelsValueLabel.setText( "" + WMpix );
        m_numberOfTotalPixelsValueLabel.setText("" + totPix );
        
        m_volumeOfCSFPixelsValueLabel.setText( "" + CSFpix * m_physPixelWidth * m_physPixelHeight * ( m_physPixelDepth + m_imageGap ) );
        m_volumeOfGMPixelsValueLabel.setText( "" + GMpix * m_physPixelWidth * m_physPixelHeight * ( m_physPixelDepth + m_imageGap ) );
        m_volumeOfWMPixelsValueLabel.setText( "" + WMpix * m_physPixelWidth * m_physPixelHeight * ( m_physPixelDepth + m_imageGap ) );
        m_volumeOfTotalPixelsValueLabel.setText("" + totPix * m_physPixelWidth * m_physPixelHeight * m_physPixelDepth );
        
        if ( totPix != ( WMpix+GMpix+CSFpix ) )
        {
            
            m_numberOfCSFPixelsValueLabel.setForeground( Color.RED );
            m_numberOfGMPixelsValueLabel.setForeground( Color.RED );
            m_numberOfWMPixelsValueLabel.setForeground( Color.RED );
            
            m_volumeOfCSFPixelsValueLabel.setForeground( Color.RED );
            m_volumeOfGMPixelsValueLabel.setForeground( Color.RED );
            m_volumeOfWMPixelsValueLabel.setForeground( Color.RED );
            
        }
        else
        {
            m_numberOfCSFPixelsValueLabel.setForeground( Color.BLACK );
            m_numberOfGMPixelsValueLabel.setForeground( Color.BLACK );
            m_numberOfWMPixelsValueLabel.setForeground( Color.BLACK );
            
            m_volumeOfCSFPixelsValueLabel.setForeground( Color.BLACK );
            m_volumeOfGMPixelsValueLabel.setForeground( Color.BLACK );
            m_volumeOfWMPixelsValueLabel.setForeground( Color.BLACK );
        }
    }
    
    /**
     * Sets the default values of the segmentation boundaries.
     * @param minimumGM
     * @param maximumGM
     * @param minimumWM
     * @param maximumWM
     * @param minimumCSF
     * @param maximumCSF 
     */
    public void setDefaultSliderValues( int minimumGM, int maximumGM, 
                                        int minimumWM, int maximumWM, 
                                        int minimumCSF, int maximumCSF )
    {

        m_maximalCSFPixValueSlider.setValue( maximumCSF );        
        m_maximalGMPixValueSlider.setValue( maximumGM );
        m_maximalWMPixValueSlider.setValue( maximumWM );
        
        m_minimalCSFPixValueSlider.setValue( minimumCSF );
        m_minimalGMPixValueSlider.setValue( minimumGM );    
        m_minimalWMPixValueSlider.setValue( minimumWM );
        
        validateSliderValues( );
    }
    
    
    public void validateSliderValues( )
    {
        // not implemented yet.
    }
    
    
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_segmentationValueDefinitionPanel = new javax.swing.JPanel();
        m_minimalGMPixValueLabel = new javax.swing.JLabel();
        m_minimalGMPixValueFormattedTextField = new javax.swing.JFormattedTextField();
        m_minimalGMPixValueSlider = new javax.swing.JSlider();
        m_maximalGMPixValueLabel = new javax.swing.JLabel();
        m_maximalGMPixValueFormattedTextField = new javax.swing.JFormattedTextField();
        m_maximalGMPixValueSlider = new javax.swing.JSlider();
        m_minimalWMPixValueLabel = new javax.swing.JLabel();
        m_minimalWMPixValueFormattedTextField = new javax.swing.JFormattedTextField();
        m_minimalWMPixValueSlider = new javax.swing.JSlider();
        m_maximalWMPixValueLabel = new javax.swing.JLabel();
        m_maximalWMPixValueFormattedTextField = new javax.swing.JFormattedTextField();
        m_maximalWMPixValueSlider = new javax.swing.JSlider();
        m_minimalCSFPixValueLabel = new javax.swing.JLabel();
        m_minimalCSFPixValueFormattedTextField = new javax.swing.JFormattedTextField();
        m_minimalCSFPixValueSlider = new javax.swing.JSlider();
        m_maximalCSFPixValueLabel = new javax.swing.JLabel();
        m_maximalCSFPixValueFormattedTextField = new javax.swing.JFormattedTextField();
        m_maximalCSFPixValueSlider = new javax.swing.JSlider();
        m_buttonPanel = new javax.swing.JPanel();
        m_acceptAndQuitButton = new javax.swing.JButton();
        m_quitButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        m_minPixValLabel = new javax.swing.JLabel();
        m_maxPixValLabel = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        m_minPixValExcitedVolumeLabel = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        m_maxPixValExcitedVolumeLabel = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        m_GMColorOverlaySelectionButton = new javax.swing.JButton();
        m_WMColorOverlaySelectionButton = new javax.swing.JButton();
        m_CSFColorOverlaySelectionButton = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        m_maximalCSFPixValueLabel1 = new javax.swing.JLabel();
        m_transparencyValueFormattedTextField = new javax.swing.JFormattedTextField();
        m_transparancyOverlayPixValueSlider = new javax.swing.JSlider();
        m_createReportButton = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        m_numberOfGMPixelsLabel = new javax.swing.JLabel();
        m_numberOfWMPixelsLabel = new javax.swing.JLabel();
        m_numberOfCSFPixelsLabel = new javax.swing.JLabel();
        m_numberOfTotalPixelsLabel = new javax.swing.JLabel();
        m_numberOfGMPixelsValueLabel = new javax.swing.JLabel();
        m_numberOfWMPixelsValueLabel = new javax.swing.JLabel();
        m_numberOfCSFPixelsValueLabel = new javax.swing.JLabel();
        m_numberOfTotalPixelsValueLabel = new javax.swing.JLabel();
        jPanel6 = new javax.swing.JPanel();
        m_tissueTypePixelIntensityOrderingComboBox = new javax.swing.JComboBox<>();
        jPanel7 = new javax.swing.JPanel();
        m_volumeOfGMPixelsLabel = new javax.swing.JLabel();
        m_volumerOfWMPixelsLabel = new javax.swing.JLabel();
        m_volumeOfCSFPixelsLabel = new javax.swing.JLabel();
        m_volumeOfTotalPixelsLabel = new javax.swing.JLabel();
        m_volumeOfGMPixelsValueLabel = new javax.swing.JLabel();
        m_volumeOfWMPixelsValueLabel = new javax.swing.JLabel();
        m_volumeOfCSFPixelsValueLabel = new javax.swing.JLabel();
        m_volumeOfTotalPixelsValueLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Define pixel Intensity boundaries for WM/GM/CSF segmentation");

        m_segmentationValueDefinitionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Segmentation thressholds"));

        m_minimalGMPixValueLabel.setText("Minimum GM pixel value");

        m_minimalGMPixValueFormattedTextField.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(java.text.NumberFormat.getIntegerInstance())));
        m_minimalGMPixValueFormattedTextField.setText("0");
        m_minimalGMPixValueFormattedTextField.setToolTipText("");
        m_minimalGMPixValueFormattedTextField.setCursor(new java.awt.Cursor(java.awt.Cursor.TEXT_CURSOR));

        m_maximalGMPixValueLabel.setText("Maximum GM pixel value");

        m_maximalGMPixValueFormattedTextField.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(java.text.NumberFormat.getIntegerInstance())));
        m_maximalGMPixValueFormattedTextField.setText("0");

        m_minimalWMPixValueLabel.setText("Minimum WM pixel value");

        m_minimalWMPixValueFormattedTextField.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(java.text.NumberFormat.getIntegerInstance())));
        m_minimalWMPixValueFormattedTextField.setText("0");

        m_maximalWMPixValueLabel.setText("Maximum WM pixel value");

        m_maximalWMPixValueFormattedTextField.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(java.text.NumberFormat.getIntegerInstance())));
        m_maximalWMPixValueFormattedTextField.setText("0");

        m_minimalCSFPixValueLabel.setText("Minimum CSF pixel value");

        m_minimalCSFPixValueFormattedTextField.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(java.text.NumberFormat.getIntegerInstance())));
        m_minimalCSFPixValueFormattedTextField.setText("0");

        m_maximalCSFPixValueLabel.setText("Maximum CSF pixel value");

        m_maximalCSFPixValueFormattedTextField.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(java.text.NumberFormat.getIntegerInstance())));
        m_maximalCSFPixValueFormattedTextField.setText("0");

        javax.swing.GroupLayout m_segmentationValueDefinitionPanelLayout = new javax.swing.GroupLayout(m_segmentationValueDefinitionPanel);
        m_segmentationValueDefinitionPanel.setLayout(m_segmentationValueDefinitionPanelLayout);
        m_segmentationValueDefinitionPanelLayout.setHorizontalGroup(
            m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                        .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                                .addComponent(m_minimalCSFPixValueLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(m_minimalCSFPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 105, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                                .addComponent(m_maximalCSFPixValueLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_maximalCSFPixValueFormattedTextField)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_minimalCSFPixValueSlider, javax.swing.GroupLayout.DEFAULT_SIZE, 297, Short.MAX_VALUE)
                            .addComponent(m_maximalCSFPixValueSlider, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                        .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                                .addComponent(m_minimalWMPixValueLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(m_minimalWMPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 105, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                                .addComponent(m_maximalWMPixValueLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_maximalWMPixValueFormattedTextField)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_minimalWMPixValueSlider, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_maximalWMPixValueSlider, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                        .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                                .addComponent(m_maximalGMPixValueLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_maximalGMPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 111, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                                .addComponent(m_minimalGMPixValueLabel)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addComponent(m_minimalGMPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 109, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_minimalGMPixValueSlider, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_maximalGMPixValueSlider, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
                .addContainerGap())
        );

        m_segmentationValueDefinitionPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {m_maximalCSFPixValueLabel, m_maximalGMPixValueLabel, m_maximalWMPixValueLabel, m_minimalCSFPixValueLabel, m_minimalGMPixValueLabel, m_minimalWMPixValueLabel});

        m_segmentationValueDefinitionPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {m_maximalCSFPixValueFormattedTextField, m_maximalGMPixValueFormattedTextField, m_maximalWMPixValueFormattedTextField, m_minimalCSFPixValueFormattedTextField, m_minimalGMPixValueFormattedTextField, m_minimalWMPixValueFormattedTextField});

        m_segmentationValueDefinitionPanelLayout.setVerticalGroup(
            m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_segmentationValueDefinitionPanelLayout.createSequentialGroup()
                .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_minimalGMPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_minimalGMPixValueLabel)
                        .addComponent(m_minimalGMPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_maximalGMPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_maximalGMPixValueLabel)
                        .addComponent(m_maximalGMPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_minimalWMPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_minimalWMPixValueLabel)
                        .addComponent(m_minimalWMPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_maximalWMPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_maximalWMPixValueLabel)
                        .addComponent(m_maximalWMPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_minimalCSFPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_minimalCSFPixValueLabel)
                        .addComponent(m_minimalCSFPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_maximalCSFPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(m_segmentationValueDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_maximalCSFPixValueLabel)
                        .addComponent(m_maximalCSFPixValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
        );

        m_acceptAndQuitButton.setText("Accept and Quit");
        m_acceptAndQuitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_acceptAndQuitButtonActionPerformed(evt);
            }
        });

        m_quitButton.setText("Quit");
        m_quitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_quitButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_buttonPanelLayout = new javax.swing.GroupLayout(m_buttonPanel);
        m_buttonPanel.setLayout(m_buttonPanelLayout);
        m_buttonPanelLayout.setHorizontalGroup(
            m_buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_buttonPanelLayout.createSequentialGroup()
                .addGap(18, 18, 18)
                .addComponent(m_quitButton, javax.swing.GroupLayout.PREFERRED_SIZE, 118, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_acceptAndQuitButton, javax.swing.GroupLayout.PREFERRED_SIZE, 118, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18))
        );
        m_buttonPanelLayout.setVerticalGroup(
            m_buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_buttonPanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(m_buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_acceptAndQuitButton)
                    .addComponent(m_quitButton))
                .addContainerGap())
        );

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Image stack & excited volume properties"));

        jLabel3.setText("Minimal pixel value of total image stack:");

        m_minPixValLabel.setText("here goes minimal value");

        m_maxPixValLabel.setText("here goes maximal value");

        jLabel6.setText("Maximal pixel value of total image stack:");

        jLabel4.setText("Minimal pixel value of excited volume:");

        m_minPixValExcitedVolumeLabel.setText("here goes minimal value");

        jLabel7.setText("Maximal pixel value of excited volume:");

        m_maxPixValExcitedVolumeLabel.setText("here goes maximal value");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_minPixValLabel))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel6)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_maxPixValLabel))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jLabel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabel7, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addGap(18, 18, 18)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_minPixValExcitedVolumeLabel)
                            .addComponent(m_maxPixValExcitedVolumeLabel))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {jLabel3, jLabel6});

        jPanel1Layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {m_maxPixValLabel, m_minPixValLabel});

        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(m_minPixValLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(m_maxPixValLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(m_minPixValExcitedVolumeLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(m_maxPixValExcitedVolumeLabel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Color of overlay"));

        m_GMColorOverlaySelectionButton.setForeground(new java.awt.Color(255, 0, 0));
        m_GMColorOverlaySelectionButton.setText("Select GM overlay color");
        m_GMColorOverlaySelectionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_GMColorOverlaySelectionButtonActionPerformed(evt);
            }
        });

        m_WMColorOverlaySelectionButton.setForeground(new java.awt.Color(0, 153, 0));
        m_WMColorOverlaySelectionButton.setText("Select WM overlay color");
        m_WMColorOverlaySelectionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_WMColorOverlaySelectionButtonActionPerformed(evt);
            }
        });

        m_CSFColorOverlaySelectionButton.setForeground(new java.awt.Color(0, 0, 255));
        m_CSFColorOverlaySelectionButton.setText("Select CSF overlay color");
        m_CSFColorOverlaySelectionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_CSFColorOverlaySelectionButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_CSFColorOverlaySelectionButton, javax.swing.GroupLayout.DEFAULT_SIZE, 201, Short.MAX_VALUE)
                    .addComponent(m_WMColorOverlaySelectionButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_GMColorOverlaySelectionButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel2Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_GMColorOverlaySelectionButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_WMColorOverlaySelectionButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_CSFColorOverlaySelectionButton)
                .addGap(21, 21, 21))
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Transparancy of overlays"));

        m_maximalCSFPixValueLabel1.setText("Alpha");

        m_transparencyValueFormattedTextField.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(java.text.NumberFormat.getIntegerInstance())));
        m_transparencyValueFormattedTextField.setToolTipText("");
        m_transparencyValueFormattedTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_transparencyValueFormattedTextFieldActionPerformed(evt);
            }
        });

        m_transparancyOverlayPixValueSlider.setMaximum(255);

        m_createReportButton.setText("Create report file");
        m_createReportButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_createReportButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_maximalCSFPixValueLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 118, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(2, 2, 2)
                .addComponent(m_transparencyValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 108, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_transparancyOverlayPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, 290, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(m_createReportButton)
                .addGap(49, 49, 49))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(m_createReportButton)
                    .addComponent(m_transparancyOverlayPixValueSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(m_maximalCSFPixValueLabel1)
                        .addComponent(m_transparencyValueFormattedTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder("Segmentation results"));

        m_numberOfGMPixelsLabel.setText("#Pixels GM:");

        m_numberOfWMPixelsLabel.setText("#Pixels WM:");

        m_numberOfCSFPixelsLabel.setText("#Pixels CSF:");

        m_numberOfTotalPixelsLabel.setText("#Pixels total:");

        m_numberOfGMPixelsValueLabel.setText("#Pixels GM:");

        m_numberOfWMPixelsValueLabel.setText("#Pixels WM:");

        m_numberOfCSFPixelsValueLabel.setText("#Pixels CSF:");

        m_numberOfTotalPixelsValueLabel.setText("#Pixels total:");

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(m_numberOfCSFPixelsLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_numberOfCSFPixelsValueLabel))
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(m_numberOfWMPixelsLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_numberOfWMPixelsValueLabel))
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(m_numberOfGMPixelsLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_numberOfGMPixelsValueLabel))
                    .addGroup(jPanel5Layout.createSequentialGroup()
                        .addComponent(m_numberOfTotalPixelsLabel)
                        .addGap(18, 18, 18)
                        .addComponent(m_numberOfTotalPixelsValueLabel)
                        .addGap(0, 0, Short.MAX_VALUE)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfGMPixelsLabel)
                    .addComponent(m_numberOfGMPixelsValueLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfWMPixelsLabel)
                    .addComponent(m_numberOfWMPixelsValueLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfCSFPixelsLabel)
                    .addComponent(m_numberOfCSFPixelsValueLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfTotalPixelsLabel)
                    .addComponent(m_numberOfTotalPixelsValueLabel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder("Tissue brightness ordering"));

        m_tissueTypePixelIntensityOrderingComboBox.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "Free", "GM < WM < CSF", "GM < CSF < WM", "WM < GM < CSF", "WM < CSF < GM", "CSF < WM < GM", "CSF < GM < WM" }));
        m_tissueTypePixelIntensityOrderingComboBox.setMaximumSize(new java.awt.Dimension(149, 23));
        m_tissueTypePixelIntensityOrderingComboBox.setMinimumSize(new java.awt.Dimension(149, 23));
        m_tissueTypePixelIntensityOrderingComboBox.setPreferredSize(new java.awt.Dimension(149, 23));

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_tissueTypePixelIntensityOrderingComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_tissueTypePixelIntensityOrderingComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder("Segmentation results"));

        m_volumeOfGMPixelsLabel.setText("Volume GM:");

        m_volumerOfWMPixelsLabel.setText("Volume WM:");

        m_volumeOfCSFPixelsLabel.setText("Volume CSF:");

        m_volumeOfTotalPixelsLabel.setText("Volume total:");

        m_volumeOfGMPixelsValueLabel.setText("#Volume GM:");

        m_volumeOfWMPixelsValueLabel.setText("#Volume WM:");

        m_volumeOfCSFPixelsValueLabel.setText("#Volume CSF");

        m_volumeOfTotalPixelsValueLabel.setText("#Volume total");

        javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
        jPanel7.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel7Layout.createSequentialGroup()
                            .addComponent(m_volumeOfCSFPixelsLabel)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_volumeOfCSFPixelsValueLabel))
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel7Layout.createSequentialGroup()
                            .addComponent(m_volumeOfTotalPixelsLabel)
                            .addGap(18, 18, 18)
                            .addComponent(m_volumeOfTotalPixelsValueLabel)))
                    .addGroup(jPanel7Layout.createSequentialGroup()
                        .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_volumerOfWMPixelsLabel)
                            .addComponent(m_volumeOfGMPixelsLabel))
                        .addGap(25, 25, 25)
                        .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_volumeOfGMPixelsValueLabel)
                            .addComponent(m_volumeOfWMPixelsValueLabel))))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_volumeOfGMPixelsLabel)
                    .addComponent(m_volumeOfGMPixelsValueLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_volumerOfWMPixelsLabel)
                    .addComponent(m_volumeOfWMPixelsValueLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_volumeOfCSFPixelsLabel)
                    .addComponent(m_volumeOfCSFPixelsValueLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_volumeOfTotalPixelsLabel)
                    .addComponent(m_volumeOfTotalPixelsValueLabel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jPanel7, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
            .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(m_segmentationValueDefinitionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel6, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
            .addComponent(m_buttonPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPanel6, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(m_segmentationValueDefinitionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3)
                .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel7, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_buttonPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_quitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_quitButtonActionPerformed
        dispose( );
    }//GEN-LAST:event_m_quitButtonActionPerformed

    private void m_acceptAndQuitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_acceptAndQuitButtonActionPerformed
        
        acceptAndQuit( );
        
    }//GEN-LAST:event_m_acceptAndQuitButtonActionPerformed

    private void acceptAndQuit( )
    {
        // Make this values persistent so that users do not have to define the 
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MAXIMAL_PIXEL_VALUE, "" + m_maximalCSFPixValueSlider.getValue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MINIMAL_PIXEL_VALUE, "" + m_minimalCSFPixValueSlider.getValue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MAXIMAL_PIXEL_VALUE, "" + m_maximalWMPixValueSlider.getValue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MINIMAL_PIXEL_VALUE, "" + m_minimalWMPixValueSlider.getValue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MAXIMAL_PIXEL_VALUE, "" + m_maximalGMPixValueSlider.getValue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MINIMAL_PIXEL_VALUE, "" + m_minimalGMPixValueSlider.getValue( ) );
        
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_DISPLAY_TRANSPARENCY, "" + m_transparancyOverlayPixValueSlider.getValue( ) );
        
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_RED_COMPONENT , "" + m_gmOverlayColor.getRed( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_BLUE_COMPONENT , "" + m_gmOverlayColor.getBlue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_GREEN_COMPONENT , "" + m_gmOverlayColor.getGreen( ) );
        
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_RED_COMPONENT , "" + m_wmOverlayColor.getRed( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_BLUE_COMPONENT , "" + m_wmOverlayColor.getBlue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_GREEN_COMPONENT , "" + m_wmOverlayColor.getGreen( ) );
        
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_RED_COMPONENT , "" + m_csfOverlayColor.getRed( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_BLUE_COMPONENT , "" + m_csfOverlayColor.getBlue( ) );
        defs.setProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_GREEN_COMPONENT , "" + m_csfOverlayColor.getGreen( ) );
        
        defs.setProperty( TDFDFittingDefaults.TISSUE_TYPE_ORDER , m_selectedTissueOrdering );
        
        m_globalModel.setSVSGMSegmentVolume( new Float( m_volumeOfGMPixelsValueLabel.getText( ) ) );
        m_globalModel.setSVSWMSegmentVolume( new Float( m_volumeOfWMPixelsValueLabel.getText( ) ) );
        m_globalModel.setSVSCSFSegmentVolume( new Float( m_volumeOfCSFPixelsValueLabel.getText( ) ) );
        
        m_globalModel.setMinimalGMSegmentationPixelValue( m_minimalGMPixValueSlider.getValue( ) );
        m_globalModel.setMinimalWMSegmentationPixelValue( m_minimalWMPixValueSlider.getValue( ) );
        m_globalModel.setMinimalCSFSegmentationPixelValue( m_minimalCSFPixValueSlider.getValue( ) );
        
        m_globalModel.setMaximalGMSegmentationPixelValue( m_maximalGMPixValueSlider.getValue( ) );
        m_globalModel.setMaximalWMSegmentationPixelValue( m_maximalWMPixValueSlider.getValue( ) );
        m_globalModel.setMaximalCSFSegmentationPixelValue( m_maximalCSFPixValueSlider.getValue( ) );
        
        defs.save( );
        dispose( );
    }
    
    
    
    private void m_GMColorOverlaySelectionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_GMColorOverlaySelectionButtonActionPerformed
        Color c = JColorChooser.showDialog( null, "Choose a color for GM overlay", m_gmOverlayColor );
        if ( c != null )
        {
            int trans = new Integer( m_transparencyValueFormattedTextField.getText( ) );
            Color colorToSet = new Color( c.getRed( ), c.getGreen( ), c.getBlue( ), trans );
            m_globalModel.getCSIGrid( ).setSegmentationGMOverlayColor( colorToSet );
            m_gmOverlayColor = colorToSet;
            m_GMColorOverlaySelectionButton.setForeground( c );
            updateGUI( );
        }
    }//GEN-LAST:event_m_GMColorOverlaySelectionButtonActionPerformed

    private void m_WMColorOverlaySelectionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_WMColorOverlaySelectionButtonActionPerformed
        Color c = JColorChooser.showDialog( null, "Choose a color for WM Overlay", m_wmOverlayColor );
        if ( c != null )
        {
            int trans = new Integer( m_transparencyValueFormattedTextField.getText( ) );
            Color colorToSet = new Color( c.getRed( ), c.getGreen( ), c.getBlue( ), trans );
            m_globalModel.getCSIGrid( ).setSegmentationWMOverlayColor( colorToSet );
            m_wmOverlayColor = colorToSet;
            m_WMColorOverlaySelectionButton.setForeground( c );
            updateGUI( );
        }
    }//GEN-LAST:event_m_WMColorOverlaySelectionButtonActionPerformed

    private void m_CSFColorOverlaySelectionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_CSFColorOverlaySelectionButtonActionPerformed
        Color c = JColorChooser.showDialog( null, "Choose a color for CSF Overlay", m_csfOverlayColor );
        if ( c != null )
        {
            int trans = new Integer( "" + m_transparencyValueFormattedTextField.getText( ) );
            Color colorToSet = new Color( c.getRed( ), c.getGreen( ), c.getBlue( ),trans );
            m_globalModel.getCSIGrid( ).setSegmentationCSFOverlayColor( colorToSet );
            m_csfOverlayColor = colorToSet;
            m_CSFColorOverlaySelectionButton.setForeground( c );
            updateGUI( );
        }
    }//GEN-LAST:event_m_CSFColorOverlaySelectionButtonActionPerformed

    private void m_transparencyValueFormattedTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_transparencyValueFormattedTextFieldActionPerformed
       // do nothing
    }//GEN-LAST:event_m_transparencyValueFormattedTextFieldActionPerformed

    /**
     * Returns the filename and directory selected by the user.
     * @param path
     * @param subDirString
     * @param fileDefaultName
     * @param buttonText
     * @return 
     */
    protected String getFilePathAndName( String path, String subDirString, 
                                         String fileDefaultName, 
                                         String buttonText )
    {
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        String outputPath = new String( path + System.getProperty( "file.separator" ) + subDirString );

        File f = new File( outputPath );
        if ( f.exists( ) == false )
        {
            f.mkdir( );
        }

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        fileChooser.setSelectedFile( new File( fileDefaultName ) );

        result = fileChooser.showDialog( this, buttonText );


        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }
    
    private void m_createReportButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_createReportButtonActionPerformed
        
        
        SimpleDateFormat formatter= new SimpleDateFormat( "yyyy-MM-dd 'at' HH:mm:ss z" );  
        Date date = new Date( System.currentTimeMillis( ) );   
        
        TextFileWriter writer =  null;
        try 
        {
            // set the default filename and try to eliminate some regular gabage.
            
            String filename =  m_globalModel.getPatientName( ).replaceAll( " ", "_" );
            filename = filename.replaceAll( "^", "-" );
            filename = filename.replaceAll( "__", "_" );
            
            filename = filename + ".txt";
            String outputFileName = getFilePathAndName( System.getProperty( "user.home" ), "jMRUIResults", filename, "Create segmentation report" );
            File file = new File( outputFileName );
            if ( file.getParentFile( ).exists( ) == false )
            {
                file.getParentFile( ).mkdir( );
            }   
            
            writer = new TextFileWriter( new FileWriter( outputFileName ) );
            
            writer.writeStringLn( "Report of:\t" + m_globalModel.getSpectroscopyFile( ) );
            writer.writeStringLn( "Patient name:\t" + m_globalModel.getPatientName( ) );
            writer.writeStringLn( "Patient date of birth:\t" + m_globalModel.getPatientBirthDate( ) );
            writer.writeStringLn( "Study date:\t" + m_globalModel.getStudyDate( ) );
            writer.writeStringLn( "Spectroscopic voxel size (computed):\t" + this.m_volumeOfTotalPixelsValueLabel.getText( ) );
            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "Minimal GM intensity thresshold  :\t" + m_minimalGMPixValueSlider.getValue( ) );
            writer.writeStringLn( "Maximal GM intensity thresshold  :\t" + m_maximalGMPixValueSlider.getValue( ) );
            writer.writeStringLn( "Minimal WM intensity thresshold  :\t" + m_minimalWMPixValueSlider.getValue( ) );
            writer.writeStringLn( "Maximal WM intensity thresshold  :\t" + m_maximalWMPixValueSlider.getValue( ) );
            writer.writeStringLn( "Minimal CSF intensity thresshold :\t" + m_minimalCSFPixValueSlider.getValue( ) );
            writer.writeStringLn( "Maximal CSF intensity thresshold :\t" + m_maximalCSFPixValueSlider.getValue( ) );
            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "Volume of WM :\t" + m_volumeOfWMPixelsValueLabel.getText( ) );
            writer.writeStringLn( "Volume of GM :\t" +  m_volumeOfGMPixelsValueLabel.getText( ) );
            writer.writeStringLn( "Volume of CSF:\t" +  m_volumeOfCSFPixelsValueLabel.getText( ) );
            float totalSpecVolume = ( new Float( m_volumeOfGMPixelsValueLabel.getText( ) ) + new Float( m_volumeOfWMPixelsValueLabel.getText( ) ) + new Float( m_volumeOfCSFPixelsValueLabel.getText( ) ) );
            writer.writeStringLn( "Total SVS Volume:\t" + totalSpecVolume );
            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "%Fraction WM :\t" + ( new Float( m_volumeOfWMPixelsValueLabel.getText( ) ) * 100f / totalSpecVolume ) );
            writer.writeStringLn( "%Fraction GM :\t" + ( new Float( m_volumeOfGMPixelsValueLabel.getText( ) ) * 100f / totalSpecVolume ) );
            writer.writeStringLn( "%Fraction CSF:\t" + ( new Float( m_volumeOfCSFPixelsValueLabel.getText( ) ) * 100f / totalSpecVolume ) );
            writer.writeStringLn( "===========================================" );
            writer.writeStringLn( "Report created by : " + System.getProperty( "user.name" ) + " on " + formatter.format( date ) );
            ////////////////////////////////////////////////////////////////////
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger(ImageSegmentationWithinSelectedSpectroscopicRegionFrame.class.getName()).log(Level.SEVERE, null, ex);
        } 
        finally 
        {
            try 
            {
                writer.close( );
            } 
            catch ( IOException ex ) 
            {
                Logger.getLogger( ImageSegmentationWithinSelectedSpectroscopicRegionFrame.class.getName( ) ).log( Level.SEVERE,  null, ex );
            }
        }
    }//GEN-LAST:event_m_createReportButtonActionPerformed

    /**
     * Checks for previous defined segmentation pixel intensity threshold 
     * values.
     */
    public void getAndSetTDFDFitDefaultValues( ) 
    {
         // Make this values persistent so that users do not have to define the 
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        if ( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MAXIMAL_PIXEL_VALUE ) != null )
        {
            m_maximalCSFPixValueFormattedTextField.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MAXIMAL_PIXEL_VALUE  ) ) );
            m_maximalCSFPixValueSlider.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MAXIMAL_PIXEL_VALUE  ) ) );
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MINIMAL_PIXEL_VALUE ) != null )
        {
            m_minimalCSFPixValueFormattedTextField.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MINIMAL_PIXEL_VALUE  ) ) );
            m_minimalCSFPixValueSlider.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_CSF_MINIMAL_PIXEL_VALUE  ) ) );
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MAXIMAL_PIXEL_VALUE ) != null )
        {
            m_maximalWMPixValueFormattedTextField.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MAXIMAL_PIXEL_VALUE  ) ) );
            m_maximalWMPixValueSlider.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MAXIMAL_PIXEL_VALUE  ) ) );
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MINIMAL_PIXEL_VALUE ) != null )
        {
            m_minimalWMPixValueFormattedTextField.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MINIMAL_PIXEL_VALUE  ) ) );
            m_minimalWMPixValueSlider.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_WM_MINIMAL_PIXEL_VALUE  ) ) );
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MAXIMAL_PIXEL_VALUE ) != null )
        {
            m_maximalGMPixValueFormattedTextField.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MAXIMAL_PIXEL_VALUE  ) ) );
            m_maximalGMPixValueSlider.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MAXIMAL_PIXEL_VALUE  ) ) );
        }
        
        if ( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MINIMAL_PIXEL_VALUE ) != null )
        {
            m_minimalGMPixValueFormattedTextField.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MINIMAL_PIXEL_VALUE  ) ) );
            m_minimalGMPixValueSlider.setValue( new Integer( defs.getProperty( TDFDFittingDefaults.IMAGE_SEGMENTATION_GM_MINIMAL_PIXEL_VALUE  ) ) );
        }
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(ImageSegmentationWithinSelectedSpectroscopicRegionFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(ImageSegmentationWithinSelectedSpectroscopicRegionFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(ImageSegmentationWithinSelectedSpectroscopicRegionFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(ImageSegmentationWithinSelectedSpectroscopicRegionFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new ImageSegmentationWithinSelectedSpectroscopicRegionFrame(null).setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JButton m_CSFColorOverlaySelectionButton;
    private javax.swing.JButton m_GMColorOverlaySelectionButton;
    private javax.swing.JButton m_WMColorOverlaySelectionButton;
    private javax.swing.JButton m_acceptAndQuitButton;
    private javax.swing.JPanel m_buttonPanel;
    private javax.swing.JButton m_createReportButton;
    private javax.swing.JLabel m_maxPixValExcitedVolumeLabel;
    private javax.swing.JLabel m_maxPixValLabel;
    private javax.swing.JFormattedTextField m_maximalCSFPixValueFormattedTextField;
    private javax.swing.JLabel m_maximalCSFPixValueLabel;
    private javax.swing.JLabel m_maximalCSFPixValueLabel1;
    private javax.swing.JSlider m_maximalCSFPixValueSlider;
    private javax.swing.JFormattedTextField m_maximalGMPixValueFormattedTextField;
    private javax.swing.JLabel m_maximalGMPixValueLabel;
    private javax.swing.JSlider m_maximalGMPixValueSlider;
    private javax.swing.JFormattedTextField m_maximalWMPixValueFormattedTextField;
    private javax.swing.JLabel m_maximalWMPixValueLabel;
    private javax.swing.JSlider m_maximalWMPixValueSlider;
    private javax.swing.JLabel m_minPixValExcitedVolumeLabel;
    private javax.swing.JLabel m_minPixValLabel;
    private javax.swing.JFormattedTextField m_minimalCSFPixValueFormattedTextField;
    private javax.swing.JLabel m_minimalCSFPixValueLabel;
    private javax.swing.JSlider m_minimalCSFPixValueSlider;
    private javax.swing.JFormattedTextField m_minimalGMPixValueFormattedTextField;
    private javax.swing.JLabel m_minimalGMPixValueLabel;
    private javax.swing.JSlider m_minimalGMPixValueSlider;
    private javax.swing.JFormattedTextField m_minimalWMPixValueFormattedTextField;
    private javax.swing.JLabel m_minimalWMPixValueLabel;
    private javax.swing.JSlider m_minimalWMPixValueSlider;
    private javax.swing.JLabel m_numberOfCSFPixelsLabel;
    private javax.swing.JLabel m_numberOfCSFPixelsValueLabel;
    private javax.swing.JLabel m_numberOfGMPixelsLabel;
    private javax.swing.JLabel m_numberOfGMPixelsValueLabel;
    private javax.swing.JLabel m_numberOfTotalPixelsLabel;
    private javax.swing.JLabel m_numberOfTotalPixelsValueLabel;
    private javax.swing.JLabel m_numberOfWMPixelsLabel;
    private javax.swing.JLabel m_numberOfWMPixelsValueLabel;
    private javax.swing.JButton m_quitButton;
    private javax.swing.JPanel m_segmentationValueDefinitionPanel;
    private javax.swing.JComboBox<String> m_tissueTypePixelIntensityOrderingComboBox;
    private javax.swing.JSlider m_transparancyOverlayPixValueSlider;
    private javax.swing.JFormattedTextField m_transparencyValueFormattedTextField;
    private javax.swing.JLabel m_volumeOfCSFPixelsLabel;
    private javax.swing.JLabel m_volumeOfCSFPixelsValueLabel;
    private javax.swing.JLabel m_volumeOfGMPixelsLabel;
    private javax.swing.JLabel m_volumeOfGMPixelsValueLabel;
    private javax.swing.JLabel m_volumeOfTotalPixelsLabel;
    private javax.swing.JLabel m_volumeOfTotalPixelsValueLabel;
    private javax.swing.JLabel m_volumeOfWMPixelsValueLabel;
    private javax.swing.JLabel m_volumerOfWMPixelsLabel;
    // End of variables declaration//GEN-END:variables

   
    
}
