
package org.spectrim.hlsvd;

import org.spectrim.tdfdfit.TDFDFittingDefaults;
import org.spectrim.utilities.ClientSocket;
import java.io.IOException;
import java.util.HashMap;


/**
 * This class enables the spectral estimation by HLSVDPro, calls the 
 * @author Ron de Beer and Johannes Slotboom (c) 2018
 */


public class Compute_HLSVDPro_batch 
{
	
    private int len = 6;
    private int batch_size;
    // private ReadWriteMRSFilesRon readwritefiles;
    
    
    double[] signal_re = new double[1024];
    double[] signal_im = new double[1024];
    
    private double[] ampl;
    private double[] freq;
    private double[] damp;
    private double[] phase;

    private double[] recon_re = new double[1024]; 
    private double[] recon_im = new double[1024]; 
    private double[] filtered_re = new double[1024]; 
    private double[] filtered_im = new double[1024]; 
    private String   hlsvdFilterRange = "";
    
    private int      ndp;
    private double   step;
    private double   larmor;
    private int      numberOfResonanceLines;
    private String   signal_name;
    private int      HankelMatrixSize;
    // Attribute to access the results of the HLSVD
    private HLSVDResults_batch m_HLSVDResults;
    private String FilterMode;
    private Double ExtremeBroadLimit;
    
    static 
    {   
        TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance();
        defaults.load( );
        
        String libCallFortranPathAndFileName   = defaults.getProperty( TDFDFittingDefaults.CALL_FORTRAN_PATH_AND_DLL_NAME );
//        System.load( libCallFortranPathAndFileName  );
        
        
        String libHLSVDPathAndFileName   = defaults.getProperty( TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME );
        System.load( libHLSVDPathAndFileName  );

        String libHLSVDPathAndFileName2   = defaults.getProperty( TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME );
        System.load( libHLSVDPathAndFileName2  );

        
//        System.load( "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits\\callfortran_hlsvd_arr.dll" );
//        System.load( "C:\\Users\\johan\\Documents\\development\\java\\Netbeans\\spectrIm-QMRS\\spectrIm\\native\\target_hlsvd_native_64bits\\libhlsvd.dll" );
    }

    /* 
     * Constructor Compute_Native_HLSVD. 
     */
    public Compute_HLSVDPro_batch( HashMap<String, String> pars, double[] signal_re, double[] signal_im ) 
    {

        numberOfResonanceLines = Integer.parseInt(pars.get("hlsvdComponents"));

        signal_name = "";
        ndp = Integer.parseInt(pars.get("ndp"));
        step   = Double.parseDouble(pars.get("step"));
        larmor = Double.parseDouble(pars.get("larmor"));

        this.HankelMatrixSize = Integer.parseInt(pars.get("hankelSize"));
        this.FilterMode = pars.get("hlsvdFilterMode");
        this.ExtremeBroadLimit = Double.parseDouble( pars.get("broadComponent") );
        this.batch_size = Integer.parseInt(pars.get("batchSize"));
        this.hlsvdFilterRange = pars.get("hlsvdRangeFrequencies");
                
        // First to do: give arrays the correct length
        this.signal_re = new double[ndp*this.batch_size];
        this.signal_im = new double[ndp*this.batch_size];

        this.ampl = new double[numberOfResonanceLines*this.batch_size];
        this.freq = new double[numberOfResonanceLines*this.batch_size];
        this.damp = new double[numberOfResonanceLines*this.batch_size];
        this.phase = new double[numberOfResonanceLines*this.batch_size];

        this.recon_re   = new double[ndp*this.batch_size]; 
        this.recon_im   = new double[ndp*this.batch_size]; 
        this.filtered_re = new double[ndp*this.batch_size]; 
        this.filtered_im = new double[ndp*this.batch_size]; 

        // generate a new test signal
        recon_re = new double[ndp*this.batch_size];
        recon_im = new double[ndp*this.batch_size];
        filtered_re = new double[ndp*this.batch_size];
        filtered_im = new double[ndp*this.batch_size];

        // Complex conjugate must be taken! otherwise positive 
        // and negative frequencies are wrong
        for ( int i = 0; i < ndp*this.batch_size; i++ )
        {
            this.signal_re[i] = signal_re[i];
            this.signal_im[i] = signal_im[i];
        }
        
        for( int i = 0; i < numberOfResonanceLines*this.batch_size; i++ )
        {
            ampl[i]     = 0.0;
            freq[i]     = 0.0;
            damp[i]     = 0.0;
            phase[i]    = 0.0;
        }
                
    }
    
    
    void computeHLSVDPro_batch()
    {
        ClientSocket socket;
        try {
            socket = new ClientSocket(33356);
        } catch (IOException e) {
            socket = null;
            System.out.println( "ERROR connecting to socket" );
        }
        if(socket!=null){
            try{
                socket.write_str("DATA");

                socket.write_str(hlsvdFilterRange);
                socket.write_doubleArray(signal_re);
                socket.write_doubleArray(signal_im);
                socket.writeInt(ndp);
                socket.writeInt(numberOfResonanceLines);
                socket.writeDouble(step);
                socket.writeInt(HankelMatrixSize);
                socket.writeInt(batch_size);
                socket.write_str("PROCESS");
                if ( FilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES ) == 0 ){
                    socket.write_str("WATER+BROAD");
                    socket.writeDouble(ExtremeBroadLimit);
                }
                else if(FilterMode.compareTo( TDFDFittingDefaults.HLSVD_FILTER_EXTREME_BROAD_LINES_ONLY ) == 0 ){
                    socket.write_str("BROAD");
                    socket.writeDouble(ExtremeBroadLimit);
                }
                else{
                    socket.write_str("WATER");
                }

                freq = socket.readDoubleArray(numberOfResonanceLines*batch_size);
                damp = socket.readDoubleArray(numberOfResonanceLines*batch_size);
                ampl = socket.readDoubleArray(numberOfResonanceLines*batch_size);
                phase = socket.readDoubleArray(numberOfResonanceLines*batch_size);
                filtered_re = socket.readDoubleArray(ndp*batch_size);
                filtered_im = socket.readDoubleArray(ndp*batch_size);
                socket.write_str("FINISHED");
                socket.closeConnection();
            } catch (IOException e){
                socket.closeConnection();
                socket = null;
            }
        }

        m_HLSVDResults = new HLSVDResults_batch( ndp, numberOfResonanceLines, batch_size );
        m_HLSVDResults.setSignalRealPart( signal_re );
        m_HLSVDResults.setSignalImaginaryPart( signal_im );
        m_HLSVDResults.setSignalName( signal_name );
        m_HLSVDResults.setNumberOfDataPoints( ndp );
        m_HLSVDResults.setNumberOfResonlineLines( numberOfResonanceLines );
        m_HLSVDResults.setStepSize( step );
        m_HLSVDResults.setLarmorFrequency( larmor );
        m_HLSVDResults.setAmplitudesHLSVD( ampl );
        m_HLSVDResults.setFrequenciesHLSVD( freq );
        m_HLSVDResults.setDampingsHLSVD( damp );
        m_HLSVDResults.setPhasesHLSVD( phase );
        m_HLSVDResults.setReconstructedSignalRealPart( recon_re );
        m_HLSVDResults.setReconstructedSignalImagenaryPart( recon_im );
        m_HLSVDResults.setFilteredSignalRealPart( filtered_re );
        m_HLSVDResults.setFilteredSignalImagenaryPart( filtered_im );
    }

    /**
     * Returns the results of the HLSVD call.
     * @return 
     */
    public HLSVDResults_batch getHLSVDResults( )
    {
        return this.m_HLSVDResults;
    }
    
}
