/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.hlsvd;

import java.io.File;
import java.text.DecimalFormat;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import org.spectrim.tdfdfit.MRSignalProperties;
import org.spectrim.tdfdfit.TDFDFittingDefaults;

/**
 * Graphical interface to set HLSVD fitting defaults.
 * @author johan
 */
public class HLSVDDefaultsDefinerFrame extends javax.swing.JFrame {

    /**
     * Creates new form HLSVDDefaultsDefinerFrame.
     */
    public HLSVDDefaultsDefinerFrame( ) 
    {
        initComponents( );
        
        DecimalFormat formatter = new DecimalFormat( "#0.000E0" );
        
        TDFDFittingDefaults defaults        = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String tdSampleStepSize             = "" + formatter.format( 1.0/MRSignalProperties.getInstance( ).getAcquistitionBandwidth( ) );
        
        String larmorFrequency              = "" + MRSignalProperties.getInstance( ).getResonanceFrequency( );
        String numberOfPointsInHLSVD        = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS );
        String hankelMatrixSize             = defaults.getProperty( TDFDFittingDefaults.HLSVD_HANKEL_MATRIX_SIZE );
        String numberOfHLSVDComponents      = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS );
        String hlsvdMinimumFilterFrequency  = defaults.getProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY );
        String hlsvdMaximumFilterFrequency  = defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY );
        String hlsvdRangesFilterFrequency   = defaults.getProperty( TDFDFittingDefaults.HLSVD_RANGES_FILTER_FREQUENCY );
        String extremeLineWidth             = defaults.getProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD );
        String filterMethod                 = defaults.getProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE );
        String hlsvdPPMReferenceValue       = defaults.getProperty( TDFDFittingDefaults.HLSVD_PPM_REFERENCE_VALUE );
        String libHLSVDArrPathAndFileName   = defaults.getProperty( TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME );
        String callFortranHLSVDArrPathAndFileName = defaults.getProperty( TDFDFittingDefaults.CALL_FORTRAN_PATH_AND_DLL_NAME );
        String batch_size                   = defaults.getProperty( TDFDFittingDefaults.HLSVDPRO_BATCH_SIZE );
        String HLSVDPRO_version             = defaults.getProperty( TDFDFittingDefaults.HLSVDPRO_VERSION );
        String PythonExec                   = defaults.getProperty( TDFDFittingDefaults.PYTHON_EXECUTABLE_FILENAME );
        
        // Set the values
        m_sampleTimeTextField.setText( tdSampleStepSize );
        m_larmorFrequencyTextField.setText( larmorFrequency );
        m_hlsvdTextField.setText( numberOfPointsInHLSVD );
        m_hankelMatrixSizeTextField.setText( hankelMatrixSize );
        m_hlsvdSinusoidsTextField.setText( numberOfHLSVDComponents );
        m_minimumFilterFrequencyTextField.setText( hlsvdMinimumFilterFrequency );
        m_maximumFilterFrequencyTextField.setText( hlsvdMaximumFilterFrequency );
        m_rangesFilterFrequencyTextField.setText( hlsvdRangesFilterFrequency );
        m_setExtremeLineWidthThressholdTextField.setText( extremeLineWidth );
        m_referenceFrequencyPPMTextField.setText( hlsvdPPMReferenceValue );
        m_batch_size_value_TextField.setText( batch_size );
        
        if ( libHLSVDArrPathAndFileName == null )
        {
            m_libhlsvd_path_and_filename_Label.setText( "<html><font color='red'>Please define the location of this library file</font></html>" );
        }
        else
        {
            m_libhlsvd_path_and_filename_Label.setText( libHLSVDArrPathAndFileName );
        }
        
        if ( callFortranHLSVDArrPathAndFileName == null )
        {
            m_callfortran_hlsvd_dll_path_and_filename_Label.setText( "<html><font color='red'>Please define the location of this library file</font></html>" );
        }
        else
        {
            m_callfortran_hlsvd_dll_path_and_filename_Label.setText( callFortranHLSVDArrPathAndFileName );
        }
        
        if ( filterMethod.compareTo( TDFDFittingDefaults.HLSVD_FILTER_WATER_ONLY ) == 0 )
        {
            this.m_eliminateWaterOnlyRadioButton.setSelected( true );
        }
        
        if ( filterMethod.compareTo( TDFDFittingDefaults.HLSVD_FILTER_EXTREME_BROAD_LINES_ONLY ) == 0 )
        {
            this.m_filterOutExtremeBroadComponentsOnlyRadioButton.setSelected( true );
        }
        
        if ( filterMethod.compareTo( TDFDFittingDefaults.HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES ) == 0 )
        {
            this.m_filterOutWaterAndExtremeBroadComponentsRadioButton.setSelected( true );
        }
        if ( HLSVDPRO_version!=null && HLSVDPRO_version.compareTo( TDFDFittingDefaults.HLSVDPRO_VERSION_PARALLEL ) == 0 )
        {
            this.m_use_parallel_HLSVDPro_RadioButton.setSelected( true );
        }
        if ( PythonExec == null )
        {
            m_python_path_and_filename.setText( "<html><font color='red'>Please define the location of the Python executable</font></html>" );
        }
        else
        {
            m_python_path_and_filename.setText( PythonExec );
        }
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        setVisible( true );
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();
        m_libhlsvd_path_and_filename_Label1 = new javax.swing.JLabel();
        m_callfortran_hlsvd_dll_path_and_filename_Label1 = new javax.swing.JLabel();
        m_numberOfTDDatapointsLabel = new javax.swing.JLabel();
        m_numberOfTDDataPointsTextField = new javax.swing.JTextField();
        m_InfoLabel_1 = new javax.swing.JLabel();
        m_stepSizeLabel = new javax.swing.JLabel();
        m_sampleTimeTextField = new javax.swing.JTextField();
        m_InfoLabe_l2 = new javax.swing.JLabel();
        m_larmorFrequencyLabel = new javax.swing.JLabel();
        m_larmorFrequencyTextField = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        m_hlsvdPointsLabel = new javax.swing.JLabel();
        m_hlsvdTextField = new javax.swing.JTextField();
        m_infoLabel_4 = new javax.swing.JLabel();
        m_hankelMatrixLabel = new javax.swing.JLabel();
        m_hankelMatrixSizeTextField = new javax.swing.JTextField();
        m_infoLabel_5 = new javax.swing.JLabel();
        m_hlsvdSinusoidsLabel = new javax.swing.JLabel();
        m_hlsvdSinusoidsTextField = new javax.swing.JTextField();
        m_infoLabel_6 = new javax.swing.JLabel();
        m_minimumFilterFrequencyLabel = new javax.swing.JLabel();
        m_minimumFilterFrequencyTextField = new javax.swing.JTextField();
        m_infoLabel_7 = new javax.swing.JLabel();
        m_minimumFilterFrequencyLabel1 = new javax.swing.JLabel();
        m_maximumFilterFrequencyTextField = new javax.swing.JTextField();
        m_infoLabel_8 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        m_applyAndQuitButton = new javax.swing.JButton();
        m_eliminateWaterOnlyRadioButton = new javax.swing.JRadioButton();
        m_filterOutExtremeBroadComponentsOnlyRadioButton = new javax.swing.JRadioButton();
        m_filterOutWaterAndExtremeBroadComponentsRadioButton = new javax.swing.JRadioButton();
        m_infoLabel_9 = new javax.swing.JLabel();
        m_setExtremeLineWidthThressholdTextField = new javax.swing.JTextField();
        m_minimumFilterFrequencyLabel2 = new javax.swing.JLabel();
        m_defineCallFortranHLSVDArrButton = new javax.swing.JButton();
        m_callfortran_hlsvd_dll_path_and_filename_Label = new javax.swing.JLabel();
        m_defineLibHLSVDArrButton = new javax.swing.JButton();
        m_libhlsvd_path_and_filename_Label = new javax.swing.JLabel();
        m_infoLabel_10 = new javax.swing.JLabel();
        m_referenceFrequencyPPMTextField = new javax.swing.JTextField();
        m_minimumFilterFrequencyLabel3 = new javax.swing.JLabel();
        m_definePythonExecLocationButton = new javax.swing.JButton();
        m_python_path_and_filename = new javax.swing.JLabel();
        m_use_parallel_HLSVDPro_RadioButton = new javax.swing.JRadioButton();
        m_batch_size_value_TextField = new javax.swing.JTextField();
        m_callfortran_hlsvd_dll_path_and_filename_Label2 = new javax.swing.JLabel();
        m_python_path_and_filename1 = new javax.swing.JLabel();
        m_batch_site_text = new javax.swing.JLabel();
        m_minimumFilterFrequencyLabel4 = new javax.swing.JLabel();
        m_rangesFilterFrequencyTextField = new javax.swing.JTextField();
        m_infoLabel_11 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();

        m_libhlsvd_path_and_filename_Label1.setText("installation path and dll/so filename");

        m_callfortran_hlsvd_dll_path_and_filename_Label1.setText("installation path and dll/so filename");

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        m_numberOfTDDatapointsLabel.setText("Number of datapoints");

        m_numberOfTDDataPointsTextField.setText("1024");

        m_InfoLabel_1.setText("(overwritten in case signal has more data points)");

        m_stepSizeLabel.setText("Step size");

        m_sampleTimeTextField.setText("0.2");
        m_sampleTimeTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_sampleTimeTextFieldActionPerformed(evt);
            }
        });

        m_InfoLabe_l2.setText("[msec] (overwritten by true value)");

        m_larmorFrequencyLabel.setText("Larmor frequency");

        m_larmorFrequencyTextField.setText("126");

        jLabel2.setText("[MHz] (overwritten by true value)");

        m_hlsvdPointsLabel.setText("Points in HLSVD");

        m_hlsvdTextField.setText("1024");

        m_infoLabel_4.setText("(value taken if smaller/equal to number of datapoints)");

        m_hankelMatrixLabel.setText("Rows of Hankel matrix");

        m_hankelMatrixSizeTextField.setText("300");

        m_infoLabel_5.setText("(value taken if smaller than number of datapoints)");

        m_hlsvdSinusoidsLabel.setText("Components HLSVD");

        m_hlsvdSinusoidsTextField.setText("30");

        m_infoLabel_6.setText("(number of damped sinusoids)");

        m_minimumFilterFrequencyLabel.setText("Minimum filter frequency");

        m_minimumFilterFrequencyTextField.setText("4.40");

        m_infoLabel_7.setText("(minimum filter frequency in ppm)");

        m_minimumFilterFrequencyLabel1.setText("Maximum filter frequency");

        m_maximumFilterFrequencyTextField.setText("7.5");
        m_maximumFilterFrequencyTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_maximumFilterFrequencyTextFieldActionPerformed(evt);
            }
        });

        m_infoLabel_8.setText("(maximum filter frequency in ppm)");

        m_applyAndQuitButton.setText("Apply and Quit");
        m_applyAndQuitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_applyAndQuitButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(282, 282, 282)
                .addComponent(m_applyAndQuitButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_applyAndQuitButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        buttonGroup1.add(m_eliminateWaterOnlyRadioButton);
        m_eliminateWaterOnlyRadioButton.setSelected(true);
        m_eliminateWaterOnlyRadioButton.setText("Filter out residual water only");
        m_eliminateWaterOnlyRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_eliminateWaterOnlyRadioButtonActionPerformed(evt);
            }
        });

        buttonGroup1.add(m_filterOutExtremeBroadComponentsOnlyRadioButton);
        m_filterOutExtremeBroadComponentsOnlyRadioButton.setText("Filter out extreme broad components");
        m_filterOutExtremeBroadComponentsOnlyRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_filterOutExtremeBroadComponentsOnlyRadioButtonActionPerformed(evt);
            }
        });

        buttonGroup1.add(m_filterOutWaterAndExtremeBroadComponentsRadioButton);
        m_filterOutWaterAndExtremeBroadComponentsRadioButton.setText("Filter out residual water and extreme broad components");
        m_filterOutWaterAndExtremeBroadComponentsRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_filterOutWaterAndExtremeBroadComponentsRadioButtonActionPerformed(evt);
            }
        });

        m_infoLabel_9.setText("(broad lines are filtered out if option selected below)");

        m_setExtremeLineWidthThressholdTextField.setText("30");
        m_setExtremeLineWidthThressholdTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_setExtremeLineWidthThressholdTextFieldActionPerformed(evt);
            }
        });

        m_minimumFilterFrequencyLabel2.setText("Broad line width limit");

        m_defineCallFortranHLSVDArrButton.setText(" Define callfortran_hlsvd_arr.dll /.so");
        m_defineCallFortranHLSVDArrButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_defineCallFortranHLSVDArrButtonActionPerformed(evt);
            }
        });

        m_callfortran_hlsvd_dll_path_and_filename_Label.setText("installation path and dll/so filename");

        m_defineLibHLSVDArrButton.setText(" Define libhlsvd.dll /.so");
        m_defineLibHLSVDArrButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_defineLibHLSVDArrButtonActionPerformed(evt);
            }
        });

        m_libhlsvd_path_and_filename_Label.setText("installation path and dll/so filename");

        m_infoLabel_10.setText("(center of spectrum in ppm)");

        m_referenceFrequencyPPMTextField.setText("4.70");

        m_minimumFilterFrequencyLabel3.setText("Reference frequency ppm");

        m_definePythonExecLocationButton.setText(" Define Python exec");
        m_definePythonExecLocationButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_definePythonExecLocationButtonActionPerformed(evt);
            }
        });

        m_python_path_and_filename.setText("Python executable path and filename");

        m_use_parallel_HLSVDPro_RadioButton.setText("Use parallel HLSVDPro version");
        m_use_parallel_HLSVDPro_RadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_use_parallel_HLSVDPro_RadioButtonActionPerformed(evt);
            }
        });

        m_batch_site_text.setText("Batch processing size");

        m_minimumFilterFrequencyLabel4.setText("Filter frequency ranges");

        m_rangesFilterFrequencyTextField.setText("4.4#5.4");
        m_rangesFilterFrequencyTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_rangesFilterFrequencyTextFieldActionPerformed(evt);
            }
        });

        m_infoLabel_11.setText("(Frequency range(s) to filter in ppm)");

        jLabel1.setText("Format: \";\" as separator between ranges. \"#\" as separator between the limits in a range.");

        jLabel3.setText("Examples: \"4.4#5.0\" (filter from 4.4 to 5.0), \"2#3.1;-4#-1\" filter from 2 to 3 and -4 to -1");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(m_eliminateWaterOnlyRadioButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_use_parallel_HLSVDPro_RadioButton)
                        .addGap(124, 124, 124))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_filterOutWaterAndExtremeBroadComponentsRadioButton)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(m_filterOutExtremeBroadComponentsOnlyRadioButton)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_batch_size_value_TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 73, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(m_stepSizeLabel)
                            .addComponent(m_larmorFrequencyLabel)
                            .addComponent(m_hlsvdPointsLabel)
                            .addComponent(m_hankelMatrixLabel)
                            .addComponent(m_minimumFilterFrequencyLabel2)
                            .addComponent(m_minimumFilterFrequencyLabel1)
                            .addComponent(m_minimumFilterFrequencyLabel)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                .addComponent(m_definePythonExecLocationButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(m_defineCallFortranHLSVDArrButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 214, Short.MAX_VALUE)
                                .addComponent(m_defineLibHLSVDArrButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                .addComponent(m_python_path_and_filename1)
                                .addGap(118, 118, 118))
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(m_callfortran_hlsvd_dll_path_and_filename_Label2, javax.swing.GroupLayout.PREFERRED_SIZE, 147, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(m_batch_site_text))
                                .addGap(42, 42, 42))))
                    .addGroup(layout.createSequentialGroup()
                        .addGap(143, 143, 143)
                        .addComponent(m_rangesFilterFrequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(m_infoLabel_11, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addGap(42, 42, 42))
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel3)
                                    .addComponent(jLabel1))
                                .addContainerGap(71, Short.MAX_VALUE))))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(m_hlsvdSinusoidsLabel)
                                    .addComponent(m_numberOfTDDatapointsLabel))
                                .addGap(39, 39, 39)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addComponent(m_numberOfTDDataPointsTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 75, Short.MAX_VALUE)
                                    .addComponent(m_hlsvdTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 75, Short.MAX_VALUE)
                                    .addComponent(m_larmorFrequencyTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 75, Short.MAX_VALUE)
                                    .addComponent(m_sampleTimeTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 75, Short.MAX_VALUE)
                                    .addComponent(m_hlsvdSinusoidsTextField)
                                    .addComponent(m_hankelMatrixSizeTextField))
                                .addGap(18, 18, 18)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(m_infoLabel_5)
                                    .addComponent(m_infoLabel_6)
                                    .addComponent(m_infoLabel_4)
                                    .addComponent(jLabel2)
                                    .addComponent(m_InfoLabe_l2)
                                    .addComponent(m_InfoLabel_1)))
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(m_minimumFilterFrequencyLabel3)
                                .addGap(18, 18, 18)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(m_maximumFilterFrequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addComponent(m_infoLabel_8))
                                    .addGroup(layout.createSequentialGroup()
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                            .addComponent(m_referenceFrequencyPPMTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 75, Short.MAX_VALUE)
                                            .addComponent(m_minimumFilterFrequencyTextField))
                                        .addGap(18, 18, 18)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addComponent(m_infoLabel_7)
                                            .addComponent(m_infoLabel_10)))
                                    .addGroup(layout.createSequentialGroup()
                                        .addComponent(m_setExtremeLineWidthThressholdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addGap(18, 18, 18)
                                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addComponent(m_callfortran_hlsvd_dll_path_and_filename_Label)
                                            .addComponent(m_infoLabel_9)
                                            .addComponent(m_libhlsvd_path_and_filename_Label)
                                            .addComponent(m_python_path_and_filename)))))
                            .addComponent(m_minimumFilterFrequencyLabel4, javax.swing.GroupLayout.PREFERRED_SIZE, 125, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfTDDatapointsLabel)
                    .addComponent(m_numberOfTDDataPointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_InfoLabel_1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_stepSizeLabel)
                    .addComponent(m_sampleTimeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_InfoLabe_l2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_larmorFrequencyLabel)
                    .addComponent(m_larmorFrequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_hlsvdPointsLabel)
                    .addComponent(m_hlsvdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_infoLabel_4))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_hankelMatrixSizeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_hankelMatrixLabel)
                    .addComponent(m_infoLabel_5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_hlsvdSinusoidsLabel)
                    .addComponent(m_hlsvdSinusoidsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_infoLabel_6))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_referenceFrequencyPPMTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_infoLabel_10)
                    .addComponent(m_minimumFilterFrequencyLabel3))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_minimumFilterFrequencyLabel)
                    .addComponent(m_minimumFilterFrequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_infoLabel_7))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_minimumFilterFrequencyLabel1)
                    .addComponent(m_maximumFilterFrequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_infoLabel_8))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_minimumFilterFrequencyLabel4)
                    .addComponent(m_rangesFilterFrequencyTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_infoLabel_11))
                .addGap(3, 3, 3)
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 14, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(15, 15, 15)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_minimumFilterFrequencyLabel2)
                    .addComponent(m_setExtremeLineWidthThressholdTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_infoLabel_9))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_defineCallFortranHLSVDArrButton)
                    .addComponent(m_callfortran_hlsvd_dll_path_and_filename_Label))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_defineLibHLSVDArrButton)
                    .addComponent(m_libhlsvd_path_and_filename_Label))
                .addGap(5, 5, 5)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_definePythonExecLocationButton)
                    .addComponent(m_python_path_and_filename))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_eliminateWaterOnlyRadioButton)
                    .addComponent(m_use_parallel_HLSVDPro_RadioButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_filterOutExtremeBroadComponentsOnlyRadioButton)
                    .addComponent(m_batch_size_value_TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_callfortran_hlsvd_dll_path_and_filename_Label2)
                    .addComponent(m_python_path_and_filename1)
                    .addComponent(m_batch_site_text))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_filterOutWaterAndExtremeBroadComponentsRadioButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_sampleTimeTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_sampleTimeTextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_sampleTimeTextFieldActionPerformed

    private void m_applyAndQuitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_applyAndQuitButtonActionPerformed
        
        TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
//        String numberOfPointsInHLSVD       = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS );
//        String hankelMatrixSize            = defaults.getProperty( TDFDFittingDefaults.HLSVD_HANKEL_MATRIX_SIZE );
//        String numberOfHLSVDComponents     = defaults.getProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS );
//        String hlsvdMinimumFilterFrequency = defaults.getProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY );
//        String hlsvdMaximumFilterFrequency = defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY );
//        String hlsvdPPMReferenceFrequency  = defaults.getProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY );
         
        defaults.setProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_DATAPOINTS, m_hlsvdTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVD_HANKEL_MATRIX_SIZE, m_hankelMatrixSizeTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVD_NUMBER_OF_SINUSOIDS, m_hlsvdSinusoidsTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVD_MINUMUM_FILTER_FREQUENCY, m_minimumFilterFrequencyTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVD_MAXIMUM_FILTER_FREQUENCY, m_maximumFilterFrequencyTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVD_RANGES_FILTER_FREQUENCY, m_rangesFilterFrequencyTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVD_EXTREME_LINE_WIDTH_THRESSHOLD, m_setExtremeLineWidthThressholdTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVD_PPM_REFERENCE_VALUE , m_referenceFrequencyPPMTextField.getText( ) );
        defaults.setProperty( TDFDFittingDefaults.HLSVDPRO_BATCH_SIZE , m_batch_size_value_TextField.getText( ) );
        defaults.save( );
        dispose( );
    }//GEN-LAST:event_m_applyAndQuitButtonActionPerformed

    private void m_setExtremeLineWidthThressholdTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_setExtremeLineWidthThressholdTextFieldActionPerformed
        if ( m_filterOutWaterAndExtremeBroadComponentsRadioButton.getModel( ).isSelected( ) )
        {
            TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
            defaults.load( );
            
            defaults.setProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE, TDFDFittingDefaults.HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES );
            defaults.save( );
        }  
    }//GEN-LAST:event_m_setExtremeLineWidthThressholdTextFieldActionPerformed

    private void m_eliminateWaterOnlyRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_eliminateWaterOnlyRadioButtonActionPerformed
        
        if ( m_eliminateWaterOnlyRadioButton.getModel( ).isSelected( ) )
        {
            TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
            defaults.load( );
            
            defaults.setProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE, TDFDFittingDefaults.HLSVD_FILTER_WATER_ONLY );
            defaults.save( );
        }    
            
    }//GEN-LAST:event_m_eliminateWaterOnlyRadioButtonActionPerformed

    private void m_filterOutExtremeBroadComponentsOnlyRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_filterOutExtremeBroadComponentsOnlyRadioButtonActionPerformed
        if ( m_filterOutExtremeBroadComponentsOnlyRadioButton.getModel( ).isSelected( ) )
        {
            TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
            defaults.load( );
            
            defaults.setProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE, TDFDFittingDefaults.HLSVD_FILTER_EXTREME_BROAD_LINES_ONLY );
            defaults.save( );
        }  
    }//GEN-LAST:event_m_filterOutExtremeBroadComponentsOnlyRadioButtonActionPerformed

    private void m_filterOutWaterAndExtremeBroadComponentsRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_filterOutWaterAndExtremeBroadComponentsRadioButtonActionPerformed
        
        if ( m_filterOutWaterAndExtremeBroadComponentsRadioButton.getModel( ).isSelected( ) )
        {
            TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
            defaults.load( );
            
            defaults.setProperty( TDFDFittingDefaults.HLSVD_FILTER_TYPE, TDFDFittingDefaults.HLSVD_FILTER_WATER_AND_EXTREME_BROAD_LINES );
            defaults.save( );
                      
        }
    }//GEN-LAST:event_m_filterOutWaterAndExtremeBroadComponentsRadioButtonActionPerformed

    private void m_defineCallFortranHLSVDArrButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_defineCallFortranHLSVDArrButtonActionPerformed
        TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String defaultFileChooserPathAndFileName = defaults.getProperty( TDFDFittingDefaults.CALL_FORTRAN_PATH_AND_DLL_NAME );
        File defaultFileChooserPathAndFile;
        
        if ( defaultFileChooserPathAndFileName != null )
        {
            defaultFileChooserPathAndFile = new File( defaultFileChooserPathAndFileName );
        }
        else
        {
            defaultFileChooserPathAndFile = new File( System.getProperty( "user.home" ) );
        }        
        
        String fileDefaultName = "call_fortran_arr.dll";
        
        if ( defaultFileChooserPathAndFile.isFile( ) )
        {
            defaultFileChooserPathAndFile = defaultFileChooserPathAndFile.getParentFile( );
            fileDefaultName = defaultFileChooserPathAndFile.getName( );
        }
        
        String buttonText = "Select";
        String callFortranHLSVDArrPathAndFileName = getFilePathAndName( defaultFileChooserPathAndFile.getAbsolutePath( ), fileDefaultName, buttonText );
        if ( callFortranHLSVDArrPathAndFileName != null )
        {
            defaults.setProperty( TDFDFittingDefaults.CALL_FORTRAN_PATH_AND_DLL_NAME, callFortranHLSVDArrPathAndFileName );
            defaults.save( );
        }
        
        m_callfortran_hlsvd_dll_path_and_filename_Label.setText( callFortranHLSVDArrPathAndFileName );
        
    }//GEN-LAST:event_m_defineCallFortranHLSVDArrButtonActionPerformed

    private void m_defineLibHLSVDArrButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_defineLibHLSVDArrButtonActionPerformed
        
        TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String defaultFileChooserPathAndFileName   = defaults.getProperty( TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME );
        File defaultFileChooserPathAndFile;
        
        if ( defaultFileChooserPathAndFileName != null )
        {
            defaultFileChooserPathAndFile = new File( defaultFileChooserPathAndFileName );
        }
        else
        {
            defaultFileChooserPathAndFile = new File( System.getProperty( "user.home" ) );
        }
        
        String fileDefaultName = "libhlsvd.dll";
        
        if ( defaultFileChooserPathAndFile.isFile( ) )
        {
            defaultFileChooserPathAndFile = defaultFileChooserPathAndFile.getParentFile( );
            fileDefaultName = defaultFileChooserPathAndFile.getName( );
        }
        
        String buttonText = "Select";
        String libHLSVDArrPathAndFileName = getFilePathAndName( defaultFileChooserPathAndFile.getAbsolutePath( ), fileDefaultName, buttonText );
        if ( libHLSVDArrPathAndFileName != null )
        {
            defaults.setProperty( TDFDFittingDefaults.LIB_HLSVD_PATH_AND_DLL_NAME, libHLSVDArrPathAndFileName );
            defaults.save( );
        }
        
        m_libhlsvd_path_and_filename_Label.setText( libHLSVDArrPathAndFileName );
    }//GEN-LAST:event_m_defineLibHLSVDArrButtonActionPerformed

    private void m_definePythonExecLocationButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_definePythonExecLocationButtonActionPerformed
        // TODO add your handling code here:
        TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
        defaults.load( );
        
        String defaultFileChooserPathAndFileName   = defaults.getProperty( TDFDFittingDefaults.PYTHON_EXECUTABLE_FILENAME );
        File defaultFileChooserPathAndFile;
        
        if ( defaultFileChooserPathAndFileName != null )
        {
            defaultFileChooserPathAndFile = new File( defaultFileChooserPathAndFileName );
        }
        else
        {
            defaultFileChooserPathAndFile = new File( System.getProperty( "user.home" ) );
        }
        
        String fileDefaultName = "";
        
        if ( defaultFileChooserPathAndFile.isFile( ) )
        {
            defaultFileChooserPathAndFile = defaultFileChooserPathAndFile.getParentFile( );
            fileDefaultName = defaultFileChooserPathAndFile.getName( );
        }
        
        String buttonText = "Select";
        String libHLSVDArrPathAndFileName = getFilePathAndName( defaultFileChooserPathAndFile.getAbsolutePath( ), fileDefaultName, buttonText );
        
        if ( libHLSVDArrPathAndFileName != null )
        {
            defaults.setProperty( TDFDFittingDefaults.PYTHON_EXECUTABLE_FILENAME, libHLSVDArrPathAndFileName );
            defaults.save( );
        }
        
        m_python_path_and_filename.setText( libHLSVDArrPathAndFileName );
    }//GEN-LAST:event_m_definePythonExecLocationButtonActionPerformed

    private void m_use_parallel_HLSVDPro_RadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_use_parallel_HLSVDPro_RadioButtonActionPerformed
        // TODO add your handling code here:
        if ( m_use_parallel_HLSVDPro_RadioButton.getModel( ).isSelected( ) )
        {
            TDFDFittingDefaults defaults    = TDFDFittingDefaults.getInstance( );
            defaults.load( );
            
            String batch_size = m_batch_size_value_TextField.getText();
            if (!"".equals(batch_size)){
                defaults.setProperty( TDFDFittingDefaults.HLSVDPRO_BATCH_SIZE, batch_size);
            }
            defaults.setProperty( TDFDFittingDefaults.HLSVDPRO_VERSION, TDFDFittingDefaults.HLSVDPRO_VERSION_PARALLEL );
            defaults.save( );
        }
    }//GEN-LAST:event_m_use_parallel_HLSVDPro_RadioButtonActionPerformed

    private void m_maximumFilterFrequencyTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_maximumFilterFrequencyTextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_maximumFilterFrequencyTextFieldActionPerformed

    private void m_rangesFilterFrequencyTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_rangesFilterFrequencyTextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_rangesFilterFrequencyTextFieldActionPerformed

    
    
    /**
     * 
     * Returns the filename and directory selected by the user.
     * 
     * @param path
     * @param subDirString
     * @param fileDefaultName
     * @param buttonText
     * @return 
     */
    protected String getFilePathAndName( String path, String fileDefaultName, String buttonText )
    {
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        String outputPath = new String( path + System.getProperty( "file.separator" ) );

        File f = new File( outputPath );
        if ( f.exists( ) == false )
        {
            f.mkdir( );
        }

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        fileChooser.setSelectedFile( new File( fileDefaultName ) );

        result = fileChooser.showDialog( this, buttonText );


        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }
    
    
    /**
     * Main (for testing purposes).
     * @param args the command line arguments.
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(HLSVDDefaultsDefinerFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(HLSVDDefaultsDefinerFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(HLSVDDefaultsDefinerFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(HLSVDDefaultsDefinerFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new HLSVDDefaultsDefinerFrame().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel m_InfoLabe_l2;
    private javax.swing.JLabel m_InfoLabel_1;
    private javax.swing.JButton m_applyAndQuitButton;
    private javax.swing.JLabel m_batch_site_text;
    private javax.swing.JTextField m_batch_size_value_TextField;
    private javax.swing.JLabel m_callfortran_hlsvd_dll_path_and_filename_Label;
    private javax.swing.JLabel m_callfortran_hlsvd_dll_path_and_filename_Label1;
    private javax.swing.JLabel m_callfortran_hlsvd_dll_path_and_filename_Label2;
    private javax.swing.JButton m_defineCallFortranHLSVDArrButton;
    private javax.swing.JButton m_defineLibHLSVDArrButton;
    private javax.swing.JButton m_definePythonExecLocationButton;
    private javax.swing.JRadioButton m_eliminateWaterOnlyRadioButton;
    private javax.swing.JRadioButton m_filterOutExtremeBroadComponentsOnlyRadioButton;
    private javax.swing.JRadioButton m_filterOutWaterAndExtremeBroadComponentsRadioButton;
    private javax.swing.JLabel m_hankelMatrixLabel;
    private javax.swing.JTextField m_hankelMatrixSizeTextField;
    private javax.swing.JLabel m_hlsvdPointsLabel;
    private javax.swing.JLabel m_hlsvdSinusoidsLabel;
    private javax.swing.JTextField m_hlsvdSinusoidsTextField;
    private javax.swing.JTextField m_hlsvdTextField;
    private javax.swing.JLabel m_infoLabel_10;
    private javax.swing.JLabel m_infoLabel_11;
    private javax.swing.JLabel m_infoLabel_4;
    private javax.swing.JLabel m_infoLabel_5;
    private javax.swing.JLabel m_infoLabel_6;
    private javax.swing.JLabel m_infoLabel_7;
    private javax.swing.JLabel m_infoLabel_8;
    private javax.swing.JLabel m_infoLabel_9;
    private javax.swing.JLabel m_larmorFrequencyLabel;
    private javax.swing.JTextField m_larmorFrequencyTextField;
    private javax.swing.JLabel m_libhlsvd_path_and_filename_Label;
    private javax.swing.JLabel m_libhlsvd_path_and_filename_Label1;
    private javax.swing.JTextField m_maximumFilterFrequencyTextField;
    private javax.swing.JLabel m_minimumFilterFrequencyLabel;
    private javax.swing.JLabel m_minimumFilterFrequencyLabel1;
    private javax.swing.JLabel m_minimumFilterFrequencyLabel2;
    private javax.swing.JLabel m_minimumFilterFrequencyLabel3;
    private javax.swing.JLabel m_minimumFilterFrequencyLabel4;
    private javax.swing.JTextField m_minimumFilterFrequencyTextField;
    private javax.swing.JTextField m_numberOfTDDataPointsTextField;
    private javax.swing.JLabel m_numberOfTDDatapointsLabel;
    private javax.swing.JLabel m_python_path_and_filename;
    private javax.swing.JLabel m_python_path_and_filename1;
    private javax.swing.JTextField m_rangesFilterFrequencyTextField;
    private javax.swing.JTextField m_referenceFrequencyPPMTextField;
    private javax.swing.JTextField m_sampleTimeTextField;
    private javax.swing.JTextField m_setExtremeLineWidthThressholdTextField;
    private javax.swing.JLabel m_stepSizeLabel;
    private javax.swing.JRadioButton m_use_parallel_HLSVDPro_RadioButton;
    // End of variables declaration//GEN-END:variables
}
