package org.spectrim.hlsvd;

import java.io.*;


/**
 * Adapted class by for use in SpectrIm 64 bits.
 * @author Ron de Beer <rondebeer1945@gmail.com>;
 */

public class ReadWriteMRSFiles 
{

    private double dreturn[];

    public void ReadWriteMRSFiles() {}

    public double[] readMruiFile (String file_name) 
    {

        try
        {
            int ndp, nr_signals;
            double dummy, filendp, filestep, filelarmor;

            FileInputStream fileinputstream = new FileInputStream(file_name);
            DataInputStream datainputstream = new DataInputStream(fileinputstream);
            dummy      = datainputstream.readDouble();	// 0
            filendp    = datainputstream.readDouble();	// 1
            filestep   = datainputstream.readDouble();	// 2
            dummy      = datainputstream.readDouble();	// 3
            dummy      = datainputstream.readDouble();	// 4
            filelarmor = datainputstream.readDouble();	// 5

            ndp = (int) filendp;			
            dreturn = new double [2*ndp + 3];

            for (int i = 0; i < 64 - 6; i++) 
            {
                dummy = datainputstream.readDouble();	// 64 doubles in fileheader
                if ( i == 7 ) 
                {					
                    nr_signals = (int) dummy;
                    if ( nr_signals == 0 ) 
                    {
                        nr_signals = 1;	 // At least one signal per file
                    }
                    System.out.println( "nr_signals in file = " + nr_signals );
                }				
            }
            for ( int i = 0; i < ndp; i++ ) 
            {
                dreturn[i]     = datainputstream.readDouble();
                dreturn[i+ndp] = datainputstream.readDouble();
            }

            dreturn [2*ndp]   = filendp;
            dreturn [2*ndp+1] = filestep;
            dreturn [2*ndp+2] = filelarmor;

            datainputstream.close();
        }
        catch (Exception e) {}

        return dreturn;
    }

    public void writeMruiFile ( String file_name, int ndp, double step, 
                                double larmor, double[][] datapoints ) 
    {

        try 
        {
            double dndp = (double) ndp;
            double[][] data = new double[2][ndp];

            for ( int i = 0; i < ndp; i++ ) 
            {
                data[0][i] = datapoints[0][i];
                data[1][i] = datapoints[1][i];
            }

            FileOutputStream fileoutputstream = new FileOutputStream(file_name);
            DataOutputStream dataoutputstream = new DataOutputStream(fileoutputstream);
            dataoutputstream.writeDouble(0.0);					// 0
            dataoutputstream.writeDouble(dndp);					// 1
            dataoutputstream.writeDouble(step);					// 2
            dataoutputstream.writeDouble(0.0);					// 3
            dataoutputstream.writeDouble(0.0);					// 4
            dataoutputstream.writeDouble(larmor);				// 5
            for (int i = 0; i < 64 - 6; i++) 
            {
                dataoutputstream.writeDouble(0.0);	// 64 doubles in fileheader
            }
            for (int i = 0; i < ndp; i++) 
            {
                dataoutputstream.writeDouble(data[0][i]);
                dataoutputstream.writeDouble(data[1][i]);
            }
            dataoutputstream.close();
        }
        catch (Exception e) {}	
    }

    public void writeBatchMruiFile( String file_name, int ndp, double step, 
                                    double larmor, int nr_signals, 
                                    double[][] datapoints ) 
    {

        try 
        {
            double dndp = (double) ndp;
            double dnr_signals = (double) nr_signals;

            double[][] data = new double[2][nr_signals*ndp];

            for (int i = 0; i < nr_signals*ndp; i++) 
            {
                data[0][i] = datapoints[0][i];
                data[1][i] = datapoints[1][i];
            }

            FileOutputStream fileoutputstream = new FileOutputStream(file_name);
            DataOutputStream dataoutputstream = new DataOutputStream(fileoutputstream);
            dataoutputstream.writeDouble(0.0);					// 0
            dataoutputstream.writeDouble(dndp);					// 1
            dataoutputstream.writeDouble(step);					// 2
            dataoutputstream.writeDouble(0.0);					// 3
            dataoutputstream.writeDouble(0.0);					// 4
            dataoutputstream.writeDouble(larmor);				// 5

            for (int i = 0; i < 64 - 6; i++) 
            {	// 64 doubles in fileheader
                if (i == 7) 
                {
                    dataoutputstream.writeDouble(dnr_signals);
                }
                else 
                {
                    dataoutputstream.writeDouble(0.0);
                }
            }

            for (int i = 0; i < nr_signals*ndp; i++) 
            {
                dataoutputstream.writeDouble(data[0][i]);
                dataoutputstream.writeDouble(data[1][i]);
            }
            dataoutputstream.close();
        }
        catch (Exception e) {}	
    }	

    public double[] readTextFile( String file_name ) 
    {

        try 
        {
            FileReader fr = new FileReader(file_name);
            BufferedReader br = new BufferedReader(fr);

            int ndp = 0;
            while (br.readLine() != null) 
            {
                ndp++;
            }

            fr.close();

            FileReader filereader = new FileReader(file_name);
            BufferedReader bufreader = new BufferedReader(filereader);

            dreturn = new double [2*ndp + 1];

            String datapoint;
            String[] splitted;
            for (int i = 0; i < ndp; i++) 
            {
                datapoint  = bufreader.readLine();
                splitted = datapoint.split("\\s+");				
                dreturn[i]     =  Double.parseDouble(splitted[1]);
                dreturn[i+ndp] =  Double.parseDouble(splitted[2]);
            }

            double dndp = (double) ndp;
            dreturn [2*ndp] = dndp;

            filereader.close();
        }
        catch (Exception e) {}	

        return dreturn;
    }

    public void writeTextFile (String file_name, int ndp, double[][] datapoints) 
    {

        try
        {			
            double[][] data = new double[2][ndp];

            for (int i = 0; i < ndp; i++) 
            {
                data[0][i] = datapoints[0][i];
                data[1][i] = datapoints[1][i];
            }

            FileWriter filewriter = new FileWriter(file_name);

            String datapoint;
            for (int i = 0; i < ndp; i++) 
            {
                datapoint = String.valueOf(data[0][i]) + "\t\t" + String.valueOf(data[1][i]) + "\n";
                filewriter.write(datapoint, 0, datapoint.length());
            }

            filewriter.close();
        }
        catch (Exception e) {}
    }

    public void writeTextFile ( String file_name, int ndp, 
                                float[] real, float[] imag ) 
    {

        try
        {			
            double[][] data = new double[2][ndp];

            for (int i = 0; i < ndp; i++) 
            {
                data[0][i] = (double)real[i];
                data[1][i] = (double)imag[i];
            }

            FileWriter filewriter = new FileWriter(file_name);

            String datapoint;
            for (int i = 0; i < ndp; i++) 
            {
                datapoint = String.valueOf(data[0][i]) + "\t\t" + String.valueOf(data[1][i]) + "\n";
                filewriter.write(datapoint, 0, datapoint.length());
            }

            filewriter.close();
        }
        catch (Exception e) {}
    }


    public double[] readOneColumnTextFile (String file_name) 
    {

        try 
        {
            FileReader fr = new FileReader(file_name);
            BufferedReader br = new BufferedReader(fr);

            int ndp = 0;
            while (br.readLine() != null) {
                    ndp++;
            }
            ndp = ndp/2;

            fr.close();

            FileReader filereader = new FileReader(file_name);
            BufferedReader bufreader = new BufferedReader(filereader);

            dreturn = new double [2*ndp + 1];

            String datapoint;
            for (int i = 0; i < 2*ndp; i++) 
            {
                datapoint  = bufreader.readLine();
                dreturn[i] =  Double.parseDouble(datapoint);
            }

            double dndp = (double) ndp;
            dreturn [2*ndp] = dndp;

            filereader.close();
        }
        catch (Exception e) {}	

        return dreturn;
    }

    public void writeOneColumnTextFile ( String file_name, int ndp, 
                                         double[][] datapoints) 
    {

        try
        {			
            double[][] data = new double[2][ndp];

            for (int i = 0; i < ndp; i++) 
            {
                data[0][i] = datapoints[0][i];
                data[1][i] = datapoints[1][i];
            }

            FileWriter filewriter = new FileWriter(file_name);

            String datapoint;
            for (int i = 0; i < ndp; i++) 
            {
                datapoint = String.valueOf(data[0][i]) + "\n";
                filewriter.write(datapoint, 0, datapoint.length());
            }
            for (int i = 0; i < ndp; i++) 
            {
                datapoint = String.valueOf(data[1][i]) + "\n";
                filewriter.write(datapoint, 0, datapoint.length());
            }
            filewriter.close();
        }
        catch (Exception e) {}
    }

    public double[] readReImTextFile( String file_name ) 
    {

        file_name = file_name.substring(0, file_name.length() - 7);

        try 
        {		
            FileReader fr = new FileReader(file_name + "_re.txt");
            BufferedReader br = new BufferedReader(fr);

            int ndp = 0;
            while (br.readLine() != null) 
            {
                ndp++;
            }

            fr.close();

            FileReader filereader_re = new FileReader(file_name + "_re.txt");
            BufferedReader bufreader_re = new BufferedReader(filereader_re);

            dreturn = new double [2*ndp + 1];

            String datapoint;
            for (int i = 0; i < ndp; i++) 
            {
                datapoint  = bufreader_re.readLine();
                dreturn[i] =  Double.parseDouble(datapoint);
            }

            filereader_re.close();

            FileReader filereader_im = new FileReader(file_name + "_im.txt");
            BufferedReader bufreader_im = new BufferedReader(filereader_im);

            for (int i = 0; i < ndp; i++) 
            {
                datapoint      = bufreader_im.readLine();
                dreturn[i+ndp] =  Double.parseDouble(datapoint);
            }

            double dndp = (double) ndp;			
            dreturn [2*ndp] = dndp;

            filereader_im.close();			
        }
        catch (Exception e) {}

        return dreturn;
    }

    public void writeReImTextFile( String file_name, int ndp, 
                                   double[][] datapoints ) 
    {

        try
        {			
            double[][] data = new double[2][ndp];

            for (int i = 0; i < ndp; i++) 
            {
                data[0][i] = datapoints[0][i];
                data[1][i] = datapoints[1][i];
            }

            FileWriter filewriter_re = new FileWriter(file_name + "_re.txt");

            String datapoint;
            for (int i = 0; i < ndp; i++) 
            {
                datapoint = String.valueOf(data[0][i]) + "\n";
                filewriter_re.write(datapoint, 0, datapoint.length());
            }

            filewriter_re.close();

            FileWriter filewriter_im = new FileWriter(file_name + "_im.txt");

            for (int i = 0; i < ndp; i++) 
            {
                datapoint = String.valueOf(data[1][i]) + "\n";
                filewriter_im.write(datapoint, 0, datapoint.length());
            }

            filewriter_im.close();

        }
        catch (Exception e) {}
    }

}
