      subroutine hlsvd(signal,ndp,Lrow,mcoL,kuser,kfit,Lsinval,
     +                     ampl,fase,damp,freq)  
c
c                  List of subroutine packages called
c     lanczos.f
c     hlsubrs.f
c     Lapack/blas 
c
c                  LIST OF IMPORTANT VARIABLES
c                  ===========================
c
c     Input from java:
c     ===============
c     signal  = complex*16 nmr signal (data)
c     ndp     = number of datapoints
c     Lrow    = number of rows of hankel data matrix hx, Lrow < ndp/2
c     mcoL    = number of columns of hx, mcoL = ndp+1-Lrow > ndp/2
c     kuser   = number of sinusoids initially wished by user 
c
c     Output to java:
c     ==============
c     sinvals = Lrow singular values of hankel data matrix 
c              (Lrow < mcoL)
c     ampl    = kfitted amplitudes, estimated by htls
c     fase    = kfitted phases, estimated by htls
c     damp    = kfitted dampings, estimated by htls
c     freq    = kfitted frequencies, estimated by htls
c     kfit    = number of sinusoids actually fitted: identical to
c               number of sinvals found by Lanczos = ndiv
c
c     Other variables: 
c     =============== 
c     x       = complex*16 work-vector, initially for the signal
c     yr,yi   = real*8 real and imag parts of signal   
c     hx      = Lmax*(2*kmax) work-matrix for ztls and zgeev
c     lsinval = Lanczos sinvals of data-matrix
c     sinval  = work-vector for Lapack subroutines
c     sinvec  = Lmax*kmax left Lanczos sinvec matrix of data-matrix,
c               and, later on, right Lapack sinvec matrix of hx
c     vvt     = (2*kfit)*(2*kfit) right sinvecs of hx in ztls
c     root    = kfit complex-valued eigvals of Z' 
c     zeta    = Lrow*kfit Vandermonde matrix 
c     kmax    = max number of sinusoids
c     Lmax    = max number of rows  < ndpmax/2
c     Lrow    = number of rows
c     ndpmax  = max number of data points
c     coef    = coefficient for optimal size of working arrays
c               for Lapack
c
c     Declarations of all variables
c     =============================
c
      implicit none  ! All variables have to be declared.
c
      integer          id1,id2,id3,id5,id6,id7,kmax,Lmax,ndpmax
      integer          coef,Lcwork,Lrwork                     ! Lapack
crdb      parameter       (id1=4096,id2=id1/2,id3=50,id5=id2/2)
      parameter       (id1=4096,id2=id1/2,id3=100,id5=id2/2)
      parameter       (id6=2*id3,id7=id5+3)
      parameter       (ndpmax=id2,Lmax=id5,kmax=id3)
      parameter       (coef=1)                                ! Lapack 
      parameter       (Lcwork=9*Lmax*coef,Lrwork=5*Lmax*coef) ! Lapack
c
      integer          i,j,kuser,kfit,Lrow,mcoL,ndp
      integer          info,rank                              ! Lapack
c     integer          ipiv(kmax)                             ! Lapack
      integer          iord,ndiv,nit,nitf                     ! Lanczos
c
      real*8           yr(ndpmax),yi(ndpmax)                  ! Lanczos
      real*8           absr,absi,scale,scalr,scali            ! Lanczos
      real*8           vr(id7,id3),vi(id7,id3)                ! Lanczos
      complex*16       u(id3,id3),unit(id3,id3)               ! Lanczos
      complex*16       zprime(id3,id3)              !lanczos and Lapack
      
      real*8           sinval(2*kmax),Lsinval(*)
      real*8           ampl(*),damp(*),freq(*),fase(*)
      real*8           pi
      real*8           rwork(Lrwork)                          ! Lapack
c
      complex*16       signal(*),x(ndpmax)
      complex*16       vc(Lmax,2*kmax)
      complex*16       cwork(Lcwork)                          ! Lapack
      complex*16       root(kmax)                       !eigvals of Z'
      complex*16       zeta(ndpmax,kmax)                !vandermonde
c
c     ******************************************************************
c
c     Calculation of sinvals & sinvecs of the hankel data matrix.
c     ==========================================================
c
c     Copy complex*16 signal to real*8 yr and yi, and to complex*16 x.
c     ---------------------------------------------------------------
c
      do 10 i = 1,ndp
         x(i) = signal(i)       ! x corrupted in amplitude computation
        yr(i) = sngl(dble(signal(i))) ! see g77.info,  Node: CMPAMBIG
   10   yi(i) = sngl(dimag(signal(i)))
c
c     Scaling of yr and yi
c     --------------------
         scalr = 0.
         scali = 0.
      do 20  i = 1,ndp
         absr  = abs(yr(i))
         absi  = abs(yi(i))
         scalr = max(absr,scalr)
         scali = max(absi,scali)
   20 continue
         scale = max(scalr,scali)
      do 30  i = 1,ndp
         yr(i) = yr(i) / scale
         yi(i) = yi(i) / scale
   30 continue
c
c     ******************************************************************
c
c     Lanczos SVD of hankel datamatrix.
c     ================================
c
c      print *,'Call Lanczos'
c
crdb      nit = 50
      nit = 100
c
      call lanczo(yr,yi,ndp,mcoL,kuser,nit,id7,vr,vi,lsinval,ndiv,nitf)
	  
c
      kfit = kuser                     ! kuser was provided by the user      
      if (ndiv .lt. kuser) kfit = ndiv ! ndiv actually found by Lanczos
      write (*,'(''ndiv ='',I5,''kuser ='',I5)') kfit,kuser
c
c     Undoing of scaling effect on sinvals. Also, division by ndp!
c     ------------------------------------  ---------------------
      do 35    i = 1,kfit
      lsinval(i) = scale * lsinval(i)/dble(ndp)
   35 continue
crdb   
      print *,'Begin sinvals from lanczos'
      do 37 i = 1, 50
        j = (i-1)*4
        if (j.GT.kfit-4) goto 1234
        write (*,'(i4, e14.4, i4, e14.4, i4, e14.4, i4, e14.4)')
     +  1+j, lsinval(1+j), 2+j, lsinval(2+j),
     +  3+j, lsinval(3+j), 4+j, lsinval(4+j)
   37 continue
1234  continue
      print *,'End sinvals from lanczos'
crdb          
c
c     Store real*8 vr and vi in complex*16 sinvec
c     -------------------------------------------
      do 40     i = 1, Lrow
      do 45     j = 1, kfit
          vc(i,j) = dcmplx(vr(i,j),vi(i,j))
 45   continue
 40   continue
c
      iord=1
      write(*,'(/,'' Lanczos ended'',/)')
c
c     ******************************************************************
c
c     Calculation of the Z' matrix, with ...
c     ======================================
c     print *,'Computing matrix Z-prime with ...'
c
      call Zcalc(kfit,kmax,Lmax,Lrow,vc,zprime,u,unit)
c
c     print *,'Computing matrix Z-prime with TLS'
c
c     call ztls(Lmax,Lrow,kmax,kfit,sinval,sinvec,hx,Lcwork,
c    +           cwork,rwork,v12,v22,vvt,ipiv)
c
c     ******************************************************************
c
c     Diagonalization of Z' (=hx), yielding the the 'signal'poles',
c     ===========================================================
c     also called roots.
c     =================
c
c     zgeev - procedure, which calculates eigvals and eigvecs
c     of a general matrix - only eigvals are used and computed
c     into variable root
c 
c      print *,'Call LAPACK - zgeev'
      call zgeev('N','N',kfit,zprime,kmax,root,vc,Lmax,vc,Lmax,
     +           cwork,Lcwork,rwork,info)
c      print *,'Lapack info from  zgeev: ',info
c 
c     ******************************************************************
c
c     Calculation of dampings (damp) and frequencies (freq) from roots
c 
      pi         = 4.0d0*datan2(1.0d0,1.0d0)    
      do 77    i = 1,kfit
         damp(i) = dlog(cdabs(root(i)))
         freq(i) = datan2(dimag(root(i)),dble(root(i))) / (2.0d0*pi)   
 77   continue                  !10-8-99 dreal -> dble
c
c     ******************************************************************
c
c     Calculation of complex-valued amplitudes , using the 
c     pseudoinverse of the Lrow*kfit Vandermonde matrix zeta.
c
c     First calculation of zeta:
c
      call vanmon(ndpmax,ndp,kmax,kfit,root,zeta)
c
c      print *,'Call LAPACK - zgelss'
c     zgells writes solution in space of vector x,
c     but the vector 'signal' is preserved. 
c     5-8-99: rcond was -1.0; g77 makes rank = 0!! Not with SUN.
c             So I set rcond = +1.0d-10!!!!
c
      call zgelss(ndp,kfit,1,zeta,ndpmax,x,ndpmax,
     +            sinval,1.0d-10,rank,cwork,Lcwork,rwork,info)
c
crdb
      print *,'Begin sinval from zgelss'
      do 79 i = 1, 50
        j = (i-1)*4
        if (j.GT.kfit-4) goto 4321
        write (*,'(i4, e14.4, i4, e14.4, i4, e14.4, i4, e14.4)')
     +  1+j, sinval(1+j), 2+j, sinval(2+j),
     +  3+j, sinval(3+j), 4+j, sinval(4+j)
   79 continue
4321  continue      
      print *,'End sinval from zgelss'
crdb
      write (*,'(''Lapack info from zgelss '',i3)') info
      write (*,'(''Rank of function-matrix: '',i3)') rank
crdb      
      if (rank.LT.kfit) write (*,'(''Rank less than kfit!'')')
crdb
      do  88  i = 1,kfit
      ampl(i) = cdabs (x(i))
   88 fase(i) = datan2(dimag(x(i)),dble(x(i))) ! 10-8-99 dreal -> dble
c
c  
c     ******************************************************************
c
      return 
      end
c 
c     &&&&&&&&&&&&&&&  end of subroutine hltls  &&&&&&&&&&&&&&&&&&

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c=======================================================================
      subroutine Zcalc(kfit,kmax,Lmax,Lrow,vc,zprime,u,unit)
c
c     31-08-1999: Calculation of Z-matrix moved to separate subroutine.
c
      implicit none
c
      integer    i,j,k,kfit,kmax,Lmax,Lrow,m
c
      real*8     uot,zero
c
      complex*16 cero,sum,temp
      complex*16 vc(Lmax,*),u(kmax,*),unit(kmax,*),zprime(kmax,*)
c
      zero = 0.0d0
      cero = dcmplx(zero,zero)
      m    = Lrow
c
      do 60       i = 1,kfit
         do 60    j = 1,kfit
                sum = cero
            do 50 k = 1,m-1
                sum = sum+ conjg(vc(k,i))*vc(k+1,j)
c!              sum = sum+dconjg(vc(k,i))*vc(k+1,j)
   50 continue
             u(i,j) = sum
   60 continue
c
          sum = cero
      do 70 i = 1,kfit
c         sum = sum+ conjg(vc(m,i))*vc(m,i)
          sum = sum+dconjg(vc(m,i))*vc(m,i)
   70 continue
          uot = 1.0d0-dble (sum)
c
      do 80         i = 1,kfit
         do 80      j = 1,kfit
            temp      = dcmplx(0.d0,0.d0)
                              if (j.eq.i) 
     +      temp      = dcmplx(1.d0,0.d0)
c           unit(i,j) = temp +  conjg(vc(m,i))*vc(m,j)/uot
            unit(i,j) = temp + dconjg(vc(m,i))*vc(m,j)/uot
   80 continue
c
      do 100      i = 1,kfit
         do 100   j = 1,kfit
                sum = cero
            do 90 k = 1,kfit
                sum = sum + unit(i,k)*u(k,j)
   90       continue
        zprime(i,j) = sum
  100 continue
c
      return
      end     

