/*
 * File: $Id: MirrorAxisPositioningControl.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 05.04.2004, 11:22 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 -2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * MirrorAxisPositioningControl.java
 *
 * Created on March 5, 2004, 11:59 AM
 */

package org.spectrim.imagegraphicobjects;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Point;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import java.util.LinkedList;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JLabel;

/**
 *
 * Allows the interactive setting of the mirror axis.
 *
 * @author  hans
 */
public class MirrorAxisPositioningControl extends JFrame implements ChangeListener, ActionListener
{
    
    /** 
     * Creates a new instance of MirrorAxisPositioningControl 
     */
    public MirrorAxisPositioningControl( String title, JFrame parent, int width, int height )  
    {
        m_parent = parent;
        
        m_imageWidth = width;
        m_imageHeight = height;
        
        setTitle( title );
       
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        // set up the panel
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        m_slidersPanel = new JPanel();
        m_slidersPanel.setLayout( new BorderLayout( ) );
        
        instantiateMirrorAxisAnglePanel( );
        
        m_contentPanel.add( m_slidersPanel, BorderLayout.NORTH );
        
        instantiateCancelApplyPanel( );
        instantiateMirrorAxisHorizontalPositionPanel( );
        instantiateMirrorAxisVerticalPositionPanel( );
        
        setContentPane( m_contentPanel );   
        pack( );
        
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = m_parent.getLocation( );
        Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
        setVisible( true );
    }
    
    /**
     * Instantiates the panel containing the controls that allows to change the
     * angle of the mirror axis.
     */
    private void instantiateMirrorAxisAnglePanel( )
    {
        m_mirrorAxisAnglePanel = new JPanel( );
        m_mirrorAxisAnglePanel.setLayout( new BoxLayout( m_mirrorAxisAnglePanel, BoxLayout.Y_AXIS ) );
        m_mirrorAxisAnglePanel.setBorder( BorderFactory.createCompoundBorder(
                                           BorderFactory.createEmptyBorder( 10, 10, 10, 10 ),
                                           BorderFactory.createTitledBorder( "Angle" ) )
                                          );
        
        m_mirrorAxisAnglePanel.setPreferredSize( new Dimension( 430, 130 ) );
        m_mirrorAxisAnglePanel.setMaximumSize( new Dimension( 430, 130 ) );
        m_mirrorAxisAnglePanel.setMinimumSize( new Dimension( 430, 130 ) );
               
        m_mirrorAxisAngleSliderPanel = new JPanel( );
        m_mirrorAxisAngleSliderPanel.setLayout( new BoxLayout( m_mirrorAxisAngleSliderPanel, BoxLayout.X_AXIS ) ); 
        
        m_mirrorAxisAngleSliderLabel = new JLabel( "Mirror axis angle" );
        m_mirrorAxisAngleSliderPanel.add( m_mirrorAxisAngleSliderLabel );
        
        m_mirrorAxisAngleSliderPanel.add( Box.createHorizontalGlue( ) );
        
        m_mirrorAxisAngleSlider = new JSlider( JSlider.HORIZONTAL, -90, 90, 0 );
        m_mirrorAxisAngleSlider.setMajorTickSpacing( 90 );
        m_mirrorAxisAngleSlider.setMinorTickSpacing( 10 );
        m_mirrorAxisAngleSlider.setPaintTicks( true );
        m_mirrorAxisAngleSlider.setPaintLabels( true );
        m_mirrorAxisAngleSlider.addChangeListener( this );
        m_mirrorAxisAngleSlider.setMinimumSize( m_mirrorAxisAngleSlider.getPreferredSize( ) );
        
        m_mirrorAxisAngleSlider.setMinimumSize( new Dimension( 250, 50 ) );
        m_mirrorAxisAngleSlider.setMaximumSize( new Dimension( 250, 50 ) );
        m_mirrorAxisAngleSlider.setPreferredSize( new Dimension( 250, 50 ) );
        
        m_mirrorAxisAngleSliderPanel.add( m_mirrorAxisAngleSlider );
        
        m_mirrorAxisAnglePanel.add( m_mirrorAxisAngleSliderPanel );
       
        m_slidersPanel.add( m_mirrorAxisAnglePanel, BorderLayout.NORTH );
    }
    
    
    /**
     * Instantiates the panel containing the controls that allows to change the
     * horizontal position of the mirror axis.
     */
    private void instantiateMirrorAxisHorizontalPositionPanel( )
    {
        m_mirrorAxisHorizontalPositionPanel = new JPanel( );
        m_mirrorAxisHorizontalPositionPanel.setLayout( new BoxLayout( m_mirrorAxisHorizontalPositionPanel, BoxLayout.Y_AXIS ) );
        m_mirrorAxisHorizontalPositionPanel.setBorder( BorderFactory.createCompoundBorder(
                                                       BorderFactory.createEmptyBorder( 10, 10, 10, 10 ),
                                                       BorderFactory.createTitledBorder( "Horizontal position" ) )
                                                      );
        
        m_mirrorAxisHorizontalPositionPanel.setPreferredSize( new Dimension( 430, 130 ) );
        m_mirrorAxisHorizontalPositionPanel.setMaximumSize( new Dimension( 430, 130 ) );
        m_mirrorAxisHorizontalPositionPanel.setMinimumSize( new Dimension( 430, 130 ) );
               
        m_mirrorAxisHorizontalPositionSliderPanel = new JPanel( );
        m_mirrorAxisHorizontalPositionSliderPanel.setLayout( new BoxLayout( m_mirrorAxisHorizontalPositionSliderPanel, BoxLayout.X_AXIS ) ); 
        
        m_mirrorAxisHorizontalPositionSliderLabel = new JLabel( "Mirror axis horizontal position" );
        m_mirrorAxisHorizontalPositionSliderPanel.add( m_mirrorAxisAngleSliderLabel );
        
        m_mirrorAxisHorizontalPositionSliderPanel.add( Box.createHorizontalGlue( ) );
        
        m_mirrorAxisHorizontalPositionSlider = new JSlider( JSlider.HORIZONTAL, 0, m_imageWidth, m_imageWidth/2 );
        m_mirrorAxisHorizontalPositionSlider.setMajorTickSpacing( 64 );
        m_mirrorAxisHorizontalPositionSlider.setMinorTickSpacing( 16 );
        m_mirrorAxisHorizontalPositionSlider.setPaintTicks( true );
        m_mirrorAxisHorizontalPositionSlider.setPaintLabels( true );
        m_mirrorAxisHorizontalPositionSlider.addChangeListener( this );
        m_mirrorAxisHorizontalPositionSlider.setMinimumSize( m_mirrorAxisHorizontalPositionSlider.getPreferredSize( ) );
        
        m_mirrorAxisHorizontalPositionSlider.setMinimumSize( new Dimension( 250, 50 ) );
        m_mirrorAxisHorizontalPositionSlider.setMaximumSize( new Dimension( 250, 50 ) );
        m_mirrorAxisHorizontalPositionSlider.setPreferredSize( new Dimension( 250, 50 ) );
        
        m_mirrorAxisHorizontalPositionSliderPanel.add( m_mirrorAxisHorizontalPositionSlider );
        
        m_mirrorAxisHorizontalPositionPanel.add( m_mirrorAxisHorizontalPositionSliderPanel );
       
        m_slidersPanel.add( m_mirrorAxisHorizontalPositionPanel, BorderLayout.CENTER );
    }
    

    
    /**
     * Instantiates the panel containing the controls that allows to change the
     * horizontal position of the mirror axis.
     */
    private void instantiateMirrorAxisVerticalPositionPanel( )
    {
        m_mirrorAxisVerticalPositionPanel = new JPanel( );
        m_mirrorAxisVerticalPositionPanel.setLayout( new BoxLayout( m_mirrorAxisVerticalPositionPanel, BoxLayout.Y_AXIS ) );
        m_mirrorAxisVerticalPositionPanel.setBorder( BorderFactory.createCompoundBorder(
                                                     BorderFactory.createEmptyBorder( 10, 10, 10, 10 ),
                                                     BorderFactory.createTitledBorder( "Vertical position" ) )
                                                   );
        
        m_mirrorAxisVerticalPositionPanel.setPreferredSize( new Dimension( 430, 130 ) );
        m_mirrorAxisVerticalPositionPanel.setMaximumSize( new Dimension( 430, 130 ) );
        m_mirrorAxisVerticalPositionPanel.setMinimumSize( new Dimension( 430, 130 ) );
               
        m_mirrorAxisVerticalPositionSliderPanel = new JPanel( );
        m_mirrorAxisVerticalPositionSliderPanel.setLayout( new BoxLayout( m_mirrorAxisVerticalPositionSliderPanel, BoxLayout.X_AXIS ) ); 
        
        m_mirrorAxisVerticalPositionSliderLabel = new JLabel( "Mirror axis vertical position" );
        m_mirrorAxisVerticalPositionSliderPanel.add( m_mirrorAxisVerticalPositionSliderLabel );
        
        m_mirrorAxisVerticalPositionSliderPanel.add( Box.createHorizontalGlue( ) );
        
        m_mirrorAxisVerticalPositionSlider = new JSlider( JSlider.HORIZONTAL, 0, m_imageHeight, m_imageHeight / 2 );
        m_mirrorAxisVerticalPositionSlider.setMajorTickSpacing( 64 );
        m_mirrorAxisVerticalPositionSlider.setMinorTickSpacing( 16 );
        m_mirrorAxisVerticalPositionSlider.setPaintTicks( true );
        m_mirrorAxisVerticalPositionSlider.setPaintLabels( true );
        m_mirrorAxisVerticalPositionSlider.addChangeListener( this );
        m_mirrorAxisVerticalPositionSlider.setMinimumSize( m_mirrorAxisVerticalPositionSlider.getPreferredSize( ) );
        
        m_mirrorAxisVerticalPositionSlider.setMinimumSize( new Dimension( 250, 50 ) );
        m_mirrorAxisVerticalPositionSlider.setMaximumSize( new Dimension( 250, 50 ) );
        m_mirrorAxisVerticalPositionSlider.setPreferredSize( new Dimension( 250, 50 ) );
        
        m_mirrorAxisVerticalPositionSliderPanel.add( m_mirrorAxisVerticalPositionSlider );
        
        m_mirrorAxisVerticalPositionPanel.add( m_mirrorAxisVerticalPositionSliderPanel );
       
        m_slidersPanel.add( m_mirrorAxisVerticalPositionPanel, BorderLayout.SOUTH );
    }
    
    /**
     * Instantiates an apply/ cancel panel.
     */
    private void instantiateCancelApplyPanel( )
    {
        // Selection panel definitions
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_selectionPanel, BorderLayout.SOUTH );
    }
    
    /**
     * Called whenever a registered component raises an actionEvent.
     * @param e
     */
    @Override
     public void actionPerformed( ActionEvent e ) 
    {
       
        Object sender = e.getSource( );
       
       // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
            dispose( );
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
            dispose( );
        }
    } 
    
    /**
     * Called whenever a registered component raises a changeEvent.
     */    
    @Override
    public void stateChanged( ChangeEvent e ) 
    {
        
        Object sender = e.getSource( );
        
        // Handles responses on horizontal slider changed and released event
        if ( sender == m_mirrorAxisAngleSlider )
        {
            if ( !m_mirrorAxisAngleSlider.getValueIsAdjusting( ) )
            {
                Iterator it = m_listeners.iterator( );
                m_deltaAngle = m_mirrorAxisAngleSlider.getValue( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ANGLE_CHANGE_EVENT, "" ) );
                }         
            }
        }        
            
        // Handles responses on horizontal slider changed and released event
        if ( sender == m_mirrorAxisHorizontalPositionSlider )
        {
             if ( !m_mirrorAxisHorizontalPositionSlider.getValueIsAdjusting( ) )
            {
                Iterator it = m_listeners.iterator( );
                m_deltaHorizontalShift = m_mirrorAxisHorizontalPositionSlider.getValue( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, HORIZONTAL_POSITION_CHANGE_EVENT, "" ) );
                }         
            }
        }   
        
        // Handles responses on horizontal slider changed and released event
        if ( sender == m_mirrorAxisVerticalPositionSlider )
        {
             if ( !m_mirrorAxisVerticalPositionSlider.getValueIsAdjusting( ) )
            {
                Iterator it = m_listeners.iterator( );
                m_deltaVerticalShift = m_mirrorAxisVerticalPositionSlider.getValue( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VERTICAL_POSITION_CHANGE_EVENT, "" ) );
                }
            }
        }   
    }
    
    /**
     * Returns the rotation angle of the mirror axis: a vertical axis equals 0 degrees
     * @return 
     **/
    public int getRotationAngle( )
    {
        return m_deltaAngle;
    }

    /**
     * Returns the horizontal translation of the mirror axis
     * @return 
     **/
    public int getHorizontalTranslation( )
    {
        return m_deltaHorizontalShift;
    }

    /**
     * Returns the vertical translation of the mirror axis
     * @return 
     **/
    public int getVerticalTranslation( )
    {
        return m_deltaVerticalShift;
    }
    
    /**
     * Adds an actionlistener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    
    // (private) Attributes
    private JFrame m_parent;
    
    private JPanel m_contentPanel;
    
    private JButton m_applyButton;
    private JButton m_cancelButton;
    private JPanel m_selectionPanel;
    
    private JPanel m_mirrorAxisAnglePanel;
    private JPanel m_mirrorAxisAngleSliderPanel;
    private JLabel m_mirrorAxisAngleSliderLabel;
    private JSlider m_mirrorAxisAngleSlider;
    
    private JPanel m_mirrorAxisHorizontalPositionPanel;
    private JPanel m_mirrorAxisHorizontalPositionSliderPanel;
    private JLabel m_mirrorAxisHorizontalPositionSliderLabel;
    private JSlider m_mirrorAxisHorizontalPositionSlider;
    
    private JPanel m_mirrorAxisVerticalPositionPanel;
    private JPanel m_mirrorAxisVerticalPositionSliderPanel;
    private JLabel m_mirrorAxisVerticalPositionSliderLabel;
    private JSlider m_mirrorAxisVerticalPositionSlider;
    
    private JPanel m_slidersPanel;
    
    // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    
    // events raised by accept and cancel button
    public final static int ACCEPT_EVENT = 0;
    public final static int CANCEL_EVENT = 1;
    
    // events raised by controls on gui
    public final static int ANGLE_CHANGE_EVENT = 2;
    public final static int HORIZONTAL_POSITION_CHANGE_EVENT = 3;
    public final static int VERTICAL_POSITION_CHANGE_EVENT = 4;
    
    private int m_deltaAngle = 0;
    private int m_deltaHorizontalShift = 0;
    private int m_deltaVerticalShift = 0;
    
    // In order to be able to evaluate to min/max
    private int m_imageWidth;
    private int m_imageHeight;
    
    // For proper serialization
    private static final long serialVersionUID = 9982244496669213L;
}
