/*
 * File: $Id: Point3D.java,v 1.2 2006/09/01 09:13:23 hans Exp $
 * History
 * Date: 19.10.2004  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2004 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * Point3D.java
 *
 * Created on October 19, 2004, 3:08 PM
 */
package org.spectrim.imagegraphicobjects;

import java.text.DecimalFormat;

/**
 * 
 * Utility class handling 3D coordinates and points.
 * 
 * @author hans
 */
public class Point3D {

    /**
     * Creates a new instance of Point3D.
     */
    public Point3D( ) 
    {
    }

    /**
     * Creates a new instance of Point3D.
     * @param p
     */
    public Point3D( Point3D p ) 
    {
        m_x1 = p.m_x1;
        m_x2 = p.m_x2;
        m_x3 = p.m_x3;
    }

    /**
     * Creates a new instance of Point3D.
     * @param x1
     * @param x2
     * @param x3
     */
    public Point3D(double x1, double x2, double x3) 
    {
        m_x1 = x1;
        m_x2 = x2;
        m_x3 = x3;
    }

    /**
     * 
     * Creates a new instance of Point3D.
     * 
     * @param x1
     * @param x2
     * @param x3
     */
    public Point3D( int x1, int x2, int x3 ) 
    {
        m_x1 = (double) x1;
        m_x2 = (double) x2;
        m_x3 = (double) x3;
    }

    /**
     * Copies an instance of Point3D.
     * @param p
     */
    public void setPoint( Point3D p ) 
    {
        m_x1 = p.m_x1;
        m_x2 = p.m_x2;
        m_x3 = p.m_x3;
    }
    
    /**
     * 
     * Performs a comparison of two points.
     * 
     * @param p
     * @return 
     */
    public boolean isSame( Point3D p )
    {
        if(
            m_x1 == p.m_x1 &&
            m_x2 == p.m_x2 &&
            m_x3 == p.m_x3
          )
            return true;
        else
            return false;
    }

    /**
     * 
     * Translate a given point about 'trans'.
     * 
     * @param trans
     * @return 
     */
    public Point3D translate( Point3D trans ) 
    {

        m_x1 = m_x1 + trans.m_x1;
        m_x2 = m_x2 + trans.m_x2;
        m_x3 = m_x3 + trans.m_x3;

        return this;
    }

    /**
     * Rotates a point p angle degrees about the X1 axis.
     * @param p
     * @param angle
     * @return 
     */
    public Point3D rotateX1(Point3D p, double angle) 
    {

        m_x1 = p.m_x1;
        m_x2 = p.m_x2 * Math.cos(angle) + p.m_x3 * Math.sin(angle);
        m_x3 = -p.m_x2 * Math.sin(angle) + p.m_x3 * Math.cos(angle);

        return this;
    }

    /**
     * Rotates a point p angle degrees about the X2 axis.
     * @param p
     * @param angle
     * @return 
     */
    public Point3D rotateX2(Point3D p, double angle) {

        m_x1 = p.m_x1 * Math.cos(angle) - p.m_x3 * Math.sin(angle);
        m_x2 = p.m_x2;
        m_x3 = p.m_x1 * Math.sin(angle) + p.m_x3 * Math.cos(angle);

        return this;
    }

    /**
     * Rotates a point p angle degrees about the X3 axis.
     * @param p
     * @param angle
     * @return 
     */
    public Point3D rotateX3(Point3D p, double angle) {

        m_x1 = p.m_x1 * Math.cos(angle) + p.m_x2 * Math.sin(angle);
        m_x2 = -p.m_x1 * Math.sin(angle) + p.m_x2 * Math.cos(angle);
        m_x3 = p.m_x3;
        return this;
    }

    /**
     * Normalizes a vector to a Point3D.
     * @param p
     * @return 
     */
    public static Point3D normalize( Point3D p ) 
    {
        Point3D norm = new Point3D();

        double length = Math.sqrt(p.m_x1 * p.m_x1 + p.m_x2 * p.m_x2 + p.m_x3 * p.m_x3);
        if (Math.abs(length) > 1.e-10) {
            norm.m_x1 = p.m_x1 / length;
            norm.m_x2 = p.m_x2 / length;
            norm.m_x3 = p.m_x3 / length;
        }
        return norm;
    }

    /**
     * Normalizes a vector to a Point3D.
     * @return 
     */
    public Point3D normalize() 
    {
        double length = Math.sqrt(this.m_x1 * this.m_x1 + this.m_x2 * this.m_x2 + this.m_x3 * this.m_x3);
        if (Math.abs(length) > 1.e-10) {
            this.m_x1 = this.m_x1 / length;
            this.m_x2 = this.m_x2 / length;
            this.m_x3 = this.m_x3 / length;
        }
        return this;
    }

    /**
     * Rotates a point p about an axis through the origin about an axis through
     * the origin and rotAxis (normalized) using a different implementation of
     * Rodrigues formula.
     * @param P
     * @param rotAxis
     * @param angle
     * @return 
     */
    public Point3D _rotate3D(Point3D P, Point3D rotAxis, double angle) {

        Point3D normRotAxis = rotAxis.normalize(rotAxis);

        Point3D rotP = new Point3D();

        double[][] Q = new double[3][3];

        double x = normRotAxis.m_x1;
        double y = normRotAxis.m_x2;
        double z = normRotAxis.m_x3;

        Q[0][0] = 1 + (1 - Math.cos(angle)) * (x * x - 1);
        Q[0][1] = -z * Math.sin(angle) + (1 - Math.cos(angle)) * x * y;
        Q[0][2] = y * Math.sin(angle) + (1 - Math.cos(angle)) * x * z;

        Q[1][0] = z * Math.sin(angle) + (1 - Math.cos(angle)) * x * y;
        Q[1][1] = 1 + (1 - Math.cos(angle)) * (y * y - 1);
        Q[1][2] = -x * Math.sin(angle) + (1 - Math.cos(angle)) * y * z;

        Q[2][0] = -y * Math.sin(angle) + (1 - Math.cos(angle)) * x * z;
        Q[2][1] = x * Math.sin(angle) + (1 - Math.cos(angle)) * y * z;
        Q[2][2] = 1 + (1 - Math.cos(angle)) * (z * z - 1);

        rotP.m_x1 = Q[0][0] * P.m_x1 + Q[1][0] * P.m_x2 + Q[2][0] * P.m_x3;
        rotP.m_x2 = Q[0][1] * P.m_x1 + Q[1][1] * P.m_x2 + Q[2][1] * P.m_x3;
        rotP.m_x3 = Q[0][2] * P.m_x1 + Q[1][2] * P.m_x2 + Q[2][2] * P.m_x3;

        return rotP;
    }

    /**
     * Rotates a point p about an axis through the origin about an axis through
     * the origin and rotAxis (normalized) using Rodrigues formula
     */
//    public static Point3D rotate3D( Point3D P, Point3D rotAxis, double angle )
//    {
//
//        Point3D normRotAxis = rotAxis.normalize( rotAxis );
//
//        Point3D rotP = new Point3D( );
//
//        double sn = Math.sin( angle );
//        double cs = 1.0 - Math.cos( angle );
//
//        double[][] A  = new double[3][3];
//        double[][] AA = new double[3][3];
//        double[][] Q  = new double[3][3];
//
//        A[0][0] = 0.0;
//        A[1][0] = -normRotAxis.m_x3 * sn;
//        A[2][0] = normRotAxis.m_x2 * sn;
//
//        A[0][1] = normRotAxis.m_x3 * sn;
//        A[1][1] = 0.0;
//        A[2][1] = -normRotAxis.m_x1 * sn;
//
//        A[0][2] = -normRotAxis.m_x2 * sn;
//        A[1][2] = normRotAxis.m_x1 * sn;
//        A[2][2] = 0.0;
//
//        AA[0][0] = ( - normRotAxis.m_x2 * normRotAxis.m_x2 - normRotAxis.m_x3 * normRotAxis.m_x3 ) * cs;
//        AA[1][0] = ( normRotAxis.m_x1 * normRotAxis.m_x2 ) * cs;
//        AA[2][0] = ( normRotAxis.m_x1 * normRotAxis.m_x3 ) * cs;
//
//        AA[0][1] = ( normRotAxis.m_x1 * normRotAxis.m_x2 ) * cs;
//        AA[1][1] = ( - normRotAxis.m_x1 * normRotAxis.m_x1 - normRotAxis.m_x3 * normRotAxis.m_x3 ) * cs;
//        AA[2][1] = ( normRotAxis.m_x2 * normRotAxis.m_x3 ) * cs;
//
//        AA[0][2] = ( normRotAxis.m_x1 * normRotAxis.m_x3 ) * cs;
//        AA[1][2] = ( normRotAxis.m_x2 * normRotAxis.m_x3 ) * cs;
//        AA[2][2] = ( - normRotAxis.m_x1 * normRotAxis.m_x1 - normRotAxis.m_x2 * normRotAxis.m_x2 ) * cs;
//
//        Q[0][0] = 1.0 + A[0][0] + AA[0][0];
//        Q[1][0] =       A[1][0] + AA[1][0];
//        Q[2][0] =       A[2][0] + AA[2][0];
//
//        Q[0][1] =       A[0][1] + AA[0][1];
//        Q[1][1] = 1.0 + A[1][1] + AA[1][1];
//        Q[2][1] =       A[2][1] + AA[2][1];
//
//        Q[0][2] =       A[0][2] + AA[0][2];
//        Q[1][2] =       A[1][2] + AA[1][2];
//        Q[2][2] = 1.0 + A[2][2] + AA[2][2];
//
//        rotP.m_x1 = Q[0][0] * P.m_x1 + Q[1][0] * P.m_x2 + Q[2][0] * P.m_x3;
//        rotP.m_x2 = Q[0][1] * P.m_x1 + Q[1][1] * P.m_x2 + Q[2][1] * P.m_x3;
//        rotP.m_x3 = Q[0][2] * P.m_x1 + Q[1][2] * P.m_x2 + Q[2][2] * P.m_x3;
//
//        return rotP;
//    }
    
    /**
     * 
     * @param P
     * @param rotAxis
     * @param angle
     * @return 
     */
    public static Point3D rotate3D( Point3D P, Point3D rotAxis, double angle ) 
    {
        Point3D res = new Point3D();
        double dotprod = dotProduct(P, rotAxis);
        double cos = Math.cos(angle);
        double sin = Math.sin(angle);
        res.m_x1 = rotAxis.m_x1 * dotprod * (1 - cos) + P.m_x1 * cos + (rotAxis.m_x2 * P.m_x3 - rotAxis.m_x3 * P.m_x2) * sin;
        res.m_x2 = rotAxis.m_x2 * dotprod * (1 - cos) + P.m_x2 * cos + (rotAxis.m_x3 * P.m_x1 - rotAxis.m_x1 * P.m_x3) * sin;
        res.m_x3 = rotAxis.m_x3 * dotprod * (1 - cos) + P.m_x3 * cos + (rotAxis.m_x1 * P.m_x2 - rotAxis.m_x2 * P.m_x1) * sin;

        return res;
    }

    /**
     * Multiplies every element of the point with the same factor alpha.
     * @param alpha
     */
    public void scalarMultiply(double alpha) 
    {
        m_x1 = alpha * m_x1;
        m_x2 = alpha * m_x2;
        m_x3 = alpha * m_x3;
    }

    /**
     * Multiplies every element of the point with the same factor alpha 'this'
     * is not changed!
     * @param alpha
     * @return 
     */
    public Point3D scalarMultiplication(double alpha) 
    {
        double x1_out = alpha * m_x1;
        double x2_out = alpha * m_x2;
        double x3_out = alpha * m_x3;

        return new Point3D(x1_out, x2_out, x3_out);
    }

     /**
     * Computes a vector perpendicular to both the input vectors a and b.
     * @param a
     * @param b
     * @return 
     */
    public Point3D computeNormalVector( Point3D a, Point3D b )
    {
        Point3D c = new Point3D( );
        
        c.m_x1 = a.m_x2 * b.m_x3 - a.m_x3 * b.m_x2;
        c.m_x2 = a.m_x3 * b.m_x1 - a.m_x1 * b.m_x3;
        c.m_x3 = a.m_x1 * b.m_x2 - a.m_x2 * b.m_x1;
        c = c.normalize( c );
        
        return c;
    }
    
    /**
     * Computes a vector perpendicular to both the input vectors a and b.
     * @param b
     * @return 
     */
    public Point3D computeNormalVector(Point3D b) {
        Point3D c = new Point3D();

        c.m_x1 = this.m_x2 * b.m_x3 - this.m_x3 * b.m_x2;
        c.m_x2 = this.m_x3 * b.m_x1 - this.m_x1 * b.m_x3;
        c.m_x3 = this.m_x1 * b.m_x2 - this.m_x2 * b.m_x1;
        c = c.normalize(c);

        return c;
    }

    /**
     * Projects a 3D point a on a plane given by Plane: 
     * x = p + lambda * r1 + mhu * r2.
     * @param a
     * @param p
     * @param r1
     * @param r2
     * @return 
     */
    public Point3D projectPointOnPlane( Point3D a, Point3D p, 
                                        Point3D r1, Point3D r2) 
    {
        Point3D n = r1.computeNormalVector(r2);

        double k = n.m_x1 * p.m_x1 + n.m_x2 * p.m_x2 + n.m_x3 * p.m_x3;

        double lambda = (k - n.m_x1 * a.m_x1 - n.m_x2 * a.m_x2 - n.m_x3 * a.m_x3) / (n.m_x1 * n.m_x1 + n.m_x2 * n.m_x2 + n.m_x3 * n.m_x3);

        // Substitute lambda in vector representation of a line
        Point3D pp = new Point3D();

        pp.m_x1 = lambda * n.m_x1 + a.m_x1;
        pp.m_x2 = lambda * n.m_x2 + a.m_x2;
        pp.m_x3 = lambda * n.m_x3 + a.m_x3;

        return pp;
    }

    /**
     * Returns the length of the vector to the 3DPoint.
     * @return 
     */
    public double length() 
    {
        return Math.sqrt(m_x1 * m_x1 + m_x2 * m_x2 + m_x3 * m_x3);
    }

    /**
     * Returns the string representation of the point.
     *
     * @return
     */
    public String asString() 
    {
        m_formatter = new DecimalFormat("###.#");

        String asString = "(" + m_formatter.format(m_x1) + " , "
                + m_formatter.format(m_x2) + " , "
                + m_formatter.format(m_x3) + " )";

        return asString;
    }
    
    /**
     * Computes the dot product.
     * @param v1
     * @param v2
     * @return 
     */
    public static double dotProduct(Point3D v1, Point3D v2) 
    {
        double res;
        res = v1.m_x1 * v2.m_x1 + v1.m_x2 * v2.m_x2 + v1.m_x3 * v2.m_x3;
        return res;
    }

    /**
     * 
     * Computes the distance between two points.
     * 
     * @param v1
     * @param v2
     * @return 
     */
    public static double getDistance(Point3D v1, Point3D v2)
    {
        return Math.sqrt(Math.pow(v1.m_x1-v2.m_x1,2)+Math.pow(v1.m_x2-v2.m_x2, 2)+Math.pow(v1.m_x3-v2.m_x3, 2));
    }

    /**
     * Computes the cross product of two vectors.
     * @param v1
     * @param v2
     * @return 
     */
    public static Point3D crossProduct(Point3D v1, Point3D v2) {
        Point3D res = new Point3D();
        res.m_x1 = v1.m_x2 * v2.m_x3 - v1.m_x3 * v2.m_x2;
        res.m_x2 = v1.m_x3 * v2.m_x1 - v1.m_x1 * v2.m_x3;
        res.m_x3 = v1.m_x1 * v2.m_x2 - v1.m_x2 * v2.m_x1;
        return res;
    }
    
    /**
     * Normalizes a vector.
     * @param v1
     * @return 
     */
    public static double normVect(Point3D v1) {
        double res;
        res = dotProduct(v1, v1);
        return res;
    }

    /**
     * Cosine between 2 angles, given a reference vector.
     * @param v2
     * @param ref
     * @return 
     */ 
    public double getCos( Point3D v2, Point3D ref ) 
    {
        double res;
        v2 = v2.normalize(v2);
        ref = ref.normalize(ref);
        res = dotProduct(crossProduct(this.normalize(this), v2), ref);
        return res;
    }
    
    /**
     * Computes the inverse tangent of a vector.
     * @param v2
     * @return 
     */
    public double atan2(Point3D v2) 
    {
        double res;
        res = Math.atan2(crossProduct(this, v2).length(), dotProduct(this, v2));
        return res;
    }

    /**
     * Returns the angle between two points.
     * @param v2
     * @param ref
     * @return 
     */
    public double getAngle( Point3D v2, Point3D ref ) 
    {
        double angle;
        if (this.m_x1 == v2.m_x1 && this.m_x2 == v2.m_x2 && this.m_x3 == v2.m_x3) {
            angle = 0;
        } else {
            Point3D crossp = crossProduct(this, v2);
            if (crossp.m_x1 == 0 && crossp.m_x2 == 0 && crossp.m_x3 == 0) {
                angle = Math.PI;
            } else {
                double res = this.atan2(v2);
                double sign = dotProduct(crossp, ref);
                angle = res * sign / Math.abs(sign);
            }
        }
        return angle;
    }

    /**
     * Adds a vector to this.
     * @param aux
     * @return 
     */
    public Point3D addPoint( Point3D aux ) 
    {
        Point3D res = new Point3D();
        res.m_x1 = this.m_x1 + aux.m_x1;
        res.m_x2 = this.m_x2 + aux.m_x2;
        res.m_x3 = this.m_x3 + aux.m_x3;
        return res;
    }
    /**
     * Subtracts a vector to this.
     * @param aux
     * @return 
     */
    public Point3D subtractPoint(Point3D aux) 
    {
        Point3D res = new Point3D();
        res.m_x1 = this.m_x1 - aux.m_x1;
        res.m_x2 = this.m_x2 - aux.m_x2;
        res.m_x3 = this.m_x3 - aux.m_x3;
        return res;
    }

    /**
     * Clones this 3D-point.
     * @return 
     */
    public Point3D Clone() 
    {
        Point3D res = new Point3D();
        res.m_x1 = this.m_x1;
        res.m_x2 = this.m_x2;
        res.m_x3 = this.m_x3;
        return res;
    }

    /**
     * 
     * @param p1
     * @param p2
     * @param p3
     * @param i1
     * @param i2
     * @param i3
     * @return 
     */
    public static Point3D vectorCombination( Point3D p1, Point3D p2, Point3D p3, 
                                             int i1, int i2, int i3 ) 
    {
        Point3D res = new Point3D();
        res.m_x1 = i1 * p1.m_x1 + i2 * p2.m_x1 + i3 * p3.m_x1;
        res.m_x2 = i1 * p1.m_x2 + i2 * p2.m_x2 + i3 * p3.m_x2;
        res.m_x3 = i1 * p1.m_x3 + i2 * p2.m_x3 + i3 * p3.m_x3;
        return res;
    }
    
    /**
     * 
     * @param p1
     * @param p2
     * @param p3
     * @param d1
     * @param d2
     * @param d3
     * @return 
     */
    public static Point3D vectorCombination( Point3D p1, Point3D p2, Point3D p3, 
                                             double d1, double d2, double d3) 
    {
        Point3D res = new Point3D();
        res.m_x1 = d1 * p1.m_x1 + d2 * p2.m_x1 + d3 * p3.m_x1;
        res.m_x2 = d1 * p1.m_x2 + d2 * p2.m_x2 + d3 * p3.m_x2;
        res.m_x3 = d1 * p1.m_x3 + d2 * p2.m_x3 + d3 * p3.m_x3;
        return res;
    }
    
    /**
     * Computes the intersection point of segment A and B which are each expressed by two points.
     * This function returns null if the two segments are not intersecting, or if the intersection point lies outside of the segments
     * @param A0 first point of segment A
     * @param A1 second point of segment A
     * @param B0 first point of segment B
     * @param B1 second point of segment B
     * source: http://stackoverflow.com/questions/2316490/the-algorithm-to-find-the-point-of-intersection-of-two-3d-line-segment
     */
    public static Point3D computeIntersectionOfSegments(Point3D A0, Point3D A1, Point3D B0, Point3D B1)
    {
        Point3D da = A1.subtractPoint(A0); 
        Point3D db = B1.subtractPoint(B0);
        Point3D dc = B0.subtractPoint(A0);
        Point3D dd = A0.subtractPoint(B0);

        if (Math.abs(Point3D.dotProduct(dc, Point3D.crossProduct(da, db))) > 0.00000001) // lines are not coplanar
            return null;
        double s0 = Point3D.dotProduct(Point3D.crossProduct(dc,db),Point3D.crossProduct(da,db)) / Point3D.normVect(Point3D.crossProduct(da,db));
        double s1 = Point3D.dotProduct(Point3D.crossProduct(dd,da),Point3D.crossProduct(db,da)) / Point3D.normVect(Point3D.crossProduct(db,da));
//        System.out.println(s);
        if (s0 >= 0.0 && s0 <= 1.0 && s1 >= 0.0 && s1 <= 1.0)
        {
            return A0.addPoint(da.scalarMultiplication(s0));
        }

        return null;
    }
    
    /**
     * Returns the normal vector of the segment ab in the plane abc.
     * @param a
     * @param b
     * @param c
     * @return 
     */
    public static Point3D getNormal(Point3D a, Point3D b, Point3D c)
    {
        Point3D s = Point3D.crossProduct(b.subtractPoint(c), a.subtractPoint(c));
        Point3D n = Point3D.crossProduct(s, b.subtractPoint(a));
        n.normalize();
        return n;
    }
    
    // Private attributes of this class
    public double m_x1 = 0.0;
    public double m_x2 = 0.0;
    public double m_x3 = 0.0;
    DecimalFormat m_formatter;
}
