/*
 * File: $Id: PolygonClipboard.java,v 1.1.1.1 2006/04/19 09:38:47 hans Exp $
 * History
 * Date: 04.03.2003  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.imagegraphicobjects;

import java.awt.event.*;
import java.util.*;

/**
 *  Singleton class that contains a copy of a polygon object;
 *  This utility class allows the copying of polygons amongst
 *  different SliceCanvasPolydraw objects.
 *
 *  @author  slotboom
 */

public class PolygonClipboard 
{
    /**
     * Returns the one and only instance of PolygonClipboard.
     * @return 
     */
    public static PolygonClipboard getInstance( )
    {
        return theInstance;
    }
    
    /** 
     * Creates a new instance of PolygonClipboard .
     */
    protected PolygonClipboard( ) 
    {
        // initializing instance fields
    }
    
    /**
     * Returns the clipboard polygon.
     */
    public RegioPolygon getPolygon( )
    {
        return m_polygon;
    }

    /**
     * Returns the zoom factor of the polygon.
     * @return 
     */
    public float getZoom( )
    {
        return m_zoom;
    }
    
    /**
     * Returns the clipboard polygon.
     * @return 
     */
    public LinkedList getPolygonList( )
    {
        return m_polygonList;
    }

    /**
     * Sets the clipboard polygon.
     * @param polygon
     * @param zoom
     */
    public void setPolygon( RegioPolygon polygon, float zoom )
    {
        m_polygon = new RegioPolygon( polygon );
        m_zoom = zoom;
        if ( m_polygon != null )
        {
            notifyListeners( ADDED_SINGLE_POLYGON_TO_CLIPBOARD );
        }
    }
    
    /**
     * Sets the clipboard polygon list.
     * @param polygons
     * @param zoomfactor
     */
    public void setPolygonList( LinkedList polygons, float zoomfactor )
    {
        m_polygonList = new LinkedList<RegioPolygon>( );
        m_zoom = zoomfactor;
        
        Iterator it = polygons.iterator( );
        while ( it.hasNext( ) )
        {
            RegioPolygon polygon = new RegioPolygon( (RegioPolygon)it.next( ) );
            m_polygonList.add( polygon );
        }
        
        if ( m_polygonList.size( ) > 1 )
        {
            notifyListeners( ADDED_MULTIPLE_POLYGONS_TO_CLIPBOARD );
        }
    }
    
    /**
     * Notifies all registered listeners.
     */
    private void notifyListeners( int eventType  )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }

    /**
     * Adds an action listener to the list of action listeners who are notified
     * when a polygon is added to the clipboard.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }
    
    // static private attribute is the instance itself
    private static final PolygonClipboard theInstance = new PolygonClipboard( );
    
    // Members to add one polygon to the clipboard
    private RegioPolygon m_polygon;
    private float m_zoom;
    
    // Members to add a list of polygons to the clipboard
    private LinkedList<RegioPolygon> m_polygonList;
    
    // Listeners to changes in the segmentation gui control
    private LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    // Event typs of the clipboard object
    public final static int ADDED_SINGLE_POLYGON_TO_CLIPBOARD = 0;
    public final static int ADDED_MULTIPLE_POLYGONS_TO_CLIPBOARD = 1;
}