/*
 * File: $Id: RegioPolygonConnections.java,v 1.2 2006/11/09 10:39:19 hans Exp $
 * History
 * Date: 08.04.2009  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2009 by
 *
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imagegraphicobjects;

import java.util.LinkedList;

/**
 * This class handles the LinkedList which contain information on two which
 * a regio polygon is linked. Is used in lesion counting algorithm
 * @author slotboom
 */
public class RegioPolygonConnections
{
    /**
     * Default constructor creates a new RegioPolygonConnections object.
     */
    public  RegioPolygonConnections( )
    {
    }

    /**
     * Constructor creates a new RegioPolygonConnections object.
     * @param sliceInstanceNumber
     * @param polygonIndexNumber
     */
    public  RegioPolygonConnections( int sliceInstanceNumber, int polygonIndexNumber )
    {
        m_sliceInstanceNumber = sliceInstanceNumber;
        m_polygonIndexNumber = polygonIndexNumber;
    }

    /**
     * Sets the slice instance number the connector belongs to.
     * @param sliceInstanceNumber
     */
    public void setSliceInstanceNumber( int sliceInstanceNumber )
    {
        m_sliceInstanceNumber = sliceInstanceNumber;
    }

    /**
     * Sets the polygon index number the connector belongs to
     * @param polygonIndexNumber
     */
    public void setPolygonIndexNumber( int polygonIndexNumber )
    {
        m_polygonIndexNumber = polygonIndexNumber;
    }

    /**
     * Adds a RegioPolygon object to the list of regio polygons
     * @param poly
     */
    public void addConnectedRegioPolygon( RegioPolygon poly )
    {
        m_connectedPolygons.add( poly );
    }

    /**
     * Clears the list of connected polygons.
     */
    public void clearConnectedPolygonsFromList( )
    {
        m_connectedPolygons.clear( );
    }

    /**
     *
     * Sets the m_volumeNumberBelongingTo attribute.
     *
     * @param volumeNumber
     */
    public void setVolumeNumberBelongingTo( int volumeNumber )
    {
        m_volumeNumberBelongingTo = volumeNumber;
    }

    /**
     * Returns the volume number the region belongs to.
     * @return
     */
    public int getVolumeNumberBelongingTo( )
    {
        return m_volumeNumberBelongingTo;
    }

    /**
     * Returns the list of connected polygons.
     * @return
     */
    public LinkedList<RegioPolygon> getConnectedPolygons( )
    {
        return m_connectedPolygons;
    }


    private int m_sliceInstanceNumber;
    private int m_polygonIndexNumber;
    private int m_volumeNumberBelongingTo = -1;

    private LinkedList<RegioPolygon> m_connectedPolygons = new LinkedList<>( );
}
