/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomDefaults;
import org.spectrim.signalprocessing.SignalTools;
import org.spectrim.signalprocessing.SignalWriter;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import javax.swing.JFrame;

/**
 *
 * Computes the histogram of the selected image segment overlay.
 *
 * @author slotboom
 */
public class ImageOverlaySegmentHistogramCalculator
{
    /**
     * 
     * Constructor allowing the user to compute histograms of the defined image
     * overlay segments
     * 
     * @param parent
     * @param imageSegmentsOverlayArrayList
     * @param imagePixelValues
     * @param selectedTissueTypes
     * @param imageWidth
     * @param imageHeight
     * @param numberOfImages
     * @param nordicScaleFactor
     */
    public ImageOverlaySegmentHistogramCalculator( JFrame parent,
                                                   ArrayList< LinkedList<boolean[][]> > imageSegmentsOverlayArrayList,
                                                   int[] imagePixelValues, 
                                                   int[] selectedTissueTypes,
                                                   int imageWidth, 
                                                   int imageHeight, 
                                                   int numberOfImages, 
                                                   float nordicScaleFactor )
    {

        m_imageWidth = imageWidth;
        m_imageHeight = imageHeight;
        m_numberOfImages = numberOfImages;
        m_nordicScaleFactor = nordicScaleFactor;
        m_parent = parent;

        // create a 3D structure to determine quickly pixel connectivity

        for ( int tissueTypeNumber = 0;  tissueTypeNumber < selectedTissueTypes.length ; tissueTypeNumber++ )
        {
            // set counter to zero for each image segment type..
            m_totalImagePixels = 0;

            //LinkedList<boolean[][]> currentOverlaysLinkedList = imageSegmentsOverlayArrayList.get( segmentNumber );
            int imageSizeOffset;
            int imageHeightOffset;
            int totalOffset;

            // determine to total pixel count of the image segment
            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                boolean[][] currentBooleanOverlay = imageSegmentsOverlayArrayList.get( i ).get( selectedTissueTypes[tissueTypeNumber] );

                imageSizeOffset = i * imageWidth * imageHeight;

                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    imageHeightOffset = j * m_imageHeight;

                    for ( int k = 0; k < m_imageWidth; k++ )
                    {
                        if ( currentBooleanOverlay[k][j] == true )
                        {
                            m_totalImagePixels++;
                        }
                    }
                }
            }

            int[] numberOfImagePixels = new int[m_totalImagePixels];
            int simpleArrayCounter = 0;
            // determine to total pixel count of the image segment

            for ( int i = 0; i < m_numberOfImages; i++ )
            {
                boolean[][] currentBooleanOverlay = imageSegmentsOverlayArrayList.get( i ).get( selectedTissueTypes[tissueTypeNumber] );;

                imageSizeOffset = i * imageWidth * imageHeight;

                for ( int j = 0; j < m_imageHeight; j++ )
                {
                    imageHeightOffset = j * m_imageHeight;

                    for ( int k = 0; k < m_imageWidth; k++ )
                    {

                        totalOffset = imageSizeOffset + imageHeightOffset + k;

                        if ( currentBooleanOverlay[k][j] == true )
                        {

                            numberOfImagePixels[simpleArrayCounter] = imagePixelValues[ totalOffset ];
                            simpleArrayCounter++;
                        }
                    }
                }
            }

            Curve hist = calculateHistogram( numberOfImagePixels, nordicScaleFactor );

            DicomDefaults fdef = DicomDefaults.getInstance( );
            fdef.load( );

            String path = fdef.getProperty( "Dicom mount path" );

            File newDir = new File( path + System.getProperty( "file.separator" ) + "histogram" );

            if  ( !newDir.exists( ) )
            {
                newDir.mkdir( );
            }

            SignalWriter swrHist = new SignalWriter( m_parent, "Write image segment histrogram",
                                                     hist.getXvalues( ), hist.getYvalues( ),
                                                     newDir.getAbsolutePath( ),
                                                     "HistogramOfSegmentNumber" + selectedTissueTypes[tissueTypeNumber] + ".txt" );

        }

    }

    /**
     * Calculates the histogram of the intensity distribution of the integer
     * values given by integer array values.
     */
    private Curve calculateHistogram( int[] values, float nordicScalingFactor )
    {
        int numberOfClasses = 0;

        float maxValue = SignalTools.getMaximum( values ) * m_nordicScaleFactor;
        float minValue = SignalTools.getMinimum( values ) * m_nordicScaleFactor;

        float classWidth = 0f;
        int[] occurences;

        float[] xVals = null;
        float[] yVals = null;

        float rangeWidth = ( maxValue - minValue );

        if ( rangeWidth != 0 )
        {
            numberOfClasses = ( values.length / 10 );

            if ( numberOfClasses < 10 )
            {
                numberOfClasses = 10;
            }

            if ( numberOfClasses > 1024 )
            {
                numberOfClasses = 1024;
            }

            classWidth = (float)rangeWidth / (float)numberOfClasses;

            occurences = new int[ numberOfClasses + 1];

            for ( int i = 0; i < values.length ; i++ )
            {
                if ( values[i] != 0 )
                {
                    int xVal = (int)( ( values[i] - minValue )  /  classWidth );
                    occurences[ xVal ]++;
                }
            }

            xVals = new float[ 2 * occurences.length ];
            yVals = new float[ 2 * occurences.length ];

            for ( int i = 0; i < occurences.length; i++ )
            {
                xVals[2*i] = ( minValue + (float)i * classWidth ) * nordicScalingFactor;
                yVals[2*i] = occurences[i];
                xVals[2*i + 1] = ( minValue + (float)(i+1) * classWidth ) * nordicScalingFactor;
                yVals[2*i + 1] = occurences[i];
            }

            XYPlotFrame xyplot = new XYPlotFrame( "Histogram", null );

            xyplot.addCurve( new Curve( xVals, yVals ), true );

            xyplot.repaint( );

        }

        return new Curve( xVals, yVals );
    }

    private final int m_imageWidth;
    private final int m_imageHeight;
    private final int m_numberOfImages;
    private int m_totalImagePixels = 0;
    private float m_nordicScaleFactor = 1f;
    private final JFrame m_parent;
}
