/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * ImageStackTextureAnalyzerFrame.java
 *
 * Created on Jun 1, 2011, 4:24:40 PM
 */

package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomImageSeriesData;
import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.dicom.FileInfo;
import org.spectrim.dicom.MultiDicomSliceView;
import org.spectrim.imagegraphicobjects.RegioPolygon;
import org.spectrim.imagegraphicobjects.RegionModeToolBar;
import org.spectrim.lib.Dialogs;
import org.spectrim.lib.EnterStringFrame;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.LinkedList;
import java.util.Vector;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.table.DefaultTableModel;
import org.spectrim.settings.GlobalSettings;
import org.spectrim.texturemaps.ImageTextureCalculator;
import rcaller.RCaller;
import rcaller.RCode;

/**
 * ImageStackTextureAnalyzerFrame is a class that enables the interactive 
 * analysis of image textures of Dicom image stacks; Simple histogram and 
 * cooccurence matrix based textures are supported.
 * @author slotboom
 */
public class ImageStackTextureAnalyzerFrame extends javax.swing.JFrame implements ActionListener
{

    /** 
     * Creates new form ImageStackTextureAnalyzerFrame.
     * @param parent
     * @param imageSeriesData
     * @param isTimeSeries
     * @param beginBaseLine
     * @param endBaseLine
     * @param maxPerf
     * @param beginReperfusion
     */
    @SuppressWarnings("LeakingThisInConstructor")
    public ImageStackTextureAnalyzerFrame( JFrame parent, 
                                           DicomImageSeriesData imageSeriesData, 
                                           boolean isTimeSeries,
                                           int beginBaseLine, 
                                           int endBaseLine, 
                                           int maxPerf, 
                                           int beginReperfusion )
    {

        setTitle( "Texture analysis frame" );

        m_beginBaseLine     = beginBaseLine;
        m_endBaseLine       = endBaseLine;
        m_maxPerf           = maxPerf;
        m_beginReperfusion  = beginReperfusion;

        m_isImageTimeSeries   = isTimeSeries;
        
        initComponents( );
        
        initToolBars( );
        
        m_textureManipulationToolBar.add( m_regionModeToolBar );

        m_multiDicomSliceView.setParent( parent );
        m_multiDicomSliceView.isTimeSeries( isTimeSeries );
        m_multiDicomSliceView.enableSyncedDisplayMode( );
        m_multiDicomSliceView.setShowToolTipText( false );          

        m_multiDicomSliceFourView.setParent( parent );
        m_multiDicomSliceFourView.isTimeSeries( isTimeSeries );
        m_multiDicomSliceFourView.enableSyncedDisplayMode( );
        m_multiDicomSliceFourView.setShowToolTipText( false );
        
        m_imageSeriesData = imageSeriesData;

        LinkedList<String> fileNames = imageSeriesData.getFileReferences( );
        LinkedList<String> fourViewData = new LinkedList<String>( );

        fourViewData.add( fileNames.get( 0 ) );
        fourViewData.add( fileNames.get( 1 ) );
        fourViewData.add( fileNames.get( 2 ) );
        fourViewData.add( fileNames.get( 3 ) );

        m_imageSeriesFourViewData = new DicomImageSeriesData( fourViewData );

        SCANalyzeDefaults defs = SCANalyzeDefaults.getInstance( );
        defs.load( );

        String lastNormalizationFactor = defs.getProperty( SCANalyzeDefaults.LAST_DTPA_NORMALIZATION_FACTOR );
        m_lastNormFactorTextField.setText( lastNormalizationFactor );

        if ( ( m_beginBaseLine == 0 ) && ( endBaseLine == 0 ) && ( maxPerf == 0 ) && ( beginReperfusion == 0 ) )
        {
            computeAndDisplayTextures( m_minumumPixelValueSlider.getValue( ),
                                       m_maximumPixelValueSlider.getValue( ),
                                       m_xLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1,
                                       m_yLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1, 
                                       false );
        }
        else
        {
            normalizeAndComputeAndDisplayTextures( m_scalingFactorOriginalValues, m_scalingFactorDifferenceImageTimeSeries,
                                                   m_minumumPixelValueSlider.getValue( ),
                                                   m_maximumPixelValueSlider.getValue( ),
                                                   m_xLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1,
                                                   m_yLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1,
                                                   false, true );
        }

        // JPopupMenu for operations on diverse textures
        m_popupMenu = new JPopupMenu( );

        m_createXYScatterPlot = new JMenuItem( "Create scatter plot.." );
        m_createXYScatterPlot.addActionListener( this );
        m_popupMenu.add( m_createXYScatterPlot );

        m_createXYScatterPlotOfROIs = new JMenuItem( "Create scatter plot of ROIs.." );
        m_createXYScatterPlotOfROIs.addActionListener( this );
        m_popupMenu.add( m_createXYScatterPlotOfROIs );

        // MouseListener popupListener = (MouseListener)( new PopupListener( ) );

        MouseListener popupListener = new PopupListener( );

        // add the listener specifically to the header
        m_textureTypeSelectionTable.addMouseListener( popupListener );
        m_textureTypeSelectionTable.getTableHeader( ).addMouseListener( popupListener );

        m_mainMenu = new JMenu( "Regions" );

        m_addArbitraryShapedROIs = new JRadioButtonMenuItem( "Add arbitrary shaped ROIs mode" );
        m_addArbitraryShapedROIs.setSelected(true);
        m_mainMenu.add( m_addArbitraryShapedROIs );
        m_addArbitraryShapedROIs.addActionListener(this);
        
        m_addCircularShapedROIs = new JRadioButtonMenuItem( "Add circular shaped ROIs mode" );
        m_mainMenu.add( m_addCircularShapedROIs );
        m_addCircularShapedROIs.addActionListener(this);
        
        m_ROITypeToAddMode = new ButtonGroup();
        m_ROITypeToAddMode.add(m_addArbitraryShapedROIs);
        m_ROITypeToAddMode.add(m_addCircularShapedROIs);
        
        m_mainMenu.addSeparator( );
        
        m_renameCurrentROIToNAWM = new JMenuItem( "Rename to healthy control tissue (NAWM)" );
        m_renameCurrentROIToNAWM.addActionListener( this );
        
        m_renameCurrentROIToMetastasisTotalTumorTissue = new JMenuItem( "Rename to total metastasis tumor tissue (TOTAL_MET)" );
        m_renameCurrentROIToMetastasisTotalTumorTissue.addActionListener( this );

        m_renameCurrentROIToGBMTotalTumorTissue = new JMenuItem( "Rename to total GBM tumor tissue (TOTAL_GBM)" );
        m_renameCurrentROIToGBMTotalTumorTissue.addActionListener( this );

        m_renameCurrentROIToNonEnhancing_MET_Edema = new JMenuItem( "Rename to non enhancing MET edema (MET_NET_EDE)" );
        m_renameCurrentROIToNonEnhancing_MET_Edema.addActionListener( this );

        m_renameCurrentROIToNonEnhancing_GBM_Edema = new JMenuItem( "Rename to non enhancing GBM edema (GBM_NET_EDE)" );
        m_renameCurrentROIToNonEnhancing_GBM_Edema.addActionListener( this );
        
        m_renameCurrentROITo_CET_NECR_MET_TumorTissue = new JMenuItem( "Rename to enhancing & necrotic MET tissue (MET_CET_NECR)" );
        m_renameCurrentROITo_CET_NECR_MET_TumorTissue.addActionListener( this );
        
        m_renameCurrentROITo_CET_NECR_GBM_TumorTissue = new JMenuItem( "Rename to enhancing & necrotic GBM tissue (GBM_CET_NECR)" );
        m_renameCurrentROITo_CET_NECR_GBM_TumorTissue.addActionListener( this );

        m_deleteAllROIsInAllViews = new JMenuItem( "Delete all ROIs in all views!" );
        m_deleteAllROIsInAllViews.addActionListener( this );
        
        
        m_loadROIsMenuItem = new JMenuItem( "Load ROIs" );
        m_loadROIsMenuItem.addActionListener( this );

        m_saveROIsMenuItem = new JMenuItem( "Save ROIs" );
        m_saveROIsMenuItem.addActionListener( this );

        m_mainMenu.add( m_loadROIsMenuItem );
        m_mainMenu.add( m_saveROIsMenuItem );

        
        m_createScreenShotOfMultiView = new JMenuItem( "Create screenshot of multiview." );
        m_createScreenShotOfMultiView.addActionListener( this );
        m_mainMenu.add( m_createScreenShotOfMultiView );
        
        m_mainMenu.addSeparator( );

        m_mainMenu.add(m_renameCurrentROIToNAWM );
        m_mainMenu.add( m_renameCurrentROIToMetastasisTotalTumorTissue );
        m_mainMenu.add( m_renameCurrentROIToGBMTotalTumorTissue );
        m_mainMenu.add( m_renameCurrentROIToNonEnhancing_MET_Edema );
        m_mainMenu.add( m_renameCurrentROIToNonEnhancing_GBM_Edema );
        m_mainMenu.add( m_renameCurrentROITo_CET_NECR_MET_TumorTissue );
        m_mainMenu.add( m_renameCurrentROITo_CET_NECR_GBM_TumorTissue );
        
        m_mainMenu.addSeparator( );
        
        m_mainMenu.add( m_deleteAllROIsInAllViews );
        
        m_viewMenu = new JMenu( "View" );
        m_itemShowToolTipText = new JRadioButtonMenuItem( "Show tooltips" );
        m_itemShowToolTipText.addActionListener( this );
        m_viewMenu.add( m_itemShowToolTipText );
        m_itemShowToolTipText.setSelected( false );
        
        
        
        m_menuBar = new JMenuBar( );
        m_menuBar.add( m_mainMenu );
        m_menuBar.add( m_viewMenu );
        
        setJMenuBar( m_menuBar );
        
        if ( m_isImageTimeSeries == true )
        {
            this.m_textureMapsViewingTabbedPane.setTitleAt(0, "Time series");
            this.m_textureMapsViewingTabbedPane.setTitleAt(1, "Perfusion interval averages");
        }
        
        if ( m_isImageTimeSeries == false )
        {
            this.m_textureMapsViewingTabbedPane.setTitleAt(0, "Image stack");
            this.m_textureMapsViewingTabbedPane.setTitleAt(1, "not used");
        }
        
        // set the default selected multiview
        m_selectedMultiDicomSliceViewTabNumber = 0;
        m_selectedMultiDicomSliceView = m_multiDicomSliceView;
        
        pack( );
        setVisible( true );
        
        loadDataBaseReferences( );
    }
    
    
    /**
     * Loads database references to dbase text files in the filesystem. 
     */
    private void loadDataBaseReferences( )
    {
        
        m_diagnoseTestAlternative_A_ComboBox.removeAllItems( );
        m_diagnoseTestAlternative_B_ComboBox.removeAllItems( );
        m_diagnosis2Dbase_ComboBox.removeAllItems( );
        
        // Directory path where dbases can be found..
        String path_cad_base =  System.getProperty( "user.home" ) +
                                System.getProperty( "file.separator" ) + "CAD";
        
        File cad_folder = new File( path_cad_base );
        
        if ( cad_folder.exists( ) == false )
        {
            cad_folder.mkdir( );
        }
        
        String path_cad_all = path_cad_base + System.getProperty( "file.separator" ) + "CAD_dbases_all";
        File cad_db_folder_all = new File( path_cad_all );
        
        if ( cad_db_folder_all.exists( ) == false )
        {
            cad_db_folder_all.mkdir( );
        }

        String path_cad_1_i_pp = path_cad_base + System.getProperty( "file.separator" ) + "CAD_dbases_1_i_pp";
        File cad_db_folder_1_i_pp = new File( path_cad_1_i_pp );
        
        if ( cad_db_folder_1_i_pp.exists( ) == false )
        {
            cad_db_folder_1_i_pp.mkdir( );
        }
        
        String path_cad_2_predict = path_cad_base + System.getProperty( "file.separator" ) + "CAD_entry_2_predict";
        File path_db_cad_2_predict = new File( path_cad_2_predict );
        
        if ( path_db_cad_2_predict.exists( ) == false )
        {
            path_db_cad_2_predict.mkdir( );
        }
         
        String files;
        File[] listOfFiles = cad_db_folder_all.listFiles( );
        
        if ( listOfFiles != null )
        {
            for ( int i = 0; i < listOfFiles.length; i++ ) 
            {
                if ( listOfFiles[i].isFile( ) ) 
                {
                    files = listOfFiles[i].getName( );
                    System.out.println( files );
                    m_diagnoseTestAlternative_A_ComboBox.addItem( files.substring( 0, files.indexOf('.') ) );
                    m_diagnoseTestAlternative_B_ComboBox.addItem( files.substring( 0, files.indexOf('.') ) );
                    m_diagnosis2Dbase_ComboBox.addItem( files.substring( 0, files.indexOf('.') ) );
                }
            }
        }
    }
    
    /**
     * Implements the listener to the mouse events of clicking around
     * in the table.
     */
    public class PopupListener extends MouseAdapter
    {
        @Override
        public void mousePressed( MouseEvent e )
        {
            showPopup(e);
        }

        @Override
        public void mouseReleased( MouseEvent e )
        {
            showPopup(e);
        }

        private void showPopup( MouseEvent e )
        {
            if ( e.isPopupTrigger( ) )
            {
                m_popupMenu.show( e.getComponent( ), e.getX( ), e.getY( ) );
            }
        }
    }


    /**
     * Initializes the toolbar.
     */
    private void initToolBars( )
    {

        m_regionModeToolBar = new RegionModeToolBar( );
        // listen to actions of this toolbar!
        m_regionModeToolBar.addActionListener( this );

    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane2 = new javax.swing.JScrollPane();
        jPanel18 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel3 = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel6 = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        m_textureTypeSelectionTable = new javax.swing.JTable();
        jPanel10 = new javax.swing.JPanel();
        jPanel14 = new javax.swing.JPanel();
        m_minimumIntensityThressholdSlider = new javax.swing.JSlider();
        jPanel15 = new javax.swing.JPanel();
        m_maximumIntensityThressholdSlider = new javax.swing.JSlider();
        jPanel16 = new javax.swing.JPanel();
        m_applyThresholdingButton = new javax.swing.JButton();
        jPanel5 = new javax.swing.JPanel();
        jPanel7 = new javax.swing.JPanel();
        m_imageZoomFactorComboBox = new javax.swing.JComboBox();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        m_colorTableSelectionComboBox = new javax.swing.JComboBox();
        m_showScaleCheckBox = new javax.swing.JCheckBox();
        m_synchronizedViewsCheckBox = new javax.swing.JCheckBox();
        jPanel11 = new javax.swing.JPanel();
        jPanel12 = new javax.swing.JPanel();
        jPanel8 = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        m_xLocalStatisticsSizeComboBox = new javax.swing.JComboBox();
        jLabel4 = new javax.swing.JLabel();
        m_yLocalStatisticsSizeComboBox = new javax.swing.JComboBox();
        jPanel9 = new javax.swing.JPanel();
        m_recalculateTexturesButton = new javax.swing.JButton();
        m_writeROIInformationButton = new javax.swing.JButton();
        m_verboseCheckBox = new javax.swing.JCheckBox();
        m_normalizeButton = new javax.swing.JButton();
        jPanel17 = new javax.swing.JPanel();
        m_lastNormFactorTextField = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        m_forceLastNormalizationButton = new javax.swing.JButton();
        jPanel13 = new javax.swing.JPanel();
        m_maximumPixelValueSlider = new javax.swing.JSlider();
        m_minumumPixelValueSlider = new javax.swing.JSlider();
        jPanel4 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        m_textureMapsViewingTabbedPane = new javax.swing.JTabbedPane();
        m_multiDicomSliceView = new org.spectrim.dicom.MultiDicomSliceView();
        m_multiDicomSliceFourView = new org.spectrim.dicom.MultiDicomSliceView();
        m_textureManipulationToolBar = new javax.swing.JToolBar();
        jPanel2 = new javax.swing.JPanel();
        jPanel19 = new javax.swing.JPanel();
        m_addCaseToCADdBaseButton = new javax.swing.JButton();
        m_diagnosis2Dbase_ComboBox = new javax.swing.JComboBox();
        jLabel8 = new javax.swing.JLabel();
        m_addTissueDBButton = new javax.swing.JButton();
        jPanel20 = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        m_diagnoseTestAlternative_A_ComboBox = new javax.swing.JComboBox();
        jLabel7 = new javax.swing.JLabel();
        m_diagnoseTestAlternative_B_ComboBox = new javax.swing.JComboBox();
        m_diagnoseButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        jScrollPane2.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        jPanel1.setMaximumSize(new java.awt.Dimension(885, 600));
        jPanel1.setPreferredSize(new java.awt.Dimension(885, 600));
        jPanel1.setRequestFocusEnabled(false);

        jSplitPane1.setDividerLocation(270);
        jSplitPane1.setMaximumSize(new java.awt.Dimension(500, 500));
        jSplitPane1.setPreferredSize(new java.awt.Dimension(851, 991));

        jPanel3.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        m_textureTypeSelectionTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Texture name", "Use in analysis"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.Boolean.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        jScrollPane3.setViewportView(m_textureTypeSelectionTable);

        jPanel10.setLayout(new javax.swing.BoxLayout(jPanel10, javax.swing.BoxLayout.Y_AXIS));

        jPanel14.setBorder(javax.swing.BorderFactory.createTitledBorder("Minimum [%]"));

        m_minimumIntensityThressholdSlider.setMajorTickSpacing(25);
        m_minimumIntensityThressholdSlider.setMinorTickSpacing(5);
        m_minimumIntensityThressholdSlider.setPaintLabels(true);
        m_minimumIntensityThressholdSlider.setPaintTicks(true);
        m_minimumIntensityThressholdSlider.setValue(0);
        m_minimumIntensityThressholdSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                m_minimumIntensityThressholdSliderStateChanged(evt);
            }
        });

        javax.swing.GroupLayout jPanel14Layout = new javax.swing.GroupLayout(jPanel14);
        jPanel14.setLayout(jPanel14Layout);
        jPanel14Layout.setHorizontalGroup(
            jPanel14Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel14Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_minimumIntensityThressholdSlider, javax.swing.GroupLayout.DEFAULT_SIZE, 209, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel14Layout.setVerticalGroup(
            jPanel14Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel14Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_minimumIntensityThressholdSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel10.add(jPanel14);

        jPanel15.setBorder(javax.swing.BorderFactory.createTitledBorder("Maximum [%]"));

        m_maximumIntensityThressholdSlider.setMajorTickSpacing(25);
        m_maximumIntensityThressholdSlider.setMinorTickSpacing(5);
        m_maximumIntensityThressholdSlider.setPaintLabels(true);
        m_maximumIntensityThressholdSlider.setPaintTicks(true);
        m_maximumIntensityThressholdSlider.setValue(100);
        m_maximumIntensityThressholdSlider.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                m_maximumIntensityThressholdSliderStateChanged(evt);
            }
        });

        javax.swing.GroupLayout jPanel15Layout = new javax.swing.GroupLayout(jPanel15);
        jPanel15.setLayout(jPanel15Layout);
        jPanel15Layout.setHorizontalGroup(
            jPanel15Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel15Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_maximumIntensityThressholdSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(15, Short.MAX_VALUE))
        );
        jPanel15Layout.setVerticalGroup(
            jPanel15Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel15Layout.createSequentialGroup()
                .addComponent(m_maximumIntensityThressholdSlider, javax.swing.GroupLayout.PREFERRED_SIZE, 79, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(23, Short.MAX_VALUE))
        );

        jPanel10.add(jPanel15);

        m_applyThresholdingButton.setText("Apply ");
        m_applyThresholdingButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_applyThresholdingButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel16Layout = new javax.swing.GroupLayout(jPanel16);
        jPanel16.setLayout(jPanel16Layout);
        jPanel16Layout.setHorizontalGroup(
            jPanel16Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel16Layout.createSequentialGroup()
                .addGap(90, 90, 90)
                .addComponent(m_applyThresholdingButton)
                .addContainerGap(102, Short.MAX_VALUE))
        );
        jPanel16Layout.setVerticalGroup(
            jPanel16Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel16Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_applyThresholdingButton)
                .addContainerGap(78, Short.MAX_VALUE))
        );

        jPanel10.add(jPanel16);

        javax.swing.GroupLayout jPanel6Layout = new javax.swing.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addGroup(jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jPanel10, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jScrollPane3, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 245, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(22, Short.MAX_VALUE))
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel6Layout.createSequentialGroup()
                .addGap(20, 20, 20)
                .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 300, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(27, 27, 27)
                .addComponent(jPanel10, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(217, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab("Texture Selector", jPanel6);

        jPanel5.setLayout(new javax.swing.BoxLayout(jPanel5, javax.swing.BoxLayout.Y_AXIS));

        jPanel7.setBorder(javax.swing.BorderFactory.createTitledBorder("Global image display properties"));

        m_imageZoomFactorComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "0.25", "0.50", "1.0", "2.0", "4.0" }));
        m_imageZoomFactorComboBox.setSelectedIndex(2);
        m_imageZoomFactorComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_imageZoomFactorComboBoxActionPerformed(evt);
            }
        });

        jLabel1.setText("Zoom factor");

        jLabel2.setText("Color table");

        m_colorTableSelectionComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Gray value", "Rainbow", "Inv. Rainbow" }));
        m_colorTableSelectionComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_colorTableSelectionComboBoxActionPerformed(evt);
            }
        });

        m_showScaleCheckBox.setText("Show scale");
        m_showScaleCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_showScaleCheckBoxActionPerformed(evt);
            }
        });

        m_synchronizedViewsCheckBox.setSelected(true);
        m_synchronizedViewsCheckBox.setText("Synchronized views");
        m_synchronizedViewsCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_synchronizedViewsCheckBoxActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel7Layout = new javax.swing.GroupLayout(jPanel7);
        jPanel7.setLayout(jPanel7Layout);
        jPanel7Layout.setHorizontalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_showScaleCheckBox)
                    .addComponent(m_synchronizedViewsCheckBox)
                    .addGroup(jPanel7Layout.createSequentialGroup()
                        .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel2)
                            .addComponent(jLabel1))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(m_imageZoomFactorComboBox, javax.swing.GroupLayout.Alignment.LEADING, 0, 133, Short.MAX_VALUE)
                            .addComponent(m_colorTableSelectionComboBox, javax.swing.GroupLayout.Alignment.LEADING, 0, 1, Short.MAX_VALUE))))
                .addContainerGap())
        );
        jPanel7Layout.setVerticalGroup(
            jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel7Layout.createSequentialGroup()
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(m_imageZoomFactorComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel7Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(m_colorTableSelectionComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_showScaleCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_synchronizedViewsCheckBox)
                .addContainerGap())
        );

        jPanel5.add(jPanel7);

        jPanel11.setBorder(javax.swing.BorderFactory.createTitledBorder("Computation parameters"));
        jPanel11.setLayout(new javax.swing.BoxLayout(jPanel11, javax.swing.BoxLayout.Y_AXIS));

        jPanel12.setLayout(new javax.swing.BoxLayout(jPanel12, javax.swing.BoxLayout.Y_AXIS));

        jPanel8.setBorder(javax.swing.BorderFactory.createTitledBorder("Local statistic range"));

        jLabel3.setText("Pixel range X");

        m_xLocalStatisticsSizeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "3", "5", "7", "9" }));
        m_xLocalStatisticsSizeComboBox.setSelectedIndex(1);

        jLabel4.setText("Pixel range Y");

        m_yLocalStatisticsSizeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "3", "5", "7", "9" }));
        m_yLocalStatisticsSizeComboBox.setSelectedIndex(1);

        javax.swing.GroupLayout jPanel8Layout = new javax.swing.GroupLayout(jPanel8);
        jPanel8.setLayout(jPanel8Layout);
        jPanel8Layout.setHorizontalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(jPanel8Layout.createSequentialGroup()
                        .addComponent(jLabel3)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_xLocalStatisticsSizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 66, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel8Layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_yLocalStatisticsSizeComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap(65, Short.MAX_VALUE))
        );
        jPanel8Layout.setVerticalGroup(
            jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(m_xLocalStatisticsSizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(jPanel8Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(m_yLocalStatisticsSizeComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(62, Short.MAX_VALUE))
        );

        jPanel12.add(jPanel8);

        jPanel9.setBorder(javax.swing.BorderFactory.createTitledBorder("ROI normalization"));

        m_recalculateTexturesButton.setText("Recalculate!");
        m_recalculateTexturesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_recalculateTexturesButtonActionPerformed(evt);
            }
        });

        m_writeROIInformationButton.setText("Write ROI infos");
        m_writeROIInformationButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_writeROIInformationButtonActionPerformed(evt);
            }
        });

        m_verboseCheckBox.setText("Verbose report");

        m_normalizeButton.setText("Normalize");
        m_normalizeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_normalizeButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel9Layout = new javax.swing.GroupLayout(jPanel9);
        jPanel9.setLayout(jPanel9Layout);
        jPanel9Layout.setHorizontalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel9Layout.createSequentialGroup()
                .addGap(30, 30, 30)
                .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_normalizeButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(m_writeROIInformationButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addComponent(m_recalculateTexturesButton, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 148, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(m_verboseCheckBox, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(65, 65, 65))
        );
        jPanel9Layout.setVerticalGroup(
            jPanel9Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel9Layout.createSequentialGroup()
                .addComponent(m_recalculateTexturesButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_writeROIInformationButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 26, Short.MAX_VALUE)
                .addComponent(m_verboseCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_normalizeButton)
                .addGap(44, 44, 44))
        );

        jPanel12.add(jPanel9);

        jPanel17.setBorder(javax.swing.BorderFactory.createTitledBorder("Force last norm. constant"));

        m_lastNormFactorTextField.setText("1.0");

        jLabel5.setText("Last normalization factor");

        m_forceLastNormalizationButton.setText("Force last normalization");
        m_forceLastNormalizationButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_forceLastNormalizationButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel17Layout = new javax.swing.GroupLayout(jPanel17);
        jPanel17.setLayout(jPanel17Layout);
        jPanel17Layout.setHorizontalGroup(
            jPanel17Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel17Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel17Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_forceLastNormalizationButton)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel17Layout.createSequentialGroup()
                        .addGroup(jPanel17Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(m_lastNormFactorTextField, javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addGap(5, 5, 5)))
                .addGap(54, 54, 54))
        );
        jPanel17Layout.setVerticalGroup(
            jPanel17Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel17Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel5)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_lastNormFactorTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(12, 12, 12)
                .addComponent(m_forceLastNormalizationButton)
                .addContainerGap(38, Short.MAX_VALUE))
        );

        jPanel12.add(jPanel17);

        jPanel11.add(jPanel12);

        jPanel13.setBorder(javax.swing.BorderFactory.createTitledBorder("Min-max-pixel values"));
        jPanel13.setLayout(new javax.swing.BoxLayout(jPanel13, javax.swing.BoxLayout.Y_AXIS));

        m_maximumPixelValueSlider.setMajorTickSpacing(25);
        m_maximumPixelValueSlider.setMinorTickSpacing(5);
        m_maximumPixelValueSlider.setPaintLabels(true);
        m_maximumPixelValueSlider.setPaintTicks(true);
        m_maximumPixelValueSlider.setSnapToTicks(true);
        m_maximumPixelValueSlider.setValue(100);
        jPanel13.add(m_maximumPixelValueSlider);

        m_minumumPixelValueSlider.setMajorTickSpacing(25);
        m_minumumPixelValueSlider.setMinorTickSpacing(5);
        m_minumumPixelValueSlider.setPaintLabels(true);
        m_minumumPixelValueSlider.setPaintTicks(true);
        m_minumumPixelValueSlider.setValue(5);
        jPanel13.add(m_minumumPixelValueSlider);

        jPanel11.add(jPanel13);

        jPanel5.add(jPanel11);

        jTabbedPane1.addTab("General", jPanel5);

        jTabbedPane1.setSelectedIndex(1);

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 275, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jTabbedPane1))
        );

        jSplitPane1.setLeftComponent(jPanel3);

        jPanel4.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        jScrollPane1.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        jScrollPane1.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);

        m_textureMapsViewingTabbedPane.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                m_textureMapsViewingTabbedPaneStateChanged(evt);
            }
        });

        javax.swing.GroupLayout m_multiDicomSliceViewLayout = new javax.swing.GroupLayout(m_multiDicomSliceView);
        m_multiDicomSliceView.setLayout(m_multiDicomSliceViewLayout);
        m_multiDicomSliceViewLayout.setHorizontalGroup(
            m_multiDicomSliceViewLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 870, Short.MAX_VALUE)
        );
        m_multiDicomSliceViewLayout.setVerticalGroup(
            m_multiDicomSliceViewLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 935, Short.MAX_VALUE)
        );

        m_textureMapsViewingTabbedPane.addTab("Time series view", m_multiDicomSliceView);

        javax.swing.GroupLayout m_multiDicomSliceFourViewLayout = new javax.swing.GroupLayout(m_multiDicomSliceFourView);
        m_multiDicomSliceFourView.setLayout(m_multiDicomSliceFourViewLayout);
        m_multiDicomSliceFourViewLayout.setHorizontalGroup(
            m_multiDicomSliceFourViewLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 870, Short.MAX_VALUE)
        );
        m_multiDicomSliceFourViewLayout.setVerticalGroup(
            m_multiDicomSliceFourViewLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 935, Short.MAX_VALUE)
        );

        m_textureMapsViewingTabbedPane.addTab("Time averaged view", m_multiDicomSliceFourView);

        jScrollPane1.setViewportView(m_textureMapsViewingTabbedPane);

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 893, Short.MAX_VALUE)
                .addContainerGap())
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 987, Short.MAX_VALUE)
        );

        jSplitPane1.setRightComponent(jPanel4);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jSplitPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 1185, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jSplitPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 991, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        m_textureManipulationToolBar.setRollover(true);

        jPanel2.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 1, 1));
        jPanel2.setLayout(new javax.swing.BoxLayout(jPanel2, javax.swing.BoxLayout.LINE_AXIS));

        jPanel19.setBorder(javax.swing.BorderFactory.createTitledBorder("Databases"));

        m_addCaseToCADdBaseButton.setText("Add case to DBase");
        m_addCaseToCADdBaseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_addCaseToCADdBaseButtonActionPerformed(evt);
            }
        });

        m_diagnosis2Dbase_ComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        jLabel8.setText("Add case as:");

        m_addTissueDBButton.setText("New Tissue Label");
        m_addTissueDBButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_addTissueDBButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel19Layout = new javax.swing.GroupLayout(jPanel19);
        jPanel19.setLayout(jPanel19Layout);
        jPanel19Layout.setHorizontalGroup(
            jPanel19Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel19Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel19Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_addCaseToCADdBaseButton)
                    .addGroup(jPanel19Layout.createSequentialGroup()
                        .addComponent(jLabel8)
                        .addGap(68, 68, 68)
                        .addComponent(m_diagnosis2Dbase_ComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_addTissueDBButton)))
                .addContainerGap(185, Short.MAX_VALUE))
        );
        jPanel19Layout.setVerticalGroup(
            jPanel19Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel19Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_addCaseToCADdBaseButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel19Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_diagnosis2Dbase_ComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8)
                    .addComponent(m_addTissueDBButton))
                .addContainerGap(42, Short.MAX_VALUE))
        );

        jPanel2.add(jPanel19);

        jPanel20.setBorder(javax.swing.BorderFactory.createTitledBorder("Diagnose by classification"));

        jLabel6.setText("Diff. Diagn. A.");

        m_diagnoseTestAlternative_A_ComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        jLabel7.setText("Diff. Diagn. B.");

        m_diagnoseTestAlternative_B_ComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));

        m_diagnoseButton.setText("Diagnose ! ");
        m_diagnoseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_diagnoseButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel20Layout = new javax.swing.GroupLayout(jPanel20);
        jPanel20.setLayout(jPanel20Layout);
        jPanel20Layout.setHorizontalGroup(
            jPanel20Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel20Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel20Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel20Layout.createSequentialGroup()
                        .addComponent(jLabel6)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_diagnoseTestAlternative_A_ComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(jPanel20Layout.createSequentialGroup()
                        .addComponent(jLabel7)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_diagnoseTestAlternative_B_ComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(m_diagnoseButton))
                .addContainerGap(253, Short.MAX_VALUE))
        );
        jPanel20Layout.setVerticalGroup(
            jPanel20Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel20Layout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addGroup(jPanel20Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(m_diagnoseTestAlternative_A_ComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel20Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel7)
                    .addComponent(m_diagnoseTestAlternative_B_ComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_diagnoseButton)
                .addContainerGap(19, Short.MAX_VALUE))
        );

        jPanel2.add(jPanel20);

        javax.swing.GroupLayout jPanel18Layout = new javax.swing.GroupLayout(jPanel18);
        jPanel18.setLayout(jPanel18Layout);
        jPanel18Layout.setHorizontalGroup(
            jPanel18Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel18Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addContainerGap())
            .addGroup(jPanel18Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel18Layout.createSequentialGroup()
                    .addContainerGap()
                    .addGroup(jPanel18Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 1185, Short.MAX_VALUE)
                        .addGroup(jPanel18Layout.createSequentialGroup()
                            .addComponent(m_textureManipulationToolBar, javax.swing.GroupLayout.DEFAULT_SIZE, 1173, Short.MAX_VALUE)
                            .addContainerGap()))))
        );
        jPanel18Layout.setVerticalGroup(
            jPanel18Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel18Layout.createSequentialGroup()
                .addContainerGap(1076, Short.MAX_VALUE)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, 151, javax.swing.GroupLayout.PREFERRED_SIZE))
            .addGroup(jPanel18Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel18Layout.createSequentialGroup()
                    .addContainerGap()
                    .addComponent(m_textureManipulationToolBar, javax.swing.GroupLayout.PREFERRED_SIZE, 42, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 1015, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addContainerGap(152, Short.MAX_VALUE)))
        );

        jScrollPane2.setViewportView(jPanel18);

        getContentPane().add(jScrollPane2, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_forceLastNormalizationButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_forceLastNormalizationButtonActionPerformed
        
        m_scalingFactorOriginalValues = new Float( m_lastNormFactorTextField.getText( ) ).floatValue( );
        m_recalculateTexturesButtonActionPerformed( null );
}//GEN-LAST:event_m_forceLastNormalizationButtonActionPerformed

    private void m_normalizeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_normalizeButtonActionPerformed
        
        
        m_scalingFactorOriginalValues = m_multiDicomSliceView.normalize( 0 );
        
        if ( m_isImageTimeSeries == true ) 
        {
            m_scalingFactorDifferenceImageTimeSeries = m_multiDicomSliceView.getNormalizeOnTimeSeriesFactor( m_endBaseLine, m_maxPerf, m_beginReperfusion );
        }
        
        SCANalyzeDefaults defs = SCANalyzeDefaults.getInstance( );
        defs.load( );
        
        String lastNormalizationFactor = defs.getProperty( SCANalyzeDefaults.LAST_DTPA_NORMALIZATION_FACTOR );
        m_lastNormFactorTextField.setText( lastNormalizationFactor );
        
        m_recalculateTexturesButtonActionPerformed( null );
}//GEN-LAST:event_m_normalizeButtonActionPerformed

    private void m_writeROIInformationButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_writeROIInformationButtonActionPerformed
        
        if ( m_isImageTimeSeries == false ) 
        {
            
//            if ( m_selectedMultiDicomSliceViewTabNumber == 0 )
//            {
//                m_multiDicomSliceView.saveAllROIInformationToFile( );
//            }
//            else
//            {
//                m_multiDicomSliceFourView.saveAllROIInformationToFile( );
//            }
            
            m_selectedMultiDicomSliceView.saveAllROIInformationToFile( );
        } 
        else 
        {
            
            float oldZoom;
            oldZoom = m_selectedMultiDicomSliceView.getGlobalZoomFactor( );
            m_selectedMultiDicomSliceView.setGlobalZoomFactor( (float)1f );
            m_selectedMultiDicomSliceView.saveAllROIInformationToFile( m_verboseCheckBox.isSelected( ), m_beginBaseLine, m_endBaseLine, m_maxPerf, m_beginReperfusion );
            m_selectedMultiDicomSliceView.setGlobalZoomFactor( oldZoom );
     
//            if ( m_selectedMultiDicomSliceViewTabNumber == 0 )
//            {
//                oldZoom = m_multiDicomSliceView.getGlobalZoomFactor( );
//                m_multiDicomSliceView.setGlobalZoomFactor( (float)1f );
//                m_multiDicomSliceView.saveAllROIInformationToFile( m_verboseCheckBox.isSelected( ), m_beginBaseLine, m_endBaseLine, m_maxPerf, m_beginReperfusion );
//                m_multiDicomSliceView.setGlobalZoomFactor( oldZoom );
//            }
//            else
//            {
//                oldZoom = m_multiDicomSliceFourView.getGlobalZoomFactor( );
//                m_multiDicomSliceFourView.setGlobalZoomFactor( (float)1f );
//                m_multiDicomSliceFourView.saveAllROIInformationToFile( m_verboseCheckBox.isSelected( ), m_beginBaseLine, m_endBaseLine, m_maxPerf, m_beginReperfusion );
//                m_multiDicomSliceFourView.setGlobalZoomFactor( oldZoom );
//            }
        }
}//GEN-LAST:event_m_writeROIInformationButtonActionPerformed

    private void m_recalculateTexturesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_recalculateTexturesButtonActionPerformed
        
        for ( int i = m_textureTypeSelectionTable.getRowCount( ); i > 0; i-- ) 
        {
            ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).removeRow( i - 1 );
        }
        
        if ( this.m_scalingFactorOriginalValues == -1.0 ) 
        {
            computeAndDisplayTextures( m_minumumPixelValueSlider.getValue( ),
                                       m_maximumPixelValueSlider.getValue( ),
                                       m_xLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1,
                                       m_yLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1, 
                                       true );
        }
        else 
        {
            
            System.out.println( "m_scalingFactor : " + m_scalingFactorOriginalValues );
            System.out.println( "m_scalingFactorDifferenceImageTimeSeries : " + m_scalingFactorDifferenceImageTimeSeries );
            
            if ( m_isImageTimeSeries == false ) 
            {
                normalizeAndComputeAndDisplayTextures( m_scalingFactorOriginalValues,
                        m_minumumPixelValueSlider.getValue( ),
                        m_maximumPixelValueSlider.getValue( ),
                        m_xLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1,
                        m_yLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1, 
                        true, false );
            } 
            else 
            {
                normalizeAndComputeAndDisplayTextures( m_scalingFactorOriginalValues,
                        m_scalingFactorDifferenceImageTimeSeries,
                        m_minumumPixelValueSlider.getValue( ),
                        m_maximumPixelValueSlider.getValue( ),
                        m_xLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1,
                        m_yLocalStatisticsSizeComboBox.getSelectedIndex( ) + 1, 
                        true, true );
            }
        }
}//GEN-LAST:event_m_recalculateTexturesButtonActionPerformed

    private void m_applyThresholdingButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_applyThresholdingButtonActionPerformed
        
        
        int[] selectedTextures = m_textureTypeSelectionTable.getSelectedRows( );
        
        for ( int i = 0; i < selectedTextures.length; i++ ) 
        {
            
            DicomSliceCanvasPolyDraw pd = m_multiDicomSliceView.getView( selectedTextures[i] );
            
            pd.applyThressholdingAndShowAsVolumeAsOverlay( m_minimumIntensityThressholdSlider.getValue( ),
                                                           m_maximumIntensityThressholdSlider.getValue( ) );
            
        }
    }//GEN-LAST:event_m_applyThresholdingButtonActionPerformed

    private void m_maximumIntensityThressholdSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_m_maximumIntensityThressholdSliderStateChanged
        
        if ( !m_maximumIntensityThressholdSlider.getValueIsAdjusting( ) ) {
            System.out.println( "Slider changed: " + m_maximumIntensityThressholdSlider.getValue( ) );
            int[] selectedTextures = m_textureTypeSelectionTable.getSelectedRows( );
            
            for ( int i = 0; i < selectedTextures.length; i++ ) {
                
                DicomSliceCanvasPolyDraw pd = m_multiDicomSliceView.getView( selectedTextures[i] );
                
                pd.applyThressholdingAndShowAsVolumeAsOverlay( m_minimumIntensityThressholdSlider.getValue( ),
                        m_maximumIntensityThressholdSlider.getValue( ) );
                
            }
            
        }
}//GEN-LAST:event_m_maximumIntensityThressholdSliderStateChanged

    private void m_minimumIntensityThressholdSliderStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_m_minimumIntensityThressholdSliderStateChanged
        
        if ( !m_minimumIntensityThressholdSlider.getValueIsAdjusting( ) ) {
            System.out.println( "Slider changed: " + m_minimumIntensityThressholdSlider.getValue( ) );
            int[] selectedTextures = m_textureTypeSelectionTable.getSelectedRows( );
            
            for ( int i = 0; i < selectedTextures.length; i++ ) {
                
                DicomSliceCanvasPolyDraw pd = m_multiDicomSliceView.getView( selectedTextures[i] );
                
                pd.applyThressholdingAndShowAsVolumeAsOverlay( m_minimumIntensityThressholdSlider.getValue( ),
                        m_maximumIntensityThressholdSlider.getValue( )  );
                
            }   
        }
}//GEN-LAST:event_m_minimumIntensityThressholdSliderStateChanged

    private void m_addCaseToCADdBaseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_addCaseToCADdBaseButtonActionPerformed
        
        if ( m_isImageTimeSeries == false ) 
        {
            String tissue_type = (String)m_diagnosis2Dbase_ComboBox.getSelectedItem( );
            float oldZoom = m_multiDicomSliceView.getGlobalZoomFactor( );
            m_multiDicomSliceView.setGlobalZoomFactor( (float)1f );
            
            // add to the database where every contour is stored
            m_multiDicomSliceView.saveAllROIInformationToCAD_dBase_File( tissue_type, false, false );
            
            // add to the database where every patient examination results in one database entry
            m_multiDicomSliceView.saveAllROIInformationToCAD_dBase_File_1_Entry_Per_Patient( tissue_type, false, false );
            
            m_multiDicomSliceView.setGlobalZoomFactor( oldZoom );
        } 
        else 
        {
            
            // this is new code and needs to be checked!!
            String tissue_type = (String)m_diagnosis2Dbase_ComboBox.getSelectedItem( );
            float oldZoom = m_multiDicomSliceFourView.getGlobalZoomFactor( );
            m_multiDicomSliceFourView.setGlobalZoomFactor( (float)1f );
            
            // add to the database where every contour is stored
            m_multiDicomSliceFourView.saveAllROIInformationToCAD_dBase_File_ForTimeSeries( tissue_type, false, false );
            
            // add to the database where every patient examination results in one database entry
            m_multiDicomSliceFourView.saveAllROIInformationToCAD_dBase_File_1_Entry_Per_Patient( tissue_type, false, false );
            
            m_multiDicomSliceFourView.setGlobalZoomFactor( oldZoom );
        } 
    }//GEN-LAST:event_m_addCaseToCADdBaseButtonActionPerformed

    private void m_addTissueDBButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_addTissueDBButtonActionPerformed
        // Directory path where dbases can be found..
        String path = System.getProperty( "user.home" ) + 
                      System.getProperty( "file.separator" ) + "CAD_dbases_all";
        stringEnterframe = null;
        stringEnterframe = new EnterStringFrame( this, "New tissue type DBase", "new tissue", "Add new tissue label" );
        stringEnterframe.addActionListener( this );
    }//GEN-LAST:event_m_addTissueDBButtonActionPerformed

    private void m_diagnoseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_diagnoseButtonActionPerformed
        
        if ( m_isImageTimeSeries == false )
        {
            // first check whether at least one reqion has been drawn
            int numberOfROIs = m_multiDicomSliceView.getCurrentSliceCanvasPolyDraw( ).getRegionNames( ).length;
            if ( numberOfROIs == 1 )
            {   
                //Test whether two different differential diagnosis were chosen
                String diagnoseA = (String)m_diagnoseTestAlternative_A_ComboBox.getSelectedItem( );
                String diagnoseB = (String)m_diagnoseTestAlternative_B_ComboBox.getSelectedItem( );
                if ( diagnoseA.compareTo(diagnoseB) != 0 )
                {
                    // first save current ROI-data to a file
                    float oldZoom = m_multiDicomSliceView.getGlobalZoomFactor( );
                    m_multiDicomSliceView.setGlobalZoomFactor( (float)1f );

                    // m_multiDicomSliceView.saveAllROIInformationToCAD_dBase_File( "to_predict", true, true );

                    m_multiDicomSliceView.saveAllROIInformationToCAD_dBase_File_1_Entry_Per_Patient( "to_predict", true, true );

                    m_multiDicomSliceView.setGlobalZoomFactor( oldZoom );


                    // now create the training database on the fly!
                    String input_path  = System.getProperty( "user.home" ) + 
                                         System.getProperty( "file.separator" ) + "CAD" +
                                         System.getProperty( "file.separator" ) + "CAD_dbases_1_i_pp" 
                                                             + System.getProperty( "file.separator" );

                    String output_path = System.getProperty( "user.home" ) + 
                                         System.getProperty( "file.separator" ) + "CAD" +
                                         System.getProperty( "file.separator" ) + "CAD_stats" 
                                                             + System.getProperty( "file.separator" );


                    File stats = new File( output_path );

                    if ( stats.exists( ) == false )
                    {
                        stats.mkdir( );
                    }

                    String fileNameAlternative_A = (String)this.m_diagnoseTestAlternative_A_ComboBox.getSelectedItem( ) + "_1_inst_pp.txt";
                    String fileNameAlternative_B = (String)this.m_diagnoseTestAlternative_B_ComboBox.getSelectedItem( ) + "_1_inst_pp.txt";

                    // Also define two label strings
                    String alternativeA= (String)this.m_diagnoseTestAlternative_A_ComboBox.getSelectedItem( );
                    String alternativeB= (String)this.m_diagnoseTestAlternative_B_ComboBox.getSelectedItem( );

                    if ( fileNameAlternative_A.compareTo(fileNameAlternative_B) != 0 )
                    {

                        String []filename={ input_path + fileNameAlternative_A, 
                                            input_path + fileNameAlternative_B };

                        try
                        {

                            File file = new File( output_path + "stats.csv" );
                            File file_new = null;

                            if ( file.exists( ) == false )
                            {
                                file.createNewFile( );
                            }
                            else
                            {
                                // if it exist first delete it
                                file.delete( );
                                file = new File( output_path + "stats.csv" );

                                // once deleted create a new one!
                                if ( file.exists( ) == false )
                                {
                                    file.createNewFile( );
                                }
                            }

                            FileWriter output = new FileWriter( file );

                            boolean first_time = true;

                            for( int i = 0; i < filename.length; i++ )
                            {
                                BufferedReader objBufferedReader = new BufferedReader( new FileReader( filename[i] ) );

                                String line;
                                int counter = 0;
                                while ( ( line = objBufferedReader.readLine( ) ) != null )
                                {
                                    line=line.replace( " ", "" );
                                    if ( ( counter != 0 ) && ( first_time == false ) )
                                    {
                                        output.write( line + "\n" );
                                    }
                                    if ( ( counter == 0 ) && ( first_time == true ) )
                                    {
                                        output.write( line + "\n" );
                                        first_time = false;
                                    }
                                    counter++;  
                                }
                                objBufferedReader.close( );

                            }
                            output.close( );     
                        }
                        catch ( Exception e ) 
                        {
                            System.out.println( "IO-error while creating R-input datafile" );
                            e.printStackTrace( );
                        }
                    }

                    try
                    {
                        // data structure for msd plot display
                        File plotFileMSD = null;
                        ImageIcon plotImageMSD = null;

                        // data structure for  
                        File varImpPlotFile = null;
                        ImageIcon varImpPlot = null;

                        // Now perform the R-code
                        RCaller r_caller = new RCaller( );

                        boolean isLinux = System.getProperty("os.name").compareTo("Linux") == 0 ? true : false;
                        r_caller.setRscriptExecutable(GlobalSettings.getInstance().getRExecutable());
                        String separator = isLinux ? "/" : "\\\\";
                        r_caller.cleanRCode();
                        
                        //r_caller.setRscriptExecutable( System.getProperty( "user.home" ) + System.getProperty( "file.separator" ) + "CAD" + System.getProperty( "file.separator" ) + "CAD_R_scripts" + System.getProperty( "file.separator" ) + "Rscript" );
                        //r_caller.setRscriptExecutable( "/usr/bin/Rscript" );

                        //OutputStream outputStreamToR = r_caller.getOutputStreamToR( );

                        RCode r_code = new RCode( );
                        r_code.R_require( "randomForest" );
                        r_code.R_require( "lattice" );
                        r_code.R_require( "Hmisc" );
                        r_code.R_require( "party" );
                        r_code.R_require( "graphics" );
                        r_code.R_require( "Runiversal" );

                        // load the training database 
                        r_code.addRCode( "class_data <- read.csv( \"/home/slotboom/CAD/CAD_stats/stats.csv\", sep=\",\")" );

                        // load the data to predict the diagnose
                        r_code.addRCode( "to_predict_data <- read.csv( \"/home/slotboom/CAD/CAD_entry_2_predict/to_predict.txt\", sep=\",\")" );

    //                    r_code.addRCode( "all_data_alternative_a <- droplevels( subset( class_data, tissue_type==\"tumor\" ) )" );
    //                    r_code.addRCode( "all_data_alternative_b <- droplevels( subset( class_data, tissue_type==\"NAWM\" ) )" );

                        r_code.addRCode( "all_data_alternative_a <- droplevels( subset( class_data, tissue_type==\"" + alternativeA + "\" ) )" );
                        r_code.addRCode( "all_data_alternative_b <- droplevels( subset( class_data, tissue_type==\"" + alternativeB + "\" ) )" );

                        r_code.addRCode( "all_data <- rbind( all_data_alternative_a, all_data_alternative_b )" );
                        r_code.addRCode( "tissue_labels <- all_data[, 1]" );

                        r_code.addRCode( "all_numeric_data <- all_data[,c(44:76)]" );
                        r_code.addRCode( "all_numeric_to_predict_data <- to_predict_data[,c(44:76)]" );

                        r_code.addRCode( "set.seed(72)" );
                        r_code.addRCode( "ranFor_class.rf <- randomForest( y=tissue_labels, x=all_numeric_data, importance=TRUE, proximity=TRUE, replace=TRUE )" );
                        r_code.addRCode( "diagnosis <- predict( ranFor_class.rf, newdata=all_numeric_to_predict_data )" );


                        r_code.addRCode( "levelsarr <- levels(diagnosis)");
                        r_code.addRCode( "diagnosis_string <- c(levelsarr[diagnosis])");

                        r_code.addRCode( "diagnosis_arr <- c(diagnosis)" );
                        //r_code.addRCode( "diagnosis_int <- c(as.numeric(diagnosis))" );


                        r_code.addRCode( "print( ranFor_class.rf )" );
                        // r_code.addRCode( "print( diagnosis )" );
                        r_code.addRCode( "round( importance( ranFor_class.rf ), 2 )" );

                        r_code.addRCode( "par(mfrow=c(2,2))" );

                        plotFileMSD = r_code.startPlot( );
                        r_code.addRCode( "MDSplot( ranFor_class.rf, tissue_labels )" );
                        r_code.endPlot( );

                        varImpPlotFile = r_code.startPlot( );
                        r_code.addRCode( "varImpPlot(ranFor_class.rf)" );
                        r_code.endPlot( );

                        r_caller.setRCode( r_code );

                        r_caller.runAndReturnResult( "diagnosis_string" );

                        //r_caller.runAndReturnResult( "ranFor_class.rf" ); 

                        System.out.println( r_caller.getParser( ).getXMLFileAsString( ) );

                        String[] diagnosis = r_caller.getParser( ).getAsStringArray( "diagnosis_string" );

                        Dialogs.messageBox( this, "The most likely diagnosis for this patient is : " + diagnosis[0] );

                        // r_caller.runOnly( );

                        plotImageMSD = r_code.getPlot( plotFileMSD );
                        r_code.showPlot( plotFileMSD );

                        varImpPlot = r_code.getPlot( varImpPlotFile );
                        r_code.showPlot( varImpPlotFile );

            //            code2.R_require("graphics");
            //                
            //            File plotFile = null;
            //            ImageIcon plotImage = null;
            //
            //            caller2.setRscriptExecutable("/usr/bin/Rscript");
            //            code2.R_require("lattice");
            //            code2.addRCode( "mfrow=c(2,2)" );
            //            plotFile = code2.startPlot( );
            //            code2.addRCode( "xyplot(rnorm(100)~1:100)" );
            //            caller2.setRCode( code2 );
            //            caller2.runOnly( );
            //            plotImage = code2.getPlot( plotFile );
            //            code2.showPlot( plotFile );    


            //            System.out.print( outputStreamToR.toString( ) );

                    }
                    catch ( IOException err )
                    {
                        System.out.println( "Cannot create plot" );
                    }
                }   // end of different tissuetypes selected
                else
                {
                    Dialogs.messageBox( this, "Select to different alternative diagnosis to test!" );
                }
            }   // end if new ROI defined
            else
            {
                Dialogs.messageBox( this, "First draw a region in a specific tissue type to diagnose!" );
            }
        }
        else
        {
            Dialogs.messageBox( this, "Not defined yet for image time series!" );
        }
    }//GEN-LAST:event_m_diagnoseButtonActionPerformed

    private void m_synchronizedViewsCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_synchronizedViewsCheckBoxActionPerformed

        if ( m_synchronizedViewsCheckBox.isSelected( ) == true ) 
        {
            m_multiDicomSliceView.enableSyncedDisplayMode( );
        } 
        else 
        {
            m_multiDicomSliceView.disableSyncedDisplayMode( );
        }
    }//GEN-LAST:event_m_synchronizedViewsCheckBoxActionPerformed

    private void m_showScaleCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_showScaleCheckBoxActionPerformed
        
        m_multiDicomSliceView.showImagePixelScales( m_showScaleCheckBox.isSelected( ) );
        m_multiDicomSliceFourView.showImagePixelScales( m_showScaleCheckBox.isSelected( ) );
        
    }//GEN-LAST:event_m_showScaleCheckBoxActionPerformed

    private void m_colorTableSelectionComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_colorTableSelectionComboBoxActionPerformed

        int useColorTable = m_colorTableSelectionComboBox.getSelectedIndex( );
        if ( useColorTable == MultiDicomSliceView.GRAY_VALUE ) {
            m_multiDicomSliceView.setGlobalColorTable( MultiDicomSliceView.GRAY_VALUE );
            m_multiDicomSliceFourView.setGlobalColorTable( MultiDicomSliceView.GRAY_VALUE );
        }
        if ( useColorTable == MultiDicomSliceView.RAINBOW ) {
            m_multiDicomSliceView.setGlobalColorTable( MultiDicomSliceView.RAINBOW );
            m_multiDicomSliceFourView.setGlobalColorTable( MultiDicomSliceView.RAINBOW );
        }
        if ( useColorTable == MultiDicomSliceView.INV_RAINBOW ) {
            m_multiDicomSliceView.setGlobalColorTable( MultiDicomSliceView.INV_RAINBOW );
            m_multiDicomSliceFourView.setGlobalColorTable( MultiDicomSliceView.INV_RAINBOW );
        }
    }//GEN-LAST:event_m_colorTableSelectionComboBoxActionPerformed

    private void m_imageZoomFactorComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_imageZoomFactorComboBoxActionPerformed

        float zoomFactor = ( new Float( "" + m_imageZoomFactorComboBox.getSelectedItem( ) ) ).floatValue( );
        m_multiDicomSliceView.setGlobalZoomFactor( (float)zoomFactor );
        m_multiDicomSliceFourView.setGlobalZoomFactor( (float)zoomFactor );
    }//GEN-LAST:event_m_imageZoomFactorComboBoxActionPerformed

    
    /**
     * This method is needed to discriminate whether the fourview is selected
     * or the texture view!
     * @param evt 
     */
    private void m_textureMapsViewingTabbedPaneStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_m_textureMapsViewingTabbedPaneStateChanged
        
        JTabbedPane sourceTabbedPane = (JTabbedPane) evt.getSource( );
        int index = sourceTabbedPane.getSelectedIndex( );
        if ( index == 0 )
        {
            m_selectedMultiDicomSliceView = this.m_multiDicomSliceView;
            m_selectedMultiDicomSliceViewTabNumber = 0;
        }
        else
        {
            m_selectedMultiDicomSliceView = this.m_multiDicomSliceFourView;
            m_selectedMultiDicomSliceViewTabNumber = 1;
        }
    }//GEN-LAST:event_m_textureMapsViewingTabbedPaneStateChanged



    /**
     * Computes the boolean overlay from the given thressholds.
     * @param evt
     */

    /**
     * Writes all information to one user defined file
     * @param evt
     */

    /**
     * Called whenever a registered component raises an ActionEvent
     * @param e
     */
    public void actionPerformed(ActionEvent e)
    {
        
        if ( e.getSource() == this.m_addArbitraryShapedROIs )
        {
            m_multiDicomSliceView.setToAddArbitraryShapeROIsMode( );
        }
        
        if ( e.getSource() == this.m_addCircularShapedROIs )
        {
            m_multiDicomSliceView.setToAddCircularShapeROIsMode( );
        }
        
        if ( e.getSource( ) == this.m_itemShowToolTipText )
        {
            if ( this.m_itemShowToolTipText.isSelected( ) == true )
            {
                m_multiDicomSliceView.setShowToolTipText( true );
                m_multiDicomSliceFourView.setShowToolTipText( true );
            }
            else
            {
                m_multiDicomSliceView.setShowToolTipText( false );
                m_multiDicomSliceFourView.setShowToolTipText( false );
            }
        }
        
        if ( e.getSource( ) == stringEnterframe  )
        {
            String newLabel = stringEnterframe.getText( );
            m_diagnoseTestAlternative_A_ComboBox.addItem( newLabel );
            m_diagnoseTestAlternative_B_ComboBox.addItem( newLabel );
            m_diagnosis2Dbase_ComboBox.addItem( newLabel );
        }
        
        if ( e.getSource( ) == m_createXYScatterPlot )
        {

            if ( m_textureTypeSelectionTable.getSelectedRowCount( ) == 2 )
            {
                System.out.println( "Two textures selected: can make a scatter plot!" );

                int[] textureNumbers = m_textureTypeSelectionTable.getSelectedRows( );

//                int[] data1 = this.m_multiDicomSliceView.getView( textureNumbers[0] ).getImagePixelData( );
//                int[] data2 = this.m_multiDicomSliceView.getView( textureNumbers[1] ).getImagePixelData( );
                
                int[] data1 = m_selectedMultiDicomSliceView.getView( textureNumbers[0] ).getImagePixelData( );
                int[] data2 = m_selectedMultiDicomSliceView.getView( textureNumbers[1] ).getImagePixelData( );

                System.out.println( "data1[].length = " + data1.length );
                System.out.println( "data2[].length = " + data2.length );

                XYPlotFrame pFrame = new XYPlotFrame( "Texture scatter plot", this );
                Curve cur = new Curve( data1, data2, Color.BLACK );
                cur.setStyle( Curve.POINTS_ONLY );

                pFrame.addCurve(cur, m_isImageTimeSeries );
            }
            else
            {
                Dialogs.messageBox( this, "Scatter plot: please select two textures!" );
            }

        }


        if ( e.getSource( ) == m_createXYScatterPlotOfROIs )
        {

            if ( m_textureTypeSelectionTable.getSelectedRowCount( ) == 2 )
            {
                System.out.println( "Two textures selected: can make a scatter plot!" );

                XYPlotFrame pFrame = new XYPlotFrame( "Texture scatter plot", this );

                int[] textureNumbers = m_textureTypeSelectionTable.getSelectedRows( );

                int[] pixVals1;
                int[] pixVals2;

                int selectedSlice   = m_selectedMultiDicomSliceView.getView( textureNumbers[0] ).getCurrentlyDisplayedImageNumber( );
                LinkedList<RegioPolygon> polygonList = m_selectedMultiDicomSliceView.getView( textureNumbers[0] ).getPolygonList( selectedSlice );
                int numPolys        = m_selectedMultiDicomSliceView.getView( textureNumbers[0] ).getNumberOfPolygons( );

                System.out.println( "Number of polygons" + numPolys );
                
                Curve cur  = null;
                Curve cur1 = null;

                for ( int i = 0; i < numPolys; i++ )
                {

                    pixVals1 =  m_selectedMultiDicomSliceView.getView( textureNumbers[0] + 1 ).getPixelValues( selectedSlice, polygonList.get( i ) );
                    pixVals2 =  m_selectedMultiDicomSliceView.getView( textureNumbers[1] + 1 ).getPixelValues( selectedSlice, polygonList.get( i ) );

                    System.out.println( "pixVals1.length = " + pixVals1.length );
                    System.out.println( "pixVals2.length = " + pixVals2.length );

                    Color col = Color.BLACK;

                    switch ( i )
                    {
                        case 0:  col = Color.BLUE;           break;
                        case 1:  col = Color.RED;            break;
                        case 2:  col = Color.CYAN;           break;
                        case 3:  col = Color.DARK_GRAY;      break;
                        case 4:  col = Color.GRAY;           break;
                        case 5:  col = Color.GREEN;          break;
                        case 6:  col = Color.MAGENTA;        break;
                        case 7:  col = Color.ORANGE;         break;
                        default: col = Color.BLACK;          break;
                    }

                    System.out.println( col.getBlue( ) + " " + col.getRed( ) + " " + col.getGreen( ) );
                    
//                    if ( i == 0 )
//                    {
                        System.out.println( "i==0" );
                        cur = new Curve( pixVals1, pixVals2, col );
                        cur.setStyle( Curve.FAT_POINTS_ONLY );
                        pFrame.addCurve( cur, false );
//                    }

//                    if ( i == 1 )
//                    {
//                        System.out.println( "i==1" );
//                        cur1 = new Curve( pixVals1, pixVals2, Color.BLUE );
//                        cur1.setStyle( Curve.POINTS_ONLY );
//                        pFrame.addCurve( cur1, false );
//                    }
                }
            }
            else
            {
                Dialogs.messageBox( this, "Scatter plot: please select two textures!" );
            }

        }

        if ( e.getSource( ) == m_multiDicomSliceFourView )
        {
            if ( e.getID( ) == MultiDicomSliceView.SELECT_IMAGE_EVENT )
            {
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.SELECT_ROI_ID );
            }

            if ( e.getID( ) == MultiDicomSliceView.ADD_ROI_MODE_SELECTED_EVENT )
            {
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.ADD_ROI_ID );
            }
            
            if ( e.getID( ) == MultiDicomSliceView.CUT_ROI_MODE_SELECTED_EVENT )
            {
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.ROI_CUT_ID );
            }
            
            if ( e.getID( ) == MultiDicomSliceView.DELETE_ROI_MODE_SELECTED_EVENT )
            {
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.DELETE_ROI_ID );
            }
            if ( e.getID( ) == MultiDicomSliceView.SELECT_ROI_MODE_SELECTED_EVENT )
            {
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.SELECT_ROI_ID );
            }

//            if ( e.getID( ) == MultiDicomSliceView.MOVE_ROI_MODE_SELECTED_EVENT )
//            {
//                //this.notifyActionListeners( DRAG_ROI_MODE_SELECTED_EVENT  );
//                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.DRAG_ROI_ID );
//            }

            if ( e.getID( ) == MultiDicomSliceView.RESIZE_ROI_MODE_SELECTED_EVENT )
            {
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.EDIT_ROI_ID );
            }

            // user has drawn a polygon, and the polygon was closed
            if ( e.getID( ) == MultiDicomSliceView.POLYGON_CLOSED_EVENT )
            {
                m_multiDicomSliceFourView.addWorkingPolygonToAllViews(false );
            }

            // user has drawn a polygon, and the polygon was closed
            if ( e.getID( ) == MultiDicomSliceView.POLYGON_CHANGED_EVENT )
            {
                m_multiDicomSliceFourView.changeAllPolygonsInAllViewsAccordingly( );
            }

            if ( e.getID( ) == MultiDicomSliceView.POLYGON_RENAME_EVENT )
            {
                m_oldPolygonName = m_multiDicomSliceFourView.getCurrentView( ).getRegioNamebeforeRenaming( );
                m_multiDicomSliceFourView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( m_oldPolygonName );
            }
        }

        
        if ( e.getSource( ) == m_multiDicomSliceView )
        {
            if ( e.getID( ) == MultiDicomSliceView.SELECT_IMAGE_EVENT )
            {
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.SELECT_ROI_ID );
            }

            if ( e.getID( ) == MultiDicomSliceView.ADD_ROI_MODE_SELECTED_EVENT )
            {
                //this.notifyActionListeners( ADD_ROI_MODE_SELECTED_EVENT );
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.ADD_ROI_ID );
            }
            if ( e.getID( ) == MultiDicomSliceView.CUT_ROI_MODE_SELECTED_EVENT )
            {
                //this.notifyActionListeners( CUT_ROI_MODE_SELECTED_EVENT );
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.ROI_CUT_ID );
            }
            if ( e.getID( ) == MultiDicomSliceView.DELETE_ROI_MODE_SELECTED_EVENT )
            {
                //this.notifyActionListeners( DELETE_ROI_MODE_SELECTED_EVENT );
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.DELETE_ROI_ID );
            }
            if ( e.getID( ) == MultiDicomSliceView.SELECT_ROI_MODE_SELECTED_EVENT )
            {
                //this.notifyActionListeners( SELECT_ROI_MODE_SELECTED_EVENT );
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.SELECT_ROI_ID );
            }

//            if ( e.getID( ) == MultiDicomSliceView.MOVE_ROI_MODE_SELECTED_EVENT )
//            {
//                //this.notifyActionListeners( DRAG_ROI_MODE_SELECTED_EVENT  );
//                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.DRAG_ROI_ID );
//            }

            if ( e.getID( ) == MultiDicomSliceView.RESIZE_ROI_MODE_SELECTED_EVENT )
            {
                //this.notifyActionListeners( RESIZE_ROI_MODE_SELECTED_EVENT );
                m_regionModeToolBar.setSelectedButton( RegionModeToolBar.EDIT_ROI_ID );
            }

            // user has drawn a polygon, and the polygon was closed
            if ( e.getID( ) == MultiDicomSliceView.POLYGON_CLOSED_EVENT )
            {
                m_multiDicomSliceView.addWorkingPolygonToAllViews( true );
                
                // to enforce synced polygons in the two multiviews:
                RegioPolygon p = new RegioPolygon( m_multiDicomSliceView.getWorkingPolygon( ) );
                m_multiDicomSliceFourView.setWorkingPolygon( p );
                m_multiDicomSliceFourView.addWorkingPolygonToAllViews( false );
//                
//                RegioPolygon pWork = new RegioPolygon( m_multiDicomSliceView.getCurrentView( ).getSelectedPolygon( ) );
//                m_multiDicomSliceFourView.deleteAllRegionsOfAllViews( );
//                m_multiDicomSliceFourView.setWorkingPolygon( pWork );
//                m_multiDicomSliceFourView.changeAllPolygonsInAllViewsAccordingly( );
            }

            // user has cut the selected polygon, into two polygons
            if ( e.getID( ) == MultiDicomSliceView.POLYGON_CUT_EVENT )
            {
                // first figure out which DicomSliceCanvasPolyDraw is selected ..
//                DicomSliceCanvasPolyDraw canvas = m_multiDicomSliceView.getCurrentSliceCanvasPolyDraw( );
//                
//                int selectedImageNumber = m_multiDicomSliceView.getCurrentSliceCanvasPolyDraw( ).getDisplayedImageNumber( );
//                
//                RegioPolygon polygonWhichWasCut = canvas.getLastCutPolygon( );
//                RegioPolygon polygonOneAfterCut = canvas.getPolygonSegmentOneAfterCut( );
//                RegioPolygon polygonTwoAfterCut = canvas.getPolygonSegementTwoAfterCut( );
                if ( this.m_isImageTimeSeries )
                {
                    m_multiDicomSliceView.cutAllROIsAccordingToLastChangedPolygonOfAllViews( );
                    
                    Vector< LinkedList<RegioPolygon>> regPols = m_multiDicomSliceView.getCurrentSliceCanvasPolyDraw( ).getAllPolygonsVector( );
                    m_multiDicomSliceFourView.cutAllROIsAccordingToLastChangedPolygonOfAllViews(regPols);
                }
                else
                {
                    // not defined yet!!
                }
                
            }
            
            // user has drawn a polygon, and the polygon was closed
            if ( e.getID( ) == MultiDicomSliceView.POLYGON_CHANGED_EVENT )
            {
                m_multiDicomSliceView.changeAllPolygonsInAllViewsAccordingly( );
                
                RegioPolygon pWork = m_multiDicomSliceView.getWorkingPolygon( );
                
                if ( pWork != null )
                {
                    RegioPolygon pWorkAdd = new RegioPolygon( pWork );
                    // m_multiDicomSliceFourView.deleteAllRegionsOfAllViews( );
                    m_multiDicomSliceFourView.setWorkingPolygon( pWorkAdd );
                    m_multiDicomSliceFourView.changeAllPolygonsInAllViewsAccordingly( );
                }
                else
                {   
                    // remove all regions from the 4View
                    m_multiDicomSliceFourView.deleteAllRegionsOfAllViews( ); 
                    // get the new list of regio polygons
                    LinkedList<RegioPolygon> definedPolygons = m_multiDicomSliceView.getView( 0 ).getPolygonList( );
                    m_multiDicomSliceFourView.setAllRegionsOfAllViews( definedPolygons );
                }
            }

            if ( e.getID( ) == MultiDicomSliceView.POLYGON_RENAME_EVENT )
            {
                m_oldPolygonName = m_multiDicomSliceView.getCurrentView( ).getRegioNamebeforeRenaming( );
                String newPolygonName = m_multiDicomSliceView.getCurrentView( ).getSelectedPolygon().getLabel( );
                m_multiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( m_oldPolygonName );
                
                // now update the 4View
                //RegioPolygon pWork = new RegioPolygon( m_multiDicomSliceView.getCurrentView( ).getSelectedPolygon( ) );
                //m_multiDicomSliceFourView.deleteAllRegionsOfAllViews( );
                //m_multiDicomSliceFourView.setWorkingPolygon( pWork );
                m_multiDicomSliceFourView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( m_oldPolygonName, newPolygonName );
            }
        }

        
        if ( e.getSource( ) == m_deleteAllROIsInAllViews )
        {
            
            m_multiDicomSliceView.deleteAllRegionsOfAllViews( );
            m_multiDicomSliceFourView.deleteAllRegionsOfAllViews( );
            
        }
        
        
        if ( e.getSource( ) == m_renameCurrentROIToNAWM )
        {
            DicomSliceCanvasPolyDraw view = m_selectedMultiDicomSliceView.getCurrentView( );
            if ( view != null )
            {
                RegioPolygon selectedPolygon = view.getSelectedPolygon( );
                if ( selectedPolygon != null )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).renameSelectedPolygon( "NAWM" );
                    m_selectedMultiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( selectedPolygon.getLabel( ) );
                }
            }
        }

        if ( e.getSource( ) == m_renameCurrentROIToMetastasisTotalTumorTissue )
        {
            DicomSliceCanvasPolyDraw view = m_selectedMultiDicomSliceView.getCurrentView( );
            if ( view != null )
            {
                RegioPolygon selectedPolygon = view.getSelectedPolygon( );
                if ( selectedPolygon != null )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).renameSelectedPolygon( "TOTAL_MET" );
                    m_selectedMultiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( selectedPolygon.getLabel( ) );
                }
            }
        }

        if ( e.getSource( ) == m_renameCurrentROIToGBMTotalTumorTissue )
        {
            DicomSliceCanvasPolyDraw view = m_selectedMultiDicomSliceView.getCurrentView( );
            if ( view != null )
            {
                RegioPolygon selectedPolygon = view.getSelectedPolygon( );
                if ( selectedPolygon != null )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).renameSelectedPolygon( "TOTAL_GBM" );
                    m_selectedMultiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( selectedPolygon.getLabel( ) );
                }
            }
        }

        if ( e.getSource( ) == m_renameCurrentROIToNonEnhancing_MET_Edema )
        {
            DicomSliceCanvasPolyDraw view = m_selectedMultiDicomSliceView.getCurrentView( );
            if ( view != null )
            {
                RegioPolygon selectedPolygon = view.getSelectedPolygon( );
                if ( selectedPolygon != null )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).renameSelectedPolygon( "NET_EDE_MET" );
                    m_selectedMultiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( selectedPolygon.getLabel( ) );
                }
            }
        }

        if ( e.getSource( ) == m_renameCurrentROIToNonEnhancing_GBM_Edema )
        {
            DicomSliceCanvasPolyDraw view = m_selectedMultiDicomSliceView.getCurrentView( );
            if ( view != null )
            {
                RegioPolygon selectedPolygon = view.getSelectedPolygon( );
                if ( selectedPolygon != null )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).renameSelectedPolygon( "NET_EDE_GBM" );
                    m_selectedMultiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( selectedPolygon.getLabel( ) );
                }
            }
        }
        
        if ( e.getSource( ) == m_renameCurrentROITo_CET_NECR_MET_TumorTissue )
        {
            DicomSliceCanvasPolyDraw view = m_selectedMultiDicomSliceView.getCurrentView( );
            if ( view != null )
            {
                RegioPolygon selectedPolygon = view.getSelectedPolygon( );
                if ( selectedPolygon != null )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).renameSelectedPolygon( "CET_NECR_MET" );
                    m_selectedMultiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( selectedPolygon.getLabel( ) );
                }
            }
        }
        
        if ( e.getSource( ) == m_renameCurrentROITo_CET_NECR_GBM_TumorTissue )
        {
            DicomSliceCanvasPolyDraw view = m_selectedMultiDicomSliceView.getCurrentView( );
            if ( view != null )
            {
                RegioPolygon selectedPolygon = view.getSelectedPolygon( );
                if ( selectedPolygon != null )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).renameSelectedPolygon( "CET_NECR_GBM" );
                    m_selectedMultiDicomSliceView.renameAllROIsAccordingToLastChangedPolygonOfAllViews( selectedPolygon.getLabel( ) );
                }
            }
        }

        if ( e.getSource( ) == m_saveROIsMenuItem )
        {
            if ( m_selectedMultiDicomSliceViewTabNumber == 0 )
            {
                m_multiDicomSliceView.getCurrentView( ).saveRegionsToFile( false );
            }
            else
            {
                Dialogs.messageBox( this, "ROI saving is not defined for the time averaged view!!" );
            }
        }
        
        if ( e.getSource( ) == m_createScreenShotOfMultiView )
        {
            m_selectedMultiDicomSliceView.createScreenshot( null );
        }

        if ( e.getSource( ) == m_loadROIsMenuItem )
        {
            if ( m_isImageTimeSeries == true )
            {
                LinkedList<RegioPolygon> polysToAdd = m_multiDicomSliceView.getCurrentView( ).loadRegionsFromFile( true, false );
                // m_multiDicomSliceView.copyAllROIsToOtherViews( polysToAdd );
                for ( int i = 0; i < polysToAdd.size( ); i++ )
                {
                    m_multiDicomSliceView.setWorkingPolygon( new RegioPolygon( polysToAdd.get( i ) ) );
               
                    m_multiDicomSliceView.addWorkingPolygonToAllViewsFromFile( );
                    
                    // correct??
                    m_multiDicomSliceFourView.setWorkingPolygon( new RegioPolygon( polysToAdd.get( i ) ) );
                    m_multiDicomSliceFourView.addWorkingPolygonToAllViewsFromFile( );
                }
            }
            else
            {
                LinkedList<RegioPolygon> polysToAdd = m_multiDicomSliceView.getCurrentView( ).loadRegionsFromFile( false, true );
                m_multiDicomSliceView.copyAllROIsToOtherViews( polysToAdd );
            }
        }

        if ( e.getSource( ) == m_regionModeToolBar )
        {

            if ( m_multiDicomSliceView.getCurrentView( ) != null )
            {
                if ( e.getID( ) == RegionModeToolBar.SELECT_ROI_ID )
                {   
                    m_selectedMultiDicomSliceView.getCurrentView( ).setMode( DicomSliceCanvasPolyDraw.SELECT_MODE );
                    m_selectedMultiDicomSliceView.getCurrentView( ).repaint( );
                }

                if ( e.getID( ) == RegionModeToolBar.ADD_ROI_ID )
                {   
                    m_selectedMultiDicomSliceView.getCurrentView( ).setMode( DicomSliceCanvasPolyDraw.POLYGON_DRAW_MODE );
                    m_selectedMultiDicomSliceView.getCurrentView( ).repaint( );
                }

                if ( e.getID( ) == RegionModeToolBar.DELETE_ROI_ID )
                {                    
                    m_selectedMultiDicomSliceView.getCurrentView( ).setMode( DicomSliceCanvasPolyDraw.DELETE_MODE );
                    m_selectedMultiDicomSliceView.getCurrentView( ).repaint( );
                }

                if ( e.getID( ) == RegionModeToolBar.ROI_CUT_ID )
                {                    
                    m_selectedMultiDicomSliceView.getCurrentView( ).setMode( DicomSliceCanvasPolyDraw.CUT_MODE );
                    m_selectedMultiDicomSliceView.getCurrentView( ).repaint( );
                }

                if ( e.getID( ) == RegionModeToolBar.EDIT_ROI_ID )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).setMode( DicomSliceCanvasPolyDraw.RESIZE_MODE );
                    m_selectedMultiDicomSliceView.getCurrentView( ).repaint( );
                }

                if ( e.getID( ) == RegionModeToolBar.DRAG_ROI_ID )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).setMode( DicomSliceCanvasPolyDraw.DRAG_MODE );
                    m_selectedMultiDicomSliceView.getCurrentView( ).repaint( );
                }

                if ( e.getID( ) == RegionModeToolBar.AFFINE_TRANSFORM_ROI_ID )
                {
                    m_selectedMultiDicomSliceView.getCurrentView( ).handleAffineTransformOfPolygon( );
                    m_selectedMultiDicomSliceView.getCurrentView( ).repaint( );
                }
            }            
        }
    }


    /**
     *
     * Computes the four views based on the time point thressholds
     *
     * @param texture
     * @param width
     * @param height
     * @return
     */

    public int[] computeFourViews( int[] texture, int width, int height )
    {

        int[] maps = new int[ width * height * 4 ];

        int[] baseline = new int[ width * height ];
        int[] inflow = new int[ width * height];
        int[] outflow = new int[ width * height ];
        int[] reperfusion = new int[ width * height ];
        
        int beginBaseline = 0;
        if ( ( this.m_isImageTimeSeries == true ) && ( m_beginBaseLine == 0 ) )
        {
            beginBaseline = 1;
        }
        
        // sum!
        for ( int i = beginBaseline; i < m_endBaseLine; i++ )
        {
            for ( int j = 0 ; j< height; j++ )
            {
                for ( int k = 0 ; k < width; k++ )
                {
                    maps[ j * width + k ] += texture[ i * width * height + j * width + k ];
                }
             }
        }

        // compute the average of first image ...
        for ( int j = 0 ; j< height; j++ )
        {
            for ( int k = 0 ; k < width; k++ )
            {
                maps[ j * width + k ] = (int)( maps[ j * width + k ] / ( m_endBaseLine - beginBaseline ) );
            }
        }

        // sum!
        for ( int i = m_endBaseLine; i <= m_maxPerf; i++ )
        {
            for ( int j = 0 ; j< height; j++ )
            {
                for ( int k = 0 ; k < width; k++ )
                {
                    maps[ width * height + j * width + k ] += texture[ i * width * height + j * width + k ];
                }
            }
        }

        // compute the average of the second image ...
        for ( int j = 0 ; j < height; j++ )
        {
            for ( int k = 0 ; k < width; k++ )
            {
                maps[ width * height + j * width + k ] = (int)( maps[ width * height + j * width + k ] / ( m_maxPerf - m_endBaseLine + 1 ) );
            }
        }

        // sum!
        for ( int i = m_maxPerf; i < m_beginReperfusion; i++ )
        {
            for ( int j = 0 ; j< height; j++ )
            {
                for ( int k = 0 ; k < width; k++ )
                {
                    maps[ (2 * width * height) + j * width + k ] += texture[ i * width * height + j * width + k ];
                }
            }
        }

        // compute the average of the third image ...
        for ( int j = 0 ; j < height; j++ )
        {
            for ( int k = 0 ; k < width; k++ )
            {
                maps[ 2 * ( width * height ) + j * width + k ] = (int)( maps[ 2 * ( width * height ) + j * width + k ] / ( m_beginReperfusion - m_maxPerf ) );
            }
        }

        // sum!
        for ( int i = m_beginReperfusion; i < m_imageSeriesData.getNumberOfImages( ); i++ )
        {
            for ( int j = 0 ; j< height; j++ )
            {
                for ( int k = 0 ; k < width; k++ )
                {
                    maps[ (3 * width * height) + j * width + k ] += texture[ i * width * height + j * width + k ];
                }
            }
        }

        // compute the average of the fourth ..
        for ( int j = 0 ; j < height; j++ )
        {

            for ( int k = 0 ; k < width; k++ )
            {
                maps[ 3 * ( width * height ) + j * width + k ] = (int)( maps[ 3 * ( width * height ) + j * width + k ] / ( m_imageSeriesData.getNumberOfImages( ) - m_beginReperfusion  ) );
            }
            
        }

        return maps;
    }

    /**
    * @param args the command line arguments
    */
    public static void main( String args[] )
    {
        java.awt.EventQueue.invokeLater(new Runnable( )
        {
            public void run( )
            {
                new ImageStackTextureAnalyzerFrame( null, null, false, 0, 0, 0, 0 ).setVisible( true );
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel10;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel13;
    private javax.swing.JPanel jPanel14;
    private javax.swing.JPanel jPanel15;
    private javax.swing.JPanel jPanel16;
    private javax.swing.JPanel jPanel17;
    private javax.swing.JPanel jPanel18;
    private javax.swing.JPanel jPanel19;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel20;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JButton m_addCaseToCADdBaseButton;
    private javax.swing.JButton m_addTissueDBButton;
    private javax.swing.JButton m_applyThresholdingButton;
    private javax.swing.JComboBox m_colorTableSelectionComboBox;
    private javax.swing.JButton m_diagnoseButton;
    private javax.swing.JComboBox m_diagnoseTestAlternative_A_ComboBox;
    private javax.swing.JComboBox m_diagnoseTestAlternative_B_ComboBox;
    private javax.swing.JComboBox m_diagnosis2Dbase_ComboBox;
    private javax.swing.JButton m_forceLastNormalizationButton;
    private javax.swing.JComboBox m_imageZoomFactorComboBox;
    private javax.swing.JTextField m_lastNormFactorTextField;
    private javax.swing.JSlider m_maximumIntensityThressholdSlider;
    private javax.swing.JSlider m_maximumPixelValueSlider;
    private javax.swing.JSlider m_minimumIntensityThressholdSlider;
    private javax.swing.JSlider m_minumumPixelValueSlider;
    private org.spectrim.dicom.MultiDicomSliceView m_multiDicomSliceFourView;
    private org.spectrim.dicom.MultiDicomSliceView m_multiDicomSliceView;
    private javax.swing.JButton m_normalizeButton;
    private javax.swing.JButton m_recalculateTexturesButton;
    private javax.swing.JCheckBox m_showScaleCheckBox;
    private javax.swing.JCheckBox m_synchronizedViewsCheckBox;
    private javax.swing.JToolBar m_textureManipulationToolBar;
    private javax.swing.JTabbedPane m_textureMapsViewingTabbedPane;
    private javax.swing.JTable m_textureTypeSelectionTable;
    private javax.swing.JCheckBox m_verboseCheckBox;
    private javax.swing.JButton m_writeROIInformationButton;
    private javax.swing.JComboBox m_xLocalStatisticsSizeComboBox;
    private javax.swing.JComboBox m_yLocalStatisticsSizeComboBox;
    // End of variables declaration//GEN-END:variables


    // Attributes for displaying
    private DicomImageSeriesData m_imageSeriesData;

    private DicomImageSeriesData m_imageSeriesFourViewData;

    private RegionModeToolBar    m_regionModeToolBar;
    private boolean              m_isImageTimeSeries = false;
    private static JPopupMenu    m_popupMenu;
    private JMenuItem            m_createXYScatterPlot;
    private JMenuItem            m_createXYScatterPlotOfROIs;

    private String               m_oldPolygonName;


    private int     m_beginBaseLine;
    private int     m_endBaseLine;
    private int     m_maxPerf;
    private int     m_beginReperfusion;
    private float   m_scalingFactorOriginalValues = -1.0f;
    private float   m_scalingFactorDifferenceImageTimeSeries = 1.0f;

    private JMenuBar    m_menuBar;
    private JMenu       m_mainMenu;
    private JMenu       m_viewMenu;
    private JMenuItem   m_renameCurrentROIToNAWM;
    private JMenuItem   m_renameCurrentROIToMetastasisTotalTumorTissue;
    private JMenuItem   m_renameCurrentROIToGBMTotalTumorTissue;
    
    private JMenuItem   m_renameCurrentROIToNonEnhancing_GBM_Edema;
    private JMenuItem   m_renameCurrentROIToNonEnhancing_MET_Edema;
    
    private JMenuItem   m_renameCurrentROITo_CET_NECR_GBM_TumorTissue;
    private JMenuItem   m_renameCurrentROITo_CET_NECR_MET_TumorTissue;
    
    private JMenuItem   m_deleteAllROIsInAllViews;
    
    private JRadioButtonMenuItem m_itemShowToolTipText;
    private JMenuItem   m_loadROIsMenuItem;
    private JMenuItem   m_saveROIsMenuItem;
    private JMenuItem   m_createScreenShotOfMultiView;
    private EnterStringFrame stringEnterframe;
    
    private JRadioButtonMenuItem m_addArbitraryShapedROIs;
    private JRadioButtonMenuItem m_addCircularShapedROIs;
    
    private ButtonGroup m_ROITypeToAddMode;
        
    private MultiDicomSliceView m_selectedMultiDicomSliceView;
    private int                 m_selectedMultiDicomSliceViewTabNumber = 0;
    
    /**
     *
     * Computes the textures based on the values minimumValue and maximumValue
     * that define
     * @param minimumValue
     * @param maximumValue
     */
    private void computeAndDisplayTextures( int minimumPixelValueRangeValue, int maximumPixelValueRangeValue,
                                            int xStatisticsPollingRange, int yStatisticsPollingRange, 
                                            boolean computeCoocAlso )
    {

        m_multiDicomSliceView.removeAllViews( );
        m_multiDicomSliceFourView.removeAllViews( );

        ImageTextureCalculator calc =  new ImageTextureCalculator( m_imageSeriesData,
                                       minimumPixelValueRangeValue, maximumPixelValueRangeValue,
                                       xStatisticsPollingRange, yStatisticsPollingRange );



// ===========================================================================================================

        int[] texture = calc.computeTexture( ImageTextureCalculator.ORIGINAL_IMAGE_DATA_NAME );

        FileInfo finfo = m_imageSeriesData.getFileInfo( 0 );
        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         ORIGINAL" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );
        
        int[] fourViewTextureMaps;
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         ORIGINAL" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RAW_DIFFERENCE_NAME );

        finfo = m_imageSeriesData.getFileInfo( 0 );
        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         DIFFEREN" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "         DIFFEREN" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================


        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "RAW", true } );



        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_AVERAGE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        AVERAGE " );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "AVERAGE", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        AVERAGE "  );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================


        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_VARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VARIANCE" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "VARIANCE", true } );

        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        VARIANCE"  );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_STANDARD_DEVIATION_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        STDEV___" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "ST. DEV.", true } );

        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        STDEV___" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================


        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_SKEWNESS_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        SKEWNESS" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "SKEWNESS", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        SKEWNESS" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_KURTOSIS_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        KURTOSIS" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );
        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "KURTOSIS", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        KURTOSIS" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_VARIANCEOFVARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VAVA____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "VAR. OF VAR.", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VAVA____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================

        // RECT_GLCM_CONTRAST_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_CONTRAST_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CCON____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM CON", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CCON____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        
        
        // RECT_GLCM_DISSIMILARITY_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_DISSIMILARITY_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CDIS____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM DIS", true } );                
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CDIS____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================        

        // RECT_GLCM_HOMOGENEITY_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_HOMOGENEITY_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CHOM____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM HOM", true } );                
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CHOM____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================        
       
        // RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CASM____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM ASM", true } ); 
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CASM____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================        
        
        // RECT_GLCM_ENERGY_ENE_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ENERGY_ENE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CENE____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Energy", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CENE____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================        
                                
        // RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CMAX____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Max Probability", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CMAX____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================        
                
        // RECT_GLCM_ENTROPY_EN_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ENTROPY_EN_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CENT____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Entropy", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CENT____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================        

        // RECT_GLCM_MEAN_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_MEAN_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CMEA____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Mean", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CMEA____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }

// ===========================================================================================================        
        
        // RECT_GLCM_VARIANCE_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_VARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CVAR____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Variance", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CVAR____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }

// ===========================================================================================================        
        
        // RECT_GLCM_CORRELATION_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_CORRELATION_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CCOR____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Correlation", true } );         
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CCOR____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
// ===========================================================================================================        

        m_multiDicomSliceView.setupLayout( );
        m_multiDicomSliceFourView.setupLayout( );
        
        m_multiDicomSliceView.addActionListener( this );
        m_multiDicomSliceFourView.addActionListener( this );
        
    }

    /**
     *
     * Computes the textures based on the values minimumValue and maximumValue
     * that define
     * @param minimumValue
     * @param maximumValue
     */
    private void normalizeAndComputeAndDisplayTextures( float scalingFactorOriginalData,
                                                        int minimumPixelValueRangeValue, int maximumPixelValueRangeValue,
                                                        int xStatisticsPollingRange, int yStatisticsPollingRange,
                                                        boolean computeCoocAlso, boolean computeDifferenceImages )
    {

        
        m_multiDicomSliceView.removeAllViews( );
        m_multiDicomSliceFourView.removeAllViews( );
        
        ImageTextureCalculator calc =  new ImageTextureCalculator( scalingFactorOriginalData, 1.0f,
                                                                   m_imageSeriesData,
                                                                   minimumPixelValueRangeValue, maximumPixelValueRangeValue,
                                                                   xStatisticsPollingRange, yStatisticsPollingRange, 
                                                                   computeCoocAlso, computeDifferenceImages,
                                                                   m_beginBaseLine, m_endBaseLine, 
                                                                   m_maxPerf, m_beginReperfusion );


// ===========================================================================================================

        int[] texture = calc.computeTexture( ImageTextureCalculator.ORIGINAL_IMAGE_DATA_NAME );

        FileInfo finfo = m_imageSeriesData.getFileInfo( 0 );
        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         ORIGINAL" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );
        
        int[] fourViewTextureMaps;
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         ORIGINAL" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RAW_DIFFERENCE_NAME );

        finfo = m_imageSeriesData.getFileInfo( 0 );
        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         DIFFEREN" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "         DIFFEREN" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================


        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "RAW", true } );

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_AVERAGE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        AVERAGE " );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "AVERAGE", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        AVERAGE "  );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================


        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_VARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VARIANCE" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "VARIANCE", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        VARIANCE"  );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_STANDARD_DEVIATION_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        STDEV___" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "ST. DEV.", true } );

        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        STDEV___" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================


        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_SKEWNESS_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        SKEWNESS" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "SKEWNESS", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        SKEWNESS" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_KURTOSIS_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        KURTOSIS" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );
        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "KURTOSIS", true } );
       
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        KURTOSIS" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_VARIANCEOFVARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        VAVA____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "VAR. OF VAR.", true } );
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VAVA____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================

        // RECT_GLCM_CONTRAST_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_CONTRAST_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CCON____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM CON", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CCON____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        
        
        // RECT_GLCM_DISSIMILARITY_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_DISSIMILARITY_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CDIS____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM DIS", true } );                
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                 "        CDIS____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        

        // RECT_GLCM_HOMOGENEITY_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_HOMOGENEITY_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CHOM____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM HOM", true } );                
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CHOM____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        
       
        // RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CASM____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM ASM", true } ); 
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CASM____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        
        
        // RECT_GLCM_ENERGY_ENE_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ENERGY_ENE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CENE____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Energy", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CENE____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        
                                
        // RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CMAX____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Max Probability", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CMAX____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        
                
        // RECT_GLCM_ENTROPY_EN_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ENTROPY_EN_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CENT____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Entropy", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CENT____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        

        // RECT_GLCM_MEAN_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_MEAN_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CMEA____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Mean", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CMEA____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }

// ===========================================================================================================        
        
        // RECT_GLCM_VARIANCE_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_VARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CVAR____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Variance", true } );
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CVAR____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }

// ===========================================================================================================        
        
        // RECT_GLCM_CORRELATION_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_CORRELATION_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CCOR____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Correlation", true } );         
        
        if ( m_isImageTimeSeries == true )
        {
            fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
            m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                                  "        CCOR____" );
            m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
            m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
            m_multiDicomSliceFourView.getCurrentView( ).update( true );
        }
        
// ===========================================================================================================        

        m_multiDicomSliceView.setupLayout( );
        m_multiDicomSliceFourView.setupLayout( );
        
        m_multiDicomSliceView.addActionListener( this );
        m_multiDicomSliceFourView.addActionListener( this );
        
    }



    /**
     *
     * Computes the textures based on the values minimumValue and maximumValue
     * that define
     * @param minimumValue
     * @param maximumValue
     */
    private void normalizeAndComputeAndDisplayTextures( float scalingFactorOriginalData, float scalingFactorDiffenceData,
                                                        int minimumPixelValueRangeValue, int maximumPixelValueRangeValue,
                                                        int xStatisticsPollingRange, int yStatisticsPollingRange,
                                                        boolean computeCoocAlso, boolean computeDifferenceImages )
    {

        m_multiDicomSliceView.removeAllViews( );
        m_multiDicomSliceFourView.removeAllViews( );
        
        ImageTextureCalculator calc =  new ImageTextureCalculator( scalingFactorOriginalData, scalingFactorDiffenceData,
                                                                   m_imageSeriesData,
                                                                   minimumPixelValueRangeValue, maximumPixelValueRangeValue,
                                                                   xStatisticsPollingRange, yStatisticsPollingRange,
                                                                   computeCoocAlso, computeDifferenceImages,
                                                                   m_beginBaseLine, m_endBaseLine, 
                                                                   m_maxPerf, m_beginReperfusion  );

        if ( m_isImageTimeSeries == true )
        {
            System.out.println( "Also normalizing difference images with factor " + scalingFactorDiffenceData );
//            calc.computeDifferenceImages( scalingFactorDiffenceData );
        }

// ===========================================================================================================

        int[] texture = calc.computeTexture( ImageTextureCalculator.ORIGINAL_IMAGE_DATA_NAME );

        FileInfo finfo = m_imageSeriesData.getFileInfo( 0 );
        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         ORIGINAL" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        int[] fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         ORIGINAL" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RAW_DIFFERENCE_NAME );

        finfo = m_imageSeriesData.getFileInfo( 0 );
        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "         DIFFEREN" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "         DIFFEREN" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================


        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "RAW", true } );



        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_AVERAGE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        AVERAGE " );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "AVERAGE", true } );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        AVERAGE "  );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================


        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_VARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VARIANCE" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).setNordicScaleFactor( 1.0f );
        //interact here??
        
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "VARIANCE", true } );


        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        VARIANCE"  );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_STANDARD_DEVIATION_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        STDEV___" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "ST. DEV.", true } );


        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ),  "        STDEV___" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================


        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_SKEWNESS_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        SKEWNESS" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "SKEWNESS", true } );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        SKEWNESS" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_KURTOSIS_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        KURTOSIS" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );
        ( (DefaultTableModel)(m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "KURTOSIS", true } );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        KURTOSIS" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================

        texture = calc.computeTexture( ImageTextureCalculator.RECT_REGION_VARIANCEOFVARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VAVA____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "VAR. OF VAR.", true } );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), "        VAVA____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================

        // RECT_GLCM_CONTRAST_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_CONTRAST_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CCON____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM CON", true } );
        
        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CCON____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );
        
// ===========================================================================================================        
        
        // RECT_GLCM_DISSIMILARITY_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_DISSIMILARITY_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CDIS____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM DIS", true } );                

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CDIS____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================        

        // RECT_GLCM_HOMOGENEITY_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_HOMOGENEITY_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CHOM____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM HOM", true } );                
        
        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CHOM____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );
        
// ===========================================================================================================        
       
        // RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ANGULAR_SECOND_MOMENT_ASM_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CASM____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM ASM", true } ); 
        
        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CASM____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================        
        
        // RECT_GLCM_ENERGY_ENE_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ENERGY_ENE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CENE____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Energy", true } );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CENE____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================        
                                
        // RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_MAXIMUM_PROBABILITY_MAX_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CMAX____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Max Probability", true } );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CMAX____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================        
                
        // RECT_GLCM_ENTROPY_EN_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_ENTROPY_EN_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CENT____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Entropy", true } );
        
        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CENT____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================        

        // RECT_GLCM_MEAN_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_MEAN_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CMEA____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Mean", true } );

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CMEA____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );


// ===========================================================================================================        
        
        // RECT_GLCM_VARIANCE_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_VARIANCE_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CVAR____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Variance", true } );
        
        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CVAR____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );


// ===========================================================================================================        
        
        // RECT_GLCM_CORRELATION_NAME
        texture = calc.computeTexture( ImageTextureCalculator.RECT_GLCM_CORRELATION_NAME );

        m_multiDicomSliceView.addNewView( m_imageSeriesData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                          "        CCOR____" );
        m_multiDicomSliceView.getCurrentView( ).setImageSeriesPixelData( texture );
        m_multiDicomSliceView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceView.getCurrentView( ).update( true );

        ( (DefaultTableModel)( m_textureTypeSelectionTable.getModel( ) ) ).addRow( new Object[]{ "GLCM Correlation", true } );         

        fourViewTextureMaps = computeFourViews( texture, finfo.getImageWidth( ), finfo.getImageHeight( ) );
        m_multiDicomSliceFourView.addNewView( m_imageSeriesFourViewData, new Dimension( finfo.getImageWidth( ), finfo.getImageHeight( ) ), 
                                              "        CCOR____" );
        m_multiDicomSliceFourView.getCurrentView( ).setImageSeriesPixelData( fourViewTextureMaps );
        m_multiDicomSliceFourView.getCurrentView( ).displayDicomImage( 1 );
        m_multiDicomSliceFourView.getCurrentView( ).update( true );

// ===========================================================================================================        

        m_multiDicomSliceView.setupLayout( );
        m_multiDicomSliceFourView.setupLayout( );
        
        m_multiDicomSliceView.addActionListener( this );
        m_multiDicomSliceFourView.addActionListener( this );

        
    }



}
