/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.imageprocessing;

import org.spectrim.lib.Convert;

import java.awt.Color;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.LinkedList;
import javax.swing.JFileChooser;

/**
 * MHAFileOverlayReader class handles the reading of mha-files;
 * These files carry information on (semi transpararent)
 * colored overlays over the Dicom files.
 * @author slotboom
 */
public class MHAFileOverlayReader 
{
    
    /**
     * Default constructor MHAFileOverlayReader.
     */
    public MHAFileOverlayReader( )
    {
        // do nothing, just construct
    }
    
    /**
     * Default constructor MHAFileOverlayReader.
     * @param defaultPath
     */
    public MHAFileOverlayReader( String defaultPath )
    {
        String selected_mha_file_reference = getFilePathAndName( defaultPath, "Load mha overlay" );
        loadMHAOverlayFile( selected_mha_file_reference );
    }
    
    /**
     * 
     * Prompts the user to load a mha overlay file 
     * 
     * @param defaultPath
     * @return 
     */
    public boolean doLoading( String defaultPath )
    {
        String selected_mha_file_reference = getFilePathAndName( defaultPath, "Load mha overlay" );
        return loadMHAOverlayFile( selected_mha_file_reference );
    }
    
    /**
     * Returns the filename and directory selected by the user.
     * @param defaultPath
     * @param buttonText
     * @return 
     */
    protected String getFilePathAndName( String defaultPath, String buttonText )
    {
        JFileChooser fileChooser = new JFileChooser( );
        String selectedFilePathAndName = "";

        // Create a File object containing the canonical path of the
        // desired directory
        
        String inputPath = defaultPath;

        File f = new File( inputPath );
        if ( f.exists( ) == false )
        {
            f.mkdir( );
        }

        // Set the current directory
        fileChooser.setCurrentDirectory( f );

        int result;
        result = fileChooser.showDialog( null, buttonText );


        File selected_file = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_file = fileChooser.getSelectedFile( );
                                                selectedFilePathAndName = selected_file.getPath( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return selectedFilePathAndName;
    }
    

    /**
     * Loads a mha-file from disc; For each image of a slice stack one 
     * overlay file is assumed having extension '.mha' .
     * @param fileName
     * @return 
     */
    private boolean loadMHAOverlayFile( String fileName )
    {
        byte[] pixelData = null;
        
        // Create a new instance for image segment overlays
        m_imageSegmentsOverlayList = new ArrayList< >( );
                
        File inputPath = new File( fileName );
        
        if ( inputPath.exists( ) )
        {
            try
            {
                try
                {
                    //  if file alreay exists than delete it before writing to it!
                    File f = new File( fileName );
                        
                    if ( f.exists( ) )
                    {
                          
                        RandomAccessFile reader = new RandomAccessFile( new File( fileName ) , "r" );
                            
                        if ( parseHeader( reader ) == true )
                        {
                            
                            int numberOfBytesToRead = m_dim_1_size * m_dim_2_size * m_dim_3_size * m_numberOfBytesPerElement;
                            
                            pixelData = new byte[ numberOfBytesToRead ];
                            
                            if ( reader.getFilePointer( ) + numberOfBytesToRead == reader.length( ) )
                            {
                                reader.readFully( pixelData );
                            }
                            else
                            {
                                System.out.println( "Cannot read the mha" );
                            }
                        }
                        
                        reader.close( );
                    }
                    else
                    {
                        return false;
                    }
                }
                catch ( FileNotFoundException e )
                {
                    return false;
                }
             }
             catch( IOException ioe )
             {
                 return false;
             }
        }
        
        m_imageSegmentColors = defaultColors;
        m_alpha = 0.3f;
        
        // ASL -> Sagital images
        if ( m_anatomicalOrientationAsString.trim( ).compareTo( "ASL" ) == 0 )
        {
            // set the thresshold to true for all pixels otherwise control cannot paint the overlays properly
            m_thressholdedImageSegmentOverlays = new int[m_dim_3_size][m_dim_1_size][m_dim_2_size];
        }
        
        // RAI -> Axial images
        if ( m_anatomicalOrientationAsString.trim( ).compareTo( "RAI" ) == 0 )
        {
            m_thressholdedImageSegmentOverlays = new int[m_dim_3_size][m_dim_2_size][m_dim_1_size];
        }
        
        //m_thressholdedImageSegmentOverlays = new int[m_dim_3_size][m_dim_2_size][m_dim_1_size];
        
        //m_thressholdedImageSegmentOverlays = new int[m_dim_2_size][m_dim_1_size][m_dim_3_size];
        
        if ( pixelData != null )
        {
            // ASL -> Sagital images
            if ( m_anatomicalOrientationAsString.trim( ).compareTo( "ASL" ) == 0 )
            {
                int counter = 0;
                for ( int i = 0; i < m_dim_3_size; i++ )
                {
                    for ( int j = 0; j < m_dim_2_size; j++ )
                    {
                        for ( int k = 0; k < m_dim_1_size; k++ )
                        {
                            if ( m_numberOfBytesPerElement == 2 )
                            {
                                m_thressholdedImageSegmentOverlays[m_dim_3_size-i-1][j][k] = Convert.byte2intl( pixelData, counter);
                                counter += 2;
                            }

                            if ( m_numberOfBytesPerElement == 1 )
                            {
                                m_thressholdedImageSegmentOverlays[m_dim_3_size-i-1][j][k] = (int)pixelData[counter];
                                counter++;
                            }
                        }
                    }
                }   
            }
            
            // RAI -> Axial images
            if ( m_anatomicalOrientationAsString.trim( ).compareTo( "RAI" ) == 0 )
            {
                int counter = 0;
                for ( int i = 0; i < m_dim_1_size; i++ )
                {
                    for ( int j = 0; j < m_dim_2_size; j++ )
                    {
                        for ( int k = 0; k < m_dim_3_size; k++ )
                        {
                            if ( m_numberOfBytesPerElement == 2 )
                            {
                                m_thressholdedImageSegmentOverlays[k][j][m_dim_1_size-i-1] = Convert.byte2intl( pixelData, counter);
                                counter += 2;
                            }

                            if ( m_numberOfBytesPerElement == 1 )
                            {
                                m_thressholdedImageSegmentOverlays[k][j][m_dim_1_size-i-1] = (int)pixelData[counter];
                                counter++;
                            }
                        }
                    }
                }
            }          
        }
        
        System.out.println( "mha file loaded!" );
        
        return true;
        
    }
    
    /**
     * 
     * This method parses the header of the mha-file.
     * @param reader
     * @return 
     * @throws java.io.IOException
     */
    public boolean parseHeader( RandomAccessFile reader ) throws IOException
    {
        
        String testString;
                            
        testString = reader.readLine( );
        int locator = testString.indexOf( "=" );
        
        // Example
        m_objectType = (testString.substring( locator + 1 ) ).trim( );
        
        // NDims = 3
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_numberOfDimensions = ( new Integer( (testString.substring( locator + 1 ) ).trim( ) ) ).intValue( );
        
        // BinaryData = True
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        if ( (testString.substring( locator + 1 ) ).trim( ).compareTo( "True" ) == 0 )
        {
            m_binaryDataBoolean = true;
        }
        else
        {
            m_binaryDataBoolean = false;
        }
        
        // BinaryDataByteOrderMSB = False
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        if ( (testString.substring( locator + 1 ) ).trim( ).compareTo( "True" ) == 0 )
        {
            m_binaryDataByteOrderMSB = true;
        }
        else
        {
            m_binaryDataByteOrderMSB = false;
        }
        
        // CompressedData = False
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        if ( (testString.substring( locator + 1 ) ).trim( ).compareTo( "True" ) == 0 )
        {
            m_compressedData = true;
        }
        else
        {
            m_compressedData = false;
        }
        
        // TransformMatrix = 1 -2.0398e-010 2.14392e-011 2.05104e-010 0.994522 -0.104528 -1.26195e-018 0.104528 0.994522
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_transformMatrixAsString = testString.substring( locator + 1 );
        // TO DO parse individual values if needed. currently not implemented
        
        //Offset = -89.0901 -116.598 -75.1377
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_offsetAsString = testString.substring( locator + 1 );
        
        
        //CenterOfRotation = 0 0 0
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_centerOfRotationAsString = testString.substring( locator + 1 );
        
        //AnatomicalOrientation = RAI
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_anatomicalOrientationAsString = testString.substring( locator + 1 );
           
        //ElementSpacing = 0.390625 0.390625 5.49999
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_elementSpacingAsString = testString.substring( locator + 1 );  
        
        //DimSize = 512 512 23
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_elementDimSizeAsString = testString.substring( locator + 1 ).trim();
        
        String substr1 = m_elementDimSizeAsString.substring( 0, m_elementDimSizeAsString.indexOf( " " ) );
        m_dim_1_size = new Integer( substr1.trim( ) ).intValue( );
        
        String rest = m_elementDimSizeAsString.substring( m_elementDimSizeAsString.indexOf( " " ) + 1 );
        String substr2 = rest.substring( 0, m_elementDimSizeAsString.indexOf( " " ) ).trim( );
        
        m_dim_2_size = new Integer( substr2.trim( ) ).intValue( );
        m_dim_3_size = new Integer( rest.substring( m_elementDimSizeAsString.indexOf( " " ) ).trim( ) );
         
        // ElementType = MET_SHORT
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_elementTypeAsString = testString.substring( locator + 1 ).trim();      
        
        
        if ( m_elementTypeAsString.compareTo( "MET_FLOAT" ) == 0 )
        {
            m_numberOfBytesPerElement = 4;
        }
        
        if ( m_elementTypeAsString.compareTo( "MET_SHORT" ) == 0 )
        {
            m_numberOfBytesPerElement = 2;
        }
        
        if ( m_elementTypeAsString.compareTo( "MET_UCHAR" ) == 0 )
        {
            m_numberOfBytesPerElement = 1;
        }
        
        // ElementDataFile = LOCAL
        testString = reader.readLine( );
        locator = testString.indexOf( "=" );
        m_elementDataFile = testString.substring( locator + 1 );     
             
        return true;
    }
    
    /**
     * Returns a boolean overlay .
     * @return 
     */
    public boolean[][][] getBooleanOverlay( )
    {
        
        m_booleanOverlay = new boolean[m_dim_3_size][m_dim_2_size][m_dim_1_size];
        
        // ASL -> Sagital images
        if ( m_anatomicalOrientationAsString.trim( ).compareTo( "ASL" ) == 0 )
        { 
            
            m_booleanOverlay = new boolean[m_dim_3_size][m_dim_2_size][m_dim_1_size];
            
            if ( m_thressholdedImageSegmentOverlays != null )
            {
                for ( int i = 0; i < m_dim_3_size; i++ )
                {
                    for ( int j = 0; j < m_dim_2_size; j++ )
                    {
                        for ( int k = 0; k < m_dim_1_size; k++ )
                        {
                            if ( m_thressholdedImageSegmentOverlays[i][j][k] != 0 )
                            {
                                m_booleanOverlay[i][j][k] = true;
                            }
                            else
                            {
                                m_booleanOverlay[i][j][k] = false;
                            }
                        }
                    }
                }
            }
        }
        
        // "RAI" -> Sagital images
        if ( m_anatomicalOrientationAsString.trim( ).compareTo( "RAI" ) == 0 )
        { 
            m_booleanOverlay = new boolean[m_dim_1_size][m_dim_2_size][m_dim_3_size];
            if ( m_thressholdedImageSegmentOverlays != null )
            {
                for ( int i = 0; i < m_dim_1_size; i++ )
                {
                    for ( int j = 0; j < m_dim_2_size; j++ )
                    {
                        for ( int k = 0; k < m_dim_3_size; k++ )
                        {
                            if ( m_thressholdedImageSegmentOverlays[i][j][k] != 0 )
                            {
                                m_booleanOverlay[i][j][k] = true;
                            }
                            else
                            {
                                m_booleanOverlay[i][j][k] = false;
                            }
                        }
                    }
                }
            }
        }
        
        return m_booleanOverlay;
        
    }
    
    
    /**
     * Some test code.
     * @param args the command line arguments.
     */
    public static void main( String args[] )
    {
        MHAFileOverlayReader reader = new MHAFileOverlayReader( );
        
        String fileName = "/home/slotboom/data/TEST-MHA-DATA/Flair-1.3.12.2.1107.5.2.12.21144.4.0.1617133518531229.mha";
        reader.loadMHAOverlayFile( fileName );
    }
    
    
    // Attributes
    private String      m_objectType = null;
    
    private ArrayList<LinkedList<boolean[][]>> m_imageSegmentsOverlayList;
    
    private int         m_numberOfDimensions;
    
    private boolean     m_binaryDataBoolean;
    private boolean     m_binaryDataByteOrderMSB;
    private boolean     m_compressedData;
    
    private String      m_transformMatrixAsString;
    private String      m_offsetAsString;
    private String      m_centerOfRotationAsString;
    private String      m_anatomicalOrientationAsString;
    private String      m_elementSpacingAsString;
    private String      m_elementDimSizeAsString;
    
    private int         m_dim_1_size;
    private int         m_dim_2_size;
    private int         m_dim_3_size;
    
    private String      m_elementTypeAsString;
    private String      m_elementDataFile;

    
    private Color[]     m_imageSegmentColors;
    
    private Color[]     defaultColors = { Color.BLUE, Color.RED, Color.GREEN, Color.ORANGE, Color.PINK, Color.MAGENTA, Color.CYAN,
                                          Color.BLUE.darker( ).darker(), Color.RED.darker( ).darker( ), Color.GREEN.darker( ).darker( ), 
                                          Color.ORANGE.darker( ).darker( ), Color.PINK.darker( ).darker( ), 
                                          Color.MAGENTA.darker( ).darker( ), Color.CYAN.darker( ).darker( ) }; 
    
    private int[][][]   m_thressholdedImageSegmentOverlays;
    private boolean[][][]   m_booleanOverlay;
    private float       m_alpha;
    private int         m_numberOfBytesPerElement = -1;
}