/*
 * File: $Id: MaximumMask.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 20.07.2004 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


/*
 * MaximumMask.java
 *
 * Created on July 20, 2004, 7:42 AM
 */

package org.spectrim.imageprocessing;

/**
 * Every pixel of and image is set to maskedValue if 
 * larger than maximumValue.
 * @author  hans
 */
public class MaximumMask 
{
    
    /** 
     * Creates a new instance of MaximumMask. 
     * @param maximumValue
     * @param maskedValue
     */
    public MaximumMask( int maximumValue, int maskedValue ) 
    {
        m_maxValue = maximumValue;
        m_maskValue = maskedValue;
    }
    
    /**
     * Performs the masking of the int[] image. 
     * @param maskedImage
     * @return 
     */
    public int[] applyMaximumMask( int[] maskedImage )
    {        
        for ( int i = 0; i < maskedImage.length; i++ )
        {
            if ( maskedImage[i] > m_maxValue )
            {
                maskedImage[i] = m_maskValue;
            }
        }
        return maskedImage;
    }
    
     /**
     * Performs the masking of the int[] image. 
     * @param maskedImage
     * @return 
     */
    public float[] applyMaximumMask( float[] maskedImage )
    {        
        for ( int i = 0; i < maskedImage.length; i++ )
        {
            if ( maskedImage[i] > m_maxValue )
            {
                maskedImage[i] = m_maskValue;
            }
        }
        return maskedImage;
    }
    
    /**
     * Performs the masking of the int[] image. 
     * @param inputMask
     * @param image
     * @return 
     */
    public int[] applyMaximumMask( int[] inputMask, int[] image )
    {        
        for ( int i = 0; i < image.length; i++ )
        {
            if ( image[i] > m_maxValue )
            {
                inputMask[i] = m_maskValue;
            }
        }
        return inputMask;
    }
    
    /**
     * Performs the masking of the float[] image. 
     * @param inputMask
     * @param image
     * @return 
     */
    public int[] applyMaximumMask( int[] inputMask, float[] image )
    {   
        for ( int i = 0; i < image.length; i++ )
        {
            if ( image[i] > m_maxValue )
            {
                inputMask[i] = m_maskValue;
            }
        }
        return inputMask;
    }
    
    
    // attributes
    int m_maxValue;
    int m_maskValue;
    
}
