/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.imageprocessing;

import javax.swing.table.DefaultTableModel;

/**
 *
 * @author slotboom
 */
public class MeasuredDistancesTableModel  extends DefaultTableModel
{

    /**
     * Returns the number of columns
     * @return 
     */
    @Override
    public int getColumnCount( )
    {
        return m_numberOfColumns;
    }

    /**
     * Returns the number of rows
     * @return 
     */
    @Override
    public int getRowCount( )
    {
        return m_numberOfRows;
    }

    /**
     * Returns the column name.
     * @param col
     * @return 
     */
    @Override
    public String getColumnName( int col )
    {
        return m_columnNames[col];
    }

    /**
     * Returns the value of a cell at (col,row).
     * @param col
     * @return 
     */
    @Override
    public Object getValueAt( int row, int col )
    {
        return m_data[row][col];
    }

    /**
     * JTable uses this method to determine the default renderer/
     * editor for each cell;  If we didn't implement this method,
     * then the last column would contain text ("true"/"false"),
     * rather than a check box.
     * @param c
     * @return 
     */
    @Override
    public Class getColumnClass( int c )
    {
        return getValueAt( 0, c ).getClass( );
    }

    /**
     * Returns whether a cell can be edited or not.
     * @param col
     * @return 
     */
    @Override
    public boolean isCellEditable( int row, int col )
    {
        //Note that the m_data/cell address is constant,
        //no matter where the cell appears onscreen.
        if ( col < 1 )
        {
            return false;
        }
        else
        {
            return true;
        }
    }

    /**
     * Sets the value of a cell at (row,col).
     * @param value
     * @param row
     * @param col
     */
    @Override
    public void setValueAt( Object value, int row, int col )
    {
        m_data[row][col] = value;
        fireTableCellUpdated( row, col );
    }

    /**
     * Clears the data in the table and forces an update.
     */
    public void clearTable( )
    {
        m_data = new Object[0][0];
        fireTableDataChanged( );
    }

    /**
     * Set data vector size.
     * @param rows
     * @param columns
     */
    public void setDataVectorSize( int rows, int columns )
    {
        m_numberOfRows = rows;
        m_numberOfColumns = columns;

        m_data = new Object[rows][columns];
    }


    /**
     * Set the data vector.
     * @param rows
     * @param columns
     * @param data
     */
    public void setDataVector( int rows, int columns, Object[][] data )
    {
        m_numberOfRows = rows;
        m_numberOfColumns = columns;

        m_data = new Object[rows][columns];

        for ( int i = 0; i < getRowCount( ); i++ )
        {
            for ( int j = 0; j < getColumnCount( ); j++ )
            {
                m_data[i][j] = data[i][j];
            }
        }

        fireTableDataChanged( );
    }
    
    /**
     * Removes row with row number 'row'.
     * @param row 
     */
    @Override
    public void removeRow( int row )
    {
        Object[][] m_data_new = new Object[m_numberOfRows - 1][m_numberOfColumns];
        int intCounter = -1;

        for ( int i = 0; i < getRowCount( ); i++ )
        {
            if ( i == row )
            {
                continue;
            } 
            else
            {
                intCounter++;
            }
            for ( int j = 0; j < getColumnCount( ); j++ )
            {
                m_data_new[intCounter][j] = m_data[i][j];
            }
        }

        m_data = null;
        m_data = m_data_new;

        m_numberOfRows--;
        
        fireTableDataChanged( );

    }


    // Attributes of the class
    private String[] m_columnNames = { "Number",
                                       "Slice number",
                                       "Name",
                                       "Start Point x",
                                       "Start Point y",
                                       "Start 3D-point",
                                       "End point x",
                                       "End point y",
                                       "End 3D-point",
                                       "Distance" };

    private Object[][] m_data = null;

    int m_numberOfRows = 0;
    int m_numberOfColumns = 10;

    private static final long serialVersionUID = 6482334568125123L;

}
