/*
 * File: $Id: RegionTTesterFrame.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 03.09.2004  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/**
 * RegionTTesterFrame.java
 *
 * Created on September 3, 2004, 1:39 PM
 */

package org.spectrim.imageprocessing;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.ListSelectionModel;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.LinkedList;

import org.spectrim.imagegraphicobjects.RegioPolygon;


/**
 *
 * @author  hans
 */
public class RegionTTesterFrame extends JFrame implements ActionListener
{ 
    
    /** 
     * Creates a new instance of RegionTTesterFrame. 
     * @param parent
     * @param imageData
     * @param polygonList
     * @param imageWidth
     * @param imageHeight
     * @param zoom
     * @param nordicScaleFactor
     */
    public RegionTTesterFrame( JFrame parent, int[] imageData, 
                               LinkedList polygonList, int imageWidth, 
                               int imageHeight, float zoom, 
                               float nordicScaleFactor )
    {
        
        m_parent = parent;
        m_nordicScalingFactor = nordicScaleFactor;
        
        m_data = new int[imageData.length];
        System.arraycopy( imageData, 0, m_data, 0, imageData.length );
        
        m_polygonList = (LinkedList)polygonList.clone();
        
        m_imageWidth = imageWidth;
        m_imageHeight = imageHeight;
        m_zoom = zoom;
        
        createPanels( );
        
        // pack and show ..
        pack( );
        setVisible( true );
        
        if ( polygonList.size( ) > 1 )
        {
            RegionTTester tTest = new RegionTTester( (RegioPolygon)polygonList.get( 0 ), 
                                                     (RegioPolygon)polygonList.get( 1 ), 
                                                     imageData, m_imageWidth, m_imageHeight, 
                                                     m_zoom, m_nordicScaleFactor );
            
            m_reportString = tTest.getReportString( );
            
            this.m_reportTextArea.append( m_reportString );
        }
        
    }
    
    /**
     * Creates all necessary panels.
     */
    private void createPanels( )
    {
        
        m_collectionPanel = new JPanel( );
        
        createReportPanel( );
        createControlPanel( );
        m_splitPane = new JSplitPane( JSplitPane.VERTICAL_SPLIT, m_reportPanel, m_controlPanel );
        
        getContentPane( ).setLayout( new BorderLayout( ) );
        getContentPane( ).add( m_splitPane, BorderLayout.CENTER );
    }

    /**
     * Creates the report panel.
     */
    private void createReportPanel( )
    {
        m_reportPanel = new JPanel( );
        
        m_reportPanel.setLayout( new BoxLayout( m_reportPanel, BoxLayout.X_AXIS ) );
        
        m_reportTextArea = new JTextArea( );
        
        m_mainTextScrollPane = new JScrollPane( m_reportTextArea,
                                                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                                JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        
        m_reportPanel.add( m_mainTextScrollPane, BorderLayout.CENTER );
        
        m_reportPanel.setMinimumSize( new Dimension( 500, 400 ) );
        m_reportPanel.setPreferredSize( new Dimension( 500, 400 ) );
        
    }
    
    /**
     * Creates control panel.
     */
    private void createControlPanel( )
    {
        m_controlPanel = new JPanel( );
        
        m_controlPanel.setLayout( new BoxLayout( m_controlPanel, BoxLayout.Y_AXIS ) );
        
        m_roiSelectionPanel = new JPanel( );
        m_roiSelectionPanel.setLayout( new BoxLayout( m_roiSelectionPanel, BoxLayout.X_AXIS ) );
        
        
        String[] items = new String[m_polygonList.size( )];

        for ( int i = 0; i < m_polygonList.size( ); i++ )
        {
            items[i] = ( (RegioPolygon)m_polygonList.get( i ) ).getLabel( );
        }
        
        m_firstROIChooser = new JList( items );
        m_firstROIChooser.setSelectionMode( ListSelectionModel.SINGLE_SELECTION  );
        m_firstROIChooser.setPreferredSize( new Dimension( 200, 200 ) );
        m_firstROIChooserListScrollerPane = new JScrollPane( m_firstROIChooser, 
                                                             JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                                             JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        
        m_firstROIChooser.setSelectedIndex( 0 );
        
        m_secondROIChooser = new JList( items );
        m_secondROIChooser.setSelectionMode( ListSelectionModel.SINGLE_SELECTION  );
        m_secondROIChooser.setPreferredSize( new Dimension( 200, 200 ) );
        m_secondROIChooserListScrollerPane = new JScrollPane( m_secondROIChooser, 
                                                             JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                                             JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        m_secondROIChooser.setSelectedIndex( 1 );
        
        m_roiSelectionPanel.add( Box.createHorizontalGlue( ) );
        m_roiSelectionPanel.add( m_firstROIChooserListScrollerPane );
        m_roiSelectionPanel.add( Box.createHorizontalGlue( ) );
        m_roiSelectionPanel.add( m_secondROIChooserListScrollerPane );
        m_roiSelectionPanel.add( Box.createHorizontalGlue( ) );
        
        m_controlPanel.add( m_roiSelectionPanel );
        
        m_buttonPanel = new JPanel( );
        
        m_buttonPanel.setLayout( new BoxLayout( m_buttonPanel, BoxLayout.X_AXIS ) );
        
        m_calcTTestButton = new JButton( "t-Test !" ) ;
        m_calcTTestButton.addActionListener( this );
        
        m_clearReportButton = new JButton( "Clear" );
        m_clearReportButton.addActionListener( this );
        
        m_quitButton = new JButton( "Quit" ) ;
        m_quitButton.addActionListener( this );
        
        m_buttonPanel.add( Box.createHorizontalGlue( ) );
        m_buttonPanel.add( m_calcTTestButton );
        
        m_buttonPanel.add( Box.createHorizontalGlue( ) );
        m_buttonPanel.add( m_clearReportButton );
        
        m_buttonPanel.add( Box.createHorizontalGlue( ) );
        m_buttonPanel.add( m_quitButton );
        
        m_buttonPanel.add( Box.createHorizontalGlue( ) );
        
        m_buttonPanel.setMinimumSize( new Dimension( 500, 150 ) );
        m_buttonPanel.setPreferredSize( new Dimension( 500, 150 ) );
        
        m_controlPanel.add( m_buttonPanel );
    }
    
    /**
     * Is called whenever a registered component evokes an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        if ( e.getSource( ) == this.m_quitButton )
        {
            dispose( );
        }
        
        if ( e.getSource( ) == this.m_clearReportButton )
        {
            m_reportTextArea.selectAll();
            m_reportTextArea.replaceSelection( "" );
            m_reportString = "";
        }
        
        if ( e.getSource( ) == this.m_calcTTestButton )
        {
             // Clear the report text area
             m_reportTextArea.selectAll();
             m_reportTextArea.replaceSelection( "" );
             m_reportString = "";
             
             RegioPolygon regPol1 = (RegioPolygon)m_polygonList.get( m_firstROIChooser.getSelectedIndex( ) );
             RegioPolygon regPol2 = (RegioPolygon)m_polygonList.get( m_secondROIChooser.getSelectedIndex( ) );
             
             m_tTest = new RegionTTester( regPol1,  regPol2, m_data, m_imageWidth, m_imageHeight, m_zoom, m_nordicScaleFactor );
            
             m_reportString = m_tTest.getReportString( );
            
            this.m_reportTextArea.append( m_reportString );
             
        }
        
    }
    
    // Attributes of the class
    private final JFrame m_parent;
    
    private JPanel m_collectionPanel;
    
    private JSplitPane m_splitPane;
    
    private JPanel m_controlPanel;
    private JPanel m_reportPanel;
    
    private JTextArea m_reportTextArea;
    private JScrollPane m_mainTextScrollPane;
    
    private JList m_firstROIChooser;
    private JScrollPane m_firstROIChooserListScrollerPane;
    private JList m_secondROIChooser;
    private JScrollPane m_secondROIChooserListScrollerPane;
    private JPanel m_roiSelectionPanel;
    
    private JPanel m_buttonPanel;
    
    private JButton m_calcTTestButton;
    private JButton m_clearReportButton;
    private JButton m_quitButton;
    
    private final LinkedList m_polygonList;
    private final int[] m_data;
    private final int m_imageWidth;
    private final int m_imageHeight;
    private final float m_zoom;
    
    private String m_reportString = "";
    private RegionTTester m_tTest;
    private final float m_nordicScaleFactor = 1f;
    
    private final float m_nordicScalingFactor;
    
    // For proper serialization
    private static final long serialVersionUID = 1661295898889393L;
}
