/*
 * File: $Id: SCANalyzeDefaults.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 30.08.2004, 13:06  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.imageprocessing;


import java.util.Properties;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;

import java.io.IOException;
 
/**
 * Container class handling the persistent storage and access of important
 * default values.
 * @author  hans
 */
public class SCANalyzeDefaults extends Properties
{
    
    /** 
        Creates a new instance of SCANalyzeDefaults.
     */
    private SCANalyzeDefaults( )
    {
        setProperty( LAST_SEARCH_PATH, System.getProperty( "user.home" ) );
    }
    
    /**
     * Returns the one and only instance of SCANalyzeDefaults.
     * @return 
     */
    public static SCANalyzeDefaults getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new SCANalyzeDefaults( );
        }
        return theInstance;
    }

    /**
     * Loads the user preferences file.
     */ 
    public void load( )
    {
        String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "SCANalyze.ini";
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
        
            // now the ini file must exist with default contents, so load it
            
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            try
            {
                OutputStream os = new BufferedOutputStream( new FileOutputStream(  userPreferenceFileName ) );
                this.save( os, "// This is a generated file: do not edit!!" );
                os.close( );
            }
            catch ( IOException ioe )
            { 
                // should hopefully never occur
            }
        }
        
        // now the ini file must exist with default contents, so load it
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( userPreferenceFileName ) );
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            // should never occur
        }
    }
    
   /**
     * Saves the user preferences file.
     */ 
    public void save( )
    {
        try
        {
            String userPreferenceFileName = System.getProperty( "user.dir" ) + System.getProperty( "file.separator" ) + "SCANalyze.ini";
            OutputStream out = new BufferedOutputStream( new FileOutputStream( userPreferenceFileName ) );
            this.save( out, "//This is a generated file : do not edit" );
        }
        catch ( IOException e )
        {
            // should not occur
        }
    }
    
    // static private attribute is the instance itself (Singleton!)
    private static  SCANalyzeDefaults theInstance = null;
    
    public final static String LAST_SEARCH_PATH = "Dicom mount path";

    public final static String LAST_DTPA_PATH = "Dicom DTPA path";
    public final static String LAST_DTPA_NORMALIZATION_FACTOR = "Last used DTPA normalization factor";

    // Must be declader somehow
    private static final long serialVersionUID = 3487495895819393L;
}
