/*
 * File: $Id: SegmentationResultsAnalyzerPanel.java,v 1.1.1.1 2008/04/18 09:38:48 hans Exp $
 * History
 * Date: 18.04.2008  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2008 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */ 

/*
 * SegmentationResultsAnalyzerPanel.java
 *
 * Created on April 18, 2008, 7:54 AM
 */

package org.spectrim.imageprocessing;

import org.spectrim.dicom.DicomSliceCanvasPolyDraw;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JPanel;

/**
 * Enables the structured graphical analysis of segmentation results.
 * @author  hans
 */
public class SegmentationResultsAnalyzerPanel extends JPanel implements ActionListener
{
    
    /** 
     * Creates new form SegmentationResultsAnalyzerPanel. 
     * @param image
     */
    public SegmentationResultsAnalyzerPanel( DicomSliceCanvasPolyDraw image ) 
    {
        // we need a reference to draw the overlays resulting from the segmentation
        m_segmentationResultsViewer = image;
        initComponents( );
        
        m_overlayPropertiesTable.addActionListener( this );
        m_segmentationResultsViewer.addActionListener( this );
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_kernalTypeButtongroup = new javax.swing.ButtonGroup();
        m_applyToSementsButtongroup = new javax.swing.ButtonGroup();
        m_segmentationResultsTabbedPane = new javax.swing.JTabbedPane();
        m_overlayPropertiesPanel = new javax.swing.JPanel();
        m_overlayPropertiesTableScrollPane = new javax.swing.JScrollPane();
        m_overlayPropertiesTable = new org.spectrim.imageprocessing.OverlayPropertiesTable();
        m_segementTypesLabel = new javax.swing.JLabel();
        m_morphologicFilterSettingsPanel = new javax.swing.JPanel();
        m_filterSpecificationPanel = new javax.swing.JPanel();
        m_filtersSizeXLabel = new javax.swing.JLabel();
        m_circularRadioButton = new javax.swing.JRadioButton();
        m_filterSizeXTextField = new javax.swing.JTextField();
        m_rectangularRadioButton = new javax.swing.JRadioButton();
        m_filterSizeYLabel = new javax.swing.JLabel();
        m_filterSizeYTextField = new javax.swing.JTextField();
        m_standardStructuringElement_N4RadioButton = new javax.swing.JRadioButton();
        m_applyFilterToSelectionPanel = new javax.swing.JPanel();
        m_applyToAllRadioButton = new javax.swing.JRadioButton();
        m_applyToSelectedSegmentOnlyRadioButton = new javax.swing.JRadioButton();
        jPanel1 = new javax.swing.JPanel();
        m_erodeButton = new javax.swing.JButton();
        m_dilateButton = new javax.swing.JButton();
        m_undoAllMorphologicFilteringButton = new javax.swing.JButton();
        m_applyMorphologicFilterButton = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        m_segmentationResultsTabbedPane.setMaximumSize(new java.awt.Dimension(512, 255));
        m_segmentationResultsTabbedPane.setMinimumSize(new java.awt.Dimension(512, 255));
        m_segmentationResultsTabbedPane.setPreferredSize(new java.awt.Dimension(512, 355));

        m_overlayPropertiesTableScrollPane.setViewportView(m_overlayPropertiesTable);

        m_segementTypesLabel.setText("Segment types"); // NOI18N

        org.jdesktop.layout.GroupLayout m_overlayPropertiesPanelLayout = new org.jdesktop.layout.GroupLayout(m_overlayPropertiesPanel);
        m_overlayPropertiesPanel.setLayout(m_overlayPropertiesPanelLayout);
        m_overlayPropertiesPanelLayout.setHorizontalGroup(
            m_overlayPropertiesPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_overlayPropertiesPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(m_segementTypesLabel)
                .addContainerGap(412, Short.MAX_VALUE))
            .add(m_overlayPropertiesTableScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 541, Short.MAX_VALUE)
        );
        m_overlayPropertiesPanelLayout.setVerticalGroup(
            m_overlayPropertiesPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_overlayPropertiesPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(m_segementTypesLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(m_overlayPropertiesTableScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 235, Short.MAX_VALUE))
        );

        m_segmentationResultsTabbedPane.addTab("Overlay Selection", m_overlayPropertiesPanel);

        m_filterSpecificationPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Filter-size & shape"));

        m_filtersSizeXLabel.setText("Morphologic filtersize X"); // NOI18N

        m_kernalTypeButtongroup.add(m_circularRadioButton);
        m_circularRadioButton.setText("Circular"); // NOI18N

        m_filterSizeXTextField.setText("1"); // NOI18N

        m_kernalTypeButtongroup.add(m_rectangularRadioButton);
        m_rectangularRadioButton.setText("Rectangular"); // NOI18N

        m_filterSizeYLabel.setText("Morphologic filtersize Y"); // NOI18N

        m_filterSizeYTextField.setText("1"); // NOI18N

        m_kernalTypeButtongroup.add(m_standardStructuringElement_N4RadioButton);
        m_standardStructuringElement_N4RadioButton.setSelected(true);
        m_standardStructuringElement_N4RadioButton.setText("Standard N4-structuring element");

        org.jdesktop.layout.GroupLayout m_filterSpecificationPanelLayout = new org.jdesktop.layout.GroupLayout(m_filterSpecificationPanel);
        m_filterSpecificationPanel.setLayout(m_filterSpecificationPanelLayout);
        m_filterSpecificationPanelLayout.setHorizontalGroup(
            m_filterSpecificationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_filterSpecificationPanelLayout.createSequentialGroup()
                .add(m_filtersSizeXLabel)
                .add(18, 18, 18)
                .add(m_filterSizeXTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 54, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 33, Short.MAX_VALUE)
                .add(m_filterSizeYLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(m_filterSizeYTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 54, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
            .add(m_filterSpecificationPanelLayout.createSequentialGroup()
                .add(m_circularRadioButton)
                .add(18, 18, 18)
                .add(m_rectangularRadioButton)
                .add(18, 18, 18)
                .add(m_standardStructuringElement_N4RadioButton)
                .addContainerGap())
        );
        m_filterSpecificationPanelLayout.setVerticalGroup(
            m_filterSpecificationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_filterSpecificationPanelLayout.createSequentialGroup()
                .addContainerGap(19, Short.MAX_VALUE)
                .add(m_filterSpecificationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(m_filtersSizeXLabel)
                    .add(m_filterSizeXTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(m_filterSizeYTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(m_filterSizeYLabel))
                .add(18, 18, 18)
                .add(m_filterSpecificationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(m_circularRadioButton)
                    .add(m_rectangularRadioButton)
                    .add(m_standardStructuringElement_N4RadioButton)))
        );

        m_applyFilterToSelectionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Apply to"));

        m_applyToSementsButtongroup.add(m_applyToAllRadioButton);
        m_applyToAllRadioButton.setSelected(true);
        m_applyToAllRadioButton.setText("Apply to all segments"); // NOI18N

        m_applyToSementsButtongroup.add(m_applyToSelectedSegmentOnlyRadioButton);
        m_applyToSelectedSegmentOnlyRadioButton.setText("Selected segmenttype only"); // NOI18N

        org.jdesktop.layout.GroupLayout m_applyFilterToSelectionPanelLayout = new org.jdesktop.layout.GroupLayout(m_applyFilterToSelectionPanel);
        m_applyFilterToSelectionPanel.setLayout(m_applyFilterToSelectionPanelLayout);
        m_applyFilterToSelectionPanelLayout.setHorizontalGroup(
            m_applyFilterToSelectionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, m_applyFilterToSelectionPanelLayout.createSequentialGroup()
                .add(m_applyToAllRadioButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 207, Short.MAX_VALUE)
                .add(77, 77, 77)
                .add(m_applyToSelectedSegmentOnlyRadioButton)
                .addContainerGap())
        );
        m_applyFilterToSelectionPanelLayout.setVerticalGroup(
            m_applyFilterToSelectionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_applyFilterToSelectionPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(m_applyToSelectedSegmentOnlyRadioButton)
                .add(m_applyToAllRadioButton))
        );

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Actions"));

        m_erodeButton.setText("Erode"); // NOI18N
        m_erodeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_erodeButtonActionPerformed(evt);
            }
        });

        m_dilateButton.setText("Dilate"); // NOI18N
        m_dilateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_dilateButtonActionPerformed(evt);
            }
        });

        m_undoAllMorphologicFilteringButton.setText("Undo All"); // NOI18N

        m_applyMorphologicFilterButton.setText("Apply!"); // NOI18N

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(m_erodeButton)
                .add(18, 18, 18)
                .add(m_dilateButton)
                .add(18, 18, 18)
                .add(m_undoAllMorphologicFilteringButton)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 200, Short.MAX_VALUE)
                .add(m_applyMorphologicFilterButton)
                .add(32, 32, 32))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                .add(m_erodeButton)
                .add(m_dilateButton)
                .add(m_undoAllMorphologicFilteringButton)
                .add(m_applyMorphologicFilterButton))
        );

        org.jdesktop.layout.GroupLayout m_morphologicFilterSettingsPanelLayout = new org.jdesktop.layout.GroupLayout(m_morphologicFilterSettingsPanel);
        m_morphologicFilterSettingsPanel.setLayout(m_morphologicFilterSettingsPanelLayout);
        m_morphologicFilterSettingsPanelLayout.setHorizontalGroup(
            m_morphologicFilterSettingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, m_filterSpecificationPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(m_applyFilterToSelectionPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        m_morphologicFilterSettingsPanelLayout.setVerticalGroup(
            m_morphologicFilterSettingsPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(m_morphologicFilterSettingsPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(m_filterSpecificationPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(m_applyFilterToSelectionPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(24, Short.MAX_VALUE))
        );

        m_segmentationResultsTabbedPane.addTab("Morphologic Filtering", m_morphologicFilterSettingsPanel);

        add(m_segmentationResultsTabbedPane, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents

    
    
    /**
     * User pressed the erode button and wants to perfrom the erode operation
     *  on the selected overlay maps
     * / 
     * @param evt
     */
    private void m_erodeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_erodeButtonActionPerformed
        
        
        MorphologicFilters morphFilter = null;
        
        if ( m_standardStructuringElement_N4RadioButton.isSelected( ) == false )
        {
            // user wants a non default N4 structuring element
            if ( this.m_rectangularRadioButton.isSelected() == true )
            {
                morphFilter = new MorphologicFilters( new Integer( m_filterSizeXTextField.getText( ) ).intValue( ), 
                                                      new Integer( m_filterSizeYTextField.getText( ) ).intValue( ) );
            }
            if ( m_circularRadioButton.isSelected( ) == true )
            {
                morphFilter = new MorphologicFilters( new Integer( m_filterSizeXTextField.getText( ) ).intValue( ) );
            }
        }
        else
        {
            // user wants a default N4 structuring element
            morphFilter = new MorphologicFilters( );
        }
        
        // find out which segments are selected
        ArrayList< LinkedList<boolean[][]> > allSegments = m_segmentationResultsViewer.getImageSegmentOverlayLinkedLists( );
        
        ArrayList< LinkedList<boolean[][]> > filteredSegments = new ArrayList< LinkedList< boolean[][]> >( );
        
        if ( allSegments.size( ) > 0 )
        {
            
            for ( int i = 0; i < m_segmentationResultsViewer.getNumberOfImages( ); i++ )
            {
                // First get the list of all image overlays
                LinkedList< boolean[][] > imageSegmentList = allSegments.get( i );
                LinkedList< boolean[][] > filteredSegmentList = new LinkedList<boolean[][]>( );
                
                for ( int j = 0; j < imageSegmentList.size( ); j++ )
                {
                    boolean[][] imageSegment = imageSegmentList.get( j );
                    boolean[][] filteredSegment = new boolean[m_segmentationResultsViewer.getImageWidth( )][m_segmentationResultsViewer.getImageHeight( )];
                    filteredSegment = morphFilter.erode( imageSegment, 
                                                         m_segmentationResultsViewer.getImageWidth( ), 
                                                         m_segmentationResultsViewer.getImageHeight( ) );
                
                    filteredSegmentList.add( filteredSegment );
                }
                
                filteredSegments.add( filteredSegmentList ); 
            }
            
        }
        
        // set the new overlays in the 
        m_segmentationResultsViewer.setImageSegmentOverlayLinkedLists( filteredSegments, defaultColors, m_overlayPropertiesTable.getSelectedRows( ), 0.3f );
        m_segmentationResultsViewer.repaint( );
    }//GEN-LAST:event_m_erodeButtonActionPerformed

private void m_dilateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_dilateButtonActionPerformed
        
        MorphologicFilters morphFilter = null;
        
        if ( m_standardStructuringElement_N4RadioButton.isSelected( ) == false )
        {
            // user wants a non default N4 structuring element
            if ( this.m_rectangularRadioButton.isSelected() == true )
            {
                morphFilter = new MorphologicFilters( new Integer( m_filterSizeXTextField.getText( ) ).intValue( ), 
                                                      new Integer( m_filterSizeYTextField.getText( ) ).intValue( ) );
            }
            if ( m_circularRadioButton.isSelected( ) == true )
            {
                morphFilter = new MorphologicFilters( new Integer( m_filterSizeXTextField.getText( ) ).intValue( ) );
            }
        }
        else
        {
            // user wants a default N4 structuring element
            morphFilter = new MorphologicFilters( );
        }
        // find out which segments are selected
        ArrayList< LinkedList<boolean[][]> > allSegments = m_segmentationResultsViewer.getImageSegmentOverlayLinkedLists( );
        
        ArrayList< LinkedList<boolean[][]> > filteredSegments = new ArrayList< LinkedList< boolean[][]> >( );
        
        if ( allSegments.size( ) > 0 )
        {
            
            for ( int i = 0; i < m_segmentationResultsViewer.getNumberOfImages( ); i++ )
            {
                // First get the list of all image overlays
                LinkedList< boolean[][] > imageSegmentList = allSegments.get( i );
                LinkedList< boolean[][] > filteredSegmentList = new LinkedList<boolean[][]>( );
                
                for ( int j = 0; j < imageSegmentList.size( ); j++ )
                {
                    boolean[][] imageSegment = imageSegmentList.get( j );
                    boolean[][] filteredSegment = new boolean[m_segmentationResultsViewer.getImageWidth( )][m_segmentationResultsViewer.getImageHeight( )];
                    filteredSegment = morphFilter.dilate( imageSegment, 
                                                          m_segmentationResultsViewer.getImageWidth( ), 
                                                          m_segmentationResultsViewer.getImageHeight( ) );
                
                    filteredSegmentList.add( filteredSegment );
                }
                
                filteredSegments.add( filteredSegmentList ); 
            }
        }
        
        // set the new overlays in the 
        m_segmentationResultsViewer.setImageSegmentOverlayLinkedLists( filteredSegments, defaultColors, m_overlayPropertiesTable.getSelectedRows( ), 0.3f );
        m_segmentationResultsViewer.repaint( );
}//GEN-LAST:event_m_dilateButtonActionPerformed

    
    
    /**
     * Adds an action listener to the list of action listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_listeners.add( e );
    }
    
    /**
     * Notify all actionListeners.
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }
    
    /**
     * Updates the table with image segment types. Each region the user has 
     * drawn represents one tissue type 
     * @param tissueTypeList
     * @param computed
     */
    public void setImageSegmentTypes( LinkedList <ImageParametersWithinARegioPolygon> tissueTypeList, 
                                      boolean computed )
    {
        OverlayPropertiesTableModel model = (OverlayPropertiesTableModel) m_overlayPropertiesTable.getModel( );
                
        int numberOfColumns = model.getColumnCount( );
        int numberOfRows = tissueTypeList.size( );
        
        model.clearTable( );
        model.setDataVectorSize( numberOfRows, numberOfColumns );
        
        Object[][] data = new Object[numberOfRows][numberOfColumns];
            
        for ( int i = 0; i < numberOfRows ; i++ )
        {
            ImageParametersWithinARegioPolygon tissuePar = tissueTypeList.get( i );
            
            data[i][0] = "Tissue" + i;
            data[i][1] = defaultColors[i % 14];
            data[i][2] = "0";
            data[i][3] = "1";
            if ( computed == false )
            {
                data[i][4] = "<html><FONT COLOR = RED>not computed</html>";
            }
            else
            {
                data[i][4] = "<html><FONT COLOR = GREEN>computed</html>";
            }
        }
        
        model.setDataVector( numberOfRows, numberOfColumns, data );
        
    }

    /**
     * Returns the array of Colors used by the object.
     * @return 
     */
    public Color[] getColors( )
    {
        return defaultColors;
    }

    /**
     * This method is called whenever a registered component evoke an 
     * action event.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
         Object sender = e.getSource( );
         if ( sender instanceof ProbabilityBasedSegmentationCalculator )
         {
             // the segmentation finished, the results are ready to be displayed
             if ( e.getID( ) == ProbabilityBasedSegmentationCalculator.PROBABILISTIC_SEGMENTATION_PERFORMED_EVENT )
             {
                 m_calc = (ProbabilityBasedSegmentationCalculator)sender;
                 
                 // 
                 m_segmentationResultsViewer.clearSegmentationResultsOfPriorAttempts( );
                 
                // get the alpha from the user in the future!
                 float alpha = 0.3f;
                 int imageNumber = m_segmentationResultsViewer.getCurrentlyDisplayedImageNumber( );
                 BufferedImage thressholdImage = m_calc.getBinaryPixelIntensityThressholdOverlayAsBufferedImage( imageNumber, Color.YELLOW, alpha );
                 
                 m_segmentationResultsViewer.setBinaryPixelIntensityThressholdOverlays( m_calc.getBinaryPixelIntensityThressholdOverlays( ) );
                 // m_segmentationResultsViewer.setBinaryPixelIntensityThressholdOverlayAsBufferedImage( thressholdImage );
                
                 // BufferedImage compositeImageSegments = m_calc.getCompositeImageOfSelectedTissueTypes( defaultColors, m_overlayPropertiesTable.getSelectedRows( ), alpha );
                 
                 // following does not include the critical Ttesting just the map with quick classifications
                 // old BufferedImage compositeImageSegments = m_calc.getCompositeImageOfSelectedTissueTypes( imageNumber, defaultColors, m_overlayPropertiesTable.getSelectedRows( ), alpha );
                 ArrayList< LinkedList<boolean[][]> > overlays = m_calc.getProbabilisticSegmentedTissueOverLays( );
                 
                 m_segmentationResultsViewer.setImageSegmentOverlayLinkedLists( overlays, defaultColors, m_overlayPropertiesTable.getSelectedRows( ), 0.3f );
                 
                 m_segmentationResultsViewer.repaint( );
             }
         }
         
        // Also listen to changes in selection of the overlay properties table 
        if ( sender == m_overlayPropertiesTable )
        {

            if ( e.getID( ) == OverlayPropertiesTable.OVERLAYCOLOR_CHANGE_EVENT )
            {
                int[] selRows = m_overlayPropertiesTable.getSelectedRows( );
                Color color = m_overlayPropertiesTable.getLastSelectedColor( );
                m_segmentationResultsViewer.setSelectedOverlayTypeNumbersAndColors( selRows, color );
                m_segmentationResultsViewer.repaint( );
            }

            if ( e.getID( ) == OverlayPropertiesTable.TABLE_SELECTED_EVENT )
            {
                
               int[] selRows = m_overlayPropertiesTable.getSelectedRows( );

               m_segmentationResultsViewer.setSelectedOverlayTypeNumbers( selRows );
               m_segmentationResultsViewer.repaint( );
                   
               // sent notification to inform ProbabilityBasedSegmentationAnalyzerFrame
               // that there was an update
               notifyActionListeners( TABLE_SELECTION_CHANGE_EVENT );
               
            }
            
            if ( e.getID( ) == OverlayPropertiesTable.PERFORM_BOOLEAN_AND_OPERATION )
            {
                m_segmentationResultsViewer.performBooleanOperation( OverlayPropertiesTable.PERFORM_BOOLEAN_AND_OPERATION );
            }
            
            if ( e.getID( ) == OverlayPropertiesTable.PERFORM_BOOLEAN_OR_OPERATION )
            {
                m_segmentationResultsViewer.performBooleanOperation( OverlayPropertiesTable.PERFORM_BOOLEAN_OR_OPERATION );
            }
            
            if ( e.getID( ) == OverlayPropertiesTable.PERFORM_BOOLEAN_EXOR_OPERATION )
            {
                m_segmentationResultsViewer.performBooleanOperation( OverlayPropertiesTable.PERFORM_BOOLEAN_EXOR_OPERATION );
            }
            
            if ( e.getID( ) == OverlayPropertiesTable.PERFORM_BOOLEAN_NOT_OPERATION )
            {
                m_segmentationResultsViewer.performBooleanOperation( OverlayPropertiesTable.PERFORM_BOOLEAN_NOT_OPERATION );
            }
            
            
        }
         
        // react on events raised by the viewer 
        if ( sender == m_segmentationResultsViewer )
        {
             if ( e.getID( ) == DicomSliceCanvasPolyDraw.IMAGE_SEGMENT_REMOVED_EVENT )
             {
                 int[] rows = m_overlayPropertiesTable.getSelectedRows( );
                 
                 OverlayPropertiesTableModel model = (OverlayPropertiesTableModel) m_overlayPropertiesTable.getModel( );
                 for ( int i = rows.length - 1; i > -1; i-- )
                 {
                     model.removeRow( rows[i] );
                 }
                 
                 m_segmentationResultsViewer.repaint( );
             }
             
             
             if ( e.getID( ) == DicomSliceCanvasPolyDraw.ADDED_NEW_COMPUTED_OVERLAY_EVENT )
             {
                int numberOfOverlays = ( new Integer( e.getActionCommand( ) ) ).intValue( );
                 
                OverlayPropertiesTableModel model = (OverlayPropertiesTableModel) m_overlayPropertiesTable.getModel( );
                
                Object[][] data = new Object[numberOfOverlays][5];
                
                for ( int i = 0; i < numberOfOverlays ; i++ )
                {
                    data[i][0] = "Tissue" + i;
                    data[i][1] = defaultColors[i % 14];
                    data[i][2] = "0";
                    data[i][3] = "1";
                    data[i][4] = "<html><FONT COLOR = GREEN>computed</html>";
                }
     
                model.setDataVector( numberOfOverlays, 5, data );
                
                m_overlayPropertiesTable.selectAll( );
                
                int[] selRows = m_overlayPropertiesTable.getSelectedRows( );
               
                m_segmentationResultsViewer.setSelectedOverlayTypeNumbers( selRows );
                m_segmentationResultsViewer.repaint( );
             }
             
             
             if ( e.getID( ) == DicomSliceCanvasPolyDraw.OVERLAYS_READ_FROM_FILE_EVENT )
             {
                int numberOfOverlays = ( new Integer( e.getActionCommand( ) ) ).intValue( );
                 
                OverlayPropertiesTableModel model = (OverlayPropertiesTableModel) m_overlayPropertiesTable.getModel( );
                
                Object[][] data = new Object[numberOfOverlays][5];
                
                for ( int i = 0; i < numberOfOverlays ; i++ )
                {
                    data[i][0] = "Tissue" + i;
                    data[i][1] = defaultColors[i % 14];
                    data[i][2] = "0";
                    data[i][3] = "1";
                    data[i][4] = "<html><FONT COLOR = GREEN>computed</html>";
                }
     
                model.setDataVector( numberOfOverlays, 5, data );
                
                m_overlayPropertiesTable.selectAll( );
                
                int[] selRows = m_overlayPropertiesTable.getSelectedRows( );
                
                m_segmentationResultsViewer.setSelectedOverlayTypeNumbers( selRows );
                m_segmentationResultsViewer.repaint( );
            }
        } 
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel m_applyFilterToSelectionPanel;
    private javax.swing.JButton m_applyMorphologicFilterButton;
    private javax.swing.JRadioButton m_applyToAllRadioButton;
    private javax.swing.JRadioButton m_applyToSelectedSegmentOnlyRadioButton;
    private javax.swing.ButtonGroup m_applyToSementsButtongroup;
    private javax.swing.JRadioButton m_circularRadioButton;
    private javax.swing.JButton m_dilateButton;
    private javax.swing.JButton m_erodeButton;
    private javax.swing.JTextField m_filterSizeXTextField;
    private javax.swing.JLabel m_filterSizeYLabel;
    private javax.swing.JTextField m_filterSizeYTextField;
    private javax.swing.JPanel m_filterSpecificationPanel;
    private javax.swing.JLabel m_filtersSizeXLabel;
    private javax.swing.ButtonGroup m_kernalTypeButtongroup;
    private javax.swing.JPanel m_morphologicFilterSettingsPanel;
    private javax.swing.JPanel m_overlayPropertiesPanel;
    private org.spectrim.imageprocessing.OverlayPropertiesTable m_overlayPropertiesTable;
    private javax.swing.JScrollPane m_overlayPropertiesTableScrollPane;
    private javax.swing.JRadioButton m_rectangularRadioButton;
    private javax.swing.JLabel m_segementTypesLabel;
    private javax.swing.JTabbedPane m_segmentationResultsTabbedPane;
    private javax.swing.JRadioButton m_standardStructuringElement_N4RadioButton;
    private javax.swing.JButton m_undoAllMorphologicFilteringButton;
    // End of variables declaration//GEN-END:variables
    
    // Listeners to changes in the segmentation gui control
    private final LinkedList<ActionListener> m_listeners = new LinkedList<>( );
    
    private final DicomSliceCanvasPolyDraw m_segmentationResultsViewer;
    
    private static final long serialVersionUID = 99822466327598273L;
    
    private final Color[] defaultColors = { Color.BLUE, Color.RED, Color.GREEN, 
                                            Color.ORANGE, Color.PINK, Color.MAGENTA, 
                                            Color.CYAN, Color.BLUE.darker( ).darker(), 
                                            Color.RED.darker( ).darker( ), 
                                            Color.GREEN.darker( ).darker( ), 
                                            Color.ORANGE.darker( ).darker( ), 
                                            Color.PINK.darker( ).darker( ), 
                                            Color.MAGENTA.darker( ).darker( ), 
                                            Color.CYAN.darker( ).darker( ) };
    
    ProbabilityBasedSegmentationCalculator m_calc;
    
    public final static int ERODE_IMAGE_EVENT = 0;
    public final static int TABLE_SELECTION_CHANGE_EVENT = 1;
}
