/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.io.nifti;

import org.spectrim.imagegraphicobjects.Point3D;
import java.awt.image.WritableRaster;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import niftijio.NiftiHeader;
import niftijio.NiftiVolume;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.registration.OutputListener;
import org.spectrim.settings.RegistrationSettings;
import org.spectrim.view.ImageSlice;
import org.spectrim.view.ImageStack;
import org.spectrim.view.OverlaySlice;
import org.spectrim.view.OverlayStack;

/**
 *
 * @author nuno
 */
public class NiftiHandler {

    public static ArrayList<WritableRaster> getRastersFromNifti(String file) {
        ArrayList<WritableRaster> rasters = new ArrayList<>();
        try {
            NiftiVolume.read(file, rasters);

//            System.out.println("time:" + (System.currentTimeMillis() - start));
//            for (int k = 0; k < volume.header.dim[3]; k++) {
//                BufferedImage im = new BufferedImage(volume.header.dim[1], volume.header.dim[2], BufferedImage.TYPE_BYTE_GRAY);
//                WritableRaster raster = im.getRaster();
//                for (int i = 0; i < volume.header.dim[1]; i++) {
//                    for (int j = 0; j < volume.header.dim[2]; j++) {
//                        double[] val = {volume.data[i][j][k][0]};
//                        raster.setPixel(i, j, val);
//                    }
//                }
//                res.add(raster);
//            }
        } catch (IOException ex) {
            Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        return rasters;
    }

    public static ImageStack getImagesFromNifti(String file) {
        ImageStack imStack = new ImageStack();
        ArrayList<WritableRaster> rasters = new ArrayList<>();
        try {
            NiftiVolume.read(file, rasters);
            NiftiHeader header = getNiftiHeader(file);
            double[] orient = getOrientation(header);
            float[] space = getSpacing(header);
            Point3D orx = new Point3D(orient[0], orient[1], orient[2]);
            Point3D ory = new Point3D(orient[3], orient[4], orient[5]);
//          Revert order in Sagital            
            if (!(orx.m_x1 > orx.m_x2 && orx.m_x1 > orx.m_x3)) {
                Collections.reverse(rasters);
            }

            for (int i = 0; i < rasters.size(); i++) {
                double[] pos = getPosition(header, i);
                Point3D p = new Point3D(pos[0], pos[1], pos[2]);
                ImageSlice aux = new ImageSlice(orx, ory, p, header.dim[1], header.dim[2], space[0], space[1], rasters.get(i), file);
                imStack.add(aux);
            }
        } catch (IOException ex) {
            Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        return imStack;
    }

    public static OverlayStack getOverlaysFromNifti(String file) {
        OverlayStack imStack = new OverlayStack();
        ArrayList<WritableRaster> rasters = new ArrayList<>();
        try {
            NiftiVolume.read(file, rasters, 1000);
            NiftiHeader header = getNiftiHeader(file);
            double[] orient = getOrientation(header);
            float[] space = getSpacing(header);
            Point3D orx = new Point3D(orient[0], orient[1], orient[2]);
            Point3D ory = new Point3D(orient[3], orient[4], orient[5]);
//          Revert order in Sagital            
            if (!(orx.m_x1 > orx.m_x2 && orx.m_x1 > orx.m_x3)) {
                Collections.reverse(rasters);
            }

            for (int i = 0; i < rasters.size(); i++) {
                double[] pos = getPosition(header, i);
                Point3D p = new Point3D(pos[0], pos[1], pos[2]);
                OverlaySlice aux = new OverlaySlice(orx, ory, p, header.dim[1], header.dim[2], space[0], space[1], rasters.get(i), file);
                imStack.add(aux);
            }
        } catch (IOException ex) {
            Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        return imStack;
    }

    public static NiftiHeader getNiftiHeader(String file) {
        NiftiHeader header;
        try {
            header = NiftiHeader.read(file);
            return header;
        } catch (IOException ex) {
            Logger.getLogger(DicomHandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public static double[] getPosition(NiftiHeader header, int slicenr) {
        double[] res = new double[3];
        double[] orient = getOrientation(header);
//         Check if it is sagital
        if (orient[1] > orient[0] && orient[1] > orient[2]) {
            res[0] = -(header.srow_x[3] + header.srow_x[1] * (header.dim[2] - 1)) - (header.dim[3] - 1 - slicenr) * header.srow_x[2];
            res[1] = -(header.srow_y[3] + header.srow_y[1] * (header.dim[2] - 1) + slicenr * header.srow_y[2]);
            res[2] = header.srow_z[3] + header.srow_z[1] * (header.dim[2] - 1) + slicenr * header.srow_z[2];
        } else { // coronal and axial
            res[0] = -(header.srow_x[3] + header.srow_x[1] * (header.dim[2] - 1) + slicenr * header.srow_x[2]);
            res[1] = -(header.srow_y[3] + header.srow_y[1] * (header.dim[2] - 1) + slicenr * header.srow_y[2]);
            res[2] = header.srow_z[3] + header.srow_z[1] * (header.dim[2] - 1) + slicenr * header.srow_z[2];
        }
        return res;
    }

    public static double[] getOrientation(NiftiHeader header) {
        double[] res = new double[6];
        double val1 = Math.sqrt(Math.pow(header.srow_x[0], 2) + Math.pow(header.srow_y[0], 2) + Math.pow(header.srow_z[0], 2));
        double val2 = Math.sqrt(Math.pow(header.srow_x[1], 2) + Math.pow(header.srow_y[1], 2) + Math.pow(header.srow_z[1], 2));
        if (header.srow_x[0] != 0f) { //avoid -0.0
            res[0] = -header.srow_x[0] / val1;
        }
        if (header.srow_y[0] != 0f) {
            res[1] = -header.srow_y[0] / val1;
        }
        res[2] = header.srow_z[0] / val1;
        res[3] = header.srow_x[1] / val2;
        res[4] = header.srow_y[1] / val2;
        if (header.srow_z[1] != 0f) {
            res[5] = -header.srow_z[1] / val2;
        }
        return res;
    }

    public static float[] getSpacing(NiftiHeader header) {
        float[] res = new float[3];
        res[0] = (float) Math.sqrt(Math.pow(header.srow_x[0], 2) + Math.pow(header.srow_y[0], 2) + Math.pow(header.srow_z[0], 2));
        res[1] = (float) Math.sqrt(Math.pow(header.srow_x[1], 2) + Math.pow(header.srow_y[1], 2) + Math.pow(header.srow_z[1], 2));
        res[2] = (float) Math.sqrt(Math.pow(header.srow_x[2], 2) + Math.pow(header.srow_y[2], 2) + Math.pow(header.srow_z[2], 2));
        return res;
    }

    public static String dcm2nii(ArrayList<String> imagefiles, String outputFolder, OutputListener listener) {
        String newFile = "";
        try {
            List<String> command = new ArrayList<String>();
            command.add(RegistrationSettings.getInstance().getDicom2niiInstallPath());
            command.add("-g");
            command.add("n");
            command.add("-r");
            command.add("n");
            command.add("-v");
            command.add("n");
            command.add("-4");
            command.add("n");
            command.add("-o");
            command.add(outputFolder);
            for (int i = 0; i < imagefiles.size(); i++) {
                command.add(imagefiles.get(i));
            }

            ProcessBuilder builder = new ProcessBuilder(command);
//            Map<String, String> environ = builder.environment();
//            environ.put(null, null);
//            builder.directory(new File{});

            final Process process = builder.start();
            InputStream is = process.getInputStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);
            String line;

            while ((line = br.readLine()) != null) {
                if (listener != null) {
                    listener.print(line);
                }
                if (line.startsWith("Saving ")) {
                    newFile = line.split(" ")[1];
                }
            }
        } catch (IOException e1) {
            e1.printStackTrace();
        }
        return newFile;
    }

    public static void main(String args[]) throws IOException {
        NiftiHeader head = getNiftiHeader("E:\\cases\\reg_files\\res1414673188050.nii");
        System.out.println(head);
    }
}
