/*
 * File: $Id: CustomFileDialog.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 31.01.2003  Authors: Martin Buerki - Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.lib;
 
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Cursor;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JDialog;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory;
import javax.swing.ListSelectionModel;

import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;

import java.util.StringTokenizer;


public class CustomFileDialog extends JDialog implements ActionListener,
							 ItemListener,
							 NamePatternListener 
{

    /**
     * Constructor of an (obsolete!) CustomFileDialog.
     * @param parent
     * @param dir
     * @param pattern 
     */
    public CustomFileDialog( JFrame parent, String dir, String pattern ) 
    {
        super( parent, "Select a file", true );
        
        this.getContentPane( ).setLayout( new BorderLayout( ) );
        
        topPanel = new JPanel( new BorderLayout( ) );
        this.getContentPane( ).add( topPanel, BorderLayout.NORTH );
        topPanel.add( subPanel1 = new JPanel( ), BorderLayout.NORTH );
        subPanel1.add( new JLabel( "Path" ) );
        subPanel1.add( tfCurrentPath = new JTextField( dir, 40 ) );
        tfCurrentPath.addActionListener( this );
        topPanel.add( subPanel2 = new JPanel( ), BorderLayout.CENTER );
        subPanel2.add( new JLabel( "Filter" ) );
        m_filterTextField = new JTextField( pattern, 30 );
        subPanel2.add( m_filterTextField );
	
        m_filterTextField.addActionListener( this );
	
        topPanel.add( formatPanel = new ImageFormatPanel( topPanel, this ), BorderLayout.SOUTH );
        formatPanel.addNamePatternListener( this );
                
        // test centerPanel = new JPanel( new GridLayout( 1, 2 ) );
        centerPanel = new JPanel( );
        centerPanel.setLayout( new BoxLayout( centerPanel, BoxLayout.X_AXIS ) );
        this.getContentPane( ).add( centerPanel, BorderLayout.CENTER );

	
        dirList = new JList( new DefaultListModel( ) );
        
        JScrollPane dirListScrollPane = new JScrollPane( dirList, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        
        dirListScrollPane.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        dirListScrollPane.setPreferredSize( new Dimension( 300, 150 ) );
        centerPanel.add( dirListScrollPane );
        

        
        dirList.addListSelectionListener( new ListSelectionListener( )
        {
            @Override
            public void valueChanged( ListSelectionEvent e )
            {
                //qqq setDirectory( currentDir + fSep + dirList.getSelectedItem( ) );

                setDirectory( currentDir + fSep + (String)dirList.getSelectedValue( ) );

                Globals globs = Globals.getInstance( );
                
                globs.defaultPath = currentDir;
                
                currentFile = null;
            }
        } );
        
        cPanelR = new JPanel( new BorderLayout( ) );

        cPanelR.add( cbListFiles = new JCheckBox( "List contents" ),
                                                   BorderLayout.NORTH );        
        cbListFiles.addItemListener( this );
	
        fileList = new JList( new DefaultListModel( ) );
        
        
        
        JScrollPane fileListScrollPane = new JScrollPane( fileList, 
                                             JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
                                             JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
        
        cPanelR.add( fileListScrollPane, BorderLayout.CENTER );
        
        cPanelR.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        
        centerPanel.add( cPanelR );
        
        fileListScrollPane.setPreferredSize( new Dimension( 400, 150 ) );
        
        // fileList.setBorder( BorderFactory.createEmptyBorder( 2, 2, 2, 2 ) );
        
        MouseListener mouseListener = new MouseAdapter( )
        {
            @Override
            public void mouseClicked( MouseEvent e )
            {
                if ( e.getClickCount( ) == 2 )
                {
                    int index = fileList.locationToIndex( e.getPoint( ) );
                    // TO DO
                    String s = (String)fileList.getModel( ).getElementAt( index );
                    
                    int k=0; 
                    int first = -1;
                    while ( k < fileList.getModel( ).getSize( ) ) 
                    {
                        if ( first == -1 && fileList.isSelectedIndex( k ) ) first = k;
                        if ( s.equals( fileList.getModel( ).getElementAt( k ) ) ) break;
                        k++;
                    }
                    for ( int i = first + 1; i <= k; i++ ) 
                    {
                        fileList.setSelectedIndex( i );
                    }
                }
            }
        };
        

        fileList.addListSelectionListener( new ListSelectionListener( )
        {
            @Override
            public void valueChanged( ListSelectionEvent e )
            {
                currentFile = (String)fileList.getSelectedValue( );
            }
        } );

        
        buttonPanel = new ButtonPanel( this );
        this.getContentPane( ).add( buttonPanel, BorderLayout.SOUTH );
        fileFilter = new FileFilter( pattern );
        dirFilter = new DirectoryFilter( );
        fSep = System.getProperty( "file.separator" );
        setDirectory( dir );

        normalSize = new Dimension( 450, 500 );
        expandedSize = new Dimension( 450, 690 );

        formatPanel.setWindowHookSizes( normalSize, expandedSize );
        setSize( normalSize );
        pack( );
        // Put the Dialog nicely in the middle of its parent
        Point parentLoc = parent.getLocation( );
        Point centerParent = new Point( parent.getX( ) + ( parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                        parent.getY( ) + ( parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
        setLocation( centerParent );
    }

    /**
     * Browse a directory.
     */
    public void browseDirectory( )
    {
        cbListFiles.setVisible( true );
        //qqq cbListFiles.setState( lastListFilesState );
        cbListFiles.setSelected( lastListFilesState );
        // fileList.setEnabled( false );
        setTitle( "Set directory" );
        setVisible( true );
        // pack( );
    }

    /**
     * Opens files obeying a specific pattern. 
     * @param pattern
     * @param multipleSelection 
     */
    public void openFiles( String pattern, boolean multipleSelection ) 
    {
        cbListFiles.setVisible( false );
        //qqq lastListFilesState = cbListFiles.getState( );
        lastListFilesState = cbListFiles.isSelected( );
        //qqq cbListFiles.setState( true );
        cbListFiles.setSelected( true );
        
        fileList.setEnabled( true );
        setTitle( "Select files" );
        setFilter( pattern );
        setMultipleSelection( multipleSelection );
        setVisible( true );
    }

    /**
     * 
     */
    private void readDirList( ) 
    {
        dirList.removeAll( );
        SortedStringList ssl = new SortedStringList( dirFile.list( dirFilter ) );

        DefaultListModel lst = (DefaultListModel)dirList.getModel( );
        
        lst.removeAllElements( );
        lst.addElement( ".." );
        for ( int k = 0; k < ssl.getSize( ); k++ )
        {
            lst.addElement( ssl.getItem( k ) );
        }
        
        // also update the file JList component displaying the files
        readFileList( );
        
    }

    /**
     * Reads the fileList elements.
     */
    private void readFileList( ) 
    {
        fileList.removeAll( );
        
        //qqq if ( !cbListFiles.getState( ) ) return;
        if ( !cbListFiles.isSelected( ) )
        {
            fileList.setVisible( false );
            DefaultListModel lst = (DefaultListModel)fileList.getModel( );
            lst.clear( );
            return;
        }
        else
        {
            fileList.setVisible( true );
            
            if ( ( getDirectory( ) == null  ) || ( ( getDirectory() ).compareTo( "" ) == 0 ) )
            {
                currentDir = System.getProperty( "user.home" );
                setDirectory( currentDir );
                
            }
            else
            {
                currentDir = getDirectory( );
            }
            
            dirFile = new File( currentDir );
            
            if ( dirFile.isDirectory( ) == true )
            {
                if ( dirFile.list( fileFilter ) != null )
                {
                    this.setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
                
                    fileList.removeAll( );
                
                    SortedStringList ssl = new SortedStringList( dirFile.list( fileFilter ) );
                    // fileList.setVisible( false );
        
                    DefaultListModel lst = (DefaultListModel)fileList.getModel( );
                    lst.clear( );
                
                    for ( int k = 0; k < ssl.getSize( ); k++ ) 
                    {
                        lst.addElement( ssl.getItem( k ) );
                    }
                    this.setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
                    // fileList.setVisible( true );
                }
            }
            else
            {
                dirFile = new File( System.getProperty( "user.dir" ) );
                this.setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
                
                fileList.removeAll( );
                
                SortedStringList ssl = new SortedStringList( dirFile.list( fileFilter ) );
                // fileList.setVisible( false );
        
                DefaultListModel lst = (DefaultListModel)fileList.getModel( );
                lst.clear( );
                
                for ( int k = 0; k < ssl.getSize( ); k++ ) 
                {
                    lst.addElement( ssl.getItem( k ) );
                }
                this.setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
                // fileList.setVisible( true );
            }
        }
    }


    /**
     * Sets the custom directory.
     * @param dir
     * @return
     */
    public boolean setDirectory( String dir ) 
    {
        dirFile = new File( dir );
        if ( dirFile.isDirectory( ) )
        {
            try
            {
                currentDir = dirFile.getCanonicalPath( );
            }
            catch ( IOException e )
            {
                return false;
            }
            tfCurrentPath.setText( currentDir );

            Globals globs = Globals.getInstance( );
            globs.defaultPath = currentDir;

            readDirList( );
            readFileList( );
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Returns the currently selected Directory as String.
     * @return Custom directory as string
     */
    public String getDirectory( )
    {
        return currentDir;
    }

    /**
     * Returns the selected file.
     * @return 
     */
    public String getSelectedFile( )
    {
        return currentFile;
    }

    /**
     * Returns the selected files.
     * @return 
     */
    public String[] getSelectedFiles( )
    {
        int[] selectedIndices = fileList.getSelectedIndices( );
        String[] selectedFiles = new String[selectedIndices.length];
        for ( int  i = 0; i < selectedIndices.length; i++ )
        {
            selectedFiles[i] = (String)fileList.getModel( ).getElementAt( selectedIndices[i] );
        }
        return selectedFiles;
    }

    public int getFileFormat( )
    {
        return formatPanel.getFileFormat( );
    }

    /**
     * Returns the ImageFormatInfo object.
     * @return 
     */
    public ImageFormatInfo getImageFormatInfo( )
    {
        return formatPanel.getImageFormatInfo( );
    }

    
    /**
     * Sets filter string; but code is unclear?? (rem by hs).
     * @param pattern
     */
    public void setFilter( String pattern )
    {
        patternRoot = pattern;
        //String pat = pattern + formatPanel.getFileExtension( )
        //              + formatPanel.getZipExtension( );
        String pat = m_filterTextField.getText(  );
        fileFilter.setPattern( pat );
        readFileList( );
    }
    
    /**
     * Sets new filter string.
     * @param pattern
     */
    public void setNewFilterText( String pattern )
    {
        patternRoot = pattern;
        fileFilter.setPattern( pattern );
        readFileList( );
    }
    
    /**
     * Update the String-pattern manually. 
     * @param pattern 
     */
    public void setFilterManual( String pattern )
    {
        m_filterTextField.setText( pattern );
        fileFilter.setPattern( pattern );
        readFileList( );
    }

    /**
     * 
     * @param mode 
     */
    public void setMultipleSelection( boolean mode ) 
    {
        if ( mode == false )
        {
            fileList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        }
        else
        {
            fileList.setSelectionMode( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION );
        }
    }
    
    /**
     * Is called in response of an ActionEvent. 
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender == m_filterTextField )
        {
            setFilter( m_filterTextField.getText( ) );
        }
        else if ( sender == tfCurrentPath )
        {
            setDirectory( tfCurrentPath.getText( ) );
        }
        
        else if ( sender == fileList )
        {
            String s = e.getActionCommand( ); /* the string of the double clicked list entry */
            int k=0;
            int first = -1;
            while ( k < fileList.getModel( ).getSize( ) )
            {
                if ( first == -1 && fileList.isSelectedIndex( k ) ) first = k;
                if ( s.equals( fileList.getModel( ).getElementAt( k ) ) ) break;
                k++;
            }
            //if ( k >= fileList.getItemCount( ) ) return;
            //if ( first < 0 ) first = 0;
            for ( int i = first + 1; i <= k; i++ )
            {
                //qqq fileList.select( i );
                fileList.setSelectedIndex( i );
            }
        }
    }
    
    /**
     * Is called whenever an item changes its state.
     * @param e 
     */
    @Override
    public void itemStateChanged( ItemEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender == cbListFiles )
        {
            readFileList( );
        }
    }

    /**
     * Sets the filter to 'patterRoot'.
     */
    @Override
    public void namePatternChanged( ) 
    {
        setFilter( patternRoot );
    }

    /**
     * Constructor of the 'FileFilter' class.
     */
    class FileFilter implements FilenameFilter 
    {	
        String pattern;
        private String[] tokens;
        private int numTokens;
        
	/**
         * Alternative constructor.
         * @param pattern 
         */
        FileFilter( String pattern )
        {
            setPattern( pattern );
        }

        /**
         * Sets the filter pattern string.
         * @param pattern 
         */
        public void setPattern( String pattern )
        {
            StringTokenizer S = new StringTokenizer( pattern, "*" );
            numTokens = S.countTokens( );
            tokens = new String[numTokens];
            for ( int k = 0; k < numTokens; k++ )
            {
                tokens[k] = S.nextToken( );
            }
        }
        
        /**
         * Called to accept the filter.
         * @param dir
         * @param name
         * @return 
         */
        @Override
        public boolean accept( File dir, String name )
        {
            // reject directories
            File f = new File( dir, name );
            if ( f.isDirectory( ) ) return false;

            boolean match = true;
            int lastIndex = 0;
            int k = 0;
            while ( k < numTokens && match )
            {
                int i = name.indexOf(tokens[k], lastIndex);
                if ( i == -1 ) match = false;
                else lastIndex = ( i + tokens[k++].length( ) );
            }
            return match;
        }
    }

    /**
     * Class that defines a filter function for directories.
     */
    class DirectoryFilter implements FilenameFilter 
    {
        public DirectoryFilter( )
        {
        }

        @Override
        public boolean accept( File dir, String name )
        {
            File f = new File( dir, name );
            return ( f.isDirectory( ) );
        }
    }

    /**
     * Helper class for a Button panel (obsolete)
     * Should be replaced by a JOptionPane.
     */
    class ButtonPanel extends JPanel implements ActionListener 
    {

        ButtonPanel( CustomFileDialog parent )
        {
            this.parent = parent;
            add( bOK = new JButton( "OK" ) );
            bOK.addActionListener( this );
            add( bUpdate = new JButton( "Update" ) );
            bUpdate.addActionListener( this );
            add( bCancel = new JButton( "Cancel" ) );
            bCancel.addActionListener( this );
        }

        /**
         * Called in response to an ActionEvent.
         * @param e 
         */
        @Override
        public void actionPerformed( ActionEvent e )
        {
            if ( e.getSource( ) == bOK )
            {
                parent.setVisible( false );
            }
            else if ( e.getSource( ) == bUpdate )
            {
                parent.setFilter( parent.m_filterTextField.getText( ) );
            }
            else if ( e.getSource( ) == bCancel )
            {
                currentFile = null;
                int[] ind = parent.fileList.getSelectedIndices( );
                parent.fileList.clearSelection( );
                parent.setVisible( false );
            }
        }
        
      	CustomFileDialog parent;
        JButton bOK, bUpdate, bCancel;
        
        // Must be declarated somehow
        private static final long serialVersionUID = 3487495895819390L;

    }
    
    // Attributes.
    private final JTextField tfCurrentPath; //, tfSelection;
    
    // text field to set the filter string
    private final JTextField m_filterTextField;
    
    private final JCheckBox cbListFiles;
    private JList dirList, fileList;
    
    private final JPanel        topPanel, centerPanel, cPanelR, 
                                subPanel1, subPanel2;
    private final ButtonPanel   buttonPanel;
    private ImageFormatPanel    formatPanel;

    private String              currentDir;
    private String              currentFile;
    private File                dirFile;
    private boolean             lastListFilesState = false;
    private final Dimension     normalSize;
    private final Dimension     expandedSize;
    private String              patternRoot;

    private String              fSep;

    private final FileFilter fileFilter;
    private final DirectoryFilter dirFilter;
    
    // Must be declarated somehow
    private static final long serialVersionUID = 3487495895819391L;
    
}