/*
 * File: $Id: ImageFormatInfo.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 30. November 2002, 08:03  Authors: Martin Buerki - Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.lib;


/**
 * Utility/container class to handle image format information.
 * (obsolete)
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */

class ImageFormatInfo 
{
    
    /**
     * Default constructor. 
     */
    public ImageFormatInfo( )
    {
    }
    
    /**
     * Alternative constructor of an ImageFormatInfo-object.
     * @param f
     * @param w
     * @param h
     * @param hs
     * @param bpp
     * @param bs
     * @param ext
     * @param c 
     */
    public ImageFormatInfo( int f,	/* the image format */
                            int w,	/* image width */
                            int h,	/* image height */
                            int hs,	/* header size */
                            int bpp,	/* bytes per pixel */
                            boolean bs,	/* byte swap */
                            String ext,	/* file name extension */
                            int c	/* compression */
                          )
    {

        format = f;
        m_width = w;
        m_height = h;
        headerSize = hs;
        bytesPerPixel = bpp;
        byteSwap = bs;
        fileExtension = ext;
        compression = c;
    }

    /**
     * Returns the format attribute.
     * @return 
     */
    public int getFormat( ) 
    {
        return format;
    }

    /**
     * Set the zip extension.
     * @return 
     */
    public String getZipExtension( ) 
    {
        if ( compression == COMPRESSION_NONE ) return "";
        else if ( compression == COMPRESSION_GZIP ) return ".gz";
        else return null;
    }

    /**
     * Set the 'format' which is a type attribute.
     * @param format 
     */
    public void setFormatValues( int format ) 
    {
        Globals globs = Globals.getInstance( );

        switch ( format )
        {
            case Globals.NUMARIS:
                m_width = 0;
                m_height = 0;
                headerSize = 6144;
                bytesPerPixel = 2;
                byteSwap = false;
                fileExtension = ".ima";
                break;
            case Globals.ACR_NEMA:
                m_width = 0;
                m_height = 0;
                headerSize = 0;
                bytesPerPixel = 2;
                byteSwap = true;
                fileExtension = "";
                break;
            case Globals.DICOM:
                m_width = 0;
                m_height = 0;
                headerSize = 0;
                bytesPerPixel = 2;
                byteSwap = false;
                fileExtension = ".dcm";
                break;
            case Globals.CUSTOM:
                m_width = 0;
                m_height = 0;
                headerSize = 0;
                bytesPerPixel = 2;
                byteSwap = false;
                fileExtension = "";
                break;
            default:
                // the required format is not known, therefore 
                // skip the last line, i.e. do not set the format value
                return;
        }
        this.format = format;
    }

    /**
     * Copy values to helper member 'rem' variables.
     */
    public void rememberFormatValues( ) 
    {
        remFormat           = format;
        remImageWidth       = m_width;
        remImageHeight      = m_height;
        remHeaderSize       = headerSize;
        remBytesPerPixel    = bytesPerPixel;
        remFileExtension    = fileExtension;
        remByteSwap         = byteSwap;
        remCompression      = compression;
    }

    /**
     * Restore format values.
     */
    public void restoreFormatValues( ) 
    {
        format          = remFormat;
        m_width         = remImageWidth;
        m_height        = remImageHeight;
        headerSize      = remHeaderSize;
        bytesPerPixel   = remBytesPerPixel;
        fileExtension   = remFileExtension;
        byteSwap        = remByteSwap;
        compression     = remCompression;
    }
    
    /**
     * Returns the image width.
     */
    public int getWidth( )
    {
        return m_width;
    }
    

    /**
     * Returns the image height.
     * @return 
     */
    public int getHeight( )
    {
        return m_width;
    }

    /**
     * Returns the image width.
     * @param width 
     */
    public void setWidth( int width )
    {
        m_width = width;
    }
    
    /**
     * Returns the image height. 
     * @param height 
     */
    public void setHeight( int height )
    {
        m_height = height;
    }
    
    // Attributes
    public static final int COMPRESSION_NONE = 0;
    public static final int COMPRESSION_GZIP = 1;
    
    // Slice width
    private int     m_width;
    // Slice height
    private int     m_height;
    
    public int      headerSize;
    public int      bytesPerPixel;
    public boolean  byteSwap;
    public String   fileExtension;
    public int      compression;

    private int     format;

    private int     remFormat;
    private int     remImageWidth;
    private int     remImageHeight;
    private int     remHeaderSize;
    private int     remBytesPerPixel;
    private String  remFileExtension;
    private boolean remByteSwap;
    private int     remCompression;
    
}
