/*
 * File: $Id: ImageFormatPanel.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 30. November 2002, 08:03  Authors: Martin Buerki - Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.lib;
 
import java.awt.Container;
import java.awt.Window;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;

import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import java.util.Vector;



class ImageFormatPanel extends JPanel implements ActionListener, ItemListener 
{


    /**
     * Class defining the handling of image formats;
     * (obsolete).
     * @param parent
     * @param win 
     */
    public ImageFormatPanel( Container parent, Window win ) 
    {
	
        this.parent = parent;
        windowHook = win;
// 	normalWinSize = new Dimension(550, 100);
// 	expandedWinSize = new Dimension(550, 300);
        multicaster = new Vector<NamePatternListener>( );

        setLayout( new BorderLayout( ) );
        
        basicPanel = new JPanel( );
	
        add( basicPanel, BorderLayout.NORTH );
        basicPanel.add( new JLabel( "Image format" ) );
	//qqq basicPanel.add( cFileFormat = new Choice( ) );
        basicPanel.add( cFileFormat = new JComboBox( ) );
        cFileFormat.addItemListener( this );
        	
        basicPanel.add( cbZipped = new JCheckBox( "Zipped" ) );
        cbZipped.addItemListener( this );
        basicPanel.add( bAdvanced = new JButton( "Advanced >>>" ) );
        bAdvanced.addActionListener( this );
 
        add( advancedPanel = new JPanel( new GridLayout( 0, 2 ) ), BorderLayout.CENTER );
        advancedPanel.add( cbByteSwap = new JCheckBox( "Byte swap", false ) );
        advancedPanel.add( new JLabel( " " ) );
        advancedPanel.add( new JLabel( "Image Width" ) );
        advancedPanel.add( tfImageWidth = new JTextField( "", 10 ) );
        advancedPanel.add( new JLabel( "Image Height" ) );
        advancedPanel.add( tfImageHeight = new JTextField( "", 10 ) );
        advancedPanel.add( new JLabel("Bytes per pixel" ) );
        advancedPanel.add( tfBytesPerPixel = new JTextField( "", 10 ) );
        advancedPanel.add( new JLabel( "File extension" ) );
        advancedPanel.add( tfFileExtension = new JTextField( "", 30 ) );
        tfFileExtension.addActionListener( this );
    	advancedPanel.add( new JLabel( "File header size" ) );
        advancedPanel.add( tfHeaderSize = new JTextField( "", 30 ) );
        advancedPanel.setVisible( false );

        Globals globs = Globals.getInstance( );

        for ( int k = 0; k < globs.fileFormats.length; k++ )
        {
            cFileFormat.addItem( globs.fileFormats[k] );
        }
        
        cFileFormat.setSelectedItem( globs.fileFormats[globs.m_defaultFileFormat] );
	
        setFormatValues( getFileFormat( ) );
    }

    /**
     * Defines diverse window sizes.
     * @param normalSize
     * @param expandedSize 
     */
    public void setWindowHookSizes( Dimension normalSize, Dimension expandedSize ) 
    {
	normalWinSize = new Dimension( normalSize );
	expandedWinSize = new Dimension( expandedSize );
    }

    /**
     * Returns the image format information object.
     * @return 
     */
    public ImageFormatInfo getImageFormatInfo( ) 
    {
        int f = getFileFormat( );
        int w = getImageWidth( );
        int h = getImageHeight( );
        int hs = getHeaderSize( );
        int bpp = getBytesPerPixel( );
        boolean bs = isByteSwap( );
        String ext = getFileExtension( );
        int c = getCompression( );

        return new ImageFormatInfo( f, w, h, hs, bpp, bs, ext, c );
    }

    /**
     * Returns the file format related integer value. 
     * @return 
     */
    public int getFileFormat( ) 
    {
        return cFileFormat.getSelectedIndex( );
    }

    /**
     * Returns the file extension. 
     * @return 
     */
    public String getFileExtension( ) 
    {
        return tfFileExtension.getText( );
    }
    
    /**
     * Returns the zip extension.
     * @return 
     */
    public String getZipExtension( ) 
    {
        if ( isZipped( ) ) return ".gz";
        else return "";
    }

    /**
     * Returns whether file is zipped or not. 
     * @return 
     */
    public int getCompression( ) 
    {
        if ( isZipped( ) )
        { 
            return ImageFormatInfo.COMPRESSION_GZIP;
        }
        else return ImageFormatInfo.COMPRESSION_NONE;
    }

    /**
     * Returns boolean whether file is zipped or not.
     * @return 
     */
    public boolean isZipped( ) 
    {
        return cbZipped.isSelected( );
        //qqq return cbZipped.getState();
    }

    /**
     * Returns whether the bytes in the binary file are swapped or not.
     * @return 
     */
    public boolean isByteSwap( ) 
    {
        return cbByteSwap.isSelected( );
        //qqq return cbByteSwap.getState();
    }

    /**
     * Returns the image width attribute.
     * @return 
     */
    public int getImageWidth( ) 
    {
        try
        {
            return Integer.valueOf( tfImageWidth.getText( ) ).intValue( );
        }
        catch ( NumberFormatException e )
        {
            return 0;
        }
    }
    
    /**
     * Returns the image height attribute.
     * @return
     */ 
    public int getImageHeight( ) 
    {
        try
        {
            return Integer.valueOf( tfImageHeight.getText( ) ).intValue( );
        }
        catch ( NumberFormatException e )
        {
            return 0;
        }
    }

    /**
     * 
     * @return 
     */
    public int getBytesPerPixel( ) 
    {
        try
        {
            return Integer.valueOf( tfBytesPerPixel.getText( ) ).intValue( );
        }
        catch ( NumberFormatException e )
        {
            return 0;
        }
    }

    /**
     * Returns size of header in bytes.
     * @return 
     */
    public int getHeaderSize( ) 
    {
        try
        {
            return Integer.valueOf( tfHeaderSize.getText( ) ).intValue( );
        }
        catch ( NumberFormatException e )
        {
            return 0;
        }
    }

    /**
     * Copies all attributes of this class.
     */
    public void rememberFormatValues( ) 
    {
        remImageWidth = tfImageWidth.getText( );
        remImageHeight = tfImageHeight.getText( );
        remHeaderSize = tfHeaderSize.getText( );
        remBytesPerPixel = tfBytesPerPixel.getText( );
        remFileExtension = tfFileExtension.getText( );
        //qqq remByteSwap = cbByteSwap.getState();
        remByteSwap = cbByteSwap.isSelected( );
    }

    /**
     * Resets the old values onto the GUI.
     */
    public void resetFormatValues( ) 
    {
        tfImageWidth.setText( remImageWidth );
        tfImageHeight.setText( remImageHeight );
        tfHeaderSize.setText( remHeaderSize );
        tfBytesPerPixel.setText( remBytesPerPixel );
        tfFileExtension.setText( remFileExtension );
        //qqq cbByteSwap.setState(remByteSwap);
        cbByteSwap.setSelected( remByteSwap );
    }

    
    /**
     * Sets the format of the GUI.
     * @param format 
     */
    public void setFormatValues( int format ) 
    {

        Globals globs = Globals.getInstance( );

        if ( format == globs.NUMARIS )
        {
            tfImageWidth.setEnabled( false );
            tfImageWidth.setBackground( Color.gray );
            tfImageWidth.setText( "" );
            tfImageHeight.setEnabled( false );
            tfImageHeight.setBackground( Color.gray );
            tfImageHeight.setText( "" );
            tfHeaderSize.setEnabled( false );
            tfHeaderSize.setBackground( Color.gray );
            tfHeaderSize.setText( "6144" );
            tfBytesPerPixel.setEnabled( false );
            tfBytesPerPixel.setBackground( Color.gray );
            tfBytesPerPixel.setText( "2" );
            cbByteSwap.setSelected( false );
            // qqq cbByteSwap.setState(false);
            tfFileExtension.setText( ".ima" );
            //cbByteSwap.setEnabled(false);
            //Globals.setNumarisValues();
        }
        else if ( format == globs.ACR_NEMA )
        {
            tfImageWidth.setEnabled( true );
            tfImageWidth.setBackground( Color.white );
            tfImageHeight.setEnabled( true );
            tfImageHeight.setBackground( Color.white );
            tfBytesPerPixel.setEnabled( false );
            tfBytesPerPixel.setBackground(Color.gray);
            tfBytesPerPixel.setText( "2" );
            //qqq cbByteSwap.setState(true);
            cbByteSwap.setSelected( true );
            tfFileExtension.setText( "" );
            tfHeaderSize.setEnabled( false );
            tfHeaderSize.setBackground( Color.gray );
            tfHeaderSize.setText( "" );
            //cbByteSwap.setEnabled(false);
            //Globals.setAcrNemaValues();
        }
        else if ( format == globs.DICOM )
        {
            tfImageWidth.setEnabled( false );
            tfImageWidth.setBackground( Color.gray );
            tfImageWidth.setText( "" );
            tfImageHeight.setEnabled( false );
            tfImageHeight.setBackground( Color.gray );
            tfImageHeight.setText( "" );
            tfBytesPerPixel.setEnabled( false );
            tfBytesPerPixel.setBackground( Color.gray );
            tfBytesPerPixel.setText( "2" );
            // cbByteSwap.setState(false);
            cbByteSwap.setSelected( false );
            tfFileExtension.setText( ".dcm" );
           
            tfHeaderSize.setEnabled( false );
            tfHeaderSize.setBackground( Color.gray );
            tfHeaderSize.setText( "" );
            //cbByteSwap.setEnabled(false);
            //Globals.setAcrNemaValues();
        }
        else if ( format == globs.CUSTOM )
        {
            tfImageWidth.setEnabled( true );
            tfImageWidth.setBackground( Color.white );
            tfImageHeight.setEnabled( true );
            tfImageHeight.setBackground( Color.white );
            tfBytesPerPixel.setEnabled( true );
            tfBytesPerPixel.setBackground( Color.white );
            tfFileExtension.setText( "" );
            cbByteSwap.setEnabled( true );
            tfHeaderSize.setEnabled( true );
            tfHeaderSize.setBackground( Color.white );
            tfHeaderSize.setText( "" );
        }
    }

    /**
     * Is called in response to an ActionEvent.
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
	Object sender = e.getSource( );
        if ( sender == bAdvanced )
        {
            if ( advancedPanel.isVisible( ) )
            {
                advancedPanel.setVisible( false );
                bAdvanced.setText( "Advanced >>>" );
                windowHook.setSize( normalWinSize );
            }
            else
            {
                advancedPanel.setVisible( true );
                bAdvanced.setText( "Advanced <<<" );
                windowHook.setSize( expandedWinSize );
            }
            parent.doLayout( );
        }
        else if ( sender == tfFileExtension )
        {
            castNamePatternEvent( );
        }
    }

    /**
     * Is called when an item changed its value.
     * @param e 
     */
    @Override
    public void itemStateChanged( ItemEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender == cbZipped )
        {
            castNamePatternEvent( );
        }
        else if ( sender == cFileFormat )
        {
            setFormatValues( getFileFormat( ) );
            castNamePatternEvent( );
        }
    }

    /**
     *
     * Adds name to pattern listener.
     * @param l 
     **/
    public void addNamePatternListener( NamePatternListener l ) 
    {
        if ( ! multicaster.contains( l ) )
        { 
            multicaster.addElement( l );
        }
    }

    /**
     * Removes name from pattern listener.
     * @param l 
     */
    public void removeNamePatternListener( NamePatternListener l ) 
    {
        multicaster.removeElement( l );
    }

    /**
     * Evokes the name pattern event to all listeners.
     */
    private void castNamePatternEvent( ) 
    {
        for ( int i=0; i < multicaster.size( ); i++ )
        {
            multicaster.elementAt( i ).namePatternChanged( );
        }
    }
    
    
    // Attributes.
    
    JPanel basicPanel;
    JPanel advancedPanel;
    JButton bAdvanced;
    JTextField tfBytesPerPixel;
    JTextField tfImageWidth;
    JTextField tfImageHeight;
    JTextField tfHeaderSize;
    JCheckBox cbByteSwap;
    JComboBox cFileFormat;
    // Choice cFileFormat;
    JTextField tfFileExtension;
    JCheckBox cbZipped;

    Container parent;
    Window windowHook;
    private Vector <NamePatternListener> multicaster;
    private Dimension normalWinSize;
    private Dimension expandedWinSize;

    private String remImageWidth;
    private String remImageHeight;
    private String remHeaderSize;
    private String remBytesPerPixel;
    private String remFileExtension;
    private boolean remByteSwap;
    
    // Must be declarated somehow
    private static final long serialVersionUID = 3487495895813493L;
}
