/*
 * File: $Id: StatusPanel.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date:   Author: Martin Buerki and Hans Slotboom  
 *
 * Changes:
 *
 * Copyright 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
*/

package org.spectrim.lib;

import java.awt.Dimension;
import java.awt.FlowLayout;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.Timer;

/**
 * Constructs a StatusPanel object with two text fields that can show texts
 */

public class StatusPanel extends JPanel implements ActionListener
{

    /**
     *  Constructs a StatusPanel
     * @param parent
     */
    public StatusPanel( JFrame parent ) 
    {
	this.parent = parent;
        this.setLayout( new BoxLayout( this,BoxLayout.Y_AXIS ) );
        m_collectionPanel = new JPanel( );
        
	m_collectionPanel.setLayout( new BoxLayout( m_collectionPanel, BoxLayout.X_AXIS ) );
        //setPreferredSize( new Dimension( ( parent.getSize( ).width ) - 10, 30  ) );
        m_leftPanel.setBorder( BorderFactory.createLoweredBevelBorder( ) );
        m_leftPanel.setLayout( new FlowLayout(  ) ); 
        m_leftMessage = new JLabel( " " );

	m_leftPanel.add( m_leftMessage );
        m_rightPanel.setBorder( BorderFactory.createLoweredBevelBorder( ) );
        m_rightPanel.setLayout( new FlowLayout( ) );
        m_rightPanel.add( m_rightMessage = new JLabel( " " ) );
        
        m_collectionPanel.add( m_leftPanel );
        m_collectionPanel.add( m_rightPanel );
        
        add( m_collectionPanel );
        
        m_rightTimer = new Timer( 2000, this );
        m_rightTimer.start( );
        m_leftTimer = new Timer( 2000, this );
        m_leftTimer.start( );
    }

    /**
     * Sets the right side message string
     * @param msg
     */    
    public synchronized void setLeftMessage( String msg ) 
    {
	m_leftMessage.setText( msg );
        m_leftTimer.stop( );
        m_leftTimer.start( );
    }
    
    /**
     * Sets the right side message string without a time out; remains visible
     * until a new string is displayed or the program terminates
     * @param msg
     */    
    public synchronized void setLeftMessageNoTimeOut( String msg ) 
    {
        // stop firing of timer, since this triggers the clearing of 
        // the information line
        m_leftTimer.stop( );
        // write the message string
	m_leftMessage.setText( msg );
    }
    
    /**
     * Sets the right side message string without time out; remains visible
     * until a new string is displayed or the program terminates
     * @param msg
     */    
    public synchronized void setRightMessageNoTimeOut( String msg )
    {
        // stop firing of timer, since this triggers the clearing of 
        // the information line
        m_rightTimer.stop( );
	// write the message string
        m_rightMessage.setText( msg );
    }
    
    /**
     * Sets the right side message string
     * @param msg
     */    
    public synchronized void setRightMessage( String msg ) 
    {
	m_rightMessage.setText( msg );
        m_rightTimer.stop( );
        m_rightTimer.start( );
    }

    /**
     * Clears the left side message
     */
    public void clearLeftMessage( )
    {
	m_leftMessage.setText( " " );
    }
    
    /**
     * Clears the right side message
     */
    public void clearRightMessage( ) 
    {
	m_rightMessage.setText( " " );
    }
    
    /**
     * Is called whenever an registered component raises a ActionEvent
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
       
        // the timer started on the moment the user rotated the mouse wheel
        // was timed out: now the GUI update can take place
        if ( sender == m_rightTimer )
        {
            clearRightMessage( );
        }

        if ( sender == m_leftTimer )
        {    
            clearLeftMessage( ) ;
        }
    }
    
    private JFrame parent;
    private JLabel m_leftMessage;
    private JLabel m_rightMessage;
    private JPanel m_collectionPanel;
    private JPanel m_leftPanel = new JPanel( );
    private JPanel m_rightPanel = new JPanel( );
    private Timer m_rightTimer;
    private Timer m_leftTimer;
    
    
    // Must be declarated somehow
    private static final long serialVersionUID = 34874578952319393L;
}
