/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.main;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JOptionPane;

/**
 * This DataLoadOptionFrame enables the user to define how the data has to be 
 * loaded that contains more than one instance.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class DataLoadOptionFrame extends javax.swing.JFrame {
    
    private LinkedList<String>          m_fileReferences;
    private LinkedList<ActionListener>  m_actionListeners = new LinkedList<>( );
    private int                         m_instanceNumberToLoad = -1;
    
    // Member variables needed for notification
    
    public final static int             LOAD_SINGLE_INSTANCE        = 0;
    public final static String          LOAD_SINGLE_INSTANCE_CMD    = "LOAD_DEFAULT_SINGLE_INSTANCE";
    
    public final static int             LOAD_ALL_AND_SUM            = 1;
    public final static String          LOAD_ALL_AND_SUM_CMD        = "LOAD_ALL_AND_AVERAGE";
    
    public final static int             LOAD_SINGLE_INSTANCE_OF_MULTIPLE     = 2;
    public final static String          LOAD_SINGLE_INSTANCE_OF_MULTIPLE_CMD = "LOAD_SINGLE_INSTANCE_OF_MULTIPLE";
    
    public final static int             LOAD_DIFF_1_MINUS_2      = 3;
    public final static String          LOAD_DIFF_1_MINUS_2_CMD  = "LOAD_DIFF_1_MINUS_2";
    
    public final static int             LOAD_DIFF_2_MINUS_1      = 4;
    public final static String          LOAD_DIFF_2_MINUS_1_CMD  = "LOAD_DIFF_2_MINUS_1";
    
    
    /**
     * Creates new form DataLoadOptionFrame.
     * @param fileReferences
     * @param seriesDescription
     */
    public DataLoadOptionFrame( LinkedList<String> fileReferences, 
                                String seriesDescription ) 
    {
        initComponents( );
        m_fileReferences = fileReferences;
        int numberOfInstances = m_fileReferences.size( );
        
        
        if  (seriesDescription != null )
        {
            if ( seriesDescription.contains( "-" ) == true )
            {
                seriesDescription = seriesDescription.substring( seriesDescription.lastIndexOf( "-" ) + 1, 
                                                                 seriesDescription.length( ) ).trim( );
            }
        }
        else
        {
            seriesDescription = "Not defined";
        }
        
        m_seriesNameValueLabel.setText( seriesDescription );
        m_numberOfResponsesValueLabel.setText( "" + numberOfInstances );
        m_totalNumberOfResponsesValueLabel.setText( "" + numberOfInstances );
        
        if ( numberOfInstances > 2 )
        {
            m_loadAndSumRadioButton.setEnabled(true);
            m_loadAndSumRadioButton.setSelected(true);
            m_loadSingleInstanceNumberRadioButton.setEnabled(true);
            m_loadDifference_1_minus_2_RadioButton.setEnabled(false);
            m_loadDifference_2_minus_1_RadioButton.setEnabled(true);
        }
        
        if ( numberOfInstances == 2 )
        {
            m_loadAndSumRadioButton.setEnabled(true);
            m_loadSingleInstanceNumberRadioButton.setEnabled(true);
            m_loadDifference_1_minus_2_RadioButton.setEnabled(true);
            m_loadDifference_1_minus_2_RadioButton.setSelected(true);
            m_loadDifference_2_minus_1_RadioButton.setEnabled(true);
        }
        
        m_instanceNumberToLoadTextField.setEnabled(false);
        m_responseOneTextField.setEnabled(false);
        m_responseTwoTextField.setEnabled(false);
        
        setDefaultCloseOperation( DISPOSE_ON_CLOSE );
        
        pack( );

    }
    
    /**
     * Returns the linked list of file references to be loaded.
     * @return 
     */
    public LinkedList<String> getFileReferences( )
    {
        return m_fileReferences;
    }
            
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_optionSelectionButtonGroup = new javax.swing.ButtonGroup();
        jPanel1 = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        m_loadAndSumRadioButton = new javax.swing.JRadioButton();
        m_loadSingleInstanceNumberRadioButton = new javax.swing.JRadioButton();
        jLabel1 = new javax.swing.JLabel();
        m_totalNumberOfResponsesValueLabel = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        m_instanceNumberToLoadTextField = new javax.swing.JTextField();
        m_loadDifference_1_minus_2_RadioButton = new javax.swing.JRadioButton();
        m_loadDifference_2_minus_1_RadioButton = new javax.swing.JRadioButton();
        m_responseOneTextField = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        m_responseTwoTextField = new javax.swing.JTextField();
        jPanel3 = new javax.swing.JPanel();
        m_cancelLoadDataButton = new javax.swing.JButton();
        m_loadDataButton = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        m_seriesNameTitleLabel = new javax.swing.JLabel();
        m_seriesNameValueLabel = new javax.swing.JLabel();
        m_numberOfResponsesTitleLabel = new javax.swing.JLabel();
        m_numberOfResponsesValueLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Multiple spectra responses selected"));

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Options"));

        m_optionSelectionButtonGroup.add(m_loadAndSumRadioButton);
        m_loadAndSumRadioButton.setText(" Load all and sum responses");
        m_loadAndSumRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadAndSumRadioButtonActionPerformed(evt);
            }
        });

        m_optionSelectionButtonGroup.add(m_loadSingleInstanceNumberRadioButton);
        m_loadSingleInstanceNumberRadioButton.setText("Load single response number");
        m_loadSingleInstanceNumberRadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadSingleInstanceNumberRadioButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("of");

        m_totalNumberOfResponsesValueLabel.setText("-1");

        jLabel2.setText("responses.");

        m_instanceNumberToLoadTextField.setText("1");

        m_optionSelectionButtonGroup.add(m_loadDifference_1_minus_2_RadioButton);
        m_loadDifference_1_minus_2_RadioButton.setText("Load and compute difference (first minus second instance number).");

        m_optionSelectionButtonGroup.add(m_loadDifference_2_minus_1_RadioButton);
        m_loadDifference_2_minus_1_RadioButton.setText("Load and compute difference (second minus first instance number).");
        m_loadDifference_2_minus_1_RadioButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadDifference_2_minus_1_RadioButtonActionPerformed(evt);
            }
        });

        m_responseOneTextField.setText("2");

        jLabel3.setText("-");

        m_responseTwoTextField.setText("1");

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_loadAndSumRadioButton)
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addComponent(m_loadSingleInstanceNumberRadioButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_instanceNumberToLoadTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 83, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jLabel1)
                        .addGap(18, 18, 18)
                        .addComponent(m_totalNumberOfResponsesValueLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 39, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel2))
                    .addComponent(m_loadDifference_1_minus_2_RadioButton)
                    .addGroup(jPanel2Layout.createSequentialGroup()
                        .addComponent(m_loadDifference_2_minus_1_RadioButton)
                        .addGap(33, 33, 33)
                        .addComponent(m_responseOneTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 44, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(jLabel3)
                        .addGap(18, 18, 18)
                        .addComponent(m_responseTwoTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 44, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(213, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap(30, Short.MAX_VALUE)
                .addComponent(m_loadAndSumRadioButton)
                .addGap(18, 18, 18)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_loadSingleInstanceNumberRadioButton)
                    .addComponent(jLabel1)
                    .addComponent(m_totalNumberOfResponsesValueLabel)
                    .addComponent(jLabel2)
                    .addComponent(m_instanceNumberToLoadTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addComponent(m_loadDifference_1_minus_2_RadioButton)
                .addGap(18, 18, 18)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_loadDifference_2_minus_1_RadioButton)
                    .addComponent(m_responseOneTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3)
                    .addComponent(m_responseTwoTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Actions"));

        m_cancelLoadDataButton.setText("Cancel");
        m_cancelLoadDataButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_cancelLoadDataButtonActionPerformed(evt);
            }
        });

        m_loadDataButton.setText("Load");
        m_loadDataButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadDataButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGap(131, 131, 131)
                .addComponent(m_cancelLoadDataButton, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 333, Short.MAX_VALUE)
                .addComponent(m_loadDataButton, javax.swing.GroupLayout.PREFERRED_SIZE, 90, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(154, 154, 154))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_cancelLoadDataButton)
                    .addComponent(m_loadDataButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("General"));

        m_seriesNameTitleLabel.setText("Selected series name: ");

        m_seriesNameValueLabel.setText("series name");

        m_numberOfResponsesTitleLabel.setText("Number of responses:");

        m_numberOfResponsesValueLabel.setText("-1");

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_seriesNameTitleLabel)
                    .addComponent(m_numberOfResponsesTitleLabel))
                .addGap(78, 78, 78)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(m_seriesNameValueLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_numberOfResponsesValueLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 70, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_seriesNameTitleLabel)
                    .addComponent(m_seriesNameValueLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfResponsesTitleLabel)
                    .addComponent(m_numberOfResponsesValueLabel))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel3, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel4, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_cancelLoadDataButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_cancelLoadDataButtonActionPerformed
        dispose( );
    }//GEN-LAST:event_m_cancelLoadDataButtonActionPerformed

    private void m_loadSingleInstanceNumberRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadSingleInstanceNumberRadioButtonActionPerformed
        m_instanceNumberToLoadTextField.setEnabled(true);
        m_responseOneTextField.setEnabled(false);
        m_responseTwoTextField.setEnabled(false);
    }//GEN-LAST:event_m_loadSingleInstanceNumberRadioButtonActionPerformed

    private void m_loadDataButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadDataButtonActionPerformed
        if ( m_loadSingleInstanceNumberRadioButton.isSelected( ) == true )
        {
            m_instanceNumberToLoad = new Integer( m_instanceNumberToLoadTextField.getText( ) ).intValue( );
           
            boolean indexesOK = true;
            if ( ( m_instanceNumberToLoad < 1 ) || ( m_instanceNumberToLoad > m_fileReferences.size( ) ) )
            {
                JOptionPane.showMessageDialog( this, "Single response number outside valid range! [1, " + m_fileReferences.size( ) + "]" );
                indexesOK = false;
            }
            
            
            if ( indexesOK == true )
            {
                notifyActionListeners( this, LOAD_SINGLE_INSTANCE, LOAD_SINGLE_INSTANCE_CMD );
                dispose( );
            }      
        }
        
        if ( m_loadAndSumRadioButton.isSelected( ) == true )
        {
            notifyActionListeners( this, LOAD_ALL_AND_SUM, LOAD_ALL_AND_SUM_CMD );
            dispose( );
        }
        
        if ( m_loadDifference_1_minus_2_RadioButton.isSelected( ) == true )
        {
            notifyActionListeners( this, LOAD_DIFF_1_MINUS_2, LOAD_DIFF_1_MINUS_2_CMD );
            dispose( );
        }
        
        if ( m_loadDifference_2_minus_1_RadioButton.isSelected( ) == true )
        {
            int responseOneTextFieldAsInt = new Integer( m_responseOneTextField.getText( ) ).intValue( );
            int responseTwoTextFieldAsInt = new Integer( m_responseTwoTextField.getText( ) ).intValue( );
            boolean indexesOK = true;
            
            if ( ( responseOneTextFieldAsInt < 1 ) || ( responseOneTextFieldAsInt > m_fileReferences.size( ) ) )
            {
                JOptionPane.showMessageDialog( this, "First response number(s) outside valid range! [1, " + m_fileReferences.size( ) + "]" );
                indexesOK = false;
            }
            
            if ( ( responseTwoTextFieldAsInt < 1 ) || ( responseTwoTextFieldAsInt > m_fileReferences.size( ) ) )
            {
                JOptionPane.showMessageDialog( this, "Second response number(s) outside valid range! [1, " + m_fileReferences.size( ) + "]" );
                indexesOK = false;
            }
            
            if ( indexesOK == true )
            {
                notifyActionListeners( this, LOAD_DIFF_2_MINUS_1, LOAD_DIFF_2_MINUS_1_CMD );
                dispose( );
            }
        }
        
    }//GEN-LAST:event_m_loadDataButtonActionPerformed

    private void m_loadAndSumRadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadAndSumRadioButtonActionPerformed
      m_responseOneTextField.setEnabled(false);
      m_responseTwoTextField.setEnabled(false);
      m_instanceNumberToLoadTextField.setEnabled(false);
    }//GEN-LAST:event_m_loadAndSumRadioButtonActionPerformed

    private void m_loadDifference_2_minus_1_RadioButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadDifference_2_minus_1_RadioButtonActionPerformed
      m_responseOneTextField.setEnabled(true);
      m_responseTwoTextField.setEnabled(true);
      m_instanceNumberToLoadTextField.setEnabled(false);
    }//GEN-LAST:event_m_loadDifference_2_minus_1_RadioButtonActionPerformed

 

    /**
     * Adds an action listener to this class.
     * @param e
     */
     public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    /**
     * 
     * Method called to notify registered classes of certain events.
     * 
     * @param obj
     * @param eventType
     * @param command 
     */
    private void notifyActionListeners( Object obj, int eventType,  String command )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, command ) );
        }
    }
     
    /**
     * Returns the instance number to be loaded of multiple instances of the
     * same series.
     * @return 
     */
    public int getSelectedInstanceNumber( )
    {
        return m_instanceNumberToLoad;
    }
    
    public int getFirstOfTwoResponses( )
    {
        return Integer.parseInt(m_responseOneTextField.getText( ) );
    }
    
    public int getSecondOfTwoResponses( )
    {
        return Integer.parseInt(m_responseTwoTextField.getText( ) );
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JButton m_cancelLoadDataButton;
    private javax.swing.JTextField m_instanceNumberToLoadTextField;
    private javax.swing.JRadioButton m_loadAndSumRadioButton;
    private javax.swing.JButton m_loadDataButton;
    private javax.swing.JRadioButton m_loadDifference_1_minus_2_RadioButton;
    private javax.swing.JRadioButton m_loadDifference_2_minus_1_RadioButton;
    private javax.swing.JRadioButton m_loadSingleInstanceNumberRadioButton;
    private javax.swing.JLabel m_numberOfResponsesTitleLabel;
    private javax.swing.JLabel m_numberOfResponsesValueLabel;
    private javax.swing.ButtonGroup m_optionSelectionButtonGroup;
    private javax.swing.JTextField m_responseOneTextField;
    private javax.swing.JTextField m_responseTwoTextField;
    private javax.swing.JLabel m_seriesNameTitleLabel;
    private javax.swing.JLabel m_seriesNameValueLabel;
    private javax.swing.JLabel m_totalNumberOfResponsesValueLabel;
    // End of variables declaration//GEN-END:variables
}
