/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.main;

import java.util.ArrayList;
import java.util.HashMap;
import org.spectrim.tdfdfit.NumericTDFDFitResults;

/**
 * Interface that enables the definition of fitting results in jMRUI as
 * well as in SpiectrIm.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public interface ResultsStoreInterface 
{
        
    /**
     * Adds the results from TDFDfit to the current HashMap.
     * @param tdfdfitResults
     * @param errorMap
     * @param noiseStdMap
     */
    public void addTDFDFitResultsData( HashMap<String,NumericTDFDFitResults> tdfdfitResults,
                                       HashMap<String, float[]> errorMap,
                                       HashMap<String, Float> noiseStdMap );
    
    /**
     * Clears the results map.
     */
    public void clearData( ); 

    /**
     * Removes the voxels from t
     * @param voxels 
     */
    public void removeAllVoxels( ArrayList<String> voxels );
    
    /**
     * Removes specific voxel from voxels belonging voxelName. 
     * @param voxelName
     */
    public void removeVoxel( String voxelName );

    /**
     * Returns the metabolites and their features at the given coordinate.
     * @param coord
     * @return 
     */
    public VoxelMetabolites getMetabolitesAt( String coord ); 

    /**
     * Returns all the names of the available metabolites.
     * @return 
     */
    public String[] getAvailableMetabolites( ); 

    /**
     * Returns the given feature for the given coordinates and the given
     * metabolite; NaN if not existing.
     * @param coord
     * @param metab
     * @param feature
     * @return 
     */
    public double getFeature( String coord, String metab, int feature ); 

    /**
     * Returns a featureMap containing all the values for the given feature and
     * metabolite.
     * @param metab
     * @param feature
     * @return 
     */
    public HashMap<String, Double> getFeatureMap( String metab, int feature ); 
   
    /**
     * Returns a Map containing the global fitting error
     * @return 
     */
    public HashMap<String, Double> getGlobalErrorMap( ); 
 
    /**
     * Returns the map with metabolites and features.
     * @return 
     */
    public HashMap<String, VoxelMetabolites> getVoxelsMetabolites( ); 

    /**
     * Returns the list of the voxels as ArrayList.
     * @return 
     */
    public ArrayList<String> getAvailableVoxels( ); 
    
}