/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.main;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Properties;
import org.spectrim.tdfdfit.TDFDFittingDefaults;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.TDFDFIT_MODELS_BASE_PATH;

/**
 * This properties class holds the information needed to initialize 
 * all settings for the ClinicalViewer, named "Clinical SpectrIm".
 * @author Hans
 */
public class SpectrImClinicalQMRSBoot extends Properties 
{
    /** 
     * Creates a new instance of TDFDFittingDefaults. 
     */
    private SpectrImClinicalQMRSBoot( )
    {
        String sep =  System.getProperty( "file.separator" );
        setProperty( SPECTRIM_CLINICAL_QMRS_BASE_PATH, System.getProperty( "user.home" ) + sep + ".SpectrImClinicalQMRS" + sep +  "SpectrimClinicalQMRS.ini" );       
    }
    
    /**
     * Returns the one and only instance of MRFittingDefaults.
     * @return 
     */
    public static SpectrImClinicalQMRSBoot getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new SpectrImClinicalQMRSBoot( );
            theInstance.load( );
        }
        return theInstance;
    }
    
    
    /**
     * Loads the user preferences file.
     */ 
    public void load( )
    {
        String sep =  System.getProperty( "file.separator" );
        m_spectrImClinicalBootFileName = System.getProperty( "user.home" ) + sep + ".SpectrImClinicalBoot" + sep +  "SpectrimClinicalQMRSBoot.ini";
        String defaultSpectrimClinicalPath = System.getProperty( "user.home" ) + sep + ".SpectrImClinicalQMRS";
        
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( m_spectrImClinicalBootFileName ) );
        
            // now the ini file must exist with default contents, so load it
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            try
            {
                File f = new File( m_spectrImClinicalBootFileName );
                if ( f.exists( ) == false )
                {
                    File TDFDDir = new File( f.getParent( ) );
                    if ( TDFDDir.exists( ) == false )
                    {
                        TDFDDir.mkdir( );
                        if ( TDFDDir.exists( ) )
                        {
                            // This boot settings file stores only one piece of infromation
                            // namely where the .SpectrImClinicalQMRS is located
                            // this gives the user the opportunity to set this to a different 
                            // location in the file system than in user.home
                            // since this directory can contain a huge amount of data
                            // and this should be stored in this directory since
                            // this is often on a SSD drive.
                            setProperty( SPECTRIM_CLINICAL_QMRS_BASE_PATH, defaultSpectrimClinicalPath );
                        }
                    }
                } 
                
                OutputStream os = new BufferedOutputStream( new FileOutputStream(  m_spectrImClinicalBootFileName ) );
                this.save( os, "// This is a generated file: do not edit!!" );
                os.close( );
            }
            catch ( IOException ioe )
            { 
                System.out.println( "IO-error occured while attempting to write the SpectrimClinicalQMRSBoot.ini file." );
                // should hopefully never occur!
            }
        }
        
        // now the ini file must exist with default contents, so load it
        try
        {
            InputStream is = new BufferedInputStream( new FileInputStream( m_spectrImClinicalBootFileName ) );
            this.load( is );
            is.close( );
        }
        catch ( IOException e )
        {
            // should never occur
        }
    }
    
   /**
     * Saves the user preferences file.
     */ 
    public void save( )
    {
        try
        {            
            OutputStream out = new BufferedOutputStream( new FileOutputStream( m_spectrImClinicalBootFileName ) );
            this.save( out, "//This is a generated file : do not edit" );
        }
        catch ( IOException e )
        {
             System.out.println( "IO-error occured while attempting to write the SpectrimClinicalQMRSBoot.ini file." );
        }
    }
    
    
    
    
    // static private attribute is the instance itself (Singleton!)
    public final static String SPECTRIM_CLINICAL_QMRS_BASE_PATH     = "Spectrim clinical base path";
    private String m_spectrImClinicalBootFileName;
    private static  SpectrImClinicalQMRSBoot theInstance            = null;
}
