/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.selectionprofile;

import com.jmatio.types.MLArray;
import com.jmatio.types.MLNumericArray;
import java.io.File;
import java.util.HashMap;
import java.util.LinkedHashMap;
import org.spectrim.settings.GlobalSettings;
import org.spectrim.utilities.MatlabIO;

/**
 *
 * @author nunopbarros
 */
public class SelectionProfileCorrection {

    private final double TE;
    private final double[] excitationVolumeDimensions;
    private final HashMap<String, HashMap<String, Double>> correctionFactors;

//  TODO: verify fieldStrength, Vendor, Model, Orientation,.... 
    public SelectionProfileCorrection(double TE, double[] excitationVolDims) throws NoSuchFieldException {
        this.TE = TE;
        this.excitationVolumeDimensions = excitationVolDims;
        correctionFactors = new HashMap<>();
        if (!readCorrectionFile("Cho") || !readCorrectionFile("Cr") || !readCorrectionFile("Lac") || !readCorrectionFile("NAA")) {
            throw new NoSuchFieldException("Correction Factors for TE" + TE + " and dimensions " + (int) Math.round(excitationVolumeDimensions[0]) + "x" + (int) Math.round(excitationVolumeDimensions[1]));
        }
    }

    private boolean readCorrectionFile(String metabName) {
        File file = new File(GlobalSettings.getInstance().getSpectrimRootDir() + File.separator + "selectionProfiles" + File.separator + "TE" + (int) TE + "_"
                + (int) Math.round(excitationVolumeDimensions[0]) + "_" + (int) Math.round(excitationVolumeDimensions[1]) + "_" + metabName + ".mat");
        if (file.exists()) {
            HashMap<String, MLArray> data = MatlabIO.readMAT(file.toString());
            MLNumericArray<Double> Xlim = (MLNumericArray<Double>) data.get("Xlim");
            MLNumericArray<Double> Ylim = (MLNumericArray<Double>) data.get("Ylim");
            MLNumericArray<Double> values = (MLNumericArray<Double>) data.get("mapVals");

            for (int i = Xlim.get(0).intValue(); i <= Xlim.get(1).intValue(); i++) {
                for (int j = Ylim.get(0).intValue(); j <= Ylim.get(1).intValue(); j++) {
                    double val = values.getReal(i - Xlim.get(0).intValue(), j - Ylim.get(0).intValue()).floatValue();
                    String coord = i + "_" + j + "_" + 0;
                    if (!correctionFactors.containsKey(coord)) {
                        correctionFactors.put(coord, new HashMap<String, Double>());
                    }
                    correctionFactors.get(coord).put(metabName, val);
                }
            }
            return true;
        }
        return false;
    }

    public double correctValue(double val, String coord, String metab) {
        double res = Double.NaN;
        double fact = getCorrectionFactor(coord, metab);
        if (!Double.isNaN(fact)) {
            return val / fact;
        }
        return res;
    }

    public double getCorrectionFactor(String coord, String metab) {
        double res = Double.NaN;
        metab = translateMetaboliteName(metab);
        if (correctionFactors.containsKey(coord) && correctionFactors.get(coord).containsKey(metab)) {
            return correctionFactors.get(coord).get(metab);
        }
        return res;
    }

    private String translateMetaboliteName(String original) {
        original = original.toLowerCase();
        if (original.contains("cho") || original.contains("ala") || original.contains("gly") || original.contains("myo")) {
            original = "Cho";
        } else if (original.contains("lip") || original.contains("lac") || original.contains("1.3") || original.contains("0.9")) {
            original = "Lac";
        } else if (original.contains("cre")) {
            original = "Cr";
        } else if (original.contains("naa") || original.contains("glu")) {
            original = "NAA";
        }
        return original;
    }

    public boolean checkMetaboliteName(String name) {
        if (translateMetaboliteName(name).matches("Cho")
                || translateMetaboliteName(name).matches("Lac")
                || translateMetaboliteName(name).matches("Cr")
                || translateMetaboliteName(name).matches("NAA")) {
            return true;
        }
        return false;
    }
}
