/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.settings;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.DefaultCellEditor;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import org.spectrim.main.ClinicalViewer;
import org.spectrim.model.GlobalModel;
import org.spectrim.settings.processing.PreprocessingAction;
import org.spectrim.settings.processing.PreprocessingActionUI;
import org.spectrim.settings.processing.ProcessingAction;
import org.spectrim.settings.processing.QuantificationAction;
import org.spectrim.settings.processing.QuantificationActionUI;
import org.spectrim.settings.processing.QuantificationTypeSelectionFrame;
import org.spectrim.settings.processing.ResultsAction;
import org.spectrim.settings.processing.ResultsActionUI;
import org.spectrim.settings.processing.SeriesFilter;
import org.spectrim.settings.processing.SeriesFilterUI;

/**
 * This class handles all settable properties of the application.
 * Adapted by JS 11.2018.
 * @author student
 */
public class SettingsGUI extends javax.swing.JFrame implements ActionListener 
{

    private static SettingsGUI theInstance = null;
    // report
    private String prstPath;
    private String imgFolderPath;
    // global
    private String rExe;
    private String CSIGrdPrstFolder;
    private double signalFreeFDMin;
    private double signalFreeFDMax;
    // registration
    private String f3dInstallPath;
    private String aladinInstallPath;
    private String regTransformInstallPath;
    private String dicom2niiInstallPath;
    private String registrationOutputFolder;
    private String roiRModel;
    private String coregistrationLibrary;

    public SettingsGUIListener listener;

    //Processing Actions
    private ArrayList<SeriesFilter> seriesFilters = new ArrayList<>();
    private ArrayList<PreprocessingAction> preprocessingActions = new ArrayList<>();
    private ArrayList<QuantificationAction> quantificationActions = new ArrayList<>();
    private ArrayList<ResultsAction> resultsActions = new ArrayList<>();
    private ArrayList<ProcessingAction> processingActions = new ArrayList<>();

    DefaultListModel seriesFiltersListModel = new DefaultListModel();
    DefaultListModel preprocessingActionsListModel = new DefaultListModel();
    DefaultListModel quantificationActionsListModel = new DefaultListModel();
    DefaultListModel resultsActionsListModel = new DefaultListModel();

    JComboBox comboSeriesFilter = new JComboBox(new String[]{"Select Series Filter"});
    JComboBox comboPreprocessingAction = new JComboBox(new String[]{"Select Pre-processing Action"});
    JComboBox comboQuantificationAction = new JComboBox(new String[]{"Select Quantification Action"});
    JComboBox comboResultsAction = new JComboBox(new String[]{"Select Results Action"});

    private DefaultTableModel tableProcessingPipelinesModel;

    String oldname;                 // Needed to update processing actions table
    QuantificationTypeSelectionFrame quantSelFrame;
    GlobalModel model;
    /**
     * Creates new form GlobalSettingsInterface.
     */
    private SettingsGUI( SettingsGUIListener listener ) 
    {
        initVariables( );
        initComponents( );
        this.setVisible( true );
        this.jTabbedPane1.remove( 3 );
        this.jFormattedTextFieldSignalFreeFDMin.setValue( signalFreeFDMin );
        this.jFormattedTextFieldSignalFreeFDMax.setValue( signalFreeFDMax );
        this.listener = listener;
        model = ( ( ClinicalViewer)listener).getModel( );
        this.jListSelectionFilters.setModel( seriesFiltersListModel );
        this.jListPreProcessingActions.setModel( preprocessingActionsListModel );
        this.jListQuantificationActions.setModel( quantificationActionsListModel );
        this.jListResultsActions.setModel( resultsActionsListModel );

        tableProcessingPipelinesModel = (DefaultTableModel) this.jTableProcessingPipelines.getModel( );
        this.jTableProcessingPipelines.getColumnModel( ).getColumn( 1 ).setCellEditor( new DefaultCellEditor( comboSeriesFilter ) );
        this.jTableProcessingPipelines.getColumnModel( ).getColumn( 2 ).setCellEditor( new DefaultCellEditor( comboPreprocessingAction ) );
        this.jTableProcessingPipelines.getColumnModel( ).getColumn( 3 ).setCellEditor( new DefaultCellEditor( comboQuantificationAction ) );
        this.jTableProcessingPipelines.getColumnModel( ).getColumn( 4 ).setCellEditor( new DefaultCellEditor( comboResultsAction));
        updateColumnsProcessingPipelinesTable();
        this.jTableProcessingPipelines.getSelectionModel( ).addListSelectionListener( new ListSelectionListener( ) 
        {
            @Override
            public void valueChanged( ListSelectionEvent e ) 
            {
                jButtonDeleteProcessingPipeline.setEnabled( !jTableProcessingPipelines.getSelectionModel().isSelectionEmpty( ) );
            }
        });
        
        loadProcessingPipeline( );
        
        getRootPane( ).setDefaultButton( this.jButtonOk );
                  
        Action applyAndExit = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e ) 
            {
                applyChanges( );
                ((SettingsGUI)e.getSource()).listener.updateProcessingActions( );
                dispose( );
            }
        };

        Action exitNoUpdate = new AbstractAction( ) 
        {
            @Override
            public void actionPerformed( ActionEvent e ) 
            {
               dispose( );
            }
        };


        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( "ENTER" ),
                                           "applyAndExit");
        getRootPane( ).getActionMap( ).put( "loadAndExit",
                                             applyAndExit  );

        getRootPane( ).getInputMap( ).put( KeyStroke.getKeyStroke( "ESCAPE" ),
                                           "Exit without loading" );
        getRootPane( ).getActionMap( ).put( "Exit without loading",
                                             exitNoUpdate );
        
    }

    public static SettingsGUI getInstance( SettingsGUIListener listener ) 
    {
        if (theInstance == null) 
        {
            theInstance = new SettingsGUI( listener );
        }
        theInstance.setVisible( true );
        return theInstance;
    }

    private void initVariables( ) 
    {
        // Report
        prstPath = ReportSettings.getInstance( ).getPrstsPath( ).toString( );
        imgFolderPath = ReportSettings.getInstance ().getImgOutputFolder( ).toString( );
        // Global Settings
        CSIGrdPrstFolder = GlobalSettings.getInstance( ).getCSIGrdPrstFolder( );
        rExe = GlobalSettings.getInstance().getRExecutable();
        signalFreeFDMin = GlobalSettings.getInstance().getSignalFreeFDMin();
        signalFreeFDMax = GlobalSettings.getInstance().getSignalFreeFDMax();
        // registration
        f3dInstallPath = RegistrationSettings.getInstance().getF3dInstallPath();
        aladinInstallPath = RegistrationSettings.getInstance().getAladinInstallPath();
        regTransformInstallPath = RegistrationSettings.getInstance().getRegTransformInstallPath();
        dicom2niiInstallPath = RegistrationSettings.getInstance().getDicom2niiInstallPath();
        registrationOutputFolder = RegistrationSettings.getInstance().getOutputFolder();
        roiRModel = RegistrationSettings.getInstance().getRoiRModel();
        coregistrationLibrary = RegistrationSettings.getInstance().getCoregistrationLibrary();
    }

    private void loadProcessingPipeline( ) 
    {
        for (SeriesFilter filter : ProcessingSettings.getInstance().getSeriesFilters()) {
            addSeriesSelectionActionUI(filter.clone(), -1);
        }
        for (PreprocessingAction preproAction : ProcessingSettings.getInstance().getPreprocessingActions()) {
            addPreprocessingActionUI(preproAction.clone(), -1);
        }
        for (QuantificationAction quantAction : ProcessingSettings.getInstance().getQuantificationActions()) {
            addQuantificationActionUI(quantAction.clone(), -1);
        }
        for (ResultsAction resAction : ProcessingSettings.getInstance().getResultsActions()) {
            addResultsActionUI(resAction.clone(), -1);
        }
        stopEdittingProcessingActionsTable();
        for (ProcessingAction proAction : ProcessingSettings.getInstance().getProcessingActions()) {
            tableProcessingPipelinesModel.addRow(new Object[]{proAction.getName(),
                proAction.getSeriesFilter() == null ? "-Select Series Filter-" : proAction.getSeriesFilter().getName(),
                proAction.getPreprocessingAction() == null ? "-Select Pre-processing Action-" : proAction.getPreprocessingAction().getName(),
                proAction.getQuantificationAction() == null ? "-Select Quantification Action-" : proAction.getQuantificationAction().getName(),
                proAction.getResultsAction() == null ? "-Select Results Action-" : proAction.getResultsAction().getName()});
        }
    }

    String selectDir(String currentDir) {
        JFileChooser jdirChooser = new JFileChooser();
        jdirChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        jdirChooser.setFileHidingEnabled(false);
        jdirChooser.setCurrentDirectory(new File(currentDir));
        jdirChooser.setDialogTitle("Select new folder");
        int returnVal = jdirChooser.showDialog(this, "Select");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            return jdirChooser.getSelectedFile().toString();
        } else {
            return currentDir;
        }
    }

    public String selectFile(String currentFile, String ext) {
        JFileChooser jFileChooser = new JFileChooser();
        jFileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        jFileChooser.setCurrentDirectory(new File(currentFile));
        if (!"".equals(ext)) {
            FileNameExtensionFilter filter = new FileNameExtensionFilter(ext + " Files", ext.toUpperCase(), ext);
            jFileChooser.setFileFilter(filter);
        }
        int returnVal = jFileChooser.showOpenDialog(this);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            return jFileChooser.getSelectedFile().toString();
        } else {
            return currentFile;
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        buttonGroupAppodisationType = new javax.swing.ButtonGroup();
        jPanelOverallContainer = new javax.swing.JPanel();
        /*UIManager.put("TabbedPane.contentAreaColor",ColorUIResource.RED);
        UIManager.put("TabbedPane.selected",ColorUIResource.RED);
        UIManager.put("TabbedPane.background",ColorUIResource.RED);
        UIManager.put("TabbedPane.shadow",ColorUIResource.RED);*/
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanelProcessingActions = new javax.swing.JPanel();
        jScrollPane3 = new javax.swing.JScrollPane();
        jListSelectionFilters = new javax.swing.JList();
        jScrollPane1 = new javax.swing.JScrollPane();
        jListPreProcessingActions = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        jListQuantificationActions = new javax.swing.JList();
        jButtonAddNewSeriesFilter = new javax.swing.JButton();
        jButtonDeleteSeriesFilter = new javax.swing.JButton();
        jButtonEditSeriesFilter = new javax.swing.JButton();
        jButtonDuplicateSeriesFilter = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jButtonDeletePreprocessingAction = new javax.swing.JButton();
        jButtonAddNewPreprocessingAction = new javax.swing.JButton();
        jButtonEditPreprocessingAction = new javax.swing.JButton();
        jButtonDuplicatePreprocessingAction = new javax.swing.JButton();
        jButtonEditQuantificationAction = new javax.swing.JButton();
        jButtonAddNewQuantificationAction = new javax.swing.JButton();
        jButtonDeleteQuantificationAction = new javax.swing.JButton();
        jButtonDuplicateQuantificationAction = new javax.swing.JButton();
        jScrollPane4 = new javax.swing.JScrollPane();
        jTableProcessingPipelines = new javax.swing.JTable();
        jSeparator2 = new javax.swing.JSeparator();
        jPanel3 = new javax.swing.JPanel();
        jButtonMoreInfo = new javax.swing.JButton();
        jButtonAddNewProcessingPipeline = new javax.swing.JButton();
        jButtonDeleteProcessingPipeline = new javax.swing.JButton();
        jLabelGnrlReportOutputFolder = new javax.swing.JLabel();
        jScrollPane5 = new javax.swing.JScrollPane();
        jListResultsActions = new javax.swing.JList();
        jLabel9 = new javax.swing.JLabel();
        jButtonAddNewResultsAction = new javax.swing.JButton();
        jButtonDeleteResultsAction = new javax.swing.JButton();
        jButtonEditResultsAction = new javax.swing.JButton();
        jButtonDuplicateResultsAction = new javax.swing.JButton();
        jPanelReports = new javax.swing.JPanel();
        jTextFieldRprtImgFolder = new javax.swing.JTextField();
        jButtonGrnlReportOutputFolder = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jTextFieldPrstFldr = new javax.swing.JTextField();
        jButton1 = new javax.swing.JButton();
        jLabel8 = new javax.swing.JLabel();
        jPanel5 = new javax.swing.JPanel();
        jPanelGeneral = new javax.swing.JPanel();
        jLabelCSIPrstFolder = new javax.swing.JLabel();
        jTextFieldCSIPrstFolder = new javax.swing.JTextField();
        jButtonCSIPrstFolder = new javax.swing.JButton();
        jLabelRExecutable = new javax.swing.JLabel();
        jTextFieldRExecutable = new javax.swing.JTextField();
        jButtonRExecutable = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        jPanel1 = new javax.swing.JPanel();
        jFormattedTextFieldSignalFreeFDMin = new javax.swing.JFormattedTextField();
        jLabel2 = new javax.swing.JLabel();
        jFormattedTextFieldSignalFreeFDMax = new javax.swing.JFormattedTextField();
        jLabel4 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jLabelRExecutable1 = new javax.swing.JLabel();
        jTextFieldRExecutable1 = new javax.swing.JTextField();
        jPanelRegistration = new javax.swing.JPanel();
        jPanelRegistrationGeneral = new javax.swing.JPanel();
        jLabelRegistrationGeneral = new javax.swing.JLabel();
        jPanelRegistrationGeneralContent = new javax.swing.JPanel();
        jPanelRegistrationOutputFolder = new javax.swing.JPanel();
        jLabelRegistrationOutputFolder = new javax.swing.JLabel();
        jPanelRegistrationOutputFolderContent = new javax.swing.JPanel();
        jTextFieldRegistrationOutputFolder = new javax.swing.JTextField();
        jButtonRegistrationOutputFolder = new javax.swing.JButton();
        jLabelRegistrationOutputFolder1 = new javax.swing.JLabel();
        jPanelCoregistrationLibrary = new javax.swing.JPanel();
        jTextFieldCoregistrationLibrary = new javax.swing.JTextField();
        jButtonCoregistrationLibrary = new javax.swing.JButton();
        jSeparator4 = new javax.swing.JSeparator();
        jPanelNiftyReg = new javax.swing.JPanel();
        jLabelNiftyRegTitle = new javax.swing.JLabel();
        jPanelNiftyRegContent = new javax.swing.JPanel();
        jPanelAladin = new javax.swing.JPanel();
        jLabelAladin = new javax.swing.JLabel();
        jPanelAladinContent = new javax.swing.JPanel();
        jTextFieldAladin = new javax.swing.JTextField();
        jButtonAladin = new javax.swing.JButton();
        jPanelf3d = new javax.swing.JPanel();
        jLabelf3d = new javax.swing.JLabel();
        jPanelf3dContent = new javax.swing.JPanel();
        jTextFieldf3d = new javax.swing.JTextField();
        jButtonf3d = new javax.swing.JButton();
        jPanelRegTrsf = new javax.swing.JPanel();
        jLabelRegTrsf = new javax.swing.JLabel();
        jPanelRegTrsfContent = new javax.swing.JPanel();
        jTextFieldRegTrsf = new javax.swing.JTextField();
        jButtonRegTrsf = new javax.swing.JButton();
        jSeparator3 = new javax.swing.JSeparator();
        jPanelmicron = new javax.swing.JPanel();
        jLabelMicron = new javax.swing.JLabel();
        jPanelMicronContent = new javax.swing.JPanel();
        jPanelDicom2Nii = new javax.swing.JPanel();
        jLabelDicom2Nii = new javax.swing.JLabel();
        jPanelDicom2NiiContent = new javax.swing.JPanel();
        jTextFieldDicom2Nii = new javax.swing.JTextField();
        jButtonDicom2Nii = new javax.swing.JButton();
        jPanelRoiReg = new javax.swing.JPanel();
        jLabelRoiRegistration = new javax.swing.JLabel();
        jPanelRoiRegContent = new javax.swing.JPanel();
        jPanelRoiModel = new javax.swing.JPanel();
        jLabelRoiModel = new javax.swing.JLabel();
        jPanelRoiModelContent = new javax.swing.JPanel();
        jTextFieldRoiModel = new javax.swing.JTextField();
        jButtonRoiModel = new javax.swing.JButton();
        jPanelMenu = new javax.swing.JPanel();
        jButtonOk = new javax.swing.JButton();
        jButtonApply = new javax.swing.JButton();
        jButtonCancel = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("SpectrIm - Settings");
        setBackground(java.awt.Color.white);

        jPanelOverallContainer.setBackground(java.awt.Color.white);
        jPanelOverallContainer.setLayout(new java.awt.GridLayout(1, 0));

        jTabbedPane1.setBackground(java.awt.Color.white);
        jTabbedPane1.setTabLayoutPolicy(javax.swing.JTabbedPane.SCROLL_TAB_LAYOUT);
        jTabbedPane1.setTabPlacement(javax.swing.JTabbedPane.LEFT);

        jPanelProcessingActions.setLayout(new java.awt.GridBagLayout());

        jListSelectionFilters.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListSelectionFilters.setToolTipText("<html>Series Filters can be used to select the spectroscopy<br>series for which the defined Processing Actions apply</html>");
        jScrollPane3.setViewportView(jListSelectionFilters);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 5, 0);
        jPanelProcessingActions.add(jScrollPane3, gridBagConstraints);

        jListPreProcessingActions.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListPreProcessingActions.setToolTipText("<html>Pre-processing Actions allow defining several pre-processing steps<br>\n(e.g. Apodization, Auto-phasing, etc.) that can be included in a Processing Action</html>");
        jScrollPane1.setViewportView(jListPreProcessingActions);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 5, 0);
        jPanelProcessingActions.add(jScrollPane1, gridBagConstraints);

        jListQuantificationActions.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListQuantificationActions.setToolTipText("<html>A Quantification Action allows the user to set up the quantification step by defining the <br>\nparameters of the quantification (.op file) and the source of the metabolite model file</html>");
        jScrollPane2.setViewportView(jListQuantificationActions);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 5, 0);
        jPanelProcessingActions.add(jScrollPane2, gridBagConstraints);

        jButtonAddNewSeriesFilter.setText("Add New");
        jButtonAddNewSeriesFilter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddNewSeriesFilterActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonAddNewSeriesFilter, gridBagConstraints);

        jButtonDeleteSeriesFilter.setText("Delete");
        jButtonDeleteSeriesFilter.setEnabled(false);
        jButtonDeleteSeriesFilter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDeleteSeriesFilterActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDeleteSeriesFilter, gridBagConstraints);

        jButtonEditSeriesFilter.setText("Edit");
        jButtonEditSeriesFilter.setEnabled(false);
        jButtonEditSeriesFilter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonEditSeriesFilterActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonEditSeriesFilter, gridBagConstraints);

        jButtonDuplicateSeriesFilter.setText("Duplicate");
        jButtonDuplicateSeriesFilter.setEnabled(false);
        jButtonDuplicateSeriesFilter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDuplicateSeriesFilterActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDuplicateSeriesFilter, gridBagConstraints);

        jLabel5.setText("Series Filters:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jLabel5, gridBagConstraints);

        jLabel6.setText("Pre-processing Actions:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 0);
        jPanelProcessingActions.add(jLabel6, gridBagConstraints);

        jLabel7.setText("Quantification Actions:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 0);
        jPanelProcessingActions.add(jLabel7, gridBagConstraints);

        jButtonDeletePreprocessingAction.setText("Delete");
        jButtonDeletePreprocessingAction.setEnabled(false);
        jButtonDeletePreprocessingAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDeletePreprocessingActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDeletePreprocessingAction, gridBagConstraints);

        jButtonAddNewPreprocessingAction.setText("Add New");
        jButtonAddNewPreprocessingAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddNewPreprocessingActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonAddNewPreprocessingAction, gridBagConstraints);

        jButtonEditPreprocessingAction.setText("Edit");
        jButtonEditPreprocessingAction.setEnabled(false);
        jButtonEditPreprocessingAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonEditPreprocessingActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonEditPreprocessingAction, gridBagConstraints);

        jButtonDuplicatePreprocessingAction.setText("Duplicate");
        jButtonDuplicatePreprocessingAction.setEnabled(false);
        jButtonDuplicatePreprocessingAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDuplicatePreprocessingActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDuplicatePreprocessingAction, gridBagConstraints);

        jButtonEditQuantificationAction.setText("Edit");
        jButtonEditQuantificationAction.setEnabled(false);
        jButtonEditQuantificationAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonEditQuantificationActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonEditQuantificationAction, gridBagConstraints);

        jButtonAddNewQuantificationAction.setText("Add New");
        jButtonAddNewQuantificationAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddNewQuantificationActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonAddNewQuantificationAction, gridBagConstraints);

        jButtonDeleteQuantificationAction.setText("Delete");
        jButtonDeleteQuantificationAction.setEnabled(false);
        jButtonDeleteQuantificationAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDeleteQuantificationActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDeleteQuantificationAction, gridBagConstraints);

        jButtonDuplicateQuantificationAction.setText("Duplicate");
        jButtonDuplicateQuantificationAction.setEnabled(false);
        jButtonDuplicateQuantificationAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDuplicateQuantificationActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 7;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDuplicateQuantificationAction, gridBagConstraints);

        jTableProcessingPipelines.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Name", "Series Filter", "Pre-processing Action", "Quantification Action", "Results Action"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        jTableProcessingPipelines.setToolTipText("");
        jTableProcessingPipelines.setColumnSelectionAllowed(true);
        jTableProcessingPipelines.getTableHeader().setReorderingAllowed(false);
        jScrollPane4.setViewportView(jTableProcessingPipelines);
        jTableProcessingPipelines.getColumnModel().getSelectionModel().setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.gridwidth = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelProcessingActions.add(jScrollPane4, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelProcessingActions.add(jSeparator2, gridBagConstraints);

        jPanel3.setLayout(new java.awt.GridBagLayout());

        jButtonMoreInfo.setForeground(new java.awt.Color(0, 0, 204));
        jButtonMoreInfo.setText("More Info...");
        jButtonMoreInfo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonMoreInfoActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        jPanel3.add(jButtonMoreInfo, gridBagConstraints);

        jButtonAddNewProcessingPipeline.setText("Add New Processing Pipeline");
        jButtonAddNewProcessingPipeline.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddNewProcessingPipelineActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_END;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel3.add(jButtonAddNewProcessingPipeline, gridBagConstraints);

        jButtonDeleteProcessingPipeline.setText("Delete Selected Processing Pipeline(s)");
        jButtonDeleteProcessingPipeline.setEnabled(false);
        jButtonDeleteProcessingPipeline.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDeleteProcessingPipelineActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_END;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel3.add(jButtonDeleteProcessingPipeline, gridBagConstraints);

        jLabelGnrlReportOutputFolder.setText("Default Image Output Folder: ");
        jPanel3.add(jLabelGnrlReportOutputFolder, new java.awt.GridBagConstraints());

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 11;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_END;
        jPanelProcessingActions.add(jPanel3, gridBagConstraints);

        jListResultsActions.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListResultsActions.setToolTipText("<html>A Quantification Action allows the user to set up the quantification step by defining the <br>\nparameters of the quantification (.op file) and the source of the metabolite model file</html>");
        jScrollPane5.setViewportView(jListResultsActions);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 5, 0);
        jPanelProcessingActions.add(jScrollPane5, gridBagConstraints);

        jLabel9.setText("Results Actions:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 9;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(0, 8, 0, 0);
        jPanelProcessingActions.add(jLabel9, gridBagConstraints);

        jButtonAddNewResultsAction.setText("Add New");
        jButtonAddNewResultsAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddNewResultsActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 10;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonAddNewResultsAction, gridBagConstraints);

        jButtonDeleteResultsAction.setText("Delete");
        jButtonDeleteResultsAction.setEnabled(false);
        jButtonDeleteResultsAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDeleteResultsActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 10;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDeleteResultsAction, gridBagConstraints);

        jButtonEditResultsAction.setText("Edit");
        jButtonEditResultsAction.setEnabled(false);
        jButtonEditResultsAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonEditResultsActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 10;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonEditResultsAction, gridBagConstraints);

        jButtonDuplicateResultsAction.setText("Duplicate");
        jButtonDuplicateResultsAction.setEnabled(false);
        jButtonDuplicateResultsAction.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDuplicateResultsActionActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 10;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        jPanelProcessingActions.add(jButtonDuplicateResultsAction, gridBagConstraints);

        jTabbedPane1.addTab("Processing Pipelines", jPanelProcessingActions);

        jPanelReports.setLayout(new java.awt.GridBagLayout());

        jTextFieldRprtImgFolder.setText(imgFolderPath);
        jTextFieldRprtImgFolder.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldRprtImgFolder.setMinimumSize(new java.awt.Dimension(200, 20));
        jTextFieldRprtImgFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldRprtImgFolderActionPerformed(evt);
            }
        });
        jTextFieldRprtImgFolder.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldRprtImgFolderFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelReports.add(jTextFieldRprtImgFolder, gridBagConstraints);

        jButtonGrnlReportOutputFolder.setText("Change...");
        jButtonGrnlReportOutputFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonGrnlReportOutputFolderActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 5);
        jPanelReports.add(jButtonGrnlReportOutputFolder, gridBagConstraints);

        jLabel1.setText("Default Report Images Folder: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelReports.add(jLabel1, gridBagConstraints);

        jTextFieldPrstFldr.setText(prstPath);
        jTextFieldPrstFldr.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldPrstFldr.setMinimumSize(new java.awt.Dimension(200, 20));
        jTextFieldPrstFldr.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldPrstFldrActionPerformed(evt);
            }
        });
        jTextFieldPrstFldr.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldPrstFldrFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelReports.add(jTextFieldPrstFldr, gridBagConstraints);

        jButton1.setText("Change...");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 5);
        jPanelReports.add(jButton1, gridBagConstraints);

        jLabel8.setText("Default Preset Folder: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelReports.add(jLabel8, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanelReports.add(jPanel5, gridBagConstraints);

        jTabbedPane1.addTab("Reports", jPanelReports);

        jPanelGeneral.setLayout(new java.awt.GridBagLayout());

        jLabelCSIPrstFolder.setText("Grid Selection Preset Folder: ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelGeneral.add(jLabelCSIPrstFolder, gridBagConstraints);

        jTextFieldCSIPrstFolder.setEditable(false);
        jTextFieldCSIPrstFolder.setText(CSIGrdPrstFolder);
        jTextFieldCSIPrstFolder.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldCSIPrstFolder.setPreferredSize(new java.awt.Dimension(50, 20));
        jTextFieldCSIPrstFolder.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldCSIPrstFolderFocusLost(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelGeneral.add(jTextFieldCSIPrstFolder, gridBagConstraints);

        jButtonCSIPrstFolder.setText("Change...");
        jButtonCSIPrstFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCSIPrstFolderActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 5);
        jPanelGeneral.add(jButtonCSIPrstFolder, gridBagConstraints);

        jLabelRExecutable.setText("R Executable:   ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelGeneral.add(jLabelRExecutable, gridBagConstraints);

        jTextFieldRExecutable.setEditable(false);
        jTextFieldRExecutable.setText(rExe);
        jTextFieldRExecutable.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldRExecutable.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelGeneral.add(jTextFieldRExecutable, gridBagConstraints);

        jButtonRExecutable.setText("Change...");
        jButtonRExecutable.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRExecutableActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 5);
        jPanelGeneral.add(jButtonRExecutable, gridBagConstraints);

        jLabel3.setText("FD Signal Free Region [min, max]:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelGeneral.add(jLabel3, gridBagConstraints);

        jPanel1.setLayout(new java.awt.GridBagLayout());

        jFormattedTextFieldSignalFreeFDMin.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter()));
        jFormattedTextFieldSignalFreeFDMin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jFormattedTextFieldSignalFreeFDMinActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel1.add(jFormattedTextFieldSignalFreeFDMin, gridBagConstraints);

        jLabel2.setText("ppm");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(0, 0, 0, 15);
        jPanel1.add(jLabel2, gridBagConstraints);

        jFormattedTextFieldSignalFreeFDMax.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter()));
        jFormattedTextFieldSignalFreeFDMax.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jFormattedTextFieldSignalFreeFDMaxActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        jPanel1.add(jFormattedTextFieldSignalFreeFDMax, gridBagConstraints);

        jLabel4.setText("ppm");
        jPanel1.add(jLabel4, new java.awt.GridBagConstraints());

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 5);
        jPanelGeneral.add(jPanel1, gridBagConstraints);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_START;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelGeneral.add(jPanel2, gridBagConstraints);

        jLabelRExecutable1.setText("R Executable:   ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.LINE_START;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 0, 0);
        jPanelGeneral.add(jLabelRExecutable1, gridBagConstraints);

        jTextFieldRExecutable1.setEditable(false);
        jTextFieldRExecutable1.setText(rExe);
        jTextFieldRExecutable1.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldRExecutable1.setPreferredSize(new java.awt.Dimension(50, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelGeneral.add(jTextFieldRExecutable1, gridBagConstraints);

        jTabbedPane1.addTab("Other", jPanelGeneral);

        jPanelRegistration.setLayout(new javax.swing.BoxLayout(jPanelRegistration, javax.swing.BoxLayout.PAGE_AXIS));

        jPanelRegistrationGeneral.setLayout(new java.awt.BorderLayout());

        jLabelRegistrationGeneral.setFont(new java.awt.Font("Ubuntu", 1, 15)); // NOI18N
        jLabelRegistrationGeneral.setText("General");
        jPanelRegistrationGeneral.add(jLabelRegistrationGeneral, java.awt.BorderLayout.NORTH);

        jPanelRegistrationGeneralContent.setLayout(new java.awt.GridLayout(0, 1));

        jPanelRegistrationOutputFolder.setLayout(new java.awt.GridLayout(0, 2));

        jLabelRegistrationOutputFolder.setText("Registration Output Folder: ");
        jPanelRegistrationOutputFolder.add(jLabelRegistrationOutputFolder);

        jPanelRegistrationOutputFolderContent.setLayout(new javax.swing.BoxLayout(jPanelRegistrationOutputFolderContent, javax.swing.BoxLayout.LINE_AXIS));

        jTextFieldRegistrationOutputFolder.setText(registrationOutputFolder);
        jTextFieldRegistrationOutputFolder.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldRegistrationOutputFolder.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldRegistrationOutputFolderFocusLost(evt);
            }
        });
        jPanelRegistrationOutputFolderContent.add(jTextFieldRegistrationOutputFolder);

        jButtonRegistrationOutputFolder.setText("Change...");
        jButtonRegistrationOutputFolder.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRegistrationOutputFolderActionPerformed(evt);
            }
        });
        jPanelRegistrationOutputFolderContent.add(jButtonRegistrationOutputFolder);

        jPanelRegistrationOutputFolder.add(jPanelRegistrationOutputFolderContent);

        jLabelRegistrationOutputFolder1.setText("Coregistration .so/.dll: ");
        jPanelRegistrationOutputFolder.add(jLabelRegistrationOutputFolder1);

        jPanelCoregistrationLibrary.setLayout(new javax.swing.BoxLayout(jPanelCoregistrationLibrary, javax.swing.BoxLayout.LINE_AXIS));

        jTextFieldCoregistrationLibrary.setText(coregistrationLibrary);
        jTextFieldCoregistrationLibrary.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldCoregistrationLibrary.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldCoregistrationLibraryActionPerformed(evt);
            }
        });
        jTextFieldCoregistrationLibrary.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldCoregistrationLibraryFocusLost(evt);
            }
        });
        jPanelCoregistrationLibrary.add(jTextFieldCoregistrationLibrary);

        jButtonCoregistrationLibrary.setText("Change...");
        jButtonCoregistrationLibrary.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCoregistrationLibraryActionPerformed(evt);
            }
        });
        jPanelCoregistrationLibrary.add(jButtonCoregistrationLibrary);

        jPanelRegistrationOutputFolder.add(jPanelCoregistrationLibrary);

        jPanelRegistrationGeneralContent.add(jPanelRegistrationOutputFolder);

        jPanelRegistrationGeneral.add(jPanelRegistrationGeneralContent, java.awt.BorderLayout.CENTER);

        jPanelRegistration.add(jPanelRegistrationGeneral);
        jPanelRegistration.add(jSeparator4);

        jPanelNiftyReg.setLayout(new java.awt.BorderLayout());

        jLabelNiftyRegTitle.setFont(new java.awt.Font("Ubuntu", 1, 15)); // NOI18N
        jLabelNiftyRegTitle.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        jLabelNiftyRegTitle.setText("Nifty Reg");
        jPanelNiftyReg.add(jLabelNiftyRegTitle, java.awt.BorderLayout.NORTH);

        jPanelNiftyRegContent.setLayout(new java.awt.GridLayout(0, 1));

        jPanelAladin.setLayout(new java.awt.GridLayout(1, 0));

        jLabelAladin.setText("Aladin Installation Path: ");
        jPanelAladin.add(jLabelAladin);

        jPanelAladinContent.setLayout(new javax.swing.BoxLayout(jPanelAladinContent, javax.swing.BoxLayout.LINE_AXIS));

        jTextFieldAladin.setText(aladinInstallPath);
        jTextFieldAladin.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldAladin.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldAladinFocusLost(evt);
            }
        });
        jPanelAladinContent.add(jTextFieldAladin);

        jButtonAladin.setText("Change...");
        jButtonAladin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAladinActionPerformed(evt);
            }
        });
        jPanelAladinContent.add(jButtonAladin);

        jPanelAladin.add(jPanelAladinContent);

        jPanelNiftyRegContent.add(jPanelAladin);

        jPanelf3d.setLayout(new java.awt.GridLayout(1, 0));

        jLabelf3d.setText("F3D Installation Path: ");
        jPanelf3d.add(jLabelf3d);

        jPanelf3dContent.setLayout(new javax.swing.BoxLayout(jPanelf3dContent, javax.swing.BoxLayout.LINE_AXIS));

        jTextFieldf3d.setText(f3dInstallPath);
        jTextFieldf3d.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldf3d.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldf3dFocusLost(evt);
            }
        });
        jPanelf3dContent.add(jTextFieldf3d);

        jButtonf3d.setText("Change...");
        jButtonf3d.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonf3dActionPerformed(evt);
            }
        });
        jPanelf3dContent.add(jButtonf3d);

        jPanelf3d.add(jPanelf3dContent);

        jPanelNiftyRegContent.add(jPanelf3d);

        jPanelRegTrsf.setLayout(new java.awt.GridLayout(1, 0));

        jLabelRegTrsf.setText("Resample Installation Path: ");
        jPanelRegTrsf.add(jLabelRegTrsf);

        jPanelRegTrsfContent.setLayout(new javax.swing.BoxLayout(jPanelRegTrsfContent, javax.swing.BoxLayout.LINE_AXIS));

        jTextFieldRegTrsf.setText(regTransformInstallPath);
        jTextFieldRegTrsf.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldRegTrsf.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldRegTrsfFocusLost(evt);
            }
        });
        jPanelRegTrsfContent.add(jTextFieldRegTrsf);

        jButtonRegTrsf.setText("Change...");
        jButtonRegTrsf.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRegTrsfActionPerformed(evt);
            }
        });
        jPanelRegTrsfContent.add(jButtonRegTrsf);

        jPanelRegTrsf.add(jPanelRegTrsfContent);

        jPanelNiftyRegContent.add(jPanelRegTrsf);

        jPanelNiftyReg.add(jPanelNiftyRegContent, java.awt.BorderLayout.CENTER);

        jPanelRegistration.add(jPanelNiftyReg);
        jPanelRegistration.add(jSeparator3);

        jPanelmicron.setLayout(new java.awt.BorderLayout());

        jLabelMicron.setFont(new java.awt.Font("Ubuntu", 1, 15)); // NOI18N
        jLabelMicron.setText("Micron");
        jPanelmicron.add(jLabelMicron, java.awt.BorderLayout.NORTH);

        jPanelMicronContent.setLayout(new java.awt.GridLayout(0, 1));

        jPanelDicom2Nii.setLayout(new java.awt.GridLayout(1, 0));

        jLabelDicom2Nii.setText("Dicom2Nii Installation Path: ");
        jPanelDicom2Nii.add(jLabelDicom2Nii);

        jPanelDicom2NiiContent.setLayout(new javax.swing.BoxLayout(jPanelDicom2NiiContent, javax.swing.BoxLayout.LINE_AXIS));

        jTextFieldDicom2Nii.setText(dicom2niiInstallPath);
        jTextFieldDicom2Nii.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldDicom2Nii.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldDicom2NiiFocusLost(evt);
            }
        });
        jPanelDicom2NiiContent.add(jTextFieldDicom2Nii);

        jButtonDicom2Nii.setText("Change...");
        jButtonDicom2Nii.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonDicom2NiiActionPerformed(evt);
            }
        });
        jPanelDicom2NiiContent.add(jButtonDicom2Nii);

        jPanelDicom2Nii.add(jPanelDicom2NiiContent);

        jPanelMicronContent.add(jPanelDicom2Nii);

        jPanelmicron.add(jPanelMicronContent, java.awt.BorderLayout.CENTER);

        jPanelRegistration.add(jPanelmicron);

        jPanelRoiReg.setLayout(new java.awt.BorderLayout());

        jLabelRoiRegistration.setFont(new java.awt.Font("Ubuntu", 1, 15)); // NOI18N
        jLabelRoiRegistration.setText("RoI Registration");
        jPanelRoiReg.add(jLabelRoiRegistration, java.awt.BorderLayout.NORTH);

        jPanelRoiRegContent.setLayout(new java.awt.GridLayout(0, 1));

        jPanelRoiModel.setLayout(new java.awt.GridLayout(1, 0));

        jLabelRoiModel.setText("Roi R model file: ");
        jPanelRoiModel.add(jLabelRoiModel);

        jPanelRoiModelContent.setLayout(new javax.swing.BoxLayout(jPanelRoiModelContent, javax.swing.BoxLayout.LINE_AXIS));

        jTextFieldRoiModel.setText(roiRModel);
        jTextFieldRoiModel.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jTextFieldRoiModel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldRoiModelActionPerformed(evt);
            }
        });
        jTextFieldRoiModel.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldRoiModelFocusLost(evt);
            }
        });
        jPanelRoiModelContent.add(jTextFieldRoiModel);

        jButtonRoiModel.setText("Change...");
        jButtonRoiModel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRoiModelActionPerformed(evt);
            }
        });
        jPanelRoiModelContent.add(jButtonRoiModel);

        jPanelRoiModel.add(jPanelRoiModelContent);

        jPanelRoiRegContent.add(jPanelRoiModel);

        jPanelRoiReg.add(jPanelRoiRegContent, java.awt.BorderLayout.CENTER);

        jPanelRegistration.add(jPanelRoiReg);

        jTabbedPane1.addTab("Registration", jPanelRegistration);

        jPanelOverallContainer.add(jTabbedPane1);
        jTabbedPane1.getAccessibleContext().setAccessibleDescription("");
        //jTabbedPane1.setBackgroundAt(0, Color.WHITE);

        getContentPane().add(jPanelOverallContainer, java.awt.BorderLayout.CENTER);

        jPanelMenu.setBackground(java.awt.Color.white);
        jPanelMenu.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelMenu.setMaximumSize(new java.awt.Dimension(2147483647, 30));
        jPanelMenu.setPreferredSize(new java.awt.Dimension(500, 30));
        jPanelMenu.setLayout(new java.awt.GridBagLayout());

        jButtonOk.setText("Ok");
        jButtonOk.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jButtonOk.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonOkActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_END;
        gridBagConstraints.weightx = 1.0;
        jPanelMenu.add(jButtonOk, gridBagConstraints);

        jButtonApply.setText("Apply");
        jButtonApply.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jButtonApply.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonApplyActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_END;
        jPanelMenu.add(jButtonApply, gridBagConstraints);

        jButtonCancel.setText("Cancel");
        jButtonCancel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCancelActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.FIRST_LINE_END;
        jPanelMenu.add(jButtonCancel, gridBagConstraints);

        getContentPane().add(jPanelMenu, java.awt.BorderLayout.PAGE_END);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonOkActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonOkActionPerformed
        applyChanges();
        listener.updateProcessingActions();
        this.dispose();
    }//GEN-LAST:event_jButtonOkActionPerformed

    private void jButtonCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonCancelActionPerformed
        theInstance = null;
        this.dispose();
    }//GEN-LAST:event_jButtonCancelActionPerformed

    private void jButtonApplyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonApplyActionPerformed
        applyChanges();
        listener.updateProcessingActions();
    }//GEN-LAST:event_jButtonApplyActionPerformed

    private void jButtonRoiModelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRoiModelActionPerformed
        roiRModel = selectFile(roiRModel, "RData");
        jTextFieldRoiModel.setText(roiRModel);
    }//GEN-LAST:event_jButtonRoiModelActionPerformed

    private void jTextFieldRoiModelFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldRoiModelFocusLost
        roiRModel = jTextFieldRoiModel.getText();
    }//GEN-LAST:event_jTextFieldRoiModelFocusLost

    private void jTextFieldRoiModelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldRoiModelActionPerformed
        roiRModel = jTextFieldRoiModel.getText();
    }//GEN-LAST:event_jTextFieldRoiModelActionPerformed

    private void jButtonDicom2NiiActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDicom2NiiActionPerformed
        if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")) {
            dicom2niiInstallPath = selectFile(dicom2niiInstallPath, "exe");
        } else {
            dicom2niiInstallPath = selectFile(dicom2niiInstallPath, "");
        }
        jTextFieldDicom2Nii.setText(dicom2niiInstallPath);
    }//GEN-LAST:event_jButtonDicom2NiiActionPerformed

    private void jTextFieldDicom2NiiFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldDicom2NiiFocusLost
        dicom2niiInstallPath = jTextFieldDicom2Nii.getText();
    }//GEN-LAST:event_jTextFieldDicom2NiiFocusLost

    private void jButtonRegTrsfActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRegTrsfActionPerformed
        if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")) {
            regTransformInstallPath = selectFile(regTransformInstallPath, "exe");
        } else {
            regTransformInstallPath = selectFile(regTransformInstallPath, "");
        }
        jTextFieldRegTrsf.setText(regTransformInstallPath);
    }//GEN-LAST:event_jButtonRegTrsfActionPerformed

    private void jTextFieldRegTrsfFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldRegTrsfFocusLost
        regTransformInstallPath = jTextFieldRegTrsf.getText();
    }//GEN-LAST:event_jTextFieldRegTrsfFocusLost

    private void jButtonf3dActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonf3dActionPerformed
        if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")) {
            f3dInstallPath = selectFile(f3dInstallPath, "exe");
        } else {
            f3dInstallPath = selectFile(f3dInstallPath, "");
        }
        jTextFieldf3d.setText(f3dInstallPath);
    }//GEN-LAST:event_jButtonf3dActionPerformed

    private void jTextFieldf3dFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldf3dFocusLost
        f3dInstallPath = jTextFieldf3d.getText();
    }//GEN-LAST:event_jTextFieldf3dFocusLost

    private void jButtonAladinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAladinActionPerformed
        if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")) {
            aladinInstallPath = selectFile(aladinInstallPath, "exe");
        } else {
            aladinInstallPath = selectFile(aladinInstallPath, "");
        }
        jTextFieldAladin.setText(aladinInstallPath);
    }//GEN-LAST:event_jButtonAladinActionPerformed

    private void jTextFieldAladinFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldAladinFocusLost
        aladinInstallPath = jTextFieldAladin.getText();
    }//GEN-LAST:event_jTextFieldAladinFocusLost

    private void jButtonCoregistrationLibraryActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonCoregistrationLibraryActionPerformed
        if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")) {
            coregistrationLibrary = selectFile(coregistrationLibrary, "dll");
        } else {
            coregistrationLibrary = selectFile(coregistrationLibrary, "so");
        }
        jTextFieldCoregistrationLibrary.setText(coregistrationLibrary);
    }//GEN-LAST:event_jButtonCoregistrationLibraryActionPerformed

    private void jTextFieldCoregistrationLibraryFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldCoregistrationLibraryFocusLost
        coregistrationLibrary = jTextFieldCoregistrationLibrary.getText();
    }//GEN-LAST:event_jTextFieldCoregistrationLibraryFocusLost

    private void jTextFieldCoregistrationLibraryActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldCoregistrationLibraryActionPerformed
        coregistrationLibrary = jTextFieldCoregistrationLibrary.getText();
    }//GEN-LAST:event_jTextFieldCoregistrationLibraryActionPerformed

    private void jButtonRegistrationOutputFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRegistrationOutputFolderActionPerformed
        registrationOutputFolder = selectDir(registrationOutputFolder);
        jTextFieldRegistrationOutputFolder.setText(registrationOutputFolder);
    }//GEN-LAST:event_jButtonRegistrationOutputFolderActionPerformed

    private void jTextFieldRegistrationOutputFolderFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldRegistrationOutputFolderFocusLost
        registrationOutputFolder = jTextFieldRegistrationOutputFolder.getText();
    }//GEN-LAST:event_jTextFieldRegistrationOutputFolderFocusLost

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        prstPath = selectDir(prstPath);
        jTextFieldPrstFldr.setText(prstPath);
    }//GEN-LAST:event_jButton1ActionPerformed

    private void jTextFieldPrstFldrFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldPrstFldrFocusLost
        prstPath = jTextFieldPrstFldr.getText();
    }//GEN-LAST:event_jTextFieldPrstFldrFocusLost

    private void jTextFieldPrstFldrActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldPrstFldrActionPerformed
        prstPath = jTextFieldPrstFldr.getText();
    }//GEN-LAST:event_jTextFieldPrstFldrActionPerformed

    private void jButtonGrnlReportOutputFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonGrnlReportOutputFolderActionPerformed
        imgFolderPath = selectDir(imgFolderPath);
        jTextFieldRprtImgFolder.setText(imgFolderPath);
    }//GEN-LAST:event_jButtonGrnlReportOutputFolderActionPerformed

    private void jTextFieldRprtImgFolderFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldRprtImgFolderFocusLost
        imgFolderPath = jTextFieldRprtImgFolder.getText();
    }//GEN-LAST:event_jTextFieldRprtImgFolderFocusLost

    private void jTextFieldRprtImgFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldRprtImgFolderActionPerformed
        imgFolderPath = jTextFieldRprtImgFolder.getText();
    }//GEN-LAST:event_jTextFieldRprtImgFolderActionPerformed

    private void jButtonDeleteProcessingPipelineActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDeleteProcessingPipelineActionPerformed
        stopEdittingProcessingActionsTable();
        int[] selection = this.jTableProcessingPipelines.getSelectedRows();
        for (int i = 0; i < selection.length; i++) {
            selection[i] = jTableProcessingPipelines.convertRowIndexToModel(selection[i]);
        }
        Arrays.sort(selection);
        for (int i = (selection.length - 1); i >= 0; i--) {
            this.tableProcessingPipelinesModel.removeRow(selection[i]);
        }
    }//GEN-LAST:event_jButtonDeleteProcessingPipelineActionPerformed

    private void jButtonAddNewProcessingPipelineActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddNewProcessingPipelineActionPerformed
        stopEdittingProcessingActionsTable();
        tableProcessingPipelinesModel.addRow(new Object[]{"New Processing Action", "-Select Series Filter-", "-Select Pre-processing Action-", "-Select Quantification Action-","-Select Results Action-"});
    }//GEN-LAST:event_jButtonAddNewProcessingPipelineActionPerformed

    private void jButtonMoreInfoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonMoreInfoActionPerformed
        JOptionPane.showMessageDialog(this,
            "<html>Processing pipelines allow the user to define how certain types of spectroscopy series are processed. A Processing<br>"
            + "pipeline may include a Series Filter, a Pre-processing Action and a Quantification Action. Once a new spectroscopy series<br>"
            + "is loaded, only the Processing Actions that pass its Series Filter are visible to the user. The absence of a Series Filter<br>"
            + "will cause the Processing Action to be visible for any spectroscopy series.<br><br>"
            + "<b>Note:</b> For Pre-processing and Quantification Actions to be used, they should be included in a Processing pipeline.</html>"
            , "Processing pipelines", JOptionPane.INFORMATION_MESSAGE);
    }//GEN-LAST:event_jButtonMoreInfoActionPerformed

    private void jButtonDuplicateQuantificationActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDuplicateQuantificationActionActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListQuantificationActions.getSelectedIndex();
        QuantificationAction copy = quantificationActions.get(index).clone();
        copy.setName(copy.getName() + " - duplicate");
        QuantificationActionUI quantAction = new QuantificationActionUI(this, copy, -1);
    }//GEN-LAST:event_jButtonDuplicateQuantificationActionActionPerformed

    private void jButtonDeleteQuantificationActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDeleteQuantificationActionActionPerformed
        
        stopEdittingProcessingActionsTable( );
        int index = this.jListQuantificationActions.getSelectedIndex();
        String oldname = quantificationActions.get(index).getName();
        quantificationActionsListModel.remove(index);
        this.quantificationActions.remove(index);
        if ( quantificationActions.size( ) > 0 ) 
        {
            if ( index == quantificationActions.size( ) ) 
            {
                this.jListQuantificationActions.setSelectedIndex( index - 1 );
            } 
            else 
            {
                this.jListQuantificationActions.setSelectedIndex( index );
            }
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
        replaceElementsTableProcessingPipelines( 3, oldname, "" );
    }//GEN-LAST:event_jButtonDeleteQuantificationActionActionPerformed

    private void jButtonAddNewQuantificationActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddNewQuantificationActionActionPerformed
        stopEdittingProcessingActionsTable( );
        this.comboSeriesFilter.setSelectedIndex( comboSeriesFilter.getSelectedIndex( ) );
        this.comboPreprocessingAction.setSelectedIndex( comboPreprocessingAction.getSelectedIndex( ) );
        this.comboQuantificationAction.setSelectedIndex( comboQuantificationAction.getSelectedIndex( ) );
        
        // Show first quantification method selector.
        quantSelFrame = new QuantificationTypeSelectionFrame( );
        quantSelFrame.addActionListener( this );
        // QuantificationActionUI quantAction = new QuantificationActionUI( this, new QuantificationAction( QuantificationTypeSelectionFrame.TDFDFIT_SELECTED ), -1 );
        
    }//GEN-LAST:event_jButtonAddNewQuantificationActionActionPerformed

    private void jButtonEditQuantificationActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonEditQuantificationActionActionPerformed
        stopEdittingProcessingActionsTable( );
        int index = this.jListQuantificationActions.getSelectedIndex( );
        oldname = quantificationActions.get(index).getName( );
        QuantificationActionUI quantAction = new QuantificationActionUI( this, quantificationActions.get(index), index );
        // Show first quantification method selector.
        quantSelFrame = new QuantificationTypeSelectionFrame( );
        quantSelFrame.addActionListener( this );
    }//GEN-LAST:event_jButtonEditQuantificationActionActionPerformed

    private void jButtonDuplicatePreprocessingActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDuplicatePreprocessingActionActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListPreProcessingActions.getSelectedIndex();
        PreprocessingAction copy = preprocessingActions.get(index).clone();
        copy.setName(copy.getName() + " - duplicate");
        PreprocessingActionUI preprocessingActionUI = new PreprocessingActionUI(this, copy, -1);
    }//GEN-LAST:event_jButtonDuplicatePreprocessingActionActionPerformed

    private void jButtonEditPreprocessingActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonEditPreprocessingActionActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListPreProcessingActions.getSelectedIndex();
        oldname = preprocessingActions.get(index).getName();
        PreprocessingActionUI preprocessingActionUI = new PreprocessingActionUI(this, preprocessingActions.get(index), index);
    }//GEN-LAST:event_jButtonEditPreprocessingActionActionPerformed

    private void jButtonAddNewPreprocessingActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddNewPreprocessingActionActionPerformed
        stopEdittingProcessingActionsTable();
        this.comboSeriesFilter.setSelectedIndex(comboSeriesFilter.getSelectedIndex());
        this.comboPreprocessingAction.setSelectedIndex(comboPreprocessingAction.getSelectedIndex());
        this.comboQuantificationAction.setSelectedIndex(comboQuantificationAction.getSelectedIndex());
        PreprocessingActionUI preprocessingActionUI = new PreprocessingActionUI(this, new PreprocessingAction(model), -1);
    }//GEN-LAST:event_jButtonAddNewPreprocessingActionActionPerformed

    private void jButtonDeletePreprocessingActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDeletePreprocessingActionActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListPreProcessingActions.getSelectedIndex();
        String oldname = preprocessingActions.get(index).getName();
        preprocessingActionsListModel.remove(index);
        this.preprocessingActions.remove(index);
        if (preprocessingActions.size() > 0) {
            if (index == preprocessingActions.size()) {
                this.jListPreProcessingActions.setSelectedIndex(index - 1);
            } else {
                this.jListPreProcessingActions.setSelectedIndex(index);
            }
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
        replaceElementsTableProcessingPipelines( 2, oldname, "" );
    }//GEN-LAST:event_jButtonDeletePreprocessingActionActionPerformed

    private void jButtonDuplicateSeriesFilterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDuplicateSeriesFilterActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListSelectionFilters.getSelectedIndex();
        SeriesFilter copy = seriesFilters.get(index).clone();
        copy.setName(copy.getName() + " - duplicate");
        SeriesFilterUI seriesFilterUI = new SeriesFilterUI(this, copy, -1);
    }//GEN-LAST:event_jButtonDuplicateSeriesFilterActionPerformed

    private void jButtonEditSeriesFilterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonEditSeriesFilterActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListSelectionFilters.getSelectedIndex();
        oldname = seriesFilters.get(index).getName();
        SeriesFilterUI seriesFilterUI = new SeriesFilterUI(this, seriesFilters.get(index), index);
    }//GEN-LAST:event_jButtonEditSeriesFilterActionPerformed

    private void jButtonDeleteSeriesFilterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDeleteSeriesFilterActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListSelectionFilters.getSelectedIndex();
        String oldname = seriesFilters.get(index).getName();
        seriesFiltersListModel.remove(index);
        this.seriesFilters.remove(index);
        if (seriesFilters.size() > 0) {
            if (index == seriesFilters.size()) {
                this.jListSelectionFilters.setSelectedIndex(index - 1);
            } else {
                this.jListSelectionFilters.setSelectedIndex(index);
            }
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
        replaceElementsTableProcessingPipelines( 1, oldname, "" );
    }//GEN-LAST:event_jButtonDeleteSeriesFilterActionPerformed

    private void jButtonAddNewSeriesFilterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddNewSeriesFilterActionPerformed
        stopEdittingProcessingActionsTable();
        SeriesFilterUI seriesFilterUI = new SeriesFilterUI(this, new SeriesFilter(), -1);
    }//GEN-LAST:event_jButtonAddNewSeriesFilterActionPerformed

    private void jFormattedTextFieldSignalFreeFDMaxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jFormattedTextFieldSignalFreeFDMaxActionPerformed
        try {
            this.jFormattedTextFieldSignalFreeFDMax.commitEdit();
            signalFreeFDMax = Double.valueOf(this.jFormattedTextFieldSignalFreeFDMax.getValue().toString());
        } catch (ParseException ex) {
            Logger.getLogger(SettingsGUI.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_jFormattedTextFieldSignalFreeFDMaxActionPerformed

    private void jFormattedTextFieldSignalFreeFDMinActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jFormattedTextFieldSignalFreeFDMinActionPerformed
        try {
            this.jFormattedTextFieldSignalFreeFDMin.commitEdit();
            signalFreeFDMin = Double.valueOf(this.jFormattedTextFieldSignalFreeFDMin.getValue().toString());
        } catch (ParseException ex) {
            Logger.getLogger(SettingsGUI.class.getName()).log(Level.SEVERE, null, ex);
        }
    }//GEN-LAST:event_jFormattedTextFieldSignalFreeFDMinActionPerformed

    private void jButtonRExecutableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRExecutableActionPerformed
        if (System.getProperty("os.name").toUpperCase().contains("WINDOWS")) {
            rExe = selectFile(rExe, "exe");
        } else {
            rExe = selectFile(rExe, "");
        }
        jTextFieldRExecutable.setText(rExe);
    }//GEN-LAST:event_jButtonRExecutableActionPerformed

    private void jButtonCSIPrstFolderActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonCSIPrstFolderActionPerformed
        CSIGrdPrstFolder = selectDir(CSIGrdPrstFolder);
        jTextFieldCSIPrstFolder.setText(CSIGrdPrstFolder);
    }//GEN-LAST:event_jButtonCSIPrstFolderActionPerformed

    private void jTextFieldCSIPrstFolderFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldCSIPrstFolderFocusLost
        CSIGrdPrstFolder = jTextFieldCSIPrstFolder.getText();
    }//GEN-LAST:event_jTextFieldCSIPrstFolderFocusLost

    private void jButtonAddNewResultsActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddNewResultsActionActionPerformed
        stopEdittingProcessingActionsTable();
        ResultsActionUI resultsActionUI = new ResultsActionUI(this, new ResultsAction(), -1);
    }//GEN-LAST:event_jButtonAddNewResultsActionActionPerformed

    private void jButtonDeleteResultsActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDeleteResultsActionActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListResultsActions.getSelectedIndex();
        String oldname = resultsActions.get(index).getName();
        resultsActionsListModel.remove(index);
        this.resultsActions.remove(index);
        if (resultsActions.size() > 0) {
            if (index == resultsActions.size()) {
                this.jListResultsActions.setSelectedIndex(index - 1);
            } else {
                this.jListResultsActions.setSelectedIndex(index);
            }
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
        replaceElementsTableProcessingPipelines( 1, oldname, "" );
    }//GEN-LAST:event_jButtonDeleteResultsActionActionPerformed

    private void jButtonEditResultsActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonEditResultsActionActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListResultsActions.getSelectedIndex();
        oldname = resultsActions.get(index).getName();
        ResultsActionUI resultsActionUI = new ResultsActionUI(this, resultsActions.get(index), index);
    }//GEN-LAST:event_jButtonEditResultsActionActionPerformed

    private void jButtonDuplicateResultsActionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonDuplicateResultsActionActionPerformed
        stopEdittingProcessingActionsTable();
        int index = this.jListResultsActions.getSelectedIndex();
        ResultsAction copy = resultsActions.get(index).clone();
        copy.setName(copy.getName() + " - duplicate");
        ResultsActionUI resultsActionUI = new ResultsActionUI(this, copy, -1);
    }//GEN-LAST:event_jButtonDuplicateResultsActionActionPerformed

    public void addSeriesSelectionActionUI( SeriesFilter filter, int number ) 
    {
        if ( number == -1 ) 
        { 
            //New element
            seriesFiltersListModel.addElement( filter.getName( ) );
            this.seriesFilters.add( filter );
            this.jListSelectionFilters.setSelectedIndex( seriesFiltersListModel.getSize( ) - 1 );
        } 
        else //editing previous element
        {
            replaceElementsTableProcessingPipelines( 1, oldname, filter.getName( ) );
            seriesFiltersListModel.set( number, filter.getName( ) );
            this.seriesFilters.set( number, filter );
            this.jListSelectionFilters.setSelectedIndex( number );
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
    }

    public boolean isSeriesFilterNameRepeated( String newName, int previousPosition ) 
    {
        for ( int i = 0; i < seriesFilters.size( ); i++ ) 
        {
            SeriesFilter filter = seriesFilters.get( i );
            if ( filter.getName( ).trim( ).toLowerCase( ).matches(newName.trim().toLowerCase()) && i != previousPosition ) 
            {
                return true;
            }
        }
        return false;
    }

    public void addPreprocessingActionUI( PreprocessingAction action, int number ) 
    {
        if ( number == -1 ) 
        { //New element
            preprocessingActionsListModel.addElement( action.getName( ) );
            this.preprocessingActions.add( action );
            this.jListPreProcessingActions.setSelectedIndex( preprocessingActionsListModel.getSize() - 1 );
        } 
        else //editing previous element
        {
            replaceElementsTableProcessingPipelines( 2, oldname, action.getName( ) );
            preprocessingActionsListModel.set( number, action.getName( ) );
            this.preprocessingActions.set( number, action );
            this.jListPreProcessingActions.setSelectedIndex( number );
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
    }

    public boolean isPreprocessingActionNameRepeated(String newName, int previousPosition) {
        for (int i = 0; i < preprocessingActions.size(); i++) {
            PreprocessingAction action = preprocessingActions.get(i);
            if (action.getName().trim().toLowerCase().matches(newName.trim().toLowerCase()) && i != previousPosition) {
                return true;
            }
        }
        return false;
    }

    public boolean isQuantificationActionNameRepeated(String newName, int previousPosition) {
        for (int i = 0; i < quantificationActions.size(); i++) {
            QuantificationAction action = quantificationActions.get(i);
            if (action.getName().trim().toLowerCase().matches(newName.trim().toLowerCase()) && i != previousPosition) {
                return true;
            }
        }
        return false;
    }

    public void addQuantificationActionUI( QuantificationAction action, int number ) 
    {
        if ( number == -1 ) 
        { //New element
            quantificationActionsListModel.addElement( action.getName( ) );
            this.quantificationActions.add( action );
            this.jListQuantificationActions.setSelectedIndex( quantificationActionsListModel.getSize( ) - 1 );
        } 
        else //editing previous element
        {
            replaceElementsTableProcessingPipelines( 3, oldname, action.getName( ) );
            quantificationActionsListModel.set( number, action.getName( ) );
            this.quantificationActions.set( number, action );
            this.jListQuantificationActions.setSelectedIndex( number );
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
    }

    public boolean isResultsActionNameRepeated(String newName, int previousPosition) {
        for (int i = 0; i < resultsActions.size(); i++) {
            ResultsAction action = resultsActions.get(i);
            if (action.getName().trim().toLowerCase().matches(newName.trim().toLowerCase()) && i != previousPosition) {
                return true;
            }
        }
        return false;
    }

    public void addResultsActionUI( ResultsAction action, int number ) 
    {
        if (number == -1) 
        { //New element
            resultsActionsListModel.addElement( action.getName( ) );
            this.resultsActions.add( action );
            this.jListResultsActions.setSelectedIndex( resultsActionsListModel.getSize( ) - 1 );
        } 
        else //editing previous element
        {
            replaceElementsTableProcessingPipelines( 3, oldname, action.getName( ) );
            resultsActionsListModel.set( number, action.getName( ) );
            this.resultsActions.set( number, action );
            this.jListResultsActions.setSelectedIndex( number );
        }
        updateProcessingActionsButtonsStatus( );
        updateColumnsProcessingPipelinesTable( );
    }

    private void applyChanges( ) 
    {
        // Report
        ReportSettings.getInstance( ).setImgOutptFolder( new File( imgFolderPath ) );
        ReportSettings.getInstance( ).setPrstsPath( new File( prstPath ) );
        ReportSettings.getInstance( ).saveProperties( );

        // Global settings
        GlobalSettings.getInstance( ).setCSIGrdPrstFolder( CSIGrdPrstFolder );
        GlobalSettings.getInstance( ).setRExecutable( rExe );
        GlobalSettings.getInstance( ).setSignalFreeFDMin( Double.valueOf( this.jFormattedTextFieldSignalFreeFDMin.getValue( ).toString( ) ) );
        GlobalSettings.getInstance( ).setSignalFreeFDMax( Double.valueOf( this.jFormattedTextFieldSignalFreeFDMax.getValue( ).toString( ) ) );
        GlobalSettings.getInstance( ).saveParamChanges( );

        // Registration settings
        RegistrationSettings.getInstance( ).setAladinInstallPath( aladinInstallPath );
        RegistrationSettings.getInstance( ).setF3dInstallPath( f3dInstallPath );
        RegistrationSettings.getInstance( ).setDicom2niiInstallPath( dicom2niiInstallPath );
        RegistrationSettings.getInstance( ).setRegTransformInstallPath( regTransformInstallPath );
        RegistrationSettings.getInstance( ).setOutputFolder( registrationOutputFolder );
        RegistrationSettings.getInstance( ).setRoiRModel( roiRModel );
        RegistrationSettings.getInstance( ).setCoregistrationLibrary( coregistrationLibrary );
        RegistrationSettings.getInstance( ).saveProperties( );

        //Processing Actions
        this.stopEdittingProcessingActionsTable( );
        this.processingActions.clear( );
        for ( int i = 0; i < this.tableProcessingPipelinesModel.getRowCount(); i++ ) 
        {
            String name = (String) tableProcessingPipelinesModel.getValueAt( i, 0 );
            ProcessingAction action = new ProcessingAction( );
            action.setName( name );
            String nameFilter = (String) tableProcessingPipelinesModel.getValueAt(i, 1);
            String namePrepro = (String) tableProcessingPipelinesModel.getValueAt(i, 2);
            String nameQuantif = (String) tableProcessingPipelinesModel.getValueAt(i, 3);
            String nameRes = (String) tableProcessingPipelinesModel.getValueAt(i, 4);
            for ( SeriesFilter filter : this.seriesFilters ) 
            {
                if ( filter.getName( ).matches( nameFilter ) ) 
                {
                    action.setSeriesFilter( filter );
                    break;
                }
            }
            for ( PreprocessingAction actionPrepro : this.preprocessingActions ) 
            {
                if ( actionPrepro.getName( ).matches( namePrepro ) ) 
                {
                    action.setPreprocessingAction( actionPrepro );
                    break;
                }
            }
            for ( QuantificationAction actionQuantif : this.quantificationActions ) 
            {
                if ( actionQuantif.getName( ).matches( nameQuantif ) ) 
                {
                    selectedQuantificationAction = actionQuantif;
                    action.setQuantificationAction( actionQuantif );
                    break;
                }
            }
            for ( ResultsAction actionRes : this.resultsActions ) 
            {
                if (actionRes.getName( ).matches( nameRes ) ) 
                {
                    action.setResultsAction( actionRes );
                    break;
                }
            }
            processingActions.add( action );
        }

        ArrayList<SeriesFilter> copySeriesFilters = new ArrayList<>( );
        ArrayList<PreprocessingAction> copyPreprocessingActions = new ArrayList<>( );
        ArrayList<QuantificationAction> copyQuantificationActions = new ArrayList<>( );
        ArrayList<ResultsAction> copyResultsActions = new ArrayList<>( );

        for ( SeriesFilter filter : seriesFilters ) 
        {
            copySeriesFilters.add( filter.clone( ) );
        }
        for ( PreprocessingAction preproAction : preprocessingActions ) 
        {
            copyPreprocessingActions.add( preproAction.clone( ) );
        }
        for ( QuantificationAction quantAction : quantificationActions ) 
        {
            copyQuantificationActions.add( quantAction.clone( ) );
        }
        for ( ResultsAction resAction : resultsActions ) 
        {
            copyResultsActions.add( resAction.clone( ) );
        }

        ProcessingSettings.getInstance( ).setSeriesFilters( copySeriesFilters );
        ProcessingSettings.getInstance( ).setPreprocessingActions( copyPreprocessingActions );
        ProcessingSettings.getInstance( ).setQuantificationActions( copyQuantificationActions );
        ProcessingSettings.getInstance( ).setResultsActions( copyResultsActions );

        ProcessingSettings.getInstance( ).setProcessingActions( new ArrayList<>( processingActions ) ); // Only shallow copy needed
        ProcessingSettings.getInstance( ).saveProperties( );

    }

    /**
     * @param args the command line arguments
     */
//    public static void main(String args[]) {
//        /* Set the Nimbus look and feel */
//        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
//        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
//         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
//         */
//        try {
//            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
//                if ("Nimbus".equals(info.getName())) {
//                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
//                    break;
//                }
//            }
//        } catch (ClassNotFoundException ex) {
//            java.util.logging.Logger.getLogger(SettingsGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//        } catch (InstantiationException ex) {
//            java.util.logging.Logger.getLogger(SettingsGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//        } catch (IllegalAccessException ex) {
//            java.util.logging.Logger.getLogger(SettingsGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
//            java.util.logging.Logger.getLogger(SettingsGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
//        }
//        //</editor-fold>
//        //</editor-fold>
//        //</editor-fold>
//        //</editor-fold>
//
//        /* Create and display the form */
//        java.awt.EventQueue.invokeLater(new Runnable() {
//            public void run() {
//                new SettingsGUI().setVisible(true);
//            }
//        });
//    }
//<editor-fold defaultstate="collapsed" desc="GUI variables">
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupAppodisationType;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButtonAddNewPreprocessingAction;
    private javax.swing.JButton jButtonAddNewProcessingPipeline;
    private javax.swing.JButton jButtonAddNewQuantificationAction;
    private javax.swing.JButton jButtonAddNewResultsAction;
    private javax.swing.JButton jButtonAddNewSeriesFilter;
    private javax.swing.JButton jButtonAladin;
    private javax.swing.JButton jButtonApply;
    private javax.swing.JButton jButtonCSIPrstFolder;
    private javax.swing.JButton jButtonCancel;
    private javax.swing.JButton jButtonCoregistrationLibrary;
    private javax.swing.JButton jButtonDeletePreprocessingAction;
    private javax.swing.JButton jButtonDeleteProcessingPipeline;
    private javax.swing.JButton jButtonDeleteQuantificationAction;
    private javax.swing.JButton jButtonDeleteResultsAction;
    private javax.swing.JButton jButtonDeleteSeriesFilter;
    private javax.swing.JButton jButtonDicom2Nii;
    private javax.swing.JButton jButtonDuplicatePreprocessingAction;
    private javax.swing.JButton jButtonDuplicateQuantificationAction;
    private javax.swing.JButton jButtonDuplicateResultsAction;
    private javax.swing.JButton jButtonDuplicateSeriesFilter;
    private javax.swing.JButton jButtonEditPreprocessingAction;
    private javax.swing.JButton jButtonEditQuantificationAction;
    private javax.swing.JButton jButtonEditResultsAction;
    private javax.swing.JButton jButtonEditSeriesFilter;
    private javax.swing.JButton jButtonGrnlReportOutputFolder;
    private javax.swing.JButton jButtonMoreInfo;
    private javax.swing.JButton jButtonOk;
    private javax.swing.JButton jButtonRExecutable;
    private javax.swing.JButton jButtonRegTrsf;
    private javax.swing.JButton jButtonRegistrationOutputFolder;
    private javax.swing.JButton jButtonRoiModel;
    private javax.swing.JButton jButtonf3d;
    private javax.swing.JFormattedTextField jFormattedTextFieldSignalFreeFDMax;
    private javax.swing.JFormattedTextField jFormattedTextFieldSignalFreeFDMin;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JLabel jLabelAladin;
    private javax.swing.JLabel jLabelCSIPrstFolder;
    private javax.swing.JLabel jLabelDicom2Nii;
    private javax.swing.JLabel jLabelGnrlReportOutputFolder;
    private javax.swing.JLabel jLabelMicron;
    private javax.swing.JLabel jLabelNiftyRegTitle;
    private javax.swing.JLabel jLabelRExecutable;
    private javax.swing.JLabel jLabelRExecutable1;
    private javax.swing.JLabel jLabelRegTrsf;
    private javax.swing.JLabel jLabelRegistrationGeneral;
    private javax.swing.JLabel jLabelRegistrationOutputFolder;
    private javax.swing.JLabel jLabelRegistrationOutputFolder1;
    private javax.swing.JLabel jLabelRoiModel;
    private javax.swing.JLabel jLabelRoiRegistration;
    private javax.swing.JLabel jLabelf3d;
    private javax.swing.JList jListPreProcessingActions;
    private javax.swing.JList jListQuantificationActions;
    private javax.swing.JList jListResultsActions;
    private javax.swing.JList jListSelectionFilters;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanelAladin;
    private javax.swing.JPanel jPanelAladinContent;
    private javax.swing.JPanel jPanelCoregistrationLibrary;
    private javax.swing.JPanel jPanelDicom2Nii;
    private javax.swing.JPanel jPanelDicom2NiiContent;
    private javax.swing.JPanel jPanelGeneral;
    private javax.swing.JPanel jPanelMenu;
    private javax.swing.JPanel jPanelMicronContent;
    private javax.swing.JPanel jPanelNiftyReg;
    private javax.swing.JPanel jPanelNiftyRegContent;
    private javax.swing.JPanel jPanelOverallContainer;
    private javax.swing.JPanel jPanelProcessingActions;
    private javax.swing.JPanel jPanelRegTrsf;
    private javax.swing.JPanel jPanelRegTrsfContent;
    private javax.swing.JPanel jPanelRegistration;
    private javax.swing.JPanel jPanelRegistrationGeneral;
    private javax.swing.JPanel jPanelRegistrationGeneralContent;
    private javax.swing.JPanel jPanelRegistrationOutputFolder;
    private javax.swing.JPanel jPanelRegistrationOutputFolderContent;
    private javax.swing.JPanel jPanelReports;
    private javax.swing.JPanel jPanelRoiModel;
    private javax.swing.JPanel jPanelRoiModelContent;
    private javax.swing.JPanel jPanelRoiReg;
    private javax.swing.JPanel jPanelRoiRegContent;
    private javax.swing.JPanel jPanelf3d;
    private javax.swing.JPanel jPanelf3dContent;
    private javax.swing.JPanel jPanelmicron;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JSeparator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JTable jTableProcessingPipelines;
    private javax.swing.JTextField jTextFieldAladin;
    private javax.swing.JTextField jTextFieldCSIPrstFolder;
    private javax.swing.JTextField jTextFieldCoregistrationLibrary;
    private javax.swing.JTextField jTextFieldDicom2Nii;
    private javax.swing.JTextField jTextFieldPrstFldr;
    private javax.swing.JTextField jTextFieldRExecutable;
    private javax.swing.JTextField jTextFieldRExecutable1;
    private javax.swing.JTextField jTextFieldRegTrsf;
    private javax.swing.JTextField jTextFieldRegistrationOutputFolder;
    private javax.swing.JTextField jTextFieldRoiModel;
    private javax.swing.JTextField jTextFieldRprtImgFolder;
    private javax.swing.JTextField jTextFieldf3d;
    // End of variables declaration//GEN-END:variables

    //</editor-fold>
    private void updateProcessingActionsButtonsStatus() 
    {
        this.jButtonDeleteSeriesFilter.setEnabled(!this.jListSelectionFilters.isSelectionEmpty());
        this.jButtonEditSeriesFilter.setEnabled(!this.jListSelectionFilters.isSelectionEmpty());
        this.jButtonDuplicateSeriesFilter.setEnabled(!this.jListSelectionFilters.isSelectionEmpty());
        this.jButtonDeletePreprocessingAction.setEnabled(!this.jListPreProcessingActions.isSelectionEmpty());
        this.jButtonEditPreprocessingAction.setEnabled(!this.jListPreProcessingActions.isSelectionEmpty());
        this.jButtonDuplicatePreprocessingAction.setEnabled(!this.jListPreProcessingActions.isSelectionEmpty());
        this.jButtonDeleteQuantificationAction.setEnabled(!this.jListQuantificationActions.isSelectionEmpty());
        this.jButtonEditQuantificationAction.setEnabled(!this.jListQuantificationActions.isSelectionEmpty());
        this.jButtonDuplicateQuantificationAction.setEnabled(!this.jListQuantificationActions.isSelectionEmpty());
        this.jButtonDeleteResultsAction.setEnabled(!this.jListResultsActions.isSelectionEmpty());
        this.jButtonEditResultsAction.setEnabled(!this.jListResultsActions.isSelectionEmpty());
        this.jButtonDuplicateResultsAction.setEnabled(!this.jListResultsActions.isSelectionEmpty());
    }

    
    /**
     * Updates the columns for Processing pipelines table.
     */
    private void updateColumnsProcessingPipelinesTable( ) 
    {
        this.comboSeriesFilter.removeAllItems( );
        this.comboPreprocessingAction.removeAllItems( );
        this.comboQuantificationAction.removeAllItems( );
        this.comboResultsAction.removeAllItems( );
        this.comboSeriesFilter.addItem( "-Select Series Filter-" );
        this.comboPreprocessingAction.addItem( "-Select Pre-processing Action-" );
        this.comboQuantificationAction.addItem( "-Select Quantification Action-" );
        this.comboResultsAction.addItem( "-Select Results Action-" );
        for ( SeriesFilter filter : this.seriesFilters ) 
        {
            comboSeriesFilter.addItem( filter.getName( ) );
        }
        for ( PreprocessingAction action : this.preprocessingActions ) 
        {
            comboPreprocessingAction.addItem( action.getName( ) );
        }
        for ( QuantificationAction action : this.quantificationActions ) 
        {
            comboQuantificationAction.addItem( action.getName( ) );
        }
        for ( ResultsAction action : this.resultsActions )
        {
            comboResultsAction.addItem( action.getName( ) );
        }
    }
    
    /**
     * Updates the columns for Processing pipelines table.
     */
    private void replaceElementsTableProcessingPipelines( int column, String oldname, String newname ) 
    {
        this.jTableProcessingPipelines.clearSelection( );
        for ( int i = 0; i < this.tableProcessingPipelinesModel.getRowCount( ); i++ ) 
        {
            String nameAux = (String) tableProcessingPipelinesModel.getValueAt( i, column );
            if ( nameAux.trim( ).toLowerCase( ).matches( oldname.trim( ).toLowerCase( ) ) ) 
            {
                if ( newname.matches( "" ) ) 
                {
                    switch (column) 
                    {
                        case 0:
                            tableProcessingPipelinesModel.setValueAt( comboSeriesFilter.getItemAt( 0 ), i, column );
                            break;
                        case 1:
                            tableProcessingPipelinesModel.setValueAt( comboPreprocessingAction.getItemAt( 0 ), i, column );
                            break;
                        case 2:
                            tableProcessingPipelinesModel.setValueAt( comboQuantificationAction.getItemAt( 0 ), i, column );
                            break;
                        case 3:
                            tableProcessingPipelinesModel.setValueAt( comboResultsAction.getItemAt( 0 ), i, column );
                            break;
                    }
                } 
                else 
                {
                    tableProcessingPipelinesModel.setValueAt( newname, i, column );
                }
            }
        }
    }

    private void stopEdittingProcessingActionsTable( ) 
    {
//        this.comboSeriesFilter.setSelectedIndex(comboSeriesFilter.getSelectedIndex());
//        this.comboPreprocessingAction.setSelectedIndex(comboPreprocessingAction.getSelectedIndex());
//        this.comboQuantificationAction.setSelectedIndex(comboQuantificationAction.getSelectedIndex());
        TableCellEditor editor = this.jTableProcessingPipelines.getCellEditor( );
        if ( editor != null ) 
        {
            editor.stopCellEditing( );
        }
    }
    
    /**
     * Respond to registered ActionEvents!
     * @param e 
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        if ( e.getSource( ) == quantSelFrame )
        {
            if ( e.getID( ) == QuantificationTypeSelectionFrame.APPLY_BUTTON_PRESSED_ID )
            {
                if ( quantSelFrame.getSelectedQuantificationType( ) == QuantificationTypeSelectionFrame.QUEST_SELECTED )
                {
                    QuantificationActionUI quantAction = new QuantificationActionUI( this, new QuantificationAction( QuantificationTypeSelectionFrame.QUEST_SELECTED ), -1 );
                }
                if ( quantSelFrame.getSelectedQuantificationType( ) == QuantificationTypeSelectionFrame.TDFDFIT_SELECTED )
                {
                    selectedQuantificationAction = new QuantificationAction( QuantificationTypeSelectionFrame.TDFDFIT_SELECTED );
                    
                    selectedQuantificationAction.setName( quantSelFrame.getSelectedQuantificationModel( ) );
                    
                    selectedQuantificationAction.setTDFDFitQuantificationModelName( quantSelFrame.getSelectedQuantificationModel( ) );
                    selectedQuantificationAction.setTDFDFitQuantificationModelPathName( quantSelFrame.getSelectedQuantificationModelPath( ) );
                    
                    QuantificationActionUI quantAction = new QuantificationActionUI( this, selectedQuantificationAction, -1 );
                }
            }
        }
    }
    
    public QuantificationAction getSelectedQuantificationAction( )
    {
        return selectedQuantificationAction;
    }
    
    public GlobalModel getGlobalModel( )
    {
        return model;
    }
    
    
    QuantificationAction selectedQuantificationAction;
}
