/*
 * File: $Id: FormattedTextField.java,v 1.1.1.1 2006/04/19 09:38:48 hans Exp $
 * History
 * Date: 04.02.2003  Authors: Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import javax.swing.JTextField;


import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

import java.awt.Toolkit;
import java.awt.Dimension;

import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;

import java.text.NumberFormat;
import java.text.DecimalFormat;
import java.text.ParseException;

public class FormattedTextField extends JTextField
{
    /**
     * Instantiates a FormattedTextField.
     * @param format
     * @param minval
     * @param maxval 
     */
    public FormattedTextField( String format, double minval, double maxval )
    {
        this( new DecimalFormat( format ), minval, maxval );
    }

    /**
     * Instantiates a FormattedTextField.
     * @param nf
     * @param minval
     * @param maxval 
     */
    public FormattedTextField( NumberFormat nf, double minval, double maxval )
    {
        format = nf;
        minValue = minval;
        maxValue = maxval;

        addFocusListener( new FocusAdapter( )
        {
            public void focusLost( FocusEvent fe )
            {
                String text = getText( );

		if ( text.length( ) == 0 || text.equals( "-" ) )
		{
                    setText( currentText );
		}
                else if ( !text.equals( currentText ) )
                {
                    setText( text );
                }
            }
        });
    }

    /**
     * Creates a default document model.
     * @return 
     */
    protected Document createDefaultModel( )
    {
        return new PlainDocument( )
        {
            public void insertString( int offs, String str, AttributeSet a )
            throws BadLocationException
            {
                char[] source = str.toCharArray( );
                char[] result = new char[source.length];
                int j=0;
                for ( int i=0; i < result.length; i++ )
                {
                    if ( Character.isDigit( source[i] ) || ( source[i] == '-' && offs == 0 && i == 0 ) || source[i] == '.' )
                    {
                        result[j++]= source[i];
                    }
                    else
                    {
                        Toolkit.getDefaultToolkit( ).beep( );
                    }
                }
                if ( j > 0 )
                {
                    super.insertString( offs, new String( result, 0, j ), a );
                    try
                    {
                        String newText = super.getText( 0, super.getLength( ) );
                        if ( !newText.equals( "-" ) )
                        {
                            double value = format.parse( newText ).doubleValue( );
                            double correct = testValue( value );
                            if ( value != correct )
                            {
                                setValue( correct );
                                Toolkit.getDefaultToolkit( ).beep( );
                            }
                        }
                    }
                    catch ( ParseException nfe )
                    {
                        setText( currentText );
                        Toolkit.getDefaultToolkit( ).beep( );
                    }
                }
                else if ( super.getLength( ) == 0 )
                {
                    super.insertString( 0, currentText, a );
                }
            }
            
            // For proper serialization
            private static final long serialVersionUID = 1424325898889393L;
        };
       
    }

    /**
     * Sets a specific value to the max or min value if it is outside the range.
     * @param dVal
     * @return 
     */
    private double testValue( double dVal )
    {
        if ( dVal > maxValue )
        {
            dVal = maxValue;
        }
        if ( dVal < minValue )
        {
            dVal = minValue;
        }
        return dVal;
    }
    
    /**
     * Method to check whether maximum value has been set.
     */
    public boolean isMaximumValueSet( )
    {
        if ( getValue( ) == maxValue)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Method to check whether minimum value has been set.
     */
    public boolean isMinimumValueSet( )
    {
        if ( getValue( ) == minValue)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Sets the text.
     * @param text 
     */
    public void setText( String text )
    {
	try
        {
            currentValue = format.parse( text ).doubleValue( );
            super.setText( text );
            currentText = text;
        }
        catch ( ParseException pe )
        {
            Toolkit.getDefaultToolkit( ).beep( );
        }
    }

    /**
     * Sets the value to be displayed.
     * @return 
     */
    public void setValue( double newVal )
    {
        currentValue = newVal;
        currentText = format.format( newVal );
        super.setText( currentText );
    }
    
    /**
     * Returns the currently displayed value.
     * @return 
     */
    public double getValue( )
    {
        return currentValue;
    }
    
    /**
     * Sets the maximum value the text field can display.
     * @param max_val maximum allowed value
     */
    public void setMaximumValue( double max_val )
    {
        maxValue = max_val;
        if ( currentValue > max_val )
        {
            currentValue = max_val;
            setValue( currentValue );
        }
    }

    /**
     * Sets the minimum value the text field can display.
     * @param min_val minimum allowed value
     */
    public void setMinimumValue( double min_val )
    {
        minValue = min_val;
        if ( currentValue < min_val )
        {
            currentValue = min_val;
            setValue( currentValue );
        }
    }
    
    private NumberFormat format;
    private String currentText;
    private double maxValue, minValue, currentValue;
    
    // For proper serialization
    private static final long serialVersionUID = 1482324598889393L;
}
