/* 
 * File: $Id: MRFitStrategy.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 09.12.2003, 9:33  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 
/*
 * MRSpectrumFitting.java
 *
 * Created on December 9, 2003, 9:33 AM
 */

package org.spectrim.tdfdfit;

import java.io.RandomAccessFile;
import java.io.IOException;

import java.util.Vector;


/**
 *
 * The class containing the list of all FittingStrategyStep objects.
 *
 * @author  hans
 */
public class MRFitStrategy 
{
    
    /** 
     * Creates a new instance of MRFitStrategy. 
     */
    private MRFitStrategy( )
    {
        m_fittingStrategySteps = new Vector<MRFitStrategyStep>( );
        
        MRFitStrategyStep stratStep1 = new MRFitStrategyStep( true, false, false, false, false );
        m_fittingStrategySteps.add( stratStep1 );
        
        MRFitStrategyStep stratStep2 = new MRFitStrategyStep( false, true, false, true, false );
        m_fittingStrategySteps.add( stratStep2 );
        
        MRFitStrategyStep stratStep3 = new MRFitStrategyStep( true, false, false, false, false );
        m_fittingStrategySteps.add( stratStep3 );
        
        MRFitStrategyStep stratStep4 = new MRFitStrategyStep( true, false, true, false, true );
        m_fittingStrategySteps.add( stratStep4 );
        
        MRFitStrategyStep stratStep5 = new MRFitStrategyStep( true, true, true, true, true );
        m_fittingStrategySteps.add( stratStep5 );
    }
    
    /**
     * Returns the one and only instance of MRFitStrategy.
     * @return 
     */
    public static MRFitStrategy getInstance( )
    {
        if ( theInstance == null )
        {
            theInstance = new MRFitStrategy( );
        }
        return theInstance;
    }
    
    /**
     * Sets the default fitting ranges based on the total number of points.
     * @param numberOfSamplePoints
     */
    public void setDefaultFittingRanges( int numberOfSamplePoints )
    {
        for ( int i = 0; i < m_fittingStrategySteps.size( ) ; i++ )
        {
            MRFitStrategyStep stratStep = m_fittingStrategySteps.get( i );
            stratStep.deleteAllFittingRanges( );
            stratStep.addFittingRange( (int)( ( 470./1024. ) * numberOfSamplePoints ) , (int)( ( 950./1024. ) * numberOfSamplePoints ) );
        }
    }
    
    
    /**
     * Writes the fitting strategy to file.
     * @param writer
     */
    public void write( RandomAccessFile writer )
    {
        try
        {
            writer.writeUTF( "<MRFitStrategy>" );
            
            writer.writeInt( m_fittingStrategySteps.size( ) );
            
            for ( int i = 0 ; i < m_fittingStrategySteps.size( ) ; i++ )
            {
                m_fittingStrategySteps.get( i ).write( writer );
            }
        
            writer.writeUTF( "</MRFitStrategy>" );
        }
        catch ( IOException e )
        {
            
        }
    }
    
    
    /**
     * Writes the fitting strategy to file.
     * @param reader
     */
    public void read( RandomAccessFile reader )
    {
        // Clear the vector profilactic
        m_fittingStrategySteps.clear( );
        
        try
        {
            String tag = reader.readUTF( );
            
            // System.out.println( "tag = " + tag );
            
            if ( tag.compareTo( "<MRFitStrategy>" ) == 0 )
            {
                
                int fitStategySteps = reader.readInt( );
                
                while ( tag.compareTo( "</MRFitStrategy>" ) != 0)
                {
                    for ( int i = 0 ; i < fitStategySteps ; i++ )
                    {
                        MRFitStrategyStep fitStratStep = new MRFitStrategyStep( );
                        fitStratStep.read( reader );
                        m_fittingStrategySteps.add( fitStratStep );
                    }
                    tag = reader.readUTF( );
                }
            }
            
        }
        catch ( IOException e )
        {
            
        }
    }
    
    /**
     * Returns an MRFitStrategyStep with fitStrategyNumber.
     * @param strategyStepNumber
     * @return 
     */ 
    public MRFitStrategyStep getFitStrategyStep( int strategyStepNumber )
    {
        return m_fittingStrategySteps.get( strategyStepNumber );
    }
    
     /**
      * Adds a MRFitStrategyStep with fitStrategyNumber.
     * @param strategyStep
      */ 
    public void addFitStrategyStep( MRFitStrategyStep strategyStep )
    {
        m_fittingStrategySteps.add( strategyStep );
    }
    
    /**
     * Removes a MRFitStrategyStep at index.
     * @param index
     */ 
    public void removeFitStrategyStep( int index )
    {
        m_fittingStrategySteps.removeElementAt( index );
    }

    /**
     * Removes a MRFitStrategyStep at index.
     * @param index
     * @param mrFitStrat
     */ 
    public void insertFitStrategyStep( int index, MRFitStrategyStep mrFitStrat )
    {
        m_fittingStrategySteps.insertElementAt(mrFitStrat, index );
    }
    
    /**
     * Set the default truncation after point number.
     * @param truncate
     */
    public void setTDTruncation( int truncate )
    {
        for ( int i = 0 ; i < m_fittingStrategySteps.size( ) ; i++ )
        {
            MRFitStrategyStep step = m_fittingStrategySteps.get( i );
            step.setTruncateAfterPointNumber( truncate );
        }
    }
     
    /**
     * Returns an MRFitStrategyStep with fitStrategyNumber.
     * @return 
     */ 
    public int getNumberOfFitStrategySteps(  )
    {
        return this.m_fittingStrategySteps.size( );
    }
    
    // Static private attribute is the instance itself (Singleton!)
    private static  MRFitStrategy theInstance = null;
    
    private Vector<MRFitStrategyStep> m_fittingStrategySteps;
    private int m_currentFittingStrategyStep = 1;
}