/*
 * File: $Id: MRFitStrategyPanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 04.12.2003, 23:05  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;

import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.BoxLayout;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.JCheckBox;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import java.util.Vector;
import javax.swing.JLabel;

/**
 * Allows the user to define the fit strategy to be applied graphically
 * 
 * @author  hans
 */
public class MRFitStrategyPanel extends JPanel implements ActionListener, ItemListener
{

    /** 
     * Creates a new instance of MRFitStrategyPanel. 
     * @param parentFrame
     */
    public MRFitStrategyPanel( JFrame parentFrame ) 
    {
        m_parentFrame = parentFrame;
        setLayout( new BorderLayout( ) );
        
        m_collectionPanel = new JPanel( );
        
        m_collectionPanel.setLayout( new BoxLayout( m_collectionPanel, BoxLayout.Y_AXIS ) );
        m_collectionPanel.setBorder( BorderFactory.createCompoundBorder( 
                                     BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                     BorderFactory.createTitledBorder( "Fit Strategy Definition" ) ) );
        
        m_fitStategyStepNumberSelecionPanel = new JPanel( );
        m_fitStategyStepNumberSelecionPanel.setLayout( new BoxLayout( m_fitStategyStepNumberSelecionPanel, BoxLayout.X_AXIS ) );
        
        m_fitStrategyStepNumberCombo = new JComboBox( );
        m_fitStrategyStepNumberCombo.setPreferredSize( new Dimension( 150, 25 ) );
        m_fitStrategyStepNumberCombo.setMaximumSize( new Dimension( 150, 25 ) );
        m_fitStrategyStepNumberCombo.setMinimumSize( new Dimension( 150, 25 ) );
        m_fitStrategyStepNumberCombo.addActionListener( this );
        
        m_fitStategyStepNumberSelecionPanel.add( m_fitStrategyStepNumberCombo );
        
        m_fitWhichParameterGroupsPanel = new JPanel( );
        m_fitWhichParameterGroupsPanel.setLayout( new BoxLayout( m_fitWhichParameterGroupsPanel, BoxLayout.X_AXIS ) );
        m_fitWhichParameterGroupsPanel.setBorder( BorderFactory.createCompoundBorder(
                                                  BorderFactory.createTitledBorder( "Parameter groups to fit" ),
                                                  BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
                
        m_fitWhichParameterGroupsPanel.add( Box.createHorizontalGlue( ) );
        m_fitArea= new JCheckBox( "Area" );
        m_fitArea.addActionListener( this );
        
        m_fitWhichParameterGroupsPanel.add( m_fitArea );
        m_fitWhichParameterGroupsPanel.add( Box.createHorizontalGlue( ) );
        
        m_fitOffset = new JCheckBox( "Offset" );
        m_fitOffset.addActionListener( this );
        
        m_fitWhichParameterGroupsPanel.add( m_fitOffset );
        m_fitWhichParameterGroupsPanel.add( Box.createHorizontalGlue( ) );
        
        m_fitGaussWidth = new JCheckBox( "Gauss width" );
        m_fitGaussWidth.addActionListener( this );
        
        m_fitWhichParameterGroupsPanel.add( m_fitGaussWidth );
        m_fitWhichParameterGroupsPanel.add( Box.createHorizontalGlue( ) );
        
        m_fitPhase = new JCheckBox( "Phase" );
        m_fitPhase.addActionListener( this );
        
        m_fitWhichParameterGroupsPanel.add( m_fitPhase );
        m_fitWhichParameterGroupsPanel.add( Box.createHorizontalGlue( ) );
        
        m_fitLorentz = new JCheckBox( "Lorentz width" );
        m_fitLorentz.addActionListener( this );
        
        m_fitWhichParameterGroupsPanel.add( m_fitLorentz );
        m_fitWhichParameterGroupsPanel.add( Box.createHorizontalGlue( ) );
        
        m_fitStrategyOptionsPanel = new JPanel( );
        m_fitStrategyOptionsPanel.setLayout( new BoxLayout(m_fitStrategyOptionsPanel, BoxLayout.Y_AXIS ) );
        m_fitStrategyOptionsPanel.setBorder( BorderFactory.createCompoundBorder(
                                             BorderFactory.createTitledBorder( "Properties" ),
                                             BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
                
        JPanel firstOptionsPanel = new JPanel( );
        firstOptionsPanel.setLayout( new BoxLayout( firstOptionsPanel, BoxLayout.X_AXIS ) );
        
        // firstOptionsPanel.add( Box.createHorizontalGlue( ) );
        m_fitAlgorithmToApplyCombo = new JComboBox( );
        m_fitAlgorithmToApplyCombo.setPreferredSize( new Dimension( 150, 25 ) );
        m_fitAlgorithmToApplyCombo.setMaximumSize( new Dimension( 150, 25 ) );
        m_fitAlgorithmToApplyCombo.setMinimumSize( new Dimension( 150, 25 ) );
        m_fitAlgorithmToApplyCombo.addItemListener( this );
        
        m_fitAlgorithmToApplyCombo.addItem( "Random" );
        m_fitAlgorithmToApplyCombo.addItem( "Conjugate gradients" );
        m_fitAlgorithmToApplyCombo.setSelectedIndex( 0 );
        
        firstOptionsPanel.add( m_fitAlgorithmToApplyCombo );
        firstOptionsPanel.add( Box.createHorizontalGlue( ) );
        m_fitChannelTypeCombo = new JComboBox( );
        m_fitChannelTypeCombo.setPreferredSize( new Dimension( 150, 25 ) );
        m_fitChannelTypeCombo.setMaximumSize( new Dimension( 150, 25 ) );
        m_fitChannelTypeCombo.setMinimumSize( new Dimension( 150, 25 ) );
        
        m_fitChannelTypeCombo.addItemListener( this );
        
        m_fitChannelTypeCombo.addItem( "Absorption channel" );
        m_fitChannelTypeCombo.addItem( "Dispersion channel" );
        m_fitChannelTypeCombo.addItem( "Complex" );
        m_fitChannelTypeCombo.addItem( "Magnitude" );
        m_fitChannelTypeCombo.addItem( "Power" );
        
        firstOptionsPanel.add( m_fitChannelTypeCombo );
        firstOptionsPanel.add( Box.createHorizontalGlue( ) );
        
        m_fitWeightingTypeCombo= new JComboBox( );
        m_fitWeightingTypeCombo.addItem( "Uniform weighting" );
        m_fitWeightingTypeCombo.addItem( "Non uniform weighting" );
        m_fitWeightingTypeCombo.setSelectedIndex( 0 );
        m_fitWeightingTypeCombo.setPreferredSize( new Dimension( 160, 25 ) );
        m_fitWeightingTypeCombo.setMaximumSize( new Dimension( 160, 25 ) );
        m_fitWeightingTypeCombo.setMinimumSize( new Dimension( 160, 25 ) );
        
        m_fitWeightingTypeCombo.addItemListener( this );
        
        firstOptionsPanel.add( m_fitWeightingTypeCombo );
        firstOptionsPanel.add( Box.createHorizontalGlue( ) );
        
        JPanel secondOptionsPanel = new JPanel( );
        secondOptionsPanel.setLayout( new BoxLayout( secondOptionsPanel, BoxLayout.X_AXIS ) );
        
        m_applyConvolution = new JCheckBox( "Apply convolution" );
        m_applyConvolution.addActionListener( this );
        
        m_applyLineShapeFit = new JCheckBox( "Apply non analytic lineshape fit" );
        m_applyLineShapeFit.addActionListener( this );
        m_applyLineShapeFit.setEnabled( false );
        
        secondOptionsPanel.setBorder( BorderFactory.createEmptyBorder( 15, 5, 15, 5 ) );
        
        secondOptionsPanel.add( Box.createHorizontalGlue( ) );
        secondOptionsPanel.add( m_applyConvolution );
        secondOptionsPanel.add( Box.createHorizontalGlue( ) );
        secondOptionsPanel.add( m_applyLineShapeFit );
        secondOptionsPanel.add( Box.createHorizontalGlue( ) );
        
        JPanel thirdOptionsPanel = new JPanel( );
        thirdOptionsPanel.setLayout( new BoxLayout( thirdOptionsPanel, BoxLayout.X_AXIS ) );
    
        double[] stepSize = new double[1];
        stepSize[0] = 5.0;
        
        m_numberOfIterations = new SpinBox( "Iterations", 1, stepSize, 
                                            1.0, 1000.0,  
                                            "--", 100.0, 4, "0" );
        m_numberOfIterations.addActionListener( this );
        
        thirdOptionsPanel.add( m_numberOfIterations );
        
        double[] stepSize1 = new double[1];
        double[] stepSize2 = new double[1];
        stepSize1[0] = 128.0;
        stepSize2[0] = 1.0;
        
        m_timeDomainTruncationPoint = new SpinBox( "Truncation", 1, stepSize1, 
                                                   0.0, 8192.0,  
                                                   "--", 1024.0, 4, "0" );
        m_timeDomainTruncationPoint.addActionListener( this );
        
        m_timeDomainIgnoreFirstPoints = new SpinBox( "Ignore", 1, stepSize2, 
                                                     0.0, 512.0,  
                                                     "--", 0.0, 4, "0" );
        
        m_timeDomainIgnoreFirstPoints.addActionListener( this );
        
        thirdOptionsPanel.add( m_timeDomainTruncationPoint );
        thirdOptionsPanel.add( m_timeDomainIgnoreFirstPoints );
        
        m_fitStrategyOptionsPanel.add( firstOptionsPanel );
        m_fitStrategyOptionsPanel.add( secondOptionsPanel );
        m_fitStrategyOptionsPanel.add( thirdOptionsPanel );
        
        createRangesPanel( );
        createGeneralOffsetDefinitionPanel();
        createButtonPanel( );
        
        m_collectionPanel.add( m_fitStategyStepNumberSelecionPanel );
        m_collectionPanel.add( m_fitWhichParameterGroupsPanel );
        m_collectionPanel.add( m_fitStrategyOptionsPanel );
        m_collectionPanel.add( m_fitStrategyRangesPanel );
        m_collectionPanel.add( m_buttonPanel );
        
        m_collectionPanel.add( m_generalOffsetDefinitionPanel );
        
        m_collectionPanel.add( Box.createVerticalGlue( ) );
        add( m_collectionPanel );
        
        // setGUIValues( 0 ); 
    }

    
    /**
     * Updates the GUI.
     */
    public void upDateGUI( )
    {
        setGUIValues( m_currentStrategyNumber );
    }
    
    
    /**
     * There always exists a default model having a FitStrategy singleton object
     * associated to it.
     */
    private void getGUIValues( )
    {
        m_currentStrategyStep.setFitArea( m_fitArea.isSelected( ) );
        m_currentStrategyStep.setFitOffset( m_fitOffset.isSelected( ) );
        m_currentStrategyStep.setFitGauss( m_fitGaussWidth.isSelected( ) );
        m_currentStrategyStep.setFitPhase( m_fitPhase.isSelected( ) );
        m_currentStrategyStep.setFitLorentz( m_fitLorentz.isSelected( ) );
        
        m_currentStrategyStep.setApplyConvolution( m_applyConvolution.isSelected( ) );
        m_currentStrategyStep.setApplyLineShapeFit( m_applyLineShapeFit.isSelected( ) );
    }
    
    /**
     * There always exists a default model having a FitStrategy singleton object
     * associated to it.
     */
    private void setGUIValues( int strategyNumber )
    {
        
        m_currentStrategyNumber = strategyNumber;
        
        MRFitStrategy mrstrat = MRFitStrategy.getInstance( );
        m_currentStrategyStep = mrstrat.getFitStrategyStep( m_currentStrategyNumber );
        
        defineFitStrategyStepCombo( mrstrat.getNumberOfFitStrategySteps( ) );
        
        
        m_fitStrategyStepNumberCombo.setSelectedIndex( m_currentStrategyNumber );
        
        m_fitArea.setSelected( m_currentStrategyStep.getFitArea( ) );
        m_fitOffset.setSelected( m_currentStrategyStep.getFitOffset( ) );
        m_fitGaussWidth.setSelected( m_currentStrategyStep.getFitGauss( ) );
        m_fitPhase.setSelected( m_currentStrategyStep.getFitPhase( ) );
        m_fitLorentz.setSelected( m_currentStrategyStep.getFitLorentz( ) );
        
        m_fitAlgorithmToApplyCombo.setSelectedIndex( m_currentStrategyStep.getFitAlgorithType( ) - 1 );
        
        m_fitChannelTypeCombo.setSelectedIndex( m_currentStrategyStep.getFittingType( ) );
        
        m_fitWeightingTypeCombo.setSelectedIndex( m_currentStrategyStep.getLeastSquaresWeightingType( ) );
        
        m_applyConvolution.setSelected( m_currentStrategyStep.getApplyConvolution( ) );
        m_applyLineShapeFit.setSelected( m_currentStrategyStep.getApplyLineShapeFit( ) );
        
        m_numberOfIterations.setTextField( "" + m_currentStrategyStep.getNumberOfIterations( ) );
        
        m_timeDomainTruncationPoint.setTextField( "" + m_currentStrategyStep.getTruncateAfterPointNumber( ) );
        
        m_timeDomainIgnoreFirstPoints.setTextField( "" + m_currentStrategyStep.getIgnoreFirstSamplePointsNumber( ));
        
        m_minimumMaximumTableModel = new DefaultTableModel( );
        
        m_minimumMaximumTableModel.addColumn( "Interval Number" );
        m_minimumMaximumTableModel.addColumn( "Minimum [pnts]" );
        m_minimumMaximumTableModel.addColumn( "Maximum [pnts]" );
        
        for ( int i = 0 ; i < m_currentStrategyStep.getNumberOfRanges( ) ; i++ )
        {
            m_minimumMaximumTableModel.insertRow( i, new Object[]{ "" + ( i + 1), 
                                                  "" + m_currentStrategyStep.getFittingRangeMinimum( i ), 
                                                  "" + m_currentStrategyStep.getFittingRangeMaximum( i ) } );
        }
        
        m_minimumMaximumIntervals.setModel( m_minimumMaximumTableModel );
        m_minimumMaximumIntervals.updateUI( );
        
        setSelectedRow( 0 );
        
    }
    
    /**
     * Fills the fit strategy step combo box with the appropriate 
     * number of entries.
     */
    private void defineFitStrategyStepCombo( int steps )
    {   
        m_fitStrategyStepNumberCombo.removeAllItems( );
        
        for ( int j = 0 ; j < steps ; j++ )
        {
            m_fitStrategyStepNumberCombo.addItem( "Fit Strategy Step " + ( j + 1 ) );
        }
    }
    
    /**
     * Creates the ButtonPanel.
     */
    private void createButtonPanel( )
    {
        m_buttonPanel = new JPanel( );
        m_buttonPanel.setLayout( new BoxLayout( m_buttonPanel, BoxLayout.X_AXIS ) );
        m_buttonPanel.setBorder( BorderFactory.createCompoundBorder(
                                 BorderFactory.createTitledBorder( "Fit strategy step" ),
                                 BorderFactory.createEmptyBorder( 5, 5, 15, 5 ) ) ); 
        
        m_addStrategyStepButton = new JButton( "Add" );
        m_addStrategyStepButton.addActionListener( this );
        m_addStrategyStepButton.setToolTipText( "Adds a new fit strategy step to the list of strategy steps" );
        m_removeStrategyStepButton = new JButton( "Remove" );
        m_removeStrategyStepButton.addActionListener( this );
        m_removeStrategyStepButton.setToolTipText( "Removes the currently selected fit stategy step" );
        m_insertStrategyStepButton = new JButton( "Insert" );
        m_insertStrategyStepButton.setToolTipText( "Inserts a new fitstrategy step on the currently selected fit stategy step" );
        m_insertStrategyStepButton.addActionListener( this );
        
        m_buttonPanel.add( Box.createHorizontalGlue( ) );
        m_buttonPanel.add( m_addStrategyStepButton );
        m_buttonPanel.add( Box.createHorizontalGlue( ) );
        m_buttonPanel.add( m_removeStrategyStepButton );
        m_buttonPanel.add( Box.createHorizontalGlue( ) );
        m_buttonPanel.add( m_insertStrategyStepButton );
        m_buttonPanel.add( Box.createHorizontalGlue( ) );    
    }
    
    /**
     * Create a general offset to apply to model before fitting.
     */
    private void createGeneralOffsetDefinitionPanel( )
    {
        m_generalOffsetDefinitionPanel = new JPanel( );
        m_generalOffsetDefinitionPanel.setLayout( new BoxLayout( m_generalOffsetDefinitionPanel, BoxLayout.Y_AXIS ) );
        
        m_generalOffsetDefinitionPanel.setBorder( BorderFactory.createCompoundBorder(
                                                  BorderFactory.createTitledBorder( "Define global chemical shift." ),
                                                  BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
        
        m_globalOffsetDefintionButtonCollectionPanel = new JPanel( );
        m_globalOffsetDefintionButtonCollectionPanel.setLayout( new BoxLayout( m_globalOffsetDefintionButtonCollectionPanel, BoxLayout.X_AXIS ) );
        
        m_defineResonanceOfInterest = new JButton( "Define resonance of interest" );
        m_defineResonanceOfInterest.addActionListener(this);
        m_defineResonanceOfInterest.setEnabled( false );
        
        m_correctLocationOfResonanceOfInterest = new JButton( "Define correct location of resonance of interest" );
        m_correctLocationOfResonanceOfInterest.addActionListener( this );
        m_correctLocationOfResonanceOfInterest.setEnabled( false );
        
        m_globalOffsetDefintionButtonCollectionPanel.add( Box.createHorizontalGlue( ) );
        m_globalOffsetDefintionButtonCollectionPanel.add( m_defineResonanceOfInterest );
        m_globalOffsetDefintionButtonCollectionPanel.add( Box.createHorizontalGlue( ) );
        m_globalOffsetDefintionButtonCollectionPanel.add( m_correctLocationOfResonanceOfInterest );
        m_globalOffsetDefintionButtonCollectionPanel.add( Box.createHorizontalGlue( ) );
        
        // Actual offset frequency definition panel
        m_actualFrequencyDefinitionPanel = new JPanel( new GridLayout( 1,2 ) );
        m_actualFrequencyDefinitionLabel = new JLabel( "Actual offset of reference : " );
        m_actualFrequencyDefinitionValue = new JLabel( "" + m_actualOffsetValue );
        
        //m_actualFrequencyDefinitionPanel.add( Box.createHorizontalGlue( ) );
        m_actualFrequencyDefinitionPanel.add( m_actualFrequencyDefinitionLabel );
        //m_actualFrequencyDefinitionPanel.add( Box.createHorizontalGlue( ) );
        m_actualFrequencyDefinitionPanel.add( m_actualFrequencyDefinitionValue );
        //m_actualFrequencyDefinitionPanel.add( Box.createHorizontalGlue( ) );
        
        // Expected offset frequency definition panel
        m_expectedOffsetFrequencyDefinitionPanel = new JPanel( new GridLayout( 1,2 ) );
        m_expectedOffsetFrequencyDefinitionLabel = new JLabel( "Expected offset of reference : " );
        m_expectedOffsetFrequencyDefinitionValue = new JLabel( "" + m_expectedOffsetValue );
        
        m_expectedOffsetFrequencyDefinitionPanel.add( m_expectedOffsetFrequencyDefinitionLabel );
        m_expectedOffsetFrequencyDefinitionPanel.add( m_expectedOffsetFrequencyDefinitionValue );
        
        
        m_globalOffsetValueToApplyDefinitionPanel = new JPanel( new GridLayout( 1,2 ) );
        m_globalOffsetValueToApplyDefinitionLabel = new JLabel( "Global offset to apply : " );
        m_globalOffsetValueToApplyDefinitionValue = new JLabel( "" + m_globalOffsetValueToApply );
        
        m_globalOffsetValueToApplyDefinitionPanel.add( m_globalOffsetValueToApplyDefinitionLabel );
        m_globalOffsetValueToApplyDefinitionPanel.add( m_globalOffsetValueToApplyDefinitionValue );
        
        // Add all the panels together...
        m_generalOffsetDefinitionPanel.add( m_globalOffsetDefintionButtonCollectionPanel );
        m_generalOffsetDefinitionPanel.add( m_actualFrequencyDefinitionPanel );
        m_generalOffsetDefinitionPanel.add( m_expectedOffsetFrequencyDefinitionPanel );
        m_generalOffsetDefinitionPanel.add( m_globalOffsetValueToApplyDefinitionPanel );
        
        m_generalOffsetDefinitionPanel.setEnabled( false );
        
    }
    
    /**
     * Creates the ranges panel.
     */
    private void createRangesPanel( )
    {
        m_fitStrategyRangesPanel = new JPanel( );
        m_fitStrategyRangesPanel.setLayout( new BoxLayout( m_fitStrategyRangesPanel, BoxLayout.X_AXIS ) );
        
        m_fitStrategyRangesPanel.setBorder( BorderFactory.createCompoundBorder(
                                            BorderFactory.createTitledBorder( "Ranges" ),
                                            BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
        

        Vector<String> columnNames = new Vector<String>( );
        
        columnNames.add( "minimum" );
        columnNames.add( "maximum" );
        
        m_minimumMaximumTableModel = new DefaultTableModel( );
        
        m_minimumMaximumTableModel.addColumn( "Interval Number" );
        m_minimumMaximumTableModel.addColumn( "Minimum [pnts]" );
        m_minimumMaximumTableModel.addColumn( "Maximum [pnts]" );

        m_minimumMaximumIntervals = new JTable( m_minimumMaximumTableModel )
        {
            public boolean isCellEditable(int rowIndex, int vColIndex) 
            {
                return false;
            }
            
            // For proper serialization
            private static final long serialVersionUID = 1482325812865123L;
        };
        
        m_minimumMaximumIntervals.setColumnSelectionAllowed( false );
        m_minimumMaximumIntervals.setRowSelectionAllowed( true );
        m_minimumMaximumIntervals.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        
        setSelectedRow( 0 );
        
        JScrollPane m_tableScrollPane = new JScrollPane( );
        m_tableScrollPane.getViewport( ).setView( m_minimumMaximumIntervals );        
        m_tableScrollPane.setPreferredSize( new Dimension( 270, 200 ) );
        m_tableScrollPane.setPreferredSize( new Dimension( 270, 200 ) );
        m_tableScrollPane.setPreferredSize( new Dimension( 270, 200 ) );
        
        m_minimumMaximumIntervals.setPreferredSize( new Dimension( 260, 175 ) );
        m_minimumMaximumIntervals.setMinimumSize( new Dimension( 260, 175 ) );
        m_minimumMaximumIntervals.setMaximumSize( new Dimension( 260, 175 ) );

        JPanel buttonPanel = new JPanel( );
        
        buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.Y_AXIS ) );
        
        m_addIntervalButton = new JButton( "Add" );
        m_addIntervalButton.addActionListener( this );
        m_addIntervalButton.setPreferredSize( new Dimension( 70, 25 ) );
        m_addIntervalButton.setMaximumSize( new Dimension( 70, 25 ) );
        m_addIntervalButton.setMinimumSize( new Dimension( 70, 25 ) );
        
        m_removeIntervalButton = new JButton( "Delete" );
        m_removeIntervalButton.addActionListener( this );
        m_removeIntervalButton.setPreferredSize( new Dimension( 70, 25 ) );
        m_removeIntervalButton.setMinimumSize( new Dimension( 70, 25 ) );
        m_removeIntervalButton.setMaximumSize( new Dimension( 70, 25 ) );
        
        m_insertIntervalButton = new JButton( "Insert" );
        m_insertIntervalButton.addActionListener( this );
        
        m_insertIntervalButton.setPreferredSize( new Dimension( 70, 25 ) );
        m_insertIntervalButton.setMinimumSize( new Dimension( 70, 25 ) );
        m_insertIntervalButton.setMaximumSize( new Dimension( 70, 25 ) );
        
        m_editIntervalButton = new JButton( "Edit" );
        m_editIntervalButton.addActionListener( this );
        m_editIntervalButton.setPreferredSize( new Dimension( 70, 25 ) );
        m_editIntervalButton.setMinimumSize( new Dimension( 70, 25 ) );
        m_editIntervalButton.setMaximumSize( new Dimension( 70, 25 ) );
        
        buttonPanel.add( m_editIntervalButton );
        buttonPanel.add( m_addIntervalButton );
        buttonPanel.add( m_removeIntervalButton );
        buttonPanel.add( m_insertIntervalButton );
        
        buttonPanel.setBorder( BorderFactory.createEmptyBorder( 5, 20, 5, 20 ) );
        m_fitStrategyRangesPanel.add( m_tableScrollPane );
        m_fitStrategyRangesPanel.add( buttonPanel );
    }
    
    
    /**
     * Called when registered object raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {   
        Object sender = e.getSource( );
        
        if ( sender == m_defineResonanceOfInterest )
        {
            notifyActionListeners( this, DEFINE_GLOBAL_OFFSET_ANCHOR_EVENT, "Define where resonance should be" );
        }
        
        if ( sender == m_correctLocationOfResonanceOfInterest )
        {
            notifyActionListeners( this, DEFINE_GLOBAL_OFFSET_EVENT, "Define where resonance is actually now" );
        }
        
        if ( sender == m_fitStrategyStepNumberCombo )
        {
            if ( m_fitStrategyStepNumberCombo.getItemCount( ) > 0 )
            {
                setGUIValues( m_fitStrategyStepNumberCombo.getSelectedIndex( ) );
                setSelectedRow( 0 );
            }
        }
        
        if ( ( sender == m_fitArea ) || ( sender == m_fitOffset ) || 
             ( sender == m_fitGaussWidth ) || ( sender == m_fitPhase ) || 
             ( sender == m_fitLorentz ) || ( sender == m_applyConvolution ) 
             || ( sender == m_applyLineShapeFit ) )
        {
            getGUIValues( );
        }
        
        if ( sender == m_timeDomainTruncationPoint )
        {
             m_currentStrategyStep.setTruncateAfterPointNumber( (int)m_timeDomainTruncationPoint.getTextFieldValue( ) );
        }
        
        if ( sender == m_timeDomainIgnoreFirstPoints )
        {
             m_currentStrategyStep.setIgnoreFirstSamplePointsNumber( (int)m_timeDomainIgnoreFirstPoints.getTextFieldValue( ) );
        }
        
        if ( sender == m_numberOfIterations )
        {
            m_currentStrategyStep.setNumberOfIterations( (int)m_numberOfIterations.getTextFieldValue( ) );
        }
        
        if ( sender == m_addStrategyStepButton )
        {
            MRFitStrategyStep fitStratStep = new MRFitStrategyStep( );
            MRFitStrategy mrstrat = MRFitStrategy.getInstance( );
            mrstrat.addFitStrategyStep( fitStratStep );
            
            defineFitStrategyStepCombo( mrstrat.getNumberOfFitStrategySteps( ) );
            
            setGUIValues( mrstrat.getNumberOfFitStrategySteps( ) - 1 );
        }
        
        if ( sender == m_removeStrategyStepButton )
        {
            MRFitStrategy mrstrat = MRFitStrategy.getInstance( );
            mrstrat.removeFitStrategyStep( m_fitStrategyStepNumberCombo.getSelectedIndex( ) );
            defineFitStrategyStepCombo( mrstrat.getNumberOfFitStrategySteps( ) );
            setGUIValues( 0 );
        }
        
        if ( sender == m_insertStrategyStepButton )
        {
            MRFitStrategy mrstrat = MRFitStrategy.getInstance( );
            MRFitStrategyStep fitStratStep = new MRFitStrategyStep( );
            int insertAt = m_fitStrategyStepNumberCombo.getSelectedIndex( );
            mrstrat.insertFitStrategyStep( insertAt, fitStratStep );
            defineFitStrategyStepCombo( mrstrat.getNumberOfFitStrategySteps( ) );
            setGUIValues( insertAt );
        }
        
        if ( sender == m_addIntervalButton )
        {
            m_rangeEditMode = ADD_RANGE_INTERVAL_MODE;
            MRSignalProperties cond = MRSignalProperties.getInstance( );
           
            RangeDefinitionFrame rangeFrame = new RangeDefinitionFrame( "Define frequency range", m_parentFrame, 0f, cond.getNumberOfSamplePoints( ), "0" );
            rangeFrame.addActionListener( this );  
        }
        
        if ( sender == m_editIntervalButton )
        {
            m_rangeEditMode = EDIT_RANGE_INTERVAL_MODE;
            MRSignalProperties cond = MRSignalProperties.getInstance( );
            
            // Get a reference to the current 
            MRFitStrategy mrFitStrat = MRFitStrategy.getInstance( );
            MRFitStrategyStep mrFitStratStep = mrFitStrat.getFitStrategyStep( this.m_currentStrategyNumber ); 
            int min = mrFitStratStep.getFittingRangeMinimum( m_currentIntervalNumber );
            int max = mrFitStratStep.getFittingRangeMaximum( m_currentIntervalNumber );
            
            RangeDefinitionFrame rangeFrame = new RangeDefinitionFrame( "Edit frequency range", m_parentFrame, (float)0 , cond.getNumberOfSamplePoints( ) , "0" );
            rangeFrame.addActionListener( this );
        }
        
        if ( sender == m_insertIntervalButton )
        {
            m_rangeEditMode = INSERT_RANGE_INTERVAL_MODE;
            MRSignalProperties cond = MRSignalProperties.getInstance( );
           
            RangeDefinitionFrame rangeFrame = new RangeDefinitionFrame( "Define frequency range", m_parentFrame, 0f, cond.getNumberOfSamplePoints( ), "0" );
            rangeFrame.addActionListener( this );
            
            // Update the GUI
             this.setGUIValues( m_currentStrategyNumber );
            
        }
        
        if ( sender == this.m_removeIntervalButton )
        {
            
            m_currentIntervalNumber = m_minimumMaximumIntervals.getSelectedRow( );
            // Update the the model                                   
            MRFitStrategy mrFitStrat = MRFitStrategy.getInstance( );
            mrFitStrat.getFitStrategyStep( this.m_currentStrategyNumber ).removeFittingRange( m_currentIntervalNumber );
             
             // Update the GUI
             this.setGUIValues( m_currentStrategyNumber );
        }
        
        if ( sender instanceof RangeDefinitionFrame )
        {
            if ( e.getID( ) == RangeDefinitionFrame.ACCEPT_EVENT )
            {
                RangeDefinitionFrame rangeDefiner = (RangeDefinitionFrame)e.getSource( );
                int max = (int)rangeDefiner.getCurrentMaximum( );
                int min = (int)rangeDefiner.getCurrentMinimum( );

                
                if ( m_rangeEditMode == this.EDIT_RANGE_INTERVAL_MODE )
                {
                    m_currentIntervalNumber = m_minimumMaximumIntervals.getSelectedRow( );
                    
                    // Update the the model                                   
                    MRFitStrategy mrFitStrat = MRFitStrategy.getInstance( );
                    mrFitStrat.getFitStrategyStep( this.m_currentStrategyNumber ).removeFittingRange( m_currentIntervalNumber  );
                    mrFitStrat.getFitStrategyStep( this.m_currentStrategyNumber ).insertFittingRange( m_currentIntervalNumber, min, max );
                    
                    // Update the GUI
                    this.setGUIValues( m_currentStrategyNumber );
                    
                    m_rangeEditMode = NOT_RANGE_INTERVAL_MODE;                                
                }
                
                if ( m_rangeEditMode == ADD_RANGE_INTERVAL_MODE )
                {
                    m_minimumMaximumTableModel.addRow( new Object[] { "" + ( m_minimumMaximumTableModel.getRowCount( ) + 1 ), 
                                                       "" + min, "" + max } );
                    // Also update the model                                   
                    MRFitStrategy mrFitStrat = MRFitStrategy.getInstance( );
                    mrFitStrat.getFitStrategyStep( this.m_currentStrategyNumber ).addFittingRange( min, max );                                                       
                    
                    setSelectedRow( m_minimumMaximumIntervals.getRowCount( ) - 1 );
                    m_rangeEditMode = NOT_RANGE_INTERVAL_MODE;                                
                }
                
                if ( m_rangeEditMode == INSERT_RANGE_INTERVAL_MODE )
                {
                    m_minimumMaximumTableModel.insertRow( m_minimumMaximumIntervals.getSelectedRow( ), ( new Object[] { "" + ( m_minimumMaximumTableModel.getRowCount( ) + 1 ), 
                                                       "" + min, "" + max } ) );
                    // Also update the model                                   
                    MRFitStrategy mrFitStrat = MRFitStrategy.getInstance( );
                    
                    m_currentIntervalNumber= m_minimumMaximumIntervals.getSelectedRow( );
                    
                    mrFitStrat.getFitStrategyStep( this.m_currentStrategyNumber ).insertFittingRange( m_currentIntervalNumber, min, max );

                    m_rangeEditMode = NOT_RANGE_INTERVAL_MODE;
                }
            }
        }
    }

    /**
     * Adds an ActionListener to the list of listeners.
     * @param listener
     */
    public void addActionListener( ActionListener listener )
    {
        m_actionListeners.add( listener );
    }
    
    /**
     * Notifies registered ActionListeners.
     */
    private void notifyActionListeners( Object obj, int eventType, String actionCommand )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, actionCommand ) );
        }
    }
    
    
    
    /**
     * Selects a row in a JTable, not very intuitive but works ...
     */
    private void setSelectedRow( int selectRowNumber )
    {
        if ( m_minimumMaximumIntervals != null )
        {
            if ( m_minimumMaximumIntervals.getRowCount( ) > 0 )
            {
                m_minimumMaximumIntervals.removeRowSelectionInterval( 0, m_minimumMaximumIntervals.getRowCount( ) - 1 ); 
                m_minimumMaximumIntervals.addRowSelectionInterval( selectRowNumber, selectRowNumber ); 
            }
        }
    }
    
    /**
     * Is called when ever a registered component raises an ItemEvent.
     * @param itemEvent
     */
    @Override
    public void itemStateChanged( ItemEvent itemEvent ) 
    {
        Object sender = itemEvent.getSource( );
        
        if ( sender == m_fitAlgorithmToApplyCombo )
        {
            if ( m_currentStrategyStep != null )
            {
                if ( m_fitAlgorithmToApplyCombo.getSelectedIndex( ) == 0 )
                {
                    m_currentStrategyStep.setFitAlgorithType( MRFitStrategyStep.RANDOMN );
                }
                else
                {
                    m_currentStrategyStep.setFitAlgorithType( MRFitStrategyStep.CONJUGATE_GRADIENTS );
                }  
            }
        }
        
        if ( sender == m_fitChannelTypeCombo )
        {
            if ( m_currentStrategyStep != null )
            {
                if ( m_fitChannelTypeCombo.getSelectedIndex( ) == 0 )
                {
                    m_currentStrategyStep.setFittingType( MRFitStrategyStep.ABSORPTION_CHANNEL_FIT );
                }
                if ( m_fitChannelTypeCombo.getSelectedIndex( ) == 1 )
                {
                    m_currentStrategyStep.setFittingType( MRFitStrategyStep.DISPERSION_CHANNEL_FIT );
                }
                if ( m_fitChannelTypeCombo.getSelectedIndex( ) == 2 )
                {
                    m_currentStrategyStep.setFittingType( MRFitStrategyStep.COMPLEX_FIT );
                }
                if ( m_fitChannelTypeCombo.getSelectedIndex( ) == 3 )
                {
                    m_currentStrategyStep.setFittingType( MRFitStrategyStep.MAGNITUDE_FIT );
                }
                if ( m_fitChannelTypeCombo.getSelectedIndex( ) == 4 )
                {
                    m_currentStrategyStep.setFittingType( MRFitStrategyStep.POWER_FIT );
                }
            }
        }
        
        if ( sender == m_fitWeightingTypeCombo )
        {
            if ( m_currentStrategyStep != null )
            {
                m_currentStrategyStep.setLeastSquaresWeightingType(  m_fitWeightingTypeCombo.getSelectedIndex( ) );
            }
        }
    }
    
    /**
     * Sets the expected offset frequency of reference component in ppm
     * @param m_lastSelectedOffsetValue 
     */
    void setExpectedOffsetValue( float lastSelectedOffsetValue ) 
    {
        m_expectedOffsetValue = lastSelectedOffsetValue;
        m_globalOffsetValueToApply = m_expectedOffsetValue - m_actualOffsetValue;
        
        m_expectedOffsetFrequencyDefinitionValue.setText( "" + m_expectedOffsetValue + "  ppm" );
        m_globalOffsetValueToApplyDefinitionValue.setText ( "" + m_globalOffsetValueToApply + "  ppm" );
    }
    
    /**
     * Sets the expected offset frequency of reference component in ppm
     * @param m_lastSelectedOffsetValue 
     */
    void setActualOffsetValue( float lastSelectedOffsetValue ) 
    {
        m_actualOffsetValue = lastSelectedOffsetValue;
        m_globalOffsetValueToApply = m_expectedOffsetValue - m_actualOffsetValue;
        m_actualFrequencyDefinitionValue.setText( "" + m_actualOffsetValue + "  ppm" );
        m_globalOffsetValueToApplyDefinitionValue.setText ( "" + m_globalOffsetValueToApply  + "  ppm");
    }
    
    
    
    
    // Attributes 
    JFrame m_parentFrame;
    MRFitStrategyStep m_currentStrategyStep;
    
    int m_currentStrategyNumber = 1;
    int m_currentIntervalNumber = 0;
    
    JComboBox m_fitStrategyStepNumberCombo;
    
    JPanel m_collectionPanel;
    JPanel m_fitStategyStepNumberSelecionPanel;
    
    JPanel m_fitWhichParameterGroupsPanel;
    JPanel m_fitStrategyOptionsPanel;
    JPanel m_fitStrategyRangesPanel;
    JPanel m_generalOffsetDefinitionPanel;
    JPanel m_buttonPanel;
    JButton m_defineResonanceOfInterest;
    JButton m_correctLocationOfResonanceOfInterest;
    
    JCheckBox m_fitArea;
    JCheckBox m_fitOffset;
    JCheckBox m_fitGaussWidth;
    JCheckBox m_fitPhase;
    JCheckBox m_fitLorentz;
    
    JComboBox m_fitAlgorithmToApplyCombo;
    JComboBox m_fitChannelTypeCombo;
    
    JComboBox m_fitWeightingTypeCombo;
    
    JCheckBox m_applyConvolution;
    JCheckBox m_applyLineShapeFit;
    
    SpinBox m_numberOfIterations;
    SpinBox m_timeDomainTruncationPoint;
    SpinBox m_timeDomainIgnoreFirstPoints;
    
    JTable m_minimumMaximumIntervals;
    DefaultTableModel m_minimumMaximumTableModel;
    
    JButton m_addIntervalButton;
    JButton m_removeIntervalButton;
    JButton m_insertIntervalButton;
    JButton m_editIntervalButton;

    JButton m_addStrategyStepButton;
    JButton m_removeStrategyStepButton;
    JButton m_insertStrategyStepButton;
    
    JPanel m_globalOffsetDefintionCollectionPanel;
    
    // The following two values determine the default global offset of the
    // model: the default difference of the two values is 0.0, so the default 
    // tdfdfit model is assumed to apply ...
    private float m_expectedOffsetValue         = 0.0f;
    private float m_actualOffsetValue           = 0.0f;
    private float m_globalOffsetValueToApply    = 0.0f;
    
    private int m_rangeEditMode = NOT_RANGE_INTERVAL_MODE;
    
    public final static int NOT_RANGE_INTERVAL_MODE = -1;
    
    public final static int ADD_RANGE_INTERVAL_MODE = 0;
    public final static int EDIT_RANGE_INTERVAL_MODE = 1;
    public final static int INSERT_RANGE_INTERVAL_MODE = 2;
    
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<ActionListener>( );
    
    // The anchor envokes the definition of the offset location where the resonance
    // is expected to be or should be.
    public final static int DEFINE_GLOBAL_OFFSET_ANCHOR_EVENT = 0;
    // The anchor envokes the definition of the offset location where the resonance
    // is currently found in the measurement.
    public final static int DEFINE_GLOBAL_OFFSET_EVENT = 1;
    
    JPanel m_globalOffsetDefintionButtonCollectionPanel;
    
    JPanel m_actualFrequencyDefinitionPanel;
    JLabel m_actualFrequencyDefinitionLabel;
    JLabel m_actualFrequencyDefinitionValue;
    
    JPanel m_expectedOffsetFrequencyDefinitionPanel;
    JLabel m_expectedOffsetFrequencyDefinitionLabel;
    JLabel m_expectedOffsetFrequencyDefinitionValue;
   
    JPanel m_globalOffsetValueToApplyDefinitionPanel;
    JLabel m_globalOffsetValueToApplyDefinitionLabel;
    JLabel m_globalOffsetValueToApplyDefinitionValue;        
    // For proper serialization
    private static final long serialVersionUID = 6482325898865123L;
}
