/*
 * File: $Id: MRFitStrategyStep.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 08.12.2003, 10:29  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;

import java.io.RandomAccessFile;
import java.io.IOException;

import java.util.Vector;


/**
 *
 * Class containing the attributes of a tdfdfit fit strategy step
 * Simple container class with setters and getter functionality
 *
 * @author  hans
 */

public class MRFitStrategyStep 
{
    
    /** 
     * Creates a new instance of MRFitStrategyStep 
     */
    public MRFitStrategyStep( )
    {
        m_fitAlgorithmType = CONJUGATE_GRADIENTS;
        
        MRSignalProperties cond = MRSignalProperties.getInstance( );
        m_truncateTDSignalAfterPointNumber = cond.getNumberOfSamplePoints( );
        
        m_fitArea = true;
        m_fitOffset = true;
        m_fitGauss = true;
        m_fitPhase = true;
        m_fitLorentz = true;
        
        m_applyLineShapeFit = false;
        m_leastSquaresWeightingType = UNIFORM_LS_WEIGHTING;
        m_applyConvolution = false;
        m_numberOfIterations = 100;
        m_truncateTDSignalAfterPointNumber = MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
        m_fittingType = COMPLEX_FIT;
        
        m_minimumRangeValue = new Vector<Integer>( );
        m_maximumRangeValue = new Vector<Integer>( );
        
        m_fittingRanges = 1;
        
        m_minimumRangeValue.add( new Integer( (int)( ( 470./1024. ) * cond.getNumberOfSamplePoints( ) ) ) );
        m_maximumRangeValue.add( new Integer( (int)( ( 800./1024. ) * cond.getNumberOfSamplePoints( ) ) ) );
        
    }
    
    /** 
     * Creates a new instance of MRFitStrategyStep 
     * @param fitArea
     * @param fitOffset
     * @param fitGauss
     * @param fitPhase
     * @param fitLorentz
     */
    public MRFitStrategyStep( boolean fitArea, boolean fitOffset, boolean fitGauss, boolean fitPhase, boolean fitLorentz )
    {
        m_fitAlgorithmType = CONJUGATE_GRADIENTS;
        
        MRSignalProperties cond = MRSignalProperties.getInstance( );
        m_truncateTDSignalAfterPointNumber = cond.getNumberOfSamplePoints( );
        m_ignoreFirstSamplePointsNumber = 0;
        
        m_fitArea = fitArea;
        m_fitOffset = fitOffset;
        m_fitGauss = fitGauss;
        m_fitPhase = fitPhase;
        m_fitLorentz = fitLorentz;
        
        m_applyLineShapeFit = false;
        m_leastSquaresWeightingType = UNIFORM_LS_WEIGHTING;
        m_applyConvolution = false;
        m_numberOfIterations = 100;
        m_truncateTDSignalAfterPointNumber = MRSignalProperties.getInstance( ).getNumberOfSamplePoints( );
        m_fittingType = COMPLEX_FIT;
        
        m_minimumRangeValue = new Vector<Integer>( );
        m_maximumRangeValue = new Vector<Integer>( );
        
        m_minimumRangeValue.add( new Integer( (int)( ( 470./1024. ) * cond.getNumberOfSamplePoints( ) ) ) );
        m_maximumRangeValue.add( new Integer( (int)( ( 800./1024. ) * cond.getNumberOfSamplePoints( ) ) ) );
        
        m_fittingRanges = 1;
    }
    

    /**
     * Sets the default time domain signal truncation point
     * @param ignore
     */
    public void setIgnoreFirstSamplePointsNumber( int ignore )
    {
        m_ignoreFirstSamplePointsNumber = ignore;
    }        
            
    /**
     * Returns the default time domain signal truncation point
     * @return 
     */
    public int getIgnoreFirstSamplePointsNumber( )
    {
        return m_ignoreFirstSamplePointsNumber;
    }    
           
    /**
     * Set the fitting algorithm type
     * @param fitType
     */
    public void setFitAlgorithType( int fitType )
    {
        m_fitAlgorithmType = fitType;
    }

    /**
     * Get the fitting algorithm type
     * @return 
     */
    public int getFitAlgorithType( )
    {
        return m_fitAlgorithmType;
    }

    /**
     * Sets the truncation point number
     * @param truncate
     */
    public void setTruncateAfterPointNumber( int truncate )
    {
        m_truncateTDSignalAfterPointNumber = truncate;
    }

    /**
     * Get the truncation point number
     * @return 
     */
    public int getTruncateAfterPointNumber( )
    {
        return m_truncateTDSignalAfterPointNumber;
    }

    /**
     * Set the area parameter group to be fitted
     * @param fitArea
     */
    public void setFitArea( boolean fitArea )
    {
        m_fitArea = fitArea;
    }
    
    /**
     * Set the area parameter group to be fitted
     * @return 
     */
    public boolean getFitArea( )
    {
        return m_fitArea;
    }
    
    /**
     * Set the offset parameter group to be fitted
     * @param fitOffset
     */
    public void setFitOffset( boolean fitOffset )
    {
        m_fitOffset = fitOffset;
    }
    
    /**
     * Set the offset parameter group to be fitted
     * @return 
     */
    public boolean getFitOffset( )
    {
        return m_fitOffset;
    }

    /**
     * Set the gauss parameter group to be fitted
     * @param fitGauss
     */
    public void setFitGauss( boolean fitGauss )
    {
        m_fitGauss = fitGauss;
    }
    
    /**
     * Set the gauss parameter group to be fitted
     * @return 
     */
    public boolean getFitGauss( )
    {
        return m_fitGauss;
    }

    /**
     * Set the phase parameter group to be fitted
     * @param fitPhase
     */
    public void setFitPhase( boolean fitPhase )
    {
        m_fitPhase = fitPhase;
    }
    
    /**
     * Set the lorentz parameter group to be fitted
     * @return 
     */
    public boolean getFitPhase( )
    {
        return m_fitPhase;
    }
    
    /**
     * Set the lorentz parameter group to be fitted
     * @param fitLorentz
     */
    public void setFitLorentz( boolean fitLorentz )
    {
        m_fitLorentz = fitLorentz;
    }
    
    /**
     * Set the lorentz parameter group to be fitted
     * @return 
     */
    public boolean getFitLorentz( )
    {
        return m_fitLorentz;
    }
    
    /**
     * Set apply line shape fit parameter; if set to true it applies 
     * a line shape fit
     * @param doLineShapeFit
     */
    public void setApplyLineShapeFit( boolean doLineShapeFit )
    {
        m_applyLineShapeFit = doLineShapeFit;
    }
    
    /**
     * Returns whether additional line shape fitting should be applied
     * @return 
     */
    public boolean getApplyLineShapeFit( )
    {
        return m_applyLineShapeFit;
    }
    
    /**
     * Sets the least squares weighting type
     * @param weightingType
     */ 
    public void setLeastSquaresWeightingType( int weightingType )
    {
        m_leastSquaresWeightingType = weightingType;
    }
    
    /**
     * Gets the least squares weighting type
     * @return 
     */ 
    public int getLeastSquaresWeightingType( )
    {
        return m_leastSquaresWeightingType;
    }
    
    /**
     * Sets the least squares weighting type
     * @param apply
     */ 
    public void setApplyConvolution( boolean apply )
    {
        m_applyConvolution = apply;
    }

    /**
     * Gets the least squares weighting type
     * @return 
     */ 
    public boolean getApplyConvolution( )
    {
        return m_applyConvolution;
    }
    
    /**
     * Sets the least squares weighting type
     * @param iterNumber
     */ 
    public void setNumberOfIterations( int iterNumber )
    {
        m_numberOfIterations = iterNumber;
    }

    /**
     * Gets the least squares weighting type
     * @return 
     */ 
    public int getNumberOfIterations( )
    {
        return m_numberOfIterations;
    }
    
    /**
     * Set fitting type
     * @param fittingType
     */
    public void setFittingType( int fittingType )
    {
        m_fittingType = fittingType;
    }
    
    /**
     * Set fitting type
     * @return 
     */
    public int getFittingType( )
    {
        return m_fittingType;
    }
    
    /**
     * Adds a new fitting range to the vector of ranges
     * @param minimum
     * @param maximum
     */
    public void addFittingRange( int minimum, int maximum )
    {
        // add checks has to be added; leave it out for the time being for
        // time reasons
        m_minimumRangeValue.add( new Integer( minimum ) );
        m_maximumRangeValue.add( new Integer( maximum ) );
        
        m_fittingRanges = m_minimumRangeValue.size( );
        
    }
    
    /**
     * Adds a new fitting range to the vector of ranges
     * @param index
     * @param minimum
     * @param maximum
     */
    public void insertFittingRange( int index, int minimum, int maximum )
    {
        // add checks has to be added; leave it out for the time being for
        // time reasons
        m_minimumRangeValue.insertElementAt( new Integer( minimum ), index );
        m_maximumRangeValue.insertElementAt( new Integer( maximum ), index );
        
        m_fittingRanges = m_minimumRangeValue.size( );
    }
    
    /**
     * Removes fitting range index
     * @param index
     */
    public void removeFittingRange( int index )
    {
        // add checks has to be added; leave it out for the time being for
        // time reasons
        m_minimumRangeValue.removeElementAt( index );
        m_maximumRangeValue.removeElementAt( index );
        
        m_fittingRanges = m_minimumRangeValue.size( );
        
    }
    
    /**
     * Deletes all the previously defined fitting ranges
     */
    public void deleteAllFittingRanges( )
    {
        m_minimumRangeValue.clear( );
        m_maximumRangeValue.clear( );
    }
    
    /**
     * Returns the maximum of fitting range "number"
     * @param number
     * @return 
     */
    public int getFittingRangeMaximum( int number )
    {
        return m_maximumRangeValue.get( number ).intValue( );
    }

    /**
     * Returns the minimum of fitting range "number"
     * @param number
     * @return 
     */
    public int getFittingRangeMinimum( int number )
    {
        return m_minimumRangeValue.get( number ).intValue( );
    }

    /**
     * Returns the maximum of fitting range "number"
     * @param number
     * @param maximum
     */
    public void setFittingRangeMaximum( int number, int maximum )
    {
        m_maximumRangeValue.set( number, new Integer( maximum ) );
    }

    /**
     * Returns the minimum of fitting range "number"
     * @param number
     * @param minimum
     */
    public void setFittingRangeMinimum( int number, int minimum )
    {
        m_minimumRangeValue.set( number, new Integer( minimum ) );
    }
    
    /**
     * Deletes the range number "number" from the list of ranges
     * @param number
     */
    public void deleteFittingRange( int number )
    {
        m_minimumRangeValue.remove( number );
        m_maximumRangeValue.remove( number );
        m_fittingRanges = m_minimumRangeValue.size( );
    }
    
    /**
     * Returns the number of ranges
     * @return 
     */
    public int getNumberOfRanges( )
    {
        if ( m_maximumRangeValue != null )
        {
            return this.m_maximumRangeValue.size();
        }
        else
        {
           return 0;
        }
    }
    
    /**
     * Read the attributes from a RandomAccesFile
     */
    void read( RandomAccessFile in )
    {
        // clear vector members profilactic
        m_minimumRangeValue.clear( );
        m_maximumRangeValue.clear( );
        
        try
        {
            // reads opening tag
            in.readUTF( );
            
            m_fitAlgorithmType = in.readInt( );
            
            m_truncateTDSignalAfterPointNumber = in.readInt( );
             m_ignoreFirstSamplePointsNumber   = in.readInt( );
             
            m_fitArea = in.readBoolean( );
            m_fitOffset = in.readBoolean( );
            m_fitGauss = in.readBoolean( );
            m_fitPhase = in.readBoolean( );
            m_fitLorentz = in.readBoolean( );
            
            m_applyLineShapeFit = in.readBoolean( );
            m_leastSquaresWeightingType = in.readInt( );
            m_applyConvolution = in.readBoolean( );
            
            m_numberOfIterations = in.readInt( );
            m_fittingType = in.readInt( );
            m_fittingRanges = in.readInt( );
            
            for ( int i = 0; i < m_fittingRanges; i++ )
            {
                m_minimumRangeValue.add( new Integer( in.readInt( ) ) );
                m_maximumRangeValue.add( new Integer( in.readInt( ) ) );
            }
            
            m_fittingRanges = m_minimumRangeValue.size( );
            
            // reads closing tag
            in.readUTF( );
        }
        catch ( IOException e )
        {

        }
    }
    
    /**
     * Writes attributes to RandomAccessFile
     */
    void write( RandomAccessFile out )
    {
        // try to write all essential attributes to file
        try
        {
            out.writeUTF( "<MRFitStrategyStep>" );
            
            out.writeInt( m_fitAlgorithmType );
            out.writeInt( m_truncateTDSignalAfterPointNumber );
            out.writeInt( m_ignoreFirstSamplePointsNumber );
            
            out.writeBoolean( m_fitArea );
            out.writeBoolean( m_fitOffset );
            out.writeBoolean( m_fitGauss );
            out.writeBoolean( m_fitPhase );
            out.writeBoolean( m_fitLorentz );
            
            out.writeBoolean( m_applyLineShapeFit );
            out.writeInt( m_leastSquaresWeightingType );
            out.writeBoolean( m_applyConvolution );
            
            out.writeInt( m_numberOfIterations );
            out.writeInt( m_fittingType );
            out.writeInt( m_fittingRanges );
            
            for ( int i = 0; i < m_fittingRanges; i++ )
            {
                out.writeInt( m_minimumRangeValue.get( i ).intValue( ) );
                out.writeInt( m_maximumRangeValue.get( i ).intValue( ) );
            }
            
            out.writeUTF( "</MRFitStrategyStep>" );
        }
        catch ( IOException e )
        {
            
        }
    }
    
    
    // Defines which type of minimizer has to be used
    private int m_fitAlgorithmType;
    
    // Truncates time domain signal after point number
    private int m_truncateTDSignalAfterPointNumber;
    
    // Ignore the first m_ignore
    private int     m_ignoreFirstSamplePointsNumber;
    private boolean m_fitArea;
    private boolean m_fitOffset;
    private boolean m_fitGauss;
    private boolean m_fitPhase;
    private boolean m_fitLorentz;
    
    private boolean m_applyLineShapeFit;
    
    private int m_leastSquaresWeightingType;
    
    private boolean m_applyConvolution;
    
    private int m_numberOfIterations;
    
    // Indicates which channel that will be fitted
    private int m_fittingType;
    
    // Fitting ranges
    private int m_fittingRanges;
    private Vector<Integer> m_minimumRangeValue;
    private Vector<Integer> m_maximumRangeValue;
    
    public final static int UNIFORM_LS_WEIGHTING = 0;
    public final static int NON_UNIFORM_LS_WEIGHTING = 1;
    
    // Algorithm type to be used
    public final static int RANDOMN = 1;
    public final static int CONJUGATE_GRADIENTS = 2;
    
    // Fit type
    public final static int ABSORPTION_CHANNEL_FIT = 0;
    public final static int DISPERSION_CHANNEL_FIT = 1;
    public final static int COMPLEX_FIT = 2;
    public final static int MAGNITUDE_FIT = 3;
    public final static int POWER_FIT = 4;
    
    
}
