/*
 * File: $Id: MRMeasuredPatientSpectrumSeriesViewer.java,v 1.3 2006/11/08 12:18:40 hans Exp $
 * History
 * Date: 24.03.2005, 11:26  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2006 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * MRMeasuredPatientSpectrumSeriesViewer.java
 *
 * Created on March 24, 2005, 11:26 AM
 */ 

package org.spectrim.tdfdfit;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;
import java.awt.Rectangle;
import java.awt.Robot;

import java.awt.image.BufferedImage;


import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.InputEvent;

import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelListener;
import java.awt.event.MouseWheelEvent;

import java.io.File;
import java.io.IOException;



import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JFileChooser;

import java.text.NumberFormat;
import java.text.DecimalFormat;
import javax.imageio.ImageIO;

import java.awt.print.Book;
import java.awt.print.Printable;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import org.spectrim.dicom.DicomDirectory;

import org.spectrim.lib.SwingWorker;
import org.spectrim.lib.Dialogs;
import org.spectrim.model.GlobalModel;
import org.spectrim.xyplot.XYPlotRange;
import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotSlider;
import org.spectrim.xyplot.XYPlot;



/**
 * Class that allows viewing and manipulation of patient spectra.
 *
 * @author  hans
 */

public class MRMeasuredPatientSpectrumSeriesViewer extends SingleVoxelSpectrumSeriesViewer implements ActionListener, MouseListener, MouseMotionListener, Printable, MouseWheelListener
{
    
    /** 
     * Creates a new instance of MRMeasuredSpectrumModelViewer. 
     * @param title
     * @param parent
     * @param ddir
     */
    public MRMeasuredPatientSpectrumSeriesViewer( String title, JFrame parent, DicomDirectory ddir ) 
    {
        super( title, parent, ddir );
        
        m_parent = parent;
        m_dicomDir = ddir;
        
        // super.addActionListener( this );
        
        //addMouseListener( this );
        //addMouseMotionListener( this );
        addMouseWheelListener( this );
        
        // This class adds functionality to the popupmenu of the base classes
        addPopupMenuItems( );
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        if ( def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT ).compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
        {
            this.setDisplayXReversed( true ); // Call the base class function to indicate that a ppm scale is drawn reversed
        }
    }
    
    public void setGlobalModel( GlobalModel model )
    {
        m_globalModel = model;
    }
    
    /**
     * Adds entry point for user interactive spectrum model editing.
     */
    private void addPopupMenuItems( )
    {
        m_popupmenu.addSeparator( );
        
        m_saveScreenShot = new JMenuItem( "Save screenshot .." );
        m_saveScreenShot.setToolTipText( "Saves a copy of the currently displayed spectrum to a file" );
        m_saveScreenShot.addActionListener( this );
        
        m_popupmenu.add( m_saveScreenShot );
        
        m_permanentStorage = new JMenu( "Export to file" );
        
        m_storeDisplayedSignalOnly = new JMenuItem( "Currently displayed response" );
        m_storeDisplayedSignalOnly.addActionListener( this );
        m_storeDisplayedSignalOnly.setToolTipText( "Exports currently displayed signal only" );
        
        m_storeAllRepsonses = new JMenuItem( "All responses" );
        m_storeAllRepsonses.addActionListener( this );
        m_storeAllRepsonses.setToolTipText( "Exports all responses" );
        
        m_permanentStorage.add( m_storeDisplayedSignalOnly );
        m_permanentStorage.add( m_storeAllRepsonses );
        
        m_popupmenu.add( m_permanentStorage );
        
        m_popupmenu.addSeparator( );
        
        m_performSVSSpectrumStabilityTests = new JMenuItem( "Perform SVS Stability and Quality tests .." );
        m_performSVSSpectrumStabilityTests.addActionListener( this );
        m_performSVSSpectrumStabilityTests.setToolTipText( "Performs a battery of (statistic) tests to check measured dataset" );
        
        m_popupmenu.add( m_performSVSSpectrumStabilityTests );
               
        m_spectralQuantification = new JMenu( "Quantification");
        
        m_fitAllSpectraMenuItem = new JMenuItem( "Fit all spectra separate" );
        m_fitAllSpectraMenuItem.setToolTipText("Fits all spectra of the series using current model; can take a while ");
        m_fitAllSpectraMenuItem.addActionListener( this );
        
        m_spectralQuantification.add( m_fitAllSpectraMenuItem );
        
        m_fitAllSpectraWithOptionsMenuItem = new JMenuItem( "Fit all spectra separate with options .." );
        m_fitAllSpectraWithOptionsMenuItem.setToolTipText( "Fits all spectra of the series using current model amd additional filtering options");
        m_fitAllSpectraWithOptionsMenuItem.addActionListener( this );
        m_spectralQuantification.add( m_fitAllSpectraWithOptionsMenuItem );
        
        m_fitCurrentSpectrumMenuItem = new JMenuItem( "Fit currently displayed spectrum only" );
        m_fitCurrentSpectrumMenuItem.setToolTipText( "Fit currently displayed spectrum only with current model");
        m_fitCurrentSpectrumMenuItem.addActionListener( this );
        
        m_spectralQuantification.add( m_fitCurrentSpectrumMenuItem );
        
        m_fitSumOfSpectraMenuItem= new JMenuItem( "Fit sum of spectra .." );
        m_fitSumOfSpectraMenuItem.setToolTipText( "Sum a spectral number range and then fit with current model" );
        m_fitSumOfSpectraMenuItem.addActionListener( this );
        
        m_spectralQuantification.add( m_fitSumOfSpectraMenuItem );
        
        m_popupmenu.add( m_spectralQuantification );
        m_popupmenu.addSeparator( );
        
        m_displaySignalType = new JMenu( "Display" );
        
        m_displayOneInstanceOfPatientSpectrum = new JRadioButtonMenuItem( "Patient spectrum (one instance)" );
        m_displayOneInstanceOfPatientSpectrum.addActionListener(this);
        m_displayOneInstanceOfPatientSpectrum.setEnabled( false );
        m_displaySignalType.add( m_displayOneInstanceOfPatientSpectrum );
        
        m_displaySumOfUserDefinedInstancesOfPatientSpectrum = new JRadioButtonMenuItem( "Patient spectrum (user defined sum of instances)" );
        m_displaySumOfUserDefinedInstancesOfPatientSpectrum.addActionListener(this);
        m_displaySumOfUserDefinedInstancesOfPatientSpectrum.setEnabled( false );
        m_displaySignalType.add( m_displaySumOfUserDefinedInstancesOfPatientSpectrum );
        
        m_displaySumOfInstancesOfPatientSpectrum = new JRadioButtonMenuItem( "Patient spectrum (sum of all instances)" );
        m_displaySumOfInstancesOfPatientSpectrum.addActionListener(this);
        m_displaySumOfInstancesOfPatientSpectrum.setEnabled( false );
        m_displaySignalType.add( m_displaySumOfInstancesOfPatientSpectrum );
        
        m_displayIntegralOfFrequencyRangePlot = new JMenuItem( "Integral of offset range .. ");
        m_displayIntegralOfFrequencyRangePlot.addActionListener( this );
        m_displayIntegralOfFrequencyRangePlot.setToolTipText( "Displays integral of selected frequency range of all spectrum series instances" );
        
        m_displaySignalType.add( m_displayIntegralOfFrequencyRangePlot );
        
        m_displayIntegralOfFrequencyRangeBaselineCorrectedPlot = new JMenuItem( "Integral of offset range baseline corrected .. ");
        m_displayIntegralOfFrequencyRangeBaselineCorrectedPlot.addActionListener( this );
        m_displayIntegralOfFrequencyRangeBaselineCorrectedPlot.setToolTipText( "Displays integral of selected frequency range of all spectrum series instances; baseline is corrected by linear interpolation of endpoints" );
        
        m_displaySignalType.add( m_displayIntegralOfFrequencyRangeBaselineCorrectedPlot );
        
        
        m_displayAverageOfFrequencyRangePlot = new JMenuItem( "Average of offset range .. ");
        m_displayAverageOfFrequencyRangePlot.addActionListener( this );
        m_displayAverageOfFrequencyRangePlot.setToolTipText( "Displays average value of selected frequency range of all spectrum series instances" );
        
        m_displaySignalType.add( m_displayAverageOfFrequencyRangePlot );
        
        m_displayReference = new JRadioButtonMenuItem( "Patient Water reference spectrum" );         
        m_displayReference.addActionListener( this );
        m_displaySignalType.add( m_displayReference );
        m_displayReference.setEnabled( false );
        
        // add to the signal types to button group
        m_signalDisplaySelector.add( m_displayOneInstanceOfPatientSpectrum );
        m_signalDisplaySelector.add( m_displaySumOfInstancesOfPatientSpectrum );
        m_signalDisplaySelector.add( m_displayReference );
        
        m_printSpectrum = new JMenuItem( "Print" );
        m_printSpectrum.addActionListener( this );
        m_printSpectrum.setEnabled( false );
        m_popupmenu.add( m_printSpectrum );
        
        m_viewDicomHeader = new JMenu( "View DICOM header" );
        
        m_viewDicomHeaderSpectrum = new JMenuItem( "Spectrum" );
        m_viewDicomHeaderSpectrum.setEnabled( false );
        m_viewDicomHeaderSpectrum.addActionListener( this );

        m_viewDicomHeaderReference = new JMenuItem( "Water reference" );
        m_viewDicomHeaderReference.setEnabled( false );
        m_viewDicomHeaderReference.addActionListener( this );
        
        m_viewDicomHeader.add( m_viewDicomHeaderSpectrum );
        m_viewDicomHeader.add( m_viewDicomHeaderReference );
        
        m_viewSiemensSpecificDicomHeader = new JMenu( "View Siemens Specific DICOM header" );
        
        m_viewSiemensSpecificDicomHeaderSpectrum = new JMenuItem( "Spectrum" );
        m_viewSiemensSpecificDicomHeaderSpectrum.setEnabled( false );
        m_viewSiemensSpecificDicomHeaderSpectrum.addActionListener( this );

        m_viewSiemensSpecificDicomHeaderReference = new JMenuItem( "Water reference" );
        m_viewSiemensSpecificDicomHeaderReference.setEnabled( false );
        m_viewSiemensSpecificDicomHeaderReference.addActionListener( this );
        
        m_viewSiemensSpecificDicomHeader.add( m_viewSiemensSpecificDicomHeaderSpectrum );
        m_viewSiemensSpecificDicomHeader.add( m_viewSiemensSpecificDicomHeaderReference );
        
        m_importToPatientDatabaseMenuItem = new JMenuItem( "Import spectrum to database .." );
        m_importToPatientDatabaseMenuItem.setEnabled( false );
        m_importToPatientDatabaseMenuItem.addActionListener( this );
        
        m_popupmenu.add( m_importToPatientDatabaseMenuItem );
        
        m_popupmenu.add( m_viewDicomHeader );
        m_popupmenu.add( m_viewSiemensSpecificDicomHeader );
        
        m_popupmenu.add( m_displaySignalType );
    }

    
    /**
     * Method that allows user to save screen shots of "this" to be stored
     * in png-formatted file.
     */
    private void performScreenShotCreation( )
    {
        try
        {
            m_screenshotFileReference = getScreenShotFileReference( );
                    
            String fileNameBase = m_screenshotFileReference.getPath( );
                    
            for ( int i = 0 ; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
            {
                m_currentlyDisplayedInstance = i;
                
                displaySpectrum( );
                
                this.paintComponent(this.getGraphics( ) );
                
                try
                {
                    Dimension dim = this.getSize( );
                    Point pos = this.getLocationOnScreen( );
                
                    Rectangle screenRect = new Rectangle( pos.x, pos.y, dim.width, dim.height );
                
                    Robot robot = new Robot( );
            
                    try
                    {
                        BufferedImage image = robot.createScreenCapture(screenRect); 
                        // save an image to a PNG file
                        if ( ( i > -1 ) && ( i < 10 ) )
                        {
                            ImageIO.write( image, "png", new File( fileNameBase + "00" + m_currentlyDisplayedInstance + ".png" ) );
                        }
                        if ( ( i > 9 ) && ( i < 100 ) )
                        {
                            ImageIO.write( image, "png", new File( fileNameBase + "0" + m_currentlyDisplayedInstance + ".png" ) );
                        }
                        if ( i > 99 ) 
                        {
                            ImageIO.write( image, "png", new File( fileNameBase + m_currentlyDisplayedInstance + ".png" ) );
                        }
                    }
                    catch( IOException ioe )
                    {
                        System.err.println( "write: " + ioe.getMessage( ) );
                    }
                        
                }
                catch ( Exception exception )
                { 
                    System.out.println( exception );
                }
            }
        }
        catch ( OutOfMemoryError e )
        {
             Dialogs d = new Dialogs( );
             d.messageBox( m_parent, "<HTML> An out of memory error occurred  <BR> Close unused windows to free memory </HTML>" );
        }
    }
    
    /**
     * Save all screen shots in a separate file; 
     * Action is performed in a separate thread.
     */
    public void storeAllScreenShotsInSperateFiles( )
    {
       
        int copyOfCurrentlyDisplayedInstance = m_currentlyDisplayedInstance;
       
        final SwingWorker worker = new SwingWorker( )
        {
            @Override
            public Object construct( )
            {
                performScreenShotCreation( );          
                return null;
            }
                
            // runs in the event dispatching thread
            @Override
            public void finished( )
            {
                Dialogs d = new Dialogs( );
                d.messageBox( m_parent, "<HTML> All screenshots stored in seperate files </HTML>" );
            }
 
        };
        
        worker.start( );
    }
    
    /**
     * Called whenever an action event occurs.
     */
    @Override
    public void actionPerformed( ActionEvent e )
    {
        
        Object sender = e.getSource( );

        // special event raised by super class: update of display should
        // take place; this is a hack (correct), for the time being allowed
        if ( e.getID( ) == -999 )
        {
            displaySpectrum( );
        }
        
        if ( sender == m_saveScreenShot )
        {
            storeAllScreenShotsInSperateFiles( );   
        }
        
        if ( sender == m_storeDisplayedSignalOnly )
        {
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            getAbsorptionSignalToWrite( );
            
            float[] abs = getAbsorptionSignalToWrite( );
            float[] disp = getDispersionSignalToWrite( );
            
            SignalWriter swrA = new SignalWriter( "Save Displayed Complex Signal", abs, fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
        }
        
        if ( sender == m_performSVSSpectrumStabilityTests )
        {
           
           determineCurveFromInstances( false );
           int min, max;
           
           if ( m_indexMinValue < m_indexMaxValue )
           {
               min = m_indexMinValue;
               max = m_indexMaxValue;
           }
           else
           {
               min = m_indexMaxValue;
               max = m_indexMinValue;
           }
           
           min=0; max=1024;
           
           SingleVoxelSignalStabilityAnalyzer svsDataAnalyzer = new SingleVoxelSignalStabilityAnalyzer( min, max, m_measuredPatientSpectrumSeries );
           //svsDataAnalyzer.analyzeStandardizedMomentsAbsNoise( );
           
           svsDataAnalyzer.analyzeStandardizedMomentsAbs( );
           
           //svsDataAnalyzer.computeBias( );
           
           
           //svsDataAnalyzer.computeARSOSOutputPDFs( 11 );
           //svsDataAnalyzer.analyzeStandardizedMomentsAbs( );
           //svsDataAnalyzer.analyzeStandardizedMomentsPhase( );
           
           // String reportString = svsDataAnalyzer.performStabilityAnalysis( );
        }
        
        if ( sender == m_displayOneInstanceOfPatientSpectrum )
        {
            displaySpectrum( );
        }
        
        if ( sender == m_displaySumOfUserDefinedInstancesOfPatientSpectrum )
        {
             m_displaySeriesRangeDefiner = new FittingSumOfSpectrumSeriesRangeDefiner( m_measuredPatientSpectrumSeries.getNumberOfInstances( ) );
             m_displaySeriesRangeDefiner.addActionListener( this );
        }
        
        if ( sender == m_displayReference )
        {
            displayReferenceWaterSpectrum( );
        }
        
        if ( sender == m_displaySumOfInstancesOfPatientSpectrum )
        {
            displaySumPatientSpectrum( );
        }
        
        if ( sender == m_fitCurrentSpectrumMenuItem )
        {
            
            MRSpectrumFitter specFitter = new MRSpectrumFitter( m_parent );
            
            specFitter.setGlobalModel( m_globalModel );
            
            SingleVoxelSpectralData currentInstance = m_measuredPatientSpectrumSeries.getCurrentInstance( m_currentlyDisplayedInstance );
            //C String patName = FitGuruApp.getInstance( ).getDicomDir( ).getPatientName( );
            
            String patName = null;
            
            //specFitter.setSpectrumToFit( m_measuredPatientSpectrumFileName + "instance " +  m_currentlyDisplayedInstance , currentInstance );
            String instStr = m_currentlyDisplayedInstance + "_of_" + m_measuredPatientSpectrumSeries.getNumberOfInstances( ) + " - ";
            specFitter.setSpectrumToFit( patName + "instance " +  m_currentlyDisplayedInstance, instStr , currentInstance );
            specFitter.performSpectralFitting( );
            
        }
        
        if ( sender == m_fitSumOfSpectraMenuItem )
        {
            m_fittingSeriesRangeDefiner = new FittingSumOfSpectrumSeriesRangeDefiner( m_measuredPatientSpectrumSeries.getNumberOfInstances( ) );
            m_fittingSeriesRangeDefiner.addActionListener( this );
            
        }
        
        if ( sender == m_displaySeriesRangeDefiner )
        {
            if ( e.getID( ) == FittingSumOfSpectrumSeriesRangeDefiner.OK_EVENT )
            {
                int display_minimum = m_displaySeriesRangeDefiner.getMinimumOfSeriesRange( );
                int display_maximum = m_displaySeriesRangeDefiner.getMaximumOfSeriesRange( );
                
                displaySumPatientSpectrum( display_minimum, display_maximum );
                
            }
            
            if ( e.getID( ) == FittingSumOfSpectrumSeriesRangeDefiner.CANCEL_EVENT )
            {
                // no calcution wanted by user
            }
        }
        
        
        if ( sender == m_fittingSeriesRangeDefiner )
        {
            if ( e.getID( ) == FittingSumOfSpectrumSeriesRangeDefiner.OK_EVENT )
            {
                int minimum = m_fittingSeriesRangeDefiner.getMinimumOfSeriesRange( );
                int maximum = m_fittingSeriesRangeDefiner.getMaximumOfSeriesRange( );
                
                // String patName = FitGuruApp.getInstance( ).getDicomDir( ).getPatientName( );
                 
                String patName = null;
                
                MRSpectrumSeriesFitter specSeriesFitter = new MRSpectrumSeriesFitter( patName, m_parent );
                
                specSeriesFitter.setSpectralSeriesRangeAverageToFit( m_measuredPatientSpectrumSeries, minimum, maximum );
                specSeriesFitter.performSpectrumSeriesRangeAverageFit( );
                
            }
            
            if ( e.getID( ) == FittingSumOfSpectrumSeriesRangeDefiner.CANCEL_EVENT )
            {
                // no calcution wanted by user
            }
        }
        
        if ( sender == m_fitAveragedSignalsMenuItem )
        {
            // not implemented yet
        }
        
        if ( sender == m_fitAllSpectraWithOptionsMenuItem )
        {
            if ( m_quantificationOptionDefiner == null )
            {
                 m_quantificationOptionDefiner = new TimeSeriesQuantificationOptionsDefiner( m_parent );
                 m_quantificationOptionDefiner.addActionListener( this );
            }
        }
        
        if ( sender == m_quantificationOptionDefiner )
        {
            if ( e.getID( ) == TimeSeriesQuantificationOptionsDefiner.CANCEL_EVENT )
            {
                // no calculation wanted by the user
            }
            if ( e.getID( ) == TimeSeriesQuantificationOptionsDefiner.APPLY_EVENT )
            {
                
                int filterSize = m_quantificationOptionDefiner.getFilterSize( );
                int filterType = m_quantificationOptionDefiner.getFilterType( );
                        
                //c String patName = FitGuruApp.getInstance( ).getDicomDir( ).getPatientName( );
                
                String patName = null;
                
                MRSpectrumSeriesFitter specSeriesFitter = new MRSpectrumSeriesFitter( patName, m_parent );
                
                specSeriesFitter.setSpectralSeriesToFit( m_measuredPatientSpectrumSeries );
                
                specSeriesFitter.performSpectrumSeriesFit( filterType, filterSize );
            }
        }
        
        
        if ( sender == m_fitAllSpectraMenuItem )
        {
            
            // String patName = FitGuruApp.getInstance( ).getDicomDir( ).getPatientName( );
            
            String patName = null;
            
            MRSpectrumSeriesFitter specSeriesFitter = new MRSpectrumSeriesFitter( patName, m_parent );
            
            SingleVoxelSpectralData specData = getNamedResponse( "quality corrected" );
            
            specData.setDicomHeaderInfo( m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ) );
            
            specSeriesFitter.setSpectralSeriesToFit( m_measuredPatientSpectrumSeries );
            specSeriesFitter.performSpectrumSeriesFit( );
        }
        
        if ( sender == m_printSpectrum )
        {
             print( );
        }
        
        if ( sender == m_viewDicomHeaderSpectrum )
        {
//            DicomHeaderViewer view = new DicomHeaderViewer( m_parent, m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ) );
        }
        
        if ( sender == m_viewDicomHeaderReference )
        {
//            DicomHeaderViewer view = new DicomHeaderViewer( m_parent, m_measuredPatientWaterReferenceSpectrum.getDicomHeaderInfo( ) );
        }
        
        if ( sender == m_viewSiemensSpecificDicomHeaderSpectrum )
        {
//            DicomSiemensSpecificFileHeaderViewer view = new DicomSiemensSpecificFileHeaderViewer( m_parent, m_measuredPatientSpectrumSeries.getSiemensSpecificDicomHeaderInfo( ) );
        }
        
        if ( sender == m_viewSiemensSpecificDicomHeaderReference )
        {
//            DicomSiemensSpecificFileHeaderViewer view = new DicomSiemensSpecificFileHeaderViewer( m_parent, m_measuredPatientWaterReferenceSpectrum.getSiemensSpecificDicomHeaderInfo( ) );
        }
        
        if ( sender == m_displayIntegralOfFrequencyRangeBaselineCorrectedPlot )
        {
            String unitStr = "";
            
            TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
            def.load( );
            
            // which unit type does the user want to be displayed is stored and updated
            // in the MRFittingDefault class
            String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
            {
                unitStr = "pnts";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
            {
                unitStr = "Hz";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                unitStr = "ppm";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
            {
                unitStr = "pnts";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
            {
                unitStr = "sec";
            }
            
            Curve response = determineCurveFromInstances( true );
            
            NumberFormat format = new DecimalFormat( "#.##" );
            
            String title = "Integral offsetrange (" + format.format( m_floatMinX ) + unitStr + " , " + 
                                                      format.format( m_floatMaxX ) + unitStr +
                                                     " ) = (array indices [ " + m_indexMinValue + 
                                                     " , " + m_indexMaxValue + " ] )";
            
            OneDSignalDisplayerAndProcessor frame = new OneDSignalDisplayerAndProcessor( title, m_parent );
            
            float average = SignalTools.mean( response.getYvalues( ) );
            float st_dev = (float)Math.sqrt( SignalTools.computeVariance( response.getYvalues( ) ) / ( response.getYvalues( ).length  - 1f ) );
            
            XYPlot plotArea = frame.getPlotArea( );
            
            plotArea.setInfoString( "Average = " + average + " | " + "Variance : " + st_dev + "  |  %-Variance : " + ( st_dev * 100f / average ) );
            
            frame.addCurve( response, true );
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            // Write signal to disc if user wants it
            SignalWriter sw = new SignalWriter( "Write signal",  response.getXvalues( ), response.getYvalues( ), fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ), null );
            
        }
        
        if ( sender ==  m_displayIntegralOfFrequencyRangePlot )
        {
            String unitStr = "";
            
            TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
            def.load( );
            
            // which unit type does the user want to be displayed is stored and updated
            // in the MRFittingDefault class
            String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
            {
                unitStr = "pnts";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
            {
                unitStr = "Hz";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                unitStr = "ppm";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
            {
                unitStr = "pnts";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
            {
                unitStr = "sec";
            }
            
            // no baseline correction so call with argument false
            Curve response = determineCurveFromInstances( false );
            
            NumberFormat format = new DecimalFormat( "#.##" );
            
            String title = "Integral offsetrange (" + format.format( m_floatMinX ) + unitStr + " , " + 
                                                     format.format( m_floatMaxX ) + unitStr + 
                                                     " ) = (array indices [ " + m_indexMinValue + 
                                                     " , " + m_indexMaxValue + " ] )";
            
            OneDSignalDisplayerAndProcessor frame = new OneDSignalDisplayerAndProcessor( title, m_parent );
            
            float average = SignalTools.mean( response.getYvalues( ) );
            float st_dev = (float)Math.sqrt( SignalTools.computeVariance( response.getYvalues( ) ) / ( response.getYvalues( ).length  - 1f ) );
            
            XYPlot plotArea = frame.getPlotArea( );
            
            plotArea.setInfoString( "Average = " + average + " | " + "Variance : " + st_dev + "  |  %-Variance : " + ( st_dev * 100f / average ) );
            
            frame.addCurve( response, true );
            
            TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
            fdef.load( );
            
            // Write signal to disc if user wants it
            SignalWriter sw = new SignalWriter( "Write signal",  response.getXvalues( ), 
                                                response.getYvalues( ), 
                                                fdef.getProperty( "Dicom mount path" ) + 
                                                System.getProperty( "file.separator" ), null );
            
        }
        
        if ( sender ==  m_displayAverageOfFrequencyRangePlot )
        {
         
            String unitStr = "";
            
            TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
            def.load( );
            
            // which unit type does the user want to be displayed is stored and updated
            // in the MRFittingDefault class
            String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
            {
                unitStr = "pnts";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
            {
                unitStr = "Hz";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                unitStr = "ppm";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
            {
                unitStr = "pnts";
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
            {
                unitStr = "sec";
            }
            
            Curve response = determineCurveFromInstances( false );
            
            NumberFormat format = new DecimalFormat( "#.##" );

            format.format( m_floatMinX );
            
            String title = "Average offsetrange (" + format.format( m_floatMinX ) + unitStr + " , " + 
                                                     format.format( m_floatMaxX ) + unitStr + 
                                                     " ) = (array indices [ " + m_indexMinValue + 
                                                     " , " + m_indexMaxValue + " ] )";
            
            OneDSignalDisplayerAndProcessor frame = new OneDSignalDisplayerAndProcessor( title, m_parent );
            
            float average = SignalTools.mean( response.getYvalues( ) );
            float st_dev = (float)Math.sqrt( SignalTools.computeVariance( response.getYvalues( ) ) / ( response.getYvalues( ).length  - 1f ) );
            
            XYPlot plotArea = frame.getPlotArea( );
            
            plotArea.setInfoString( "Average = " + average + "<BR>" + "Variance : " + st_dev + "<BR>" + "%-Variance : " + ( st_dev * 100f / average )  );
            
            frame.addCurve( response, true );
            
        }
        
        super.actionPerformed( e );
    }
    
    
    /**
     * Returns the array values of the absorption channel of the 
     * currently displayed signal.
     */
    private float[] getAbsorptionSignalToWrite( )
    {
        float[] abs;
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        String displayDomain = def.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
         
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            abs = m_measuredPatientSpectrumSeries.getAbsorptionChannelFD( m_currentlyDisplayedInstance );            
        }
        else
        {
            abs = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( m_currentlyDisplayedInstance );
        }
        
        return abs;
    }
    
    
    /**
     * Returns the array values of the absorption channel of the 
     * currently displayed signal.
     */
    private float[] getDispersionSignalToWrite( )
    {
        float[] abs;
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        String displayDomain = def.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
         
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            abs = m_measuredPatientSpectrumSeries.getAbsorptionChannelFD( m_currentlyDisplayedInstance );            
        }
        else
        {
            abs = m_measuredPatientSpectrumSeries.getAbsorptionChannelTD( m_currentlyDisplayedInstance );
        }
        
        return abs;
    }
    
    
    /**
     * Determines the curve of spectral integrals from all instances.
     */
    private Curve determineCurveFromInstances( boolean doCorrectBaseline )
    {
        
        int minXValue = ( (XYPlotSlider)m_plotSliderList.getFirst( ) ).getCurrentPosition( );
        int maxXValue = ( (XYPlotSlider)m_plotSliderList.getLast( ) ).getCurrentPosition( );
        
        float minX = -1f;
        float maxX = -1f;
        
        // compute the true float values at the sliders
        Point2D.Float pmin = this.determineValuesAtPositionXY( new Point( minXValue, getHeight( ) / 2 ) );
        Point2D.Float pmax = this.determineValuesAtPositionXY( new Point( maxXValue, getHeight( ) / 2 ) );
        
        // order them properly
        if ( pmin.x > pmax.x )
        {
            float val = pmin.x;
            minX = pmax.x;
            maxX = val;
        }
        else
        {
            minX = pmin.x;
            maxX = pmax.x;
        }
        
        // compute the array indices values corresponding to x
        m_indexMinValue = minXValue = computeXArrayIndex( minX );
        m_indexMaxValue = maxXValue = computeXArrayIndex( maxX );
        
        m_floatMinX = minX;
        m_floatMaxX = maxX;
        
        // order them properly again
        if ( minXValue > maxXValue )
        {
            int val = minXValue;
            minXValue = maxXValue;
            maxXValue = val;
        }
        else
        {
            minXValue = minXValue;
            maxXValue = maxXValue;
        }
        
        int cntr = 0;
        float[] xArrayValues = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        float[] yArrayValues = new float[m_measuredPatientSpectrumSeries.getNumberOfInstances( )];
        
        for ( int i = 0; i < m_measuredPatientSpectrumSeries.getNumberOfInstances( ); i++ )
        {
            SingleVoxelSpectralData specData = m_measuredPatientSpectrumSeries.getCurrentInstance( i );
                        
            xArrayValues[cntr]= (float)i;
            
            if ( doCorrectBaseline == false )
            {
                yArrayValues[cntr]= getIntegralOfCurrentlyDisplayedSignal( minXValue, maxXValue, specData );
            }
            else
            {
                yArrayValues[cntr]= getIntegralOfCurrentlyDisplayedSignalBaselineCorrected( minXValue, maxXValue, specData );
            }
            cntr++;
        }
               
        return new Curve( xArrayValues, yArrayValues, Color.BLACK );
    }
    
    /**
     * Returns the integral over the indicated range and over the currently
     * displayed signal.
     * @param min
     * @param max
     * @param specData
     * @return 
     */
    public float getIntegralOfCurrentlyDisplayedSignal( int min, int max, SingleVoxelSpectralData specData )
    {
        
        float integral = 0f;
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        // which unit type does the user want to be displayed is stored and updated
        // in the MRFittingDefault class
        String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        String displayChannel = def.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        String displayDomain = def.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
         
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getAbsorptionCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summation( specData.getAbsorptionCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summation( specData.getAbsorptionCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getDispersionCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summation( specData.getDispersionCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summation( specData.getDispersionCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getMagnitudeCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summation( specData.getMagnitudeCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summation( specData.getMagnitudeCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPowerCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPowerCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPowerCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPhaseCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPhaseCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPhaseCurveFDPPM( ).getYvalues( ), min, max );
                }
            }
        }
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getAbsorptionCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getAbsorptionCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getDispersionCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getDispersionCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getMagnitudeCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getMagnitudeCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPowerCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPowerCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPhaseCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summation( specData.getPhaseCurveTDSeconds( ).getYvalues( ), min, max );
                    
                }
            }
            
        }
        
        return integral;
    }
    
    /**
     * Returns the integral over the indicated range and over the currently
     * displayed signal.
     * @param min
     * @param max
     * @param specData
     * @return 
     */
    public float getIntegralOfCurrentlyDisplayedSignalBaselineCorrected( int min, int max, SingleVoxelSpectralData specData )
    {
        
        float integral = 0f;
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        // which unit type does the user want to be displayed is stored and updated
        // in the MRFittingDefault class
        String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        String displayChannel = def.getProperty( TDFDFittingDefaults.DISPLAY_CHANNEL );
        String displayDomain = def.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
         
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getAbsorptionCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getAbsorptionCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getAbsorptionCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getDispersionCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getDispersionCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getDispersionCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getMagnitudeCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getMagnitudeCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getMagnitudeCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPowerCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPowerCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPowerCurveFDPPM( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPhaseCurveFDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPhaseCurveFDHertz( ).getYvalues( ), min, max );
                }
        
                if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPhaseCurveFDPPM( ).getYvalues( ), min, max );
                }
            }
        }
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        {
            if ( displayChannel.compareTo( TDFDFittingDefaults.ABSORPTION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getAbsorptionCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getAbsorptionCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.DISPERSION ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getDispersionCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getDispersionCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.MAGNITUDE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getMagnitudeCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getMagnitudeCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }
            
            if ( displayChannel.compareTo( TDFDFittingDefaults.POWER ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPowerCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPowerCurveTDSeconds( ).getYvalues( ), min, max );
                }
            }

            if ( displayChannel.compareTo( TDFDFittingDefaults.PHASE ) == 0 )
            {
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPhaseCurveTDPoints( ).getYvalues( ), min, max );
                }
                if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
                {
                    integral = SignalTools.summationWithBaselineCorrection( specData.getPhaseCurveTDSeconds( ).getYvalues( ), min, max );
                    
                }
            }
            
        }
        
        return integral;
    }
    
    
    
    
    /**
     * Computes the X index for the given float value.
     */
    private int computeXArrayIndex( float xValue )
    {
        
        int retValue = -1;
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        // which unit type does the user want to be displayed is stored and updated
        // in the MRFittingDefault class
        String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        String displayDomain = def.getProperty( TDFDFittingDefaults.DISPLAY_DOMAIN );
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.FREQ_DOMAIN ) == 0 )
        {
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_POINTS ) == 0 )
            {
                //retValue = (int)( xValue +  getTotalNumberOfDisplayPoints( ) / 2 );
                retValue = computeIndex( xValue );
            }
            
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_HZ ) == 0 )
            {
                // retValue = (int)( ( xValue + getSampleFrequency( ) / 2 ) * getTotalNumberOfDisplayPoints( ) / getSampleFrequency( ) );
                retValue = computeIndex( xValue );
            }
        
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                retValue = computeIndex( xValue );
            }
        }
        
        if ( displayDomain.compareTo( TDFDFittingDefaults.TIME_DOMAIN ) == 0 )
        { 
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_POINTS ) == 0 )
            {
                retValue = (int)( xValue + getTotalNumberOfDisplayPoints( ) / 2 );
            }
            if ( displayType.compareTo( TDFDFittingDefaults.TIME_DOMAIN_SECONDS ) == 0 )
            {
               retValue = (int)( xValue * getSampleFrequency( ) );  
            }            
        }
        
        return retValue;
        
    }
    
    
    /**
     * Computes the array index position from a value x.
     */
    private int computeIndex( float ppmValue )
    {
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        // which unit type does the user want to be displayed is stored and updated
        // in the MRFittingDefault class
        String displayType = def.getProperty( TDFDFittingDefaults.DISPLAY_UNIT );
        
        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
        {
            int val;
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                val = m_xyForcedPlotRange.computeIndexValueReversed( ppmValue, getTotalNumberOfDisplayPoints( ) );
            }
            else
            {
                val = m_xyForcedPlotRange.computeIndexValue( ppmValue, getTotalNumberOfDisplayPoints( ) );
            }
            return val;
        }
        else
        {
            int val;
            if ( displayType.compareTo( TDFDFittingDefaults.FREQUENCY_DOMAIN_PPM ) == 0 )
            {
                val = m_xyAutoPlotRange.computeIndexValueReversed( ppmValue, getTotalNumberOfDisplayPoints( ) );
            }
            else
            {
                val = m_xyAutoPlotRange.computeIndexValue( ppmValue, getTotalNumberOfDisplayPoints( ) );
            }
            
            return val;            
        }

    }
    
    
    
    /**
     * Displays the measured patient spectrum, if defined.
     */
    public void displaySpectrum( )
    {
        if ( this.m_measuredPatientSpectrumSeries != null )
        {
            
            m_displayOneInstanceOfPatientSpectrum.setSelected( true );
            
            m_responseList.clear( );
            m_responseNameList.clear( );
            m_responseListBuffer.clear( );
        
            deleteAllCurves( );
            
            setTitle( m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientName( ) + " (" +
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientBirthDate() + ") - " + 
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesDescription( ) + " -  Series # : " + 
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) + " - " +
                      "Instance " + ( m_currentlyDisplayedInstance + 1 ) + " of " + ( m_measuredPatientSpectrumSeries.getNumberOfInstances( ) )
                );
            
            
            
            m_measuredPatientSpectrumSeries.setCurveColor( Color.black );
            
            SingleVoxelMeasuredSpectralData instance = m_measuredPatientSpectrumSeries.getCurrentInstance( m_currentlyDisplayedInstance );
            
            addSpectralDataSet( "measured patient spectrum", instance, true );
        }
        else
        {
            // cannot show measured patient spectrum 
        }
    }
    
    
    /**
     * Displays the measured patient spectrum, if defined.
     */
    public void displayReferenceWaterSpectrum( )
    {
        if ( this.m_measuredPatientSpectrumSeries != null )
        {
            m_displayReference.setSelected( true );
            m_responseList.clear( );
            m_responseNameList.clear( );
            m_responseListBuffer.clear( );
        
            deleteAllCurves( );
        
            m_measuredPatientWaterReferenceSpectrum.setCurveColor( Color.black );
        
            addSpectralDataSet( "measured patient water reference spectrum", m_measuredPatientWaterReferenceSpectrum, true );
        }
        else
        {
            // cannot show measured patient spectrum 
        }
    }
    
    /**
     * Displays the measured patient spectrum, if defined.
     */
    public void displaySumPatientSpectrum( )
    {
        if ( m_measuredPatientSpectrumSeries != null )
        {
            
            m_displaySumOfInstancesOfPatientSpectrum.setSelected( true );
            m_responseList.clear( );
            m_responseNameList.clear( );
            m_responseListBuffer.clear( );
        
            deleteAllCurves( );
            
            setTitle( m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientName( ) + " (" +
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientBirthDate() + ") - " + 
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesDescription( ) + " -  Series # : " + 
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) + " - " +
                      "Sum of all <" + ( m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ) + "> instances"
                );
            
            m_measuredPatientSumSpectrum.setCurveColor( Color.black );
            m_measuredPatientSumSpectrum.setCurveColor( Color.black );
        
            addSpectralDataSet( "sum spectrum", m_measuredPatientSumSpectrum, true );
        }
        else
        {
            // cannot show measured patient spectrum 
        }
    }
    
    
    /**
     * Displays the sum of the measured patient spectrum, if defined;
     * Added are the spectral series instances from minimum till maximum.
     * @param minimum
     * @param maximum
     */
    public void displaySumPatientSpectrum( int minimum, int maximum )
    {
        if ( m_measuredPatientSpectrumSeries != null )
        {   
            // Define the attribute containing the sum spectrum
            m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( minimum, maximum );
            
            m_displaySumOfInstancesOfPatientSpectrum.setSelected( true );
            m_responseList.clear( );
            m_responseNameList.clear( );
            m_responseListBuffer.clear( );
        
            deleteAllCurves( );
            
            setTitle( m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientName( ) + " (" +
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientBirthDate() + ") - " + 
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesDescription( ) + " -  Series # : " + 
                      m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) + " - " +
                      "Sum of instances range [ " + minimum + " , " + maximum + " ]"
                );
            
            m_measuredPatientSumSpectrum.setCurveColor( Color.black );
            m_measuredPatientSumSpectrum.setCurveColor( Color.black );
        
            addSpectralDataSet( "sum spectrum", m_measuredPatientSumSpectrum, true );
            
            
        }
        else
        {
            // cannot show measured patient spectrum 
        }
    }
    
    
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param specData
     */
    public void setMeasuredSpectrumToPreview( SingleVoxelMeasuredSpectralData specData )
    {
        
        m_displayOneInstanceOfPatientSpectrum.setSelected( true );
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        
        deleteAllCurves( );
        
        specData.setCurveColor( Color.black );
        
        addSpectralDataSet( "measured", specData, true );
        
        m_viewDicomHeaderSpectrum.setEnabled( true );
        m_viewSiemensSpecificDicomHeaderSpectrum.setEnabled( true );
    }
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param fileName
     * @param specDataSeries
     */
    public void setMeasuredSpectrumSeries( String fileName, SingleVoxelSpectralDataSeries specDataSeries )
    {
        
        m_measuredPatientSpectrumFileName = fileName;
        
        m_displayOneInstanceOfPatientSpectrum.setSelected( true );
        
        m_measuredPatientSpectrumSeries = specDataSeries;
        
        
        // Define the attribute containing the sum spectrum
        m_measuredPatientSumSpectrum = m_measuredPatientSpectrumSeries.getSumSpectrum( );
        
        setTitle( m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientName( ) + " (" +
                  m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getPatientBirthDate() + ") - " + 
                  m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesDescription( ) + " -  Series # : " + 
                  m_measuredPatientSpectrumSeries.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) + " - " +
                      "Instance " + ( m_currentlyDisplayedInstance + 1 ) + " of " + ( m_measuredPatientSpectrumSeries.getNumberOfInstances( ) )
                );
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        deleteAllCurves( );
        
        m_measuredPatientSpectrumSeries.setCurveColor( Color.black );
        
        addSpectralDataSet( "spectrum [" + m_currentlyDisplayedInstance  + " ]", 
                            m_measuredPatientSpectrumSeries.getPersistentInstance( m_currentlyDisplayedInstance ), true );
        
        clearAllLeftUpperInformationLines( );
        
        addLeftLowerInformationLine( "voxel size : " + m_measuredPatientSpectrumSeries.getVoxelDimensions( ) );
        
        m_viewDicomHeaderSpectrum.setEnabled( true );
        m_viewSiemensSpecificDicomHeaderSpectrum.setEnabled( true );
        m_importToPatientDatabaseMenuItem.setEnabled( true );
        m_displaySumOfUserDefinedInstancesOfPatientSpectrum.setEnabled( true );
        m_displayOneInstanceOfPatientSpectrum.setEnabled( true );
        this.m_displaySumOfInstancesOfPatientSpectrum.setEnabled( true );
        m_printSpectrum.setEnabled( true );
        
        addTwoHorizontalXYPlotSliders( );
    }
    
    /**
     * Sets the measured reference spectrum object in the m_modelSpectrumViewer
     * The reference spectrum, is the spectrum that is measured in order to obtain
     * prior knowledge information.
     * @param fileName
     * @param specData
     */
    public void setMeasuredPatientWaterReferenceSpectrum( String fileName, SingleVoxelMeasuredSpectralData specData )
    {
        
        m_measuredPatientWaterReferenceSpectrumFileName = fileName;
        
        m_displayReference.setSelected( true );
        
        m_measuredPatientWaterReferenceSpectrum = new SingleVoxelMeasuredSpectralData( );
        m_measuredPatientWaterReferenceSpectrum.setSpectralDataPointsTD( specData.getAbsorptionChannelTD( ),
                                                                         specData.getDispersionChannelTD( ) );
        
        m_measuredPatientWaterReferenceSpectrum.setDicomHeaderInfo( specData.getDicomHeaderInfo( ) );
        m_measuredPatientWaterReferenceSpectrum.setSiemensSpecificDicomHeaderInfo( specData.getSiemensSpecificDicomHeaderInfo( ) );
        if ( m_measuredPatientSpectrumSeries != null )
        {
            m_measuredPatientSpectrumSeries.applyQuality( false, m_measuredPatientWaterReferenceSpectrum );
        }
        
        m_responseList.clear( );
        m_responseNameList.clear( );
        m_responseListBuffer.clear( );
        deleteAllCurves( );
        
        addSpectralDataSet( "quality corrected [" + m_currentlyDisplayedInstance + " ]", m_measuredPatientSpectrumSeries.getPersistentInstance( m_currentlyDisplayedInstance ), true );
        m_viewDicomHeaderReference.setEnabled( true );
        m_viewSiemensSpecificDicomHeaderReference.setEnabled( true );
        m_displayReference.setEnabled( true );
        
    }
    
    /**
     * Is called whenever the mouse wheel is rotated.
     * @param e
     */
    @Override
    public void mouseWheelMoved( MouseWheelEvent e )
    {
        int diff = e.getWheelRotation( );

        if ( ( m_currentlyDisplayedInstance - diff < m_measuredPatientSpectrumSeries.getNumberOfInstances( ) ) && ( m_currentlyDisplayedInstance - diff > -1 ) )
        {
            m_currentlyDisplayedInstance -= diff;
            // envokes the refresh of the display
            displaySpectrum( );
        }
    }

    
    /**
     * Called when the mouse button is pressed.
     */
    @Override
    public void mousePressed( MouseEvent mouseEvent ) 
    {

        // Is only of interest if left button is pressed.
        if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
        {
            m_mouseButtonPressedX = mouseEvent.getX( );
            m_mouseButtonPressedY = mouseEvent.getY( );
        }

        super.mousePressed( mouseEvent );
    }
    
    /**
     * Called when the mouse button is pressed.
     */
    @Override
    public void mouseReleased( MouseEvent mouseEvent )
    {

        if ( m_sliderIsSelected == false )
        {
            if ( m_mouseDragged == true )
            {
                m_mouseDragged = false;
                // is only of interest if left button is released
                if ( ( mouseEvent.getModifiers( ) & InputEvent.BUTTON1_MASK ) == InputEvent.BUTTON1_MASK )
                {
            
                    m_mouseButtonReleasedX = mouseEvent.getX( );
                    m_mouseButtonReleasedY = mouseEvent.getY( );
            
                    // continue only if the start and stop position are not the same
                    if ( ( Math.abs( m_mouseButtonPressedX - m_mouseButtonReleasedX ) > 5 ) && 
                         ( Math.abs( m_mouseButtonPressedY - m_mouseButtonReleasedY ) > 5 ) )
                    {
                        Point2D.Float start = determineValuesAtPosition( new Point( m_mouseButtonPressedX, m_mouseButtonPressedY ) );
                        Point2D.Float stop = determineValuesAtPosition( new Point( m_mouseButtonReleasedX, m_mouseButtonReleasedY ) );
                
                        if ( ( start.x < stop.x ) && ( start.y < stop.y ) )
                        {
                            m_xyForcedPlotRange = new XYPlotRange( start.x, stop.x, start.y, stop.y );
                        }
                        if ( ( start.x < stop.x ) && ( start.y > stop.y ) )
                        {
                            m_xyForcedPlotRange = new XYPlotRange( start.x, stop.x, stop.y, start.y );   
                        }
                        if ( ( start.x > stop.x ) && ( start.y > stop.y ) )
                        {
                            m_xyForcedPlotRange = new XYPlotRange( stop.x, start.x, stop.y, start.y );
                        }
                        if ( ( start.x > stop.x ) && ( start.y < stop.y ) )
                        {   
                            m_xyForcedPlotRange = new XYPlotRange( stop.x, start.x, start.y, stop.y );   
                        }
                    
                        this.m_XYPlotRangeSelectorMenuItem.setSelected( true );
                    
                        changeDisplayedSignal( );
                    }
                }
            }
        }
        
        super.mouseReleased( mouseEvent );
    }
    
    /**
     * Called when the mouse button is clicked.
     * @param mouseEvent
     */
    @Override
    public void mouseClicked( MouseEvent mouseEvent ) 
    {
        super.mouseClicked( mouseEvent );
    }
    
    /**
     * Called when the mouse is dragged.
     * @param mouseEvent
     */
    @Override
    public void mouseDragged( MouseEvent mouseEvent )
    {
        m_mouseDragged = true;
        super.mouseDragged( mouseEvent );
    }
    
    /**
     * Called when the mouse enters the control.
     * @param mouseEvent
     */    
    @Override
    public void mouseEntered( MouseEvent mouseEvent )
    {
        super.mouseEntered( mouseEvent );
    }
    
    /**
     * Called when the mouse exits.
     * @param mouseEvent
     */    
    @Override
    public void mouseExited( MouseEvent mouseEvent )
    {
        super.mouseExited( mouseEvent );
    }
    
    /**
     * Called when the mouse button is released.
     * @param mouseEvent
     */    
    @Override
    public void mouseMoved( MouseEvent mouseEvent )
    {
        createStatusLineMessageMouseLocation( mouseEvent );
        super.mouseMoved( mouseEvent );
    }

    
    /**
     * Creates a status line containing the current location of the mouse pointer.
     */
    private void createStatusLineMessageMouseLocation( MouseEvent mouseEvent )
    {
        int xPos = mouseEvent.getX( );
        int yPos = mouseEvent.getY( );
        
        if ( ( xPos > this.getSize( ).getWidth( ) / 10 ) && ( xPos < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( yPos > this.getSize( ).getHeight( ) / 10 ) && ( yPos < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)xPos - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)yPos - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                xPosCorrected = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            else
                            {
                                xPosCorrected = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );
                            }
                            
                            yPosCorrected = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                xPosCorrected = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                xPosCorrected = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            
                            yPosCorrected = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                        if ( m_statusPanel != null )
                        {
                            m_statusPanel.setLeftMessage( "( x, y ) =  ( " + xPosCorrected + " , " + yPosCorrected + " )" );
                        }
                    }
                }
            }
        }
    }
    
    /**
     * Determines the selected values at the position indicated by the
     * mouseEvent.
     */
    private void determineValuesAtCursorPosition( int xPos, int yPos )
    {
        
        if ( ( xPos > this.getSize( ).getWidth( ) / 10 ) && ( xPos < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( yPos > this.getSize( ).getHeight( ) / 10 ) && ( yPos < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {
                    if ( m_curves.size( ) > 0 )
                    {
                        float xPosCorrected = ( ( (float)xPos - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)yPos - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                        
                        // lorentzian amplitude must be corrected in order to get the correct height in freq. domain
                        m_lastSelectedAmplitudeValue = m_lastSelectedAmplitudeValue * 3.141592f * 2.0f / 1024.0f;
                    }
                }
            }
        }
    }
    
    /**
     * Determines the selected values at the position indicated by the
     * screen-point Point p.
     */
    private Point2D.Float determineValuesAtPosition( Point p )
    {
        float selectedOffsetValue = 0f;
        float selectedAmplitudeValue = 0f;
        
        if ( ( p.x > this.getSize( ).getWidth( ) / 10 ) && ( p.x < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( p.y > this.getSize( ).getHeight( ) / 10 ) && ( p.y < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {

                    if ( m_curves.size( ) > 0 )
                    {
                        /*
                        float xPosCorrected = ( ( (float)p.x - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)p.y - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_displayXReversed == false )
                        {
                            selectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                        }
                        else
                        {
                            selectedOffsetValue = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                        }
                        selectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        
                        */
                        
                        float xPosCorrected = ( ( (float)p.x - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)p.y - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            m_lastSelectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                m_lastSelectedOffsetValue = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            
                            m_lastSelectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                        
                        // lorentzian amplitude must be corrected in order to get the correct height in freq. domain
                        // m_lastSelectedAmplitudeValue = m_lastSelectedAmplitudeValue * 3.141592f * 2.0f / 1024.0f;
                        
                        
                    }                    
                }
            }
        }
        
        return new Point2D.Float( m_lastSelectedOffsetValue, m_lastSelectedAmplitudeValue ); 
    }
    
    
    /**
     * Determines the selected values at the position indicated by the
     * screen-point Point p : no effect on member variables.
     */
    private Point2D.Float determineValuesAtPositionXY( Point p )
    {
        float selectedOffsetValue = 0f;
        float selectedAmplitudeValue = 0f;
        
        if ( ( p.x > this.getSize( ).getWidth( ) / 10 ) && ( p.x < 9 * this.getSize( ).getWidth( ) / 10 ) )
        {   
            if ( ( p.y > this.getSize( ).getHeight( ) / 10 ) && ( p.y < 9 * this.getSize( ).getHeight( ) / 10 ) )
            {
                float xrange = 8.0f * (float)getSize( ).getWidth( ) / 10.0f;
                float yrange = 8.0f * (float)getSize( ).getHeight( ) / 10.0f;
                
                MRSignalProperties mrcond = MRSignalProperties.getInstance( );
                
                if ( m_curves != null )
                {

                    if ( m_curves.size( ) > 0 )
                    {
                     
                        float xPosCorrected = ( ( (float)p.x - (float)this.getSize( ).getWidth( ) / 10.0f )  )  / (float)xrange;
                        float yPosCorrected = ( ( (float)p.y - (float)this.getSize( ).getHeight( ) / 10.0f )  )  / (float)yrange;
                        
                        if ( m_XYPlotRangeSelectorMenuItem.isSelected( ) == true )
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                selectedOffsetValue = m_xyForcedPlotRange.getMinimumX( ) + xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            else
                            {
                                selectedOffsetValue = m_xyForcedPlotRange.getMaximumX( ) - xPosCorrected * m_xyForcedPlotRange.getXRange( );   
                            }
                            
                            selectedAmplitudeValue = m_xyForcedPlotRange.getMaximumY( ) - yPosCorrected * m_xyForcedPlotRange.getYRange( );
                        }
                        else
                        {
                            if ( this.m_displayXReversed == false )
                            {
                                selectedOffsetValue = m_xyAutoPlotRange.getMinimumX( ) + xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            else
                            {
                                selectedOffsetValue = m_xyAutoPlotRange.getMaximumX( ) - xPosCorrected * m_xyAutoPlotRange.getXRange( );
                            }
                            
                            selectedAmplitudeValue = m_xyAutoPlotRange.getMaximumY( ) - yPosCorrected * m_xyAutoPlotRange.getYRange( );
                        }
                    }                    
                }
            }
        }
        
        return new Point2D.Float( selectedOffsetValue, selectedAmplitudeValue ); 
    }
    
    
    
    /**
     * Prints the current graphical context.
     * @param g
     * @param pf
     */
    @Override
    public int print( Graphics g, PageFormat pf, int pageIndex ) throws PrinterException 
    {
        if ( pageIndex > 0 )
        {
            return Printable.NO_SUCH_PAGE;
        }
        Graphics2D g2d = (Graphics2D)g;
        g2d.translate( pf.getImageableX( ), pf.getImageableY( ) );
                
        Rectangle componentBounds = this.getBounds(null);
        double scaleX = pf.getImageableWidth( ) / componentBounds.width;
        double scaleY = pf.getImageableHeight( ) / componentBounds.height;
        /*
        if (scaleX < 1 || scaleY < 1) 
        {
            
            if (scaleX < scaleY) 
            {
                scaleY = scaleX;
            } 
            else 
            {
                scaleX = scaleY;
            }
        }
        */
        g2d.scale( scaleX, scaleY );
        
        paintComponent( g2d );
        
        return Printable.PAGE_EXISTS;  
    }
    
    /**
     * Prints the currently displayed spectrum.
     */
    public void print( )
    {
        //--- Create a new PrinterJob object
        PrinterJob printJob = PrinterJob.getPrinterJob( );
        
        //--- Create a new book to add pages to
        Book book = new Book( );
        
        //--- Add the document page using a landscape page format
        PageFormat documentPageFormat = new PageFormat( );
        documentPageFormat.setOrientation( PageFormat.LANDSCAPE );
        book.append ( this, documentPageFormat );
        
        //--- Tell the printJob to use the Book as the Pageable object
        printJob.setPageable( book );
        
        //--- Show the print dialog box. If the user clicks the
        //--- print button, we then proceed to print, else we cancel
        //--- the process.
        try 
        {
            if ( printJob.printDialog( ) )
            {
                printJob.print( );
            }
        }
        catch ( PrinterException e )
        {
        }  
    }
    
    /**
     * Returns the number of sample points.
     * @return 
     */
    public int getTotalNumberOfDisplayPoints( )
    {
        return (int)((SingleVoxelSpectralData)m_responseList.iterator().next()).getAbsorptionChannelFD().length;
    }
    
    /**
     * Returns the number of sample points.
     * @return 
     */
    public float getSampleFrequency( )
    {
        return ((SingleVoxelSpectralData)m_responseList.iterator().next()).getSampleFrequency();
    }
    
    /**
     * Returns the file selected by the user.
     */
    private File getScreenShotFileReference( )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "File name and path to store screenshot into" ) );
        
        int result = fileChooser.showOpenDialog( this );
        
        m_screenshotFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                m_screenshotFileReference = fileChooser.getSelectedFile( );
                                                m_screenshotFileReference = new File( m_screenshotFileReference.getPath( ) );
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return m_screenshotFileReference;
    }
    
    /**
     * Enables printing: menu item controlling printing is 
     * enabled.
     */
    public void enablePrinting( )
    {
        m_printSpectrum.setEnabled( true );
    }
    
    /**
     * Enables printing if "enable" is set to true.
     * @param enable
     */
    public void enablePrinting( boolean enable )
    {
        m_printSpectrum.setEnabled( enable );
    }
    
    // Attributes of this class
    
    private JMenu m_permanentStorage;
    
    private JMenuItem m_storeDisplayedSignalOnly;        
    private JMenuItem m_storeAllRepsonses;
    
    private JMenuItem m_performSVSSpectrumStabilityTests;
    
    private JMenu m_spectralQuantification;
    private JMenuItem m_fitAllSpectraMenuItem;
    private JMenuItem m_fitAllSpectraWithOptionsMenuItem;
    private JMenuItem m_fitAveragedSignalsMenuItem;
    private JMenuItem m_fitSumOfSpectraMenuItem;
    private JMenuItem m_fitCurrentSpectrumMenuItem;
    
    private JMenuItem m_printSpectrum;
    
    private JMenu m_viewDicomHeader;
    private JMenuItem m_viewDicomHeaderSpectrum;
    private JMenuItem m_viewDicomHeaderReference;
    
    private JMenu m_viewSiemensSpecificDicomHeader;
    private JMenuItem m_viewSiemensSpecificDicomHeaderSpectrum;
    private JMenuItem m_viewSiemensSpecificDicomHeaderReference;

    private JMenu m_displaySignalType;
    
    private JRadioButtonMenuItem m_displayOneInstanceOfPatientSpectrum;
    private JRadioButtonMenuItem m_displaySumOfUserDefinedInstancesOfPatientSpectrum;
    private JRadioButtonMenuItem m_displaySumOfInstancesOfPatientSpectrum;
    
    private JMenuItem m_displayIntegralOfFrequencyRangePlot;
    private JMenuItem m_displayIntegralOfFrequencyRangeBaselineCorrectedPlot;
    private JMenuItem m_displayAverageOfFrequencyRangePlot;
    
    private JRadioButtonMenuItem m_displayReference;
    private JMenuItem m_saveScreenShot;
    
    private final ButtonGroup m_signalDisplaySelector = new ButtonGroup( );
    
    private JMenuItem m_importToPatientDatabaseMenuItem;
    
    private String m_measuredPatientWaterReferenceSpectrumFileName;
    private SingleVoxelMeasuredSpectralData m_measuredPatientWaterReferenceSpectrum;
    
    protected int m_mouseButtonPressedX;
    protected int m_mouseButtonPressedY;
    
    protected int m_mouseButtonReleasedX;
    protected int m_mouseButtonReleasedY;
    
    protected float m_lastSelectedOffsetValue;
    protected float m_lastSelectedAmplitudeValue;
    
    boolean m_mouseDragged = false;
    
    private final JFrame m_parent;

    // private int m_currentlyDisplayedInstance = 0;
    private int m_indexMinValue, m_indexMaxValue;
    private float m_floatMinX, m_floatMaxX;
    
    private File m_screenshotFileReference;
    
    private FittingSumOfSpectrumSeriesRangeDefiner m_fittingSeriesRangeDefiner;
    private FittingSumOfSpectrumSeriesRangeDefiner m_displaySeriesRangeDefiner;
    
    private TimeSeriesQuantificationOptionsDefiner m_quantificationOptionDefiner;
    
    
    // For proper serialization
    private static final long       serialVersionUID = 1482114591289213L;
    
    // Needed for water reference computations
    private final DicomDirectory    m_dicomDir;
    private GlobalModel m_globalModel;
}