/*
 * File: $Id: MRSpectrumPreProcessingPanel.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 09.02.2004, 9:55  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 - 2004 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

/*
 * MRSpectrumPreProcessingPanel.java
 *
 * Created on February 9, 2004, 9:55 AM
 */

package org.spectrim.tdfdfit;

import java.awt.BorderLayout;
import java.awt.Dimension;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.JPanel;
import org.spectrim.hlsvd.HLSVDDefaultsDefinerFrame;


/**
 * 
 * Allows user to set some preprocessing defaults.
 * 
 * @author  hans
 */
public class MRSpectrumPreProcessingPanel extends JPanel implements ActionListener
{
    
    /** 
     * Creates a new instance of MRSpectrumPreProcessingPanel. 
     */
    public MRSpectrumPreProcessingPanel( )
    {
        
        setLayout( new BorderLayout( ) );
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BorderLayout( ) );
        
        m_qualityPanel = new JPanel( );
        m_qualityPanel.setBorder( BorderFactory.createCompoundBorder(
                                  BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                  BorderFactory.createTitledBorder( "QUALITY - preprocessing" ) ) );
        
        
        m_qualityPanel.setLayout( new BoxLayout( m_qualityPanel, BoxLayout.Y_AXIS ) );
        
        m_applyQualityPanel = new JPanel( );
        m_applyQualityPanel.setLayout( new BoxLayout( m_applyQualityPanel, BoxLayout.X_AXIS ) );
        
        m_applyQualityCheckBox = new JCheckBox( "Apply QUALITY" );
        
        m_applyQualityCheckBox.addActionListener( this );
        
        m_applyQualityCheckBox.setSelected( true );
        
        m_applyQualityPanel.add( m_applyQualityCheckBox );
        
        m_qualityTypePanel = new JPanel( );
        m_qualityTypePanel.setLayout( new BoxLayout( m_qualityTypePanel, BoxLayout.X_AXIS ) );
        
        m_applyFullQuality = new JRadioButton( "Apply full QUALTIY" );
        m_applyFullQuality.addActionListener( this );
        
        m_applyPhaseQualityOnly = new JRadioButton( "Apply phase QUALITY only" );
        m_applyPhaseQualityOnly.addActionListener( this );
        
        m_qualityTypePanel.add( m_applyPhaseQualityOnly );
        m_qualityTypePanel.add( Box.createHorizontalGlue( ) );
        m_qualityTypePanel.add( m_applyFullQuality );
        
        m_buttonGroup = new ButtonGroup( );
        
        m_buttonGroup.add( m_applyFullQuality );
        m_buttonGroup.add( m_applyPhaseQualityOnly );
        
        m_applyPhaseQualityOnly.setSelected( true );
        
        m_qualityPanel.add( m_qualityTypePanel );
        m_qualityPanel.add( m_applyQualityPanel );
        
        m_defineHLSVDDefaultsPanel = new JPanel( );
        m_defineHLSVDDefaultsPanel.setLayout( new BoxLayout( m_defineHLSVDDefaultsPanel, BoxLayout.X_AXIS ) );
        m_defineHLSVDDefaultsPanel.add( Box.createHorizontalGlue( ) );
        m_setHSLVDDefaultsButton = new JButton( "Set HLSVD defaults" );
        m_defineHLSVDDefaultsPanel.add( m_setHSLVDDefaultsButton );
        m_defineHLSVDDefaultsPanel.add( Box.createHorizontalGlue( ) );
        
        m_setHSLVDDefaultsButton.addActionListener( this );
        
        m_contentPanel.add( m_qualityPanel, BorderLayout.CENTER );
        m_contentPanel.add( m_defineHLSVDDefaultsPanel, BorderLayout.SOUTH );
        add( m_contentPanel );
        
        // read the values set in the FitGuru default file
        setValues( );
    }
    
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        Object sender = e.getSource( );
        if ( sender == m_applyQualityCheckBox )
        {
            if ( m_applyQualityCheckBox.isSelected( ) )
            {
                m_applyFullQuality.setEnabled( true );
                m_applyPhaseQualityOnly.setEnabled( true );
                
                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                def.setProperty( "Apply QUALITY", "true" );
                def.save( );
            }
            else
            {
                m_applyFullQuality.setEnabled( false );
                m_applyPhaseQualityOnly.setEnabled( false );
                TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
                def.setProperty( "Apply QUALITY", "false" );
                def.save( );
            }
        }
        if ( sender == m_setHSLVDDefaultsButton )
        {
            HLSVDDefaultsDefinerFrame hldsvdDefaultsFrame = new HLSVDDefaultsDefinerFrame( ); 
        }
        
        if ( sender == this.m_applyFullQuality )
        {
            TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
            def.setProperty( TDFDFittingDefaults.QUALITY_TYPE, TDFDFittingDefaults.QUALITY_TYPE_AMPLITUDE_AND_PHASE );
            def.save( );
        }

        if ( sender == this.m_applyPhaseQualityOnly )
        {
            TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
            def.setProperty( TDFDFittingDefaults.QUALITY_TYPE, TDFDFittingDefaults.QUALITY_TYPE_PHASE_ONLY );
            def.save( );
        }  
    }
    
    /**
     * Sets the values set by the user in previous sessions.
     */
    private void setValues( )
    {
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        if ( def.getProperty( "Apply QUALITY" ).compareTo( "true" ) == 0 )
        {
            m_applyQualityCheckBox.setSelected( true );
        }
        else
        {
            m_applyQualityCheckBox.setSelected( false );
            
            m_applyFullQuality.setEnabled( false );
            m_applyPhaseQualityOnly.setEnabled( false );
            
        }
        
        if ( def.getProperty( TDFDFittingDefaults.QUALITY_TYPE ).compareTo( TDFDFittingDefaults.QUALITY_TYPE_AMPLITUDE_AND_PHASE ) == 0 )
        {
            m_applyFullQuality.setSelected( true );
        }
        else
        {
            m_applyFullQuality.setSelected( false );
        }
    }
    
    // Attributes
    
    private JPanel m_contentPanel;
    
    private JPanel m_qualityPanel;
    private JPanel m_applyQualityPanel;
    private JCheckBox m_applyQualityCheckBox;
    
    private JPanel m_qualityTypePanel;
    private JRadioButton m_applyFullQuality;
    private JRadioButton m_applyPhaseQualityOnly;
    private ButtonGroup m_buttonGroup;
    
    private JPanel m_defineHLSVDDefaultsPanel;
    private JButton m_setHSLVDDefaultsButton;
    // For proper serialization
    private static final long serialVersionUID = 1482325898866693L;
}
