/*
 * File: $Id: MRSpectrumSeriesFitter.java,v 1.3 2006/11/08 12:18:02 hans Exp $
 * History
 * Date: 22.08.2005, 12:03  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2005 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

 
/*
 * MRSpectrumSeriesFitter.java
 *
 * Created on August 22, 2005, 10:24 AM
 */

package org.spectrim.tdfdfit;

import java.io.InputStream;
import java.io.IOException;

import java.util.Iterator;

import javax.swing.JFrame;

import java.util.LinkedList;


/**
 *
 * Does the fitting of a spectrum using the spectral model that is defined
 * in the MRModelSpectrum class
 *
 * @author  hans
 */
public class MRSpectrumSeriesFitter 
{
    
    /** 
     * Creates a new instance of MRSpectrumSeriesFitter. 
     */
    public MRSpectrumSeriesFitter( String patientName, JFrame parent )
    {
        m_parent = parent;
        m_patientName = patientName;
        m_patientName = m_patientName.replace( ' ', '_');
        m_patientName = m_patientName.replace( '^', '_' );
    }
        
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     */
    public void setSpectralSeriesToFit( SingleVoxelSpectralDataSeries specSeries )
    {
        m_spectrumFileNames = specSeries.getFileReferences( );
        m_spectralDataSeries = specSeries;
    }
    
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     */
    public void setSpectralSeriesRangeAverageToFit( SingleVoxelSpectralDataSeries specSeries, int minimumSeriesNumber, int maximumSeriesNumber )
    {
        m_spectrumFileNames = specSeries.getFileReferences( );
        
        m_spectralDataSeries = specSeries;
        m_minimumOfSpectrumSeries = minimumSeriesNumber;
        m_maximumOfSpectrumSeries = maximumSeriesNumber;
    }
    
    /**
     * Compute filtered response (moving median or moving average filtered).
     */
     private SingleVoxelSpectralData computeFilteredResponse( int cnt, int filterType, int filterSize )
     {
         return m_spectralDataSeries.computeFilteredResponse( cnt, filterType, filterSize );
     }
     
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     */
    public void performSpectrumSeriesFit( int filterType, int filterSize )
    {
        
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        String spectralDatabasePath = (String)fitDef.get( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        
        m_spectrumPath = spectralDatabasePath;
        
        if ( filterType == TimeSeriesQuantificationOptionsDefiner.MOVING_AVERAGE_FILTERING )
        {
            int i = 0;
            Iterator it = m_spectrumFileNames.iterator( );
            while ( it.hasNext( ) )
            {
            
                m_currentSpectrumToFit = computeFilteredResponse( i, filterType, filterSize );
            
                // m_currentSpectrumToFit = m_spectralDataSeries.getCurrentInstance( i );
            
                m_currentFileName =  (String)it.next( );
            
                int fileNamePos = m_currentFileName.lastIndexOf( System.getProperty( "file.separator" ) ) + 1;
            
                m_currentFileName = m_currentFileName.substring( fileNamePos );
            
                m_currentFileName = m_patientName + "_" + m_currentFileName + "_" + i + "_of_" + m_spectralDataSeries.getNumberOfInstances( );
            
                // DicomImageData data = new DicomImageData( m_currentFileName );
           
                m_startParameterFileName = spectralDatabasePath + System.getProperty( "file.separator" ) + m_currentFileName.replaceAll( " ","" ) + ".par"; 
                m_currentFileName = m_currentFileName.replaceAll( " ", "" ) + ".bin";
        
                TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
                modSpec.createStartingValueFileToPath(  m_startParameterFileName );
        
                m_currentSpectrumToFit.writeAsTdfdfitFile( spectralDatabasePath.replaceAll( " ", "" ),  m_currentFileName );
            
                performSpectrumFit( );
                showFitResults( "" + i + "_of_" + m_spectralDataSeries.getNumberOfInstances( ) );
            
                i++;
            }
        }
        
        if ( filterType == TimeSeriesQuantificationOptionsDefiner.MOVING_MEDIAN_FILTERING )
        {
            System.out.println("Moving median filtering is not implemented yet ! ");
        }
        
        if ( filterType == TimeSeriesQuantificationOptionsDefiner.SIMPLE_AVERAGE_FILTERING )
        {
            
            int i = 0;
                 
            Iterator it = m_spectrumFileNames.iterator( );
            while ( i <  m_spectralDataSeries.getNumberOfInstances( ) )
            {
            
                m_currentSpectrumToFit = computeFilteredResponse( i, filterType, filterSize );
            
                // m_currentSpectrumToFit = m_spectralDataSeries.getCurrentInstance( i );
            
                m_currentFileName =  (String)it.next( );
            
                int fileNamePos = m_currentFileName.lastIndexOf( System.getProperty( "file.separator" ) ) + 1;
            
                m_currentFileName = m_currentFileName.substring( fileNamePos );
            
                m_currentFileName = m_patientName + "_" + m_currentFileName + "_" + i + "_to_" + ( i + filterSize ) + "_of_" + m_spectralDataSeries.getNumberOfInstances( );
            
                // DicomImageData data = new DicomImageData( m_currentFileName );
           
                m_startParameterFileName = spectralDatabasePath + System.getProperty( "file.separator" ) + m_currentFileName.replaceAll( " ","" ) + ".par"; 
                m_currentFileName = m_currentFileName.replaceAll( " ", "" ) + ".bin";
        
                TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
                modSpec.createStartingValueFileToPath(  m_startParameterFileName );
        
                m_currentSpectrumToFit.writeAsTdfdfitFile( spectralDatabasePath.replaceAll( " ", "" ),  m_currentFileName );
            
                performSpectrumFit( );
                showFitResults( "" + i + "_of_" + m_spectralDataSeries.getNumberOfInstances( ) );
            
                i = i + filterSize;
            }
            
        }
    }
    
    
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     */
    public void performSpectrumSeriesFit( )
    {   
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        String spectralDatabasePath = (String)fitDef.get( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        
        m_spectrumPath = spectralDatabasePath;
        
        int i = 0;
        Iterator it = m_spectrumFileNames.iterator( );
        while ( it.hasNext( ) )
        {
            m_currentSpectrumToFit = m_spectralDataSeries.getCurrentInstance( i );
            
            m_currentFileName =  (String)it.next( );
            
            int fileNamePos = m_currentFileName.lastIndexOf( System.getProperty( "file.separator" ) ) + 1;
            
            m_currentFileName = m_currentFileName.substring( fileNamePos );
            
            m_currentFileName = m_patientName + "_" + m_currentFileName + "_" + i + "_of_" + m_spectralDataSeries.getNumberOfInstances( );
            
            // DicomImageData data = new DicomImageData( m_currentFileName );
           
            m_startParameterFileName = spectralDatabasePath + System.getProperty( "file.separator" ) + m_currentFileName.replaceAll( " ","" ) + ".par"; 
            m_currentFileName = m_currentFileName.replaceAll( " ", "" ) + ".bin";
        
            TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
            modSpec.createStartingValueFileToPath(  m_startParameterFileName );
        
            m_currentSpectrumToFit.writeAsTdfdfitFile( spectralDatabasePath.replaceAll( " ", "" ),  m_currentFileName );
            
            performSpectrumFit( );
            showFitResults( "" + i + "_of_" + m_spectralDataSeries.getNumberOfInstances( ) );
            
            i++;
       }
    }
    
    /**
     * Sets the attribute containing the spectral data that should be fitted.
     */
    public void performSpectrumSeriesRangeAverageFit( )
    {   
        TDFDFittingDefaults fitDef = TDFDFittingDefaults.getInstance( );
        fitDef.load( );
        
        String spectralDatabasePath = (String)fitDef.get( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        
        m_spectrumPath = spectralDatabasePath;
        
        int i = 0;
        Iterator it = m_spectrumFileNames.iterator( );
        
        m_currentSpectrumToFit = m_spectralDataSeries.getAverageOfInstanceRangeSpectrum( m_minimumOfSpectrumSeries, m_maximumOfSpectrumSeries );
            
        m_currentFileName = (String)it.next( );
            
        int fileNamePos = m_currentFileName.lastIndexOf( System.getProperty( "file.separator" ) ) + 1;
            
        m_currentFileName = m_currentFileName.substring( fileNamePos );
        
        m_startParameterFileName = spectralDatabasePath + System.getProperty( "file.separator" ) + m_currentFileName.replaceAll( " ","" ) + ".par"; 
        m_currentFileName = m_currentFileName.replaceAll( " ","" ) + ".bin";
        
        TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        modSpec.createStartingValueFileToPath(  m_startParameterFileName );
        
        m_currentSpectrumToFit.writeAsTdfdfitFile( spectralDatabasePath.replaceAll( " ", "" ),  m_currentFileName );
            
        performSpectrumFit( );
        showFitResults( "average [" +  m_minimumOfSpectrumSeries + ", " + m_maximumOfSpectrumSeries + "]" );
    }
    
    
    /**
     * Replaces all blanks by "\  " in order to be interpreted 
     * correctly by the operating system
     * @param cmd 
     * @return 
     */
    private String generateProperCommand( String cmd )
    {
        String out = "";
        for ( int i = 0; i < cmd.length( ); i++ )
        {
            if ( cmd.substring( i, i + 1 ).compareTo( "\\" ) == 0 )
            {
                out += "\\" + "\\";
            }
            else
            {
                out += cmd.substring( i, i + 1 );
            }
        }
        return out;
    }
    
    
    /**
     * Performs the fit and keeps the interface responsive
     */
//    public void startSpectrumFit( )
//    {
//       
//       final SwingWorker worker = new SwingWorker( )
//       {
//            public Object construct( )
//            {
//                try
//                {
//                    performSpectrumSeriesFit( );
//                }
//                catch ( OutOfMemoryError e )
//                {
//                    Dialogs d = new Dialogs( );
//                    d.messageBox( m_parent, "<HTML> An out of memory error occurred  <BR> Close unused windows to free memory </HTML>" );
//                }
//                return null;
//            }
//                
//            // runs in the event dispatching thread
//            public void finished( )
//            {
//                showFitResults( );
//            }
// 
//        };
//        
//        worker.start( );
//    }
    
    /**
     * Starts the fit that corresponds to the current status of the model
     * and the loaded measured spectrum
     */
    private void performSpectrumFit( )
    {
        int returnValue;
        String cmd;

        TDFDFittingDefaults fittingDefaults = TDFDFittingDefaults.getInstance( );
        fittingDefaults.load( );
        
        String tdfdfitExecutablePath = fittingDefaults.getProperty( TDFDFittingDefaults.TDFDFIT_INSTALLATION_PATH );
        String tdfdfitExecutableName = fittingDefaults.getProperty( TDFDFittingDefaults.TDFDFIT_EXECUTABLE_NAME );
        
        cmd = tdfdfitExecutablePath + System.getProperty( "file.separator" )
            + tdfdfitExecutableName + " " + m_spectrumPath 
            + " " + m_currentFileName + " " + m_startParameterFileName;

        cmd = generateProperCommand( cmd );
               
        System.out.println( "Trying following system command : " + cmd );
        try 
        {
            Process p = Runtime.getRuntime( ).exec( cmd );
            
            InputStream in = p.getInputStream( );
            int c;
            while ( ( c = in.read( ) ) != -1 ) 
            {
                System.out.print( (char)c ) ;
            }

            returnValue = p.waitFor( );
            in.close( );
	}
        catch ( IOException ioX ) 
        {
            // not handled yet
            System.out.println( "IOException occurred" );
        }
        catch ( InterruptedException intX ) 
        {
            // not handled yet
            System.out.println( "InterruptedException occurred" );
	}
        
    }
    
    /**
     * Shows the results of the fit in a separate frame.
     */
    private void showFitResults( String instanceNumber )
    {
        MRSignalProperties mrSignalProperties = MRSignalProperties.getInstance( );
        
        String title = m_currentSpectrumToFit.getDicomHeaderInfo( ).getPatientName( ) + " (" +
                       m_currentSpectrumToFit.getDicomHeaderInfo( ).getPatientBirthDate() + ") - " + 
                       m_currentSpectrumToFit.getDicomHeaderInfo( ).getSeriesDescription( ) + " -  Series # : " + 
                       m_currentSpectrumToFit.getDicomHeaderInfo( ).getSeriesNumber( ).trim( ) + " - " +
                       "( " + instanceNumber + " )" +
                       m_currentSpectrumToFit.getDicomHeaderInfo( ).getAcquisitionDate( );
        
        FitResultViewerFrame fitViewer = new  FitResultViewerFrame( mrSignalProperties.getNumberOfSamplePoints( ), m_spectrumPath + System.getProperty( "file.separator" ) + m_currentFileName, title, true );
    }
    
    
    // Attributes of the class
    private JFrame m_parent;
    private String m_patientName;
    private LinkedList<String> m_spectrumFileNames;
    private String m_spectrumPath;
    
    private String m_startParameterFileName;
    private SingleVoxelSpectralDataSeries  m_spectralDataSeries;
    
    private SingleVoxelSpectralData m_currentSpectrumToFit;
    private String m_currentFileName;
    
    private int m_minimumOfSpectrumSeries;
    private int m_maximumOfSpectrumSeries;
}
