/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.Point2D;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import org.spectrim.lib.DualProgressBar;
import org.spectrim.model.GlobalModel;
import org.spectrim.spectra.Spectra;
import org.spectrim.spectra.Spectrum;

/**
 *
 * @author johan
 */
public class NonInteractiveBaselineRemovalExecuter implements ActionListener 
{
    
    // Attributes
    private final GlobalModel               m_model;
    private int                             m_numberOfBaselineRemovalThreadsActive;
    private boolean                         m_multiThreaded;
    private int                             n_cores = 1;
    private final JFrame                    m_parent;
    private final  Vector<Point2D.Double>   m_splineNodes;
    private float[]                         m_xValuesForInterpolation; 
    private long                            startComputationTime;
    private Vector<Object>                  m_specStringKeyVec;
    private Vector<Object>                  m_specVec;
    private int                             m_cntr;
    private int                             m_numberOfThreadsActive;
    private DualProgressBar                 m_dualProgressBar;
    private int                             m_numberOfSpectraToProcess;
    private  Vector<BaselineRemoverSwingWorker>    m_workers = new Vector<BaselineRemoverSwingWorker>( );
    private int                             m_numberOfFitTreadsStarted;
    private boolean                         m_forceProcessingToQuit;
    private int                             m_isPerformedMethodCalled;
    public final static int                 ALL_BASELINEREMOVAL_THREADS_DONE = 0;
    public final static String              ALL_BASELINEREMOVAL_THREADS_DONE_CMD = "ALL_BASELINEREMOVAL_THREADS_DONE";
    
    /**
     *
     * @param frame
     * @param model
     * @param splineNodes
     * @param xValuesForInterpolation
     */
    public NonInteractiveBaselineRemovalExecuter( JFrame frame, GlobalModel model, 
                                                  Vector<Point2D.Double> splineNodes, 
                                                  float[] xValuesForInterpolation )
    {
        m_model         = model;
        m_parent        = frame;
        m_splineNodes   = splineNodes;
        n_cores = Runtime.getRuntime( ).availableProcessors( );
        m_xValuesForInterpolation = xValuesForInterpolation;
        
        System.arraycopy( xValuesForInterpolation, 0, m_xValuesForInterpolation, 0, xValuesForInterpolation.length );
        
    }
    
    /**
     * 
     * Starts the fitting process of the CSIData object, using the model file
     * stored in TDFDFitModelFileReference String object.
     * 
     * @param multiThreaded
     * @return 
     */
    public boolean executeBaselineRemoval( boolean multiThreaded )
    {
        
        m_multiThreaded = multiThreaded;
        boolean processingPerformedSuccessfully = false;
  
        if ( m_multiThreaded == true )
        {
            performMultithreadedBaselineRemoval( );
        }
        else
        {
            performSingleThreadedBaselineRemovalMode( );
        }

        return processingPerformedSuccessfully; 
    }
     
    private synchronized boolean performBaseLineRemovalInMultiThreadMode( )
    {
        startComputationTime = System.currentTimeMillis( );
        n_cores = Runtime.getRuntime( ).availableProcessors( );
                
        //System.out.println( "Number of cores          : " + n_cores );
        Spectra spectra = m_model.getSpectra( );
        // System.out.println( "Number of spectra to fit : " + spectra.getVectorSize( ) );
        
        HashMap<String, Spectrum> data =  spectra.getData( );
        Iterator it = data.entrySet( ).iterator( );
        
        // Hashmap is not thread safe, first copy to a Vector<String>
        // and a Vector<Spectrum> 
        
        m_specStringKeyVec = new Vector<>( );
        m_specVec = new Vector<>( );
        ArrayList<String> selVoxels = m_model.getSelectedVoxels( );
        
        for ( int i = 0; i < selVoxels.size( ); i++ )
        {
            String key = selVoxels.get( i );
            Spectrum spectrum = data.get( selVoxels.get( i ) );
            
            m_specVec.add( spectrum );
            m_specStringKeyVec.add( key );
        }
        
        m_numberOfSpectraToProcess = m_specVec.size( );
                
        // Inititialize
        m_cntr = 0;
        m_numberOfThreadsActive = 0;
  
//        Date date = new Date( );
        // System.out.println( "Timestamp : " + new Timestamp( date.getTime( ) ) );
        String fileNameBase;
        
        m_dualProgressBar = new DualProgressBar( m_parent, 
                                                 n_cores, 
                                                 m_numberOfSpectraToProcess, 
                                                 new Dimension( 900, 150 ), true );
        
        m_dualProgressBar.setTitle( "Spline-based baseline removal .."  );
        m_dualProgressBar.setUpperLabel( "<HTML><BR>     Number of threads active : " + m_numberOfThreadsActive  + " (maximal number of threads : " + n_cores + ")</HTML>" );
        m_dualProgressBar.setUpperProgress( 0 );
        // m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( true );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Finished number of spectra to ptocess : " + 0 + "</HTML>" );
        
        m_dualProgressBar.addActionListener( this );
        
        while ( m_cntr < m_numberOfSpectraToProcess )
        {
            try 
            {
                Thread.sleep( 1 );
            } 
            catch ( InterruptedException ex ) 
            {
                Logger.getLogger(NonInteractiveBaselineRemovalExecuter.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
            
            if ( m_numberOfThreadsActive < n_cores )
            {
               
                m_numberOfThreadsActive++;
                m_cntr++;
                BaselineRemoverSwingWorker worker = new BaselineRemoverSwingWorker( m_cntr,  
                                                                                    (String)m_specStringKeyVec.get( m_cntr ), 
                                                                                    m_model.getSpectra( ).getVectorSize( ), 
                                                                                    (Spectrum)m_specVec.get( m_cntr ), 
                                                                                    m_splineNodes,
                                                                                    m_xValuesForInterpolation );
                m_workers.add( worker );
                
                
                m_numberOfFitTreadsStarted++;
                worker.addActionListener( this );
               
                worker.execute( );
                
                
                m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
            } 
        }
        
        // Date date2 = new Date( );
        // System.out.println( "Timestamp : " + new Timestamp( date2.getTime( ) ) );
         
        return true;
    }
    
    /**
     * This method is necessary to keep SpectrIm responsive during the fitting
     * of the spectra in parallel.
     */
    public synchronized void performMultithreadedBaselineRemoval( )
    {
        javax.swing.SwingWorker<Void, Void> worker = new javax.swing.SwingWorker<Void, Void>( ) 
        {
            @Override
            protected Void doInBackground( ) throws Exception 
            {
                performBaseLineRemovalInMultiThreadMode( );
                return null;
            }

            @Override
            protected void done( ) 
            {
                m_dualProgressBar.dispose( );
                notifyActionListeners( ALL_BASELINEREMOVAL_THREADS_DONE, ALL_BASELINEREMOVAL_THREADS_DONE_CMD );
            }
	};
	worker.execute( );
    }
    
    
    /**
     * This method does the actual fitting using one single additional 
     * java threads.
     */
    private boolean performSingleThreadedBaselineRemovalMode( ) 
    {
         
        // m_currentNumberOfParallelThreads = 1;
       
//        Spectra spectra = m_csiData.getSpectra( );
//        
//        HashMap<String, Spectrum> data =  spectra.getData( );
//        Iterator it = data.entrySet( ).iterator( );
//        
//        int cntr = 0;
//        m_numberOfThreadsActive = 1;
//        
//        while ( it.hasNext( ) ) 
//        {
//            if ( m_numberOfThreadsActive < m_currentNumberOfParallelThreads )
//            {
//                Map.Entry pair = (Map.Entry)it.next( );
//
//                // System.out.println( pair.getKey( ) + " = " + pair.getValue( ) );
//                Spectrum spectrum = (Spectrum) pair.getValue( );
//                float[] absSpectrum = spectrum.getAbsorptionTD( );
//                float[] dispSpectrum = spectrum.getDispersionTD( );
//
//                // There is a difference in internal representation of SpectrIm
//                // and the representation within the TDFDFit part:
//                // Therefore we must adapt the arrays to get proper represenatation.
//                for ( int i = 0; i < spectrum.getAbsorptionTD( ).length; i++ )
//                {
//                    absSpectrum[i] = absSpectrum[i];
//                    dispSpectrum[i] = -dispSpectrum[i];
//                }
//                System.out.println( "CISDATA file name : " + m_csiData.getPatientName( ) + "_" + m_csiData.getSeriesDescription( ) + "_" + m_csiData.getStudyDate( ) + "_" + cntr );
//                String fileNameBase = m_csiData.getPatientName( ) + "_" + m_csiData.getSeriesDescription( ) + "_" + m_csiData.getStudyDate( ) + "_" + cntr;
//                
//                SingleVoxelMeasuredSpectralData dataToFit = new SingleVoxelMeasuredSpectralData( absSpectrum, dispSpectrum );
//                TDFDFitExecutorSwingWorker worker = new TDFDFitExecutorSwingWorker( cntr, dataToFit, true, fileNameBase );                
//                worker.addActionListener( this );
//                m_workers.add( worker );
//                
//                worker.execute( );
//                
//                m_numberOfThreadsActive++;
//                // System.out.println( "Number of threads ++ : " + m_numberOfThreadsActive );
//                
//            }
//        }
        
        return true;
    }
    
  
    /**
     * Adds an ActionListener to the list of listeners.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
    
    public float[] getBaselineCorrectedSpectrumRealPart( )
    {
        return m_correctedAbsorption;
    }
    
    public float[] getBaselineCorrectedSpectrumImagPart( )
    {
        return m_correctedDispersion;
    }
     
    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( int actionID, String actionCommand )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, actionID, actionCommand ) );
        }
    }
    
    public String getKey( ) 
    {
        return m_key;
    }

    // Attributes of BaselineRemovalInThread
    String      m_key;
    int         m_samples;
    float[]     m_phaseDisplayed;
    float[]     m_correctedAbsorption;
    float[]     m_correctedDispersion;
    float[]     m_magnitudeDisplayed;
    double[]    m_xInterpolPoints; 
    float[]     m_xInterpolPointsFlt;
    double[]    xaxis;
    double[]    y;
    
    LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    public final String     BASELINE_CORRECTTION_DONE_CMD = "baseLineCorrectionDone";
    public final int        BASELINE_CORRECTTION_DONE_ID = 0;    


    @Override
    public void actionPerformed( ActionEvent ae ) 
    {
        if ( ae.getSource( ) == m_dualProgressBar )
        {
            if ( ae.getID( ) == m_dualProgressBar.FORCE_QUIT )
            {
                System.out.println( "Force processing to quit pressed! .." );
                // test code
                notifyActionListeners( BASELINE_CORRECTTION_DONE_ID, BASELINE_CORRECTTION_DONE_CMD );
                m_dualProgressBar.setVisible(false);
                m_forceProcessingToQuit = true;
            }
        }
        
        if ( ae.getSource( ) instanceof BaselineRemoverSwingWorker )
        {
            if ( ae.getID( ) == BaselineRemoverSwingWorker.IS_PERFORMED )
            {
                m_isPerformedMethodCalled++;
                
                int number = ( (BaselineRemoverSwingWorker)ae.getSource( ) ).getThreadID( );

                String key = ( (BaselineRemoverSwingWorker)ae.getSource( ) ).getKey( );
                m_model.getSpectra( ).setSpectrum( ( (BaselineRemoverSwingWorker)ae.getSource( ) ).getCorrectedSpectrumReal( ),
                                                   ( (BaselineRemoverSwingWorker)ae.getSource( ) ).getCorrectedSpectrumImag( ), key );
                
                
                m_dualProgressBar.setLowerProgress( m_cntr - m_numberOfThreadsActive );
                m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
                
                m_dualProgressBar.setUpperLabel( "<HTML><BR>     Number of threads active     : " + m_numberOfThreadsActive  + " (maximal number of threads : " + n_cores + ")</HTML>" );
                m_dualProgressBar.setLowerLabel( "<HTML><BR>     Finished number of spectra : " + ( m_cntr - m_numberOfThreadsActive ) + "</HTML>" );
                
                m_workers.remove( ( BaselineRemoverSwingWorker)ae.getSource( ) );
                m_numberOfThreadsActive--;
                                
                if ( ( m_workers.size( ) == 0 ) && ( m_isPerformedMethodCalled == m_numberOfSpectraToProcess ) )
//              if (  m_numberOfFitTreadsStarted == m_numberOfSpectraToFit  )
                {
                    long allWorkersStartedTime = System.currentTimeMillis( );
                    long computationTime = allWorkersStartedTime - startComputationTime;
                    long timePerFit = computationTime / ( m_numberOfSpectraToProcess - m_workers.size( ) );
                    int aliveWorkers =  m_workers.size( );
                    
                    try 
                    {
                        Thread.sleep( ( m_workers.size( ) + 6 ) * 150 );
                        // After waiting terminate the workers
                        for ( int i = 0; i < aliveWorkers; i++ )
                        {
                            BaselineRemoverSwingWorker worker = m_workers.elementAt(i);
                            worker.cancel( true );
                        }
                    } 
                    catch ( InterruptedException ex )
                    {
                        Logger.getLogger(NonInteractiveTDFDFitExecuter.class.getName()).log(Level.SEVERE, null, ex);
                    }
                    m_dualProgressBar.dispose( );
                }
            }
        }
    }
}
