/*
 * File: $Id: QuickSortFloat.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 11.05.2005, 8:08  Authors: Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2005 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.tdfdfit;

/**
 *
 * @author  hans
 */
public class QuickSortFloat 
{
        
    /** 
     * Creates a new instance of QuickSortFloat 
     */
    public QuickSortFloat( ) 
    {
    }
    
    /**********************************************************************
     * Swaps two elements of an array of ints.
     *
     * Parameters:
     *   array: the array
     *   index1: the index of the first element to swap
     *   index2: the index of the second element to swap
     *********************************************************************/
    static void swap( float array[], int index1, int index2) 
    {
        float temp = array[index1];
        array[index1] = array[index2];
        array[index2] = temp;
    } // end swap

    /*********************************************************************
     * Recursive method to sort a section of an array of ints in non-decreasing 
     * order.  Uses the quicksort algorithm.  
     *
     * Parameters: 
     *   data: the array to be sorted
     *   first: the first index of the section of the array to sort
     *   last: the last element of the section of the array to sort
     *********************************************************************/
    static void quickSort( float data[], int first, int last ) 
    {
        // Base Case: if this array section has 0 or 1 element, it is already sorted.
        if ( first >= last )
        {
            return;
        }
        // swap the middle element into the first position for use as the pivot value
        swap( data, first, (first + last) / 2 );
        float pivot = data[first]; // the pivot value
   
        // Indexes for moving elements in the array.  
        int lower = first+1; 
        int upper = last;
   
        // Our goal is to move lower up and move upper down until they meet.
        // We want to swap elements around so that data[first..lower] <= pivot
        // and data[upper..last] > pivot
        while ( lower <= upper ) 
        {
            // move lower up until we find an element which is > pivot
            while ( data[lower] < pivot ) // correction: my original code had "<="
            {
                lower++;
            }
            // move lower down until we find an element which is <= pivot
            while ( data[upper] > pivot )
            {
                upper--;
            }
            // At this point one of two things have happend:
            //    1. data[lower] >= pivot >= data[upper].
            //       In this case, we swap elements lower and upper.
            //    2. We have run out of elements, so lower >= upper.
            //       In this case, our job is done and we increment
            //       lower to make sure the loop will end.
            if ( lower < upper ) 
            {
                swap( data, lower, upper );
                lower++;
                upper--;
            }
            else 
            {
                lower++;
            } // end if
        } // end while
   
        // Now upper is the index of the position *before* the first
        // element that is > pivot.  This is exactly where the pivot
        // value should go.
   
        // swap the pivot value into its proper place in the array
        swap( data, upper, first );
   
        // sort the two sub-arrays (elements before & after the pivot)
        quickSort( data, first, upper - 1 );
        quickSort( data, upper+1, last );
    } // end three-parameter recursive quickSort



   /*********************************************************************
    * Sorts an array of ints in non-decreasing order.  Uses the quicksort 
    * algorithm.  This is the top-level method: sets things up and
    * then calls the recursive method that sorts a section of an array.
    *
    * Parameter: the array to be sorted
    *********************************************************************/
    public static float[] quickSort( float data[] ) 
    {
        if ( data.length < 2 ) return data; // already sorted
   
        // Move the largest element of the array into the last position.
        // This saves lots of extra checks in the recursive version.
   
        int maxIndex = 0;
        for ( int i = 1; i < data.length; i++ )
        {
            if ( data[i] > data[maxIndex] )
            {
                maxIndex = i;
            }
        }
        
        swap( data, maxIndex, data.length - 1 );
   
        // Now use the recursive version to sort the rest of the array.
        quickSort( data, 0, data.length - 2 );
        
        return data;
    } // end one-parameter quickSort

    /*********************************************************************
     * Main program for testing
     *********************************************************************/
    public static void main( String args[] ) 
    {
        float testArray[][] = new float[1024][2048];
        
        System.out.println( "array before sorting : ");
        for (int i = 0; i < 1024; i++ )
        {
            for ( int j = 0; j < 2047; j++ ) 
            {
                testArray[i][j] = (float)( 5 * Math.random( ) + 1.0f );
            }
        }
        
        long start = System.currentTimeMillis( );
        System.out.println( "Current time (start) : " + start );
        
        for (int i = 0; i < 1024; i++ )
        {
            quickSort( testArray[i] );
        }
        
        long stop = System.currentTimeMillis( );
        
        System.out.print( "Current time (stop) : " + stop );
        System.out.println( );
        System.out.println("Time needed is " + ( stop -start ) + " msec" );
        
    } // end main
    
} // end class QuickSort
