/*
 * File: $Id: SignalTools.java,v 1.1.1.1 2006/04/19 09:38:50 hans Exp $
 * History
 * Date: 30.11.2002  Authors: Martin Bürki / Hans Slotboom  
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.tdfdfit;


import org.spectrim.xyplot.Curve;
import org.spectrim.xyplot.XYPlotFrame;
import java.awt.Color;

 
public final class SignalTools
{


    /**
     * Default constructor.
     */
    public SignalTools( )
    {

    }

    public static final double CONSTANT_TOLERANCE = 1e-3;

    /**
     * Returns the fragment of an array y beginning at starting index beginIndex
     * and ending at endIndex.
     * @param y
     * @param beginIndex
     * @param endIndex
     * @return 
     */
    public static float[] getArrayFragment( float[] y, int beginIndex, int endIndex ) 
    {	
        float[] outArr = new float[ endIndex - beginIndex ];
	
        for ( int k = beginIndex; k < endIndex; k++ ) 
        {
            outArr[k-beginIndex] = y[k];  
        }
        
        return outArr;
    }

    /**
     * Returns the median value of the integer array values[].
     * @param values
     * @return
     */
    public static int getMedian( int[] values )
    {
        values = QuickSort.quickSort( values );

        return values[ values.length / 2 ];
    }

    /**
     * Return the float average value.
     * @param values
     * @param maxValues
     * @return
     */
    public static float getAverage( int[] values )
    {
        int sum = 0;
        for ( int k = 0; k < values.length; k++ )
        {
            sum += values[k];
        }

        return (float)sum/values.length;
    }


    /**
     * Returns the fragment of an array y beginning at starting index beginIndex
     * and ending at endIndex.
     * @param y
     * @param beginIndex
     * @param endIndex
     * @return 
     */
    public static double[] getArrayFragment( double[] y, int beginIndex, int endIndex ) 
    {	
        double[] outArr = new double[ endIndex - beginIndex ];
	
        for ( int k = beginIndex; k < endIndex; k++ ) 
        {
            outArr[k-beginIndex] = y[k];  
        }
        
        return outArr;
    }

    /**
     * Returns the fragment of an array y beginning at starting index beginIndex
     * and ending at endIndex.
     * @param y
     * @param beginIndex
     * @param endIndex
     * @return 
     */
    public static int[] getArrayFragment( int[] y, int beginIndex, int endIndex )
    {
        int[] outArr = new int[ endIndex - beginIndex ];

        for ( int k = beginIndex; k < endIndex; k++ )
        {
            outArr[k-beginIndex] = y[k];
        }

        return outArr;
    }
    
    /**
     * Integrates an array y sampled at distances h using Eulers' method.
     * @param y
     * @param beginIndex
     * @param endIndex
     * @return 
     */
    public static float summation( float[] y, int beginIndex, int endIndex ) 
    {	
        int N = y.length;
	
        float sum = 0;
	
        for ( int k = beginIndex; k < endIndex; k++ ) 
        {
            sum += y[k];  
        }
        
        return sum;
    }
    
    /**
     * Integrates an array y after correcting the points y for a linear offset
     * determined for the first 3 and last 3 ending points
     * The first 3 together with the last 3 points are used dor a linear regression
     * The fitted line is subtracted from the array y. This difference signal
     * is than simply integrated (summed).
     *
     * @param y
     * @param beginIndex
     * @param endIndex
     * @return 
     */
    public static float summationWithBaselineCorrection( float[] y, int beginIndex, int endIndex ) 
    {	
        int N = y.length;
	
        float sum = 0;
        
        double[] x_in = new double[6];
        double[] y_in = new double[6];
        
        x_in[0] = beginIndex - 1;
        x_in[1] = beginIndex;
        x_in[2] = beginIndex + 1;
        
        x_in[3] = endIndex - 1;
        x_in[4] = endIndex;
        x_in[5] = endIndex + 1;
        
        y_in[0] = y[ beginIndex - 1 ];
        y_in[1] = y[ beginIndex ];
        y_in[2] = y[ beginIndex + 1];
        
        y_in[3] = y[ endIndex - 1 ];
        y_in[4] = y[ endIndex ];
        y_in[5] = y[ endIndex + 1 ];
        
        MRLinearRegression regression = new MRLinearRegression( x_in, y_in, false );
	
        double[] corrected_x = new double[ endIndex - beginIndex ];
        double[] corrected_y = new double[ endIndex - beginIndex ];
        double[] uncorrected_y = new double[ endIndex - beginIndex ];
        
        int cntr = 0;
        
        for ( int j = beginIndex; j < endIndex; j++ )
        {
          corrected_x[cntr] = (double)j;
          corrected_y[cntr] = y[j] - regression.getYvalue( j );
          uncorrected_y[cntr] = y[j];
          cntr++;
        }
        
        for ( int k = 0; k < cntr; k++ ) 
        {
            sum += corrected_y[k];  
        }
        
        XYPlotFrame frame = new XYPlotFrame( "corrected curve", null );
        
        frame.addCurve( new Curve( corrected_x, corrected_y ,Color.BLACK ), true );
        frame.addCurve( new Curve( corrected_x, uncorrected_y ,Color.BLACK ), true );
        
        
        return sum;
    }
    
    
    /**
     * Integrates an array y sampled at distances h using Eulers' method.
     * @param y
     * @param h
     * @return 
     */
    public static float integrate2( float[] y, float h ) 
    {	
	int N = y.length;
	
	float sum = 0;
	for ( int k = 0; k < N; k++ ) 
        {
	    if ( k == 0 || k == N-1 )
            { 
                sum += 0.5 * y[k];
            }
	    else
            { 
                sum += y[k];
            }
	}
	return h * sum;
    }
    
    /**
     * Integrates an array y sampled at distances h using an improved algorithm.
     * @param y
     * @param h
     * @return 
     */
    public static float integrate3( float[] y, float h ) 
    {
	int N = y.length;

	float sum = 0;
	for ( int k = 0; k < N; k++ ) 
        {
	    if ( k==0 || k==N-1 )
            { 
                sum += 5 / 12 * y[k];
            }
	    else if ( k == 1 || k == N - 2 )
            { 
                sum += 13 / 12 * y[k];
            }
	    else
            { 
                sum += y[k];
            }
	}
	return h * sum;
    }

    /**
     * Returns the baseline begin an end indices of an array named 'signal'.
     * @param signal
     * @return 
     */
    public static int[] getBaseline( float[] signal ) 
    {

	float ds_threshold = 10f;

	int N = signal.length;
	float[] ds = new float[N-1];

	for ( int k = 0; k < N - 1; k++ ) 
        {
            ds[k] = signal[k+1] - signal[k];
        }

	// determine baselineStartIndex
	int baselineStartIndex = 0;
	while ( baselineStartIndex < N - 1 ) 
        {
	    if ( Math.abs(ds[baselineStartIndex]) > ds_threshold)
            {
                baselineStartIndex++;
            }
	    else
            {
                break;
            }
	}
        
	// determine baselineEndIndex
	int baselineEndIndex = baselineStartIndex+1;
	while ( baselineEndIndex < N - 1 ) 
        {
	    if ( Math.abs(ds[baselineEndIndex]) < ds_threshold )
            { 
                baselineEndIndex++;
            }
	    else
            {
                break;
            }
	}
	// System.out.println( "start baseline index" + baselineStartIndex + "end baseline index" + baselineEndIndex);
	return ( new int[] { baselineStartIndex, baselineEndIndex } );
    }
    
    /**
     * Returns true when float array y is constant (within a given tolerance)
     * else false.
     * @param y
     * @return 
     */
    public static boolean isConstant( float[] y ) 
    {
	float l = (float) y.length;
	float[] y2 = new float[y.length];
	float m = mean(y);
	float s = 0f;
	for ( int k = 0; k < y2.length; k++ ) 
        {
	    y2[k] = (y[k] - m)/l;
	    s += y2[k];
	}
	return ( Math.sqrt( s ) < CONSTANT_TOLERANCE );
    }

    /**
     * Returns the array index of the coordinate containing the maximum value.
     * @param y
     * @return 
     */
    public static int getIndexOfMax( float[] y ) 
    {
	int index = 0;
	float maxVal = y[0];

	for ( int k = 0; k < y.length; k++ ) 
        {
	    if ( y[k] > maxVal ) 
            {
		maxVal = y[k];
		index = k;
	    }
	}
	return index;
    }
       
    /**
     * Returns the cubic interpolated array index of the 
     * coordinate of the maximum value.
     * @param y
     * @return 
     */
    public static float getCubicInterpolatedIndexOfMax( float[] y ) 
    {
	int index = 0;
        float f_index = 0f;
        
	float maxVal = y[0];
        
        float a = 0f;
        float b = 0f;
        float c = 0f;
        float fa = 0f;
        float fb = 0f;
        float fc = 0f;
	
        for ( int k=0; k<y.length; k++ ) 
        {
	    if ( y[k] > maxVal ) 
            {
		maxVal = y[k];
		index = k;
	    }
	}
        try
        {
            if ( ( index < y.length - 1 ) && ( index > 1 ) )
            {
                a  = index - 1;
                b  = index;
                c  = index + 1;
                fa = y[index-1];
                fb = y[index];
                fc = y[index+1];
            
                if ( ( ( fa != fb ) && ( fb != fc ) && ( a != b ) && ( b != c ) ) )
                {
                    f_index = b - ( ( b - a ) * ( b - a ) * ( fb - fc ) - ( b - c ) * ( b - c ) * ( fb - fa ) ) / ( ( 2f * ( b - a ) * ( fb - fc ) ) - ( ( b - c ) * ( fb - fc ) ) );
                }
                else
                {
                    f_index = (float)index;
                }
            
                if ( ( f_index < (float)a ) || (f_index > (float)c ) )
                {
                    f_index = (float)index;
                }
            
            }
            else
            {
                f_index = (float)index;
            }
        }
        catch ( ArithmeticException ex )
        {
            f_index = b;
        }
        
	return f_index;
        
    }
    
    /**
     * Returns the array index of the coordinate containing the minimum value.
     * @param y
     * @return 
     */
    public static int getIndexOfLocalMinimum( float[] y ) 
    {
	float dy1,dy2;
	int k;

	for ( k = 1; k < y.length - 1; k++ ) 
        {
	    dy1 = y[k] - y[k-1];
	    dy2 = y[k+1] - y[k];
	    if ( dy1 < 0f && dy2 > 0f ) break;
	}
	
	if ( k == y.length - 1 )
        { 
            return -1;
        }
	else
        { 
            return k;
        }
    }

    /**
     * Clips the array x before index value 'start' and after index value 'end'.
     * @param x
     * @param start
     * @param end
     * @return 
     */
    public static float[] clip( float[] x, int start, int end ) 
    {
    	float[] xc = new float[ end - start + 1];
		System.arraycopy( x, start, xc, 0, xc.length );
		return xc;
    }

    
    /**
     * Clips the array x before index value 'start' and after index value 'end'.
     * @param x
     * @param start
     * @param end
     * @return 
     */
    public static double[] clip( double[] x, int start, int end ) 
    {
    	double[] xc = new double[ end - start + 1];
    	System.arraycopy( x, start, xc, 0, xc.length );
    	return xc;
    }
    
    
    /**
     * Returns an array widths starting value 'start' end value 'end' and each
     * element in between spaced by a value 'step'.
     * @param start
     * @param end
     * @param step
     * @return 
     */
    public static float[] linspace( float start, float end, float step ) 
    {

        float[] x = new float[(int)Math.floor((double)(start - end)/step)+1];

        for ( int k = 0; k < x.length; k++ )
        {
            x[k] = start + k * step;
        }
        return x;
    }
    
    /**
     * Returns an array of length 'len' having starting value 'start' and each
     * element spaced by a value 'step'.
     * @param start
     * @param step
     * @param len
     * @return 
     */
    public static float[] linspace( float start, float step, int len ) 
    {

        float[] x = new float[len];

        for ( int k = 0; k < x.length; k++ )
        {
            x[k] = start + k * step;
        }
        return x;
    }
    
    /**
     *  Calculates the concentration curve of y[k].
     * @param y
     * @param start
     * @param end
     * @param c
     * @return 
     */
    public static float calculateConcentrationCurve( float[] y, int start, int end, float c ) 
    {
        float mean = 0f;
	
        // compute mean value of baseline
        for ( int k = start; k < end; k++ )
        {
            mean += y[k];
        }
        mean /= (float)( end - start );

        // subtract the mean and invert the sign (y -> -(y-mean) = -y+mean)
        for ( int k = 0; k < y.length; k++ )
        {
            y[k] = c * (float)Math.log( (double)( y[k] / mean ) );
        }
                
        return mean;
    }
    
    /**
     * Shifts the float array about n index positions.
     * @param y
     * @param n
     * @return 
     */
    public static float[] shift( float[] y, int n ) 
    {
        float[] ys = new float[y.length];
        for ( int k = 0; k < ys.length; k++ )
        {
            if ( k - n < 0 || k - n >= ys.length )
            { 
                ys[ k ] = 0f;
            }
            else
            { 
                ys[ k ] = y[ k - n ];
            }
        }
        return ys;
    }
    
    /**
     * Concatenates array y after array x and stores result in array xy.
     * @param x
     * @param y
     * @param xy
     */ 
    public static void concatenate( float[] x, float[] y, float[] xy ) 
    {
        if ( x.length + y.length != xy.length )
        { 
            throw new IllegalArgumentException( "Size mismatch encountered durung array concatination" );
        }
        System.arraycopy( x, 0, xy, 0, x.length );
        System.arraycopy( y, 0, xy, x.length, y.length );
    }

    /**
     * Returns the concatenated array x followed by y.
     * @param x
     * @param y
     * @return 
     */
    public static float[] concatenate( float[] x, float[] y ) 
    {
        float[] xy = new float[ x.length + y.length ];
        System.arraycopy( x, 0, xy, 0, x.length );
        System.arraycopy( y, 0, xy, x.length, y.length );
        return xy;
    }

    /**
     * Returns the element-wise product of vector y1 and vector y2.
     * @param y1
     * @param y2
     * @return 
     */
    public static float[] product( float[] y1, float[] y2 ) 
    {
        int len = ( y1.length <= y2.length ? y1.length : y2.length );
        float[] y = new float[len];
        for ( int k = 0; k < len; k++ )
        { 
            y[k] = y1[k] * y2[k];
        }
        return y;
    }
    
    /**
     * Calculates element wise product of y1 and y2 and stores it in a new
     * vector y which is returned.
     * @param y1
     * @param y2
     * @param y
     */
    public static void product( float[] y1, float[] y2, float[] y ) 
    {
        if ( y1.length != y2.length ||  y1.length!=y.length )
        { 
            throw new IllegalArgumentException( "Size mismatch" );
        }
        for ( int k = 0; k < y.length; k++ )
        {
            y[k] = y1[k] * y2[k];
        }
    }

    /**
     * Performs zero padding or zero filling of an array y:
     * 'head' number of zeroes are placed before and 'tail' number of zeroes
     * are placed at the tail of the input array y. The zero filled array is
     * returned
     * @param y
     * @param head
     * @param tail
     * @return 
     */
    public static float[] zeropad( float[] y, int head, int tail ) 
    {
        float[] yp = ( float[] )y.clone( );
        if ( head > 0 )
        { 
            yp = concatenate( zeros( head ), yp );
        }
        if ( tail > 0 )
        { 
            yp = concatenate( yp, zeros( tail ) );
        }
        return yp;
    }

    /**
     * Returns a new float array of length 'len' containing zeroes.
     * @param len
     * @return 
     */
    public static float[] zeros( int len ) 
    {
        return new float[len];
    }

    /**
     * Prints a integer array 'data' on standard  output with w rows formatted.
     * @param data
     * @param w
     */
    public static void printIntMatrix( int[] data, int w ) 
    {
        int l = data.length;
        int h = 0;
        for ( int k = 0; k < l; k++ )
        {
            System.out.print( "  " + data[k] );
            h++;
            if ( h == w )
            {
                h=0;
                System.out.println( "" );
            }
        }
    }

    /**
     * Simply prints the float array to standard output space delimited.
     * @param y
     */
    public static void printVector( float[] y ) 
    {
        if ( y == null )
        {
            System.out.println( "[null]" );
            return;
        }
        int l = y.length;
        for ( int k = 0; k < l; k++ )
        {
            System.out.print( " " + y[k] );
        }
        System.out.println( );
    }

    /**
     * Adds two arrays 'y'  and 'add' and stores the result in array y again.
     * @param y
     * @param add
     */
    public static void vectorAdd( float[] y, float[] add ) 
    {
        int n;
        if ( y==null || add==null ) return;
        if ( y.length <= add.length ) n = y.length;
        else n = add.length;
        for ( int k = 0; k < n; k++ )
        {
            y[k] = y[k] + add[k];
        }
    }
    
    /**
     * Adds two arrays and returns the resulting array.
     * @param y
     * @param add
     * @return 
     */
    public static float[] signalAdd( float[] y, float[] add ) 
    {
        int n;
        if ( y == null || add == null ) return null;
        if ( y.length <= add.length ) n = y.length;
        else n = add.length;
        for ( int k = 0; k < n; k++ )
        {
            y[k] = y[k] + add[k];
        }
        
        return y;
    }

    /**
     * Subtracts two arrays and returns the result.
     * @param y
     * @param diff
     * @return 
     */
    public static float[] signalSubtract( float[] y, float[] diff ) 
    {
        int n;
        if ( y==null || diff==null ) return null;
        if ( y.length <= diff.length  ) n = y.length;
        else n = diff.length;
        for ( int k = 0; k < n; k++ )
        {
            y[k] = y[k] - diff[k];
        }
        
        return y;
    }

    /**
     * Adds a constant 'constant' to each element of the input array y2.
     * @param y
     * @param constant
     * @return 
     */
    public static float[] addConstant( float[] y, float constant ) 
    {
        float[] y2 = new float[y.length];
        for ( int k = 0; k < y2.length; k++ )
        { 
            y2[k] = y[k] + constant;
        }
        return y2;
    }

    /**
     * Multiplies the array with a factor s element wise.
     * @param y
     * @param s
     */
    public static void scale( float[] y, float s ) 
    {
        if ( s == 1f ) return;
        for ( int k = 0; k < y.length; k++ )
        {
            y[k] = y[k] * s;
        }
    }
    
    /**
     * Multiplies the array with a factor s element-wise.
     * @param y
     * @param maxValue
     */
    public static void scaleToMaxValue( float[] y, float maxValue ) 
    {
	
        float maxEl = getMaximum( y );
        
        if ( maxEl != 0f )
        {
            for ( int k = 0; k < y.length; k++ )
            {
                y[k] = ( y[k] / maxEl ) * maxValue;
            }
        }
    }
    
    
    /**
     * Scales the vector y with a factor s and returns the resulting array.
     * @param y
     * @param s
     * @return 
     */
    public static float[] scaled( float[] y, float s ) 
    {
	int n = y.length;
	float[] ys = new float[n];
	System.arraycopy( y, 0, ys, 0, n );
	scale( ys, s );
	return ys;
    }

    /**
     * Adds all elements of a float array and returns this sum.
     * @param y
     * @return 
     */
    public static float sum( float[] y ) 
    {
	if ( y == null ) return 0f;
	float sum = 0f;
	for ( int k = 0; k < y.length; k++ ) 
        {
	    sum += y[k];
	}
	return sum;
    }

    /**
     * Adds all elements of a float array and returns this sum
     * @param y
     * @return 
     */
    public static double sum( double[] y ) 
    {
    	if ( y == null ) return 0f;
    	float sum = 0f;
    	for ( int k = 0; k < y.length; k++ ) 
        {
    		sum += y[k];
        }
    	return sum;
    }
    
    
    /**
     * Adds all squared elements of a float array and returns 
     * this sum of squares.
     * @param y
     * @return 
     */
    public static double sum2( double[] y ) 
    {
    	if ( y == null ) return 0f;
		double sum2 = 0f;
		for ( int k = 0; k < y.length; k++ ) 
        {
	    	sum2 += ( y[k] * y[k] );
        }
		return sum2;
    }

    
    /**
     * Adds all squared elements of a float array and returns 
     * this sum of squares.
     * @param y
     * @return 
     */
    public static float sum2( float[] y ) 
    {
    	if ( y == null ) return 0f;
    	float sum2 = 0f;
    	for ( int k = 0; k < y.length; k++ ) 
        {
    		sum2 += ( y[k] * y[k] );
        }
    	return sum2;
    }

    /**
     * Returns the mean of a float array y.
     * @param y
     * @return 
     */
    public static float mean( float[] y ) 
    {
    	if ( y == null ) return 0f;
    	if ( y.length == 0 ) return 0f;
    	return  sum( y ) / (float)y.length;
    }

    
    /**
     * Returns the mean of a float array y.
     * @param y
     * @return 
     */
    public static double mean( double[] y ) 
    {
    	if ( y == null ) return 0f;
    	if ( y.length == 0 ) return 0f;
    	return  sum( y ) / (float)y.length;
    }
    
    
    
    /**
     * Returns the norm value of an float array y.
     * @param y
     * @return 
     */
    public static float norm( float[] y ) 
    {
	return (float)Math.sqrt( ( double )sum2( y ) );
    }
    
    /**
     * Calculates the natural logarithm of y[k].
     * @param y
     * @return 
     */
    public static float[] log_s_s0( float[] y ) 
    {
	int n = y.length;
	float[] s_log = new float[n];
	for ( int k = 0; k < y.length; k++ ) 
        {
	    s_log[k] = (float)Math.log( y[k] );
	}
	return s_log;
    }
    
    /**
     * Returns the maximum value of the array y.
     * @param y
     * @return 
     */
    public static float getMaximum( float[] y )
    {
        float maximum = -Float.MAX_VALUE;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] > maximum )
            {
                maximum = y[i];
            }
        }
        return maximum;
    }

    
    /**
     * Returns the maximum value of the array y
     * @param y
     * @return 
     */
    public static double getMaximum( double[] y )
    {
        double maximum = -Double.MAX_VALUE;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] > maximum )
            {
                maximum = y[i];
            }
        }
        return maximum;
    }
    
    /**
     * Returns the maximum value of the array y.
     * @param y
     * @return 
     */
    public static int getMaximum( int[] y )
    {
        int maximum = -Integer.MAX_VALUE;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] > maximum )
            {
                maximum = y[i];
            }
        }
        return maximum;
    }    
    
    /**
     * Returns the index value of the array y where y has its absolute maximum.
     * @param y
     * @return 
     */
    public static int getMaximumIndex( float[] y )
    {
        float maximum = -Float.MAX_VALUE;
        int maximumIndex = -1;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] > maximum )
            {
                maximum = y[i];
                maximumIndex = i;
            }
        }
        return maximumIndex;
    }

    /**
     * Returns the index value of the array y where y has its absolute maximum.
     * @param y
     * @return 
     */
    public static int getMaximumIndex( double[] y )
    {
        double maximum = -Double.MAX_VALUE;
        int maximumIndex = -1;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] > maximum )
            {
                maximum = y[i];
                maximumIndex = i;
            }
        }
        return maximumIndex;
    }
    
    
    /**
     * Returns the minimum value of the array y float version.
     * @param y
     * @return 
     */
    public static float getMinimum( float[] y )
    {
        float minimum = Float.MAX_VALUE;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] < minimum )
            {
                minimum = y[i];
            }
        }
        return minimum;
    }
    
    /**
     * Returns the minimum value of the array y (double version).
     * @param y
     * @return 
     */
    public static double getMinimum( double[] y )
    {
        double minimum = Double.MAX_VALUE;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] < minimum )
            {
                minimum = y[i];
            }
        }
        return minimum;
    }
    
    
    /**
     * Returns the minimum value of the array y (int version).
     * @param y
     * @return 
     */
    public static int getMinimum( int[] y )
    {
        int minimum = Integer.MAX_VALUE;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] < minimum )
            {
                minimum = y[i];
            }
        }
        return minimum;
    }
    
    
    /**
     * Returns the index value of the array y where y has its absolute minimum.
     * @param y
     * @return 
     */
    public static int getMinimumIndex( float[] y )
    {
        float minimum = Float.MAX_VALUE;
        int minimumIndex = -1;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] < minimum )
            {
                minimum = y[i];
                minimumIndex = i;
            }
        }
        return minimumIndex;
    }
    
    /**
     * Returns the index value of the array y where y has its absolute minimum.
     * @param y
     * @return 
     */
    public static int getMinimumIndex( double[] y )
    {
        double minimum = Double.MAX_VALUE;
        int minimumIndex = -1;
        for ( int i = 0; i < y.length; i++ )
        {
            if ( y[i] < minimum )
            {
                minimum = y[i];
                minimumIndex = i;
            }
        }
        return minimumIndex;
    }
    
    
    
    /**
     * Returns the difference between the array y[n] and an the over 'shift' points 
     * shifted array y[ n - shift]. Usefull for instance to calculate the difference
     * quotient. 
     * @param y
     * @param shift
     * @return 
     */
    public static float[] shiftDifferenceSignal( float[] y, int shift )
    {
        float[] shifted_y = new float[y.length];
        shifted_y = shift( y, 1 );
        float[] diffSignal = new float[y.length];
        for ( int i = 0; i < y.length; i++ )
        {
            diffSignal[i] = y[i] - shifted_y[i];
        }
        
        return diffSignal;
    }
    
    /**
     * Smoothen the array x using a moving average model
     * @param x
     * @return 
     */
    public static float[] threePointsSmoothenSignal( float[] x )
    {
        float[] y = new float[x.length];
        
        y[0] = 0.5f * ( x[0] + x[1] );
        for ( int i = 1; i <= x.length - 2; i++ )
        {
            y[i] = ( x[i-1] + x[i] + x[i+1] ) / 3.0f;
        }
        y[x.length-1] = 0.5f * ( x[x.length-2] + x[x.length-1] );
        
        return y;
    }
    
    /**
     * Smoothen the array x using a moving average model.
     * @param x
     * @return 
     */
    public static double[] threePointsSmoothenSignal( double[] x )
    {
        double[] y = new double[x.length];
        
        y[0] = 0.5f * ( x[0] + x[1] );
        for ( int i = 1; i <= x.length - 2; i++ )
        {
            y[i] = ( x[i-1] + x[i] + x[i+1] ) / 3.0f;
        }
        y[x.length-1] = 0.5f * ( x[x.length-2] + x[x.length-1] );
        
        return y;
    }
    
    /**
     * Smoothen the array x using a moving average model
     * @param x
     * @return 
     */
    public static double[] fivePointsSmoothenSignal( double[] x )
    {
        double[] y = new double[x.length];
        
        y[0] = ( x[x.length-2] + x[x.length-1] + x[0] + x[1] + x[2] ) / 5.0f;
        
        y[1] = ( x[x.length-1] + x[0] + x[1] + x[2] + x[3] ) / 5.0f;
        
        for ( int i = 2; i <= x.length - 3; i++ )
        {
            y[i] = ( x[i-2] + x[i-1] + x[i] + x[i+1] + x[i+2] ) / 5.0f;
        }
        
        y[x.length-2] = ( x[x.length-4] + x[x.length-3] + x[x.length-2] + x[x.length-1] + x[0] );
        y[x.length-2] = ( x[x.length-3] + x[x.length-2] + x[x.length-1] + x[0]  + x[1] );
        
        
        return y;
    }
    
    /**
     * Computes the variance of a signal (float version).
     * @param x
     * @return 
     */
    public static float computeVariance( float[] x )
    {
        float variance = 0f;
        float mean = SignalTools.mean( x );
        
        for ( int i =0; i< x.length; i++ )
        {
            variance += ( x[i] - mean ) * ( x[i] - mean );
        }
        
        return variance;
    }
    
    /**
     * Computes the variance of a signal (double version).
     * @param x
     * @return 
     */
    public static double computeVariance( double[] x )
    {
        double variance = 0f;
        double mean = SignalTools.mean( x );
        
        for ( int i =0; i< x.length; i++ )
        {
            variance += ( x[i] - mean ) * ( x[i] - mean );
        }
        
        return variance;
    }
    
    /**
     * Casts a float vector to a double vector.
     * @param x
     * @return 
     */
    public static double[] toDouble( float[] x )
    {
        double[] returnVector = new double[x.length];
        for ( int i = 0; i< x.length; i++ )
        {
            returnVector[i] = (double)x[i];
        }    
        return returnVector;
    }
    
}
