/*
 * File: $Id: TDFDFitModelTreeWithPopupMenu.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 12.06.2003 2003, 13:38    Authors: Hans Slotboom
 * Changes:
 *
 * Copyright (C) 2003 by
 *  
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */
 

package org.spectrim.tdfdfit;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;

import javax.swing.JComponent;
import javax.swing.JTree;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.JMenu;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeNode;
import javax.swing.tree.DefaultTreeModel;

import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;


import java.util.Iterator;
import java.util.LinkedList;
import java.util.Enumeration;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.lib.JCheckBoxTree;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME;

/**
 * Implementation of a JTree which also has a JPopupMenu.
 * @author  hans
 */
public class TDFDFitModelTreeWithPopupMenu extends JCheckBoxTree implements ActionListener
{
    
    /** 
     * Creates a new instance of JTreeWithPopupMenu. 
     * @param dmtn
     */
    public TDFDFitModelTreeWithPopupMenu( DefaultMutableTreeNode dmtn ) 
    {
        // call base class constructor
        super( dmtn );
        m_rootNode = dmtn;
        constructComponent( );
        setSelectionPath( new TreePath( m_rootNode ) );
        
        // This control should listen to changes in the wish the user wants
        // to display the offset units.
        TDFDFittingDefaults.getInstance( ).addActionListener( this );
        
    }
    
    /** 
     * Creates a new instance of JTreeWithPopupMenu. 
     * @param treeModel
     */
    public TDFDFitModelTreeWithPopupMenu( DefaultTreeModel treeModel ) 
    {
        super( treeModel );
        m_rootNode = (DefaultMutableTreeNode)treeModel.getRoot( );    
        constructComponent( );
        setSelectionPath( new TreePath( m_rootNode ) );
    }
    
    
    /**
     * Clears the tree and sets the JTree in its initial state.
     */
    public void clearJTree( )
    {
        DefaultTreeModel model = (DefaultTreeModel) this.getModel( );
        DefaultMutableTreeNode root = (DefaultMutableTreeNode)model.getRoot( );
        root.removeAllChildren( );
        model.reload( );
        root.setUserObject( "No model defined" );
        
        // m_rootNode = new DefaultMutableTreeNode( );
        // m_rootNode.setUserObject( "No model defined" );
        // model.setRoot( m_rootNode );
        m_rootNode = root;
        
        raiseActionEvent( MODEL_CLEARED, this, "Model cleared" );
    }
    
    
    /**
     * Does the construction of the component.
     */
    private void constructComponent( )
    {
        // Define the popup
        m_popup = new JPopupMenu( );
        
        m_deleteNode = new JMenuItem( "Remove node" );
        m_deleteNode.addActionListener( this );
        m_deleteNode.setActionCommand( MODEL_TREE_NODE_DELETE_EVENT_CMD );
        m_deleteNode.setEnabled( false );
        
        m_popup.add( m_deleteNode );
        
        m_renameNode = new JMenuItem( "Rename node" );
        m_renameNode.addActionListener( this );
        m_renameNode.setActionCommand( MODEL_TREENODE_NAME_CHANGE_EVENT_CMD );
        m_renameNode.setEnabled( false );
        
        m_popup.add( m_renameNode );
        
       
        m_copyNode = new JMenuItem( "Copy node" );
        m_copyNode.addActionListener( this );
        m_copyNode.setActionCommand( "copy node" );
        m_copyNode.setEnabled( false );  
        
        m_popup.add( m_copyNode );
        
        m_pasteNode = new JMenuItem( "Paste node" );
        m_pasteNode.addActionListener( this );
        m_pasteNode.setActionCommand( "paste node" );
        m_pasteNode.setEnabled( false );
        
        m_popup.add( m_pasteNode );
        
        m_popup.addSeparator( );
        
        m_addLineNode = new JMenuItem( "Add line" );
        m_addLineNode.addActionListener( this );
        m_addLineNode.setActionCommand( "add line node" );
        m_addLineNode.setEnabled( false );
        
        m_popup.add( m_addLineNode );
        
        m_numericPatternSubMenu = new JMenu( "Add numeric pattern" );
        m_numericPatternSubMenu.setEnabled( false );
        m_addNumericPatternNodeFromBinFile = new JMenuItem( "Add numeric pattern from TDFDFit binary signal file .." );
        m_addNumericPatternNodeFromBinFile.addActionListener( this );
        m_addNumericPatternNodeFromBinFile.setActionCommand( "add numeric pattern node from bin response" );
        m_addNumericPatternNodeFromBinFile.setEnabled( false );
        
        m_numericPatternSubMenu.add( m_addNumericPatternNodeFromBinFile );
        
        m_addNumericPatternNodeFromNPFFile = new JMenuItem( "Add numeric pattern from .npf file .." );
        m_addNumericPatternNodeFromNPFFile.addActionListener( this );
        m_addNumericPatternNodeFromNPFFile.setActionCommand( "add numeric pattern from npf file" );
        m_addNumericPatternNodeFromNPFFile.setEnabled( false );
        
        m_numericPatternSubMenu.add( m_addNumericPatternNodeFromNPFFile );
        m_numericPatternSubMenu.setEnabled( false );
        m_addNumericPatternNodeFromMRUIFile = new JMenuItem( "Add numeric pattern from .mrui file .." );
        m_addNumericPatternNodeFromMRUIFile.addActionListener( this );
        m_addNumericPatternNodeFromMRUIFile.setActionCommand( "add numeric pattern node from mrui response" );
        m_addNumericPatternNodeFromMRUIFile.setEnabled( false );
        
        m_numericPatternSubMenu.add( m_addNumericPatternNodeFromMRUIFile );
        
        m_popup.add( m_numericPatternSubMenu );
        
        m_parametericPatternSubMenu = new JMenu( "Add parametric pattern" );
        m_parametericPatternSubMenu.setEnabled ( false );
        m_addParametricPatternNodeManually = new JMenuItem( "Add parametric pattern manually" );
        m_addParametricPatternNodeManually.addActionListener( this );
        m_addParametricPatternNodeManually.setActionCommand( "add parametric pattern node" );
        m_addParametricPatternNodeManually.setEnabled( false );
        m_parametericPatternSubMenu.add( m_addParametricPatternNodeManually );
        
        m_addParametricPatternNodeFromFile = new JMenuItem( "Add parametric pattern from .ppf file .." );
        m_addParametricPatternNodeFromFile.addActionListener( this );
        m_addParametricPatternNodeFromFile.setActionCommand( "add parametric pattern node from file" );
        m_addParametricPatternNodeFromFile.setEnabled( false );
        
        m_parametericPatternSubMenu.add( m_addParametricPatternNodeFromFile );
        
        m_popup.add( m_parametericPatternSubMenu );
        
        m_addMetaboliteNode = new JMenuItem( "Add metabolite" );
        m_addMetaboliteNode.addActionListener( this );
        m_addMetaboliteNode.setActionCommand( "add metabolite node" );
        m_addMetaboliteNode.setEnabled( false );
        
        m_popup.add( m_addMetaboliteNode );
        
        m_createStartingValueFiles = new JMenuItem( "Create starting value files" );
        m_createStartingValueFiles.addActionListener( this );
        m_createStartingValueFiles.setActionCommand( "create starting value files" );
        m_createStartingValueFiles.setEnabled( false );
        
        m_popup.add( m_createStartingValueFiles );
        
        m_exportModelAsNumericPatternMenuItem = new JMenuItem( "Export model as numeric pattern (.npf)" );
        m_exportModelAsNumericPatternMenuItem.addActionListener( this );
        m_exportModelAsNumericPatternMenuItem.setActionCommand( EXPORT_AS_NUMERIC_PATTERN_CMD );
        m_exportModelAsNumericPatternMenuItem.setToolTipText( "Writes the current selection to numeric binary pattern (.bin) file" );
        m_exportModelAsNumericPatternMenuItem.setEnabled( false );
        
        m_popup.add( m_exportModelAsNumericPatternMenuItem );
        
        m_exportModelAsMetaboliteMenuItem = new JMenuItem( "Export model as metabolite (.mpf)" );
        m_exportModelAsMetaboliteMenuItem.addActionListener( this );
        m_exportModelAsMetaboliteMenuItem.setActionCommand( EXPORT_AS_METABOLITE_PATTERN_CMD );
        m_exportModelAsMetaboliteMenuItem.setToolTipText( "Writes the current selection to numeric binary pattern (.bin) file" );
        m_exportModelAsMetaboliteMenuItem.setEnabled( false );
        
        m_popup.add( m_exportModelAsMetaboliteMenuItem );
        
        m_exportModelAsParametricPatternMenuItem = new JMenuItem( "Export model as parametric pattern" );
        m_exportModelAsParametricPatternMenuItem.addActionListener( this );
        m_exportModelAsParametricPatternMenuItem.setActionCommand( EXPORT_AS_PARAMETRIC_PATTERN_CMD );
        m_exportModelAsParametricPatternMenuItem.setToolTipText( "Writes the current model to parametric pattern file" );
        m_exportModelAsParametricPatternMenuItem.setEnabled( false );
        
        m_popup.add(m_exportModelAsParametricPatternMenuItem );
        
        m_popup.setOpaque( true );
        m_popup.setLightWeightPopupEnabled( true );
        
        this.addMouseListener( 
            new MouseAdapter( )
            {
                @Override
                public void mouseReleased( MouseEvent e )
                {
                    if ( e.isPopupTrigger( ) )
                    {   
                        // popup depends on which node has been selected
                        adaptPopup( );
                        m_popup.show( (JComponent)e.getSource( ), e.getX( ), e.getY( ) );
                    }
                }
            }
        );
        
 
        // Add a tree selection listener
        addTreeSelectionListener( new TreeSelectionListener( ) 
        {
            @Override
            public void valueChanged( TreeSelectionEvent evt ) 
            {
                
                Object deselectedObj = null; 
                Object selectedObj = null;
                
                DefaultMutableTreeNode deselectedNode, selectedNode; 
                
                selectedNode = null;
                deselectedNode = null;
                
                // Get all nodes whose selection status has changed
                TreePath[] paths = evt.getPaths( );
                
                // Iterate over all affected nodes
                for ( int i = 0; i < paths.length; i++ ) 
                {
                    if ( !evt.isAddedPath( i ) )
                    {
                        // This node has been deselected
                        TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
                        MRFittingParameters fitPar = modSpec.getLastNotifiedParameters( );
                       
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode)( paths[i].getLastPathComponent( ) );
                        
                        // Mark the deselected node!
                        deselectedNode = (DefaultMutableTreeNode)node.getParent( );
                    }
                }      
                
                // Iterate through all affected nodes
                for ( int i = 0; i < paths.length; i++ ) 
                {
                    if ( evt.isAddedPath( i ) )
                    {
                        // This node has been selected
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode)( paths[i].getLastPathComponent( ) );
                        
                        selectedNode = (DefaultMutableTreeNode)node.getParent( );
                    }
                }
                
                if ( selectedNode != null )
                {
                    if ( selectedNode.equals( deselectedNode ) == true )
                    {
                        // System.out.println( "The two nodes have same parent!" );
                    }
                    else
                    {
                        // System.out.println( "The two nodes do not have the same parent!" );
                    }
                }
                
                // Iterate through all affected nodes
                // First save the node information of the deselected node!
                // in the TDFDFitPrototypeModelSpectrumParameters-object
                for ( int i = 0; i < paths.length; i++ ) 
                {
                    if ( !evt.isAddedPath( i ) )
                    {
                        // This node has been deselected
                        TDFDFitPrototypeModelSpectrumParameters modSpec = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
                        MRFittingParameters fitPar = modSpec.getLastNotifiedParameters( );
                        
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode)( paths[i].getLastPathComponent( ) );
                        // deselectedObj = node.getUserObject( );
                                           
                            if ( deselectedObj instanceof MRLineModelParameters )
                            {
                                ((MRLineModelParameters)deselectedObj).setFittingParameters( fitPar );
                            }
                            if ( deselectedObj instanceof MRNumericPattern )
                            {   
                                ((MRNumericPattern)deselectedObj).setFittingParameters( fitPar );
                            }
                            if ( deselectedObj instanceof MRParametricPattern )
                            {
                                ((MRParametricPattern)deselectedObj).setFittingParameters( fitPar );
                            }
                            if ( deselectedObj instanceof MRMetaboliteModel )
                            {   
                                ((MRMetaboliteModel)deselectedObj).setFittingParameters( fitPar );
                            }
                        
                    }
                }
                
                // Iterate through all affected nodes
                for ( int i = 0; i < paths.length; i++ ) 
                {
                    if ( evt.isAddedPath( i ) )
                    {
                        // This node has been selected
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode)( paths[i].getLastPathComponent( ) );
                        
                        // selectedObj = node.getUserObject( );
                        
                        // selectedObj = ( (TreeModelCheckBoxNodeData)node.getUserObject( ) ).getObject( );
                        
                        if ( node.isRoot( ) == true )
                        // if ( selectedObj instanceof String )
                        {
                            raiseActionEvent( ROOT_SELECTED_EVENT, (String)selectedObj, "root node selected event" );
                        } 
                        else
                        {
                            selectedObj = node.getUserObject( );
                            
                            if ( selectedObj instanceof MRLineModelParameters )
                            {
                                raiseActionEvent( LINE_MODEL_SELECTED_EVENT, selectedObj, "line model node selected event" );
                            }
                            if ( selectedObj instanceof MRNumericPattern )
                            {
                                raiseActionEvent( NUMERIC_PATTERN_SELECTED_EVENT, ( (MRNumericPattern)selectedObj ).getFittingParameters( ), "numeric pattern node selected event" );
                            }
                            if ( selectedObj instanceof MRParametricPattern )
                            {
                                raiseActionEvent( PARAMETRIC_PATTERN_SELECTED_EVENT, ( (MRParametricPattern)selectedObj ).getFittingParameters( ), "parametric pattern node selected event" );
                            }
                            if ( selectedObj instanceof MRMetaboliteModel )
                            {
                                raiseActionEvent( METABOLITE_SELECTED_EVENT, ( (MRMetaboliteModel)selectedObj ).getFittingParameters( ), "metabolite node selected event" );
                            }
                        }
                    }
                }  
            }
            
        } );     
    }
    
    /**
     * Adds an MRLineModelParameters object to the currently selected node.
     * @param linPar 
     */
    public void addMRLineModelParameters( MRLineModelParameters linPar )
    {
        TreePath path = getSelectionPath( );
        DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );
           
        DefaultMutableTreeNode node = new DefaultMutableTreeNode( linPar );
                
        // an MRLineModelParameter object node can not be child node 
        // of a MRLineModelParameter and a MRNumericPattern node
        if ( ( dmtn.getUserObject( ) instanceof MRLineModelParameters ) ||
             ( dmtn.getUserObject( ) instanceof MRNumericPattern ) )
        {
            dmtn = (DefaultMutableTreeNode)dmtn.getParent( );
        }

        createUniqueNodeName( dmtn, node );
        dmtn.add( node );
        
        raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, null ); 
    }
    
    /**
     * Adds an MRMetaboliteModel object to the currently selected node.
     * @param metaboliteFileRef 
     */
    void addMetabolite_From_MPF_File( String metaboliteFileRef ) 
    {
        MRMetaboliteModel metabolite = new MRMetaboliteModel( );
        
        // loading the parametric pattern
        if ( metaboliteFileRef.compareTo( "" ) != 0 )
        {

            DefaultMutableTreeNode node = new DefaultMutableTreeNode( metabolite );
            
            // Get the currently selected tree path
            TreePath path = getSelectionPath( );
            DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );
            createUniqueNodeName( dmtn, node );
            
            dmtn.add( node );

            // Loads the metabolite pattern
            RandomAccessFile in;
            try 
            {
                in = new RandomAccessFile( metaboliteFileRef, "rw" );
                try 
                {
                    String tag = in.readUTF( );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFitModelTreeWithPopupMenu.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
                metabolite.read( in, node );
            } 
            catch ( FileNotFoundException ex ) 
            {
                Logger.getLogger( TDFDFitModelTreeWithPopupMenu.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
            
            createUniqueNodeName( dmtn, node );
            
            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, "add parametric pattern node from file" );

            // refresh tree structure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
            setSelectionPath( new TreePath( node.getPath( ) ) );
        }
    }
        
    /**
     * Adds an MRParametricPattern object to the currently selected node.
     * @param parPatFileReference 
     */
    void addParametricPatternFrom_PPF_File( String parPatFileReference ) 
    {
        MRParametricPattern parPat = new MRParametricPattern( );
        
        // loading the parametric pattern
        if ( parPatFileReference.compareTo( "" ) != 0 )
        {

            DefaultMutableTreeNode node = new DefaultMutableTreeNode( parPat );
            
            // Get the currently selected tree path
            TreePath path = getSelectionPath( );
            DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );
            createUniqueNodeName( dmtn, node );
            
            dmtn.add( node );

            // Load the parametric pattern here!
            parPat.readModelFile( node, parPatFileReference );

            createUniqueNodeName( dmtn, node );
            
            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, "add parametric pattern node from file" );

            // Refresh the  tree structure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
            setSelectionPath( new TreePath( node.getPath( ) ) );
        }
    }
    
    /**
     *  Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {

        Object sender = e.getSource( );
        DefaultMutableTreeNode dmtn, node;
        
        TreePath path = getSelectionPath( );

        if ( path != null )
        {
            dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );
        }
        else
        {
            dmtn = m_rootNode;
        }
        if ( e.getSource( ) == TDFDFittingDefaults.getInstance( ) )
        {
            // Currrently nothing is performed
        }
        
        if ( e.getSource( ) == m_copyNode )            
        {
            MRLineModelParameters selObj = (MRLineModelParameters)getSelectedObject( );
            m_MRLineModelParametersToCopy = new MRLineModelParameters( selObj );
            m_lineToCopyAvailable = true;
        }
        
        // Paste the MRLineModelParameters-object to the currently selected object.
        if ( e.getSource( ) == m_pasteNode )            
        {
            node = new DefaultMutableTreeNode( m_MRLineModelParametersToCopy );
            createUniqueNodeName( dmtn, node );
            dmtn.add( node );
            
            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
            
            // Refresh treestructure.
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
            
            // Select the newly created treenode..
            TreePath tp = new TreePath( node.getPath( ) );
            setSelectionPath( tp );
            makeVisible( tp );
        }
        if ( e.getSource( ) instanceof MRParameterSpectrumModelViewer )
        {
            
            if ( ( e.getID( ) == MRLineModelParametersPanel.ADD_USER_DEFINED_LINE_EVENT ) && 
                 ( e.getActionCommand( ).compareTo( "add user defined line node" ) == 0 ) )
            {
                MRParameterSpectrumModelViewer viewer = (MRParameterSpectrumModelViewer)e.getSource( );
            
                MRLineModelParameters linPar = new MRLineModelParameters( );
                linPar.setArea( viewer.getLastSelectedAreaValueOfLastMouseClickedEvent( ) / 105.0f );
                
                linPar.setOffset(  viewer.convertToTDFDFitPointUnits( viewer.getLastSelectedOffsetValueOfLastMouseClickedEvent( ) ) );
                linPar.setLorentzianWidth( 3.0f );
                node = new DefaultMutableTreeNode( linPar );
                
                // An MRLineModelParameter object node can not be child node 
                // of a MRLineModelParameter and a MRNumericPattern node.
                if ( ( dmtn.getUserObject( ) instanceof MRLineModelParameters ) ||
                     ( dmtn.getUserObject( ) instanceof MRNumericPattern ) )
                {
                    dmtn = (DefaultMutableTreeNode)dmtn.getParent( );
                }
                
                createUniqueNodeName( dmtn, node );
                dmtn.add( node );
            
                raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
            
                // Refresh treestructure.
                ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
                
                // select the newly created treenode
                TreePath tp = new TreePath( node.getPath( ) );
                setSelectionPath( tp );
                makeVisible( tp );
            }
        }
        
        if ( e.getActionCommand( ).equals( MODEL_TREE_NODE_ADD_EVENT_CMD ) )
        {
            node = new DefaultMutableTreeNode( new MRLineModelParameters( ) );
            createUniqueNodeName( dmtn, node );
            dmtn.add( node );
            
            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
            
            // Refresh treestructure.
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
            
            // select the newly created treenode
            TreePath tp = new TreePath( node.getPath( ) );
            setSelectionPath( tp );
            makeVisible( tp );
        }
        
        // Adds a metabolite node.
        if ( e.getActionCommand( ).equals( "add metabolite node" ) )
        {
            node = new DefaultMutableTreeNode( new MRMetaboliteModel( ) );
            createUniqueNodeName( dmtn, node );
            dmtn.add( node );
            
            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
            // refresh treestructure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
            
            // select the newly created treenode
            TreePath tp = new TreePath( node.getPath( ) );
            setSelectionPath(tp );
            makeVisible( tp );
        }
        
        // Adds a numeric pattern model from .bin file.
        if ( e.getActionCommand( ).equals( "add numeric pattern node from bin response" ) )
        {
            
            addNumericPatternFrom_Binary_File( null );
             
//            File numericPatternBinFileName = selectNumericPatternFromBinFileReference( );
//            
//            if ( numericPatternBinFileName != null )
//            {
//                MRNumericPattern numPat = new MRNumericPattern( );
//                // Now read it from a file
//                try 
//                {
//                    RandomAccessFile in = new RandomAccessFile( numericPatternBinFileName.getAbsoluteFile( ).toString( ), "rw" );
//                    numPat.readTDFDFitFormattedResponseFile( MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ), 
//                                                             numericPatternBinFileName.getAbsoluteFile( ).toString( ) );
//                } 
//                catch ( FileNotFoundException ex ) 
//                {
//                    Logger.getLogger( TDFDFitModelTreeWithPopupMenu.class.getName( ) ).log( Level.SEVERE, null, ex );
//                }
//                
//                numPat.setName( numericPatternBinFileName.getName( ) );
//                numPat.setNumericPatternFileReference( numericPatternBinFileName.getAbsolutePath( ) );
//                node = new DefaultMutableTreeNode( numPat );
//                createUniqueNodeName( dmtn, node );
//                dmtn.add( node );
//            
//                raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
//                
//                // refresh treestructure
//                ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
//                
//                // select the newly created treenode
//                TreePath tp = new TreePath( node.getPath( ) );
//                setSelectionPath( tp );
//                makeVisible( tp );
//            }  
        }
        
        // Adds a numeric pattern model from .mrui file.
        if ( e.getActionCommand( ).equals( "add numeric pattern node from mrui response" ) )
        {
            
            File fileName = getLoadPatternFileReference( "Load from mrui file" );
            addNumericPatternFrom_MRUI_File( fileName );

        }
        
        
        // Adds a numeric pattern model from .npf file.
        if ( e.getActionCommand( ).equals( "add numeric pattern node" ) )
        {
            
            File numericPatternNodeName = selectNumericPatternFromNPFFileReference( );
            
            if ( numericPatternNodeName != null )
            {
                MRNumericPattern numPat = new MRNumericPattern( );
                // Now read it from a file
                try 
                {
                    RandomAccessFile in = new RandomAccessFile( numericPatternNodeName.getAbsoluteFile( ).toString( ), "rw" );
                    numPat.read( in );
                } 
                catch ( FileNotFoundException ex ) 
                {
                    Logger.getLogger(TDFDFitModelTreeWithPopupMenu.class.getName()).log(Level.SEVERE, null, ex);
                }
                numPat.setName( numericPatternNodeName.getName( ) );
                numPat.setNumericPatternFileReference( numericPatternNodeName.getAbsolutePath( ) );
                node = new DefaultMutableTreeNode( numPat );
                createUniqueNodeName( dmtn, node );
                dmtn.add( node );
            
                raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
                
                // refresh treestructure
                ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
                
                // select the newly created treenode
                TreePath tp = new TreePath( node.getPath( ) );
                setSelectionPath( tp );
                makeVisible( tp );
            }  
        }
        
        // This handles the event raised by the tree node.
        if ( e.getSource( ) == m_addNumericPatternNodeFromNPFFile )
        {
            File numericPatternNodeName = selectNumericPatternFromNPFFileReference( );
            if ( numericPatternNodeName != null )
            {
                MRNumericPattern numPat = new MRNumericPattern( );
                // Now read it from a file
                try 
                {
                    RandomAccessFile in = new RandomAccessFile( numericPatternNodeName.getAbsoluteFile( ).toString( ), "rw" );
                    numPat.read( in );
                } 
                catch ( FileNotFoundException ex ) 
                {
                    Logger.getLogger( TDFDFitModelTreeWithPopupMenu.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
                numPat.setName( numericPatternNodeName.getName( ) );
                numPat.setNumericPatternFileReference( numericPatternNodeName.getAbsolutePath( ) );
                node = new DefaultMutableTreeNode( numPat );
                createUniqueNodeName( dmtn, node );
                dmtn.add( node );
            
                raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
                
                // refresh treestructure
                ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
                
                // select the newly created treenode
                TreePath tp = new TreePath( node.getPath( ) );
                setSelectionPath( tp );
                makeVisible( tp );
            }  
        }
        
        // Adds a parametric pattern model.
        if ( e.getActionCommand( ).equals( "add parametric pattern node" ) )
        {
            node = new DefaultMutableTreeNode( new MRParametricPattern( ) );
            createUniqueNodeName( dmtn, node );
            dmtn.add( node );
            
            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
            // refresh treestructure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
            setSelectionPath( new TreePath( node.getPath( ) ) );
            
        }
        
        // Adds a parametric pattern model from a file.
        if ( e.getActionCommand( ).equals( "add parametric pattern node from file" ) )
        {
            MRParametricPattern parPat = new MRParametricPattern( );
            // let the user select the filename 
            String parPatFile = getParametericPatternFileName( );
            
            // loading the parametric pattern
            
            if ( parPatFile.compareTo( "" ) != 0 )
            {
                
                node = new DefaultMutableTreeNode( parPat );
            
                createUniqueNodeName( dmtn, node );
                dmtn.add( node );
                
                // load the parametric pattern here
                parPat.readModelFile( node, parPatFile );
                
                raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, e.getActionCommand( ) );
                
                // refresh treestructure
                ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );
                setSelectionPath( new TreePath( node.getPath( ) ) );
            }
        }
        
        // Handle the destruction of nodes.
        if ( e.getActionCommand( ).equals( MODEL_TREE_NODE_DELETE_EVENT_CMD ) )
        {  
            DefaultMutableTreeNode parent = (DefaultMutableTreeNode)dmtn.getParent( );
            dmtn.removeFromParent( );
            // also verify prior knowledge ...
            // refresh treestructure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)parent );
            
            setSelectionPath( new TreePath( parent.getPath( ) ) );
            
            raiseActionEvent( MODEL_TREE_NODE_DELETE_EVENT, this, e.getActionCommand( ) );
            
        }
        
        // Handle the renaming of nodes.
        if ( e.getActionCommand( ).equals( MODEL_TREENODE_NAME_CHANGE_EVENT_CMD ) )
        {
            // Show a dialog allowing the user to change the name of the component.
            m_renameControl = new StringDefinitionControl( "Rename", "New name", getSelectedNodeName( ), null );
            m_renameControl.addActionListener( this );
        }

        // Handle the creation of a starting value file.
        if ( e.getActionCommand( ).equals( CREATE_STARTING_VALUE_FILES_CMD ) )
        {
            raiseActionEvent( CREATE_STARTING_VALUE_FILES, this, CREATE_STARTING_VALUE_FILES_CMD );
        }
        
        // Export the complete model as parametric pattern.
        if ( e.getActionCommand( ).equals( EXPORT_AS_PARAMETRIC_PATTERN_CMD ) )
        {
            raiseActionEvent( EXPORT_AS_PARAMETRIC_PATTERN, this, EXPORT_AS_PARAMETRIC_PATTERN_CMD );
        }
        
        // Export the complete model as numeric pattern.
        if ( e.getActionCommand( ).equals( EXPORT_AS_NUMERIC_PATTERN_CMD ) )
        {
            raiseActionEvent( EXPORT_AS_NUMERIC_PATTERN, this, EXPORT_AS_NUMERIC_PATTERN_CMD );
        }
        
        // Export the complete model as metabolite pattern.
        if ( e.getActionCommand( ).equals( EXPORT_AS_METABOLITE_PATTERN_CMD ) )
        {
            raiseActionEvent( EXPORT_AS_METABOLITE_PATTERN, this, EXPORT_AS_METABOLITE_PATTERN_CMD );
        }
              
        // Do the actual renaming of the node.
        if ( sender == m_renameControl )
        {
            if ( e.getID( ) == StringDefinitionControl.ACCEPT_EVENT )
            {
                
                m_newNameAfterRenaming = m_renameControl.getRenamedString( );
                
                // Store the old name
                m_oldNodeNameBeforeRenaming = this.getSelectedNodeName( );
                
                setSelectedNodeName( m_newNameAfterRenaming );
                
                if ( dmtn.isRoot( ) == false )
                {
                    raiseActionEvent( MODEL_TREENODE_NAME_CHANGE_EVENT, this, m_newNameAfterRenaming );
                }
                else
                {
                    raiseActionEvent( MODEL_ROOT_NAME_CHANGE_EVENT, m_newNameAfterRenaming, e.getActionCommand( ) );
                }
                
                updateUI( );
            }
        }
    }
    
    /**
     * Selects the root node of the JTree.
     */
    public void selectRootNode( )
    {
        DefaultMutableTreeNode root = ( (DefaultMutableTreeNode)this.getModel( ).getRoot( ) );
        setSelectionPath( new TreePath( root.getPath( ) ) );
    }
    
    public DefaultMutableTreeNode getRootNode( )
    {
        return m_rootNode;
    }
    
    /**
     * Adds a numeric pattern file from .mrui file.
     * @param mruiFileReference 
     */
    public void addNumericPatternFrom_MRUI_File( File mruiFileReference )
    {
        TreePath path = getSelectionPath( );
        DefaultMutableTreeNode dmtn;
        
        dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );
        if ( mruiFileReference != null )
        {
            MRNumericPattern numPat = new MRNumericPattern( );
            numPat.setMRUIFileAsSource( mruiFileReference );

            String str = mruiFileReference.getName( );
            int begin = str.lastIndexOf( System.getProperty( "file.separator" ) ) + 1;
            int end = str.lastIndexOf( '.' );
            
            String numPatName = str.substring( begin, end );
            
            numPat.setName( numPatName );
            
            TDFDFittingDefaults defaults = TDFDFittingDefaults.getInstance( );
            defaults.load( );
            
            String outpath = defaults.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );
            outpath = outpath + System.getProperty( "file.separator" ) 
                    + "Numeric_Patterns" 
                    + System.getProperty( "file.separator" );
            
            numPat.setNumericPatternFileReference( outpath + numPatName + "_AsNumpat.npf" );
            numPat.writeAsTdfdfitFile( outpath + numPatName + ".bin" );
            numPat.writeNumericPatternAsTDFDFitDotBinFile( numPatName + "_AsNumpat.npf", false );
            try 
            {
                try ( RandomAccessFile out = new RandomAccessFile( outpath + numPatName + "_AsNumpat.npf", "rw" ) ) 
                {
                    numPat.write( out );
                }
            } 
            catch ( FileNotFoundException ex )
            {
                Logger.getLogger( MRParametricPattern.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
            catch ( IOException ex )
            {
                Logger.getLogger( MRParametricPattern.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
            
            
            DefaultMutableTreeNode node = new DefaultMutableTreeNode( numPat );
            createUniqueNodeName( dmtn, node );
            dmtn.add( node );

            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, MODEL_TREE_NODE_ADD_EVENT_CMD );

            // Refresh tree structure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );

            // Select the newly created tree node
            TreePath tp = new TreePath( node.getPath( ) );
            setSelectionPath( tp );
            makeVisible( tp );
        } 
    }
    
    /**
     * Adds a numeric pattern file from .npf file.
     * @param npfFileReference 
     */
    public void addNumericPatternFrom_NPF_File( File npfFileReference )
    {
        TreePath path = getSelectionPath( );
        DefaultMutableTreeNode dmtn;
        
        dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );
        if ( npfFileReference != null )
        {
            MRNumericPattern numPat = new MRNumericPattern( );
            RandomAccessFile in = null;
            try 
            {
                in = new RandomAccessFile( npfFileReference.getAbsoluteFile( ).toString( ), "rw" );
            } 
            catch ( FileNotFoundException ex ) 
            {
                Logger.getLogger( TDFDFitModelTreeWithPopupMenu.class.getName( ) ).log( Level.SEVERE, null, ex );
            }
            numPat.read( in );
            numPat.setName( npfFileReference.getName( ) );
            numPat.setNumericPatternFileReference( npfFileReference.getAbsolutePath( ) );
            DefaultMutableTreeNode node = new DefaultMutableTreeNode( numPat );
            createUniqueNodeName( dmtn, node );
            dmtn.add( node );

            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, MODEL_TREE_NODE_ADD_EVENT_CMD );

            // Refresh tree structure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );

            // Select the newly created tree node
            TreePath tp = new TreePath( node.getPath( ) );
            setSelectionPath( tp );
            makeVisible( tp );
        } 
    }
    
    /**
     * Updates the prior knowledge in the tree view by replacing 
     * the oldName by newName.
     * @param oldName
     * @param newName
     */
    public void updatePriorKnowledgeInTreeModel( String oldName, String newName )
    {
        TreePath path = getSelectionPath( );
        DefaultMutableTreeNode dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );
        
        DefaultMutableTreeNode parent = (DefaultMutableTreeNode)dmtn.getParent( );
        
        for ( Enumeration e = parent.children( ); e.hasMoreElements( ) ; )
        {
            DefaultMutableTreeNode node = ( DefaultMutableTreeNode )e.nextElement( );
            String childName = node.toString( );
            
            Object obj = node.getUserObject( );
            
            if ( ( childName.compareTo( oldName ) == 0 ) && ( node == dmtn ) ) 
            {
                if ( obj instanceof MRLineModelParameters )
                {   
                    // 1. change the name of the node itself
                    ((MRLineModelParameters)obj).setName( newName );
                    // 2. Change the name of the references of the node if appropriate
                    if ( ((MRLineModelParameters)obj).getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setAreaReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setGaussianWidthReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setLorentzianWidthReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setPhaseReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setOffsetReference( newName );
                    }
                }
                if ( obj instanceof MRNumericPattern )
                {   
                    // 1. change the name of the node itself
                    ((MRNumericPattern)obj).setName( newName );
                    // 2. Change the name of the references of the node if appropriate
                    if ( ((MRNumericPattern)obj).getFittingParameters( ).getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters( ).setAreaReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters( ).getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters( ).setGaussianWidthReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters( ).getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters( ).setLorentzianWidthReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters( ).getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters( ).setPhaseReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters( ).getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters( ).setOffsetReference( newName );
                    }
                    
                    
                }
                if ( obj instanceof MRParametricPattern )
                {
                    // 1. change the name of the node itself
                    ((MRParametricPattern)obj).setName( newName );
                    // 2. Change the name of the references of the node if appropriate
                    if ( ((MRParametricPattern)obj).getFittingParameters( ).getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters( ).setAreaReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters( ).getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters( ).setGaussianWidthReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters( ).getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters( ).setLorentzianWidthReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters( ).getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters( ).setPhaseReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters( ).getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters( ).setOffsetReference( newName );
                    }
                }
                
                if ( obj instanceof MRMetaboliteModel )
                {   
                    // 1. change the name of the node itself
                    ((MRMetaboliteModel)obj).setName( newName );
                    // 2. Change the name of the references of the node if appropriate
                    if ( ((MRMetaboliteModel)obj).getFittingParameters( ).getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters( ).setAreaReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters( ).getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters( ).setGaussianWidthReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters( ).getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters( ).setLorentzianWidthReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters( ).getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters( ).setPhaseReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters( ).getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters( ).setOffsetReference( newName );
                    }
                }
            }
            else
            {
                if ( obj instanceof MRLineModelParameters )
                {
                    // In this case only check names of references
                    if ( ((MRLineModelParameters)obj).getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setAreaReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setGaussianWidthReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setLorentzianWidthReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setPhaseReference( newName );
                    }
                    if ( ((MRLineModelParameters)obj).getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRLineModelParameters)obj).setOffsetReference( newName );
                    }
                }
                if ( obj instanceof MRNumericPattern )
                {   
                    // In this case only check names of references
                    if ( ((MRNumericPattern)obj).getFittingParameters().getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters().setAreaReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters().getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters().setGaussianWidthReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters().getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters().setLorentzianWidthReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters().getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters().setPhaseReference( newName );
                    }
                    if ( ((MRNumericPattern)obj).getFittingParameters().getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRNumericPattern)obj).getFittingParameters().setOffsetReference( newName );
                    }
                }
                if ( obj instanceof MRParametricPattern )
                {
                    // In this case only check names of references
                    if ( ((MRParametricPattern)obj).getFittingParameters().getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters().setAreaReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters().getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters().setGaussianWidthReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters().getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters().setLorentzianWidthReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters().getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters().setPhaseReference( newName );
                    }
                    if ( ((MRParametricPattern)obj).getFittingParameters().getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRParametricPattern)obj).getFittingParameters().setOffsetReference( newName );
                    }
                }
                if ( obj instanceof MRMetaboliteModel )
                {   
                    // In this case only check names of references
                    if ( ((MRMetaboliteModel)obj).getFittingParameters().getAreaReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters().setAreaReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters().getGaussianWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters().setGaussianWidthReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters().getLorentzianLineWidthReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters().setLorentzianWidthReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters().getPhaseReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters().setPhaseReference( newName );
                    }
                    if ( ((MRMetaboliteModel)obj).getFittingParameters().getOffsetReference( ).compareTo( oldName ) == 0 )
                    {
                        ((MRMetaboliteModel)obj).getFittingParameters().setOffsetReference( newName );
                    }
                }
            }
        }   
    }
    
    /**
     * 
     * Get the value of the name of the tree element before renaming
     * 
     * @return 
     */
    public String getStringNameBeforeRenaming( ) 
    {
        return m_oldNodeNameBeforeRenaming;
    }

    /**
     * Creates a unique child node name that is unique in the  parent branch.
     */
    private void createUniqueNodeName( DefaultMutableTreeNode parent, 
                                       DefaultMutableTreeNode newChild )
    {
        
        String newChildName = newChild.toString( );
        
        // Create a list of all names of all children
        LinkedList<String> name_list = new LinkedList<String>( );
        
        for ( Enumeration e = parent.children( ); e.hasMoreElements( ) ; )
        {
            DefaultMutableTreeNode node = ( DefaultMutableTreeNode )e.nextElement( );
            String childName = node.toString( );
            name_list.add( childName );
        }
        
        Iterator it = name_list.iterator( );
        boolean test = false;
        while ( it.hasNext( ) )
        {
            if ( ( (String)it.next( ) ).compareTo( newChildName )  == 0 )
            {
                test = true;
            }
            else
            {
                // nothing
            }
        }
        
        String testName = newChildName;
        
        if ( test == true )
        {
            testName = newChildName;
            int i=1;
            while ( isUnique( name_list, testName ) == false )
            {
                testName = newChildName + "_" + i;
                i++;
            }
        }
        
        if ( newChild.getUserObject( ) instanceof MRLineModelParameters )
        {
            ( (MRLineModelParameters)newChild.getUserObject( ) ).setName( testName );
        }
        if ( newChild.getUserObject( ) instanceof MRNumericPattern )
        {
            ( (MRNumericPattern)newChild.getUserObject( ) ).setName( testName  );
        }
        if ( newChild.getUserObject( ) instanceof MRParametricPattern )
        {
            ( (MRParametricPattern)newChild.getUserObject( ) ).setName( testName );
        }
        if ( newChild.getUserObject( ) instanceof MRMetaboliteModel )
        {
            ( (MRMetaboliteModel)newChild.getUserObject( ) ).setName( testName );
        }
    }
    
    /**
     * Returns whether the string is unique.
     */
    private boolean isUnique( LinkedList names, String name )
    {
        Iterator it = names.iterator( );
        while ( it.hasNext( ) )
        {
            String listString = (String)it.next( );
            if ( listString.compareTo( name ) == 0 )
            {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Returns the file selected by the user.
     */
    private String getParametericPatternFileName( )
    {   
        
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + 
                                                     System.getProperty( "file.separator" ) + 
                                                     "Parametric_Patterns"  );
        
        
        fileChooser.setName( "Select a parametric pattern file" );
        
        int result = fileChooser.showOpenDialog( this );
        
        File fitParametricPatternFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                fitParametricPatternFileReference = fileChooser.getSelectedFile( );
                                                break;
            case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                break;
        }

        return fitParametricPatternFileReference.getAbsolutePath( );
    }

        
    /**
     * Enables selecting a numeric pattern file reference.
     */
    private File selectNumericPatternFromNPFFileReference( )
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + System.getProperty( "file.separator" ) + "Numeric_Patterns"  );
        FileFilter filter = new FileNameExtensionFilter( "Numeric pattern file", "npf" );
        // fileChooser.addChoosableFileFilter( filter );
        fileChooser.setFileFilter( filter );
        int result = fileChooser.showOpenDialog( this );
        
        File numericPatternReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                numericPatternReference = fileChooser.getSelectedFile( );
                                                // TO DO test the validity of the selected file:
                                                // the user could select any garbage!!
                                                break;
           case JFileChooser.CANCEL_OPTION:     numericPatternReference = null;
                                                // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return numericPatternReference;
    }
        
        
    /**
     * Enables selecting a binary signal file reference to generate a 
     * numeric pattern from.
     */
    private File selectNumericPatternFromBinFileReference( )
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + 
                                                     System.getProperty( "file.separator" ) + 
                                                     "Numeric_Patterns"  );
        FileFilter filter = new FileNameExtensionFilter( "Bin file", "bin" );
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.addChoosableFileFilter( filter );
        
        int result = fileChooser.showOpenDialog( this );
        
        File numericPatternReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                numericPatternReference = fileChooser.getSelectedFile( );
                                                // TO DO test the validity of the selected file:
                                                // the user could select any garbage!!
                                                break;
           case JFileChooser.CANCEL_OPTION:     numericPatternReference = null;
                                                // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return numericPatternReference;
    }
    
     /**
     * Enables selecting a binary signal file reference to generate a 
     * numeric pattern from.
     */
    private File selectNumericPatternFromMRUIFileReference( )
    {
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + 
                                                     System.getProperty( "file.separator" ) + 
                                                     "Numeric_Patterns"  );
        
        FileFilter filter = new FileNameExtensionFilter( "MRUI file", "mrui" );
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.addChoosableFileFilter( filter );
        
        int result = fileChooser.showOpenDialog( this );
        
        File numericPatternReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                numericPatternReference = fileChooser.getSelectedFile( );
                                                // TO DO test the validity of the selected file:
                                                // the user could select any garbage!!
                                                break;
           case JFileChooser.CANCEL_OPTION:     numericPatternReference = null;
                                                // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return numericPatternReference;
    }
    
    
    /**
     * Returns the object that is currently selected.
     * @return 
     */
    public Object getSelectedObject( )
    {
        TreePath path = getSelectionPath( );
        
        // This node has been selected
        if ( path != null )
        {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)( path.getLastPathComponent( ) );
            // Object obj = node.getUserObject( );
//            if ( node.isRoot( ) )
//            {
//                return node.getUserObject( );
//            }
//            else
//            {
//                Object obj = ( (TreeModelCheckBoxNodeData)node.getUserObject( ) ).getObject( );
//                return obj;
//            }
            return node.getUserObject( );
        }
        
        return null;
    }
    
    /**
     * Returns the name of the selected node.
     * @return 
     */
    public String getSelectedNodeName( )
    {    
        TreePath path = getSelectionPath( );
        
        // This node has been selected
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)( path.getLastPathComponent( ) );
        Object obj = node.getUserObject( );
        
        if ( obj instanceof String )
        {
            return ( (String)obj ).toString( );
        }
        if ( obj instanceof MRLineModelParameters )
        {
            return ( (MRLineModelParameters)obj ).toString( );
        }
        if ( obj instanceof MRNumericPattern )
        {
            return ( (MRNumericPattern)obj ).toString( );
        }
        if ( obj instanceof MRParametricPattern )
        {
            return ( (MRParametricPattern)obj ).toString( );
        }
        if ( obj instanceof MRMetaboliteModel )
        {
            return ( (MRMetaboliteModel)obj ).toString( );
        } 
        // Return following string in case object doesnot match
        return "no matching string";
        
    }
    
    /**
     * Sets the name of the node equal 'name'.
     */
    private void setSelectedNodeName( String newName )
    {  
        TreePath path = getSelectionPath( );

        // This node has been selected
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)( path.getLastPathComponent( ) );
        Object obj = node.getUserObject( );
        
        // The root node object type is String.
        if ( node.isRoot( ) )
        {
            node.setUserObject( newName );
            obj = newName;
            MRSignalProperties MRSignalProps = MRSignalProperties.getInstance( );
            MRSignalProps.setFitModelName( newName );
            raiseActionEvent( MODEL_ROOT_NAME_CHANGE_EVENT , obj, MODEL_ROOT_NAME_CHANGE_EVENT_CMD );
        }
        
        String oldName = null;
        
        if ( obj instanceof MRLineModelParameters )
        {
            oldName = ( (MRLineModelParameters)obj ).getName( );
            ( (MRLineModelParameters)obj ).setName( newName );
        }
        
        if ( obj instanceof MRNumericPattern )
        {
            oldName = ( (MRNumericPattern)obj ).getName( );
            ( (MRNumericPattern)obj ).setName( newName );
        }
        
        if ( obj instanceof MRParametricPattern )
        {
            oldName = ( (MRParametricPattern)obj ).getName( );
            ( (MRParametricPattern)obj ).setName( newName );
        }
        
        if ( obj instanceof MRMetaboliteModel )
        {
            oldName = ( (MRMetaboliteModel)obj ).getName( );
            ( (MRMetaboliteModel)obj ).setName( newName );
        }
        
    }
    
    /**
     * Sets all JMenuItems enabled.
     */
    public void setAllPopupItemsEnabled( )
    {
        m_deleteNode.setEnabled( true );
        m_copyNode.setEnabled( true );
        m_pasteNode.setEnabled( true );
        
        m_addMetaboliteNode.setEnabled( true );
        m_addLineNode.setEnabled( true );
        m_addParametricPatternNodeManually.setEnabled( true );
        m_addParametricPatternNodeFromFile.setEnabled( true );
        m_numericPatternSubMenu.setEnabled( true );
        m_parametericPatternSubMenu.setEnabled( true );
        m_addNumericPatternNodeFromBinFile.setEnabled( true );
        m_addNumericPatternNodeFromNPFFile.setEnabled( true );
        m_addNumericPatternNodeFromMRUIFile.setEnabled( true );
        m_renameNode.setEnabled( true );
        m_createStartingValueFiles.setEnabled( true );
        m_exportModelAsParametricPatternMenuItem.setEnabled( true ); 
        m_exportModelAsNumericPatternMenuItem.setEnabled( true );
    }
    
    /**
     * Before the pop up is shown, one has to determine which node type has been
     * selected. Depending on the node selected, only child nodes of certain type 
     * can be added
     */
    public void adaptPopup( )
    {
        DefaultMutableTreeNode node;
        TreePath path = this.getSelectionPath( );
        
        // following node has been selected
        node = (DefaultMutableTreeNode)path.getLastPathComponent( );
        
        Object obj = node.getUserObject( ) ;
        
        // Check whether the node is the root node
//        if ( node.isRoot( ) && ( (String)node.getUserObject( ) ).compareTo ( "No model defined" ) != 0 )

        if ( node.isRoot( ) && ( (String)obj ).compareTo ( "No model defined" ) != 0 )
        {    
            m_deleteNode.setEnabled( false );
            m_renameNode.setEnabled( true );
            m_addLineNode.setEnabled( true );
            m_numericPatternSubMenu.setEnabled( true );
            m_parametericPatternSubMenu.setEnabled( true );
            m_addNumericPatternNodeFromBinFile.setEnabled( true );
            m_addNumericPatternNodeFromNPFFile.setEnabled( true );
            m_addNumericPatternNodeFromMRUIFile.setEnabled( true );
            m_addParametricPatternNodeManually.setEnabled( true );
            m_addParametricPatternNodeFromFile.setEnabled( true );
            m_addMetaboliteNode.setEnabled( true );
            m_createStartingValueFiles.setEnabled( true );
            m_exportModelAsParametricPatternMenuItem.setEnabled( true );
            m_exportModelAsNumericPatternMenuItem.setEnabled( true );
            m_exportModelAsMetaboliteMenuItem.setEnabled( false );
            
        }
        
        // depending on the object type, the popup shows up differently  
        Object nodeObject = node.getUserObject( );
        
        // simple line model
        if ( nodeObject instanceof MRLineModelParameters )
        {
            m_addLineNode.setEnabled( false );
            m_deleteNode.setEnabled( true );
            m_renameNode.setEnabled( true );
            m_copyNode.setEnabled( true );
            m_pasteNode.setEnabled( false );
            m_numericPatternSubMenu.setEnabled( false );
            m_parametericPatternSubMenu.setEnabled( false );
            m_addNumericPatternNodeFromBinFile.setEnabled( false );
            m_addNumericPatternNodeFromNPFFile.setEnabled( false );
            m_addNumericPatternNodeFromMRUIFile.setEnabled( false );
            m_addParametricPatternNodeManually.setEnabled( false );
            m_addParametricPatternNodeFromFile.setEnabled( false );
            m_addMetaboliteNode.setEnabled( false );
            m_createStartingValueFiles.setEnabled( false );
            m_exportModelAsParametricPatternMenuItem.setEnabled( false );
            m_exportModelAsNumericPatternMenuItem.setEnabled( true );
            m_exportModelAsMetaboliteMenuItem.setEnabled( false );
        }
        
        // numeric pattern model
        if ( nodeObject instanceof MRNumericPattern ) 
        {
            m_addLineNode.setEnabled( false );
            m_deleteNode.setEnabled( true );
            m_renameNode.setEnabled( true );
            m_copyNode.setEnabled( false );
            m_pasteNode.setEnabled( false );
            m_numericPatternSubMenu.setEnabled( false );
            m_parametericPatternSubMenu.setEnabled( false );
            m_addNumericPatternNodeFromBinFile.setEnabled( false );
            m_addNumericPatternNodeFromNPFFile.setEnabled( false );
            m_addNumericPatternNodeFromMRUIFile.setEnabled( false );
            m_addParametricPatternNodeManually.setEnabled( false );
            m_addParametricPatternNodeFromFile.setEnabled( false );
            m_addMetaboliteNode.setEnabled( false );
            m_createStartingValueFiles.setEnabled( false );
            m_exportModelAsParametricPatternMenuItem.setEnabled( false );
            m_exportModelAsNumericPatternMenuItem.setEnabled( true );
            m_exportModelAsMetaboliteMenuItem.setEnabled( false );
        }

        // parametric pattern model
        if ( nodeObject instanceof MRParametricPattern ) 
        {
            m_addLineNode.setEnabled( true );
            m_deleteNode.setEnabled( true );
            m_renameNode.setEnabled( true );
            m_copyNode.setEnabled( false );
            if ( m_lineToCopyAvailable == true )
            {
                m_pasteNode.setEnabled( true );
            }
            m_parametericPatternSubMenu.setEnabled( false );
            m_numericPatternSubMenu.setEnabled( false );
            m_addNumericPatternNodeFromBinFile.setEnabled( false );
            m_addNumericPatternNodeFromNPFFile.setEnabled( false );
            m_addNumericPatternNodeFromMRUIFile.setEnabled( false );
            m_addParametricPatternNodeManually.setEnabled( false );
            m_addParametricPatternNodeFromFile.setEnabled( false );
            m_addMetaboliteNode.setEnabled( false );
            m_exportModelAsParametricPatternMenuItem.setEnabled( false );
            m_createStartingValueFiles.setEnabled( false );
            m_exportModelAsNumericPatternMenuItem.setEnabled( true );
            m_exportModelAsMetaboliteMenuItem.setEnabled( false );
        }

        if ( nodeObject instanceof MRMetaboliteModel )
        {
            m_deleteNode.setEnabled( true );
            m_renameNode.setEnabled( true );
            m_copyNode.setEnabled( false );
            if ( m_lineToCopyAvailable == true )
            {
                m_pasteNode.setEnabled( true );
            }
            
            m_addLineNode.setEnabled( true );
            m_numericPatternSubMenu.setEnabled( false );
            m_parametericPatternSubMenu.setEnabled( true );
            m_numericPatternSubMenu.setEnabled( true );
            m_addNumericPatternNodeFromBinFile.setEnabled( true );
            m_addNumericPatternNodeFromNPFFile.setEnabled( true );
            m_addNumericPatternNodeFromMRUIFile.setEnabled( true );
            m_addParametricPatternNodeManually.setEnabled( true );
            m_addParametricPatternNodeFromFile.setEnabled( true );
            m_addMetaboliteNode.setEnabled( false );
            m_exportModelAsParametricPatternMenuItem.setEnabled( false );
            m_createStartingValueFiles.setEnabled( false );
            m_exportModelAsNumericPatternMenuItem.setEnabled( true );
            m_exportModelAsMetaboliteMenuItem.setEnabled( true );
        }
        
        // ::: TODO :::
    }
    
    /**
     * Reads the attributes from a RandomAccesFile.
     */
    void read( RandomAccessFile in )
    {
        
    }
    
    /**
     * Writes attributes to RandomAccessFile.
     */
    void write( RandomAccessFile out )
    {
        
    }
    
    /**
     * Raises actions on which registered components can take actions.
     */ 
    private void raiseActionEvent( int eventType, Object obj, String cmd )
    {
        Iterator it = m_listeners.iterator( );
        while( it.hasNext( ) )
        {
            if ( obj != null )
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( obj, eventType, "" ) );
            }
            else
            {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
            }
        }
    }
    
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /** 
     * Sets the MRModelSpectrum attribute; method is called after a model
     * spectrum has been loaded; 
     * Its purpose is to set the GUI with the information.
     * @param modelSpectrum
     */
    public void setMRModelSpectrum( TDFDFitPrototypeModelSpectrumParameters modelSpectrum )
    {
        DefaultMutableTreeNode root, node;
        
        if ( m_rootNode != null )
        {
            m_rootNode.removeAllChildren( );
        }
        
        // Refresh tree structure
        ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)m_rootNode );
        // select the newly created treenode
        setSelectionPath( new TreePath( m_rootNode.getPath( ) ) );
        
        // Add a root node again with a novel DefaultMutableTreeNode!
        if ( this.getModel( ).getRoot( ) == null )
        {
            m_rootNode = new DefaultMutableTreeNode( modelSpectrum.getModelSpectrumName( ) );
            createTreeModel( m_rootNode );
        }
        
         // Get all the simple line objects
        LinkedList modelLineList = modelSpectrum.getSimpleMRLines( );
        
        Iterator it = modelLineList.iterator( );
        while( it.hasNext( ) )
        {
            node = new DefaultMutableTreeNode( (MRLineModelParameters)it.next( ) );
            m_rootNode.add( node );
        }

        // Get all the parametric pattern objects 
        LinkedList parametricPatternList = modelSpectrum.getParametricPatterns( );
        
        Iterator it2 = parametricPatternList.iterator( );
        while( it2.hasNext( ) )
        {
            MRParametricPattern parPat = (MRParametricPattern)it2.next( );
            node = new DefaultMutableTreeNode( parPat );
            m_rootNode.add( node );
            
            LinkedList lineList = parPat.getSimpleLineList( );
            Iterator lineIter = lineList.iterator( );
            while ( lineIter.hasNext( ) )
            {
                MRLineModelParameters linPar = (MRLineModelParameters)lineIter.next( );
                DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( linPar );
                node.add( childNode );
            }
        }
        
        // Get all the numeric pattern objects 
        LinkedList numericPatternList = modelSpectrum.getNumericPatterns( );
        
        Iterator it3 = numericPatternList.iterator( );
        while( it3.hasNext( ) )
        {
            node = new DefaultMutableTreeNode( (MRNumericPattern)it3.next( ) );
            m_rootNode.add( node );
        }
        
        // Get all the metabolite objects 
        LinkedList metaboliteModelList = modelSpectrum.getMetaboliteModels( );
        
        Iterator it4 = metaboliteModelList.iterator( );
        while( it4.hasNext( ) )
        {
            MRMetaboliteModel metMod = (MRMetaboliteModel)it4.next( );
            node = new DefaultMutableTreeNode( metMod );
            m_rootNode.add( node );
            
            // Add Simple lines to the Treeview model
            LinkedList lineList = metMod.getSimpleLineList( );
            Iterator lineIter = lineList.iterator( );
            
            while ( lineIter.hasNext( ) )
            {
                MRLineModelParameters linPar = (MRLineModelParameters)lineIter.next( );
                DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( linPar );
                node.add( childNode );
            }

            // Add numeric patterns to the Treeview model
            LinkedList numPatList = metMod.getNumericPatternList( );
            Iterator numPatListIter = numPatList.iterator( );
            
            while ( numPatListIter.hasNext( ) )
            {
                MRNumericPattern numPat = (MRNumericPattern)numPatListIter.next( );
                DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( numPat );
                node.add( childNode );
            }
         
            // Add parametric patterns to the Treeview model
            LinkedList parPatList = metMod.getParametricPatternList( );
            Iterator parPatListIter = parPatList.iterator( );
            
            while ( parPatListIter.hasNext( ) )
            {
                MRParametricPattern parPat = (MRParametricPattern)parPatListIter.next( );
                DefaultMutableTreeNode childNode = new DefaultMutableTreeNode( parPat );
                node.add( childNode );
                
                // the MRParametricPattern can have one or more lines so iterate
                // over the SimpleLineLinkedList
                LinkedList simpleLineList = parPat.getSimpleLineList( );
                Iterator simpleLineListIterator = simpleLineList.iterator( );
                while ( simpleLineListIterator.hasNext( ) )
                {
                    MRLineModelParameters linePar = (MRLineModelParameters)simpleLineListIterator.next( );
                    DefaultMutableTreeNode grandChildNode = new DefaultMutableTreeNode( linePar );
                    childNode.add( grandChildNode );
                }
            }
        }
        
        // Refresh tree structure
        ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)m_rootNode );
        
        // Select the newly created treenode
        setSelectionPath( new TreePath( m_rootNode.getPath( ) ) );
        
        raiseActionEvent( MODEL_TREE_CREATED_EVENT, this, MODEL_TREE_CREATED_EVENT_CMD );
        
        expandAll( true );
        
    }
    
    /**
     * Returns a reference to the parent JFrame object.
     * @return 
     */
    public JFrame getParentFrame( )
    {
        return m_parent;
    }
    
    // If expand is true, expands all nodes in the tree.
    // Otherwise, collapses all nodes in the tree.
    public void expandAll( boolean expand ) 
    {
        TreeNode root = (TreeNode)this.getModel().getRoot();
    
        // Traverse tree from root
        expandAll( this, new TreePath( root ), expand );
    }
    /**
     * Method that traverses through all nodes and expanding or collapsing
     * the tree.
     */
    private void expandAll( JTree tree, TreePath parent, boolean expand )
    {
        // Traverse children
        TreeNode node = (TreeNode)parent.getLastPathComponent( );
        if ( node.getChildCount( ) >= 0 ) 
        {
            for ( Enumeration e=node.children( ); e.hasMoreElements( ); ) 
            {
                TreeNode n = (TreeNode)e.nextElement( );
                TreePath path = parent.pathByAddingChild( n );
                makeVisible( path );
                expandAll( tree, path, expand );
            }
        }
    
        // Expansion or collapse must be done bottom-up
        if ( expand ) 
        {
            tree.expandPath( parent );
        } 
        else 
        {
            tree.collapsePath( parent );
        }
    }
    
    /**
     * Returns the old node name.
     * @return 
     */
    public String getOldNodeNameBeforeRenaming( ) 
    {
        return m_oldNodeNameBeforeRenaming;
    }
    
    /**
     * Returns the new node name.
     * @return 
     */
    public String getNewNameAfterRenaming( ) 
    {
        return m_newNameAfterRenaming;
    }
    
    // Attributes
    private JFrame m_parent;
    private StringDefinitionControl m_renameControl;
    protected JPopupMenu m_popup;
    
    private JMenuItem m_deleteNode;
    
    private JMenuItem m_copyNode;
    private MRLineModelParameters m_MRLineModelParametersToCopy;
    
    private JMenuItem m_pasteNode;
    
    private boolean   m_lineToCopyAvailable = false;
    
    // when chosen adds a metabolite the spectroscopic model tree
    private JMenuItem m_addMetaboliteNode;
    
    // when chosen adds a line the spectroscopic model tree
    private JMenuItem m_addLineNode;
    // a parametric pattern can be read or defened manually:
    // so we need a sub menu structure
    private JMenu m_parametericPatternSubMenu;
    private JMenu m_numericPatternSubMenu;
    
    // when chosen adds a pattern to the spectroscopic model tree
    private JMenuItem m_addParametricPatternNodeManually;
    private JMenuItem m_addParametricPatternNodeFromFile;
    
    // when chosen adds a numeric pattern to the spectroscopic model tree
    private JMenuItem m_addNumericPatternNodeFromBinFile;
    private JMenuItem m_addNumericPatternNodeFromNPFFile;
    private JMenuItem m_addNumericPatternNodeFromMRUIFile;
    
    // allows renaming of a node
    private JMenuItem m_renameNode;
    // when pressed starts the creation of starting value file belonging to
    // the fit model
    private JMenuItem m_createStartingValueFiles;
    
    private JMenuItem m_exportModelAsParametricPatternMenuItem;
    
    private JMenuItem m_exportModelAsNumericPatternMenuItem;
    private JMenuItem m_exportModelAsMetaboliteMenuItem;
    
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
   
    // Possible Action Events of this class
    public final static int     MODEL_TREENODE_NAME_CHANGE_EVENT        = 0;
    public final static String  MODEL_TREENODE_NAME_CHANGE_EVENT_CMD    = "rename node";
    
    public final static int     MODEL_TREE_NODE_ADD_EVENT               = 1;
    public final static String  MODEL_TREE_NODE_ADD_EVENT_CMD           = "add line node";
    
    public final static int     MODEL_TREE_NODE_DELETE_EVENT            = 2;
    public final static String  MODEL_TREE_NODE_DELETE_EVENT_CMD        = "delete node";
    
    public final static int     LINE_MODEL_SELECTED_EVENT               = 3;
    public final static int     NUMERIC_PATTERN_SELECTED_EVENT          = 4;
    public final static int     PARAMETRIC_PATTERN_SELECTED_EVENT       = 5;
    public final static int     METABOLITE_SELECTED_EVENT               = 6;
    
    // Event needed to identific that the name of the MRModelSpectrum has changed
    public final static int     MODEL_ROOT_NAME_CHANGE_EVENT            = 7;
    public final static String  MODEL_ROOT_NAME_CHANGE_EVENT_CMD        = "root name changed event";
    
    public final static int     MODEL_TREE_CREATED_EVENT                = 8;
    public final static int     ROOT_SELECTED_EVENT                     = 9;
    
    public final static int     CREATE_STARTING_VALUE_FILES             = 10;
    public final static int     EXPORT_AS_PARAMETRIC_PATTERN            = 11;
    public final static int     EXPORT_AS_NUMERIC_PATTERN               = 12;
    public final static int     EXPORT_AS_METABOLITE_PATTERN            = 13;
    public final static int     MODEL_CLEARED                           = 14;
    DefaultMutableTreeNode      m_rootNode;
    
    public final static String  EXPORT_AS_PARAMETRIC_PATTERN_CMD    = "export model as parametric pattern";
    public final static String  EXPORT_AS_NUMERIC_PATTERN_CMD       = "export model as numeric pattern";
    public final static String  EXPORT_AS_METABOLITE_PATTERN_CMD    = "export model as metabolite pattern";
    public final static String  CREATE_STARTING_VALUE_FILES_CMD     = "create starting value files";
    private             String  m_oldNodeNameBeforeRenaming         = "";
    public final static String  MODEL_TREE_CREATED_EVENT_CMD        = "model tree created event";
    
    private String m_newNameAfterRenaming = "";

    // For proper serialization
    private static final long   serialVersionUID = 1482325898889393L;

    /**
     * Adds an MRNumericPattern from a binary file.
     * @param file 
     */
    void addNumericPatternFrom_Binary_File( File file ) 
    {
       
        DefaultMutableTreeNode dmtn, node; 
        TreePath path = getSelectionPath( );
        dmtn = (DefaultMutableTreeNode)path.getLastPathComponent( );     
        
        File numericPatternBinFileName;
        if ( file == null )
        {
            numericPatternBinFileName = selectNumericPatternFromBinFileReference( );
        }
        else
        {
            numericPatternBinFileName = file;
        }
        
        if ( numericPatternBinFileName != null )
        {
            MRNumericPattern numPat = new MRNumericPattern( );
            // Now read it from a file
            try 
            {
                RandomAccessFile in = new RandomAccessFile( numericPatternBinFileName.getAbsoluteFile( ).toString( ), "rw" );
                numPat.readTDFDFitFormattedResponseFile( MRSignalProperties.getInstance( ).getNumberOfSamplePoints( ), 
                                                         numericPatternBinFileName.getAbsoluteFile( ).toString( ) );
            } 
            catch ( FileNotFoundException ex ) 
            {
                Logger.getLogger( TDFDFitModelTreeWithPopupMenu.class.getName( ) ).log( Level.SEVERE, null, ex );
            }

            numPat.setName( numericPatternBinFileName.getName( ) );
            numPat.setNumericPatternFileReference( numericPatternBinFileName.getAbsolutePath( ) );
            node = new DefaultMutableTreeNode( numPat );
            createUniqueNodeName( dmtn, node );
            dmtn.add( node );

            raiseActionEvent( MODEL_TREE_NODE_ADD_EVENT, this, MODEL_TREE_NODE_ADD_EVENT_CMD );

            // refresh treestructure
            ( (DefaultTreeModel)this.getModel( ) ).nodeStructureChanged( (TreeNode)dmtn );

            // select the newly created treenode
            TreePath tp = new TreePath( node.getPath( ) );
            setSelectionPath( tp );
            makeVisible( tp );
        }  
    }

    /**
     * Returns the file selected by the user.
     */
    private File getLoadPatternFileReference( String approveButtonText )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        File directory = null;
        FileFilter filter = null;
        
        if ( approveButtonText.contains( "parametric" ) == true )
        {
            directory = new File( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + System.getProperty( "file.separator" ) + "Parametric_Patterns" );
            filter = new FileNameExtensionFilter( "Parametric pattern file", "ppf" );
        }
        else if ( approveButtonText.contains( "numeric" ) == true )
        {
            directory = new File( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + System.getProperty( "file.separator" ) + "Numeric_Patterns" );
            filter = new FileNameExtensionFilter( "Numeric pattern file", "npf" );
        } 
        else if ( approveButtonText.contains( "metabolite" ) == true )
        {
            directory = new File( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + System.getProperty( "file.separator" ) + "Metabolites" );
            filter = new FileNameExtensionFilter( "Metabolite file", "mbf" );
        }
        else if ( approveButtonText.contains( "mrui" ) == true )
        {
            directory = new File( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + System.getProperty( "file.separator" ) + "Numeric_Patterns" );
            filter = new FileNameExtensionFilter( "MRUI file", "mrui" );
        }
        else if ( approveButtonText.contains( "binary" ) == true )
        {
            directory = new File( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) + System.getProperty( "file.separator" ) + "tdfdfit_responses" );
            filter = new FileNameExtensionFilter( "Binary file", "bin" );
        }
        JFileChooser fileChooser;
        
        if ( !directory.exists( ) )
        {
            fileChooser = new JFileChooser( fdef.getProperty(DEFAULT_TDFDFIT_FIT_MODEL_NAME ) );
        }
        else
        {
            fileChooser = new JFileChooser( directory );
        }
        fileChooser.setApproveButtonText( approveButtonText );
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.addChoosableFileFilter( filter );
        
        int result = fileChooser.showOpenDialog( this );
        
        File fileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                fileReference = fileChooser.getSelectedFile( );

                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return fileReference;
    }
    
    
}
