/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.spectrim.data.CSIData;
import org.spectrim.dicom.DicomDirectory;
import org.spectrim.dicom.DicomSliceCanvasPolyDraw;
import org.spectrim.lib.DualProgressBar;
import org.spectrim.model.GlobalModel;
import org.spectrim.patientdatabase.MRPatientSpectrumDataBase;
import org.spectrim.randomforestregression.DataGeneratorMain;
import org.spectrim.spectra.Spectra;
import org.spectrim.spectra.Spectrum;
import static org.spectrim.tdfdfit.TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY;
import org.spectrim.xyplot.XYPlotFrame;

/**
 * This GUI enables the user to generate model datasets for training 
 * machine learning algorithms.
 * @author johan
 */
public class TDFDFitRandomModelSignalGeneratorFrame extends javax.swing.JFrame implements ActionListener, ChangeListener, KeyListener 
{

    private MRLineModelSpectralResponse m_singleVoxelModelResponse;
    private final GlobalModel m_globalModel;
    private SingleVoxelMeasuredSpectralData m_singleVoxelMeasuredResponse;
    private int m_numberOfResponses = 500;
    private int m_currentlySelectedResponseNumber = 0;
    
    private SingleVoxelMeasuredSpectralData[]   m_allGeneratedGTNoisyResponses;
    
    private SingleVoxelMeasuredSpectralData[]   m_generatedVerificationResponses; 
    private MRLineModelSpectralResponse[]       m_RFRegressionPredictedResponses;
    private MRLineModelSpectralResponse[]       m_noiselessGroundTruthParameterResponses;
    
    private SingleVoxelMeasuredSpectralData[]       m_defaultModelTDFDFittedGroundTruthParameterResponses;
    private SingleVoxelMeasuredSpectralData[]       m_rfImprovedModelTDFDFittedGroundTruthParameterResponses;
    
    public final static int OVERALL_OFFSET          = 0;
    public final static int OVERALL_GAUSSWITDTH     = 1;
    public final static int OVERALL_PHASE           = 2;
    public final static int OVERALL_LORENTZWIDTH    = 3;
    
    private String m_groundTruthSignalsFileName;
    private String m_groundTruthParametersInputFileName;
    private String m_predictedParametersFileName;
    private int m_numberOfSpectraToFit;
    private int m_cntr;
    private int m_numberOfThreadsActive;
    private DualProgressBar m_dualProgressBar;
    private final JFrame m_parentFrame;
    private static final Vector<TDFDFitExecutorSwingWorker> m_workers = new Vector<TDFDFitExecutorSwingWorker>( );
    private static final Vector<TDFDFitExecutorVariableModelSingleThreaded> m_variableModelworkers = new Vector<TDFDFitExecutorVariableModelSingleThreaded>( );
    private int m_numberOfFitTreadsStarted;
    private NumericTDFDFitResults m_numericTDFDFit = new NumericTDFDFitResults( );
    private Vector<TDFDFittingParameters> m_fittedElem = new Vector<TDFDFittingParameters>( );
    private String m_fileNameBaseForCollectingData;
    
    private boolean m_allRFregrImprovedSpectraFitted = false;
    
    private boolean m_groundtruthParameterFileLoaded = false;
    private boolean m_predictedParamterFileLoaded    = false;
    private boolean m_groundtruthTDSignalsLoaded     = false;
    private BaselineAndWaterResidualModelParameterDefinitionJFrame m_baselineAndWaterResidualModelParameterDefinitionJFrame;
    
    /**
     * Creates new form TDFDFitRandomModelSignalGenerator.
     * @param model
     */
//    public TDFDFitRandomModelSignalGeneratorFrame( CSIData clinicalSpectrumData, 
//                                                   DicomDirectory ddir,
//                                                   GlobalModel model )
    public TDFDFitRandomModelSignalGeneratorFrame( JFrame parent, GlobalModel model )
    {
        m_globalModel = model;
        m_parentFrame = parent;
        
        setTitle( "..::: TDFDFit Random Model Signal Generator :::.." );
        
        initComponents( );
        displayCurrentlySelectedModel( );
        
        m_spectrumNumberSelectionSlider.setMinimum( 0 );
        m_spectrumNumberSelectionSlider.setMaximum( m_numberOfResponses - 1 );
        m_spectrumNumberSelectionSlider.setMajorTickSpacing( m_numberOfResponses / 10 );
        m_spectrumNumberSelectionSlider.setMinorTickSpacing( m_numberOfResponses / 100 );
        m_spectrumNumberSelectionSlider.setPaintTicks( true );
        m_spectrumNumberSelectionSlider.setPaintLabels( true );
        m_spectrumNumberSelectionSlider.addChangeListener( this );
        m_spectrumNumberSelectionSlider.setValue( m_currentlySelectedResponseNumber );
        m_selectedResponseNumberTextField.addKeyListener( this );
        
        ImperfectSpectrumModelParameters imperfPar = ImperfectSpectrumModelParameters.getInstance( );
        
        m_includeRandomBaselineRolejCheckBox.setSelected( imperfPar.getIncludeHasGeneralBaselineRoleArtefacts( ) );
        m_includeBroadWaterlineResidualjCheckBox.setSelected( imperfPar.getIncludeHasResidualWaterSignals( ) );
        m_includeSmileyBaselinejCheckBox.setSelected( imperfPar.getIncludeSmileyArtefacts( ) );
        
        m_showNoisyGroundTruthSpectrumCheckBox.addActionListener( this );
        m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox.addActionListener( this );
        m_showNoiselessGroundTruthSpectrumCheckBox.addActionListener( this );
        m_showPredictedSpectrumCheckBox.addActionListener( this );
        m_showPredictedvsModelNoiselessCheckBox.addActionListener( this );
        m_showPredictedvsModelNoisyCheckBox.addActionListener( this );
        m_showImprovedTDFDFitResultsCheckBox.addActionListener( this );
        m_showPlainTDFDFitResultsCheckBox.addActionListener( this );
        
               
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        setVisible( true );
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        m_displayTypeButtonGroup = new javax.swing.ButtonGroup();
        m_spectrumDisplayerPanel = new javax.swing.JPanel();
        m_MRParameterSpectrumModelViewer = new org.spectrim.tdfdfit.MRParameterSpectrumModelViewer();
        m_spectrumToDisplaySelectionPanel = new javax.swing.JPanel();
        m_spectrumNumberSelectionSlider = new javax.swing.JSlider();
        jLabel2 = new javax.swing.JLabel();
        m_showNoisyGroundTruthSpectrumCheckBox = new javax.swing.JCheckBox();
        m_showPredictedSpectrumCheckBox = new javax.swing.JCheckBox();
        m_selectedResponseNumberTextField = new javax.swing.JTextField();
        jLabel3 = new javax.swing.JLabel();
        m_updateWithEnteredValueButton = new javax.swing.JButton();
        m_showNoiselessGroundTruthSpectrumCheckBox = new javax.swing.JCheckBox();
        m_showPredictedvsModelNoisyCheckBox = new javax.swing.JCheckBox();
        m_showPredictedvsModelNoiselessCheckBox = new javax.swing.JCheckBox();
        m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox = new javax.swing.JCheckBox();
        m_writeFitQualityNumberTablesButton = new javax.swing.JButton();
        m_showPlainTDFDFitResultsCheckBox = new javax.swing.JCheckBox();
        m_showImprovedTDFDFitResultsCheckBox = new javax.swing.JCheckBox();
        jPanel5 = new javax.swing.JPanel();
        m_setNumberOfSignalsToGereratePanel = new javax.swing.JPanel();
        m_randomlyGenerateModelSignalsButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        m_numberOfSignalsToComputeTextField = new javax.swing.JTextField();
        m_closeDialogButton = new javax.swing.JButton();
        m_featureDefinitionPanel = new javax.swing.JPanel();
        m_numberOfTDFeaturePointsLabel = new javax.swing.JLabel();
        m_numberOfTDFeaturePointsTextField = new javax.swing.JTextField();
        m_numberOfTDFeaturePointsLabel1 = new javax.swing.JLabel();
        m_minimalFDFeaturePointsTextField = new javax.swing.JTextField();
        m_maximumFDFeaturePointsLabel = new javax.swing.JLabel();
        m_maximumFDFeaturePointsTextField = new javax.swing.JTextField();
        m_useTDAbsorptionFeaturesCheckBox = new javax.swing.JCheckBox();
        m_useTDDispersionFeaturesCheckBox = new javax.swing.JCheckBox();
        m_editFDAbsortionFeauturesCheckBox = new javax.swing.JCheckBox();
        m_useFDDispersionCheckBox = new javax.swing.JCheckBox();
        m_noiseRangeLabel = new javax.swing.JLabel();
        m_noiseRangeTextField = new javax.swing.JTextField();
        jPanel4 = new javax.swing.JPanel();
        m_includeSmileyBaselinejCheckBox = new javax.swing.JCheckBox();
        m_includeBroadWaterlineResidualjCheckBox = new javax.swing.JCheckBox();
        m_setArtefactAndBaselineSimulationParametersjButton = new javax.swing.JButton();
        m_includeRandomBaselineRolejCheckBox = new javax.swing.JCheckBox();
        m_randomVariationPanel = new javax.swing.JPanel();
        m_minimalAmplitudeLabel = new javax.swing.JLabel();
        m_minimalAmplitudeFactor = new javax.swing.JTextField();
        m_maximalAmplitudeLabel = new javax.swing.JLabel();
        m_maximalAmplitudeFactor = new javax.swing.JTextField();
        m_absoluteLeftOffsetShiftLabel = new javax.swing.JLabel();
        m_absoluteLeftOffsetShift = new javax.swing.JTextField();
        m_absoluteRightOffsetShiftLabel = new javax.swing.JLabel();
        m_absoluteRightOffsetShift = new javax.swing.JTextField();
        m_absoluteMaximalPhaseVariation = new javax.swing.JTextField();
        m_absoluteMinimalPhaseVariation = new javax.swing.JTextField();
        m_absoluteMinimalPhaseLabel = new javax.swing.JLabel();
        m_absoluteMaximumPhaseLabel = new javax.swing.JLabel();
        m_absoluteMaximalLorentzVariation = new javax.swing.JTextField();
        m_absoluteMinimalLorentzVariation = new javax.swing.JTextField();
        m_absoluteMinimalLorentzLabel = new javax.swing.JLabel();
        m_absoluteMaximumLorentzLabel = new javax.swing.JLabel();
        m_absoluteMinimalLorentzLabel1 = new javax.swing.JLabel();
        m_absoluteMinimalGaussianVariation = new javax.swing.JTextField();
        m_absoluteMaximumCaussianLabel = new javax.swing.JLabel();
        m_absoluteMaximalGaussianVariation = new javax.swing.JTextField();
        m_commonFrequencyCheckBox = new javax.swing.JCheckBox();
        m_commonPhaseCheckBox = new javax.swing.JCheckBox();
        m_commonLorentzianWidthCheckBox = new javax.swing.JCheckBox();
        m_commonGaussianWidthCheckBox = new javax.swing.JCheckBox();
        jPanel1 = new javax.swing.JPanel();
        m_defineGroundtruthSignalsFileButton = new javax.swing.JButton();
        m_groundtruthSignalFileNameLabel = new javax.swing.JLabel();
        m_predictedResponseButton = new javax.swing.JButton();
        m_predictedParameterFileNameLabel = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        m_firstGTResponseNumberTextField = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        m_lastGTResponseNumberTextField = new javax.swing.JTextField();
        m_defineGroundtruthSignalsParameterFileButton = new javax.swing.JButton();
        m_groundtruthParametersInputFileNameLabel = new javax.swing.JLabel();
        m_loadAndVerifyDatajButton = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        m_displayGeneratedGroundtruthRadioButton = new javax.swing.JRadioButton();
        m_displayGroundtruthVSPredictionRadioButton = new javax.swing.JRadioButton();
        jPanel3 = new javax.swing.JPanel();
        m_applyRFImprovedModelTDFDFitlToAllGTButton = new javax.swing.JButton();
        m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);

        m_MRParameterSpectrumModelViewer.setMaximumSize(new java.awt.Dimension(2048, 2048));

        m_spectrumToDisplaySelectionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Select spectrum to display"));

        jLabel2.setText("Show spectrum #");

        m_showNoisyGroundTruthSpectrumCheckBox.setText("Noisy GT");
        m_showNoisyGroundTruthSpectrumCheckBox.setEnabled(false);

        m_showPredictedSpectrumCheckBox.setText("RFregr prediction");
        m_showPredictedSpectrumCheckBox.setEnabled(false);

        m_selectedResponseNumberTextField.setText("0");

        jLabel3.setText("Selected response number ");

        m_updateWithEnteredValueButton.setText("Update!");
        m_updateWithEnteredValueButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_updateWithEnteredValueButtonActionPerformed(evt);
            }
        });

        m_showNoiselessGroundTruthSpectrumCheckBox.setText("Noiseless GT");
        m_showNoiselessGroundTruthSpectrumCheckBox.setEnabled(false);

        m_showPredictedvsModelNoisyCheckBox.setText("Noisy difference");
        m_showPredictedvsModelNoisyCheckBox.setEnabled(false);

        m_showPredictedvsModelNoiselessCheckBox.setText("Noiseless difference");
        m_showPredictedvsModelNoiselessCheckBox.setEnabled(false);

        m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox.setText(" GT difference (noise)");
        m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox.setEnabled(false);

        m_writeFitQualityNumberTablesButton.setText("Write FQN Table");
        m_writeFitQualityNumberTablesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_writeFitQualityNumberTablesButtonActionPerformed(evt);
            }
        });

        m_showPlainTDFDFitResultsCheckBox.setText("Straight TDFDFit ");
        m_showPlainTDFDFitResultsCheckBox.setEnabled(false);

        m_showImprovedTDFDFitResultsCheckBox.setText(" Improved  TDFDFit ");
        m_showImprovedTDFDFitResultsCheckBox.setEnabled(false);

        javax.swing.GroupLayout m_spectrumToDisplaySelectionPanelLayout = new javax.swing.GroupLayout(m_spectrumToDisplaySelectionPanel);
        m_spectrumToDisplaySelectionPanel.setLayout(m_spectrumToDisplaySelectionPanelLayout);
        m_spectrumToDisplaySelectionPanelLayout.setHorizontalGroup(
            m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_spectrumToDisplaySelectionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_spectrumToDisplaySelectionPanelLayout.createSequentialGroup()
                        .addComponent(jLabel2)
                        .addGap(18, 18, 18)
                        .addComponent(m_spectrumNumberSelectionSlider, javax.swing.GroupLayout.PREFERRED_SIZE, 459, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jLabel3)
                        .addGap(28, 28, 28)
                        .addComponent(m_selectedResponseNumberTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 60, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(m_spectrumToDisplaySelectionPanelLayout.createSequentialGroup()
                        .addComponent(m_showNoisyGroundTruthSpectrumCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_showNoiselessGroundTruthSpectrumCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_showPredictedSpectrumCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_showPredictedvsModelNoiselessCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(m_showPredictedvsModelNoisyCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_showPlainTDFDFitResultsCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(m_showImprovedTDFDFitResultsCheckBox)))
                .addGap(18, 18, 18)
                .addGroup(m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(m_writeFitQualityNumberTablesButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(m_updateWithEnteredValueButton, javax.swing.GroupLayout.PREFERRED_SIZE, 111, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        m_spectrumToDisplaySelectionPanelLayout.setVerticalGroup(
            m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_spectrumToDisplaySelectionPanelLayout.createSequentialGroup()
                .addGroup(m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(m_spectrumToDisplaySelectionPanelLayout.createSequentialGroup()
                        .addComponent(m_updateWithEnteredValueButton)
                        .addGap(18, 18, 18)
                        .addComponent(m_writeFitQualityNumberTablesButton))
                    .addGroup(m_spectrumToDisplaySelectionPanelLayout.createSequentialGroup()
                        .addGroup(m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(m_selectedResponseNumberTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel3))
                            .addGroup(m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(jLabel2)
                                .addComponent(m_spectrumNumberSelectionSlider, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(18, 18, 18)
                        .addGroup(m_spectrumToDisplaySelectionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(m_showNoisyGroundTruthSpectrumCheckBox)
                            .addComponent(m_showNoiselessGroundTruthSpectrumCheckBox)
                            .addComponent(m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox)
                            .addComponent(m_showPredictedSpectrumCheckBox)
                            .addComponent(m_showPredictedvsModelNoisyCheckBox)
                            .addComponent(m_showPredictedvsModelNoiselessCheckBox)
                            .addComponent(m_showPlainTDFDFitResultsCheckBox)
                            .addComponent(m_showImprovedTDFDFitResultsCheckBox))))
                .addContainerGap(22, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout m_spectrumDisplayerPanelLayout = new javax.swing.GroupLayout(m_spectrumDisplayerPanel);
        m_spectrumDisplayerPanel.setLayout(m_spectrumDisplayerPanelLayout);
        m_spectrumDisplayerPanelLayout.setHorizontalGroup(
            m_spectrumDisplayerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(m_spectrumToDisplaySelectionPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(m_MRParameterSpectrumModelViewer, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        m_spectrumDisplayerPanelLayout.setVerticalGroup(
            m_spectrumDisplayerPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_spectrumDisplayerPanelLayout.createSequentialGroup()
                .addComponent(m_MRParameterSpectrumModelViewer, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_spectrumToDisplaySelectionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10))
        );

        jPanel5.setMaximumSize(new java.awt.Dimension(611, 869));
        jPanel5.setMinimumSize(new java.awt.Dimension(611, 869));

        m_setNumberOfSignalsToGereratePanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Generate  Data"));

        m_randomlyGenerateModelSignalsButton.setText("Generate randomly varying model signals ..");
        m_randomlyGenerateModelSignalsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_randomlyGenerateModelSignalsButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Number of signals to generate");

        m_numberOfSignalsToComputeTextField.setText("500");

        m_closeDialogButton.setText("Close dialog");
        m_closeDialogButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_closeDialogButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout m_setNumberOfSignalsToGereratePanelLayout = new javax.swing.GroupLayout(m_setNumberOfSignalsToGereratePanel);
        m_setNumberOfSignalsToGereratePanel.setLayout(m_setNumberOfSignalsToGereratePanelLayout);
        m_setNumberOfSignalsToGereratePanelLayout.setHorizontalGroup(
            m_setNumberOfSignalsToGereratePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_setNumberOfSignalsToGereratePanelLayout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jLabel1)
                .addGap(18, 18, 18)
                .addComponent(m_numberOfSignalsToComputeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 76, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_randomlyGenerateModelSignalsButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_closeDialogButton)
                .addContainerGap())
        );
        m_setNumberOfSignalsToGereratePanelLayout.setVerticalGroup(
            m_setNumberOfSignalsToGereratePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_setNumberOfSignalsToGereratePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(m_randomlyGenerateModelSignalsButton)
                .addComponent(jLabel1)
                .addComponent(m_numberOfSignalsToComputeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(m_closeDialogButton))
        );

        m_featureDefinitionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Signal features definition"));

        m_numberOfTDFeaturePointsLabel.setText("Total number of TD acquistion points");

        m_numberOfTDFeaturePointsTextField.setText("140");

        m_numberOfTDFeaturePointsLabel1.setText("Start point in FD ");

        m_minimalFDFeaturePointsTextField.setText("540");

        m_maximumFDFeaturePointsLabel.setText("End  point in FD ");

        m_maximumFDFeaturePointsTextField.setText("940");

        m_useTDAbsorptionFeaturesCheckBox.setSelected(true);
        m_useTDAbsorptionFeaturesCheckBox.setText("Use TD absorption as features ");

        m_useTDDispersionFeaturesCheckBox.setSelected(true);
        m_useTDDispersionFeaturesCheckBox.setText("Use TD dispersion as features ");

        m_editFDAbsortionFeauturesCheckBox.setSelected(true);
        m_editFDAbsortionFeauturesCheckBox.setText("Use FD absorption as features ");

        m_useFDDispersionCheckBox.setSelected(true);
        m_useFDDispersionCheckBox.setText("Use FD dispersion as features ");

        m_noiseRangeLabel.setText("Noise range");

        m_noiseRangeTextField.setText("0.5");

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("Baseline and residual water"));

        m_includeSmileyBaselinejCheckBox.setText("Include smiley artefact");
        m_includeSmileyBaselinejCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_includeSmileyBaselinejCheckBoxActionPerformed(evt);
            }
        });

        m_includeBroadWaterlineResidualjCheckBox.setText("Include broad water baseline");
        m_includeBroadWaterlineResidualjCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_includeBroadWaterlineResidualjCheckBoxActionPerformed(evt);
            }
        });

        m_setArtefactAndBaselineSimulationParametersjButton.setText("Properties for baseline and water residual..");
        m_setArtefactAndBaselineSimulationParametersjButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_setArtefactAndBaselineSimulationParametersjButtonActionPerformed(evt);
            }
        });

        m_includeRandomBaselineRolejCheckBox.setText("Include general baseline role");
        m_includeRandomBaselineRolejCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_includeRandomBaselineRolejCheckBoxActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_includeRandomBaselineRolejCheckBox)
                    .addComponent(m_includeSmileyBaselinejCheckBox)
                    .addComponent(m_includeBroadWaterlineResidualjCheckBox)
                    .addComponent(m_setArtefactAndBaselineSimulationParametersjButton))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(m_includeSmileyBaselinejCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_includeBroadWaterlineResidualjCheckBox)
                .addGap(3, 3, 3)
                .addComponent(m_includeRandomBaselineRolejCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_setArtefactAndBaselineSimulationParametersjButton)
                .addContainerGap(36, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout m_featureDefinitionPanelLayout = new javax.swing.GroupLayout(m_featureDefinitionPanel);
        m_featureDefinitionPanel.setLayout(m_featureDefinitionPanelLayout);
        m_featureDefinitionPanelLayout.setHorizontalGroup(
            m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_featureDefinitionPanelLayout.createSequentialGroup()
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_noiseRangeLabel)
                    .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(m_editFDAbsortionFeauturesCheckBox, javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(m_numberOfTDFeaturePointsLabel))
                    .addComponent(m_numberOfTDFeaturePointsLabel1)
                    .addComponent(m_maximumFDFeaturePointsLabel)
                    .addComponent(m_useTDAbsorptionFeaturesCheckBox))
                .addGap(18, 18, 18)
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addComponent(m_useFDDispersionCheckBox, javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_numberOfTDFeaturePointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 70, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(m_noiseRangeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 70, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(m_useTDDispersionFeaturesCheckBox)))
                    .addComponent(m_minimalFDFeaturePointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 70, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_maximumFDFeaturePointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 70, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(51, Short.MAX_VALUE))
        );
        m_featureDefinitionPanelLayout.setVerticalGroup(
            m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_featureDefinitionPanelLayout.createSequentialGroup()
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfTDFeaturePointsLabel)
                    .addComponent(m_numberOfTDFeaturePointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_numberOfTDFeaturePointsLabel1)
                    .addComponent(m_minimalFDFeaturePointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_maximumFDFeaturePointsLabel)
                    .addComponent(m_maximumFDFeaturePointsTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_noiseRangeLabel)
                    .addComponent(m_noiseRangeTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(4, 4, 4)
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_useTDAbsorptionFeaturesCheckBox)
                    .addComponent(m_useTDDispersionFeaturesCheckBox))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_featureDefinitionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_editFDAbsortionFeauturesCheckBox)
                    .addComponent(m_useFDDispersionCheckBox)))
            .addComponent(jPanel4, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
        );

        m_randomVariationPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("Defined random parameter variations"));

        m_minimalAmplitudeLabel.setText("Minimal amplitude factor ");

        m_minimalAmplitudeFactor.setText("0");

        m_maximalAmplitudeLabel.setText("Maximal amplitude factor ");

        m_maximalAmplitudeFactor.setText("3.0");

        m_absoluteLeftOffsetShiftLabel.setText("Absolute left offset shift");

        m_absoluteLeftOffsetShift.setText("-5.0");
        m_absoluteLeftOffsetShift.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_absoluteLeftOffsetShiftActionPerformed(evt);
            }
        });

        m_absoluteRightOffsetShiftLabel.setText("Absolute right offset shift");

        m_absoluteRightOffsetShift.setText("5.0");

        m_absoluteMaximalPhaseVariation.setText("45");

        m_absoluteMinimalPhaseVariation.setText("-45");

        m_absoluteMinimalPhaseLabel.setText("Absolute minimal phase variation");

        m_absoluteMaximumPhaseLabel.setText("Absolute maximal phase variation");

        m_absoluteMaximalLorentzVariation.setText("6");

        m_absoluteMinimalLorentzVariation.setText("-1");
        m_absoluteMinimalLorentzVariation.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_absoluteMinimalLorentzVariationActionPerformed(evt);
            }
        });

        m_absoluteMinimalLorentzLabel.setText("Absolute minimal Lorentz variation");

        m_absoluteMaximumLorentzLabel.setText("Absolute maximal Lorentzian variation");

        m_absoluteMinimalLorentzLabel1.setText("Absolute minimal Gaussian variation");

        m_absoluteMinimalGaussianVariation.setText("0");

        m_absoluteMaximumCaussianLabel.setText("Absolute maximal Gaussian variation");

        m_absoluteMaximalGaussianVariation.setText("0");

        m_commonFrequencyCheckBox.setSelected(true);
        m_commonFrequencyCheckBox.setText(" Use common frequency shift for all resonances");
        m_commonFrequencyCheckBox.setEnabled(false);

        m_commonPhaseCheckBox.setSelected(true);
        m_commonPhaseCheckBox.setText(" Use common phase variation for all resonances");
        m_commonPhaseCheckBox.setEnabled(false);

        m_commonLorentzianWidthCheckBox.setSelected(true);
        m_commonLorentzianWidthCheckBox.setText(" Use common Lorentzian width variation for all resonances");
        m_commonLorentzianWidthCheckBox.setEnabled(false);

        m_commonGaussianWidthCheckBox.setSelected(true);
        m_commonGaussianWidthCheckBox.setText(" Use common Gaussian width variation for all resonances");
        m_commonGaussianWidthCheckBox.setEnabled(false);

        javax.swing.GroupLayout m_randomVariationPanelLayout = new javax.swing.GroupLayout(m_randomVariationPanel);
        m_randomVariationPanel.setLayout(m_randomVariationPanelLayout);
        m_randomVariationPanelLayout.setHorizontalGroup(
            m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_randomVariationPanelLayout.createSequentialGroup()
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(m_absoluteMaximumLorentzLabel, javax.swing.GroupLayout.Alignment.TRAILING)
                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, m_randomVariationPanelLayout.createSequentialGroup()
                            .addComponent(m_absoluteMaximumCaussianLabel)
                            .addGap(7, 7, 7)))
                    .addGroup(m_randomVariationPanelLayout.createSequentialGroup()
                        .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(m_absoluteMinimalLorentzLabel1)
                            .addComponent(m_absoluteRightOffsetShiftLabel)
                            .addComponent(m_minimalAmplitudeLabel)
                            .addComponent(m_maximalAmplitudeLabel)
                            .addComponent(m_absoluteLeftOffsetShiftLabel)
                            .addComponent(m_absoluteMinimalLorentzLabel)
                            .addComponent(m_absoluteMaximumPhaseLabel)
                            .addComponent(m_absoluteMinimalPhaseLabel))
                        .addGap(11, 11, 11)))
                .addGap(18, 18, 18)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_minimalAmplitudeFactor, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteRightOffsetShift, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteLeftOffsetShift, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_maximalAmplitudeFactor, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteMinimalPhaseVariation, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteMinimalLorentzVariation, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteMaximalPhaseVariation, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteMaximalLorentzVariation, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteMinimalGaussianVariation, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_absoluteMaximalGaussianVariation, javax.swing.GroupLayout.PREFERRED_SIZE, 75, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_commonFrequencyCheckBox)
                    .addComponent(m_commonPhaseCheckBox)
                    .addComponent(m_commonLorentzianWidthCheckBox)
                    .addComponent(m_commonGaussianWidthCheckBox))
                .addGap(0, 0, Short.MAX_VALUE))
        );
        m_randomVariationPanelLayout.setVerticalGroup(
            m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(m_randomVariationPanelLayout.createSequentialGroup()
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_minimalAmplitudeLabel)
                    .addComponent(m_minimalAmplitudeFactor, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(4, 4, 4)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_maximalAmplitudeLabel)
                    .addComponent(m_maximalAmplitudeFactor, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteLeftOffsetShiftLabel)
                    .addComponent(m_absoluteLeftOffsetShift, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_commonFrequencyCheckBox))
                .addGap(3, 3, 3)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteRightOffsetShiftLabel)
                    .addComponent(m_absoluteRightOffsetShift, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(4, 4, 4)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteMinimalPhaseLabel)
                    .addComponent(m_absoluteMinimalPhaseVariation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_commonPhaseCheckBox))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteMaximumPhaseLabel)
                    .addComponent(m_absoluteMaximalPhaseVariation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteMinimalLorentzLabel)
                    .addComponent(m_absoluteMinimalLorentzVariation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_commonLorentzianWidthCheckBox))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteMaximumLorentzLabel)
                    .addComponent(m_absoluteMaximalLorentzVariation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteMinimalLorentzLabel1)
                    .addComponent(m_absoluteMinimalGaussianVariation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(m_commonGaussianWidthCheckBox))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(m_randomVariationPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_absoluteMaximumCaussianLabel)
                    .addComponent(m_absoluteMaximalGaussianVariation, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
        );

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Prediction verification"));

        m_defineGroundtruthSignalsFileButton.setText("Set groundtruth signals file ");
        m_defineGroundtruthSignalsFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_defineGroundtruthSignalsFileButtonActionPerformed(evt);
            }
        });

        m_groundtruthSignalFileNameLabel.setText("No groundtruth signals file defined yet!");

        m_predictedResponseButton.setText("Set predicted RFRregr parameters file ");
        m_predictedResponseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_predictedResponseButtonActionPerformed(evt);
            }
        });

        m_predictedParameterFileNameLabel.setText("No predicted parameter file defined yet!");

        jLabel4.setText("First response number ");

        m_firstGTResponseNumberTextField.setText("8000");

        jLabel5.setText("Last response number ");

        m_lastGTResponseNumberTextField.setText("10000");
        m_lastGTResponseNumberTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_lastGTResponseNumberTextFieldActionPerformed(evt);
            }
        });

        m_defineGroundtruthSignalsParameterFileButton.setText("Set groundtruth RFregr input file ");
        m_defineGroundtruthSignalsParameterFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_defineGroundtruthSignalsParameterFileButtonActionPerformed(evt);
            }
        });

        m_groundtruthParametersInputFileNameLabel.setText("No groundtruth signals file defined yet!");

        m_loadAndVerifyDatajButton.setText("Load and verify");
        m_loadAndVerifyDatajButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_loadAndVerifyDatajButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addComponent(m_predictedResponseButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_defineGroundtruthSignalsFileButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(m_defineGroundtruthSignalsParameterFileButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGap(29, 29, 29)
                                .addComponent(m_groundtruthSignalFileNameLabel))
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                                .addGap(30, 30, 30)
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(m_predictedParameterFileNameLabel)
                                    .addComponent(m_groundtruthParametersInputFileNameLabel)))))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(jLabel4)
                        .addGap(18, 18, 18)
                        .addComponent(m_firstGTResponseNumberTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(jLabel5)
                        .addGap(18, 18, 18)
                        .addComponent(m_lastGTResponseNumberTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 77, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(m_loadAndVerifyDatajButton)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_defineGroundtruthSignalsFileButton)
                    .addComponent(m_groundtruthSignalFileNameLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_defineGroundtruthSignalsParameterFileButton)
                    .addComponent(m_groundtruthParametersInputFileNameLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_predictedResponseButton)
                    .addComponent(m_predictedParameterFileNameLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel5)
                        .addComponent(m_lastGTResponseNumberTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(m_loadAndVerifyDatajButton))
                    .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel4)
                        .addComponent(m_firstGTResponseNumberTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(0, 0, Short.MAX_VALUE))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder("Select signal type to display"));

        m_displayTypeButtonGroup.add(m_displayGeneratedGroundtruthRadioButton);
        m_displayGeneratedGroundtruthRadioButton.setSelected(true);
        m_displayGeneratedGroundtruthRadioButton.setText("Display ground truth signals ");

        m_displayTypeButtonGroup.add(m_displayGroundtruthVSPredictionRadioButton);
        m_displayGroundtruthVSPredictionRadioButton.setText("Display ground truth vs predicted signals");

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGap(49, 49, 49)
                .addComponent(m_displayGeneratedGroundtruthRadioButton)
                .addGap(71, 71, 71)
                .addComponent(m_displayGroundtruthVSPredictionRadioButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(m_displayGeneratedGroundtruthRadioButton)
                .addComponent(m_displayGroundtruthVSPredictionRadioButton))
        );

        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder("Apply default TDFDFit model to ground truth and/or test dataset"));

        m_applyRFImprovedModelTDFDFitlToAllGTButton.setText("Apply RFregr improved TDFDFit model to all GT test signals");
        m_applyRFImprovedModelTDFDFitlToAllGTButton.setEnabled(false);
        m_applyRFImprovedModelTDFDFitlToAllGTButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_applyRFImprovedModelTDFDFitlToAllGTButtonActionPerformed(evt);
            }
        });

        m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton.setText("Apply same TDFDFit model to all GT test signals");
        m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton.setEnabled(false);
        m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addComponent(m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(m_applyRFImprovedModelTDFDFitlToAllGTButton)
                .addContainerGap())
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel3Layout.createSequentialGroup()
                .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton)
                    .addComponent(m_applyRFImprovedModelTDFDFitlToAllGTButton))
                .addGap(10, 10, 10))
        );

        javax.swing.GroupLayout jPanel5Layout = new javax.swing.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(m_featureDefinitionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(m_setNumberOfSignalsToGereratePanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(m_randomVariationPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .addComponent(jPanel3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel5Layout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addComponent(m_randomVariationPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, 0)
                .addComponent(m_featureDefinitionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_setNumberOfSignalsToGereratePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(22, 22, 22)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(10, 10, 10))
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(m_spectrumDisplayerPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(m_spectrumDisplayerPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jPanel5, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(0, 0, Short.MAX_VALUE))))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void m_randomlyGenerateModelSignalsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_randomlyGenerateModelSignalsButtonActionPerformed
        
        m_displayGeneratedGroundtruthRadioButton.setSelected( true );
        
        TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        m_allGeneratedGTNoisyResponses = tdfdFitPrototypeParameters.createGroundtruthModelSpectralResponsesAndRandomParameterVariationsAsCSV( this );
        
        XYPlotFrame xyPlotFrm1 = new XYPlotFrame( "Examples signals of groundtruth" , this );
        SingleVoxelMeasuredSpectralData curRespToDisplay = null;
        
        for ( int i = 0; i < m_allGeneratedGTNoisyResponses.length/20; i++ )
        {
            curRespToDisplay = m_allGeneratedGTNoisyResponses[i];
            // curRespToDisplay.addNoiseInTimeDomain( new Float( m_noiseRangeTextField.getText( ) ) );
            // curRespToDisplay.doComplexConjuation( );
            xyPlotFrm1.addCurve( curRespToDisplay.getAbsorptionCurveFDPoints( ) , false );
        }
        xyPlotFrm1.addCurve( curRespToDisplay.getAbsorptionCurveFDPoints( ) , true );
        m_showNoisyGroundTruthSpectrumCheckBox.setEnabled( true );
    }//GEN-LAST:event_m_randomlyGenerateModelSignalsButtonActionPerformed

    private void m_closeDialogButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_closeDialogButtonActionPerformed
        this.dispose(  );
    }//GEN-LAST:event_m_closeDialogButtonActionPerformed

    private void m_absoluteLeftOffsetShiftActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_absoluteLeftOffsetShiftActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_absoluteLeftOffsetShiftActionPerformed

    private void m_updateWithEnteredValueButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_updateWithEnteredValueButtonActionPerformed
        m_spectrumNumberSelectionSlider.setValue( new Integer( m_selectedResponseNumberTextField.getText( ) ) );
        showResponseNumber( m_currentlySelectedResponseNumber );
    }//GEN-LAST:event_m_updateWithEnteredValueButtonActionPerformed

    private void m_absoluteMinimalLorentzVariationActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_absoluteMinimalLorentzVariationActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_absoluteMinimalLorentzVariationActionPerformed

    private void m_defineGroundtruthSignalsFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_defineGroundtruthSignalsFileButtonActionPerformed
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String last_used_path = defs.getProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY );
        
        if ( last_used_path.compareTo( "not defined yet!" ) == 0 )
        {
            last_used_path = System.getProperty( "user.home" );
            defs.setProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY , last_used_path );
            defs.save( );
        }
        
        String inputTDSignals = "Enter name of ground truth TD input signals for the RF-regression";
        m_groundTruthSignalsFileName = getFileReference( inputTDSignals, last_used_path, "csv" );
        if ( m_groundTruthSignalsFileName != null )
        {
            defs.setProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY , new File( m_groundTruthSignalsFileName ).getParent( ) );
            defs.save( );    
        }
        else
        {
            JOptionPane.showMessageDialog( this, "No valid ground truth file was defined!" );
        }
        
        String strippedDisplayFilename = new File( m_groundTruthSignalsFileName ).getName( );
        if ( strippedDisplayFilename.length( ) > 70 )
        {
            strippedDisplayFilename = strippedDisplayFilename.substring( 0, 30 ) + "  ...  " + strippedDisplayFilename.substring( strippedDisplayFilename.length( ) - 30, strippedDisplayFilename.length( ) );
        }
            
        m_groundtruthSignalFileNameLabel.setText( strippedDisplayFilename );
        
        int numberOfSignals = findAndSetNumberOfGroundtruthSignalsInFile( );
        m_firstGTResponseNumberTextField.setText( "" + 0 );
        m_lastGTResponseNumberTextField.setText( "" + numberOfSignals );
    }//GEN-LAST:event_m_defineGroundtruthSignalsFileButtonActionPerformed

    private void m_predictedResponseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_predictedResponseButtonActionPerformed
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String last_used_path = defs.getProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY );
        
        if ( last_used_path.compareTo( "not defined yet!" ) == 0 )
        {
            last_used_path = System.getProperty( "user.home" );
            defs.setProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY , last_used_path );
            defs.save( );
        }
         
        String predPars = "Enter name of predicted parameter file of RF-regression";
        m_predictedParametersFileName = getFileReference( predPars, last_used_path, "csv" );
        
        if ( m_predictedParametersFileName != null )
        {
            defs.setProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY , new File( m_predictedParametersFileName ).getParent( ) );
            defs.save( );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "No valid  predicted paramter file was defined!" );
        }
        
        m_predictedParameterFileNameLabel.setText( new File( m_predictedParametersFileName ).getName( ) );
    }//GEN-LAST:event_m_predictedResponseButtonActionPerformed

    private void m_defineGroundtruthSignalsParameterFileButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_defineGroundtruthSignalsParameterFileButtonActionPerformed
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String last_used_path = defs.getProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY );
        
        if ( last_used_path.compareTo( "not defined yet!" ) == 0 )
        {
            last_used_path = System.getProperty( "user.home" );
            defs.setProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY , last_used_path );
            defs.save( );
        }
        
        defs.setProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY , new File( m_groundTruthSignalsFileName ).getParent( ) );
        defs.save( );
        
        
        String gtPars = "Enter name of ground truth parameter file of RF-regression";
        m_groundTruthParametersInputFileName = getFileReference( gtPars ,last_used_path, "csv" );
        
        if ( m_groundTruthParametersInputFileName != null )
        {
            defs.setProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY , new File( m_groundTruthSignalsFileName ).getParent( ) );
            defs.save( );
        }
        else
        {
            JOptionPane.showMessageDialog( this, "No valid ground truth parameter file was defined!" );
        }
        m_groundtruthParametersInputFileNameLabel.setText( new File( m_groundTruthParametersInputFileName ).getName( ) );
    }//GEN-LAST:event_m_defineGroundtruthSignalsParameterFileButtonActionPerformed

    private void m_loadAndVerifyDatajButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_loadAndVerifyDatajButtonActionPerformed
        m_displayGroundtruthVSPredictionRadioButton.setSelected(true);
        m_spectrumNumberSelectionSlider.setMinimum( 0 );
       
        int range = new Integer( m_lastGTResponseNumberTextField.getText( ) ) -  new Integer( m_firstGTResponseNumberTextField.getText( ) );
       
        m_spectrumNumberSelectionSlider.setMaximum( range - 1 );
        m_spectrumNumberSelectionSlider.setMinorTickSpacing( 100 );
        m_spectrumNumberSelectionSlider.setMajorTickSpacing( 500 );
        m_spectrumNumberSelectionSlider.setPaintLabels( false );
        
        if ( m_groundtruthSignalFileNameLabel.getText( ).compareTo( "No groundtruth signals file defined yet!" ) != 0 )
        {
            loadNoisyGroundTruthTDSignalsFile( );
        }
        
        if ( m_groundtruthParametersInputFileNameLabel.getText( ).compareTo( "No groundtruth signals file defined yet!" ) != 0 )
        {
            loadGroundTruthModelParameterFile( );
        }
        
        if ( m_predictedParameterFileNameLabel.getText( ).compareTo( "No groundtruth signals file defined yet!" ) != 0 )
        {
            loadRFRegressionPredictedModelParameterFile( );
        }
        
        m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton.setEnabled( true );
        m_applyRFImprovedModelTDFDFitlToAllGTButton.setEnabled( true );
        
        
        if ( m_groundtruthParameterFileLoaded  == true )
        {
            m_showNoisyGroundTruthSpectrumCheckBox.setEnabled( true );
            m_showNoiselessGroundTruthSpectrumCheckBox.setEnabled( true );
            m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox.setEnabled( true );
            m_showPredictedSpectrumCheckBox.setEnabled( true );
            m_showPredictedvsModelNoiselessCheckBox.setEnabled( true );
            m_showPredictedvsModelNoisyCheckBox.setEnabled( true );
        }
        


    }//GEN-LAST:event_m_loadAndVerifyDatajButtonActionPerformed

    private void m_lastGTResponseNumberTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_lastGTResponseNumberTextFieldActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_m_lastGTResponseNumberTextFieldActionPerformed

    private void m_writeFitQualityNumberTablesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_writeFitQualityNumberTablesButtonActionPerformed
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String last_used_path = defs.getProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY );
        
        String groundtruthFQN = "Enter name of all FQN values";
        
        String outputFQNFileName = getFileReference( groundtruthFQN, last_used_path, "txt" );
        
        // Write the groundtruth FQNs!
        org.spectrim.lib.TextFileWriter writer;
            
        try 
        {
            
            writer = new org.spectrim.lib.TextFileWriter( new FileWriter( outputFQNFileName ) );
            
            writer.writeString( "FQN_GT_Abs_FD" );          writer.writeString( "," );
            writer.writeString( "FQN_GT_Disp_FD" );         writer.writeString( "," );
            writer.writeString( "FQN_GT_complex_FD" );      writer.writeString( "," );
            writer.writeString( "FQN_Pred_Abs_FD" );        writer.writeString( "," );
            writer.writeString( "FQN_Pred_Disp_FD" );       writer.writeString( "," );
            writer.writeString( "FQN_Pred_complex_FD" );    writer.writeString( "," );
            writer.writeString( "var_Pred_GT_Abs_FD" );     writer.writeString( "," );
            writer.writeString( "var_Pred_GT_Disp_FD" );    writer.writeString( "," );
            writer.writeString( "var_Pred_GT_complex_FD" ); writer.writeString( "," );
            
            writer.writeString( "FQN_GT_Abs_TD" );          writer.writeString(" , ");
            writer.writeString( "FQN_GT_Disp_TD" );         writer.writeString(" , ");
            writer.writeString( "FQN_GT_complex_TD" );      writer.writeString(" , ");
            writer.writeString( "FQN_Pred_Abs_TD" );        writer.writeString(" , ");
            writer.writeString( "FQN_Pred_Disp_TD" );       writer.writeString(" , ");
            writer.writeString( "FQN_Pred_complex_TD" );    writer.writeString(" , ");
            writer.writeString( "var_Pred_GT_Abs_TD" );     writer.writeString(" , ");
            writer.writeString( "var_Pred_GT_Disp_TD" );    writer.writeString(" , ");
            writer.writeString( "var_Pred_GT_complex_TD" ); writer.writeStringLn( "" );
            
            
//            m_generatedVerificationResponses; 
//            m_RFRegressionPredictedResponses;
//            m_noiselessGroundTruthParameterResponses;
            for ( int i = 0; i < m_generatedVerificationResponses.length; i++ )
            {
                
                System.out.println( "i = " + i );
                SingleVoxelMeasuredSpectralData svsGTResponse      = m_generatedVerificationResponses[i];
                MRLineModelSpectralResponse predictedResponse      = m_RFRegressionPredictedResponses[i];
                MRLineModelSpectralResponse noiseLessGTResponse    = m_noiselessGroundTruthParameterResponses [i];
                 
                float noiselessGroundTruthStandardDevValueFD = 0f;
                float groundtruthFQNValue_FD                 = 0f;
                float predictedFQNValue_FD                   = 0f;
                
                float[] noisyGT_absFD  = svsGTResponse.m_absorptionChannelFD;
                float[] noisyGT_dispFD = svsGTResponse.m_dispersionChannelFD;
                
                float[] noiselessPredicted_absFD  = predictedResponse.m_absorptionChannelFD;
                float[] noiselessPredicted_dispFD = predictedResponse.m_dispersionChannelFD;
                 
                float[] noiselessGT_absFD  = noiseLessGTResponse.m_absorptionChannelFD;
                float[] noiselessGT_dispFD = noiseLessGTResponse.m_dispersionChannelFD;
                
                
                float varianceDiffNoisyGTAbsFD              = 0f;
                float varianceDiffNoisyGTDispFD             = 0f;  
                
                float varianceDiffNoisyPredAbsFD            = 0f;
                float variancediffNoisyPredDispFD           = 0f;
                
                float varianceDiffNoiselessGTvsPredAbsFD    = 0f;
                float varianceDiffNoiselessGTvsPredDispFD   = 0f;
                
                float noiseFDAbsVarianceValue               = 0f;
                float noiseFDDispVarianceValue              = 0f;
                
                for ( int j = new Integer( m_minimalFDFeaturePointsTextField.getText( ) ); j < new Integer( m_maximumFDFeaturePointsTextField.getText( ) ); j++ )
                {
                    varianceDiffNoisyGTAbsFD += ( noisyGT_absFD[j] - noiselessGT_absFD[j] ) * ( noisyGT_absFD[j] - noiselessGT_absFD[j] );
                    varianceDiffNoisyGTDispFD += ( noisyGT_dispFD[j] - noiselessGT_dispFD[j] ) * ( noisyGT_dispFD[j] - noiselessGT_dispFD[j] );
                    
                    varianceDiffNoisyPredAbsFD += ( noisyGT_absFD[j] - noiselessPredicted_absFD[j] ) * ( noisyGT_absFD[j] - noiselessPredicted_absFD[j] );
                    variancediffNoisyPredDispFD += ( noisyGT_dispFD[j] - noiselessPredicted_dispFD[j] ) * ( noisyGT_dispFD[j] - noiselessPredicted_dispFD[j] );
                    
                    varianceDiffNoiselessGTvsPredAbsFD  +=  ( noiselessPredicted_absFD[j] - noiselessGT_absFD[j] ) * ( noiselessPredicted_absFD[j] - noiselessGT_absFD[j] );
                    varianceDiffNoiselessGTvsPredDispFD +=  ( noiselessPredicted_dispFD[j] - noiselessGT_dispFD[j] ) * ( noiselessPredicted_dispFD[j] - noiselessGT_dispFD[j] );
                    
                }
                
                
                // make the values compatible with sample variance of noise
                
                varianceDiffNoisyGTAbsFD = varianceDiffNoisyGTAbsFD / ( new Integer( m_maximumFDFeaturePointsTextField.getText( ) ) - new Integer( m_minimalFDFeaturePointsTextField.getText( ) ) - 1 );
                varianceDiffNoisyGTDispFD = varianceDiffNoisyGTDispFD / ( new Integer( m_maximumFDFeaturePointsTextField.getText( ) ) - new Integer( m_minimalFDFeaturePointsTextField.getText( ) ) - 1 );
                
                varianceDiffNoisyPredAbsFD = varianceDiffNoisyPredAbsFD / ( new Integer( m_maximumFDFeaturePointsTextField.getText( ) ) - new Integer( m_minimalFDFeaturePointsTextField.getText( ) ) - 1 );
                variancediffNoisyPredDispFD = variancediffNoisyPredDispFD / ( new Integer( m_maximumFDFeaturePointsTextField.getText( ) ) - new Integer( m_minimalFDFeaturePointsTextField.getText( ) ) - 1 );
                
                varianceDiffNoiselessGTvsPredAbsFD = varianceDiffNoiselessGTvsPredAbsFD / ( new Integer( m_maximumFDFeaturePointsTextField.getText( ) ) - new Integer( m_minimalFDFeaturePointsTextField.getText( ) ) - 1 );
                varianceDiffNoiselessGTvsPredDispFD = varianceDiffNoiselessGTvsPredAbsFD / ( new Integer( m_maximumFDFeaturePointsTextField.getText( ) ) - new Integer( m_minimalFDFeaturePointsTextField.getText( ) ) - 1 );
                
                float meanNoiseFDAbs  = 0f;
                float meanNoiseFDDisp = 0f;
                
                float varNoiseFDAbs  = 0f;
                float varNoiseFDDisp = 0f;
                
                
                for ( int j = 0; j < 200; j++ )
                {
                    meanNoiseFDAbs += noisyGT_absFD[j];
                    meanNoiseFDDisp += noisyGT_dispFD[j];
                }
                
                meanNoiseFDAbs = meanNoiseFDAbs / 200f;
                meanNoiseFDDisp = meanNoiseFDDisp / 200f;
                
                for ( int j = 0; j < 200; j++ )
                {
                    varNoiseFDAbs  += ( noisyGT_absFD[j] - meanNoiseFDAbs ) * ( noisyGT_absFD[j] - meanNoiseFDAbs );
                    varNoiseFDDisp += ( noisyGT_dispFD[j] - meanNoiseFDDisp ) * ( noisyGT_dispFD[j] - meanNoiseFDDisp );
                }
                
                // compute the sample variance
                varNoiseFDAbs = varNoiseFDAbs / 199f;
                varNoiseFDDisp = varNoiseFDDisp / 199f;
                
                // Output parameters
                float FQN_GT_Abs_FD = varianceDiffNoisyGTAbsFD / varNoiseFDAbs;
                float FQN_GT_Disp_FD = varianceDiffNoisyGTDispFD / varNoiseFDDisp;
                float FQN_GT_complex_FD = ( varianceDiffNoisyGTAbsFD + varianceDiffNoisyGTDispFD )  / ( varNoiseFDAbs + varNoiseFDDisp );
                
                // Output parameters
                float FQN_Pred_Abs_FD = varianceDiffNoisyPredAbsFD / varNoiseFDAbs;
                float FQN_Pred_Disp_FD = variancediffNoisyPredDispFD / varNoiseFDDisp;
                float FQN_Pred_complex_FD = ( varianceDiffNoisyPredAbsFD + variancediffNoisyPredDispFD ) / ( varNoiseFDAbs + varNoiseFDDisp );
                
                // Output parameters
                float var_Pred_GT_Abs_FD = varianceDiffNoiselessGTvsPredAbsFD;
                float var_Pred_GT_Disp_FD = varianceDiffNoiselessGTvsPredDispFD;
                float var_Pred_GT_complex_FD = ( varianceDiffNoiselessGTvsPredAbsFD + varianceDiffNoiselessGTvsPredDispFD );
                
                // now write these results
                writer.writeFloat( FQN_GT_Abs_FD ); writer.writeString(",");
                writer.writeFloat( FQN_GT_Disp_FD ); writer.writeString(",");
                writer.writeFloat( FQN_GT_complex_FD ); writer.writeString(",");
                writer.writeFloat( FQN_Pred_Abs_FD ); writer.writeString(",");
                writer.writeFloat( FQN_Pred_Disp_FD ); writer.writeString(",");
                writer.writeFloat( FQN_Pred_complex_FD ); writer.writeString(",");
                writer.writeFloat( var_Pred_GT_Abs_FD ); writer.writeString(",");
                writer.writeFloat( var_Pred_GT_Disp_FD ); writer.writeString(",");
                writer.writeFloat( var_Pred_GT_complex_FD ); writer.writeString(",");
                
                ////////////////////////////////////////////////////////////////
                // Now for time domain the same!
                ////////////////////////////////////////////////////////////////
                
                float noiselessGroundTruthStandardDevValueTD = 0f;
                float groundtruthFQNValue_TD    = 0f;
                float predictedFQNValue_TD      = 0f;
                
                float[] noisyGT_absTD  = svsGTResponse.m_absorptionChannelTD;
                float[] noisyGT_dispTD = svsGTResponse.m_dispersionChannelTD;
                
                float[] noiselessPredicted_absTD  = predictedResponse.m_absorptionChannelTD;
                float[] noiselessPredicted_dispTD = predictedResponse.m_dispersionChannelTD;
                 
                float[] noiselessGT_absTD  = noiseLessGTResponse.m_absorptionChannelTD;
                float[] noiselessGT_dispTD = noiseLessGTResponse.m_dispersionChannelTD;
                
                
                float varianceDiffNoisyGTAbsTD    = 0f;
                float varianceDiffNoisyGTDispTD   = 0f;        
                
                float varianceDiffNoisyPredAbsTD  = 0f;
                float variancediffNoisyPredDispTD = 0f;
                
                float varianceDiffNoiselessGTvsPredAbsTD = 0f;
                float varianceDiffNoiselessGTvsPredDispTD = 0f;
                
                float noiseTDAbsVarianceValue  = 0f;
                float noiseTDDispVarianceValue = 0f;
                
                for ( int j = 0; j < new Integer( m_numberOfTDFeaturePointsTextField.getText( ) ); j++ )
                {
                    varianceDiffNoisyGTAbsTD += ( noisyGT_absTD[j] - noiselessGT_absTD[j] ) * ( noisyGT_absTD[j] - noiselessGT_absTD[j] );
                    varianceDiffNoisyGTDispTD += ( noisyGT_dispTD[j] - noiselessGT_dispTD[j] ) * ( noisyGT_dispTD[j] - noiselessGT_dispTD[j] );
                    
                    varianceDiffNoisyPredAbsTD += ( noisyGT_absTD[j] - noiselessPredicted_absTD[j] ) * ( noisyGT_absTD[j] - noiselessPredicted_absTD[j] );
                    variancediffNoisyPredDispTD += ( noisyGT_dispTD[j] - noiselessPredicted_dispTD[j] ) * ( noisyGT_dispTD[j] - noiselessPredicted_dispTD[j] );
                    
                    varianceDiffNoiselessGTvsPredAbsTD  +=  ( noiselessPredicted_absTD[j] - noiselessGT_absTD[j] ) * ( noiselessPredicted_absTD[j] - noiselessGT_absTD[j] );
                    varianceDiffNoiselessGTvsPredDispTD +=  ( noiselessPredicted_dispTD[j] - noiselessGT_dispTD[j] ) * ( noiselessPredicted_dispTD[j] - noiselessGT_dispTD[j] );
                }
                
                // make the values compatible with sample variance of noise
                
                varianceDiffNoisyGTAbsTD = varianceDiffNoisyGTAbsTD / ( new Integer( m_numberOfTDFeaturePointsTextField.getText( ) ) - 1 );
                varianceDiffNoisyGTDispTD = varianceDiffNoisyGTDispTD / ( new Integer( m_numberOfTDFeaturePointsTextField.getText( ) ) - 1 );
                
                varianceDiffNoisyPredAbsTD = varianceDiffNoisyPredAbsTD / ( new Integer( m_numberOfTDFeaturePointsTextField.getText( ) ) - 1 );
                variancediffNoisyPredDispTD = variancediffNoisyPredDispFD / ( new Integer( m_numberOfTDFeaturePointsTextField.getText( ) ) - 1 );
    
                varianceDiffNoiselessGTvsPredAbsTD = varianceDiffNoiselessGTvsPredAbsTD / ( new Integer( m_numberOfTDFeaturePointsTextField.getText( ) ) - 1 );
                varianceDiffNoiselessGTvsPredDispTD = varianceDiffNoiselessGTvsPredAbsTD / ( new Integer( m_numberOfTDFeaturePointsTextField.getText( ) ) - 1 );
                
                float meanNoiseTDAbs  = 0f;
                float meanNoiseTDDisp = 0f;
                float varNoiseTDAbs  = 0f;
                float varNoiseTDDisp = 0f;
                
                for ( int j = 800; j < 940; j++ )
                {
                    meanNoiseTDAbs += noisyGT_absTD[j];
                    meanNoiseTDDisp += noisyGT_dispTD[j];
                }
                
                meanNoiseTDAbs = meanNoiseTDAbs / 140f;
                meanNoiseTDDisp = meanNoiseTDDisp / 140f;
                
                for ( int j = 800; j < 940; j++ )
                {
                    varNoiseTDAbs  += ( noisyGT_absTD[j] - meanNoiseTDAbs ) * ( noisyGT_absTD[j] - meanNoiseTDAbs );
                    varNoiseTDDisp += ( noisyGT_dispTD[j] - meanNoiseTDDisp ) * ( noisyGT_dispTD[j] - meanNoiseTDDisp );
                }
                
                // compute the sample variance
                varNoiseTDAbs = varNoiseTDAbs / 140f;
                varNoiseTDDisp = varNoiseTDDisp / 140f;
                
                // Output parameters
                float FQN_GT_Abs_TD = varianceDiffNoisyGTAbsTD / varNoiseTDAbs;
                float FQN_GT_Disp_TD = varianceDiffNoisyGTDispTD / varNoiseTDDisp;
                float FQN_GT_complex_TD = ( varianceDiffNoisyGTAbsTD + varianceDiffNoisyGTDispTD ) / ( varNoiseTDAbs + varNoiseTDDisp );
                
                // Output parameters
                float FQN_Pred_Abs_TD = varianceDiffNoisyPredAbsTD / varNoiseTDAbs;
                float FQN_Pred_Disp_TD = variancediffNoisyPredDispTD / varNoiseTDDisp;
                float FQN_Pred_complex_TD = ( varianceDiffNoisyPredAbsTD + variancediffNoisyPredDispTD ) / ( varNoiseTDAbs + varNoiseTDDisp );
                
                // Output parameters
                float var_Pred_GT_Abs_TD = varianceDiffNoiselessGTvsPredAbsTD;
                float var_Pred_GT_Disp_TD = varianceDiffNoiselessGTvsPredDispTD;
                float var_Pred_GT_complex_TD = ( varianceDiffNoiselessGTvsPredAbsTD + varianceDiffNoiselessGTvsPredDispTD );
                
                // now write these results
                writer.writeFloat( FQN_GT_Abs_TD ); writer.writeString(",");
                writer.writeFloat( FQN_GT_Disp_TD ); writer.writeString(",");
                writer.writeFloat( FQN_GT_complex_TD ); writer.writeString(",");
                writer.writeFloat( FQN_Pred_Abs_TD ); writer.writeString(",");
                writer.writeFloat( FQN_Pred_Disp_TD ); writer.writeString(",");
                writer.writeFloat( FQN_Pred_complex_TD ); writer.writeString(",");
                writer.writeFloat( var_Pred_GT_Abs_TD ); writer.writeString(",");
                writer.writeFloat( var_Pred_GT_Disp_TD ); writer.writeString(",");
                writer.writeFloat( var_Pred_GT_complex_TD ); writer.writeStringLn( "" );
                
            }
            
            writer.close( );
            
        }
        catch ( IOException ex ) 
        {
            Logger.getLogger( DicomSliceCanvasPolyDraw.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
    }//GEN-LAST:event_m_writeFitQualityNumberTablesButtonActionPerformed

    
    /**
     * This method applies the default model to the test responses only.
     * @param evt 
     */
    private void m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButtonActionPerformed
        
        javax.swing.SwingWorker<Void, Void> worker = new javax.swing.SwingWorker<Void, Void>( ) 
        {
            @Override
            protected Void doInBackground( ) throws Exception 
            {
                performFittingInMultiThreadMode( );
                return null;
            }

            @Override
            protected void done( ) 
            {
//              notifyActionListeners( ALL_FITTING_THREADS_DONE, ALL_FITTING_THREADS_DONE_CMD );
                
                
                
                m_showPlainTDFDFitResultsCheckBox.setEnabled( true ); 
            }
	};
	worker.execute( );

    }//GEN-LAST:event_m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButtonActionPerformed

    
    
     /**
     * This method calls the actual fitting using multiple java threads.
     */
    private void m_applyRFImprovedModelTDFDFitlToAllGTButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_applyRFImprovedModelTDFDFitlToAllGTButtonActionPerformed
        javax.swing.SwingWorker<Void, Void> worker = new javax.swing.SwingWorker<Void, Void>( ) 
        {
            @Override
            protected Void doInBackground( ) throws Exception 
            {
                performFittingInSingleThreadModeWithImprovedStartingValues( );
                m_showImprovedTDFDFitResultsCheckBox.setSelected( true );
                m_showImprovedTDFDFitResultsCheckBox.setEnabled( true );
                return null;
            }

            @Override
            protected void done( ) 
            {
//                notifyActionListeners( ALL_FITTING_THREADS_DONE, ALL_FITTING_THREADS_DONE_CMD );
                
                m_showPredictedSpectrumCheckBox.setEnabled( true );
                m_showPredictedvsModelNoiselessCheckBox.setEnabled( true );
//                m_showPredictedvsModelNoisyCheckBox.setEnabled( true );
//                m_showImprovedTDFDFitResultsCheckBox.setEnabled( true );
                
            }
	};
	worker.execute( );
    }//GEN-LAST:event_m_applyRFImprovedModelTDFDFitlToAllGTButtonActionPerformed

    private void m_setArtefactAndBaselineSimulationParametersjButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_setArtefactAndBaselineSimulationParametersjButtonActionPerformed
        m_baselineAndWaterResidualModelParameterDefinitionJFrame = new BaselineAndWaterResidualModelParameterDefinitionJFrame( );
        m_baselineAndWaterResidualModelParameterDefinitionJFrame.addActionListener( this );
    }//GEN-LAST:event_m_setArtefactAndBaselineSimulationParametersjButtonActionPerformed

    private void m_includeSmileyBaselinejCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_includeSmileyBaselinejCheckBoxActionPerformed
        ImperfectSpectrumModelParameters imperfPar = ImperfectSpectrumModelParameters.getInstance( );
        imperfPar.setIncludeSmileyArtefacts( m_includeSmileyBaselinejCheckBox.isSelected( ) );
        imperfPar.makeInstancePersistent( );
    }//GEN-LAST:event_m_includeSmileyBaselinejCheckBoxActionPerformed

    private void m_includeBroadWaterlineResidualjCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_includeBroadWaterlineResidualjCheckBoxActionPerformed
        ImperfectSpectrumModelParameters imperfPar = ImperfectSpectrumModelParameters.getInstance( );
        imperfPar.setIncludeHasResidualWaterSignals( m_includeBroadWaterlineResidualjCheckBox.isSelected( ) );
        imperfPar.makeInstancePersistent( );
    }//GEN-LAST:event_m_includeBroadWaterlineResidualjCheckBoxActionPerformed

    private void m_includeRandomBaselineRolejCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_m_includeRandomBaselineRolejCheckBoxActionPerformed
         ImperfectSpectrumModelParameters imperfPar = ImperfectSpectrumModelParameters.getInstance( );
         imperfPar.setIncludeHasGeneralBaselineRoleArtefacts( m_includeRandomBaselineRolejCheckBox.isSelected( ) );
         imperfPar.makeInstancePersistent( );
    }//GEN-LAST:event_m_includeRandomBaselineRolejCheckBoxActionPerformed

    
     /**
     * This method calls the actual fitting using multiple java threads.
     * This can only be called if there is one single prototype model
     * to be used in all threads.
     */
    private synchronized boolean performFittingInMultiThreadMode( ) 
    {
        
        long startComputationTime = System.currentTimeMillis( );
        int n_cores = Runtime.getRuntime( ).availableProcessors( );
  
        
//        System.out.println( "Number of cores          : " + n_cores );
//        System.out.println( "Number of spectra to fit : " + m_generatedVerificationResponses.length );

        m_numberOfSpectraToFit = m_generatedVerificationResponses.length;
              
        // Inititialize
        m_cntr = 0;
        m_numberOfThreadsActive = 0;
  
        // Date date = new Date( );
        // System.out.println( "Timestamp : " + new Timestamp( date.getTime( ) ) );
        String fileNameBase;
        
        m_dualProgressBar = new DualProgressBar( m_parentFrame, 
                                                 n_cores, 
                                                 m_numberOfSpectraToFit, 
                                                 new Dimension( 900, 150 ), true );
        
        m_dualProgressBar.setTitle( "Fitting ground truth test spectra using tdfdfit .."  );
        m_dualProgressBar.setUpperLabel( "<HTML><BR>     Number of threads active : " + m_numberOfThreadsActive  + " (maximal number of threads : " + n_cores + ")</HTML>" );
        m_dualProgressBar.setUpperProgress( 0 );
        //m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( true );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Finished number of spectra : " + 0 + "</HTML>" );
        
        m_dualProgressBar.addActionListener( this );
        
        int dotCounter = 0;
        
        while ( m_cntr < m_numberOfSpectraToFit )
        // while ( m_cntr < 10 )
        {            
           
            SingleVoxelMeasuredSpectralData svsGTResponse = m_generatedVerificationResponses[ m_cntr ]; 
            
            float[] absSpectrum  = new float[ svsGTResponse.getAbsorptionChannelTD( ).length ];
            float[] dispSpectrum = new float[ svsGTResponse.getDispersionChannelTD( ).length ];
           
            // There is a difference in internal representation of SpectrIm
            // and the representation within the TDFDFit part:
            // Therefore we must adapt the arrays to get proper represenatation:
            // namely complex conjugation ....
            for ( int i = 0; i < absSpectrum.length; i++ )
            {
                absSpectrum[i]  =  svsGTResponse.getAbsorptionChannelTD( )[i];
                dispSpectrum[i] =  -svsGTResponse.getDispersionChannelTD( )[i];
            }
            
            TDFDFitPrototypeModelSpectrumParameters protoTypeModel = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
            String base = protoTypeModel.getModelSpectrumName( );
            
            base = base.substring( 0, base.lastIndexOf( "." ) -  1 );
            
            m_fileNameBaseForCollectingData = "RF_regression_" + base + "_";
            
            fileNameBase = "RF_regression_" + base + "_" + m_cntr;
            
            fileNameBase = fileNameBase.replace( " ", "_" );
            fileNameBase = fileNameBase.replace( "^", "_" );
            fileNameBase = fileNameBase.replaceAll( "�", "_" );
            
            if ( m_numberOfThreadsActive < n_cores )
            {

                m_numberOfThreadsActive++;
                
                SingleVoxelMeasuredSpectralData dataToFit = new SingleVoxelMeasuredSpectralData( absSpectrum, dispSpectrum );
                TDFDFitExecutorSwingWorker worker = new TDFDFitExecutorSwingWorker( m_cntr, dataToFit, false, fileNameBase );
                m_workers.add( worker );
                m_cntr++;
                
                worker.addActionListener( this );
                worker.execute( );
                m_numberOfFitTreadsStarted++;
                m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
            }
            
        }
        
        // System.out.println( "### Leaving do while loop!" );
        
//        Date date2 = new Date( );
//        System.out.println( "Timestamp : " + new Timestamp( date2.getTime( ) ) );
         
        return true;
    }
    
    
    /**
     * This method does the same as the method above, but it first updates the
     * starting values with the ones found by the RFRegression classifier.
     * In the current implementation that can only be done in a single threaded 
     * way.
     * @return 
     */
    private synchronized boolean performFittingInSingleThreadModeWithImprovedStartingValues( ) 
    {
        
        TDFDFittingDefaults def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        String defaultModelName = def.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        String defaultModelPath = def.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );
        
        long startComputationTime = System.currentTimeMillis( );
        
        // int n_cores = Runtime.getRuntime( ).availableProcessors( );
        int n_cores = 1;
        
        ////
        // Test code
        ////
        
//        System.out.println( "Number of cores          : " + n_cores );
//        System.out.println( "Number of spectra to fit : " + m_generatedVerificationResponses.length );
                
        m_numberOfSpectraToFit = m_generatedVerificationResponses.length;
                
        // Inititialize
        m_cntr = 0;
        m_numberOfThreadsActive = 0;
  
//        Date date = new Date( );
//        System.out.println( "Timestamp : " + new Timestamp( date.getTime( ) ) );

        String fileNameBase;
        
        m_dualProgressBar = new DualProgressBar( m_parentFrame, 
                                                 n_cores, 
                                                 m_numberOfSpectraToFit, 
                                                 new Dimension( 900, 150 ), true );
        
        m_dualProgressBar.setTitle( "Fitting ground truth test spectra using tdfdfit .."  );
        m_dualProgressBar.setUpperLabel( "<HTML><BR>     Number of threads active : " + m_numberOfThreadsActive  + " (maximal number of threads : " + n_cores + ")</HTML>" );
        m_dualProgressBar.setUpperProgress( 0 );
        // m_dualProgressBar.setUpperLabel( "        Total progress " );
        m_dualProgressBar.setUpperProgressVisible( true );
        m_dualProgressBar.setLowerLabel( "<HTML><BR>     Finished number of spectra : " + 0 + "</HTML>" );
        
        m_dualProgressBar.addActionListener( this );
        
        int dotCounter = 0;
        
//        for ( int spectrumIDCounter = 0; spectrumIDCounter < 10; spectrumIDCounter++ ) // ( m_cntr < 10 )
        for ( int spectrumIDCounter = 0; spectrumIDCounter < m_numberOfSpectraToFit; spectrumIDCounter++ ) // ( m_cntr < 10 )    
        {            
           // if ( m_cntr == m_numberOfSpectraToFit - 1 )
//           if ( m_cntr == 9 )
//           {
//                m_allRFregrImprovedSpectraFitted = true;
//           }
            
            SingleVoxelMeasuredSpectralData svsGTResponse      = m_generatedVerificationResponses[ spectrumIDCounter ]; 
            
            float[] absSpectrum  = new float[ svsGTResponse.getAbsorptionChannelTD( ).length ];
            float[] dispSpectrum = new float[ svsGTResponse.getDispersionChannelTD( ).length ];

            // There is a difference in internal representation of SpectrIm
            // and the representation within the TDFDFit part:
            // Therefore we must adapt the arrays to get proper represenatation:
            // namely complex conjugation ....
            for ( int i = 0; i < absSpectrum.length; i++ )
            {
                absSpectrum[i]  = svsGTResponse.getAbsorptionChannelTD( )[i];
                dispSpectrum[i] = -svsGTResponse.getDispersionChannelTD( )[i];
            }
            
            // set the TDFDFit model name for this test signal
            int dotPosition = defaultModelName.indexOf( "." );
            String modelNameTrunc = defaultModelName.substring( 0, dotPosition - 1 );
            
            String temporaryTDFDFitModelName = modelNameTrunc + "_test_response_" + spectrumIDCounter + ".tfm";            
            def.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, temporaryTDFDFitModelName );
            def.save( );
            
            TDFDFitPrototypeModelSpectrumParameters protoTypeModel = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
            String base = protoTypeModel.getModelSpectrumName( );
            
            // load the prototype file to be able to use it!
            loadTDFDFitPrototypeModelSpectrumFile( new File( defaultModelPath + System.getProperty( "file.separator" )+ base ) );
            
            base = base.substring( 0, base.lastIndexOf( "." ) );
            
            base = base.replace( " ", "_" );
            base = base.replace( "^", "_" );
            
            m_fileNameBaseForCollectingData = "RF_regression_improved_" + base + "_";
            
            // fileNameBase = "RF_regression_improved_" + base + "_" + spectrumIDCounter;
            fileNameBase = "RF_regression_improved_" + base;

            // m_numberOfThreadsActive++;
            SingleVoxelMeasuredSpectralData dataToFit = new SingleVoxelMeasuredSpectralData( absSpectrum, dispSpectrum );
            TDFDFitExecutorVariableModelSingleThreaded worker = new TDFDFitExecutorVariableModelSingleThreaded( spectrumIDCounter, dataToFit, false, fileNameBase );
            worker.startTDFDFitSingleThreaded( );

            m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
            
        }
        
        def = TDFDFittingDefaults.getInstance( );
        def.load( );
        
        String defaultPathToRenameTo = def.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );
        
        try 
        {
            collectAllDataAfterAllFittingIsPerformed( defaultPathToRenameTo, m_fileNameBaseForCollectingData, false );
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( TDFDFitRandomModelSignalGeneratorFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
        
        // Set the default TDFDFit model name back
        
        def.setProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME, defaultModelName );
        def.save( );
        
        // System.out.println( "### Leaving do while loop!" );
        
//        Date date2 = new Date( );
//        System.out.println( "Timestamp : " + new Timestamp( date2.getTime( ) ) );
         
        return true;
    }
    
    
    
        /**
     * Loads the TDFDFit prototype model spectrum from a file which is 
     * measured, and serves as prototype for fitting other signals with.
     */
    private void loadTDFDFitPrototypeModelSpectrumFile( File TDFDfitPrototypeModelFileReference )
    {
        if ( ( TDFDfitPrototypeModelFileReference != null ) && 
             ( TDFDfitPrototypeModelFileReference.exists( ) ) )
        {
            try
            {
                RandomAccessFile reader = new RandomAccessFile( TDFDfitPrototypeModelFileReference, "rw" );
            
                MRSignalProperties localMRSignalProperties = MRSignalProperties.getInstance( );
                
                // Load all essential object attritubutes to a file.
                localMRSignalProperties.read( reader );
                
                // Load the fitting strategy; if not defined you will get the 
                // default strategy.
                MRFitStrategy localFitStrategy = MRFitStrategy.getInstance( );
                localFitStrategy.read( reader );
                
                // Read the TDFDFitPrototypeModelSpectrum object from file.
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).read( reader );
                
                // Read the single voxel spectral data response
                SingleVoxelMeasuredSpectralData data = new SingleVoxelMeasuredSpectralData();
                data.read( reader, false );
                
                
                // The model spectrum must know its m_MRSignalProperties object
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMRSignalProperties( localMRSignalProperties );
                TDFDFitPrototypeModelSpectrumParameters.getInstance( ).setMeasuredSpectrum( data );
                
                OrderedBaseLinePointsDataModel baseLineModel = OrderedBaseLinePointsDataModel.getInstance( );
                // Reads the baseline points from file
                baseLineModel.read( reader );
                
                try {
                    reader.close( );
                } catch (IOException ex) {
                    Logger.getLogger(TDFDFitRandomModelSignalGeneratorFrame.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
            catch ( FileNotFoundException e )
            {
                // Not handled yet.
            }
        }
    }
    
    
    
    /**
     * Returns the file selected by the user.
     */
    private String getFileReference( String title, String last_used_path, String file_type )
    {   
        TDFDFittingDefaults fdef = TDFDFittingDefaults.getInstance( );
        fdef.load( );
        
        JFileChooser fileChooser = new JFileChooser( last_used_path );
        
        fileChooser.setDialogTitle( title );
        if ( file_type.compareTo( "txt" ) == 0 )
        {
            fileChooser.setFileFilter( new FileNameExtensionFilter( "text file", "txt" ) );
        }
        
        if ( file_type.compareTo( "csv" ) == 0 )
        {
            fileChooser.setFileFilter( new FileNameExtensionFilter( "CSV file", "csv" ) );
        }
        
        
        int result = fileChooser.showOpenDialog( this );
        
        File screenshotFileReference = new File( "" );
        
        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                screenshotFileReference = fileChooser.getSelectedFile( );
                                                // screenshotFileReference = new File( screenshotFileReference.getPath( ) );
                                                break;
           case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
           case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }
        
        return screenshotFileReference.getAbsolutePath( );
    }
    
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(TDFDFitRandomModelSignalGeneratorFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(TDFDFitRandomModelSignalGeneratorFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(TDFDFitRandomModelSignalGeneratorFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(TDFDFitRandomModelSignalGeneratorFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
 //               new TDFDFitRandomModelSignalGeneratorFrame().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private org.spectrim.tdfdfit.MRParameterSpectrumModelViewer m_MRParameterSpectrumModelViewer;
    private javax.swing.JTextField m_absoluteLeftOffsetShift;
    private javax.swing.JLabel m_absoluteLeftOffsetShiftLabel;
    private javax.swing.JTextField m_absoluteMaximalGaussianVariation;
    private javax.swing.JTextField m_absoluteMaximalLorentzVariation;
    private javax.swing.JTextField m_absoluteMaximalPhaseVariation;
    private javax.swing.JLabel m_absoluteMaximumCaussianLabel;
    private javax.swing.JLabel m_absoluteMaximumLorentzLabel;
    private javax.swing.JLabel m_absoluteMaximumPhaseLabel;
    private javax.swing.JTextField m_absoluteMinimalGaussianVariation;
    private javax.swing.JLabel m_absoluteMinimalLorentzLabel;
    private javax.swing.JLabel m_absoluteMinimalLorentzLabel1;
    private javax.swing.JTextField m_absoluteMinimalLorentzVariation;
    private javax.swing.JLabel m_absoluteMinimalPhaseLabel;
    private javax.swing.JTextField m_absoluteMinimalPhaseVariation;
    private javax.swing.JTextField m_absoluteRightOffsetShift;
    private javax.swing.JLabel m_absoluteRightOffsetShiftLabel;
    private javax.swing.JButton m_applyDefaultTDFDFitModelToAllGTVerificationSignalsButton;
    private javax.swing.JButton m_applyRFImprovedModelTDFDFitlToAllGTButton;
    private javax.swing.JButton m_closeDialogButton;
    private javax.swing.JCheckBox m_commonFrequencyCheckBox;
    private javax.swing.JCheckBox m_commonGaussianWidthCheckBox;
    private javax.swing.JCheckBox m_commonLorentzianWidthCheckBox;
    private javax.swing.JCheckBox m_commonPhaseCheckBox;
    private javax.swing.JButton m_defineGroundtruthSignalsFileButton;
    private javax.swing.JButton m_defineGroundtruthSignalsParameterFileButton;
    private javax.swing.JRadioButton m_displayGeneratedGroundtruthRadioButton;
    private javax.swing.JRadioButton m_displayGroundtruthVSPredictionRadioButton;
    private javax.swing.ButtonGroup m_displayTypeButtonGroup;
    private javax.swing.JCheckBox m_editFDAbsortionFeauturesCheckBox;
    private javax.swing.JPanel m_featureDefinitionPanel;
    private javax.swing.JTextField m_firstGTResponseNumberTextField;
    private javax.swing.JLabel m_groundtruthParametersInputFileNameLabel;
    private javax.swing.JLabel m_groundtruthSignalFileNameLabel;
    private javax.swing.JCheckBox m_includeBroadWaterlineResidualjCheckBox;
    private javax.swing.JCheckBox m_includeRandomBaselineRolejCheckBox;
    private javax.swing.JCheckBox m_includeSmileyBaselinejCheckBox;
    private javax.swing.JTextField m_lastGTResponseNumberTextField;
    private javax.swing.JButton m_loadAndVerifyDatajButton;
    private javax.swing.JTextField m_maximalAmplitudeFactor;
    private javax.swing.JLabel m_maximalAmplitudeLabel;
    private javax.swing.JLabel m_maximumFDFeaturePointsLabel;
    private javax.swing.JTextField m_maximumFDFeaturePointsTextField;
    private javax.swing.JTextField m_minimalAmplitudeFactor;
    private javax.swing.JLabel m_minimalAmplitudeLabel;
    private javax.swing.JTextField m_minimalFDFeaturePointsTextField;
    private javax.swing.JLabel m_noiseRangeLabel;
    private javax.swing.JTextField m_noiseRangeTextField;
    private javax.swing.JTextField m_numberOfSignalsToComputeTextField;
    private javax.swing.JLabel m_numberOfTDFeaturePointsLabel;
    private javax.swing.JLabel m_numberOfTDFeaturePointsLabel1;
    private javax.swing.JTextField m_numberOfTDFeaturePointsTextField;
    private javax.swing.JLabel m_predictedParameterFileNameLabel;
    private javax.swing.JButton m_predictedResponseButton;
    private javax.swing.JPanel m_randomVariationPanel;
    private javax.swing.JButton m_randomlyGenerateModelSignalsButton;
    private javax.swing.JTextField m_selectedResponseNumberTextField;
    private javax.swing.JButton m_setArtefactAndBaselineSimulationParametersjButton;
    private javax.swing.JPanel m_setNumberOfSignalsToGereratePanel;
    private javax.swing.JCheckBox m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox;
    private javax.swing.JCheckBox m_showImprovedTDFDFitResultsCheckBox;
    private javax.swing.JCheckBox m_showNoiselessGroundTruthSpectrumCheckBox;
    private javax.swing.JCheckBox m_showNoisyGroundTruthSpectrumCheckBox;
    private javax.swing.JCheckBox m_showPlainTDFDFitResultsCheckBox;
    private javax.swing.JCheckBox m_showPredictedSpectrumCheckBox;
    private javax.swing.JCheckBox m_showPredictedvsModelNoiselessCheckBox;
    private javax.swing.JCheckBox m_showPredictedvsModelNoisyCheckBox;
    private javax.swing.JPanel m_spectrumDisplayerPanel;
    private javax.swing.JSlider m_spectrumNumberSelectionSlider;
    private javax.swing.JPanel m_spectrumToDisplaySelectionPanel;
    private javax.swing.JButton m_updateWithEnteredValueButton;
    private javax.swing.JCheckBox m_useFDDispersionCheckBox;
    private javax.swing.JCheckBox m_useTDAbsorptionFeaturesCheckBox;
    private javax.swing.JCheckBox m_useTDDispersionFeaturesCheckBox;
    private javax.swing.JButton m_writeFitQualityNumberTablesButton;
    // End of variables declaration//GEN-END:variables

    public int getNumberOfResponsesToCalculate( ) 
    {
        return new Integer( m_numberOfSignalsToComputeTextField.getText( ) );
    }

    public double getMinimalAmplitudeFactor( ) 
    {
       return new Double( m_minimalAmplitudeFactor.getText( ) );
    }

    public double getMaximalAmplitudeFactor( ) 
    {
        return new Double( m_maximalAmplitudeFactor.getText( ) );
    }

    public double getOffsetLeftShift( ) 
    {
        return new Double( this.m_absoluteLeftOffsetShift.getText( ) );
    }

    public double getOffsetRightShift( ) 
    {
        return new Double( this.m_absoluteRightOffsetShift.getText( ) );
    }

    public double getMinimalPhaseVariation( ) 
    {
         return new Double( m_absoluteMinimalPhaseVariation.getText( ) );
    }

    public double getMaximalPhaseVariation( ) 
    {
        return new Double( m_absoluteMaximalPhaseVariation.getText( ) );
    }

    public double getMinimalGaussianVariation( ) 
    {
        return new Double( this.m_absoluteMinimalGaussianVariation.getText( ) );
    }

    public double getMaximalGaussianVariation( ) 
    {
        return new Double( m_absoluteMaximalGaussianVariation.getText( ) );
    }

    public double getMinimalLorentzianVariation( ) 
    {
        return new Double( m_absoluteMinimalLorentzVariation.getText( ) );
    }

    public double getMaximalLorentzianVariation( ) 
    {
        return new Double( m_absoluteMaximalLorentzVariation.getText( ) );
    }
    
    public int getMinimalOffsetRangeFrequencyIndexPoint( )
    {
        return new Integer( m_minimalFDFeaturePointsTextField.getText( ) ).intValue( );
    }
    
    public int getMaximalOffsetRangeFrequencyIndexPoint( )
    {
        return new Integer( m_maximumFDFeaturePointsTextField.getText( ) ).intValue( );
    }
    
    public int getNumberOfTDSignalIndexPoint( )
    {
        return new Integer( this.m_numberOfTDFeaturePointsTextField.getText( ) ).intValue( );
    }

    public float getTDNoiseLevel( ) 
    {
        return new Float( m_noiseRangeTextField.getText( ) );
    }

    private void displayCurrentlySelectedModel( ) 
    {
        TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        m_singleVoxelModelResponse = tdfdFitPrototypeParameters.getCompleteModelResponse( );
        m_singleVoxelMeasuredResponse = tdfdFitPrototypeParameters.getMeasuredSpectrum( );
        
        m_MRParameterSpectrumModelViewer.setMeasuredSpectrum( m_singleVoxelModelResponse, true );
        m_MRParameterSpectrumModelViewer.setModelSpectrum( m_singleVoxelMeasuredResponse, false );
    }

    @Override
    public void stateChanged( ChangeEvent e ) 
    {
        if ( e.getSource( ) == this.m_spectrumNumberSelectionSlider )
        {
            if ( !m_spectrumNumberSelectionSlider.getValueIsAdjusting( ) ) 
            {
                m_currentlySelectedResponseNumber = (int)m_spectrumNumberSelectionSlider.getValue( );
                this.m_selectedResponseNumberTextField.setText( "" + m_currentlySelectedResponseNumber );
                showResponseNumber( m_currentlySelectedResponseNumber );
            }
        }
    } 

    @Override
    public void keyTyped( KeyEvent e ) 
    {
    }

    @Override
    public void keyReleased( KeyEvent e ) 
    {
    }

    @Override
    public void keyPressed( KeyEvent e ) 
    {
        if( e.getKeyCode( ) == KeyEvent.VK_ENTER )
        {
             if ( e.getSource( ) == m_selectedResponseNumberTextField )
             { 
                try
                {
                    Integer.parseInt( m_selectedResponseNumberTextField.getText( ) ); 
                 
                    int value = new Integer( m_selectedResponseNumberTextField.getText( ) );
                    if  ( value > m_numberOfResponses - 1 )
                    {
                        m_spectrumNumberSelectionSlider.setValue( m_numberOfResponses );
                        m_currentlySelectedResponseNumber = m_numberOfResponses - 1;
                    }
                    else
                    {
                        m_spectrumNumberSelectionSlider.setValue( value );
                    }
                    
                    if  ( value < 0 )
                    {
                        m_spectrumNumberSelectionSlider.setValue( 0 );
                        m_currentlySelectedResponseNumber = 0;
                    }
                    else
                    {
                        m_spectrumNumberSelectionSlider.setValue( value );
                    }
                    
                }
                catch ( NumberFormatException nfe ) 
                { 
                    
                    m_spectrumNumberSelectionSlider.setValue( this.m_currentlySelectedResponseNumber );
                    m_selectedResponseNumberTextField.setText( "" + m_currentlySelectedResponseNumber );
                    System.out.println( m_selectedResponseNumberTextField.getText( ) + " is not a valid integer number" ); 
                } 
                        
             }
        }
    }

    
    
    // m_generatedVerificationResponses -> the simulated response with noise (normally 10000 pieces)
    // m_RFRegressionPredictedResponses -> array with predicted responses
    // m_noiselessGroundTruthParameterResponses -> array with ground truth model responses. 
    
    private void showResponseNumber( int currentlySelectedResponseNumber ) 
    {
        TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        m_singleVoxelModelResponse = tdfdFitPrototypeParameters.getCompleteModelResponse( );
        m_singleVoxelMeasuredResponse = tdfdFitPrototypeParameters.getMeasuredSpectrum( );
        
        
        if ( m_displayGeneratedGroundtruthRadioButton.isSelected( ) == true )
        {    
            m_MRParameterSpectrumModelViewer.setMeasuredSpectrum( m_singleVoxelModelResponse, true );
            m_MRParameterSpectrumModelViewer.setModelSpectrum( m_singleVoxelMeasuredResponse, false );
        
            if ( m_allGeneratedGTNoisyResponses != null )
            {
                if ( m_showNoisyGroundTruthSpectrumCheckBox.isSelected( ) == true )
                {
                    m_MRParameterSpectrumModelViewer.setSimulatedSpectrum( m_allGeneratedGTNoisyResponses[ currentlySelectedResponseNumber ], false, false );
                }
            }
        }
        else
        {
            boolean once = true;
            
            if ( m_generatedVerificationResponses != null )
            {
                if ( m_showNoisyGroundTruthSpectrumCheckBox.isSelected( ) == true )
                {
                    m_MRParameterSpectrumModelViewer.setSimulatedSpectrum( m_generatedVerificationResponses[ currentlySelectedResponseNumber ], once, once );
                    once = false;
                }
            }
       
            if ( m_noiselessGroundTruthParameterResponses != null )
            {
                if ( this.m_showNoiselessGroundTruthSpectrumCheckBox.isSelected( ) == true )
                {
                    m_MRParameterSpectrumModelViewer.setNoiselessGroundTruthModelResponse( m_noiselessGroundTruthParameterResponses[ currentlySelectedResponseNumber ], once, once );
                    once = false;
                }
            }
            
            if  ( ( m_generatedVerificationResponses != null ) && ( m_noiselessGroundTruthParameterResponses != null ) )
            {
               
                // Difference responses go here after
                SingleVoxelMeasuredSpectralData noisyGT    = m_generatedVerificationResponses[ currentlySelectedResponseNumber ];
                MRLineModelSpectralResponse     modelGT    = m_noiselessGroundTruthParameterResponses[ currentlySelectedResponseNumber ];
                if ( m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox.isSelected( ) == true )
                {
                    SingleVoxelMeasuredSpectralData displayData = new SingleVoxelMeasuredSpectralData( noisyGT.getAbsorptionChannelTD( ),
                                                                                                       noisyGT.getDispersionChannelTD( ) );
                    
                    displayData.subtractTimeDomainResponse( modelGT.getAbsorptionChannelTD( ), modelGT.getDispersionChannelTD( ) );
            
                    if ( m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox.isSelected( ) == true )
                    {
                        m_MRParameterSpectrumModelViewer.setNoisyGTDifferenceResponse( displayData, once, once );
                        once = false;
                    }      
                }
            }
            
            if ( m_RFRegressionPredictedResponses != null )
            {
                if ( m_showPredictedSpectrumCheckBox.isSelected( ) == true )
                {
                    m_MRParameterSpectrumModelViewer.setPredictedSpectralModelResponse( m_RFRegressionPredictedResponses[ currentlySelectedResponseNumber ], once, once );
                    once = false;
                }
            }
            
            if ( ( m_generatedVerificationResponses != null ) && ( m_RFRegressionPredictedResponses != null ) )
            {
                // Difference responses go here after
                SingleVoxelMeasuredSpectralData noisyVerificationResponse = m_generatedVerificationResponses[ currentlySelectedResponseNumber ];
                MRLineModelSpectralResponse     predictedResp             = m_RFRegressionPredictedResponses[ currentlySelectedResponseNumber ];
                if ( ( noisyVerificationResponse != null ) && ( predictedResp != null ) )
                {
                    SingleVoxelMeasuredSpectralData displayData = new SingleVoxelMeasuredSpectralData( predictedResp.getAbsorptionChannelTD( ),
                                                                                                       predictedResp.getDispersionChannelTD( ) );
                    displayData.subtractTimeDomainResponse( noisyVerificationResponse.getAbsorptionChannelTD( ), noisyVerificationResponse.getDispersionChannelTD( ) );
            
                    if ( m_showPredictedvsModelNoisyCheckBox.isSelected( ) == true )
                    {
                        m_MRParameterSpectrumModelViewer.setNoisyDifferenceResponse( displayData, once, once );
                        once = false;
                    }            
                }
            }
            
            if ( ( m_RFRegressionPredictedResponses != null ) && ( m_noiselessGroundTruthParameterResponses != null ) )
            {
                // Difference responses go here after
                MRLineModelSpectralResponse     trueNoiselessPredicted  = m_RFRegressionPredictedResponses[ currentlySelectedResponseNumber ];
                MRLineModelSpectralResponse     modelNoiselessGT        = m_noiselessGroundTruthParameterResponses[ currentlySelectedResponseNumber ];
                
                SingleVoxelMeasuredSpectralData displayData1 = new SingleVoxelMeasuredSpectralData( trueNoiselessPredicted.getAbsorptionChannelTD( ),
                                                                                                    trueNoiselessPredicted.getDispersionChannelTD( ) );
                
                if ( ( trueNoiselessPredicted != null ) && ( modelNoiselessGT != null ) )
                {
                    
                    displayData1.subtractTimeDomainResponse( modelNoiselessGT.getAbsorptionChannelTD( ), modelNoiselessGT.getDispersionChannelTD( ) );
            
                    if ( m_showPredictedvsModelNoiselessCheckBox.isSelected( ) == true )
                    {
                        m_MRParameterSpectrumModelViewer.setNoiseLessDifferenceResponse( displayData1, once, once );
                        once = false;
                    }
                }
            }
            
            if ( ( m_defaultModelTDFDFittedGroundTruthParameterResponses != null ) && ( m_defaultModelTDFDFittedGroundTruthParameterResponses.length > currentlySelectedResponseNumber ) )
            {
                if ( m_showPlainTDFDFitResultsCheckBox.isSelected( ) )
                {            
                    m_MRParameterSpectrumModelViewer.setPlainTDFDFittedSpectrum( m_defaultModelTDFDFittedGroundTruthParameterResponses[ currentlySelectedResponseNumber ], once, once );
                    once = false;
                }
            }
            
            if ( ( m_rfImprovedModelTDFDFittedGroundTruthParameterResponses != null ) && ( m_rfImprovedModelTDFDFittedGroundTruthParameterResponses.length > currentlySelectedResponseNumber ) )
            {
                if ( m_showImprovedTDFDFitResultsCheckBox.isSelected( ) )
                {            
                    m_MRParameterSpectrumModelViewer.setRFImprovedTDFDFittedSpectrum( m_rfImprovedModelTDFDFittedGroundTruthParameterResponses[ currentlySelectedResponseNumber ], once, true );
                    once = false;
                }
            }
            
            
        }
        
    }

    /**
     * Find the number of signals defined in the signal file.
    */
    private int findAndSetNumberOfGroundtruthSignalsInFile( ) 
    {
        BufferedReader br;
        int linecounter = 0;
 
        try 
        {
            br = new BufferedReader( new FileReader( m_groundTruthSignalsFileName ) );
            String line;
            
            while ( ( line = br.readLine( ) ) != null ) 
            {
                linecounter++;
            }
            
            System.out.println( "Read ground truth signals : done (" + linecounter + ")" );
            br.close( );
            
        } 
        catch ( IOException ex ) 
        {   
            // nope!
        }
        
        return linecounter;
    }
    
    
    /**
     * Loads the in a earlier step defined signals of the signals.
     */
    private void loadNoisyGroundTruthTDSignalsFile( ) 
    {
        
        m_groundtruthTDSignalsLoaded = false;
        
        TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        int sample_points = tdfdFitPrototypeParameters.getCompleteModelResponse( ).getAbsorptionChannelTD( ).length;
        
//        m_allGeneratedGTResponses = new MRLineModelSpectralResponse[ new Integer( m_lastGTResponseNumberTextField.getText( ) ) ];
        m_generatedVerificationResponses = new SingleVoxelMeasuredSpectralData[ new Integer( m_lastGTResponseNumberTextField.getText( ) ) - new Integer( this.m_firstGTResponseNumberTextField.getText( ) ) + 1];
        
        m_defaultModelTDFDFittedGroundTruthParameterResponses = new SingleVoxelMeasuredSpectralData[ new Integer( m_lastGTResponseNumberTextField.getText( ) ) - new Integer( this.m_firstGTResponseNumberTextField.getText( ) ) ];
        m_rfImprovedModelTDFDFittedGroundTruthParameterResponses = new SingleVoxelMeasuredSpectralData[ new Integer( m_lastGTResponseNumberTextField.getText( ) ) - new Integer( this.m_firstGTResponseNumberTextField.getText( ) ) ];
        
        SingleVoxelMeasuredSpectralData response = null;
        float[] absTD = new float[ sample_points ];
        float[] dispTD = new float[ sample_points ];
        
        BufferedReader br;
        int linecounter = 0;
        int tokenCounter = 0;
        int verificationResponsesCounter = 0;
        
        try 
        {
            br = new BufferedReader( new FileReader( m_groundTruthSignalsFileName ) );
            String line;
            
            while ( ( line = br.readLine( ) ) != null ) 
            {
                
                if ( ( linecounter >= new Integer( m_firstGTResponseNumberTextField.getText( ) ) ) && 
                     ( linecounter <= new Integer( m_lastGTResponseNumberTextField.getText( ) ) ) )
                {
                    if ( line.trim( ).equals( "" ) ) break;
                    StringTokenizer sTok = new StringTokenizer( line, "," );
                    while ( sTok.hasMoreTokens( ) )
                    {
                        if ( tokenCounter < sample_points )
                        {
                            absTD[tokenCounter] = Float.parseFloat( sTok.nextToken( ) );
                            tokenCounter++;
                        }
                        else
                        {
                            dispTD[tokenCounter - sample_points] = Float.parseFloat( sTok.nextToken( ) );
                            tokenCounter++;
                        }   
                    }
                    
                    response = new SingleVoxelMeasuredSpectralData( absTD, dispTD );
                    
                    m_generatedVerificationResponses[ verificationResponsesCounter ] = response;
                    verificationResponsesCounter++;
                    tokenCounter = 0;
                    
                    
                }
                linecounter++;
            }
            System.out.println( "Read ground truth signals : done (" + linecounter + ")" );
            br.close( );
            m_groundtruthTDSignalsLoaded = true;
        } 
        catch ( Exception ex ) 
        {
            ex.printStackTrace( );
        }
    }

    private void loadRFRegressionPredictedModelParameterFile( ) 
    {
        
        m_predictedParamterFileLoaded = false;
        TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        int sample_points = tdfdFitPrototypeParameters.getCompleteModelResponse( ).getAbsorptionChannelTD( ).length;
        int numberOfPredictedSpectra = new Integer( m_lastGTResponseNumberTextField.getText( ) ) - new Integer( m_firstGTResponseNumberTextField.getText( ) ) + 1;
             
        int numberOfSpectralComponentsToFit = tdfdFitPrototypeParameters.getNumberOfSpectralComponentsToFit( );
        
        m_RFRegressionPredictedResponses = new MRLineModelSpectralResponse[numberOfPredictedSpectra];
        
        float[][] predictedParametersAll = new float[4+numberOfSpectralComponentsToFit][numberOfPredictedSpectra]; 
        float[]   predictedParameters  = new float[4+numberOfSpectralComponentsToFit]; 
        MRLineModelSpectralResponse response = null;
        float[] absTD = new float[ sample_points ];
        float[] dispTD = new float[ sample_points ];
        
        BufferedReader br;
        int linecounter     = -1;
        int responsecounter = 0;
        int tokenCounter    = 0;
        int verificationResponsesCounter = 0;
        
        try 
        {
            br = new BufferedReader( new FileReader( m_predictedParametersFileName ) );
            String line;
              
            while ( ( line = br.readLine( ) ) != null ) 
            {
                linecounter++;
                StringTokenizer sTokDummy = new StringTokenizer( line, "," );
                    
                // first test if the first line is numeric or just the title
                String firstStringToken = sTokDummy.nextToken( );
                    
                boolean test = true;
                float firstValue = 0f;
                try 
                {
                    firstValue = Float.parseFloat(firstStringToken);
                } 
                catch ( NumberFormatException nfe ) 
                {
                    test = false;
                }
                // if the line contains numerical values it is OK
                if ( test == true )
                {
                  
                    if ( linecounter < numberOfPredictedSpectra )
                    {
                        System.out.println( "Predicted parameters of number: " + responsecounter );
                        if ( line.trim( ).equals( "" ) ) break;
                        StringTokenizer sTok = new StringTokenizer( line, "," );
                        while (( sTok.hasMoreTokens( ) ) )
                        // while ( sTok.hasMoreTokens( ) )
                        {
                            switch ( tokenCounter )
                            {
                                case OVERALL_OFFSET:        predictedParametersAll[OVERALL_OFFSET][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            predictedParameters[OVERALL_OFFSET] = predictedParametersAll[OVERALL_OFFSET][responsecounter];
                                                            break;

                                case OVERALL_GAUSSWITDTH:   predictedParametersAll[OVERALL_GAUSSWITDTH][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            predictedParameters[OVERALL_GAUSSWITDTH] = predictedParametersAll[OVERALL_GAUSSWITDTH][responsecounter];
                                                            break;

                                case OVERALL_PHASE:         predictedParametersAll[OVERALL_PHASE][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            predictedParameters[OVERALL_PHASE] = predictedParametersAll[OVERALL_PHASE][responsecounter];
                                                            break;

                                case OVERALL_LORENTZWIDTH:  predictedParametersAll[OVERALL_LORENTZWIDTH][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            predictedParameters[OVERALL_LORENTZWIDTH] = predictedParametersAll[OVERALL_LORENTZWIDTH][responsecounter];
                                                            break;

                                default:                    predictedParametersAll[tokenCounter][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            predictedParameters[tokenCounter] = predictedParametersAll[tokenCounter][responsecounter];

                            }
                            tokenCounter++;
                        }

                        response = tdfdFitPrototypeParameters.calculateCompleteModelSpectralResponseBasedUponRandomForestRegressionPredictedParameters( predictedParameters );
                        
                        SingleVoxelMeasuredSpectralData originalMeasuredModelSpectrum = tdfdFitPrototypeParameters.getMeasuredSpectrum( );
                        tdfdFitPrototypeParameters.setMeasuredSpectrum( m_generatedVerificationResponses[responsecounter] );
                        // Update a (hopefully improved!) TDFDFitModel file and write it to filesystem ..
                        tdfdFitPrototypeParameters.updatePrototypeModeParametersBasedUponRandomForestRegressionPredictedParameters( predictedParameters );        
                        saveImprovedStartingValuesTDFDFitModelAsTo_TFM_File( responsecounter, tdfdFitPrototypeParameters );
                        // set back parameters ...
                        tdfdFitPrototypeParameters.restoreModelParametersToDefault( );
                        // set backprototype signal
                        tdfdFitPrototypeParameters.setMeasuredSpectrum( originalMeasuredModelSpectrum );
                        m_RFRegressionPredictedResponses[ responsecounter ] = response;
                        verificationResponsesCounter++;
                        System.out.println( "Read predicted response # " + responsecounter );
                        responsecounter++;
                        tokenCounter = 0;
                        
                    }
                }
            }
            
            br.close( );
            m_predictedParamterFileLoaded = true;
        } 
        catch ( IOException | NumberFormatException ex ) 
        {
            // to do
        }
    }
    
    private void loadGroundTruthModelParameterFile( ) 
    {    
        
        m_groundtruthParameterFileLoaded = false;
        TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        int sample_points = tdfdFitPrototypeParameters.getCompleteModelResponse( ).getAbsorptionChannelTD( ).length;
        int numberOfPredictedSpectra = new Integer( m_lastGTResponseNumberTextField.getText( ) ) - new Integer( m_firstGTResponseNumberTextField.getText( ) ) + 1;
          
        int numberOfSpectralComponentsToFit = tdfdFitPrototypeParameters.getNumberOfSpectralComponentsToFit( );
        
        m_noiselessGroundTruthParameterResponses = new MRLineModelSpectralResponse[numberOfPredictedSpectra];
        
        float[][] gtParametersAll = new float[4+numberOfSpectralComponentsToFit][numberOfPredictedSpectra]; 
        float[]   gtParameters  = new float[4+numberOfSpectralComponentsToFit]; 
        MRLineModelSpectralResponse response = null;
        float[] absTD = new float[ sample_points ];
        float[] dispTD = new float[ sample_points ];
        
        BufferedReader br;
        int linecounter = -1;
        int tokenCounter = 0;
        int responsecounter = 0;
        int verificationResponsesCounter = 0;
        
        try 
        {
            br = new BufferedReader( new FileReader( m_groundTruthParametersInputFileName ) );
            String line;
            
            while ( ( line = br.readLine( ) ) != null ) 
            {
                linecounter++;
                if ( linecounter < numberOfPredictedSpectra )
                {
                    
                    System.out.println( "Groundtruth parameters : " + linecounter );
                    if ( line.trim( ).equals( "" ) ) break;
                    StringTokenizer sTokDummy = new StringTokenizer( line, "," );
                    
                    // first test if the first line is numeric or just the title
                    String firstStringToken = sTokDummy.nextToken( );
                    
                    boolean test = true;
                    float firstValue = 0f;
                    try 
                    {
                        firstValue = Float.parseFloat( firstStringToken );
                    } 
                    catch ( NumberFormatException nfe ) 
                    {
                        test = false;
                    }
                    
                    // if the line contains numerical values it is OK
                    if ( test == true )
                    {
                        
                        StringTokenizer sTok = new StringTokenizer( line, "," );
                        
                        while ( ( sTok.hasMoreTokens( ) ) )
                        {    
                            switch ( tokenCounter )
                            {
                                case OVERALL_OFFSET:        gtParametersAll[OVERALL_OFFSET][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            gtParameters[OVERALL_OFFSET] = gtParametersAll[OVERALL_OFFSET][responsecounter];
                                                            break;

                                case OVERALL_GAUSSWITDTH:   gtParametersAll[OVERALL_GAUSSWITDTH][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            gtParameters[OVERALL_GAUSSWITDTH] = gtParametersAll[OVERALL_GAUSSWITDTH][responsecounter];
                                                            break;

                                case OVERALL_PHASE:         gtParametersAll[OVERALL_PHASE][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            gtParameters[OVERALL_PHASE] = gtParametersAll[OVERALL_PHASE][responsecounter];
                                                            break;

                                case OVERALL_LORENTZWIDTH:  gtParametersAll[OVERALL_LORENTZWIDTH][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            gtParameters[OVERALL_LORENTZWIDTH] = gtParametersAll[OVERALL_LORENTZWIDTH][responsecounter];
                                                            break;

                                default:                    gtParametersAll[tokenCounter][responsecounter] = Float.parseFloat( sTok.nextToken( ) );
                                                            gtParameters[tokenCounter] = gtParametersAll[tokenCounter][responsecounter];
                            }
                            tokenCounter++;
                        }
                    
                    
                        // First compute the ground thruth response
                        response = tdfdFitPrototypeParameters.calculateCompleteModelSpectralResponseBasedUponRandomForestRegressionPredictedParameters( gtParameters );

                        m_noiselessGroundTruthParameterResponses[ responsecounter ] = response;
                        verificationResponsesCounter++;
                        responsecounter++;
                        tokenCounter = 0;
                        System.out.println( "Read groundtruth response # " + linecounter );

                    }
                }
            }
            
            br.close( );
            m_groundtruthParameterFileLoaded = true;
            
        }
        catch ( Exception ex ) 
        {
            ex.printStackTrace( );
        }
    }

    /**
     * Saves the TDFDfit prototype model to a file with a new file name.
     * @param responseNumber
     * @param tdfdFitPrototypeParameters
     */
    public void saveImprovedStartingValuesTDFDFitModelAsTo_TFM_File( int responseNumber, 
                                                                     TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters )
    {
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        // Get reference to the actually used default model path and model name ..
        String resultPath    = defs.getProperty( TDFDFittingDefaults.DEFAULT_USED_TDFDFIT_MODEL_PATH );
        String modelNameBase = defs.getProperty( TDFDFittingDefaults.DEFAULT_TDFDFIT_FIT_MODEL_NAME );
        
        int dotPosition = modelNameBase.lastIndexOf( "." );
        modelNameBase = modelNameBase.substring( 0, dotPosition - 1 );
        
        File improvedModelOutputFile = new File( resultPath + 
                                                 System.getProperty( "file.separator" ) + 
                                                 modelNameBase + "_test_response_" + 
                                                 responseNumber );
        
        
        if ( improvedModelOutputFile != null )
        {
            try
            {
                RandomAccessFile writer;
                
                if ( improvedModelOutputFile.getName( ).endsWith( ".tfm" ) == false )
                {
                
                    writer = new RandomAccessFile( improvedModelOutputFile + ".tfm", "rw" );
                }
                else
                {
                    writer = new RandomAccessFile( improvedModelOutputFile, "rw" );    
                }
                
                // Write all essential object attritubutes to a file:
                // Write the MRSignalProperties object to file
                MRSignalProperties.getInstance( ).write( writer );
                
                // Write the fitting strategy; if not defined the 
                // default strategy will be written
                MRFitStrategy.getInstance( ).write( writer );
                
                // Write the TDFDFitPrototypeModelSpectrumParameters object to file
                tdfdFitPrototypeParameters.write( writer );
                
                m_singleVoxelMeasuredResponse = tdfdFitPrototypeParameters.getMeasuredSpectrum( );
                
                // Write the measured single voxel spectral data response
                m_singleVoxelMeasuredResponse.write( writer );
                
                // Write the baseline points to the file: has now meaning up till now ...
                OrderedBaseLinePointsDataModel baseLineModel = OrderedBaseLinePointsDataModel.getInstance( );
                baseLineModel.deleteAllPoints( );
                
                baseLineModel.addPoint( new Point2D.Double( (double)0,   (double)50  ) );
                baseLineModel.addPoint( new Point2D.Double( (double)150, (double)175 ) );
                baseLineModel.addPoint( new Point2D.Double( (double)200, (double)250 ) );
                baseLineModel.addPoint( new Point2D.Double( (double)550, (double)575 ) );
                
                baseLineModel.write( writer );
                
                try 
                {
                    writer.close( );
                } 
                catch ( IOException ex ) 
                {
                    Logger.getLogger( TDFDFitControlFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
                }
                
            }
            catch ( FileNotFoundException e )
            {
                // not handled yet
            }
        }
    } 
    
    
    public void collectAllDataAfterAllFittingIsPerformed( String path, String nameBase, boolean isStraightTDFDFitCase ) throws IOException 
    {

        System.out.println( "All fitting done !" );

        
        TDFDFitPrototypeModelSpectrumParameters tdfdFitPrototypeParameters = TDFDFitPrototypeModelSpectrumParameters.getInstance( );
        int samplePoints = tdfdFitPrototypeParameters.getCompleteModelResponse( ).getAbsorptionChannelTD( ).length;
        
        LinkedList<MRNumericPattern> numericPatternList = tdfdFitPrototypeParameters.getNumericPatterns( );
        
        MRNumericPattern firstModelNumericPattern = numericPatternList.getFirst();
        
        float referenceOffset = firstModelNumericPattern.getFittingParameters( ).getOffset( );
        float referenceGaussianLWC = firstModelNumericPattern.getFittingParameters( ).getLorentzianLineWidth( );
        float referenceLorentzianLWC = firstModelNumericPattern.getFittingParameters( ).getGaussianWidth( );
        float referencePhase = firstModelNumericPattern.getFittingParameters().getPhase( );
        
        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
        defs.load( );
        
        String resultPath = defs.getProperty( TDFDFittingDefaults.PATIENT_DATABASE_ROOT_PATH );

        List<NumericTDFDFitResults> numFitResultArray = new ArrayList<>( );

        String collectionDataPath = defs.getProperty(TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY );
        String collectParameterFileBase;
        
        if ( isStraightTDFDFitCase == false )
        {
            collectParameterFileBase = "ImprovedTDFDFitStartingModel";
        }
        else
        {
            collectParameterFileBase = "StraightTDFDFitStartingModel";
        }
        
        String output_TDFDFIT_Parameters = collectionDataPath + System.getProperty( "file.separator" ) + collectParameterFileBase + "_parameters.csv";   // asv 
        String output_TDFDFIT_Signals  = collectionDataPath + System.getProperty( "file.separator" ) + collectParameterFileBase + "_signals.csv";      // asv 
        
        
        float bandwidth = samplePoints;

        org.spectrim.lib.TextFileWriter fitParameterWriter  = null;
        org.spectrim.lib.TextFileWriter fitSignalWriter     = null;
  
        System.out.println( "Writing results to collection parameter file :" + output_TDFDFIT_Parameters );
        
        fitParameterWriter = new org.spectrim.lib.TextFileWriter( new FileWriter( output_TDFDFIT_Parameters ) );
        fitParameterWriter.setNumberFormat( "0.00000000E00" );
        
        fitSignalWriter = new org.spectrim.lib.TextFileWriter( new FileWriter( output_TDFDFIT_Signals ) );
        fitSignalWriter.setNumberFormat( "0.00000000E00" );
        
        
        if ( isStraightTDFDFitCase == false )
        {
            m_fileNameBaseForCollectingData = m_fileNameBaseForCollectingData.substring( 0, m_fileNameBaseForCollectingData.lastIndexOf( "_" ) - 1 );
            m_fileNameBaseForCollectingData = m_fileNameBaseForCollectingData.substring( 0, m_fileNameBaseForCollectingData.lastIndexOf( "_" ) );
        }
        
        
        for ( int i = 0; i < m_generatedVerificationResponses.length; i++ ) 
//        for ( int i = 0; i < 10; i++ ) 
        {
            m_numericTDFDFit = new NumericTDFDFitResults( ); 
            m_numericTDFDFit.setNumberOfPoints( samplePoints );
            
            String exlfileNameBase;
            
            if ( isStraightTDFDFitCase == true )
            {  
                exlfileNameBase = m_fileNameBaseForCollectingData + i;
                m_numericTDFDFit.setResultFileName( resultPath + System.getProperty( "file.separator" ) + exlfileNameBase + ".exl" );
            }
            else
            {
                exlfileNameBase = m_fileNameBaseForCollectingData;
                m_numericTDFDFit.setResultFileName( resultPath + System.getProperty( "file.separator" ) + exlfileNameBase + "_" + i + "_" + i + ".exl" );
            }
            
            m_numericTDFDFit.setComponentNames( TDFDFitPrototypeModelSpectrumParameters.getInstance( ).getComponentNameVector( ) );
            m_numericTDFDFit.readFile( );

            m_numericTDFDFit.collectSignalInformationFromFile( samplePoints, ".fit" );
            System.out.println( "File reader loop: " + i );

            numFitResultArray.add( m_numericTDFDFit );   
            
            // System.out.println( numFitResultArray.get( i ).getM_fileReference( ) );

            // output - file text writer
            m_fittedElem = m_numericTDFDFit.getFittedElements( );
//            TDFDFittingParameters fitParameters = m_fittedElem.get( i );
            
            double m_FQN = m_numericTDFDFit.getFitQualityNumber( );
            
            float[] absFitTD = m_numericTDFDFit.getFittedSpectrumRealTD( );
            float[] dispFitTD = m_numericTDFDFit.getFittedSpectrumImagTD( );
            
            SingleVoxelMeasuredSpectralData response = new SingleVoxelMeasuredSpectralData( absFitTD, dispFitTD );
            
            if ( isStraightTDFDFitCase == true )
            {
                m_defaultModelTDFDFittedGroundTruthParameterResponses[i] = response;
                System.out.println( " Default TDFDFit FQN : " + m_FQN );
            }
            else
            {
                m_rfImprovedModelTDFDFittedGroundTruthParameterResponses[i] = response;
                System.out.println( "Improved TDFDFit FQN : " + m_FQN );
            }
            
            
            System.out.println( m_numericTDFDFit.getM_fileReference( ) );

            for ( int j = 0; j < m_fittedElem.size( ); j++ )
            {
                TDFDFittingParameters params = m_fittedElem.get( j );
                      
                // fitParameterWriter.writeString( fitParameters.getComponentName( ) + "," );

                // fitParameterWriter.writeString( "pattern," );
                if ( j == 0 )
                {
                    fitParameterWriter.writeDouble( referenceOffset + ( ( params.getOffset( ) + bandwidth / 2f ) ) );
                    fitParameterWriter.writeString( "," );
                    fitParameterWriter.writeDouble( referenceGaussianLWC + params.getGwidth( ) );
                    fitParameterWriter.writeString( "," );
                    fitParameterWriter.writeDouble( referencePhase + params.getPhase( ) );
                    fitParameterWriter.writeString(",");
                    fitParameterWriter.writeDouble( referenceLorentzianLWC + params.getLwidth( ) );     
                    fitParameterWriter.writeString( "," );
                }
  
                fitParameterWriter.writeDouble( params.getAmplitude( ) );
                fitParameterWriter.writeString( "," );
                
            }
            fitParameterWriter.writeStringLn( "" );
            
            fitSignalWriter.writeArray( absFitTD );
            fitSignalWriter.writeString( "," );
            fitSignalWriter.writeArray( dispFitTD );
            fitSignalWriter.writeStringLn( "" );
        }

        try 
        {
            fitParameterWriter.flush( );
            fitParameterWriter.close( );
            
            fitSignalWriter.flush( );
            fitSignalWriter.close( );     
        } 
        catch ( IOException ex ) 
        {
            Logger.getLogger( DataGeneratorMain.class.getName( ) ).log( Level.SEVERE, null, ex );
        }
    }
     
     
     
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
          
        Object sender = e.getSource( );
        
        if ( sender instanceof BaselineAndWaterResidualModelParameterDefinitionJFrame )
        {
            if ( e.getID( ) == BaselineAndWaterResidualModelParameterDefinitionJFrame.QUIT_AND_SAVE )
            {
                ImperfectSpectrumModelParameters imperfPar = ImperfectSpectrumModelParameters.getInstance( );
        
                m_includeRandomBaselineRolejCheckBox.setSelected( imperfPar.getIncludeHasGeneralBaselineRoleArtefacts( ) );
                m_includeBroadWaterlineResidualjCheckBox.setSelected( imperfPar.getIncludeHasResidualWaterSignals( ) );
                m_includeSmileyBaselinejCheckBox.setSelected( imperfPar.getIncludeSmileyArtefacts( ) );
            }
        }
        
        if ( sender instanceof TDFDFitExecutorSwingWorker ) 
        {
            if ( e.getID( ) == TDFDFitExecutorSwingWorker.IS_PERFORMED ) 
            {
                // decrement the number of active threads
                m_numberOfThreadsActive--;
                m_dualProgressBar.setUpperProgress(m_numberOfThreadsActive);
                System.out.println( "Number of active threads -- : " + m_numberOfThreadsActive );
                
                if ( ( this.m_numberOfThreadsActive == 0 ) ) 
                {
                    System.out.println( "Action performed called" );
                    try 
                    {                
                        
                        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
                        defs.load( );
                        
                        String pathToStore = defs.getProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY );
                        String fileNameBase = "TDFDFitDefaultFit";
                        collectAllDataAfterAllFittingIsPerformed( pathToStore, fileNameBase, true );
                    } 
                    catch ( IOException ex ) 
                    {
                        Logger.getLogger( TDFDFitRandomModelSignalGeneratorFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }
                }
            }
        }
         
        if ( sender instanceof TDFDFitExecutorVariableModelSingleThreaded ) 
        {
            if ( e.getID( ) == TDFDFitExecutorSwingWorker.IS_PERFORMED ) 
            {
                // decrement the number of active threads
                m_numberOfThreadsActive--;
                m_dualProgressBar.setUpperProgress( m_numberOfThreadsActive );
                System.out.println( "Number of active threads -- : " + m_numberOfThreadsActive );
                
                if ( ( m_allRFregrImprovedSpectraFitted == true ) ) 
                {
                    System.out.println( "Action performed called" );
                    try 
                    {   
                        TDFDFittingDefaults defs = TDFDFittingDefaults.getInstance( );
                        defs.load( );
                        String pathToStore = defs.getProperty( TDFDFittingDefaults.LAST_RANDOMFOREST_REGRESSION_FILES_DIRECTORY );
                        String fileNameBase = "TDFDFitRFImprovedFit";
                        collectAllDataAfterAllFittingIsPerformed( pathToStore, fileNameBase, false );
                    }
                    catch ( IOException ex ) 
                    {
                        Logger.getLogger( TDFDFitRandomModelSignalGeneratorFrame.class.getName( ) ).log( Level.SEVERE, null, ex );
                    }
                }
            }
        }
        
        if ( e.getSource( ) == m_showNoisyGroundTruthSpectrumCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
        
        if ( e.getSource( ) == m_showGroundTruthDifferenceNoiseOnlySpectrumCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
        
        if ( e.getSource( ) == m_showNoiselessGroundTruthSpectrumCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
        
        if ( e.getSource( ) == m_showPredictedSpectrumCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
        
        if ( e.getSource( ) == m_showPredictedvsModelNoiselessCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
        
        if ( e.getSource( ) == m_showPredictedvsModelNoisyCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
        
        if ( e.getSource( ) == m_showPlainTDFDFitResultsCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
        
        if ( e.getSource( ) == m_showImprovedTDFDFitResultsCheckBox )
        {
            this.showResponseNumber( m_currentlySelectedResponseNumber );
        }
    }

    public boolean getIncludeSmileyArtifactIntoBaseline( ) 
    {
        return m_includeSmileyBaselinejCheckBox.isSelected( );
    }
    
    public boolean getIncludeRandomlyVaryingBaseline( ) 
    {
        return this.m_includeRandomBaselineRolejCheckBox.isSelected( );
    }
    
    public boolean getIncludeResidualWaterBaseline( )
    {
        return this.m_includeBroadWaterlineResidualjCheckBox.isSelected( );
    }
}
