/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * 
 * Container class to define the values of a by TDFDFit fitted element.
 * 
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class TDFDFittingParameters 
{
    
    
    /**
     * Default constructor.
     * @param componentName
     */
    public TDFDFittingParameters( String componentName )
    {
        m_componentName = componentName;
        // Set to default value if constructed
        m_amplitude = 0.0;
        m_offset    = 0.0;
        m_phase     = 0.0;
        m_lwidth    = 0.0;
        m_gwidth    = 0.0;
        
        // and the associated errors
        m_e_amplitude    = 0.0;
        m_e_offset       = 0.0;
        m_e_phase        = 0.0;
        m_e_lwidth       = 0.0;
        m_e_gwidth       = 0.0;
        
        m_amplitudeRef  = -1;
        m_offsetRef     = -1;
        m_gWidthRef     = -1;
        m_lWidthRef     = -1;
        m_phaseRef      = -1;
    }

    /**
     * Writes most important attributes to the DataOutputStream.
     * @param dos
     * @throws IOException 
     */
    void writeToDataOutputStream( DataOutputStream dos ) throws IOException 
    {
        
        dos.writeUTF( m_componentName );
        dos.writeInt( m_number );
        
        // System.out.println( "m_amplitude = "  + m_amplitude );
        
        // Set to default value if constructed
        dos.writeDouble( m_amplitude );
        dos.writeDouble( m_offset );
        dos.writeDouble( m_phase );
        dos.writeDouble( m_lwidth );
        dos.writeDouble( m_gwidth );
        
        // and the associated errors
        dos.writeDouble( m_e_amplitude );
        dos.writeDouble( m_e_offset );
        dos.writeDouble( m_e_phase );
        dos.writeDouble( m_e_lwidth );
        dos.writeDouble( m_e_gwidth );
        
        dos.writeInt( m_amplitudeRef );
        dos.writeInt( m_offsetRef );
        dos.writeInt( m_gWidthRef );
        dos.writeInt( m_lWidthRef );
        dos.writeInt( m_phaseRef );
    }
    
    /**
     * Writes most important attributes to the DataOutputStream.
     * @param dos
     * @throws IOException 
     */
    void readFromDataInputStream( DataInputStream dis ) throws IOException 
    {
        m_componentName = dis.readUTF( );
        m_number        = dis.readInt(  );
        
        // Set to default value if constructed
        m_amplitude = dis.readDouble(  );
        m_offset    = dis.readDouble(  );
        m_phase     = dis.readDouble(  );
        m_lwidth    = dis.readDouble(  );
        m_gwidth    = dis.readDouble(  );
        
        // and the associated errors
        m_e_amplitude   = dis.readDouble(  );
        m_e_offset      = dis.readDouble(  );
        m_e_phase       = dis.readDouble(  );
        m_e_lwidth      = dis.readDouble(  );
        m_e_gwidth      = dis.readDouble(  );
        
        m_amplitudeRef  = dis.readInt(  );
        m_offsetRef     = dis.readInt(  );
        m_gWidthRef     = dis.readInt(  );
        m_lWidthRef     = dis.readInt(  );
        m_phaseRef      = dis.readInt(  );
    }
    
    public int getNumber( ) 
    {
        return m_number;
    }

    public void setNumber( int m_number ) 
    {
        this.m_number = m_number;
    }

    public String getElementType( ) 
    {
        return m_elementType;
    }

    public void setElementType( String m_elementType ) 
    {
        this.m_elementType = m_elementType;
    }

    public double getAmplitude( ) 
    {
        return m_amplitude;
    }

    public void setAmplitude( double m_amplitude ) 
    {
        this.m_amplitude = m_amplitude;
    }

    public double getOffset( ) 
    {
        return m_offset;
    }

    public void setOffset( double m_offset ) 
    {
        this.m_offset = m_offset;
    }

    public double getPhase( ) 
    {
        return m_phase;
    }

    public void setPhase( double m_phase ) 
    {
        this.m_phase = m_phase;
    }

    public double getLwidth( ) 
    {
        return m_lwidth;
    }

    public void setLwidth( double m_lwidth ) 
    {
        this.m_lwidth = m_lwidth;
    }

    public double getGwidth( ) 
    {
        return m_gwidth;
    }

    public void setGwidth( double m_gwidth ) 
    {
        this.m_gwidth = m_gwidth;
    }

    public double getE_Amplitude( ) 
    {
        return m_e_amplitude;
    }

    public void setE_Amplitude( double m_e_amplitude ) 
    {
        this.m_e_amplitude = m_e_amplitude;
    }

    void setAmplitudeModelReference( int  amplRef ) 
    {
        m_amplitudeRef = amplRef;
    }
    
    public int getAmplitudeModelReference( ) 
    {
        return m_amplitudeRef;
    }
    
    void setPhaseModelReference( int  phaseRef ) 
    {
        m_phaseRef = phaseRef;
    }
    
    public int getPhaseModelReference( ) 
    {
        return m_phaseRef;
    }
    
    void setOffsetModelReference( int  offsetRef ) 
    {
        m_offsetRef = offsetRef;
    }
    
    public int getOffsetModelReference( ) 
    {
        return m_offsetRef;
    }
    
    void setGWidthModelReference( int  gWidthRef ) 
    {
        m_gWidthRef = gWidthRef;
    }
    
    public int getLWidthModelReference( ) 
    {
        return m_lWidthRef;
    }
    
    void setLWidthModelReference( int  lWidthRef ) 
    {
        m_lWidthRef = lWidthRef;
    }
    
    public int getGWidthModelReference( ) 
    {
        return m_gWidthRef;
    }
    
    public double getE_Offset( ) 
    {
        return m_e_offset;
    }

    public void setE_Offset( double m_e_offset ) 
    {
        this.m_e_offset = m_e_offset;
    }

    public double getE_Phase( ) 
    {
        return m_e_phase;
    }

    public void setE_Phase( double m_e_phase ) 
    {
        this.m_e_phase = m_e_phase;
    }

    public double getE_Lwidth( ) 
    {
        return m_e_lwidth;
    }

    public void setE_Lwidth( double m_e_lwidth ) 
    {
        this.m_e_lwidth = m_e_lwidth;
    }

    public double getE_Gwidth( ) 
    {
        return m_e_gwidth;
    }

    public void setE_Gwidth( double m_e_gwidth ) 
    {
        this.m_e_gwidth = m_e_gwidth;
    }
    
    /**
     * Returns the name of this component.
     * @return 
     */
    public String getComponentName( )
    {
        return m_componentName;
    }
    
    // Attributes
    int    m_number       = 0;
    String m_elementType  = "not_defined";
    
    // Attributes
    double m_amplitude      = 0.0;
    double m_offset         = 0.0;
    double m_phase          = 0.0;
    double m_lwidth         = 0.0;
    double m_gwidth         = 0.0;
        
    // And associated errors
    double m_e_amplitude    = 0.0;
    double m_e_offset       = 0.0;
    double m_e_phase        = 0.0;
    double m_e_lwidth       = 0.0;
    double m_e_gwidth       = 0.0;

    // Prior knowledge references
    int m_amplitudeRef      = -1;
    int m_offsetRef         = -1;
    int m_gWidthRef         = -1;
    int m_lWidthRef         = -1;
    int m_phaseRef          = -1;
    String m_componentName  = "none";

    
}
