/*
 * TextFileWriter.java
 *
 * Created on 17. September 2003, 13:57
 */

package org.spectrim.tdfdfit;

import java.io.IOException;
import java.io.FileWriter;
import java.io.BufferedWriter;

import java.text.DecimalFormat;

/**
 * Dedicated class to create text files.
 * @author  hans
 */
public class TextFileWriter extends BufferedWriter 
{
    
    /** 
     * Creates a new instance of TextFileWriter 
     * @param fw
     */
    public TextFileWriter( FileWriter fw ) 
    {
        super( fw );
    }
    /**
     * Writes a string to a file followed by a new line
     * @param str
     * @throws java.io.IOException
     */
    public void writeStringLn( String str ) throws IOException
    {
        write( str );
        newLine( );
    }
    
    /**
     * Writes a string to a file followed by a new line
     * @param str
     * @throws java.io.IOException
     */
    public void writeString( String str ) throws IOException
    {
        write( str );
    }

    /**
     * Writes a float to an output file
     * @param floatVal
     * @throws java.io.IOException
     */
    public void writeFloat( float floatVal ) throws IOException
    {
        double d = (double)floatVal;
        writeDouble( d );
    }
    
    /**
     * Writes a float to an output file following by a new line character
     * @param floatVal
     * @throws java.io.IOException
     */
    public void writeFloatLn( float floatVal ) throws IOException
    {
        double d = (double)floatVal;
        writeDouble( d );
        newLine( );
    }

    
    /**
     * Writes a double to an output file
     * @param doubleVal
     * @throws java.io.IOException
     */
    public void writeDouble( double doubleVal ) throws IOException
    {
        if ( m_format != null )
        {
            String str = m_format.format( doubleVal );
            write( str );
        }
        else
        {
            Double d = new Double( doubleVal );
            write( d.toString( ) );
        }
    }

    /**
     * Writes a double to an output file
     * @param doubleVal
     * @throws java.io.IOException
     */
    public void writeDoubleLn( double doubleVal ) throws IOException
    {
        if ( m_format != null )
        {
            String str = m_format.format( doubleVal );
            write( str );
            newLine( );
        }
        else
        {
            Double d = new Double( doubleVal );
            write( d.toString( ) );
            newLine( );
        }
    }

    /**
     * Writes a double to an output file
     * @param intVal
     * @throws java.io.IOException
     */
    public void writeInteger( int intVal ) throws IOException
    {
        Integer i = new Integer( intVal );
        write( i.toString( ) );
    }

    /**
     * Writes a double to an output file
     * @param intVal
     * @throws java.io.IOException
     */
    public void writeIntegerLn( int intVal ) throws IOException
    {
        Integer i = new Integer( intVal );
        write( i.toString( ) );
        newLine( );
    }

    /**
     * Sets the number format string: applies to floats as well as
     * doubles written to this file
     * @param format
     */
    public void setNumberFormat( String format )
    {
        m_format = new DecimalFormat( format );
    }
    /**
     * Writes a tab character in the file
     * @throws java.io.IOException
     */
    public void writeTab( ) throws IOException
    {
        write( "\t" );
    }
    
    DecimalFormat m_format;
}
