/*
 * TwoSignalsWriter.java
 *
 * Created on August 10, 2005, 8:27 AM
 */

package org.spectrim.tdfdfit;

import java.io.RandomAccessFile;
import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;

import org.spectrim.lib.Dialogs;
import org.spectrim.lib.MessageBox;


/**
 *
 * @author  hans
 */
public class TwoSignalsWriter 
{
    
    /** Creates a new instance of TwoSignalsWriter */
    public TwoSignalsWriter() 
    {
    }
    

    /** 
     * Creates a new instance of TwoSignalWriter and writes the two signals to 
     * one file to the filesystem.
     * 
     * The signal is a 1D signal of floating points
     *
     */
    public TwoSignalsWriter( String dialogTitle, float[] signal_1, float[] signal_2, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( new Integer( signal_1.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                writer.writeBytes( new Integer( signal_2.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                for ( int i = 0; i < signal_1.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_1[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                for ( int i = 0; i < signal_2.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_2[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }

    
    /** 
     * Creates a new instance of TwoSignalWriter and writes the two signals and  
     * xaxis information to the filesystem.
     * 
     * The signal is a 1D signal of floating points
     *
     */
    public TwoSignalsWriter( String dialogTitle, String title, float[] xaxis, float[] signal_1, float[] signal_2, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( title );
                writer.writeBytes( "\r\n" );
                
                writer.writeBytes( new Integer( signal_1.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                writer.writeBytes( new Integer( signal_2.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                for ( int i = 0; i < signal_1.length; i++ )
                {
                    writer.writeBytes( ( new Float( xaxis[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }

                for ( int i = 0; i < signal_1.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_1[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                for ( int i = 0; i < signal_2.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_2[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }

   
    /**
     * Creates a new instance of SignalWriter and writes the signal to 
     * the filesystem.
     * 
     * The signal is a 1D signal of double type 
     *
     */
    public TwoSignalsWriter( String dialogTitle, double[] signal_1, double[] signal_2, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( new Integer( signal_1.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                writer.writeBytes( new Integer( signal_2.length ).toString( ) );
                writer.writeBytes( "\r\n" );
                
                for ( int i = 0; i < signal_1.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_1[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                for ( int i = 0; i < signal_2.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_2[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }

    /**
     * Creates a new instance of SignalWriter and writes the signal to 
     * the filesystem.
     * 
     * The signal is a 1D signal of double type 
     *
     */
    public TwoSignalsWriter( String dialogTitle, String title, double[] xaxis, double[] signal_1, double[] signal_2, String path, String fileName )
    {
        
        m_path = path;
        m_dialogTitle = dialogTitle;
        m_fileName = fileName;
        
        handleOutputFileNaming( );
        
        if ( m_file != null )
        {
            try
            {
                RandomAccessFile writer = new RandomAccessFile( m_file, "rw" );
                
                writer.writeBytes( title );
                writer.writeBytes( "\r\n" );
                
                writer.writeBytes( new Integer( signal_1.length ).toString( ) );
                writer.writeBytes( "\r\n" );

                writer.writeBytes( new Integer( signal_2.length ).toString( ) );
                writer.writeBytes( "\r\n" );

                for ( int i = 0; i < signal_1.length; i++ )
                {
                    writer.writeBytes( ( new Float( xaxis[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                for ( int i = 0; i < signal_1.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_1[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                
                for ( int i = 0; i < signal_2.length; i++ )
                {
                    writer.writeBytes( ( new Float( signal_2[i] ) ).toString( ) );
                    writer.writeBytes( "\r\n" );
                }
                writer.close( );
            }
            catch ( IOException ioe )
            {
                System.out.println( ioe.getMessage( ) ); 
            }
        }
    }
    
    /**
     * Handles the file naming and creation in the Filesystem
     */
    private void handleOutputFileNaming( )
    {
        JFileChooser fileChooser = new JFileChooser( );
        
        String defaultPath = fileChooser.getCurrentDirectory( ).getPath( );
        
        
        if ( ( m_path == null ) && ( m_fileName == null ) )
        {
            int result = fileChooser.showSaveDialog( null );
        
            switch ( result )
            {
                case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                    m_file = fileChooser.getSelectedFile( );
                                                    
                                                    break;
                case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                    break;
                case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                    break;
            }
        }
        
        if ( ( m_path != null ) && ( m_fileName == null ) )
        {    
            
            fileChooser = new JFileChooser( new File( m_path ) );
            
            if ( m_dialogTitle != null )
            {
                fileChooser.setDialogTitle( m_dialogTitle );
            }
            
            int result = fileChooser.showSaveDialog( null );
        
            switch ( result )
            {
                case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                    m_file = fileChooser.getSelectedFile( );
                                                    
                                                    break;
                case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                    break;
                case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                    break;
            }
        }

        if ( ( m_path == null ) && ( m_fileName != null ) )
        {
            File constructedFile = new File( defaultPath + System.getProperty( "file.separator" ),  m_fileName );
            
            try
            {
                constructedFile.createNewFile( );
            }
            catch ( IOException e )
            {
                // nothandled
            }
            
            fileChooser = new JFileChooser( constructedFile );
            
            fileChooser.setSelectedFile( constructedFile );
            
            if ( m_dialogTitle != null )
            {
                fileChooser.setDialogTitle( m_dialogTitle );
            }
            
            int result = fileChooser.showSaveDialog( null );
        
            switch ( result )
            {
                case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                    m_file = fileChooser.getSelectedFile( );
                                                    
                                                    break;
                case JFileChooser.CANCEL_OPTION:    // Cancel or close dialog was selected
                                                    if ( constructedFile.length( ) == 0L )
                                                    {
                                                        try
                                                        {
                                                            constructedFile.delete( );    
                                                        }
                                                        catch ( SecurityException se )
                                                        {
                                                            System.out.println( se.getMessage( ) );
                                                        }
                                                    }
                                                    break;
                case JFileChooser.ERROR_OPTION:     // the selected process did not terminate properly
                                                    break;
            }
        }
        
        if ( ( m_path != null ) && ( m_fileName != null ) )
        {
            m_file = new File( m_path + System.getProperty( "file.separator" ), m_fileName );
            
            if ( m_file.exists( ) == true )
            {
                m_file.delete( );
            }
            
            try
            {    
                m_file.createNewFile( );
            }
            catch ( IOException e )
            {
                Dialogs.messageBox( null, "Could not create: " + m_file.getName( ), "File Creation Error", MessageBox.MB_OK );
            }
        }
    }
    
    // Attributes
    private File m_file;
    private String m_path;
    private String m_fileName;
    private String m_dialogTitle;
    

}
