/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.tdfdfit.mmbaseline;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.spectrim.tdfdfit.MRLineModelParameters;
import org.spectrim.tdfdfit.MRSignalProperties;

/**
 * This class defines the macro molecular baseline.
 * Its function is to make it easy to add the general-baselines to a TDFDFit
 * model in a most easy and convenient way.
 * @author Johannes Slotboom <johannes.slotboom at insel.ch>
 */
public class GeneralBaseline 
{
    /**
     * This constructs a macromolecular baseline in a easy way. 
     * N.B. only valid for 1H-spectroscopy of the brain.
     * @param frame
     * @param stiffness
     */
    public GeneralBaseline( GeneralBaselineSettingsFrame frame, int stiffness )
    {
        m_modelStiffness       = stiffness;
        m_modelDefinitionFrame = frame;
    }
    
    public List generateRequiredGeneralBaselineType( )
    {
        List baselineList = null;
        if ( m_modelStiffness == GeneralBaseline.FREE_MODEL )
        {
            // retrieve all variable from the GeneralBaselineSettingsFrame
            
            float   minPPMOffset                    = m_modelDefinitionFrame.getMinimumPPMValue( );
            float   maxPPMOffset                    = m_modelDefinitionFrame.getMaximumPPMValue( );        
            int     numberOfBaselineComponents      = m_modelDefinitionFrame.getNumberOfBaselineComponents( );
            float   defaultAmplitudeValues          = m_modelDefinitionFrame.getDefaultStartingAmplitudes( );
            float   defaultPhaseValues              = m_modelDefinitionFrame.getDefaultStartingPhases( );
            float   defaultOffsetValues             = m_modelDefinitionFrame.getDefaultStartingOffsets( );
            float   defaultLorentzianWidthValues    = m_modelDefinitionFrame.getDefaultLorentzianWidths( );
            float   defaultGaussianWidthValues      = m_modelDefinitionFrame.getDefaultGaussianWidths( );
            boolean absoluteOffsetFixed             = m_modelDefinitionFrame.areOffsetsAbsolutelyFixed( );
            boolean absolutePhasesFixed             = m_modelDefinitionFrame.areFhasesAbsolutelyFixed( );
            boolean absoluteLorentzianWidthsFixed   = m_modelDefinitionFrame.areLorentzianWidthsAbsolutelyFixed( );
            boolean absoluteGaussianWidthsFixed     = m_modelDefinitionFrame.areGaussianWidthsAbsolutelyFixed( );
            
            baselineList = getGeneralBaseLineComponentList( minPPMOffset, maxPPMOffset, numberOfBaselineComponents,
                                                            defaultAmplitudeValues, defaultPhaseValues, defaultLorentzianWidthValues, defaultGaussianWidthValues,
                                                            absoluteOffsetFixed, absolutePhasesFixed,
                                                            absoluteLorentzianWidthsFixed, absoluteGaussianWidthsFixed );
        }
        
        if ( m_modelStiffness == GeneralBaseline.VERY_RIGID_MODEL )
        {
            float   minPPMOffset                    = m_modelDefinitionFrame.getMinimumPPMValue( );
            float   maxPPMOffset                    = m_modelDefinitionFrame.getMaximumPPMValue( );        
            int     numberOfBaselineComponents      = 10;
            float   defaultAmplitudeValues          = m_modelDefinitionFrame.getDefaultStartingAmplitudes( );
            float   defaultPhaseValues              = m_modelDefinitionFrame.getDefaultStartingPhases( );
            float   defaultOffsetValues             = m_modelDefinitionFrame.getDefaultStartingOffsets( );
            float   defaultLorentzianWidthValues    = 300;
            float   defaultGaussianWidthValues      = m_modelDefinitionFrame.getDefaultGaussianWidths( );
            boolean absoluteOffsetFixed             = m_modelDefinitionFrame.areOffsetsAbsolutelyFixed( );
            boolean absolutePhasesFixed             = m_modelDefinitionFrame.areFhasesAbsolutelyFixed( );
            boolean absoluteLorentzianWidthsFixed   = m_modelDefinitionFrame.areLorentzianWidthsAbsolutelyFixed( );
            boolean absoluteGaussianWidthsFixed     = m_modelDefinitionFrame.areGaussianWidthsAbsolutelyFixed( );
            
            baselineList = getGeneralBaseLineComponentList( minPPMOffset, maxPPMOffset, numberOfBaselineComponents,
                                                            defaultAmplitudeValues, defaultPhaseValues, defaultLorentzianWidthValues, defaultGaussianWidthValues,
                                                            absoluteOffsetFixed, absolutePhasesFixed,
                                                            absoluteLorentzianWidthsFixed, absoluteGaussianWidthsFixed );
        }

        if ( m_modelStiffness == GeneralBaseline.RIGID_MODEL )
        {
            float   minPPMOffset                    = m_modelDefinitionFrame.getMinimumPPMValue( );
            float   maxPPMOffset                    = m_modelDefinitionFrame.getMaximumPPMValue( );        
            int     numberOfBaselineComponents      = 20;
            float   defaultAmplitudeValues          = m_modelDefinitionFrame.getDefaultStartingAmplitudes( );
            float   defaultPhaseValues              = m_modelDefinitionFrame.getDefaultStartingPhases( );
            float   defaultOffsetValues             = m_modelDefinitionFrame.getDefaultStartingOffsets( );
            float   defaultLorentzianWidthValues    = 250;
            float   defaultGaussianWidthValues      = m_modelDefinitionFrame.getDefaultGaussianWidths( );
            boolean absoluteOffsetFixed             = m_modelDefinitionFrame.areOffsetsAbsolutelyFixed( );
            boolean absolutePhasesFixed             = m_modelDefinitionFrame.areFhasesAbsolutelyFixed( );
            boolean absoluteLorentzianWidthsFixed   = m_modelDefinitionFrame.areLorentzianWidthsAbsolutelyFixed( );
            boolean absoluteGaussianWidthsFixed     = m_modelDefinitionFrame.areGaussianWidthsAbsolutelyFixed( );
            
            baselineList = getGeneralBaseLineComponentList( minPPMOffset, maxPPMOffset, numberOfBaselineComponents,
                                                            defaultAmplitudeValues, defaultPhaseValues, defaultLorentzianWidthValues, defaultGaussianWidthValues,
                                                            absoluteOffsetFixed, absolutePhasesFixed,
                                                            absoluteLorentzianWidthsFixed, absoluteGaussianWidthsFixed );
        }
        
        
        if ( m_modelStiffness == GeneralBaseline.FLEXIBLE_MODEL )
        {
            float   minPPMOffset                    = m_modelDefinitionFrame.getMinimumPPMValue( );
            float   maxPPMOffset                    = m_modelDefinitionFrame.getMaximumPPMValue( );        
            int     numberOfBaselineComponents      = m_modelDefinitionFrame.getNumberOfBaselineComponents();
            float   defaultAmplitudeValues          = m_modelDefinitionFrame.getDefaultStartingAmplitudes( );
            float   defaultPhaseValues              = m_modelDefinitionFrame.getDefaultStartingPhases( );
            float   defaultOffsetValues             = m_modelDefinitionFrame.getDefaultStartingOffsets( );
            float   defaultLorentzianWidthValues    = 50;
            float   defaultGaussianWidthValues      = m_modelDefinitionFrame.getDefaultGaussianWidths( );
            boolean absoluteOffsetFixed             = m_modelDefinitionFrame.areOffsetsAbsolutelyFixed( );
            boolean absolutePhasesFixed             = m_modelDefinitionFrame.areFhasesAbsolutelyFixed( );
            boolean absoluteLorentzianWidthsFixed   = m_modelDefinitionFrame.areLorentzianWidthsAbsolutelyFixed( );
            boolean absoluteGaussianWidthsFixed     = m_modelDefinitionFrame.areGaussianWidthsAbsolutelyFixed( );
            
            baselineList = getGeneralBaseLineComponentList( minPPMOffset, maxPPMOffset, numberOfBaselineComponents,
                                                            defaultAmplitudeValues, defaultPhaseValues, defaultLorentzianWidthValues, defaultGaussianWidthValues,
                                                            absoluteOffsetFixed, absolutePhasesFixed,
                                                            absoluteLorentzianWidthsFixed, absoluteGaussianWidthsFixed );
        }
        
        if ( m_modelStiffness == GeneralBaseline.VERY_FLEXIBLE_MODEL )
        {
            float   minPPMOffset                    = m_modelDefinitionFrame.getMinimumPPMValue( );
            float   maxPPMOffset                    = m_modelDefinitionFrame.getMaximumPPMValue( );        
            int     numberOfBaselineComponents      = 20;
            float   defaultAmplitudeValues          = m_modelDefinitionFrame.getDefaultStartingAmplitudes( );
            float   defaultPhaseValues              = m_modelDefinitionFrame.getDefaultStartingPhases( );
            float   defaultOffsetValues             = m_modelDefinitionFrame.getDefaultStartingOffsets( );
            float   defaultLorentzianWidthValues    = 50;
            float   defaultGaussianWidthValues      = m_modelDefinitionFrame.getDefaultGaussianWidths( );
            boolean absoluteOffsetFixed             = m_modelDefinitionFrame.areOffsetsAbsolutelyFixed( );
            boolean absolutePhasesFixed             = m_modelDefinitionFrame.areFhasesAbsolutelyFixed( );
            boolean absoluteLorentzianWidthsFixed   = m_modelDefinitionFrame.areLorentzianWidthsAbsolutelyFixed( );
            boolean absoluteGaussianWidthsFixed     = m_modelDefinitionFrame.areGaussianWidthsAbsolutelyFixed( );
            
            baselineList = getGeneralBaseLineComponentList( minPPMOffset, maxPPMOffset, numberOfBaselineComponents,
                                                            defaultAmplitudeValues, defaultPhaseValues, defaultLorentzianWidthValues, defaultGaussianWidthValues,
                                                            absoluteOffsetFixed, absolutePhasesFixed,
                                                            absoluteLorentzianWidthsFixed, absoluteGaussianWidthsFixed );
        }
        
        return baselineList;
    }      
    
    /**
     * Returns .
     * @param minPPM
     * @param maxPPM
     * @param numberOfComponents
     * @param defaultAmp
     * @param defaultPhase
     * @param defaultLorentzianWidth
     * @param defaultGaussianWidth
     * @param absoluteOffsetFixed
     * @param absolutePhaseFixed
     * @param absoluteLorentzianWidthFixed
     * @param absoluteGaussianWidthFixed
     * @return 
     */
    public List getGeneralBaseLineComponentList( float minPPM, float maxPPM, int numberOfComponents,
                                                 float defaultAmp, float defaultPhase, float defaultLorentzianWidth, float defaultGaussianWidth,
                                                 boolean absoluteOffsetFixed, boolean absolutePhaseFixed,
                                                 boolean absoluteLorentzianWidthFixed, boolean absoluteGaussianWidthFixed )
    {
        ArrayList<MRLineModelParameters> adHocBaselineList = new ArrayList<>( );
        List adHocSyncedBaselineList = Collections.synchronizedList( adHocBaselineList );
        
        MRSignalProperties props = MRSignalProperties.getInstance( );
        // Adapt the internal offset to tdfdfit internal offset representation (Hz)
        float ppmRange =  props.getAcquistitionBandwidth( ) / ( props.getResonanceFrequency( ) / 1000000f );    
        
        
        for ( int i = 0; i < numberOfComponents; i++ )
        {
            
            float currPPM = minPPM + ( ( maxPPM - minPPM ) * i ) / numberOfComponents;
            
            float internalTDFDFitOffset = - ( props.getNumberOfSamplePoints( ) / ppmRange ) * currPPM + (  props.getNumberOfSamplePoints( ) / ppmRange ) * props.getPPMValueInCenterOfSpectrum( );
        
            MRLineModelParameters par = new MRLineModelParameters( );
            
            par.setName( "General_baseline_" + i );
            par.setArea( defaultAmp );
            par.setAreaReference( "Free" ); 
            par.setOffset( internalTDFDFitOffset ); // frequency set in ppm
            par.setLorentzianWidth( defaultLorentzianWidth );
            par.setPhase( defaultPhase );
            par.setGaussianWidth( defaultGaussianWidth );
            
            par.setMinimumArea( 0.0f );
            par.setMinimumOffset( currPPM - m_deltaFreqTolerance );
            par.setMinimumLorentzianWidth( defaultLorentzianWidth - 5.0f );
            par.setMinimumPhase( 0.0f - 180f );
            par.setMinimumGaussianWidth( 0.0f );
            
            par.setMaximumArea( 100000.0f );
            par.setMaximumOffset( currPPM + m_deltaFreqTolerance );
            par.setMaximumLorentzianWidth( defaultLorentzianWidth + 5.0f );
            par.setMaximumPhase( 0.0f + 180f );
            par.setMaximumGaussianWidth( 0.0f );
            
            par.setFileReferenceName( "none" );
            par.setFitWithArbitraryLineShapeFunction( false );
            par.setFitWithReferenceLineShape( false );
            
            // Offset
            if ( absoluteOffsetFixed == true )
            {
                par.setOffsetReference( "General_baseline_" + i );
            }
            else
            {
                if ( i > 0 )
                {
                    par.setOffsetReference( "General_baseline_" + 0 );
                }
                else
                {
                    par.setOffsetReference( "Free" );
                }
            }
            
            // Phase
            if ( absolutePhaseFixed == true )
            {
                par.setPhaseReference( "General_baseline_" + i );
            }
            else
            {
                if ( i > 0 )
                {
                    par.setPhaseReference( "General_baseline_" + 0 );
                }
                else
                {
                    par.setPhaseReference( "Free" );
                }
            }
            
            // Lorentzian width
            if ( absoluteLorentzianWidthFixed == true )
            {
                par.setLorentzianWidthReference( "General_baseline_" + i );
            }
            else
            {
                if ( i > 0 )
                {
                    par.setLorentzianWidthReference( "General_baseline_" + 0 );
                }
                else
                {
                    par.setLorentzianWidthReference( "Free" );
                }
            }
            
            // Gaussian width
            if ( absoluteGaussianWidthFixed == true )
            {
                par.setGaussianWidthReference( "General_baseline_" + i );
            }
            else
            {
                if ( i > 0 )
                {
                    par.setGaussianWidthReference( "General_baseline_" + 0 );
                }
                else
                {
                    par.setGaussianWidthReference( "Free" );
                }
            }
             
            adHocSyncedBaselineList.add( par );
        }
        
        return adHocSyncedBaselineList;
    }
    
    private final ArrayList<MRLineModelParameters> m_generalBaselineList = new ArrayList<>( );       
    private final List m_mmSyncedBaselineList = Collections.synchronizedList( m_generalBaselineList );
    
    private final GeneralBaselineSettingsFrame m_modelDefinitionFrame;
    
    private final int m_modelStiffness;
    
    private final float m_deltaFreqTolerance = 0.05f;
    
    public final static int VERY_RIGID_MODEL    = 0;
    public final static int RIGID_MODEL         = 1;
    public final static int FLEXIBLE_MODEL      = 2;
    public final static int VERY_FLEXIBLE_MODEL = 3;
    public final static int FREE_MODEL          = 4;
    
}
