/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.quantitativeresults;

import javax.swing.table.DefaultTableModel;

/**
 * Class defining a table model for the normal value databases
 * that it handles: NormalDataBaseSelectionTableModel.
 * @author slotboom
 */
public class NormalDataBaseSelectionTableModel extends DefaultTableModel
{
    /**
     * Returns the number of columns.
     * @return 
     */
    @Override
    public int getColumnCount( )
    {
        return m_numberOfColumns;
    }

    /**
     * Returns the number of rows.
     * @return 
     */
    @Override
    public int getRowCount( )
    {
        return m_numberOfRows;
    }

    /**
     * Returns the column name.
     * @param col
     * @return 
     */
    @Override
    public String getColumnName( int col )
    {
        return m_columnNames[col];
    }

    /**
     * Returns the value of a cell at (col,row).
     * @param row
     * @param col
     * @return 
     */
    @Override
    public Object getValueAt( int row, int col )
    {
        return m_data[row][col];
    }

    /**
     * JTable uses this method to determine the default renderer/
     * editor for each cell.  If we didn't implement this method,
     * then the last column would contain text ("true"/"false"),
     * rather than a check box.
     * @param c
     * @return 
     */
    @Override
    public Class getColumnClass( int c )
    {
        return getValueAt( 0, c ).getClass( );
    }

    /**
     * Returns whether a cell can be edited or not.
     * @param row
     * @param col
     * @return 
     */
    @Override
    public boolean isCellEditable( int row, int col )
    {
        return false;
    }

    /**
     * Sets the value of a cell at (row,col)
     * @param value
     * @param row
     * @param col
     */
    @Override
    public void setValueAt( Object value, int row, int col )
    {
        m_data[row][col] = value;
        fireTableCellUpdated( row, col );
    }

    /**
     * Clears the data in the table and forces an update.
     */
    public void clearTable( )
    {
        m_data = new Object[0][0];
        fireTableDataChanged( );
    }

    /**
     * Set data vector size.
     * @param rows
     * @param columns
     */
    public void setDataVectorSize( int rows, int columns )
    {
        m_numberOfRows = rows;
        m_numberOfColumns = columns;

        m_data = new Object[rows][columns];
    }

    /**
     * Set the data vector.
     * @param rows
     * @param columns
     * @param data
     */
    public void setDataVector( int rows, int columns, Object[][] data )
    {
        m_numberOfRows = rows;
        m_numberOfColumns = columns;

        m_data = new Object[rows][columns];

        for ( int i = 0; i < getRowCount( ); i++ )
        {
            for ( int j = 0; j < getColumnCount( ); j++ )
            {
                m_data[i][j] = data[i][j];
            }
        }
        fireTableDataChanged( );
    }

    @Override
    public void removeRow( int row )
    {
        Object[][] m_data_new = new Object[m_numberOfRows - 1][m_numberOfColumns];
        int intCounter = -1;

        for ( int i = 0; i < getRowCount( ); i++ )
        {
            if( i == row )
            {
                continue;
            }
            else
            {
                intCounter++;
            }
            for ( int j = 0; j < getColumnCount( ); j++ )
            {
                m_data_new[intCounter][j] = m_data[i][j];
            }
        }

        m_data = null;
        m_data = m_data_new;

        m_numberOfRows--;

        fireTableDataChanged( );

    }

    /**
     * Returns the array of column names.
     * @return 
     */
    public String[] getColumnNames( )
    {
        return m_columnNames;
    }

    
    // Attributes of the class
    private final String[] m_columnNames = { "Database name",
                                            "Quantification method",
                                            "Manufacturer",
                                            "Field strength",
                                            "Scanner type",
                                            "Sequence type",
                                            "Tissue type",
                                            "Echo time [TE]",
                                            "Repetition time [TR]",
                                            "Scanner manufacturer" };

    private Object[][] m_data = null;

    int m_numberOfRows      = 0;
    int m_numberOfColumns   = 9;


    private static final long serialVersionUID = 6482334568825123L;
}
