/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.spectrim.tdfdfit.waterreferencemodelling;

import org.spectrim.lib.ColumnSorter;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Vector;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import org.dcm4che2.data.DicomObject;
import org.spectrim.dicom.DicomImageData;
import org.spectrim.dicom.FileInfo;
import org.spectrim.io.dicom.DicomHandler;
import org.spectrim.io.dicom.DicomHeader;
import org.spectrim.io.dicom.DicomHeaderLine;

/**
 * TwoCompartmentWaterReferenceDicomFileSelectionTable enables user to define
 * manually the water reference signals to be used in the quantification.
 * @author slotboom
 */
public class TwoCompartmentWaterReferenceDicomFileSelectionTable extends JTable implements  ActionListener, MouseListener
{
    
    
    public  TwoCompartmentWaterReferenceDicomFileSelectionTable( )
    {
        super( );
    
        Object[][] cellData = { { "", "", "", "", "", "", "" } };
        String[] columnNames = { "<HTML><B>File name</B></HTML>",
                                 "<HTML><B>Type</B></HTML>",
                                 "<HTML><B>Instance number</B></HTML>",
                                 "<HTML><B>Slice Position [mm]</B></HTML>",
                                 "<HTML><B>FOV [mm]x[mm]</B></HTML>",
                                 "<HTML><B>TE [ms]</B></HTML>",
                                 "<HTML><B>TR [ms]</B></HTML>"
                               };
        
        m_tableModel = new DefaultTableModel( );
        
        m_tableModel.addColumn( "<HTML><B>File name</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Type</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Instance number</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>Slice Position [mm]</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>FOV [mm]x[mm]</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>TE [ms]</B></HTML>" );
        m_tableModel.addColumn( "<HTML><B>TR [ms]</B></HTML>" );
        
        setModel( m_tableModel );
        
        // define default behavior of the JTable
        setAutoCreateColumnsFromModel( false );
        setColumnSelectionAllowed (false );
        setRowSelectionAllowed( true );
        
        constructObject( );
        
        // Initialize the toggle functions
        for ( int i = 0 ; i < 7; i++ )
        {
            m_ascending[i] = true;
        }
        
        
    }
    
    /**
     * Does the rest of the object construction work.
     */
    private void constructObject( )
    {
        m_popup = new JPopupMenu( );
        
        m_selectAllItems = new JMenuItem( "Select all items" );
        m_selectAllItems.addActionListener( this );
        
        m_deleteSelectedItems = new JMenuItem( "Delete selected items" );
        m_deleteSelectedItems.addActionListener( this );
        
        m_sortByFileName = new JMenuItem( "Sort by filename" );
        m_sortByFileName.addActionListener( this );
        
        m_sortByFileType = new JMenuItem( "Sort by file type" );
        m_sortByFileType.addActionListener( this );

        m_sortByInstanceNumber = new JMenuItem( "Sort by instance number" );
        m_sortByInstanceNumber.addActionListener( this );
        
        m_sortBySlicePosition = new JMenuItem( "Sort by slice position" );
        m_sortBySlicePosition.addActionListener( this );
        
        m_sortByFOVSize = new JMenuItem( "Sort by FOV size" );
        m_sortByFOVSize.addActionListener( this );
        
        m_sortByTE = new JMenuItem( "Sort by echo time TE" );
        m_sortByTE.addActionListener( this );
        
        m_sortByTR = new JMenuItem( "Sort by repetition time TR" );
        m_sortByTR.addActionListener( this );
        
        m_popup.add( m_selectAllItems );
        m_popup.add( m_deleteSelectedItems );
        
        m_popup.addSeparator( );
        
        m_popup.add( m_sortByFileName );
        m_popup.add( m_sortByFileType );
        m_popup.add( m_sortByInstanceNumber );
        m_popup.add( m_sortBySlicePosition );
        m_popup.add( m_sortByFOVSize );
        m_popup.add( m_sortByTE );
        m_popup.add( m_sortByTR );
        
        add( m_popup );
        
        addMouseListener( this );
        
    }
    
    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        
        Object sender = e.getSource( );
        
        if ( sender == m_selectAllItems )
        {
            this.selectAll( );
        }
        
        if ( sender == m_sortByFileName )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 0, m_ascending[0] );
            m_ascending[0] = !m_ascending[0];
        }
        
        if ( sender == m_sortByFileType )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 1, m_ascending[1] );
            m_ascending[1] = !m_ascending[1];
        }
        
        if ( sender == m_sortByInstanceNumber )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 2, m_ascending[2] );
            m_ascending[2] = !m_ascending[2];
        }
        
        if ( sender == m_sortBySlicePosition )
        {
            // since slice position in a numeric float value, alfa numeric
            // sorting does not work
            numericSortAllRowsBySlicePosition( );
            
        }
        
        if ( sender == m_sortByFOVSize )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 4, m_ascending[4] );
            m_ascending[4] = !m_ascending[4];
        }
        
        if ( sender ==  m_sortByTE )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 5, m_ascending[5] );
            m_ascending[5] = !m_ascending[5];
        }
        
        if ( sender == m_sortByTR )
        {
            sortAllRowsBy( (DefaultTableModel)this.getModel( ), 6, m_ascending[6] );
            m_ascending[6] = !m_ascending[6];
        }
        
        
        if ( sender == m_deleteSelectedItems )
        {
            deleteSelectedFileReferences( );
        }
        
    }
    
    /**
     * Method that sorts the list after image position.
     */
    private void numericSortAllRowsBySlicePosition( )
    {
        int numberOfColumns = m_tableModel.getColumnCount( );
        int numberOfRows = m_tableModel.getRowCount( );
        
        Object[][] inputData = new Object[numberOfRows][numberOfColumns];
        Object[][] sortedData = new Object[numberOfRows][numberOfColumns];
        
        float[] inputSlicePositions = new float[ numberOfRows ];
        float[] orderedSlicePositions = new float[ numberOfRows ];
        
        int[] order = new int[numberOfRows];
        
        for ( int i = 0; i < numberOfColumns; i++ )
        {
            for ( int j = 0; j < numberOfRows; j++ )
            {
                inputData[j][i] = m_tableModel.getValueAt( j, i );
            }
        }
        
        for ( int j = 0; j < numberOfRows; j++ )
        {
            inputSlicePositions[j] = new Float( m_tableModel.getValueAt( j, 3 ).toString( ) ).floatValue( );
        }
        
        orderedSlicePositions = org.spectrim.imageprocessing.QuickSort.quickSort( order, inputSlicePositions );
        
        for ( int i = 0; i < numberOfRows; i++ )
        { 
            sortedData[i][0] = inputData[order[i]][0];
            sortedData[i][1] = inputData[order[i]][1];
            sortedData[i][2] = inputData[order[i]][2];
            sortedData[i][3] = inputData[order[i]][3];
            sortedData[i][4] = inputData[order[i]][4];
            sortedData[i][5] = inputData[order[i]][5];
            sortedData[i][6] = inputData[order[i]][6];
        }
        
        for ( int i = 0; i < numberOfColumns; i++ )
        {
            for ( int j = 0; j < numberOfRows; j++ )
            {
                m_tableModel.setValueAt( sortedData[j][i], j, i );
            }
        }
        
        m_tableModel.fireTableStructureChanged( );
        
    }
    
    
    /**
     * Deletes selected file references, note not the files itself.
     */
    private void deleteSelectedFileReferences( )
    {
        int[] selectedRows = this.getSelectedRows( );
        
        for ( int i = selectedRows.length -1 ; i > -1 ; i-- )
        {
            m_tableModel.removeRow( selectedRows[i] );
        }
        
        m_tableModel.fireTableStructureChanged( );   
    }
    
    
    /**
     *  Regardless of sort order (ascending or descending), null values always appear last.
     * @param model
     * @param colIndex
     * @param ascending
      */ 
    public void sortAllRowsBy( DefaultTableModel model, int colIndex, boolean ascending ) 
    {
        Vector data = model.getDataVector( );
        Collections.sort( data, new ColumnSorter( colIndex, ascending ) );
        model.fireTableStructureChanged( );
    }
    
    /**
     * Is called whenever a mouse button is pressed.
     * @param e
     */
    @Override
    public void mousePressed( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
    }
    
    /**
     * Is called whenever a mouse button is clicked.
     * @param e
     */
    @Override
    public void mouseClicked( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever a mouse pointer enters.
     * @param e
     */
    @Override
    public void mouseEntered( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever the mouse pointer exits.
     * @param e
     */
    @Override
    public void mouseExited( MouseEvent e ) 
    {
    }
    
    /**
     * Is called whenever a mouse button is released.
     * @param e
     */
    @Override
    public void mouseReleased( MouseEvent e ) 
    {
        if ( e.isPopupTrigger( ) )
        {
            m_popup.show( e.getComponent( ), e.getX( ), e.getY( ) );
        }
    }
    
    /**
     * Ad hoc added to get things running with Philips data.
     * @param header
     * @param tag
     * @return 
     */
    protected String readHeaderValue(DicomHeader header, String tag) {
        DicomHeaderLine headerLine;
        headerLine = (DicomHeaderLine) header.get(tag);
        if (headerLine != null) {
            String val = headerLine.Value.replace("\"", "");
            return val;
        } else {
            // System.out.println("No value found for tag: " + tag);
            return "0";
        }
    }
    
    
    /**
     * Sets the cell values of the list; Since it is a public method it can be 
     * called from a other object.
     * @param fileReferences
     */
    public void setListData( LinkedList<String> fileReferences ) 
    {
        LinkedHashMap map;

        m_fileSelectedSeriesFileReferences = fileReferences;
        
        m_cellData = new Object[ m_fileSelectedSeriesFileReferences.size( ) ][ 7 ];
                
        // reimplement the constructor code
        int counter = 0;
        Iterator it = fileReferences.iterator( );
        while ( it.hasNext( ) )
        {
            String fileName = (String)it.next( );
            DicomImageData data = new DicomImageData( fileName );
            FileInfo finfo = data.getHeader( );

            double TE=0.0;
            double TR=0.0;
            double sizeDim1=0.0;
            double sizeDim2=0.0;
            double sizeDim3=0.0;
            
            DicomObject spc_case = DicomHandler.getDicomObject( fileName );
            
            if ( spc_case != null )
            {
                DicomHeader header = DicomHandler.getHeader(spc_case);
                TE = Double.valueOf(readHeaderValue(header, "(0018,9114)->(0018,9082)"));
                TR = Double.valueOf(readHeaderValue(header, "(0018,9112)->(0018,0080)"));
                sizeDim1 = Double.valueOf(readHeaderValue(header, "(0018,9126)[0]->(0018,9104)"));
                sizeDim2 = Double.valueOf(readHeaderValue(header, "(0018,9126)[1]->(0018,9104)"));
                sizeDim3 = Double.valueOf(readHeaderValue(header, "(0018,9126)[2]->(0018,9104)"));
                
                finfo.setEchoTime( "" + TE );
                finfo.setTR( "" + TR );
                finfo.setSpectroscopicVoxelDimensions( "[" + sizeDim1 + "," + sizeDim2 + "," + sizeDim3 + "]");
            }
            
                  
            m_cellData[counter][0] = fileName; 
            if ( finfo != null )
            {
                if ( finfo.isImageData( ) == true )
                {
                    m_cellData[counter][1] = "Image";
                    m_isImageSeries = true;
                }
                else
                {
                    m_cellData[counter][1] = "Spectrum";
                    m_isImageSeries = false;
                    map = data.getSiemensSpecificDataFromFile( );
                }
            
                m_cellData[counter][2] = new Integer( finfo.getInstanceNumber( ).trim( ) );

                // some siemens reformatted data do not have their slice location
                // property set
                if ( ( finfo.getSliceLocation( ) != null ) &&
                     ( finfo.getSliceLocation( ).compareTo( "N.A." ) != 0 ) &&
                     ( finfo.getSliceLocation( ).compareTo( "" ) != 0 ) )
                {
                    m_cellData[counter][3] = new Double( finfo.getSliceLocation( ) ).toString( );
                }
                else
                {
                    m_cellData[counter][3] = new Double( finfo.getInstanceNumber( ).trim( ) ).toString( );
                }

                String fov;
                if ( finfo.isImageData( ) == true )
                {
                    if ( ( finfo.getPixelWidth( ).trim( ).compareTo( "N.A." ) == 0 ) ||
                       ( finfo.getPixelHeight( ).trim( ).compareTo( "N.A." ) == 0 ) ||
                       ( finfo.getPixelWidth( ).trim( ).compareTo( "" ) == 0 ) ||
                       ( finfo.getPixelHeight( ).trim( ).compareTo( "" ) == 0 )
                       )
                    {
                        fov = "N.A.";
                    }
                    else
                    {
                        fov = "" +     ( new Double( finfo.getPixelWidth( ).trim( ) ).doubleValue( ) *
                                        new Double( finfo.getImageWidth( ) ).doubleValue( ) );
                        fov += " x " + ( new Double( finfo.getPixelHeight( ).trim( ) ).doubleValue( ) *
                                     new Double( finfo.getImageHeight( ) ).doubleValue( ) );
                    }
                }
                else
                {
                    String voxelDimensions;
                    if ( finfo.getManufacturer( ).contains( "Philips" ) == false )
                    {
                        LinkedHashMap lst = data.getSiemensSpecificDataHeaderInfo( );
                        if ( !lst.isEmpty( ) )
                        {
                            voxelDimensions = "[ " +  lst.get( "sSpecPara.sVoI.dThickness" ) + ", ";
                            voxelDimensions += lst.get( "sSpecPara.sVoI.dPhaseFOV" ) + ", ";
                            voxelDimensions += lst.get( "sSpecPara.sVoI.dReadoutFOV" ) + " ]";
                        }
                        else
                        {
                            voxelDimensions = finfo.getSpectroscopicVoxelDimensions( );
                        }
                    }
                    else
                    {
                        voxelDimensions = "[ " + 
                                                  sizeDim1 + ", ";
                               voxelDimensions += sizeDim2 + ", ";
                               voxelDimensions += sizeDim3 + " ]";
                    }
                    
                    fov = voxelDimensions;
                }

                m_cellData[counter][4] = fov;
                if ( ( finfo.isImageData( ) == true ) &&  finfo.canDisplayImagePixelData( ) == true )
                {
                    if ( finfo.getEchoTime( ).trim( ).compareTo( "N.A." ) != 0 )
                    {
                        m_cellData[counter][5] = new Double( finfo.getEchoTime( ).trim( ) ).toString( );
                    }
                    else
                    {
                        // this is sometimes the case for dicomized screenshots
                        m_cellData[counter][5] = new String( "N.A." );
                    }
                    if ( finfo.getTR( ).compareTo( "N.A." ) != 0 )
                    {
                        m_cellData[counter][6] = new Double( finfo.getTR( ) ).toString( );
                    }
                    else
                    {
                        m_cellData[counter][6] = new String( "N.A." );
                    }
                }
                else
                {
                    m_cellData[counter][5] = new String( "N.A." );
                    m_cellData[counter][6] = new String( "N.A." );
                }
            
                if ( ( finfo.isImageData( ) == false ) && ( finfo.getImageModality( ).compareTo( "MR" ) == 0 ) )
                {
                    map = data.getSiemensSpecificDataHeaderInfo( );
                    double displayTE;
                    double displayTR;

                    if ( finfo.getManufacturer( ).contains( "Philips" ) == false )
                    {
                        // So assume it is Siemens 
                        if ( (String)map.get( "alTE[0]" ) == null )
                        {
                            displayTE = new Double( finfo.getEchoTime( ) );
                        }
                        else
                        {
                            displayTE = new Double( (String)map.get( "alTE[0]" ) ).doubleValue( )/1000.0;
                        }
                        
                        if ( (String)map.get( "alTE[0]" ) == null )
                        {
                            displayTR = new Double( finfo.getTR( ) );
                        }
                        else
                        {
                            displayTR = new Double( (String)map.get( "alTR[0]" ) ).doubleValue( )/1000.0;
                        }
                        
                    }
                    else
                    {
                        displayTE = new Double( finfo.getEchoTime( ) );
                        displayTR = new Double( finfo.getTR( ) );
                    }
                    
                    m_cellData[counter][5] = new Double( displayTE );
                    m_cellData[counter][6] = new Double( displayTR );
                }
                counter++;
            }
        }
        String[] columnNames = { "<HTML><B>File name</B></HTML>",
                                 "<HTML><B>Type</B></HTML>",
                                 "<HTML><B>Instance number</B></HTML>",
                                 "<HTML><B>Slice Position [mm]</B><HTML>",
                                 "<HTML><B>FOV [mm]x[mm]</B><HTML>",
                                 "<HTML><B>TE [ms]</B><HTML>",
                                 "<HTML><B>TR [ms]</B><HTML>"
                                };
                                 
        m_tableModel = new DefaultTableModel( m_cellData, columnNames );
        
        setModel( m_tableModel );
        
        numericSortAllRowsBySlicePosition( );

        //sortAllRowsBy( (DefaultTableModel)this.getModel( ), 2, m_ascending[2] );
        //m_ascending[2] = !m_ascending[2];

        selectAll( );
        
        updateUI( );
    }
    
    /**
     * Returns whether the selected series contains image data or not.
     * @return 
     */
    public boolean isImageSeries( )
    {
        return m_isImageSeries;
    }
    
    /**
     * Returns the number of different slice positions in the selected series.
     * @return 
     */
    public int getNumberOfDifferentSlicePositions( )
    {
        if ( m_cellData != null  )
        {
            if (  m_cellData != null )
            {
                int numberOfDifferentSlicePositions = 0;
                double[] difSlicePos = new double[ m_tableModel.getRowCount( ) ];
        
                difSlicePos[0] = ( new Double( (String)m_cellData[0][3] ) ).doubleValue( );
        
                for ( int i = 1; i < m_fileSelectedSeriesFileReferences.size( ); i++ )
                {
                    boolean isDifferent = true;
                
                    for ( int j = 0; j < difSlicePos.length; j++ )
                    {
                        if ( ( new Double( (String)m_cellData[i][3] ) ).doubleValue( ) == difSlicePos[j] )
                        {
                            isDifferent = false;
                        }
                    }
                
                    if ( isDifferent == true )
                    {
                        numberOfDifferentSlicePositions++;
                        difSlicePos[numberOfDifferentSlicePositions] = ( new Double( (String)m_cellData[i][3] ) ).doubleValue( );
                    }
                }
                
                return numberOfDifferentSlicePositions + 1;
            }
        }
        return 0;
    }
    
     /**
     * Returns the file references of all selected files from the table.
     * @return
     */
    public LinkedList<String> getSelectedFileReferencesFromTable( )
    {
        m_fileSelectedSeriesFileReferences = new LinkedList<String>( );
        int[] selectedRows = this.getSelectedRows( );
        
        for ( int i = 0; i < selectedRows.length; i++ )
        {
            m_fileSelectedSeriesFileReferences.add( (String)getValueAt( selectedRows[i], 0 ) );
        }
        
        return m_fileSelectedSeriesFileReferences;
    }
    
    /**
     * Returns the linked list of strings that contain the file references 
     * in the selected series.
     * @return 
     */
    public LinkedList<String> getSelectedSeriesFileReferences( )
    {
        return m_fileSelectedSeriesFileReferences;
    }
    
    
    // Attributes
    private DefaultTableModel m_tableModel;
    
    private JPopupMenu m_popup;
    
    private JMenuItem m_selectAllItems;
    private JMenuItem m_deleteSelectedItems;
    private JMenuItem m_sortByFileName;
    private JMenuItem m_sortByFileType;
    private JMenuItem m_sortByInstanceNumber;
    private JMenuItem m_sortBySlicePosition;
    private JMenuItem m_sortByFOVSize;
    private JMenuItem m_sortByTE;
    private JMenuItem m_sortByTR;
    
    
    // needed for a toggle soring functions
    private final boolean[] m_ascending = new boolean[7];
    
    // Contains the treeset of images belonging to a series study 
    private LinkedList<String> m_fileSelectedSeriesFileReferences;
    
    // Object to store the list items of the image series data into
    Object[][] m_cellData;
    
    // variable indicating whether series contains image data or else
    boolean m_isImageSeries;
    
}
