/*
 * File: $Id: MRMeasuredSpectrumSelectionControl.java,v 1.1.1.1 2006/04/19 09:38:49 hans Exp $
 * History
 * Date: 12.09.2003, 11:14  Authors: Hans Slotboom
 *
 * Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */ 

package org.spectrim.tdfdfit.waterreferencemodelling;

import org.spectrim.dicom.DicomDefaults;
import org.spectrim.dicom.DicomDirectory;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import java.util.LinkedList;
import java.util.Iterator;


import javax.swing.JFileChooser;
import javax.swing.JFrame;

/**
 * Allows the selection of a measured DICOM spectro water reference files.
 * @author  hans
 */
public class TwoCompartmentWaterReferenceModelFittingMeasuredSpectrumSelectionControl extends TwoCompartmentWaterReferenceDicomFileSelectionControl implements ActionListener
{
    
    /** 
     * Creates a new instance of 
     * TwoCompartmentWaterReferenceModelFittingMeasuredSpectrumSelectionControl 
     * @param parent
     */
    public TwoCompartmentWaterReferenceModelFittingMeasuredSpectrumSelectionControl( JFrame parent )
    {
        super( );

        m_parent = parent;
        m_popupMenu = new JPopupMenu( );
        
        m_mountDirectoryMenuItem = new JMenuItem( "Mount directory" );
        m_mountDirectoryMenuItem.addActionListener( this );
        m_mountDirectoryMenuItem.setActionCommand( "mount directory" );
        
        m_popupMenu.add( m_mountDirectoryMenuItem );
        
        m_unMountDirectoryMenuItem = new JMenuItem( "Unmount directory" );
        m_unMountDirectoryMenuItem.addActionListener( this );
        m_unMountDirectoryMenuItem.setActionCommand( "unmount directory" );
        m_unMountDirectoryMenuItem.setEnabled( false );
        
        m_popupMenu.add( m_unMountDirectoryMenuItem );
     
        m_importTwoCompartmentWaterReferencesMenuItem = new JMenuItem( "Load two component water model spectra" );
        m_importTwoCompartmentWaterReferencesMenuItem.addActionListener( this );
        m_importTwoCompartmentWaterReferencesMenuItem.setActionCommand( "Loads two component water model spectra" );
        m_importTwoCompartmentWaterReferencesMenuItem.setEnabled( false );
        
        m_popupMenu.add( m_importTwoCompartmentWaterReferencesMenuItem );

        m_importWaterReferenceMenuItem = new JMenuItem( "Load water reference spectrum" );
        m_importWaterReferenceMenuItem.addActionListener( this );
        m_importWaterReferenceMenuItem.setActionCommand( "Loads water reference spectrum to the program."  );
        m_importWaterReferenceMenuItem.setEnabled( false );
        
        m_popupMenu.add( m_importWaterReferenceMenuItem );
    }


    /**
     * Evokes the directory selection dialog.
     */
    private void setDicomFiles( )
    {
        m_selectedDicomDirectory = getDicomDirectory( );
        if ( m_selectedDicomDirectory.getPath( ).compareTo( "" ) !=  0 )
        {
            m_dicomDir = new DicomDirectory( m_parent, m_selectedDicomDirectory, this );
        }
    }

    /**
     * Returns the DICOM directory selected by the user.
     */
    private File getDicomDirectory( )
    {
        DicomDefaults fdef = DicomDefaults.getInstance( );

        JFileChooser fileChooser = new JFileChooser( fdef.getProperty( "Dicom mount path" ) + System.getProperty( "file.separator" ) );

        fileChooser.setFileSelectionMode( JFileChooser.DIRECTORIES_ONLY );

        int result = fileChooser.showOpenDialog( this );

        File selected_dir = new File( "" );

        switch ( result )
        {
            case JFileChooser.APPROVE_OPTION:   // Approve Open button was pressed
                                                selected_dir = fileChooser.getSelectedFile( );
                                                DicomDefaults def = DicomDefaults.getInstance( );
                                                def.setProperty( "Dicom rename mount path", selected_dir.getPath( ) );
                                                def.save( );
                                                if ( selected_dir != null )
                                                {
                                                    m_dicomDirectorySet = true;
                                                }

                                                break;
            case JFileChooser.CANCEL_OPTION:     // Cancel or close dialog was selected
                                                break;
            case JFileChooser.ERROR_OPTION:      // the selected process did not terminate properly
                                                break;
        }

        return selected_dir;
    }


    /**
     * Is called whenever a registered component raises an ActionEvent.
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
       
        Object sender = e.getSource( );
        
        if ( sender == m_mountDirectoryMenuItem )
        {
            //System.out.println( "AutoTwoCompartmentFittingMeasuredSpectrumSelectionControl:: actionPerformed::mountDirectoryMenuItem" );
            setDicomFiles( );

            m_mountDirectoryMenuItem.setEnabled( false );
            m_unMountDirectoryMenuItem.setEnabled( true );
            m_directoryIsMounted = true;
        }
        
        if ( sender == m_unMountDirectoryMenuItem )
        {
            //System.out.println( "AutoTwoCompartmentFittingMeasuredSpectrumSelectionControl:: actionPerformed::unMountDirectoryMenuItem" );
            clearDicomDir( );
            m_directoryIsMounted = false;     
          
            m_importTwoCompartmentWaterReferencesMenuItem.setEnabled( false );
            m_mountDirectoryMenuItem.setEnabled( true );
            m_unMountDirectoryMenuItem.setEnabled( false );
        }
        
        if ( sender == m_importTwoCompartmentWaterReferencesMenuItem )
        {
            // System.out.println( "AutoTwoCompartmentFittingMeasuredSpectrumSelectionControl:: actionPerformed::IMPORT_MODEL_WATER_REFERENCE_FILE_EVENT" );
            notifyActionListeners( IMPORT_2COMP_MODEL_WATER_REFERENCE_FILES_EVENT );
        }

        if ( sender == m_importWaterReferenceMenuItem )
        {
            notifyActionListeners( IMPORT_WATER_REFERENCE_FILE_EVENT );
        }
        
        if ( sender == m_dicomDir )
        {
            setDicomDir( m_dicomDir );
        }
        
    }

    /**
     * Notifies actionEvent to all registered listeners.
     */
    private void notifyActionListeners( int eventType )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, "" ) );
        }
    }
    
    /**
     * Adds an action listener to the list of action listeners.
     * @param listener
     */
    @Override
    public void addActionListener( ActionListener listener )
    {
        super.addActionListener( listener );
        m_actionListeners.add( listener );
    }
    
    // Attributes
    
    private final JMenuItem m_importTwoCompartmentWaterReferencesMenuItem;
    private final JMenuItem m_importWaterReferenceMenuItem;

    private final JMenuItem m_mountDirectoryMenuItem;
    private final JMenuItem m_unMountDirectoryMenuItem;
    
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    
    
    public final static int MOUNT_DIRECTORY_EVENT = 2;
//    public final static int IMPORT_MODEL_SPECTRUM_FILE_EVENT = 3;
    public final static int IMPORT_2COMP_MODEL_WATER_REFERENCE_FILES_EVENT = 4;
    public final static int IMPORT_WATER_REFERENCE_FILE_EVENT = 10;

//    public final static int IMPORT_PATIENT_SPECTRUM_FILE_EVENT = 5;
//    public final static int IMPORT_PATIENT_WATER_REFERENCE_FILE_EVENT = 6;
//    public final static int COPY_AND_RENAME_MOUNTED_DIRECTORY_EVENT = 7;
//    public final static int CREATE_SYMBOLIC_LINKS_OF_MOUNTED_DIRECTORY_EVENT = 8;
    public final static int UNMOUNT_DIRECTORY_EVENT = 9;
//    public final static int UPLOAD_TO_SPECTRODATABASE_EVENT = 10;
    
    private final boolean m_dicomDirectoryToCopyToSet = false;
    private File m_selectedDicomDirectory;
    private DicomDirectory m_dicomDir;
    private final JFrame m_parent;

    private  boolean m_dicomDirectorySet ;

    // For proper serialization
    private static final long serialVersionUID = 6322325998125113L;
}
