/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import org.spectrim.imagegraphicobjects.Point3D;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import org.apache.commons.math3.linear.Array2DRowRealMatrix;
import org.apache.commons.math3.linear.ArrayRealVector;
import org.apache.commons.math3.linear.DecompositionSolver;
import org.apache.commons.math3.linear.QRDecomposition;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.spectrim.registration.AffineTransformation;
import org.spectrim.registration.DisplacementField;

/**
 *
 * @author nunopbarros
 */
public class ImageStack {

    private ArrayList<ImageSlice> images = new ArrayList<>();
    private double[] maxIntensity = new double[]{Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
    private double[] minIntensity = new double[]{Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    
    // Attributes to store pixel intensity values into
    private int maximumPixelValue = Integer.MIN_VALUE;
    private int minimumPixelValue = Integer.MAX_VALUE;
    
    private DisplacementField dispField;
    private AffineTransformation affTransf;
    private double spacingBetweenSlices;

    public void add(ImageSlice slice) {
        if (size() == 0 || slice.hasSameDimensions(images.get(images.size() - 1))) {
            images.add(slice);
            if (slice.getMaxVal()[0] > this.maxIntensity[0]) {
                maxIntensity[0] = slice.getMaxVal()[0];
            }
            if (slice.getMinVal()[0] < this.minIntensity[0]) {
                minIntensity[0] = slice.getMinVal()[0];
            }
            if (slice.getMaxVal()[1] > this.maxIntensity[1]) {
                maxIntensity[1] = slice.getMaxVal()[1];
            }
            if (slice.getMinVal()[1] < this.minIntensity[1]) {
                minIntensity[1] = slice.getMinVal()[1];
            }
            if (slice.getMaxVal()[2] > this.maxIntensity[2]) {
                maxIntensity[2] = slice.getMaxVal()[2];
            }
            if (slice.getMinVal()[2] < this.minIntensity[2]) {
                minIntensity[2] = slice.getMinVal()[2];
            }
        } else {
            System.out.println("Slice with different dimensions! Not added to Image Stack");
        }
        if (images.size() > 1) {
            spacingBetweenSlices = (images.get(1).imagePosition.subtractPoint(images.get(0).imagePosition)).length();
        }
    }

    public ArrayList<String> getFiles() {
        HashSet<String> hashSet = new HashSet<>();
        ArrayList<String> res = new ArrayList<>(hashSet);
        for (int i = 0; i < images.size(); i++) {
            if (!hashSet.contains(images.get(i).file)) {
                hashSet.add(images.get(i).file);
                res.add(images.get(i).file);
            }
        }
        return res;
    }

    public void addAll(Collection<ImageSlice> col) {
        Iterator<ImageSlice> iter = col.iterator();
        while (iter.hasNext()) {
            add(iter.next());
        }
    }

    public ImageSlice get(int nr) {
        return images.get(nr);
    }

    public ArrayList<ImageSlice> getImages() {
        return images;
    }

    public int[] getDimensions() {
        return new int[]{images.get(0).sizeX, images.get(0).sizeY, images.size()};
    }

    public void clear() {
        images.clear();
        maxIntensity = new double[]{Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
        minIntensity = new double[]{Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY};
    }

    public int size() {
        return images.size();
    }

    public int getMaximumPixelIntensityOfStackAsInt( ) 
    {
        if ( maximumPixelValue == Integer.MIN_VALUE ) 
        {
            for ( int i = 0; i < images.size( ); i++ ) 
            {
                
                int[] intIma = images.get( i ).getImageAsInt( );
                for ( int p = 0 ; p < intIma.length; p++  )
                
                
                if ( intIma[p] > maximumPixelValue ) 
                {
                    maximumPixelValue = intIma[p];
                }
            }
        }
        return maximumPixelValue;
    }

    public int getMinimumPixelIntensityOfStackAsInt( ) 
    {
        if ( minimumPixelValue == Integer.MAX_VALUE ) 
        {
            for ( int i = 0; i < images.size( ); i++ ) 
            {
                
                int[] intIma = images.get( i ).getImageAsInt( );
                for ( int p = 0 ; p < intIma.length; p++  )
                
                
                if ( intIma[p] < maximumPixelValue ) 
                {
                    minimumPixelValue = intIma[p];
                }
            }
        }
        return minimumPixelValue;
    }
    
    public double[] getMaxIntensity() {
//        if (minIntensity == Double.POSITIVE_INFINITY) {
//            for (int i = 0; i < images.size(); i++) {
//                if (images.get(i).getMaxVal() < minIntensity) {
//                    minIntensity = images.get(i).getMinVal();
//                };
//            }
//        }
        return minIntensity;
    }
    
    
    
    public double[] getMinIntensity() {
//        if (minIntensity == Double.POSITIVE_INFINITY) {
//            for (int i = 0; i < images.size(); i++) {
//                if (images.get(i).getMaxVal() < minIntensity) {
//                    minIntensity = images.get(i).getMinVal();
//                };
//            }
//        }
        return minIntensity;
    }

    public ImageStack getCheckboard(ImageStack ims2) {
        ImageStack res = new ImageStack();
        if (this.size() == ims2.size()) {
            for (int i = 0; i < ims2.size(); i++) {
                res.add(this.get(i).getCheckboard(ims2.get(i).image));
            }
        }
        return res;
    }

    public ImageStack getDifference(ImageStack ims2) {
        ImageStack res = new ImageStack();
        if (this.size() == ims2.size()) {
            for (int i = 0; i < ims2.size(); i++) {
                res.add(this.get(i).getDifference(ims2.get(i).image));
            }
        }
        return res;
    }

    public ImageStack getCheckboard(ArrayList<Raster> ims2) {
        ImageStack res = new ImageStack();
        if (this.size() == ims2.size()) {
            for (int i = 0; i < ims2.size(); i++) {
                res.add(this.get(i).getCheckboard(ims2.get(i)));
            }
        }
        return res;
    }

    public ImageStack getDifference(ArrayList<Raster> ims2) {
        ImageStack res = new ImageStack();
        if (this.size() == ims2.size()) {
            for (int i = 0; i < ims2.size(); i++) {
                res.add(this.get(i).getDifference(ims2.get(i)));
            }
        }
        return res;
    }

    public void setDisplacements(DisplacementField disp) {
        this.dispField = disp;
    }

    public DisplacementField getDisplacements() {
        return this.dispField;
    }

    public void setAffineTransform(AffineTransformation afft) {
        this.affTransf = afft;
    }

    public AffineTransformation getAffine() {
        return this.affTransf;
    }

    public double getSpacingBetweenSlices() {
        return spacingBetweenSlices;
    }

    public double[] getIntensitiesNearPoint(Point3D p) {
        Point3D imageOrientationX = this.images.get(0).imageOrientationX;
        Point3D imageOrientationY = this.images.get(0).imageOrientationY;
        double pixelSpacingX = this.images.get(0).pixelSpacingX;
        double pixelSpacingY = this.images.get(0).pixelSpacingY;
        int width = this.images.get(0).image.getWidth();
        int height = this.images.get(0).image.getHeight();
        double[] res = new double[5];
        Point3D imagePosition = this.images.get(0).imagePosition;
        Point3D v3 = Point3D.crossProduct(imageOrientationX, imageOrientationY);
        double[][] matrixData = {
            {imageOrientationX.m_x1, imageOrientationY.m_x1, v3.m_x1},
            {imageOrientationX.m_x2, imageOrientationY.m_x2, v3.m_x2},
            {imageOrientationX.m_x3, imageOrientationY.m_x3, v3.m_x3}};
        RealVector constants = new ArrayRealVector(new double[]{p.m_x1 - imagePosition.m_x1, p.m_x2 - imagePosition.m_x2, p.m_x3 - imagePosition.m_x3}, false);
        RealMatrix n = new Array2DRowRealMatrix(matrixData);
        DecompositionSolver solver = new QRDecomposition(n).getSolver();
        Point3D disp = null;
        try {
            RealVector solution = solver.solve(constants);
            res[0] = (int) Math.round(solution.getEntry(0) / pixelSpacingX);
            res[1] = (int) Math.round(solution.getEntry(1) / pixelSpacingY);
            res[2] = (int) Math.round(solution.getEntry(2) / spacingBetweenSlices);


            if (res[0] < width && res[0] >= 0 && res[1] < height && res[1] >= 0 && res[2] >= 0 && res[2] < this.images.size()) {
                ImageSlice slice = images.get((int) res[2]);
                double distance = (Point3D.vectorCombination(imageOrientationX, imageOrientationY, new Point3D(), res[0], res[1], 0).addPoint(slice.imagePosition)).subtractPoint(p).m_x3;
//                if (distance <= (slice.sliceThickness / 2)) {
                    res[3] = images.get((int) res[2]).image.getSample((int) res[0], (int) res[1], 0);
                    if (this.dispField != null) {
                        double[] disps = this.dispField.getDisplacement((int) res[0], (int) res[1], (int) res[2]);
                        res[4] = Math.sqrt(Math.pow(disps[0], 2) + Math.pow(disps[1], 2) + Math.pow(disps[2], 2));
                    }
//                } else {
//                    res[0] = -1;
//                    res[1] = -1;
//                    res[2] = -1;
//                }
            } else {
                res[0] = -1;
                res[1] = -1;
                res[2] = -1;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return res;
    }

    public HashMap<String, Integer> getPixelValuesForPointList(ArrayList<Point3D> points) {
        HashMap<String, Integer> vals = new HashMap<>();
        for (int i = 0; i < points.size(); i++) {
            double[] val = getIntensitiesNearPoint(points.get(i));
            if (val[0] != -1 && val[1] != -1 && val[2] != -1) {
                vals.put(((int) val[0]) + "_" + ((int) val[1]) + "_" + ((int) val[2]), ((int) val[3]));
            }
        }
        return vals;
    }

    public void getPixelValuesForPointList(ArrayList<Point3D> points, HashMap<String, Integer> vals) {
        for (int i = 0; i < points.size(); i++) {
            double[] val = getIntensitiesNearPoint(points.get(i));
            if (val[0] != -1 && val[1] != -1 && val[2] != -1) {
                vals.put(((int) val[0]) + "_" + ((int) val[1]) + "_" + ((int) val[2]), ((int) val[3]));
            }
        }
    }

    public void getPixelValuesForPointList(ArrayList<Point3D> points, HashMap<String, Integer> vals, HashMap<String, Double> disps) {
        for (int i = 0; i < points.size(); i++) {
            double[] val = getIntensitiesNearPoint(points.get(i));
            if (val[0] != -1 && val[1] != -1 && val[2] != -1) {
                vals.put(((int) val[0]) + "_" + ((int) val[1]) + "_" + ((int) val[2]), ((int) val[3]));
                disps.put(((int) val[0]) + "_" + ((int) val[1]) + "_" + ((int) val[2]), val[4]);
            }
        }
    }
}
