/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import com.pixelmed.dicom.DicomException;
import com.pixelmed.display.SourceImage;
import java.awt.image.Raster;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.ListIterator;
import org.dcm4che2.data.DicomObject;
import org.dcm4che2.io.DicomInputStream;

/**
 *
 * @author Michal Jablonski
 */
public class MultiFrameParameters {

    private String[] imageOrientation;
    private String[] imagePosition;
    private String[] pixelSpacing;
    private String[] sliceThickness;
    private String[] sopInstanceUID;

    //config
    private int numberOfFrames;
    private int rows;
    private int columns;
    private DicomObject object;
    private Raster[] images;
    File file;
    private int nrOrientations = 0;
    ArrayList<ArrayList<Integer>> orientationIndex = new ArrayList<ArrayList<Integer>>();

    public MultiFrameParameters(File dicomFile) throws IOException {

        this.file = dicomFile;
        DicomInputStream dis = new DicomInputStream(file);
        object = dis.readDicomObject();

        //get number of frames
        numberOfFrames = object.get(0x00280008).getInt(true);

        // rows and columns 
        rows = object.get(0x00280010).getInt(true);
        columns = object.get(0x00280011).getInt(true);

        //initialise
        sopInstanceUID = getValuesForAllFrames(0x52009230, 0x2005140F, 0x00080018); //ok
        imagePosition = getValuesForAllFrames(0x52009230, 0x00209113, 0x00200032); //ok
        imageOrientation = getValuesForAllFrames(0x52009230, 0x00209116, 0x00200037); //ok 
        pixelSpacing = getValuesForAllFrames(0x52009230, 0x2005140F, 0x00280030); //ok 
        sliceThickness = getValuesForAllFrames(0x52009230, 0x2005140F, 0x00180088); //ok
    }

    public int getInstanceNumber(int index) {
        return index;
    }

    private String[] getValuesForAllFrames(int tag1, int tag2, int tag3) {
        String[] current = new String[numberOfFrames];
        int path[] = {tag1, 0, tag2};
        for (int i = 0; i < numberOfFrames; i++) {
            path[1] = i;
            current[i] = "";
            String[] aux = object.get(path).getDicomObject().getStrings(tag3);

            if (aux.length == 1) {
                current[i] = aux[0];
            } else {
                for (int j = 0; j < aux.length; j++) {
                    current[i] += aux[j];
                    if (aux.length - j > 1) {
                        current[i] += "\\";
                    }
                }
            }
        }
        return current;
    }

    public int getNumberOfFrames() {
        return numberOfFrames;
    }

    public String getImageOrientationPatient(int index) {
        return imageOrientation[index];
    }

    public String getImagePosition(int index) {
        return imagePosition[index];
    }

    public int getRows() {
        return rows;
    }

    public int getColumns() {
        return columns;
    }

    public String getSliceThickness(int index) {
        return sliceThickness[index];
    }

    public String getPixelSpacing(int index) {
        return pixelSpacing[index];
    }

    public void getAllImages() throws IOException, DicomException {
        SourceImage image = new SourceImage(file.toString());
        images = new Raster[numberOfFrames];
        for (int i = 0; i < numberOfFrames; i++) {
            images[i] = image.getBufferedImage(i).getRaster();
        }
    }

    public Raster getImage(int index) {
        return images[index];
    }

    public int numberOfOrientations() {
        ArrayList<String> orientations = new ArrayList<String>();
        orientations.add(imageOrientation[0]);
        orientationIndex.clear();
        orientationIndex.add(new ArrayList<Integer>());
        orientationIndex.get(0).add(0);

        for (int i = 1; i < imageOrientation.length; i++) {
            ListIterator it = orientations.listIterator();
            int ind = 0;

            while (it.hasNext()) {
                String comp = it.next().toString();
                if (comp.equalsIgnoreCase(imageOrientation[i])) {
                    orientationIndex.get(ind).add(i);
                    ind++;
                    break;
                }
                else{
                    ind++; 
                }
                if (!it.hasNext()) {
                    ind = orientationIndex.size();
                    orientations.add(imageOrientation[i]);
                    orientationIndex.add(new ArrayList<Integer>());
                    orientationIndex.get(ind).add(i);
                    break;
                }
            }
        }
        nrOrientations = orientations.size();
        return nrOrientations;
    }

    public ArrayList<Integer> getOrientationIndex(int i) {
        return orientationIndex.get(i); 
    }
    public File getFile(){
        return file; 
    }
}
