/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.view;

import org.spectrim.colormap.ColorMap;
import org.spectrim.model.GlobalModel;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Rectangle2D;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartMouseEvent;
import org.jfree.chart.ChartMouseListener;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.Marker;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.ValueMarker;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.chart.title.TextTitle;
import org.jfree.chart.title.Title;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.HorizontalAlignment;
import org.jfree.ui.Layer;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.VerticalAlignment;
import org.spectrim.spectra.Spectra;

/**
 *
 * @author user
 */
public class SpectraPanel extends JPanel implements MouseListener, MouseMotionListener, KeyListener, FocusListener {

    private GlobalModel model;

    public enum DataType {

        ABS_BLUE,
        DISP_BLUE,
        MAG_BLUE,
        ABS_RED,
        DISP_RED,
        MAG_RED,
        ABS_BLUE_FIT,
        DISP_BLUE_FIT,
        MAG_BLUE_FIT,
        ABS_RED_FIT,
        DISP_RED_FIT,
        MAG_RED_FIT
    };

    private ArrayList<XYSeries> absorptionBlueData = new ArrayList<>();
    private ArrayList<XYSeries> dispersionBlueData = new ArrayList<>();
    private ArrayList<XYSeries> magBlueData = new ArrayList<>();
    private ArrayList<XYSeries> absorptionRedData = new ArrayList<>();
    private ArrayList<XYSeries> dispersionRedData = new ArrayList<>();
    private ArrayList<XYSeries> magRedData = new ArrayList<>();
    private ArrayList<XYSeries> absorptionBlueFitData = new ArrayList<>();
    private ArrayList<XYSeries> dispersionBlueFitData = new ArrayList<>();
    private ArrayList<XYSeries> magBlueFitData = new ArrayList<>();
    private ArrayList<XYSeries> absorptionRedFitData = new ArrayList<>();
    private ArrayList<XYSeries> dispersionRedFitData = new ArrayList<>();
    private ArrayList<XYSeries> magRedFitData = new ArrayList<>();

    private static Color absorptionBlueColor = new Color(0f, 84f / 255f, 210f / 255f, 1f);
    private static Color dispersionBlueColor = new Color(0f, 159f / 255f, 1f, 1f);
    private static Color magnitudeBlueColor = new Color(27f / 255f, 0f, 126f / 255f, 1f);
    private static Color absorptionRedColor = new Color(1f, 0f, 0f, 1f);
    private static Color dispersionRedColor = new Color(1f, 156f / 255f, 84f / 255f, 1f);
    private static Color magnitudeRedColor = new Color(128 / 255f, 0f, 0f, 1f);

    private static BasicStroke dashedStroke = new BasicStroke(1.0f,
            BasicStroke.CAP_SQUARE,
            BasicStroke.JOIN_MITER,
            10.0f,
            new float[]{10.0f, 5.0f},
            0.0f);
    private HashMap<String, XYSeries> modelLines = new HashMap<>();
    private int linecount = 0;
    private ChartPanel chartPanel;
    private JFreeChart chart;
    private double[] xAxis;
    private int[] pressedCoord;
    private boolean updatephase = false;
    private double initialphase;
    private XYSeriesCollection seriesCollection;
    private boolean phasingmode = false;
//  LIMIT VARIABLES  
    private boolean listeningLimitsSingle = false;
    private boolean listeningLimitsDouble = false;
    private int limitsSetAction;
    public static final int RANGES_1 = 1;
    public static final int ADD_LORENTZIAN = 2;
    public static final int REMOVE_PEAK = 3;
    public static final int RANGES_2 = 4;
    public static final int COLOR1 = 0;
    public static final int COLOR2 = 1;
    private double limitmin1 = -1;
    private double limitmin2 = -1;
    private double limitmax1 = -1;
    private double limitmax2 = -1;
    private TextTitle subtitle;
    private boolean mouseInteraction;

    public SpectraPanel() {
        initialize(true);
        this.setLayout(new java.awt.BorderLayout());
        this.add(chartPanel);
        mouseInteraction = true;
    }

    public SpectraPanel(boolean mouseInteraction) {
        initialize(mouseInteraction);
        this.mouseInteraction = mouseInteraction;
        this.setLayout(new java.awt.BorderLayout());
        this.add(chartPanel);
    }

    public void setModel(GlobalModel model) {
        this.model = model;
    }

//
    private void initialize(boolean mouseInteraction) {
        subtitle = new TextTitle("",
                new Font("Calibri", Font.PLAIN, 12), Color.BLACK,
                RectangleEdge.TOP, HorizontalAlignment.CENTER,
                VerticalAlignment.TOP, RectangleInsets.ZERO_INSETS);
        seriesCollection = new XYSeriesCollection();
        chart = ChartFactory.createXYLineChart("", "", "", seriesCollection, PlotOrientation.VERTICAL, false, false, false);
        chartPanel = new ChartPanel(chart);
        chartPanel.getChartRenderingInfo().setEntityCollection(null);
        chartPanel.addMouseListener(this);
        if (mouseInteraction) {
            chartPanel.addMouseMotionListener(this);
            chartPanel.addKeyListener(this);
            chartPanel.addFocusListener(this);
            chartPanel.addChartMouseListener(new ChartMouseListener() {
                @Override
                public void chartMouseClicked(final ChartMouseEvent event) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            if (xAxis != null) {
                                double clickX = chart.getXYPlot().getDomainCrosshairValue();
                                double clickY = chart.getXYPlot().getRangeCrosshairValue();
                                System.out.println("chart clicked near " + clickX + " " + clickY + " length " + xAxis.length + " max: " + xAxis[0] + " min: " + xAxis[xAxis.length - 1]);
                                if (listeningLimitsSingle || listeningLimitsDouble) {
//                            chart.getXYPlot().getDomainAxisCount();
                                    if (chart.getXYPlot().getDomainMarkers(Layer.FOREGROUND) == null) {
                                        addDomainMarker(clickX, COLOR1);
                                    } else if (chart.getXYPlot().getDomainMarkers(Layer.FOREGROUND).size() > 1) {
                                        addDomainMarker(clickX, COLOR2);
                                    } else {
                                        addDomainMarker(clickX, COLOR1);
                                    }
//                            int index = (int) Math.round((xAxis.length - 1) * ((clickX - xAxis[0]) / (xAxis[xAxis.length - 1] - xAxis[0])));

                                    if (chart.getXYPlot().getDomainMarkers(Layer.FOREGROUND).size() == 1) {
                                        limitmin1 = clickX;
                                    } else if (chart.getXYPlot().getDomainMarkers(Layer.FOREGROUND).size() == 2) {
                                        listeningLimitsSingle = false;
                                        if (clickX > limitmin1) {
                                            limitmax1 = clickX;
                                        } else {
                                            limitmax1 = limitmin1;
                                            limitmin1 = clickX;
                                        }
                                        if (model != null) {
                                            if (limitsSetAction == RANGES_1) {
                                                double min1 = limitmin1;
                                                double max1 = limitmax1;
                                                model.createColoredMap(model.getColorMap().getCurrentFunctionIndex(), new double[]{min1, max1});
                                                model.setColorMapUIRangesTxt(min1, max1);
                                            } else if (limitsSetAction == ADD_LORENTZIAN) {
//                                        model.fitLorentzian(limitmin1, limitmax1);
                                            }
                                        } else if (limitsSetAction == REMOVE_PEAK) {
                                        }
                                    } else if (chart.getXYPlot().getDomainMarkers(Layer.FOREGROUND).size() == 3) {
                                        limitmin2 = clickX;
                                    } else if (chart.getXYPlot().getDomainMarkers(Layer.FOREGROUND).size() == 4) {
                                        listeningLimitsDouble = false;
                                        if (clickX > limitmin2) {
                                            limitmax2 = clickX;
                                        } else {
                                            limitmax2 = limitmin2;
                                            limitmin2 = clickX;
                                        }
                                        if (model != null) {
                                            if (limitsSetAction == RANGES_2) {
                                                double min1 = limitmin1;
                                                double max1 = limitmax1;
                                                double min2 = limitmin2;
                                                double max2 = limitmax2;
                                                model.createColoredMap(model.getColorMap().getCurrentFunctionIndex(), new double[]{min1, max1, min2, max2});
                                                model.setColorMapUIRangesTxt(min1, max1, min2, max2);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    });
                }

                @Override
                public void chartMouseMoved(final ChartMouseEvent event) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
//                        Shape area = event.getEntity().getArea();
//                        double chartX = chart.getXYPlot().getDomainAxis().java2DToValue(event.getTrigger().getX(), area.getBounds2D(), chart.getXYPlot().getDomainAxisEdge());
//                        double chartY = chart.getXYPlot().getRangeAxis().java2DToValue(event.getTrigger().getY(), area.getBounds2D(), chart.getXYPlot().getRangeAxisEdge());
                        }
                    });
                }
            });
            XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
            renderer.setBaseShapesFilled(false);
            renderer.setDrawOutlines(false);
            renderer.setDrawSeriesLineAsPath(true);
            chart.getXYPlot().setRenderer(renderer);
        }
    }

    public double[] getZoom() {
        double xmin = chart.getXYPlot().getDomainAxis().getRange().getLowerBound();
        double xmax = chart.getXYPlot().getDomainAxis().getRange().getUpperBound();
        double ymin = chart.getXYPlot().getRangeAxis().getRange().getLowerBound();
        double ymax = chart.getXYPlot().getRangeAxis().getRange().getUpperBound();
        return new double[]{xmin, xmax, ymin, ymax};
    }

    public void setZoom(double[] range) {
        assert (range.length == 2 || range.length == 4);
        assert (range[1] > range[0]);
        if (range.length == 2) {
            chart.getXYPlot().getDomainAxis().setRange(range[0], range[1]);
            chart.getXYPlot().getRangeAxis().setAutoRange(true);
        } else if (range.length == 4) {
            assert (range[3] > range[2]);
            chart.getXYPlot().getDomainAxis().setRange(range[0], range[1]);
            chart.getXYPlot().getRangeAxis().setRange(range[2], range[3]);
        }
    }
    
    public void autoZoomVertical(){
        chart.getXYPlot().getRangeAxis().setAutoRange(true);
    }

    public double[] getYAxisRange() {
        return new double[]{(chart.getXYPlot().getRangeAxis().getLowerMargin() + 1.0) * chart.getXYPlot().getRangeAxis().getLowerBound(), (chart.getXYPlot().getRangeAxis().getUpperMargin() + 1.0) * chart.getXYPlot().getRangeAxis().getUpperBound()};
    }

    public void clearData() {
        for (DataType type : DataType.values()) {
            clearData(type);
        }
        //        setAxisVisible(false);
    }

    public void clearBlueData() {
        clearData(DataType.ABS_BLUE);
        clearData(DataType.DISP_BLUE);
        clearData(DataType.MAG_BLUE);
        clearData(DataType.ABS_BLUE_FIT);
        clearData(DataType.DISP_BLUE_FIT);
        clearData(DataType.MAG_BLUE_FIT);
        this.updateSeriesColor();
    }

    public void clearRedData() {
        clearData(DataType.ABS_RED);
        clearData(DataType.DISP_RED);
        clearData(DataType.MAG_RED);
        clearData(DataType.ABS_RED_FIT);
        clearData(DataType.DISP_RED_FIT);
        clearData(DataType.MAG_RED_FIT);
        this.updateSeriesColor();
    }

    public void clearAbsorptionData() {
        clearData(DataType.ABS_BLUE);
        clearData(DataType.ABS_RED);
        clearData(DataType.ABS_BLUE_FIT);
        clearData(DataType.ABS_RED_FIT);
        this.updateSeriesColor();
    }

    public void clearDispersionData() {
        clearData(DataType.DISP_BLUE);
        clearData(DataType.DISP_RED);
        clearData(DataType.DISP_BLUE_FIT);
        clearData(DataType.DISP_RED_FIT);
        this.updateSeriesColor();
    }

    public void clearMagnitudeData() {
        clearData(DataType.MAG_BLUE);
        clearData(DataType.MAG_RED);
        clearData(DataType.MAG_BLUE_FIT);
        clearData(DataType.MAG_RED_FIT);
        this.updateSeriesColor();
    }

    private Color getColor(DataType type) {
        switch (type) {
            case ABS_BLUE:
                return absorptionBlueColor;
            case DISP_BLUE:
                return dispersionBlueColor;
            case MAG_BLUE:
                return magnitudeBlueColor;
            case ABS_RED:
                return absorptionRedColor;
            case DISP_RED:
                return dispersionRedColor;
            case MAG_RED:
                return magnitudeRedColor;
            case ABS_BLUE_FIT:
                return absorptionBlueColor;
            case DISP_BLUE_FIT:
                return dispersionBlueColor;
            case MAG_BLUE_FIT:
                return magnitudeBlueColor;
            case ABS_RED_FIT:
                return absorptionRedColor;
            case DISP_RED_FIT:
                return dispersionRedColor;
            case MAG_RED_FIT:
                return magnitudeRedColor;
            default:
                return null;
        }
    }

    private ArrayList<XYSeries> getData(DataType type) {
        switch (type) {
            case ABS_BLUE:
                return absorptionBlueData;
            case DISP_BLUE:
                return dispersionBlueData;
            case MAG_BLUE:
                return magBlueData;
            case ABS_RED:
                return absorptionRedData;
            case DISP_RED:
                return dispersionRedData;
            case MAG_RED:
                return magRedData;
            case ABS_BLUE_FIT:
                return absorptionBlueFitData;
            case DISP_BLUE_FIT:
                return dispersionBlueFitData;
            case MAG_BLUE_FIT:
                return magBlueFitData;
            case ABS_RED_FIT:
                return absorptionRedFitData;
            case DISP_RED_FIT:
                return dispersionRedFitData;
            case MAG_RED_FIT:
                return magRedFitData;
            default:
                return null;
        }
    }

    private Stroke getStroke(DataType type) {
        switch (type) {
            case ABS_BLUE:
                return new BasicStroke();
            case DISP_BLUE:
                return new BasicStroke();
            case MAG_BLUE:
                return new BasicStroke();
            case ABS_RED:
                return new BasicStroke();
            case DISP_RED:
                return new BasicStroke();
            case MAG_RED:
                return new BasicStroke();
            case ABS_BLUE_FIT:
                return dashedStroke;
            case DISP_BLUE_FIT:
                return dashedStroke;
            case MAG_BLUE_FIT:
                return dashedStroke;
            case ABS_RED_FIT:
                return dashedStroke;
            case DISP_RED_FIT:
                return dashedStroke;
            case MAG_RED_FIT:
                return dashedStroke;
            default:
                return null;
        }
    }

    private void updateSeriesColor(DataType type) {
        XYItemRenderer renderer = chart.getXYPlot().getRenderer();
        renderer.setBaseOutlinePaint(Color.BLACK);
        ArrayList<XYSeries> data = getData(type);
        if (data != null) {
            for (int i = 0; i < data.size(); i++) {
                if (i == (data.size() - 1)) {
                    seriesCollection.setNotify(true);
                }
                if (seriesCollection.getSeries().contains(data.get(i))) {
                    renderer.setSeriesPaint(seriesCollection.indexOf(data.get(i)), getColor(type));
                    renderer.setSeriesStroke(seriesCollection.indexOf(data.get(i)), getStroke(type));
                }
            }
        }
    }

    public void updateSeriesColor() {
        for (DataType type : DataType.values()) {
            updateSeriesColor(type);
        }
    }

    public void clearData(DataType type) {
        ArrayList<XYSeries> data = getData(type);
        for (int i = 0; i < data.size(); i++) {
            if (seriesCollection.getSeries().contains(data.get(i))) {
                seriesCollection.removeSeries(data.get(i));
            }
        }
        data.clear();
//        updateSeriesColor();
    }

    public void setAxisVisible(boolean bool) {
        chart.getXYPlot().getRangeAxis().setVisible(bool);
        chart.getXYPlot().getDomainAxis().setVisible(bool);
    }

    public void setData(float[] newData, DataType type) {
        ArrayList<float[]> list = new ArrayList<>();
        list.add(newData);
        setData(list, type);
    }

    public void setData(ArrayList<float[]> newData, DataType type) {
        clearData(type);
        if (newData != null && !newData.isEmpty()) {
            for (int i = 0; i < newData.size(); i++) {
                XYSeries newSeries = new XYSeries(type.name() + i);
                for (int j = 0; j < newData.get(i).length; j++) {
                    if (i == (newData.size() - 1) && j == (newData.get(i).length - 1)) {
                        newSeries.add(xAxis[j], newData.get(i)[j], false);
                    } else {
                        newSeries.add(xAxis[j], newData.get(i)[j], true);
                    }
                }
                int ind = seriesCollection.getSeriesIndex(type.name() + i);
                if (ind >= 0) {
                    seriesCollection.removeSeries(ind);
                }
                seriesCollection.addSeries(newSeries);
                switch (type) {
                    case ABS_BLUE:
                        absorptionBlueData.add(newSeries);
                        break;
                    case DISP_BLUE:
                        dispersionBlueData.add(newSeries);
                        break;
                    case MAG_BLUE:
                        magBlueData.add(newSeries);
                        break;
                    case ABS_RED:
                        absorptionRedData.add(newSeries);
                        break;
                    case DISP_RED:
                        dispersionRedData.add(newSeries);
                        break;
                    case MAG_RED:
                        magRedData.add(newSeries);
                        break;
                    case ABS_BLUE_FIT:
                        absorptionBlueFitData.add(newSeries);
                        break;
                    case DISP_BLUE_FIT:
                        dispersionBlueFitData.add(newSeries);
                        break;
                    case MAG_BLUE_FIT:
                        magBlueFitData.add(newSeries);
                        break;
                    case ABS_RED_FIT:
                        absorptionRedFitData.add(newSeries);
                        break;
                    case DISP_RED_FIT:
                        dispersionRedFitData.add(newSeries);
                        break;
                    case MAG_RED_FIT:
                        magRedFitData.add(newSeries);
                }
            }
        }
    }

    public void setSeriesCollectionNotify(boolean bool) {
        seriesCollection.setNotify(bool);
    }

    public void setXAxis(double[] xAxis) {
        this.removeLines();
        this.xAxis = xAxis;
    }

    public void setxAxisText(String title) {
        this.chart.getXYPlot().getDomainAxis().setLabel(title);
    }

    public void setInvertedAxis(boolean invert) {
        chart.getXYPlot().getDomainAxis().setInverted(invert);
    }

    public void listenLimitsSingle(int action) {
        chart.getXYPlot().clearDomainMarkers();
        listeningLimitsSingle = true;
        this.limitsSetAction = action;
    }

    public void listenLimitsDouble(int action) {
        chart.getXYPlot().clearDomainMarkers();
        listeningLimitsDouble = true;
        this.limitsSetAction = action;
    }

    public void clearDomainMarkers() {
        chart.getXYPlot().clearDomainMarkers();
    }

    public void addDomainMarker(double pos, int color) {
        Marker markeraux = new ValueMarker(pos);
        markeraux.setStroke(new BasicStroke(1.5f));
        if (color == COLOR1) {
            markeraux.setPaint(new Color(57, 105, 138));
        } else if (color == COLOR2) {
            markeraux.setPaint(new Color(204, 117, 60));
        }
        chart.getXYPlot().addDomainMarker(markeraux);
    }

    public void restoreZoom() {
        this.chartPanel.restoreAutoBounds();
    }

//    public String addLine(float[] lpoints, String name) {
//        this.removeLines();
//        lorentz = lpoints;
//        XYSeries series = new XYSeries(name);
//        seriesCollection.addSeries(series);
//        linecount++;
//        modelLines.put("L" + linecount, series);
//        for (int i = 0; i < (this.xAxis.length - 1); i++) {
//            series.add(xAxis[i], lpoints[i], false);
//        }
//        series.add(xAxis[lpoints.length - 1], lpoints[lpoints.length - 1], true);
//        return "L" + linecount;
//    }
    public void updateLine(float[] lpoints, String id) {
        XYSeries series = modelLines.get(id);
        series.clear();
        for (int i = 0; i < (this.xAxis.length - 1); i++) {
            series.add(xAxis[i], lpoints[i], false);
        }
        series.add(xAxis[lpoints.length - 1], lpoints[lpoints.length - 1], true);
    }

    public void removeLine(String id) {
        seriesCollection.removeSeries(modelLines.get(id));
        modelLines.remove(id);
        linecount--;
    }

    public void removeLines() {
        Iterator iter = modelLines.keySet().iterator();
        String id;
        while (iter.hasNext()) {
            id = (String) iter.next();
            seriesCollection.removeSeries(modelLines.get(id));
            modelLines.remove(id);
            linecount--;
        }
    }

    @Override
    public void mouseDragged(MouseEvent me) {
        if (model != null && me.isControlDown()) {

            double phase0increment = (((float) me.getX() - (float) this.pressedCoord[0]) / 50);

            model.addPhase(new double[]{phase0increment, 0});
//            currentphase[1] = currentphase[1] + ((float) me.getY() - (float) this.pressedCoord[1]);
//            System.out.println("Phi0: " + currentphase[0] + "   Phi1: " + currentphase[1]);
            DecimalFormat numberFormat = new DecimalFormat("#0.000");
//            subtitle.setText("Zero order phase: " + numberFormat.format(currentphase[0]));
            subtitle.setText("Zero order phase: " + numberFormat.format(model.getMeanPhase()[0]) + " +- "
                    + numberFormat.format(model.getStdDevPhase()[0]));
            this.pressedCoord[0] = me.getX();
            this.pressedCoord[1] = me.getY();

            setData(model.getSpectrumData(Spectra.Mode.ABS, false, GlobalModel.SelectionGroup.BLUE), DataType.ABS_BLUE);
            setData(model.getSpectrumData(Spectra.Mode.ABS, false, GlobalModel.SelectionGroup.RED), DataType.ABS_RED);
            setData(model.getSpectrumData(Spectra.Mode.DISP, false, GlobalModel.SelectionGroup.BLUE), DataType.DISP_BLUE);
            setData(model.getSpectrumData(Spectra.Mode.DISP, false, GlobalModel.SelectionGroup.RED), DataType.DISP_RED);
            if (model.hasFittedSpectra()) {
                setData(model.getSpectrumData(Spectra.Mode.ABS, true, GlobalModel.SelectionGroup.BLUE), DataType.ABS_BLUE_FIT);
                setData(model.getSpectrumData(Spectra.Mode.ABS, true, GlobalModel.SelectionGroup.RED), DataType.ABS_RED_FIT);
                setData(model.getSpectrumData(Spectra.Mode.DISP, true, GlobalModel.SelectionGroup.BLUE), DataType.DISP_BLUE_FIT);
                setData(model.getSpectrumData(Spectra.Mode.DISP, true, GlobalModel.SelectionGroup.RED), DataType.DISP_RED_FIT);
            }
            updateSeriesColor();
        }

    }

    @Override
    public void mouseMoved(MouseEvent me) {
    }

    @Override
    public void keyTyped(KeyEvent ke) {
    }

    @Override
    public void keyPressed(KeyEvent ke) {
        if (model != null) {
            if (ke.getKeyCode() == 17 && !phasingmode && !ke.isAltDown()) {
                if (!model.isAbsorptionVisible()) {
                    setData(model.getSpectrumData(Spectra.Mode.ABS, false, GlobalModel.SelectionGroup.BLUE), DataType.ABS_BLUE);
                    setData(model.getSpectrumData(Spectra.Mode.ABS, false, GlobalModel.SelectionGroup.RED), DataType.ABS_RED);
                }
                if (!model.isDispersionVisible()) {
                    setData(model.getSpectrumData(Spectra.Mode.DISP, false, GlobalModel.SelectionGroup.BLUE), DataType.DISP_BLUE);
                    setData(model.getSpectrumData(Spectra.Mode.DISP, false, GlobalModel.SelectionGroup.RED), DataType.DISP_RED);
                }
                if (model.hasFittedSpectra()) {
                    if (!model.isAbsorptionVisible()) {
                        setData(model.getSpectrumData(Spectra.Mode.ABS, true, GlobalModel.SelectionGroup.BLUE), DataType.ABS_BLUE_FIT);
                        setData(model.getSpectrumData(Spectra.Mode.ABS, true, GlobalModel.SelectionGroup.RED), DataType.ABS_RED_FIT);
                    }
                    if (!model.isDispersionVisible()) {
                        setData(model.getSpectrumData(Spectra.Mode.DISP, true, GlobalModel.SelectionGroup.BLUE), DataType.DISP_BLUE_FIT);
                        setData(model.getSpectrumData(Spectra.Mode.DISP, true, GlobalModel.SelectionGroup.RED), DataType.DISP_RED_FIT);
                    }
                }
                DecimalFormat numberFormat = new DecimalFormat("#0.000");
                subtitle.setText("Zero order phase: " + numberFormat.format(model.getMeanPhase()[0]) + " +- "
                        + numberFormat.format(model.getStdDevPhase()[0]) + " (Drag mouse to change phase)");
                chart.addSubtitle(subtitle);
                updateSeriesColor();
                chartPanel.setMouseZoomable(false);
                phasingmode = true;
            }
        }

    }

    @Override
    public void keyReleased(KeyEvent ke) {
        if (model != null) {
            if (ke.getKeyCode() == 17) {
                chartPanel.setMouseZoomable(true);
                if (!model.isAbsorptionVisible()) {
                    clearData(DataType.ABS_BLUE);
                    clearData(DataType.ABS_BLUE_FIT);
                    clearData(DataType.ABS_RED);
                    clearData(DataType.ABS_RED_FIT);
                }
                if (!model.isDispersionVisible()) {
                    clearData(DataType.DISP_BLUE);
                    clearData(DataType.DISP_BLUE_FIT);
                    clearData(DataType.DISP_RED);
                    clearData(DataType.DISP_RED_FIT);
                }
                updateSeriesColor();
                chart.clearSubtitles();
                phasingmode = false;
            }
        }
    }

    @Override
    public void mouseClicked(MouseEvent me) {
    }

    @Override
    public void mousePressed(MouseEvent me) {
        chartPanel.grabFocus();
        this.pressedCoord = new int[]{me.getX(), me.getY()};
        if (model != null) {
//            initialphase = currentphase;
        }
    }

    @Override
    public void mouseReleased(MouseEvent me) {
        if (updatephase) {
//            model.setPhase(currentphase);
            updatephase = false;
        }
    }

    @Override
    public void mouseEntered(MouseEvent me) {
        if (!mouseInteraction) {
            setVisible(false);
            ((MouseListener) this.getParent()).mouseExited(me);
        }
    }

    @Override
    public void mouseExited(MouseEvent me) {
    }

    @Override
    public void focusGained(FocusEvent fe) {
        if (!mouseInteraction) {
            this.setVisible(false);
        }
    }

    @Override
    public void focusLost(FocusEvent fe) {
        chartPanel.setMouseZoomable(true);
        if (model != null) {
            if (!model.isAbsorptionVisible()) {
                clearData(DataType.ABS_BLUE);
                clearData(DataType.ABS_BLUE_FIT);
                clearData(DataType.ABS_RED);
                clearData(DataType.ABS_RED_FIT);
            }
            if (!model.isDispersionVisible()) {
                clearData(DataType.DISP_BLUE);
                clearData(DataType.DISP_BLUE_FIT);
                clearData(DataType.DISP_RED);
                clearData(DataType.DISP_RED_FIT);
            }
            chart.clearSubtitles();
            phasingmode = false;
        }
    }
}
