/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.spectrim.xyplot;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Iterator;
import java.util.LinkedList;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import org.spectrim.lib.Dialogs;
import org.spectrim.model.GlobalModel;

/**
 * This JFrame derived class enables easy viewing of grids of XYPlot objects.
 * It supports viewing of complex data and non complex data.
 * @author johan
 */
public class MultiXYPlotFrame extends JFrame implements ActionListener
{
    // Atrributes
    private MultiXYPlotView m_multiXYPlotView;
    private boolean m_frameResized = false;
    private LinkedList<ActionListener> m_actionListeners = new LinkedList<>( );
    public final static int    FRAME_RESIZED_EVENT_ID  = 0;
    public final static String FRAME_RESIZED_EVENT_CMD = "MULTIXYPLOTFRAME_FRAME_RESIZED_EVENT_CMD";
    private JMenuBar    m_XYPlotFrameMenuBar;
    private JMenu       m_viewMenu;
    private JMenuItem   m_printViewMenuItem;
    private JMenuItem   m_createScreenshotOfViewMenuItem;
    
    
    /**
     * Constructs a MultiXYPlotFrame object.
     * @param title
     * @param parent 
     * @param model 
     */
    public MultiXYPlotFrame( String title, JFrame parent, GlobalModel model )
    {
        // Set some default size for starters.
        setSize( 512, 512 );
        setMinimumSize( new Dimension( 512, 512 ) );
        
        m_XYPlotFrameMenuBar = new JMenuBar( ); 
        m_viewMenu = new JMenu( "View" );
        m_printViewMenuItem = new JMenuItem( "Print view .." );
        m_printViewMenuItem.addActionListener( this );
        m_viewMenu.add(  m_printViewMenuItem );
        m_createScreenshotOfViewMenuItem = new JMenuItem( "Create screenshot (png) .." );
        m_createScreenshotOfViewMenuItem.addActionListener( this );
        m_viewMenu.add(  m_createScreenshotOfViewMenuItem );
        
        m_XYPlotFrameMenuBar.add( m_viewMenu );
        setJMenuBar( m_XYPlotFrameMenuBar );
        
        
        m_multiXYPlotView = new MultiXYPlotView( this, model );
        addActionListener( m_multiXYPlotView );
        add( m_multiXYPlotView );
        
        getRootPane( ).addComponentListener( new ComponentAdapter( ) 
        {
            @Override
            public void componentResized( ComponentEvent e ) 
            {
                // This is only called when the user releases the mouse button.
                // System.out.println( "componentResized" );
                notifyActionListeners( FRAME_RESIZED_EVENT_ID, FRAME_RESIZED_EVENT_CMD );
            }
        });
        
        pack( );
        setVisible( true );
        
    }
    
    /**
     * Adds a new complex TD data set to be displayed to the list of spectra 
     * to be displayed.
     * @param spectrumID
     * @param absorptionChannelTD
     * @param dispersionChannelTD 
     */
    public void addNewTDDataset( String spectrumID, float[] absorptionChannelTD, float[] dispersionChannelTD )
    {
        
    }
    
    /**
     * Adds a new complex TD data set to be displayed to the list of spectra 
     * to be displayed.
     * @param spectrumID 
     * @param absorptionChannelFD 
     * @param dispersionChannelFD 
     */
    public void addNewFDDataset( String spectrumID, float[] absorptionChannelFD, float[] dispersionChannelFD )
    {
        
    }

  
    /**
     * Adds an ActionListener to the list of action listeners who are notified
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
        m_actionListeners.add( e );
    }
    
 
    /**
     * Notify listeners that something in the object has changed.
     */
    private void notifyActionListeners( int eventType, String eventCmd )
    {
        Iterator it = m_actionListeners.iterator( );
        while( it.hasNext( ) )
        {
            ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, eventType, eventCmd ) );
        }
    }

    @Override
    public void actionPerformed( ActionEvent e ) 
    {
        if ( e.getSource( ) == m_printViewMenuItem )
        {
            System.out.println( "The user wants to print the view!" );
            if ( System.getProperty( "os.name" ).toLowerCase( ).contains( "linux" ) == false )
            {
                m_multiXYPlotView.print( );
            }
        
            else
            {
                Dialogs.messageBox( this, "Printing under Linux is not supported!", "Printing under Linux" );
            }
        }
        
        if ( e.getSource( ) == m_createScreenshotOfViewMenuItem )
        {
            System.out.println( "The user wants to create a screenshot png image file of  the view!" );
            
            m_multiXYPlotView.createPNGFile( );
        } 
    }
}
