/*
 * File: $Id: XYPlotRange.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 20.05.2003, 09:38 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */

package org.spectrim.xyplot;
 
/**
 * This class handles image display information like scale and offset values
 * for horizontal as well as vertical display axis.
 *
 * @author  hans
 */

public class XYPlotRange 
{
    
    /** 
     * Creates a new instance of XYPlotRange. 
     */
    public XYPlotRange( ) 
    {
        m_minimumX = Float.MAX_VALUE;
        m_maximumX = -Float.MAX_VALUE;
        m_minimumY = Float.MAX_VALUE;
        m_maximumY = -Float.MAX_VALUE;
        
        m_currentMinimumX = Float.MAX_VALUE;
        m_currentMaximumX = -Float.MAX_VALUE;
        m_currentMinimumY = Float.MAX_VALUE;
        m_currentMaximumY = -Float.MAX_VALUE;
        
        m_currentXCenter = 0.0f;
        m_currentYCenter = 0.0f;
        m_currentXRangeFactor = 1.0f;
        m_currentYRangeFactor = 1.0f;
    }
        
    /** 
     * Creates a new instance of XYPlotRange with default plot range type.
     * @param x_min
     * @param x_max
     * @param y_min
     * @param y_max
     */
    public XYPlotRange( float x_min, float x_max, float y_min, float y_max ) 
    {
        m_minimumX = x_min;
        m_maximumX = x_max;
        m_minimumY = y_min;
        m_maximumY = y_max;
        
        m_currentMinimumX = m_minimumX;
        m_currentMaximumY = m_maximumY;
        m_currentMaximumX = m_maximumX;
        m_currentMinimumY = m_minimumY;
        
        m_currentXRange = Math.abs( m_maximumX - m_minimumX );
        m_currentYRange = Math.abs( m_maximumY - m_minimumY );
        
        m_currentXCenter = 0.5f * ( m_minimumX + m_maximumX );
        m_currentYCenter = 0.5f * ( m_minimumY + m_maximumY );
        m_currentXRangeFactor = m_currentXRange / ( m_maximumX - m_minimumX );
        
        // m_currentXRangeFactor = 1.0f;
        m_currentYRangeFactor = 1.0f;
    }
    
    /**
     * Resets the attributes to default values.
     */
    public void reset( )
    {
        m_minimumX = Float.MAX_VALUE;
        m_maximumX = -Float.MAX_VALUE;
        m_minimumY = Float.MAX_VALUE;
        m_maximumY = -Float.MAX_VALUE;
                
        m_currentMinimumX = m_minimumX;
        m_currentMinimumY = m_minimumY;
        m_currentMaximumX = m_maximumX;
        m_currentMaximumY = m_maximumY;
        
        m_currentXCenter = 0.0f;
        m_currentYCenter = 0.0f;
        if ( m_currentXRangeFactor < 0.0 )
        {
            m_currentXRangeFactor = -1.0f;
        }
        else
        {
            m_currentXRangeFactor = 1.0f;
        }
        m_currentYRangeFactor = 1.0f;
    }
    
    
    /**
     * Returns the minimum x-value attribute.
     * @return 
     */
    public float getMinimumX( )
    {
        return m_currentMinimumX;
    }

    /**
     * Sets the minimum x-value attribute.
     * @param minimumX
     */
    public void setMinimumX( float minimumX )
    {
        m_currentMinimumX = minimumX;
        m_minimumX = minimumX;
    }

    /**
     * Returns the maximum x-value attribute.
     * @return 
     */
    public float getMaximumX( )
    {
        return m_currentMaximumX;
    }

    /**
     * Sets the minimum x-value attribute.
     * @param maximumX
     */
    public void setMaximumX( float maximumX )
    {
        m_currentMaximumX = maximumX;
        m_maximumX = maximumX;
    }
    
    /**
     * Returns the minimum y-value attribute.
     * @return 
     */
    public float getMinimumY( )
    {
        return m_currentMinimumY;
    }

    /**
     * Sets the minimum y-value attribute.
     * @param minimumY
     */
    public void setMinimumY( float minimumY )
    {
        m_currentMinimumY = minimumY;
        m_minimumY = minimumY;
    }

    /**
     * Returns the maximum y-value attribute.
     * @return 
     */
    public float getMaximumY( )
    {
        return m_currentMaximumY;
    }

    /**
     * Sets the maximum y-value attribute.
     * @param maximumY
     */
    public void setMaximumY( float maximumY )
    {
        m_currentMaximumY = maximumY;
        m_maximumY = maximumY;
    }
    
    /**
     * Returns the m_xCenter attribute.
     * @return 
     */
    public float getXCenter( )
    {
        return m_currentXCenter;
    }

    /**
     * Sets the m_xCenter value attribute
     * @param xCenterFactor 
     *
     */
    public void setXCenter( float xCenterFactor )
    {
        m_currentXCenter = ( m_maximumX - m_minimumX ) * xCenterFactor / 100.0f;
        
        m_currentXRangeFactor = m_currentXRange / ( m_maximumX - m_minimumX );
        
        m_currentMinimumX = m_currentXCenter - 0.5f * m_currentXRangeFactor * Math.abs( m_maximumX - m_minimumX );
        m_currentMaximumX = m_currentXCenter + 0.5f * m_currentXRangeFactor * Math.abs( m_maximumX - m_minimumX );
    }
    
    /**
     * Returns the m_yCenter attribute.
     * @return 
     */
    public float getYCenter( )
    {
        return m_currentYCenter;
    }

    /**
     * Sets the m_yCenter value attribute.
     * @param yCenterFactor
     */
    public void setYCenter( float yCenterFactor )
    {
        m_currentYCenter = ( m_maximumY - m_minimumY ) * yCenterFactor / 100.0f;
        
        m_currentMinimumY = m_currentYCenter - 0.5f * m_currentYRangeFactor * ( m_maximumY - m_minimumY );
        m_currentMaximumY = m_currentYCenter + 0.5f * m_currentYRangeFactor * ( m_maximumY - m_minimumY );
    }    
    
    
    /**
     * Returns the m_xRange attribute.
     * @return 
     */
    public float getXRange( )
    {
        m_currentXRange = Math.abs( m_currentMaximumX - m_currentMinimumX );
        return Math.abs( m_currentMaximumX - m_currentMinimumX );
    }

    /**
     * Returns the m_yRange attribute.
     * @return 
     */
    public float getYRange( )
    {
        m_currentYRange = m_currentMaximumY - m_currentMinimumY;
        return m_currentMaximumY - m_currentMinimumY;
    }

    
    /**
     * Returns the m_currentXRangeFactor attribute.
     * @return 
     */
    public float getXRangeZoomFactor( )
    {
        return m_currentXRangeFactor;
    }
    
    /**
     * Returns the m_currentYRangeFactor attribute.
     * @return 
     */
    public float getYRangeZoomFactor( )
    {
        return m_currentYRangeFactor;
    }
    
    /**
     * Sets the m_xRange value attribute.
     * @param xRange
     */
    public void setXRange( float xRange )
    {
        m_currentXRange = xRange;
        
        m_currentXRangeFactor = xRange / ( m_maximumX - m_minimumX );
        
        m_currentMinimumX = m_currentXCenter - 0.5f * m_currentXRangeFactor * Math.abs( m_maximumX - m_minimumX );
        m_currentMaximumX = m_currentXCenter + 0.5f * m_currentXRangeFactor * Math.abs( m_maximumX - m_minimumX );
    }    
    
    /**
     * Sets the m_yRange value attribute.
     * @param yRange
     */
    public void setYRange( float yRange )
    {
        m_currentYRange = yRange;
        
        m_currentYRangeFactor = yRange / ( m_maximumY - m_minimumY );
        
        m_currentMinimumY = m_currentYCenter - 0.5f * m_currentYRangeFactor * ( m_maximumY - m_minimumY );
        m_currentMaximumY = m_currentYCenter + 0.5f * m_currentYRangeFactor * ( m_maximumY - m_minimumY );   
    }
    
    
    /**
     * Sets the x unit string.
     * @param xUnitString
     */
    public void setXUnitString( String xUnitString )
    {
        m_xUnitString = xUnitString;
    }

    /**
     * Sets the x unit string.
     * @param yUnitString
     */
    public void setYUnitString( String yUnitString )
    {
        m_yUnitString = yUnitString;
    }
    
    /**
     * Sets the display unit type.
     * @param displayType
     */
    public void setDisplayUnitType( int displayType )
    {
        
    }
    
    /**
     * Computes the index value at 'position' value
     * assuming m_minimumX at index position 0 and 
     * m_max position at position size.
     * @param value
     * @param size
     * @return 
     */
    public int computeIndexValue( float value, int size )
    {
        int val = Math.round( ( value - m_minimumX ) * (float)size / ( m_maximumX - m_minimumX ) );
        return val;
    }
    
     /**
     * Computes the index value at 'position' value
     * assuming m_minimumX at index position 0 and 
     * m_max position at position size.
     * @param value
     * @param size
     * @return 
     */
    public int computeIndexValueReversed( float value, int size )
    {
        int val = Math.abs( Math.round( ( m_maximumX - value ) * (float)size / ( m_maximumX - m_minimumX ) ) );
        return val;
    }
    
    
    // Basic attributes
    private float m_minimumX;
    private float m_minimumY;
    private float m_maximumX;
    private float m_maximumY;
    
    private float m_currentMinimumX;
    private float m_currentMinimumY;
    private float m_currentMaximumX;
    private float m_currentMaximumY;
    
    private float m_currentXCenter;
    private float m_currentYCenter;    
    private float m_currentXRange;
    private float m_currentYRange;    
    private float m_currentXRangeFactor;
    private float m_currentYRangeFactor;
    
    private String m_xUnitString = "[Hz]";
    private String m_yUnitString = "[a.u.]";
}
