/*
 * File: $Id: XYPlotRangeSelector.java,v 1.1.1.1 2006/04/19 09:38:51 hans Exp $
 * History
 * Date: 10.05.2003, 09:23 Hans Slotboom  Changes:
 *
 * Copyright (C) 2003 by
 * 
 * Inselspital - Department of Neuroradiology
 * CH 3010 Bern Switzerland
 */


package org.spectrim.xyplot;

import java.awt.Point;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.LinkedList;
import java.util.Iterator;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JButton;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.JLabel;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


/**
 * Allows the adjustment of the displayed horizontal and vertical display
 * ranges
 * @author  hans
 */

public class XYPlotRangeSelector extends JFrame implements ActionListener, ChangeListener 
{
    
    /**
     * Creates a new instance of XYPlotRangeSelector. 
     * @param title
     * @param parent
     */
    public XYPlotRangeSelector( String title, JFrame parent )
    {
        super( );
        m_parent = parent;
        setTitle( title );
        
        m_contentPanel = new JPanel( );
        m_contentPanel.setLayout( new BoxLayout( m_contentPanel, BoxLayout.Y_AXIS ) );

        instantiateHorizontalRangePanel( "[Hz]" );
        instantiateVerticalRangePanel( "[a.u.]" );
        
        instantiateCancelApplyPanel( );
                
        // add window listener necessary to handle the window closing 
        // event properly
        addWindowListener( 
            new WindowAdapter( )
            {
                @Override
                public void windowClosing( WindowEvent event )
                {
                   dispose( );
                }
            }
        );
        
        setContentPane( m_contentPanel );   
        
        pack( );
        // Put the Dialog nicely in the middle of its parent
        if ( m_parent != null )
        {
            Point parentLoc = m_parent.getLocation( );
            Point centerParent = new Point( m_parent.getX( ) + ( m_parent.getWidth( ) / 2 ) - this.getWidth( ) / 2,
                                            m_parent.getY( ) + ( m_parent.getHeight( ) / 2 ) - this.getHeight( ) / 2 );
            setLocation( centerParent );
        }
        setVisible( true );
    }
        
    /**
     * Instantiates an apply/ cancel panel.
     */
    private void instantiateCancelApplyPanel( )
    {
        // Selection panel definitions
        m_applyButton = new JButton( "Apply" );
        m_applyButton.addActionListener( this );
        m_applyButton.setToolTipText( "Applies the selected zerofilling factor to the currently loaded dataset" );
        m_cancelButton = new JButton( "Cancel" );
        m_cancelButton.addActionListener( this );
        m_cancelButton.setToolTipText( "Cancels this operation without applying the selected zerofilling factor" );
        
        m_selectionPanel = new JPanel( );
        
        m_selectionPanel.setLayout( new BoxLayout( m_selectionPanel, BoxLayout.X_AXIS ) );
        m_selectionPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_applyButton );
        m_selectionPanel.add( Box.createGlue( ) );
        m_selectionPanel.add( m_cancelButton );
        m_selectionPanel.add( Box.createGlue( ) );
        
        m_contentPanel.add( m_selectionPanel );
    }
    
    /**
     * Instantiates the panel containing the controls that allows horizontal range
     * selections.
     */
    private void instantiateHorizontalRangePanel( String unit_x )
    {
        m_horizontalRangeWidthPanel = new JPanel( );
        m_horizontalRangeWidthPanel.setLayout( new BoxLayout( m_horizontalRangeWidthPanel, BoxLayout.Y_AXIS ) );
        m_horizontalRangeWidthPanel.setBorder( BorderFactory.createCompoundBorder(
                                               BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                               BorderFactory.createTitledBorder( "Horizontal display parameters" ) )
                                             );
        
        m_horizontalRangeWidthPanel.setPreferredSize( new Dimension( 430, 130 ) );
        m_horizontalRangeWidthPanel.setMaximumSize( new Dimension( 430, 130 ) );
        m_horizontalRangeWidthPanel.setMinimumSize( new Dimension( 430, 130 ) );
        
        m_horizontalRangePanel = new JPanel( );
        m_horizontalRangePanel.setLayout( new BoxLayout( m_horizontalRangePanel, BoxLayout.X_AXIS ) );
        
        m_horizontalRangeLabel = new JLabel( "Horizontal range" );
        
        m_horizontalRangePanel.add( m_horizontalRangeLabel );
        
        m_horizontalRangePanel.add( Box.createHorizontalGlue( ) );
        
        m_horizontalButtonPanel = new JPanel( );
        m_horizontalButtonPanel.setLayout( new BoxLayout( m_horizontalButtonPanel, BoxLayout.X_AXIS ) );
        
        m_halfHorizontalRangeButton = new JButton( "Halve" );
        m_halfHorizontalRangeButton.setToolTipText( "Halves the display range" );
        m_halfHorizontalRangeButton.addActionListener( this );
        
        m_horizontalButtonPanel.add( Box.createHorizontalGlue( ) );
        
        m_horizontalButtonPanel.add( m_halfHorizontalRangeButton );
        
        m_doubleHorizontalRangeButton = new JButton( "Double" );
        m_doubleHorizontalRangeButton.setToolTipText( "Doubles the display range" );
        m_doubleHorizontalRangeButton.addActionListener( this );
        
        m_horizontalButtonPanel.add( Box.createHorizontalGlue( ) );
        
        m_horizontalButtonPanel.add( m_doubleHorizontalRangeButton );
        
        m_horizontalRangePanel.add( m_horizontalButtonPanel );
        
        m_horizontalButtonPanel.add( Box.createHorizontalGlue( ) );
        
        m_horizontalRangeWidthPanel.add( m_horizontalRangePanel );
        
        m_horizontalSliderPanel = new JPanel( );
        m_horizontalSliderPanel.setLayout( new BoxLayout( m_horizontalSliderPanel, BoxLayout.X_AXIS ) ); 
        
        m_horizontalCenterSliderLabel = new JLabel( "Horizontal center" );
        m_horizontalSliderPanel.add( m_horizontalCenterSliderLabel );
        
        
        m_horizontalSliderPanel.add( Box.createHorizontalGlue( ) );
        
        m_horizontalCenterSlider = new JSlider( JSlider.HORIZONTAL, -100, 100, 0 );
        m_horizontalCenterSlider.setMajorTickSpacing( 50 );
        m_horizontalCenterSlider.setMinorTickSpacing( 10 );
        m_horizontalCenterSlider.setPaintTicks( true );
        m_horizontalCenterSlider.setPaintLabels( true );
        m_horizontalCenterSlider.addChangeListener( this );
        m_horizontalCenterSlider.setMinimumSize( m_horizontalCenterSlider.getPreferredSize( ) );
        
        m_horizontalCenterSlider.setMinimumSize( new Dimension( 250, 50 ) );
        m_horizontalCenterSlider.setMaximumSize( new Dimension( 250, 50 ) );
        m_horizontalCenterSlider.setPreferredSize( new Dimension( 250, 50 ) );
        
        m_horizontalSliderPanel.add( m_horizontalCenterSlider );
        
        m_horizontalRangeWidthPanel.add( m_horizontalSliderPanel );
       
        m_contentPanel.add( m_horizontalRangeWidthPanel );
    }
    
    /**
     * Instantiates the panel containing the controls that allows vertical range
     * selections.
     */
    private void instantiateVerticalRangePanel( String unit_x )
    {
        m_verticalRangeWidthPanel = new JPanel( );
        m_verticalRangeWidthPanel.setLayout( new BoxLayout( m_verticalRangeWidthPanel, BoxLayout.Y_AXIS ) );
        m_verticalRangeWidthPanel.setBorder( BorderFactory.createCompoundBorder(
                                             BorderFactory.createEmptyBorder( 5, 5, 5, 5 ),
                                             BorderFactory.createTitledBorder( "Vertical display parameters" ) )
                                           );
        
        m_verticalRangeWidthPanel.setPreferredSize( new Dimension( 430, 130 ) );
        m_verticalRangeWidthPanel.setMaximumSize( new Dimension( 430, 130 ) );
        m_verticalRangeWidthPanel.setMinimumSize( new Dimension( 430, 130 ) );
        
        m_verticalRangePanel = new JPanel( );
        m_verticalRangePanel.setLayout( new BoxLayout( m_verticalRangePanel, BoxLayout.X_AXIS ) );
        
        m_verticalRangeLabel = new JLabel( "Vertical range" );
        
        m_verticalRangePanel.add( m_verticalRangeLabel );
        
        m_verticalRangePanel.add( Box.createHorizontalGlue( ) );
        
        m_verticalButtonPanel = new JPanel( );
        m_verticalButtonPanel.setLayout( new BoxLayout( m_verticalButtonPanel, BoxLayout.X_AXIS ) );
        
        m_halfVerticalRangeButton = new JButton( "Halve" );
        m_halfVerticalRangeButton.setToolTipText( "Halves the display range" );
        m_halfVerticalRangeButton.addActionListener( this );
        
        m_verticalButtonPanel.add( Box.createHorizontalGlue( ) );
        
        m_verticalButtonPanel.add( m_halfVerticalRangeButton );
        
        m_doubleVerticalRangeButton = new JButton( "Double" );
        m_doubleVerticalRangeButton.setToolTipText( "Doubles the display range" );
        m_doubleVerticalRangeButton.addActionListener( this );
        
        m_verticalButtonPanel.add( Box.createHorizontalGlue( ) );
        
        m_verticalButtonPanel.add( m_doubleVerticalRangeButton );
        
        m_verticalRangePanel.add( m_verticalButtonPanel );
        
        m_verticalButtonPanel.add( Box.createHorizontalGlue( ) );
        
        m_verticalRangeWidthPanel.add( m_verticalRangePanel );
        
        m_verticalSliderPanel = new JPanel( );
        m_verticalSliderPanel.setLayout( new BoxLayout( m_verticalSliderPanel, BoxLayout.X_AXIS ) ); 
        
        m_verticalCenterSliderLabel = new JLabel( "Vertical center" );
        m_verticalSliderPanel.add( m_verticalCenterSliderLabel );
        
        m_verticalSliderPanel.add( Box.createHorizontalGlue( ) );
        
        m_verticalCenterSlider = new JSlider( JSlider.HORIZONTAL, -100, 100, 0 );
        m_verticalCenterSlider.setMajorTickSpacing( 50 );
        m_verticalCenterSlider.setMinorTickSpacing( 10 );
        m_verticalCenterSlider.setPaintTicks( true );
        m_verticalCenterSlider.setPaintLabels( true );
        m_verticalCenterSlider.addChangeListener( this );
        m_verticalCenterSlider.setMinimumSize( m_verticalCenterSlider.getPreferredSize( ) );
        
        m_verticalCenterSlider.setMinimumSize( new Dimension( 250, 50 ) );
        m_verticalCenterSlider.setMaximumSize( new Dimension( 250, 50 ) );
        m_verticalCenterSlider.setPreferredSize( new Dimension( 250, 50 ) );
        
        m_verticalSliderPanel.add( m_verticalCenterSlider );
        
        m_verticalRangeWidthPanel.add( m_verticalSliderPanel );
       
        m_contentPanel.add( m_verticalRangeWidthPanel, BorderLayout.CENTER );
    }
        
    /**
     * Is called whenever a registered component raises an ActionEvent
     * @param e
     */
    @Override
    public void actionPerformed( ActionEvent e ) 
    {
       
        Object sender = e.getSource( );
       
       // handle responses on cancel button pressed event
        if ( sender == m_cancelButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, CANCEL_EVENT, "" ) );
             }
            dispose( );
        }
        // handle responses on apply button pressed event
        if ( sender == m_applyButton )
        {
            Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, ACCEPT_EVENT, "" ) );
             }
            dispose( );
        }
        
        if ( sender == m_halfHorizontalRangeButton )
        {
             Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, HORIZONTAL_HALF_RANGE_EVENT, "" ) );
             }
        }

        if ( sender == m_doubleHorizontalRangeButton )
        {
             Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, HORIZONTAL_DOUBLE_RANGE_EVENT, "" ) );
             }
        }

        if ( sender == m_halfVerticalRangeButton )
        {
             Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VERTICAL_HALF_RANGE_EVENT, "" ) );
             }
        }

        if ( sender == m_doubleVerticalRangeButton )
        {
             Iterator it = m_listeners.iterator( );
             while( it.hasNext( ) )
             {
                ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VERTICAL_DOUBLE_RANGE_EVENT, "" ) );
             }
        }
    }
    
    /**
     * Adds an Actionlistener to the list of action listeners who are notified
     * when something happens within this object.
     * @param e
     */
    public void addActionListener( ActionListener e )
    {
	m_listeners.add( e );
    }
    
    /**
     * Called whenever a registered component raises a changeEvent.
     */
    @Override
    public void stateChanged( ChangeEvent e ) 
    {
        Object sender = e.getSource( );
        
        // Handles responses on horizontal slider changed and released event
        if ( sender == m_horizontalCenterSlider )
        {
             if ( !m_horizontalCenterSlider.getValueIsAdjusting( ) )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, HORIZONTAL_CENTER_EVENT, "" ) );
                }         
            }
        }
        
        // Handles responses on vertical slider changed and released event
        if ( sender == m_verticalCenterSlider )
        {
             if ( !m_verticalCenterSlider.getValueIsAdjusting( ) )
            {
                Iterator it = m_listeners.iterator( );
                while( it.hasNext( ) )
                {
                    ( ( ActionListener )it.next( ) ).actionPerformed( new ActionEvent( this, VERTICAL_CENTER_EVENT, "" ) );
                }         
            }
        }      
        
    }
    
    /**
     * Returns the value of the horizontalCenterslider.
     * @return 
     */
    public int getXCenter( )
    {
        return m_horizontalCenterSlider.getValue( );
    }
    
    /**
     * Returns the value of the verticalCenterslider.
     * @return 
     */
    public int getYCenter( )
    {
        return m_verticalCenterSlider.getValue( );
    }
    
    
    private JFrame m_parent;
    private JPanel m_contentPanel;
    
    
    private JPanel m_verticalRangePanel;
    
    JButton m_applyButton;
    JButton m_cancelButton;
    JPanel m_selectionPanel;
    
    // Listeners to changes in this class
    private LinkedList<ActionListener> m_listeners = new LinkedList<ActionListener>( );
    
    private JPanel      m_horizontalRangeWidthPanel;
    
    // Horizontal
    private JPanel      m_horizontalRangePanel;
    private JPanel      m_horizontalButtonPanel;
    private JButton     m_halfHorizontalRangeButton;
    private JButton     m_doubleHorizontalRangeButton;
    
    private JLabel      m_horizontalRangeLabel;
    private JLabel      m_horizontalCenterSliderLabel;
    private JPanel      m_horizontalSliderPanel;
    private JSlider     m_horizontalCenterSlider;
    
    // Vertical
    
    private JPanel      m_verticalRangeWidthPanel;
    
    private JPanel      m_verticalSliderPanel;
    private JPanel      m_verticalButtonPanel;
    private JButton     m_halfVerticalRangeButton;
    private JButton     m_doubleVerticalRangeButton;    
    
    private JLabel      m_verticalCenterSliderLabel;
    private JSlider     m_verticalCenterSlider;
    private JLabel      m_verticalRangeLabel;
    

    // Event IDs raised by this control
    public final static int ACCEPT_EVENT = 0;
    public final static int CANCEL_EVENT = 1;
    
    public final static int HORIZONTAL_CENTER_EVENT = 2;
    public final static int HORIZONTAL_HALF_RANGE_EVENT = 3;
    public final static int HORIZONTAL_DOUBLE_RANGE_EVENT = 4;
    
    public final static int VERTICAL_CENTER_EVENT = 5;
    public final static int VERTICAL_HALF_RANGE_EVENT = 6; 
    public final static int VERTICAL_DOUBLE_RANGE_EVENT = 7;
    
    // For proper serialization
    private static final long serialVersionUID = 3487491268889393L;
}
